Package: knitr
Type: Package
Title: A General-Purpose Package for Dynamic Report Generation in R
Version: 1.51
Authors@R: c(
    person("Yihui", "Xie", role = c("aut", "cre"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666", URL = "https://yihui.org")),
    person("Abhraneel", "Sarma", role = "ctb"),
    person("Adam", "Vogt", role = "ctb"),
    person("Alastair", "Andrew", role = "ctb"),
    person("Alex", "Zvoleff", role = "ctb"),
    person("Amar", "Al-Zubaidi", role = "ctb"),
    person("Andre", "Simon", role = "ctb", comment = "the CSS files under inst/themes/ were derived from the Highlight package http://www.andre-simon.de"),
    person("Aron", "Atkins", role = "ctb"),
    person("Aaron", "Wolen", role = "ctb"),
    person("Ashley", "Manton", role = "ctb"),
    person("Atsushi", "Yasumoto", role = "ctb", comment = c(ORCID = "0000-0002-8335-495X")),
    person("Ben", "Baumer", role = "ctb"),
    person("Brian", "Diggs", role = "ctb"),
    person("Brian", "Zhang", role = "ctb"),
    person("Bulat", "Yapparov", role = "ctb"),
    person("Cassio", "Pereira", role = "ctb"),
    person("Christophe", "Dervieux", role = "ctb"),
    person("David", "Hall", role = "ctb"),
    person("David", "Hugh-Jones", role = "ctb"),
    person("David", "Robinson", role = "ctb"),
    person("Doug", "Hemken", role = "ctb"),
    person("Duncan", "Murdoch", role = "ctb"),
    person("Elio", "Campitelli", role = "ctb"),
    person("Ellis", "Hughes", role = "ctb"),
    person("Emily", "Riederer", role = "ctb"),
    person("Fabian", "Hirschmann", role = "ctb"),
    person("Fitch", "Simeon", role = "ctb"),
    person("Forest", "Fang", role = "ctb"),
    person(c("Frank", "E", "Harrell", "Jr"), role = "ctb", comment = "the Sweavel package at inst/misc/Sweavel.sty"),
    person("Garrick", "Aden-Buie", role = "ctb"),
    person("Gregoire", "Detrez", role = "ctb"),
    person("Hadley", "Wickham", role = "ctb"),
    person("Hao", "Zhu", role = "ctb"),
    person("Heewon", "Jeon", role = "ctb"),
    person("Henrik", "Bengtsson", role = "ctb"),
    person("Hiroaki", "Yutani", role = "ctb"),
    person("Ian", "Lyttle", role = "ctb"),
    person("Hodges", "Daniel", role = "ctb"),
    person("Jacob", "Bien", role = "ctb"),
    person("Jake", "Burkhead", role = "ctb"),
    person("James", "Manton", role = "ctb"),
    person("Jared", "Lander", role = "ctb"),
    person("Jason", "Punyon", role = "ctb"),
    person("Javier", "Luraschi", role = "ctb"),
    person("Jeff", "Arnold", role = "ctb"),
    person("Jenny", "Bryan", role = "ctb"),
    person("Jeremy", "Ashkenas", role = c("ctb", "cph"), comment = "the CSS file at inst/misc/docco-classic.css"),
    person("Jeremy", "Stephens", role = "ctb"),
    person("Jim", "Hester", role = "ctb"),
    person("Joe", "Cheng", role = "ctb"),
    person("Johannes", "Ranke", role = "ctb"),
    person("John", "Honaker", role = "ctb"),
    person("John", "Muschelli", role = "ctb"),
    person("Jonathan", "Keane", role = "ctb"),
    person("JJ", "Allaire", role = "ctb"),
    person("Johan", "Toloe", role = "ctb"),
    person("Jonathan", "Sidi", role = "ctb"),
    person("Joseph", "Larmarange", role = "ctb"),
    person("Julien", "Barnier", role = "ctb"),
    person("Kaiyin", "Zhong", role = "ctb"),
    person("Kamil", "Slowikowski", role = "ctb"),
    person("Karl", "Forner", role = "ctb"),
    person(c("Kevin", "K."), "Smith", role = "ctb"),
    person("Kirill", "Mueller", role = "ctb"),
    person("Kohske", "Takahashi", role = "ctb"),
    person("Lorenz", "Walthert", role = "ctb"),
    person("Lucas", "Gallindo", role = "ctb"),
    person("Marius", "Hofert", role = "ctb"),
    person("Martin", "Modrák", role = "ctb"),
    person("Michael", "Chirico", role = "ctb"),
    person("Michael", "Friendly", role = "ctb"),
    person("Michal", "Bojanowski", role = "ctb"),
    person("Michel", "Kuhlmann", role = "ctb"),
    person("Miller", "Patrick", role = "ctb"),
    person("Nacho", "Caballero", role = "ctb"),
    person("Nick", "Salkowski", role = "ctb"),
    person("Niels Richard", "Hansen", role = "ctb"),
    person("Noam", "Ross", role = "ctb"),
    person("Obada", "Mahdi", role = "ctb"),
    person("Pavel N.", "Krivitsky", role = "ctb", comment=c(ORCID = "0000-0002-9101-3362")),
    person("Pedro", "Faria", role = "ctb"),
    person("Qiang", "Li", role = "ctb"),
    person("Ramnath", "Vaidyanathan", role = "ctb"),
    person("Richard", "Cotton", role = "ctb"),
    person("Robert", "Krzyzanowski", role = "ctb"),
    person("Rodrigo", "Copetti", role = "ctb"),
    person("Romain", "Francois", role = "ctb"),
    person("Ruaridh", "Williamson", role = "ctb"),
    person("Sagiru", "Mati", role = "ctb", comment = c(ORCID = "0000-0003-1413-3974")),
    person("Scott", "Kostyshak", role = "ctb"),
    person("Sebastian", "Meyer", role = "ctb"),
    person("Sietse", "Brouwer", role = "ctb"),
    person(c("Simon", "de"), "Bernard", role = "ctb"),
    person("Sylvain", "Rousseau", role = "ctb"),
    person("Taiyun", "Wei", role = "ctb"),
    person("Thibaut", "Assus", role = "ctb"),
    person("Thibaut", "Lamadon", role = "ctb"),
    person("Thomas", "Leeper", role = "ctb"),
    person("Tim", "Mastny", role = "ctb"),
    person("Tom", "Torsney-Weir", role = "ctb"),
    person("Trevor", "Davis", role = "ctb"),
    person("Viktoras", "Veitas", role = "ctb"),
    person("Weicheng", "Zhu", role = "ctb"),
    person("Wush", "Wu", role = "ctb"),
    person("Zachary", "Foster", role = "ctb"),
    person("Zhian N.", "Kamvar", role = "ctb", comment = c(ORCID = "0000-0003-1458-7108")),
    person(given = "Posit Software, PBC", role = c("cph", "fnd"))
    )
Description: Provides a general-purpose tool for dynamic report generation in R
    using Literate Programming techniques.
Depends: R (>= 3.6.0)
Imports: evaluate (>= 0.15), highr (>= 0.11), methods, tools, xfun (>=
        0.52), yaml (>= 2.1.19)
Suggests: bslib, DBI (>= 0.4-1), digest, formatR, gifski, gridSVG,
        htmlwidgets (>= 0.7), jpeg, JuliaCall (>= 0.11.1), magick,
        litedown, markdown (>= 1.3), otel, otelsdk, png, ragg,
        reticulate (>= 1.4), rgl (>= 0.95.1201), rlang, rmarkdown,
        sass, showtext, styler (>= 1.2.0), targets (>= 0.6.0), testit,
        tibble, tikzDevice (>= 0.10), tinytex (>= 0.56), webshot,
        rstudioapi, svglite
License: GPL
URL: https://yihui.org/knitr/
BugReports: https://github.com/yihui/knitr/issues
Encoding: UTF-8
VignetteBuilder: litedown, knitr
SystemRequirements: Package vignettes based on R Markdown v2 or
        reStructuredText require Pandoc (http://pandoc.org). The
        function rst2pdf() requires rst2pdf
        (https://github.com/rst2pdf/rst2pdf).
Collate: 'block.R' 'cache.R' 'citation.R' 'hooks-html.R' 'plot.R'
        'utils.R' 'defaults.R' 'concordance.R' 'engine.R' 'highlight.R'
        'themes.R' 'header.R' 'hooks-asciidoc.R' 'hooks-chunk.R'
        'hooks-extra.R' 'hooks-latex.R' 'hooks-md.R' 'hooks-rst.R'
        'hooks-textile.R' 'hooks.R' 'otel.R' 'output.R' 'package.R'
        'pandoc.R' 'params.R' 'parser.R' 'pattern.R' 'rocco.R' 'spin.R'
        'table.R' 'template.R' 'utils-conversion.R' 'utils-rd2html.R'
        'utils-string.R' 'utils-sweave.R' 'utils-upload.R'
        'utils-vignettes.R' 'zzz.R'
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-19 15:19:25 UTC; yihui
Author: Yihui Xie [aut, cre] (ORCID: <https://orcid.org/0000-0003-0645-5666>,
    URL: https://yihui.org),
  Abhraneel Sarma [ctb],
  Adam Vogt [ctb],
  Alastair Andrew [ctb],
  Alex Zvoleff [ctb],
  Amar Al-Zubaidi [ctb],
  Andre Simon [ctb] (the CSS files under inst/themes/ were derived from
    the Highlight package http://www.andre-simon.de),
  Aron Atkins [ctb],
  Aaron Wolen [ctb],
  Ashley Manton [ctb],
  Atsushi Yasumoto [ctb] (ORCID: <https://orcid.org/0000-0002-8335-495X>),
  Ben Baumer [ctb],
  Brian Diggs [ctb],
  Brian Zhang [ctb],
  Bulat Yapparov [ctb],
  Cassio Pereira [ctb],
  Christophe Dervieux [ctb],
  David Hall [ctb],
  David Hugh-Jones [ctb],
  David Robinson [ctb],
  Doug Hemken [ctb],
  Duncan Murdoch [ctb],
  Elio Campitelli [ctb],
  Ellis Hughes [ctb],
  Emily Riederer [ctb],
  Fabian Hirschmann [ctb],
  Fitch Simeon [ctb],
  Forest Fang [ctb],
  Frank E Harrell Jr [ctb] (the Sweavel package at inst/misc/Sweavel.sty),
  Garrick Aden-Buie [ctb],
  Gregoire Detrez [ctb],
  Hadley Wickham [ctb],
  Hao Zhu [ctb],
  Heewon Jeon [ctb],
  Henrik Bengtsson [ctb],
  Hiroaki Yutani [ctb],
  Ian Lyttle [ctb],
  Hodges Daniel [ctb],
  Jacob Bien [ctb],
  Jake Burkhead [ctb],
  James Manton [ctb],
  Jared Lander [ctb],
  Jason Punyon [ctb],
  Javier Luraschi [ctb],
  Jeff Arnold [ctb],
  Jenny Bryan [ctb],
  Jeremy Ashkenas [ctb, cph] (the CSS file at
    inst/misc/docco-classic.css),
  Jeremy Stephens [ctb],
  Jim Hester [ctb],
  Joe Cheng [ctb],
  Johannes Ranke [ctb],
  John Honaker [ctb],
  John Muschelli [ctb],
  Jonathan Keane [ctb],
  JJ Allaire [ctb],
  Johan Toloe [ctb],
  Jonathan Sidi [ctb],
  Joseph Larmarange [ctb],
  Julien Barnier [ctb],
  Kaiyin Zhong [ctb],
  Kamil Slowikowski [ctb],
  Karl Forner [ctb],
  Kevin K. Smith [ctb],
  Kirill Mueller [ctb],
  Kohske Takahashi [ctb],
  Lorenz Walthert [ctb],
  Lucas Gallindo [ctb],
  Marius Hofert [ctb],
  Martin Modrák [ctb],
  Michael Chirico [ctb],
  Michael Friendly [ctb],
  Michal Bojanowski [ctb],
  Michel Kuhlmann [ctb],
  Miller Patrick [ctb],
  Nacho Caballero [ctb],
  Nick Salkowski [ctb],
  Niels Richard Hansen [ctb],
  Noam Ross [ctb],
  Obada Mahdi [ctb],
  Pavel N. Krivitsky [ctb] (ORCID:
    <https://orcid.org/0000-0002-9101-3362>),
  Pedro Faria [ctb],
  Qiang Li [ctb],
  Ramnath Vaidyanathan [ctb],
  Richard Cotton [ctb],
  Robert Krzyzanowski [ctb],
  Rodrigo Copetti [ctb],
  Romain Francois [ctb],
  Ruaridh Williamson [ctb],
  Sagiru Mati [ctb] (ORCID: <https://orcid.org/0000-0003-1413-3974>),
  Scott Kostyshak [ctb],
  Sebastian Meyer [ctb],
  Sietse Brouwer [ctb],
  Simon de Bernard [ctb],
  Sylvain Rousseau [ctb],
  Taiyun Wei [ctb],
  Thibaut Assus [ctb],
  Thibaut Lamadon [ctb],
  Thomas Leeper [ctb],
  Tim Mastny [ctb],
  Tom Torsney-Weir [ctb],
  Trevor Davis [ctb],
  Viktoras Veitas [ctb],
  Weicheng Zhu [ctb],
  Wush Wu [ctb],
  Zachary Foster [ctb],
  Zhian N. Kamvar [ctb] (ORCID: <https://orcid.org/0000-0003-1458-7108>),
  Posit Software, PBC [cph, fnd]
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2025-12-20 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 02:18:51 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("$",knitr_strict_list)
S3method(is_low_change,default)
S3method(knit_print,default)
S3method(knit_print,knit_asis)
S3method(knit_print,knit_asis_url)
S3method(knit_print,knitr_kable)
S3method(print,knitr_kable)
S3method(sew,character)
S3method(sew,default)
S3method(sew,error)
S3method(sew,html_screenshot)
S3method(sew,knit_asis)
S3method(sew,knit_embed_url)
S3method(sew,knit_image_paths)
S3method(sew,list)
S3method(sew,message)
S3method(sew,recordedplot)
S3method(sew,source)
S3method(sew,warning)
export(Sweave2knitr)
export(all_labels)
export(all_patterns)
export(all_rcpp_labels)
export(asis_output)
export(cache_engines)
export(clean_cache)
export(combine_words)
export(convert_chunk_header)
export(current_input)
export(dep_auto)
export(dep_prev)
export(download_image)
export(engine_output)
export(extract_raw_output)
export(fig_chunk)
export(fig_path)
export(hook_ffmpeg_html)
export(hook_gifski)
export(hook_mogrify)
export(hook_movecode)
export(hook_optipng)
export(hook_pdfcrop)
export(hook_plot_asciidoc)
export(hook_plot_custom)
export(hook_plot_html)
export(hook_plot_md)
export(hook_plot_rst)
export(hook_plot_tex)
export(hook_plot_textile)
export(hook_pngquant)
export(hook_purl)
export(hook_r2swf)
export(hook_scianimator)
export(hooks_asciidoc)
export(hooks_html)
export(hooks_jekyll)
export(hooks_latex)
export(hooks_listings)
export(hooks_markdown)
export(hooks_rst)
export(hooks_sweave)
export(hooks_textile)
export(image_uri)
export(imgur_upload)
export(include_app)
export(include_graphics)
export(include_url)
export(inline_expr)
export(is_html_output)
export(is_latex_output)
export(is_low_change)
export(kable)
export(kables)
export(knit)
export(knit2html)
export(knit2pandoc)
export(knit2pdf)
export(knit2wp)
export(knit_child)
export(knit_code)
export(knit_engines)
export(knit_exit)
export(knit_expand)
export(knit_filter)
export(knit_global)
export(knit_hooks)
export(knit_meta)
export(knit_meta_add)
export(knit_params)
export(knit_params_yaml)
export(knit_patterns)
export(knit_print)
export(knit_rd)
export(knit_rd_all)
export(knit_theme)
export(knit_watch)
export(load_cache)
export(normal_print)
export(opts_chunk)
export(opts_current)
export(opts_hooks)
export(opts_knit)
export(opts_template)
export(pandoc)
export(pandoc_from)
export(pandoc_to)
export(partition_chunk)
export(pat_asciidoc)
export(pat_brew)
export(pat_html)
export(pat_md)
export(pat_rnw)
export(pat_rst)
export(pat_tex)
export(pat_textile)
export(plot_crop)
export(purl)
export(rand_seed)
export(raw_block)
export(raw_html)
export(raw_latex)
export(raw_output)
export(read_chunk)
export(read_demo)
export(render_asciidoc)
export(render_html)
export(render_jekyll)
export(render_latex)
export(render_listings)
export(render_markdown)
export(render_rst)
export(render_sweave)
export(render_textile)
export(restore_raw_output)
export(rnw2pdf)
export(rocco)
export(rst2pdf)
export(set_alias)
export(set_header)
export(set_parent)
export(sew)
export(spin)
export(spin_child)
export(stitch)
export(stitch_rhtml)
export(stitch_rmd)
export(wrap_rmd)
export(write_bib)
import(grDevices)
import(graphics)
import(stats)
import(utils)
importFrom(xfun,file_ext)
importFrom(xfun,file_string)
importFrom(xfun,html_escape)
importFrom(xfun,is_R_CMD_check)
importFrom(xfun,is_abs_path)
importFrom(xfun,is_windows)
importFrom(xfun,loadable)
importFrom(xfun,parse_only)
importFrom(xfun,read_utf8)
importFrom(xfun,sans_ext)
importFrom(xfun,try_silent)
importFrom(xfun,with_ext)
importFrom(xfun,write_utf8)

# knitr

[![Build Status](https://travis-ci.org/yihui/knitr.svg)](https://travis-ci.org/yihui/knitr)
[![Coverage status](https://codecov.io/gh/yihui/knitr/branch/master/graph/badge.svg)](https://codecov.io/github/yihui/knitr?branch=master)
[![Downloads from the RStudio CRAN mirror](http://cranlogs.r-pkg.org/badges/knitr)](https://cran.r-project.org/package=knitr)

The R package **knitr** is a general-purpose literate programming engine,
with lightweight API's designed to give users full control of the output
without heavy coding work. It combines many features into one package with
slight tweaks motivated from my everyday use of Sweave. See the package
[homepage](https://yihui.org/knitr/) for details and examples. See
[FAQ's](https://yihui.org/knitr/faq/) for a list of
frequently asked questions (including where to ask questions).

## Installation

You can install the stable version on
[CRAN](https://cran.r-project.org/package=knitr):

```r
install.packages('knitr')
```

You can also install the development version from
[XRAN](https://xran.yihui.org), which provides daily build of **knitr**:

```r
# update all existing packages first
update.packages(ask = FALSE, repos = 'https://cran.r-project.org')
install.packages('knitr', repos = c('https://xran.yihui.org', 'https://cran.r-project.org'))
```

Note that if you want to build the source package via `R CMD INSTALL` without a previously installed version of **knitr**, you must either pre-install **knitr** from CRAN, or run `R CMD INSTALL` on this source repo, otherwise `R CMD build` will fail (which is probably a bug of base R).

## Motivation

While Sweave and related add-on packages like
[**cacheSweave**](https://cran.r-project.org/package=cacheSweave) and
[**pgfSweave**](https://cran.r-project.org/package=pgfSweave) are fairly good
engines for literate programming in R, I often feel my hands are tied.
For example:

- I stared at the source code of Sweave and wished for hundreds of times,
  *if only I could easily insert* `[width=.8\textwidth]` *between*
  `\includegraphics` *and* `{my-plot.pdf}`. (The official way in Sweave is
  `\setkeys{Gin}` but it is setting a global width, which is unrealistic
  since we often have to set widths individually; yes, you can use
  `\setkeys{Gin}` for many times, but why not just provide an option for
  each chunk?)
- I wished for many times, *if only I could use graphics devices other
  than PDF and postscript*; now the dream has come true in the official R,
  but what I was hoping for was an option as simple as `dev = 'png'` or `dev
  = 'CairoJPEG'`.
- I wished multiple plots in a code chunk could be recorded instead of only
  the last one.
- I wished there was a way to round the numbers in `\Sexpr{}` other than
  writing expressions like `\Sexpr{round(x, 3)}` for *each single* `\Sexpr{}`
- I wished I did not have to `print()` plots from.
  [**ggplot2**](https://cran.r-project.org/package=ggplot2) and a simple
  `qplot(x, y)` would just give me a plot in Sweave.
- I wished users would never need instructions on `Sweave.sty` or run into
  troubles due to the fact that LaTeX cannot find `Sweave.sty`.
- I wished **cacheSweave** could print the results of a code chunk even if
  it was cached.
- I wished [**brew**](https://cran.r-project.org/package=brew) could support
  graphics.
- I wished [**R2HTML**](https://cran.r-project.org/package=R2HTML) could
  support R code syntax highlighting.
- ...


[<img src="http://i.imgur.com/yYw46aF.jpg" align="right" alt="The book Dynamic Documents with R and knitr" />](https://amzn.com/1498716962)

The package **knitr** was designed to give the user access to every part of
the process of dealing with a literate programming document, so there is no
need to hack at any core components if you want more freedom. I have gone
through the source code of **pgfSweave** and **cacheSweave** for a couple of
times and I often feel uncomfortable with the large amount of code copied
from official R, especially when R has a new version released (I will begin
to worry if the add-on packages are still up-to-date with the official
Sweave).

## Usage

```r
library(knitr)
?knit
knit(input)
```

If options are not explicitly specified, **knitr** will try to guess
reasonable default settings. A few manuals are available such as the [main
manual](https://yihui.org/knitr/demo/manual/), and the
[graphics
manual](https://yihui.org/knitr/demo/graphics/). For a
more organized reference, see the [knitr book](https://amzn.com/1498716962).

## License

This package is free and open source software, licensed under GPL.

library(testit)

# see http://stackoverflow.com/q/18992260/559676 for the bug
assert('inline_exec only accept character result', {
  block = list(code = "function() 1", input = "inline `r function() 1`")
  res = xfun::try_silent(inline_exec(block, new.env()))
  (inherits(res, 'try-error'))
  block = list(code = "(function() 1)()", input = "inline `r (function() 1)()`")
  block$location = matrix(c(8,27), ncol = 2, byrow = TRUE)
  res = inline_exec(block, new.env())
  (res %==% "inline 1")
  block = list(code = character(0), input = "no inline")
  res = inline_exec(block, new.env())
  (res %==% "no inline")
})

assert('label_code correct adds comment on code for yaml block or parsed param', {
  oldW = getOption('width')
  options(width = 20)
  (label_code("1+1", list(params.src = "test, eval=TRUE")) %==% "## ----test, eval=TRUE----\n1+1\n")
  options(width = oldW)
  (label_code("1+1", list(params.chunk = c("#| label: test", "#| eval: true"))) %==%
      "## --------\n#| label: test\n#| eval: true\n1+1\n")
})

library(testit)

# helper function to convert raw src to params list
read_params = function(src, evaluate = TRUE) {
  lines = strsplit(src, "\n")[[1]]
  knit_params(lines, evaluate = evaluate)
}

# helper function to convert raw src yaml to params list
read_params_yaml = function(src, evaluate = TRUE) {
  knit_params_yaml(src, evaluate = evaluate)
}

params = read_params('
---
params:
  a: 10
  b: 20
---
'
)
assert('basic params parsing works', {
  (params[[1]]$name %==% 'a')
  (params[[1]]$value %==% 10L)
  (params[[2]]$name %==% 'b')
  (params[[2]]$value %==% 20L)
  (flatten_params(params) %==% list(a = 10L, b = 20L))
})

# test date custom type (these deprecated and here for backwards compt) --
params = read_params('
---
params:
  start: !date 2015-01-01
  end: !datetime 2015-01-01 12:30:00
---
'
)
assert('date/time params parsing works', {
  (params[[1]]$name %==% 'start')
  (inherits(params[[1]]$value, 'Date'))
  (params[[1]]$value %==% as.Date("2015-01-01"))
  (params[[2]]$name %==% 'end')
  (inherits(params[[2]]$value, 'POSIXct'))
  (params[[2]]$value %==% as.POSIXct("2015-01-01 12:30:00", tz = "GMT"))
})

params = read_params('
---
params:
  file1:
    value: data1.csv
---
'
)
assert('sub-option params parsing works', {
  (params[[1]]$name %==% 'file1')
  (params[[1]]$value %==% 'data1.csv')
})

params = read_params('
---
params:
  regions:
    value: [North, South]
---
'
)
assert('prams with length > 1 works', {
  (length(params[[1]]$value) %==% 2L)
  (params[[1]]$value[[2]] %==% 'South')
})

## test including additional parameter attributes --------------------------

params = read_params('
---
params:
  regions:
    value: [North, South]
    choices: [North, South, East, West]
    label: "Select Regions"
---
'
)
assert('other types of params can be parsed', {
  (params[[1]]$choices %==% c('North', 'South', 'East', 'West'))
  (params[[1]]$label %==% "Select Regions")
})

params = read_params('
---
params:
  x: 1
  y: 2
  z: 3
  n: 4
  Y: 5
  N: 6
---
'
)

assert('y/Y/n/N are not converted to booleans', {
  (unname(unlist(lapply(params, `[[`, 'name'))) %==% c('x', 'y', 'z', 'n', 'Y', 'N'))
})


params = read_params('
---
params:
  today: !r Sys.Date()
  now: !expr Sys.time()
  x: 10
---
'
)
assert('params as expressions can be parsed', {
  (!is.null(params[[1]]$expr))
  (inherits(params[[1]]$value, 'Date'))
  (!is.null(params[[2]]$expr))
  (inherits(params[[2]]$value, 'POSIXct'))
  (is.null(params[[3]]$expr))
})

params = read_params('
---
params:
  today: !r Sys.Date()
---
', evaluate = FALSE)
assert('unevaluated expressions in params can be parsed', {
  (params$today$expr %==% "Sys.Date()")
  (class(params$today$value) %==% "expression")
})

params = read_params_yaml('
params:
  x: 1
  today: !r Sys.Date()
  posixlt: !r strptime("2015-01-01", format = "%Y-%m-%d")
  list: [1,2,3]
#  map: { a: 1, b: 2 }
  map: { value: { a: 1, b: 2 } }
')
# The direct map value is not supported; an explicit value field is necessary
assert('yaml parameters can be handled', {
  (params$x$value %==% 1L)
  (params$today$expr %==% "Sys.Date()")
  (inherits(params$today$value, 'Date'))
  (params$posixlt$expr %==% 'strptime("2015-01-01", format = "%Y-%m-%d")')
  (inherits(params$posixlt$value, 'POSIXlt'))
})

library(testit)

render_jekyll('prettify')
options(prompt = '> ', continue = '+ ')

# see http://stackoverflow.com/q/18992260/559676 for the bug
assert('the source hook pastes source code into one character string', {
  (knit_hooks$get('source')(c('1+1', '2-2'), opts_chunk$get()) %==%
    "\n\n<pre><code class=\"prettyprint \">1+1\n2-2</code></pre>\n\n")
  (knit_hooks$get('source')(c('1+1+\n1', '2-2'), opts_chunk$merge(list(prompt = TRUE))) %==%
    "\n\n<pre><code class=\"prettyprint \">&gt; 1+1+\n+ 1\n&gt; 2-2</code></pre>\n\n")
})

render_markdown()

img_output = function(path, opts = list()) {
  opts = opts_chunk$merge(opts)
  sew(knit_print(include_graphics(path, error = FALSE)), opts)
}

assert('include_graphics() includes custom images correctly', {
  (img_output('a.png') %==% '![](a.png)')
  (img_output(c('a.png', 'b.png'), list(fig.show = 'hold')) %==% '![](a.png)![](b.png)')
  (img_output('a.png', list(fig.cap = 'foo bar')) %==% '![foo bar](a.png)')
  (img_output('a.png', list(out.width = '50%')) %==% '<img src="a.png" width="50%" />')
  (img_output('a.pdf', list(out.width = '300px')) %==% '<embed src="a.pdf" width="300px" type="application/pdf" />')
})

hook_src = knit_hooks$get("source")
options_ = list(engine = "r", prompt = FALSE, highlight = TRUE)

assert('Length of fences are satisfied', {
  (hook_src("", options_) %==% "\n\n``` r\n\n```\n\n")
  (hook_src("```", options_) %==% "\n\n```` r\n```\n````\n\n")
})

assert('Attributes for source can be specified class.source and attr.source', {
  (hook_src("1", c(options_, class.source = "a b")) %==% "\n\n```{.r .a .b}\n1\n```\n\n")
  (hook_src("1", c(options_, attr.source = ".a .b")) %==% "\n\n```{.r .a .b}\n1\n```\n\n")
  (hook_src("1", c(options_, class.source = "a", attr.source = "b='1'")) %==%
    "\n\n```{.r .a b='1'}\n1\n```\n\n")
  (hook_src("1", c(options_, attr.source = ".a b='1'")) %==%
    "\n\n```{.r .a b='1'}\n1\n```\n\n")
})

assert('class.source and attr.source works also with collapse = TRUE', {
  hook_chunk = hooks_markdown()$chunk
  (hook_chunk("```{.r .a b=1}\n1\n```", c(options_, collapse = TRUE)) %==%
      "```{.r .a b=1}\n1\n```")
  (hook_chunk("```{.r .a b=1}\n1\n```\n```{.r .a b=1}\n1\n```",
              c(options_, collapse = TRUE)) %==%
      "```{.r .a b=1}\n1\n1\n```")
})

hook_out = knit_hooks$get("output")

assert('Attributes for source can be specified class.source and attr.source', {
  (hook_out("1\n", c(options_, class.output = "a b")) %==%
    "\n\n```{.a .b}\n1\n```\n\n")
  (hook_out("1\n", c(options_, attr.output = ".a .b")) %==%
    "\n\n```{.a .b}\n1\n```\n\n")
  (hook_out("1\n", c(options_, class.output = "a", attr.output = "b='1'")) %==%
    "\n\n```{.a b='1'}\n1\n```\n\n")
  (hook_out("1\n", c(options_, attr.output = ".a b='1'")) %==%
    "\n\n```{.a b='1'}\n1\n```\n\n")
})

hook_chunk = knit_hooks$get("chunk")

assert('Chunks are enclosed by fenced divs when needed.', {
  (hook_chunk('', list(class.chunk=NULL)) %==% '')
  (hook_chunk('', list(class.chunk="")) %==% '::: \n\n:::')
  (hook_chunk('', list(class.chunk="foo")) %==% '::: foo\n\n:::')
  (hook_chunk(':::', list(class.chunk="foo")) %==% ':::: foo\n:::\n::::')
})

knit_hooks$restore()


x = "1.png"
w = h = 1
ex = "style='margin: 0;'"
cap = "foo"
opt = function(
  w = NULL, h = NULL, ex = NULL, cap = NULL, show = 'asis',
  fig.align = 'default', ...
) {
  list(
    out.width = w, out.height = h, out.extra = ex,
    fig.cap = cap, fig.show = show, fig.align = fig.align, ...
  )
}

assert("Include a plot by pandoc md", {
  (hook_plot_md_pandoc(x, opt()) %==% "![](1.png)")
  (hook_plot_md_pandoc(x, opt(w = w)) %==% sprintf("![](1.png){width=%s}", w))
  (hook_plot_md_pandoc(x, opt(h = h)) %==% sprintf("![](1.png){height=%s}", h))
  (hook_plot_md_pandoc(x, opt(cap = cap)) %==% sprintf("![%s](1.png)", cap))
  (hook_plot_md_pandoc(x, opt(ex = ex)) %==% sprintf("![](1.png){%s}", ex))
  (hook_plot_md_pandoc(x, opt(w = w, cap = cap, ex = ex)) %==%
    sprintf("![%s](1.png){width=%s %s}", cap, w, ex))
})

assert("fig.alt does not break office document", {
  old = opts_knit$get('rmarkdown.pandoc.to')
  opts_knit$set(rmarkdown.pandoc.to = "docx")
  (suppressWarnings(hook_plot_md(x, opt())) %==% "![](1.png)")
  (suppressWarnings(hook_plot_md(x, opt(fig.alt = "bar"))) %==% "![](1.png)")
  opts_knit$set('rmarkdown.pandoc.to' = old)
})

#!/usr/bin/env Rscript

if (!nzchar(Sys.which('lyx')) || system('lyx -version') != 0L) q()

call_lyx = function(file) {
  res = sapply(sprintf('lyx -e %s %s', c('knitr', 'r', 'pdf2'), file),
               system, ignore.stdout = TRUE, USE.NAMES = FALSE)
  unlink(sub('\\.lyx$', '.R', file))
  stopifnot(identical(res, integer(3L)))
}

for (i in list.files(pattern = '\\.lyx$')) {
  message(i)
  call_lyx(i)
  flush.console()
}

knit_script = normalizePath('../bin/knit', mustWork = TRUE)
knit_cmd = function(file) {
  paste('Rscript', shQuote(knit_script), shQuote(file))
}
test_cmd = function(cmd) {
  stopifnot(identical(system(cmd, ignore.stdout = TRUE), 0L))
}

for (i in list.files(pattern = '\\.Rmd')) {
  message(i)
  cmd = if (i == 'knitr-minimal.Rmd') {
    sprintf("Rscript -e 'library(knitr);opts_knit$set(base.url=\"http://animation.r-forge.r-project.org/ideas/\");knit(\"%s\")'", i)
  } else knit_cmd(i)
  test_cmd(cmd)
  flush.console()
}

test_cmd(sprintf("Rscript -e 'library(knitr);spin(\"knitr-spin.R\", precious=TRUE)'"))

setwd('child')
for (i in c('knitr-main.Rnw', 'knitr-parent.Rnw')) {
  test_cmd(knit_cmd(i))
}
unlink('*.tex')
test_cmd(knit_cmd('knitr-main.Rmd'))
setwd('..')

file.remove('child/knitr-main.html')

help(package = 'YourPackage', help_type = 'html')
# or see a standalone list of vignettes
browseVignettes('YourPackage')

if (TRUE) cat('_hello_ **markdown**!', '\n')

1+1
10:1
rnorm(5)^2
strsplit('hello, markdown vignettes', '')

n=300; set.seed(123)
par(mar=c(4,4,.1,.1))
plot(rnorm(n), rnorm(n), pch=21, cex=5*runif(n), col='white', bg='gray')

library(testit)

dep_list$restore()
knit_code$restore()


assert('find_symbols() identifies all symbols', {
  (find_symbols('x = x + 1; rnorm(1, std = z)') %==% c('x', 'rnorm', 'z'))
})

knit_lazy = function(lazy = TRUE) {
  in_dir(tempdir(), {
    txt = c(sprintf('```{r test, cache=TRUE, cache.lazy=%s}', lazy),
            'x1 = Sys.time()', '```')
    knit(text = txt, quiet = TRUE)
    x2 = x1
    Sys.sleep(0.1)
    knit(text = txt, quiet = TRUE)
    x1 == x2  # x1 should not be updated
  })
}
assert('cache.lazy = TRUE/FALSE works', {
  (knit_lazy(TRUE))
  (knit_lazy(FALSE))
})

knit_code$set(a = 1, b = 2, c = 3)
assert('dep_prev() sets dependencies on previous chunks', {
  # dependency is empty now
  (dep_list$get() %==% list())
  # b/c depend on a, c depends on b
  dep_prev()
  (dep_list$get() %==% list(a = c('b', 'c'), b = 'c'))
})
dep_list$restore()
knit_code$restore()

assert('dep_auto() solves dependencies', {
  # dependency is empty now
  (dep_list$get() %==% list())

  # base rmd text
  rmd0 = c(
    '```{r, autodep=TRUE, cache=TRUE}',
    'x = %s',
    '```',
    '```{r, autodep=TRUE, cache=TRUE}',
    'print(x)',
    '```'
  )

  td = tempfile()
  dir.create(td, showWarnings = FALSE, recursive = TRUE)

  rmd1 = sprintf(rmd0, 'runif(1)')
  rmd2 = sprintf(rmd0, '"a"')

  # without child document
  in_dir(td, {
    # with cache, the result should reproduce
    knit1 = knit(text = rmd1, quiet = TRUE)
    (knit(text = rmd1, quiet = TRUE) %==% knit1)

    # on updating `x`, the printed result should change
    knit2 = knit(text = rmd2, quiet = TRUE)
    print2 = gsub('\n.*', '', gsub('.*\n##', '##', knit2))
    (print2 %==% '## [1] "a"')
  })
})
dep_list$restore()
knit_code$restore()

assert('dep_auto() solves dependencies of child documents', {
  # dependency is empty now
  (dep_list$get() %==% list())

  # base rmd text
  rmd0 = c(
    '```{r, autodep=TRUE, cache=TRUE}',
    'x = %s',
    '```',
    '```{r, autodep=TRUE, cache=TRUE}',
    'print(x)',
    '```'
  )
  rmd1 = sprintf(rmd0, 'runif(1)')
  rmd2 = sprintf(rmd0, '"a"')

  td = tempfile()
  dir.create(td, showWarnings = FALSE, recursive = TRUE)

  # with child document
  parent = c(
    '```{r, child="child.Rmd"}',
    '```'
  )
  in_dir(td, {
    # with cache, the result should reproduce
    writeLines(rmd1, 'child.Rmd')
    knit1 = knit(text = parent, quiet = TRUE)
    (knit(text = parent, quiet = TRUE) %==% knit1)

    # on updating `x`, the printed result should change
    writeLines(rmd2, 'child.Rmd')
    knit2 = knit(text = parent, quiet = TRUE)
    print2 = gsub('\n.*', '', gsub('.*\n##', '##', knit2))
    (print2 %==% '## [1] "a"')
  })
})
dep_list$restore()
knit_code$restore()

library(testit)

kable2 = function(...) as.character(kable(...))

assert('kable() works on data frames/matrices of one row', {
  (kable2(data.frame(x = 1, y = 1), format = 'pipe') %==%
     c('|  x|  y|', '|--:|--:|', '|  1|  1|'))
})

m = matrix(1:2, nrow = 1, dimnames = list('a', c('x', 'y')))
assert('kable() does not discard row names when there is only one row', {
  (kable2(m) %==% c('|   |  x|  y|', '|:--|--:|--:|', '|a  |  1|  2|'))
})

assert('kable() can assign a column name for row names', {
  (kable2(m, col.names = c('z', colnames(m))) %==%
     c('|z  |  x|  y|', '|:--|--:|--:|', '|a  |  1|  2|'))
})

assert('kable() recycles the align argument correctly', {
  (kable2(m, align = 'c') %==%
     c('|   | x | y |', '|:--|:-:|:-:|', '|a  | 1 | 2 |'))
})

assert('kable() align with strings correctly', {
  (kable2(m, align = c('c', 'r')) %==% kable2(m, align = 'cr'))
})

assert('kable() works on character data frames', {
  (kable2(data.frame(x = 'a')) %==% c('|x  |', '|:--|', '|a  |'))
})

assert("kable() works on NA's", {
  (kable2(data.frame(x = c(NA, FALSE))) %==%
     c('|x     |', '|:-----|', '|NA    |', '|FALSE |'))
})

assert('kable() works with the jira format', {
  (kable2(m, 'jira') %==% c('||   ||  x||  y||', '|a  |  1|  2|'))
})

assert('kable() works with the org format', {
  (kable2(m, 'org') %==%
    c('|   |  x|  y|', '|<l>|<r>|<r>|', '|---+---+---|', '|a  |  1|  2|'))
  m2 = m; colnames(m2) = NULL  # no column names
  (kable2(m2, 'org') %==% c('|<l>|<r>|<r>|', '|a  |  1|  2|'))
  # the case of only one column
  (kable2(m[, 1, drop = FALSE], 'org', row.names = FALSE) %==%
    c('|  x|', '|<r>|', '|---|', '|  1|'))
})

assert('kable() does not add extra spaces to character columns', {
  (kable2(data.frame(x = c(1.2, 4.87), y = c('fooooo', 'bar')), 'latex') %==% '
\\begin{tabular}{r|l}
\\hline
x & y\\\\
\\hline
1.20 & fooooo\\\\
\\hline
4.87 & bar\\\\
\\hline
\\end{tabular}')
})

assert('kable() escapes LaTeX special characters by default', {
  (kable2(data.frame(x = c('10%', '5%'), col_name = c('3_8', '40_6')), 'latex') %==% '
\\begin{tabular}{l|l}
\\hline
x & col\\_name\\\\
\\hline
10\\% & 3\\_8\\\\
\\hline
5\\% & 40\\_6\\\\
\\hline
\\end{tabular}')
})

assert("kable() doesn't escape LaTeX special characters when escape = FALSE", {
  (kable2(data.frame(x = c('10%', '5%'), col_name = c('3_8', '40_6')), 'latex', escape = FALSE) %==% '
\\begin{tabular}{l|l}
\\hline
x & col_name\\\\
\\hline
10% & 3_8\\\\
\\hline
5% & 40_6\\\\
\\hline
\\end{tabular}'
  )
})

assert('kable() adds {} before [] when booktabs = TRUE', {
  (kable2(data.frame(x = c('[0, 1]', '(1, 2]'), y = c(35, 62)), 'latex', booktabs = TRUE) %==% '
\\begin{tabular}{lr}
\\toprule
x & y\\\\
\\midrule
{}[0, 1] & 35\\\\
(1, 2] & 62\\\\
\\bottomrule
\\end{tabular}'
   )
})

assert('kable(format = "latex", linesep = ...) works', {
  (kable2(data.frame(x = 1:4), 'latex', linesep  = c('', '', '\\midrule')) %==% '
\\begin{tabular}{r}
\\hline
x\\\\
\\hline
1\\\\
2\\\\
3\\\\
\\midrule
4\\\\
\\hline
\\end{tabular}')
})

assert('kable() does not trim escaped spaces at the end', {
  (kable2(data.frame(x = '\\ '), 'latex', escape =FALSE ) %==% '
\\begin{tabular}{l}
\\hline
x\\\\
\\hline
\\ \\\\
\\hline
\\end{tabular}')
})

assert('kable() escapes HTML special characters by default', {
  (kable2(data.frame(x = c('10<>', '5&2'), y = c('3>8', '"40"')), 'html') %==%
     '<table>\n <thead>\n  <tr>\n   <th style="text-align:left;"> x </th>\n   <th style="text-align:left;"> y </th>\n  </tr>\n </thead>\n<tbody>\n  <tr>\n   <td style="text-align:left;"> 10&lt;&gt; </td>\n   <td style="text-align:left;"> 3&gt;8 </td>\n  </tr>\n  <tr>\n   <td style="text-align:left;"> 5&amp;2 </td>\n   <td style="text-align:left;"> "40" </td>\n  </tr>\n</tbody>\n</table>')
})

assert('kable() doesn\'t escape HTML special characters when escape = FALSE', {
  (kable2(data.frame(x = c('10<>', '5&2'), y = c('3>8', '"40"')), 'html', escape = FALSE) %==%
     '<table>\n <thead>\n  <tr>\n   <th style="text-align:left;"> x </th>\n   <th style="text-align:left;"> y </th>\n  </tr>\n </thead>\n<tbody>\n  <tr>\n   <td style="text-align:left;"> 10<> </td>\n   <td style="text-align:left;"> 3>8 </td>\n  </tr>\n  <tr>\n   <td style="text-align:left;"> 5&2 </td>\n   <td style="text-align:left;"> "40" </td>\n  </tr>\n</tbody>\n</table>')
})

assert('kable(digits = vector) works on numeric matrices', {
  (kable2(matrix(c(1.1, 1.2, 2.3, 2.4), 2, dimnames = list(NULL, c('a', 'b'))), digits = c(0, 1)) %==%
     c('|  a|   b|', '|--:|---:|', '|  1| 2.3|', '|  1| 2.4|'))
})

assert('kable() works on matrices with duplicate row names', {
  (kable2(matrix(c(1, 1, 1, 1), ncol = 2, dimnames = list(c('A', 'A'), c('B', 'B')))) %==%
     c('|   |  B|  B|', '|:--|--:|--:|', '|A  |  1|  1|', '|A  |  1|  1|'))
})

assert('kable() works on matrices with NA colname', {
  (kable2(matrix(c(1, 1, 1, 1), ncol = 2, dimnames = list(c('A', NA), c('B', NA)))) %==%
     c('|   |  B| NA|', '|:--|--:|--:|', '|A  |  1|  1|', '|NA |  1|  1|'))
})

# edge cases (should not error)
x1 = matrix(NA, 0, 0)
x2 = matrix(NA, 0, 1)
x3 = matrix(NA, 1, 0)
for (f in c('simple', 'html', 'latex', 'rst', 'jira', 'org')) {
  kable(x1, f)
  kable(x2, f)
  kable(x3, f)
}

colnames(x2) = 'a'
assert('kable(, "pipe") works for a 0 row 1 column matrix', {
  (kable2(x2, 'pipe') %==% c('|a  |', '|:--|'))
})

assert('kable(, "pipe") works for a 1-column matrix without column names', {
  (kable2(matrix(1), 'pipe') %==% c('|   |', '|--:|', '|  1|'))
})

assert('kable(, "simple") generates a pipe table for a 0 row data.frame', {
  (kable2(data.frame(x = character(0), y = integer(0)), 'simple') %==%
     c('|x  |  y|', '|:--|--:|'))
})

assert('kable(, "simple") generates a simple table for a 1-column object', {
  # if the object has column names, indent the table by one space
  (kable2(data.frame(x = 1), 'simple') %==% c('   x', ' ---', '   1'))
  # if it doesn't have column names, generate the usual table (without indent)
  (kable2(matrix(1), 'simple') %==% c('---', '  1', '---'))
})

assert('kable(, "simple", caption = "Table Caption") works for a 1-column matrix', {
  x4 = matrix(1:2, ncol = 1, dimnames = list(NULL, 'a'))
  (kable2(x4, 'simple', caption = 'Table Caption') %==%
    c('Table: Table Caption', '', '   a', ' ---', '   1', '   2'))
})

assert('kable() works on an irregular matrix', {
  (kable2(matrix(list('a', 2, 3, 4), nrow = 2), col.names = c('a', 'b')) %==%
     c('|a  |b  |', '|:--|:--|', '|a  |3  |', '|2  |4  |'))
})

assert('has_rownames() works', {
  (!has_rownames(matrix(1:4, 2)))
  (!has_rownames(iris))
  (has_rownames(mtcars))
  (!has_rownames(as.data.frame(matrix(nrow = 0, ncol = 3))))
})

op = options(knitr.kable.NA = '')
assert('kable() can display NA as empty strings', {
  (kable2(matrix(c(1, NA, 3, 4), nrow = 2), col.names = c('a', 'b')) %==%
     c('|  a|  b|', '|--:|--:|', '|  1|  3|', '|   |  4|'))
})
options(op)

assert('kable() can apply formatting to custom objects', {
  d = tibble::tibble(x = structure(
    'print_me', .to_upper = TRUE, class = 'make_upper'
  ))

  format.make_upper = function(x, ...) {
    xout = unclass(x)
    if (isTRUE(attr(x, '.to_upper'))) {
      xout = toupper(xout)
    }
    as.character(xout)
  }
  registerS3method('format', 'make_upper', format.make_upper, environment(format))

  (kable2(d) %==% c('|x        |', '|:--------|', '|PRINT_ME |'))
})

library(testit)

assert("alt text is included in LaTeX output", {
  # no alt text
  (hook_plot_tex('foo.pdf', list(fig.align = 'center', fig.show = 'asis')) %==%
     '\n\n{\\centering \\includegraphics{foo} \n\n}\n\n')

  # alt text
  (hook_plot_tex('foo.pdf', list(fig.alt = 'Alt', fig.align = 'center',
                                 fig.show = 'asis')) %==%
     '\n\n{\\centering \\includegraphics[alt={Alt}]{foo} \n\n}\n\n')

  # with width
  (hook_plot_tex('foo.pdf', list(fig.alt = 'Alt', fig.align = 'center',
                                 fig.show = 'asis', out.width = '\\maxwidth')) %==%
     '\n\n{\\centering \\includegraphics[width=\\maxwidth,alt={Alt}]{foo} \n\n}\n\n')
})

library(testit)

opts_knit$set(out.format = 'markdown')
assert('parse_block() parses the language engine from ```{lang}', {
  (parse_block(NULL, '', 'r, foo, a=1,')$params %==% alist(label = 'foo', a = 1))
  (parse_block(NULL, '', 'Rcpp, foo, a=1,')$params %==% alist(label = 'foo', a = 1, engine = 'Rcpp'))
})

res = split_file(
  c('abc', '```{r foo}', '1+1', '```{r bar}', '2+2', '```', 'def'),
  patterns = all_patterns$md
)
assert('split_file() treats ``` as part of code chunk instead of beginning of text chunk', {
  # the foo chunk does not have a closing mark
  (c(knit_code$get('foo')) %==% '1+1')  # use c() to drop attributes
  (c(knit_code$get('bar')) %==% '2+2')
  # before knitr v1.6, the text chunk was c('', 'def')
  (res[[4]][['input']] %==% 'def')
})
opts_knit$restore()
knit_code$restore(); knit_concord$restore()

res = parse_inline(c('aaa \\Sexpr{x}', 'bbb \\Sexpr{NA} and \\Sexpr{1+2}',
                     'another expression \\Sexpr{rnorm(10)}'), all_patterns$rnw)
assert('parse_inline() parses inline text', {
  (res$code %==% c('x', 'NA', '1+2', 'rnorm(10)'))
  (nchar(res$input) %==% 81L)
  # empty inline code is not recognized
  (parse_inline('\\Sexpr{}', all_patterns$rnw)$code %==% character(0))
  # can use > in HTML inline code
  (parse_inline('<!--rinline "<a>" -->', all_patterns$html)$code %==% ' "<a>" ')
})

res = parse_inline('inline expressions `r pi+1`, +r cos(0)+ in AsciiDoc',
                   all_patterns$asciidoc)
assert('both `r expression` and +r expression+ work for AsciiDoc', {
  (res$code %==% c('pi+1', 'cos(0)'))
})

knit_code$restore()

read_chunk(lines = c('1+1'))
assert('read_chunk() does not discard code without chunk headers', {
  (knit_code$get() %==% list('unnamed-chunk-1' = '1+1'))
})

knit_code$restore()

read_chunk(lines = c('# ---- foo ----', '1+1'))
assert('read_chunk() can identify chunk labels', {
  (knit_code$get() %==% list(foo = '1+1'))
})

knit_code$restore()

# chunk references with <<>> --------------------------------------------------

knit_code$restore(list(
  a = '1+1', b = '2-2', c = c('if (T)', '  <<a>>'), d = c('function() {', '  <<c>>', '}')
))
pc = function(x) parse_chunk(x, all_patterns$rnw$ref.chunk)

assert('parse_chunk() preserves indentation', {
  (pc(c('3*3', '<<a>>', ' <<b>>', 'if (T)', '  <<a>>')) %==% c("3*3", "1+1", " 2-2", "if (T)", "  1+1" ))
  (pc('<<d>>') %==% c('function() {', '  if (T)', '    1+1', '}'))
})

assert('parse_chunk() ignores labels not found in knit_code', {
  # chunk 'e' doesn't exist
  (pc(c('3*3', '<<a>>', '  <<e>>', '<<b>>  ')) %==% c("3*3", "1+1", "  <<e>>", "2-2"))
})

knit_code$restore()

# duplication of labels

knit_code$restore(list(a = '1+1'))

assert('duplicated labels are not allowed by default', {
  (!has_error(parse_block(NULL, '', 'label = "a"')))
  (has_error(parse_block('2+2', '', 'label = "a"')))
  (has_error(parse_block(NULL, '','label = "a", code = "2+2"')))
  (has_error(parse_block(NULL, '','label = "a", file = "dummy.R"')))
})
op = options(knitr.duplicate.label = 'allow')
assert('duplicated labels are allowed after setting an option', {
  (!has_error(parse_block('2+2', '', 'label = "a"')))
  (!has_error(parse_block(NULL, '','label = "a", code = "2+2"')))
  (!has_error(parse_block(NULL, '','label = "a", file = "dummy.R"')))
})
options(op)

knit_code$restore()

## stolen from Romain's highlight package (v0.3.2)

# styler assistant for latex
styler_assistant_latex = function(x) {

  styles = sapply(x, function(item) {
    settings = names(item)
    has = function(s, value) {
      s %in% settings && grepl(value, item[[s]])
    }
    start = end = ''
    if ('color' %in% settings) {
      start = str_c(start, '\\textcolor[rgb]{', col2latexrgb(item[['color']]), '}{')
      end = str_c(end, '}')
    }
    if (has('font-weight', 'bold')) {
      start = str_c(start, '\\textbf{')
      end = str_c('}', end)
    }
    if (has('font-style', 'italic')) {
      start = str_c(start, '\\textit{')
      end = str_c('}', end)
    }
    if (has('text-decoration', 'underline')) {
      start = str_c(start, '\\underline{')
      end = str_c('}', end)
    }
    sprintf('%s#1%s', start, end)
  })
  sprintf('\\newcommand{\\hl%s}[1]{%s}%%', names(x), styles)
}

col2latexrgb = function(hex) {
  col = col2rgb(hex)[, 1]/255
  paste(col, collapse = ',')
}

library(testit)

# getting and setting themes

formats = c("markdown", "latex")
for (frm in formats) {
  opts_knit$set(out.format = frm)
  thm = knit_theme$get('acid')
  knit_theme$set(thm)
  assert('knit_theme$get returns theme(s)', {
    (identical(length(knit_theme$get()),
              length(dir(system.file("themes", package = "knitr")))))
    (!is.null(thm))
  })

  assert('knit_theme$set updates highlight and background', {
    (thm$highlight %==% as.character(opts_knit$get('header')[1]))
    (thm$background %==% opts_chunk$get('background'))
  })
  opts_knit$restore()
  opts_chunk$restore()
}

# style -> theme

tl = theme2list('acid.style')
assert('theme2list parsing from .style to CSS', {
  (tl$Description %==% "Acid")
  (tl$Canvas$Colour %==% "#eeeeee")
  (isTRUE(tl$Number$Bold))
  (is.null(tl$Keywords))
})

# theme -> css

css = list2css(tl)
assert('css is generated for a theme', {
  (grepl(tl$Canvas$Colour, css[1]))
  (grepl("font-weight: bold;", css[2]))
  (grepl(tl$String$Colour, css[3]))
})

fdir = file.path(tempdir(), "themes")
dir.create(fdir)
res = lapply(1:2, function(i) {
  fn = paste0(fdir, "/acid", i, '.theme', collapse = "")
  writeLines(readLines('acid.style'), con = fn)
  return()
})

cssdir = file.path(tempdir(), "css")
dir.create(cssdir)
suppressMessages(themes2css(fdir, cssdir))
assert('themes2css creates two css files from test data', {
  (length(dir(cssdir)) == 2)
  (dir(cssdir) %==% c("acid1.css", "acid2.css"))
})

library(testit)

assert('detect_pattern() automatically detects syntax patterns', {
  (detect_pattern('<<>>=') %==% 'rnw')
  (detect_pattern('<<foo, bar=TRUE>>=') %==% 'rnw')
  (detect_pattern('% begin.rcode') %==% 'tex')
  (detect_pattern('<!--begin.rcode') %==% 'html')
  (detect_pattern('``` {r}') %==% 'md')
  (detect_pattern('asdf', 'rnw') %==% 'rnw')
  (is.null(detect_pattern('foo')))
})

assert('group_pattern() checks if a pattern contains a group', {
  (group_pattern('(.*)'))
  (!group_pattern('()'))
  (!group_pattern('abc'))
  (!group_pattern(NULL))
})

ce_rnw = all_patterns$rnw$chunk.end
assert('patterns for Rnw', {
  (grep(ce_rnw, '  @') %==% 1L) # spaces before @
  (grep(ce_rnw, '@  ') %==% 1L) # spaces after @
  (grep(ce_rnw, '@ %asdf') %==% 1L) # comments after %
  (grep(ce_rnw, '@ asdf') %==% integer()) # only spaces/comments allowed
  (grep(ce_rnw, ' @ a% sdf') %==% integer())
})

cb_md = all_patterns$md$chunk.begin
assert('patterns for md', {
  # Chunk options
  (grepl(cb_md, '```{r}'))
  (grepl(cb_md, '```{r label}'))
  (grepl(cb_md, '```{r, eval=FALSE}'))
  (grepl(cb_md, '```{awk}'))
  # Pandoc's fenced code attributes
  (!grepl(cb_md, '```{.class}'))
  (!grepl(cb_md, '```{#id}'))
  (!grepl(cb_md, '```{style="color: red"}'))
  # Pandoc's raw attribute
  (!grepl(cb_md, '```{=latex}'))
})

library(testit)

z = new_defaults(list(a = 1))

assert('$set() and $get() methods set/get values', {
  (z$get() %==% list(a = 1))
  ({z$set(a = 2); z$get('a')} %==% 2)
  ({z$set(b = 'f'); z$get(c('a', 'b'))} %==% list(a = 2, b = 'f'))
})

assert('$merge() does not change the object but returns a list of new values', {
  ({z$merge(list(b = 'g')); z$get('b')} %==% 'f')
  (z$merge(list(b = 'g')) %==% list(a = 2, b = 'g'))
})

assert('$get(default=TRUE) returns the initial value', {
  (z$get(default = TRUE) %==% list(a = 1))
})

assert('$get(names) keeps the names if drop = FALSE and one or more names do not exist', {
  (z$get(c('a', 'c')) %==% list(a = 2, c = NULL))
  (z$get('c') %==% NULL)
  (z$get('c', drop = FALSE) %==% list(c = NULL))
})

assert('$delete() deletes keys from the list', {
  z$set(b1 = TRUE, b2 = FALSE); z$delete(c('b1', 'b2'))
  (intersect(c('b1', 'b2'), names(z$get())) %==% character())
})

assert('$restore() restores to the initial value', {
  ({z$restore(); z$get()} %==% list(a = 1))
})

assert('$append() returns appended the chunk option', {
  z$set(d = 1)
  z$append(d = 2)
  (z$get('d') %==% c(1, 2))
})

z = new_defaults()

# named argument in set()
z$set(a = list(b = 2, c = 'qwer'))

assert('a named argument of list in $set() method is not treated as a list of options', {
  (length(z$get()) %==% 1L)
  # unnamed argument in set()
  ({z$restore(); z$set(list(b = 2, c = 'qwer')); z$get()} %==% list(b = 2, c = 'qwer'))
})

library(testit)

tangle_text = function(text, out.format = 'markdown') {
  optk = opts_knit$get()
  opts_knit$set(out.format = out.format)
  on.exit(opts_knit$set(optk), add = TRUE)
  purl(text = text)
}

# Test that when there is no pattern specified, no pattern found, and the file
# is tangled, purl() returns an empty string rather than the original text.
# https://github.com/yihui/knitr/pull/1660
assert('when LP pattern not found in the input, purl() returns an empty string', {
  (tangle_text('There is no code.') %==% '')
})

#' Download and convert a theme from eclipsecolorthemes.org to CSS
#'
#' This function uses the \pkg{XML} package to parse the theme as an XML file,
#' then converts to a CSS file using a brew template in the \pkg{knitr} package.
#' The CSS file can be further parsed with \code{knit_theme$get()}, and the
#' result will be ready for \code{knit_theme$set()} to set the highlighting
#' theme.
#' @param id id of theme to save as CSS
#' @return Path to the CSS file converted from the website.
#' @references \url{http://www.eclipsecolorthemes.org/}
#' @author Ramnath Vaidyanathan
#' @seealso \code{\link{knit_theme}}
#' @export
#' @examples ## http://www.eclipsecolorthemes.org/?view=theme&id=1
#' \dontrun{
#' opts_knit$set(out.format = 'latex')
#' (css = eclipse_theme(1))
#' thm = knit_theme$get(css)
#' knit_theme$set(thm)
#' }
eclipse_theme = function(id){
  xml_file = tempfile()
  url = "http://www.eclipsecolorthemes.org/?view=empty&action=download&theme=%s&type=xml"
  download.file(sprintf(url, id), xml_file)
  xml_to_css(xml_file)
}


#' Parse XML file of eclipse theme
#'
#' @param xml_file path to xml file of eclipse theme
#' @return return a list containing theme name and syntax highlighting tokens.
#' @noRd
xml_to_list = function(xml_file){
  library(XML)
  doc      = xmlParse(xml_file)
  docname  = xmlAttrs(xmlRoot(doc), 'colorTheme[@name]')['name']
  docname  = make.names(tolower(docname))
  doclist  = xmlToList(doc)
  list(doclist = doclist, docname = docname)
}

#' Convert XML file with eclipse theme to a CSS style file for use by highlight
#'
#' @param xml_file path to xml file of eclipse theme
#' @return saves css file of eclipse theme to the themes folder of knitr
#' @noRd
##  TODO: Parse more formatting variables like font-weight, font-style etc.
xml_to_css = function(xml_file){
  xml_list   = xml_to_list(xml_file)
  doccolors  = lapply(xml_list$doclist, "[", 'color')
  style      = xml_list$docname
  css_file   = file.path(tempdir(), sprintf("%s.css", style))
  template   = system.file('themes', 'eclipse.brew', package = 'knitr')
  with(doccolors, knit(template, css_file))
  message('Theme ', style, ' saved to ', css_file)
  css_file
}

options(digits = 4)
rnorm(20)
fit = lm(dist ~ speed, data = cars)
b = coef(fit)

summary(fit)$coefficients

par(mar=c(4, 4, 1, .1))
plot(cars, pch = 20)
abline(fit, col = 'red')

print(citation('knitr'), style = 'html')

library(testit)

assert('validation of sql queries works', {
  (!is_sql_update_query('SELECT 1'))
  (!is_sql_update_query('SELECT * FROM foo'))
  (!is_sql_update_query(' SELECT 1'))
  (!is_sql_update_query('\nSELECT 1'))
  (!is_sql_update_query('\tSELECT 1'))
  (!is_sql_update_query(c('-- Some SQL', 'SELECT 1')))
  (!is_sql_update_query(c('/* ', '   Some SQL', '*/', 'SELECT 1')))
  (!is_sql_update_query(c('   /* ', '      Some SQL', '   */', 'SELECT 1')))

  (is_sql_update_query('UPDATE foo SET a=1'))
  (is_sql_update_query(' UPDATE foo SET a=1'))
  (is_sql_update_query('\n\nUPDATE foo SET a=1'))
  (is_sql_update_query('\tUPDATE foo SET a=1'))
  (is_sql_update_query('DELETE FROM foo'))
  (is_sql_update_query('INSERT INTO foo values(1)'))

  (is_sql_update_query(c('-- SELECT 1', 'INSERT INTO foo values(1)')))

  (is_sql_update_query(c('/*SELECT 1*/', '   INSERT INTO foo values(1)')))

  (is_sql_update_query(c(
    '/*', '   Insert records into table', '*/', '', '   INSERT INTO foo values(1)'
  )))

  (is_sql_update_query('update foo set a=1'))
  (is_sql_update_query('delete from foo'))
  (is_sql_update_query('insert into foo values(1)'))
})

library(testit)

op = options(digits = 3, scipen = 0, knitr.digits.signif = TRUE)

assert('format_sci() uses correct number of significant digits', {
  (format_sci(1) %==% '1')
  (format_sci(0) %==% '0')
  (format_sci(3.1415e2) %==% '314')
  (format_sci(3.1415) %==% '3.14')
})

options(op)

op = options(digits = 14, scipen = 0, knitr.digits.signif = TRUE)
assert('format_sci() prints numerics at maximum number of significant digits', {
  (format_sci(3.14159265358979) %==% '3.1415926535898')
})
options(op)

op = options(digits = 4, scipen = 0)

assert('format_sci() turns numbers into scientific notations', {
  (format_sci(c(1.84e8, 1e5, 2.34e3)) %==% c('1.84\\times 10^{8}', '10^{5}', '2340'))
  (format_sci(1.23456789 * 10^-5) %==% '1.2346\\times 10^{-5}')
  (format_sci(10^-5) %==% '10^{-5}')
  (format_sci(10^5) %==% '10^{5}')
  (format_sci(-10^-5) %==% '-10^{-5}')
  (format_sci(-10^5) %==% '-10^{5}')
  (format_sci(9.87654e6, 'html') %==% '9.8765 &times; 10<sup>6</sup>')
  (format_sci(9.87654e6, 'rst') %==% '9.8765 |times| 10 :sup:`6`')
  (format_sci(letters) %==% letters)
  (format_sci(NA_real_) %==% NA_character_)
})

assert('format_sci() coerces non-numeric and non-double values to characters', {
  (format_sci(Sys.Date()) == as.character(Sys.Date()))
  (format_sci(1000000L) == '1000000')
})

# https://github.com/yihui/knitr/issues/1625
assert('format_sci() does not convert roman numerals to arabic numerals', {
  (format_sci(as.roman(c(1, 4, 7, 33, 100))) %==% c('I', 'IV', 'VII', 'XXXIII', 'C'))
})

assert('format_sci() for Rnw does not add \\ensuremath{} at all', {
  (!grepl('[\\]ensuremath', format_sci(c(1e4, 1.2345e10, 2 * pnorm(-(3:4)), -Inf))))
})

assert('the inline hook for Rnw applies \\ensuremath{} correctly', {
  (.inline.hook.tex(1e4) == '\\ensuremath{10^{4}}')
  (.inline.hook.tex(-Inf) == '\\ensuremath{-\\infty{}}')
  (.inline.hook.tex(-1.23) == '-1.23')
  (.inline.hook.tex(c(1.2345e10, 2 * pnorm(-(3:4)))) ==
    "\\ensuremath{1.2345\\times 10^{10}}, 0.0027, \\ensuremath{6.3342\\times 10^{-5}}")
})

assert('Infinity and NaN are formatted correctly', {
  (format_sci(-Inf) %==% '-\\infty{}')
  (format_sci(-Inf, 'html') %==% '-&infin;')
  (format_sci(-Inf, 'rst') %==% '-Inf')
  (format_sci(NaN) %==% 'NaN')
})

assert('sanitize_fn() warns against spaces in filenames', {
  (has_warning(sanitize_fn('figure/a b')))
})

options(op)

assert('fig_path() sanitizes paths', {
  (sanitize_fn('fig/foo') %==% 'fig/foo')
  (sanitize_fn('figure/a b  c', FALSE) %==% 'figure/a_b__c')
  (sanitize_fn('fig space/a.b') %==% 'fig_space/a.b')
  (sanitize_fn('../c.d') %==% '../c.d')
  (sanitize_fn('./../c..d') %==% './../c..d')
  (sanitize_fn('C:/foo/bar') %==% 'C:/foo/bar')
})

assert('fig_chunk() generates figure filenames for a code chunk', {
  (fig_chunk('foo') %==% 'figure/foo-1')
  (fig_chunk('foo', 'pdf') %==% 'figure/foo-1.pdf')
  (fig_chunk('foo', 'png', 2) %==% 'figure/foo-2.png')
  (fig_chunk('foo', 'svg', 1:5) %==% sprintf('figure/foo-%d.svg', 1:5))
  (fig_chunk('foo', fig.path = 'my_figure/') %==% 'my_figure/foo-1')
  (fig_chunk('foo', '.pdf') %==%'figure/foo-1.pdf')
})

assert('all_figs() generates all figure paths for a code chunk', {
  opts = list(fig.path = 'abc/', label = 'foo', fig.num = 3)
  (all_figs(opts, '.svg') %==% sprintf('abc/foo-%d.svg', 1:3))
  (all_figs(opts, c('png', 'pdf'))  %==% apply(
    expand.grid(1:3, c('.png', '.pdf')), 1, function(x) {
      paste0(c('abc/foo-', x), collapse = '')
    }
  ))
})

assert('escape_latex() escapes special LaTeX characters', {
  (escape_latex('# $ % & ~ _ ^ \\ { }') %==%
     '\\# \\$ \\% \\& \\textasciitilde{} \\_ \\textasciicircum{} \\textbackslash{} \\{ \\}'
  )
  (escape_latex('a b', spaces = TRUE) %==% 'a b')
  (escape_latex('a   b', spaces = TRUE) %==% 'a \\ \\ b')
})

assert('indent_block() works when the first element is empty (#790)', {
  (indent_block(c('', 'a')) %==% c('    ', '    a'))
  (indent_block(c('', '')) %==% c('    ', '    '))
})

assert('current_input() returns NULL by default', {
  (is.null(current_input()))
  (suppressWarnings(is.null(current_input(TRUE))))
})

assert('color_def() generates LaTeX code to define a color variable', {
  (color_def(NA) %==% '')
  (color_def('red') %==% '\\definecolor{shadecolor}{rgb}{1, 0, 0}')
  (color_def('#00ff00') %==% '\\definecolor{shadecolor}{rgb}{0, 1, 0}')
  (color_def('.5,.6,.7', 'fgcolor') %==% '\\definecolor{fgcolor}{rgb}{.5, .6, .7}')
})

opts = list(
  fig.cap = 'Figure "caption" <>.', fig.lp = 'Fig:', label = 'foo'
)
assert('.img.cap() generates the figure caption and alt attribute', {
  (.img.cap(list(fig.cap = NULL), FALSE) %==% "")
  (.img.cap(opts, FALSE) %==% opts$fig.cap)
  (.img.cap(opts, TRUE, TRUE)  %==% 'Figure "caption" &lt;&gt;.')

  opts$fig.alt = 'Figure "alternative text" <>.'

  (.img.cap(opts, TRUE, TRUE)  %==% 'Figure "alternative text" &lt;&gt;.')
  (.img.cap(opts, FALSE)  %==% opts$fig.cap)

  (.img.cap(list(fig.cap = '', fig.alt = "alt"), FALSE) %==% "")
  (.img.cap(list(fig.cap = '', fig.alt = "alt"), TRUE) %==% "alt")
})

z = as.strict_list(list(a = 1, aa = 2, bbb = 3))
assert('as.strict_list() does not allow partial matching', {
  (z$b %==% NULL)
  (z$bbb %==% 3)
})

out = c('*hello*', raw_output('<special>content</special> *protect* me!'), '*world*')
pre = extract_raw_output(out)
pre$value = gsub('[*]([^*]+)[*]', '<em>\\1</em>', pre$value)  # think this as Pandoc conversion
# raw output was protected from the conversion (e.g. *protect* was not converted)
assert('restore_raw_output() restores raw output', {
  (restore_raw_output(pre$value, pre$chunks) %==%
    '<em>hello</em>\n<special>content</special> *protect* me!\n<em>world</em>')
})

assert('raw_block() returns a raw attribute block for Pandoc', {
  (c(raw_latex('\\emph{x}')) %==% '\n```{=latex}\n\\emph{x}\n```\n')
  (c(raw_html('<i>foo</i>')) %==% '\n```{=html}\n<i>foo</i>\n```\n')
})

assert('block_class() turns a character vector into Pandoc attributes for code block classes', {
  (block_class(NULL) %==% NULL)
  (block_class('a') %==% 'a')
  (block_class('a', '.b') %==% '.a')
  (block_class('a b') %==% c('.a', '.b'))
  (block_class(c('a', 'b')) %==% c('.a', '.b'))
})

assert('when collapse is TRUE, class.* and attr.* become NULL except for class.source and attr.source', {
  keys = unlist(lapply(
    c('class.', 'attr.'), paste0, c('source', 'output', 'message', 'warning', 'error')
  ))
  keys_source = c('class.source', 'attr.source')
  opts = fix_options(opts_chunk$merge(c(setNames(as.list(keys), keys), list(collapse = TRUE))))
  (opts[keys_source] %==% as.list(setNames(keys_source, keys_source)))
  (!any(names(opts) %in% setdiff(keys, keys_source)))
  rm(keys, keys_source, opts)
})

assert('default strip.white is conditional to collapse', {
  opts = opts_chunk$get(default = TRUE)
  (fix_options(opts)$strip.white %==% TRUE)
  opts$collapse = TRUE
  (fix_options(opts)$strip.white %==% FALSE)
  opts$strip.white = TRUE
  (fix_options(opts)$strip.white %==% TRUE)
  rm(opts)
})

assert('pandoc_to gets the current Pandoc format', {
  opts = opts_knit$get(c('rmarkdown.pandoc.to', 'rmarkdown.pandoc.from'))
  opts_knit$delete(c("rmarkdown.pandoc.from", "rmarkdown.pandoc.to"))
  (pandoc_from() %==% 'markdown')
  (pandoc_to() %==% NULL)
  opts_knit$set(rmarkdown.pandoc.from = 'gfm', rmarkdown.pandoc.to = 'docx')
  (pandoc_from() %==% 'gfm')
  (pandoc_to() %==% 'docx')
  (pandoc_to('docx'))
  (!pandoc_to('pptx'))
  (pandoc_to(c('docx', 'pptx')))
  opts_knit$set(opts)
})

assert('comment_out() add prefix and newlines if asked', {
  (comment_out("a") %==% "## a\n")
  (comment_out("ab cd") %==% "## ab cd\n")
  (comment_out("ab cd\n") %==% "## ab cd\n")
  (comment_out("") %==% "## \n")
  (comment_out("\n") %==% "## \n")
  (comment_out(c("a", "b")) %==% c("## a\n", "## b\n"))
  (comment_out(c("a", "b"), which = 2) %==% c("a\n", "## b\n"))
  (comment_out("a", newline = FALSE) %==% "## a")
  (comment_out("a", prefix = "$") %==% "$ a\n")
  (comment_out("a", prefix = NULL) %==% "a\n")
})

assert('remove_urls() removes the link', {
  (remove_urls(c('[a](b)', '[a b](c)')) %==% c('a', 'a b'))
  (remove_urls('a [b](c) d.') %==% 'a b d.')
  (remove_urls('a [b](c) d [e f+g](h) i.') %==% 'a b d e f+g i.')
  (remove_urls('a [b](c) `[d](e)` f.') %==% 'a b `[d](e)` f.')
})

assert('options using `-` are converted to `.` and default value replaced', {
  opts = opts_chunk$merge(list('fig-cap' = 'caption', 'out-width' = 300))
  (is.null(dot_names(opts)[['fig-cap']]))
  (is.null(dot_names(opts)[['out-width']]))
  (dot_names(opts)[['fig.cap']] %==% 'caption')
  (dot_names(opts)[['out.width']] %==% 300)
  rm(opts)
})

assert('fig.format and fig.dpi', {
  opts = opts_chunk$merge(list('fig-format' = 'svg', 'fig-dpi' = 750))
  (is.null(dot_names(opts)[['fig-format']]))
  (is.null(dot_names(opts)[['fig-dpi']]))
  (dot_names(opts)[['dev']] %==% 'svg')
  (dot_names(opts)[['dpi']] %==% 750)
  rm(opts)
})

assert('options using `.` are converted to `-` and default value replaced', {
  opts = opts_chunk$merge(list('fig.cap' = 'caption', 'out.width' = 300))
  (is.null(dash_names(opts)[['fig.cap']]))
  (is.null(dash_names(opts)[['out.width']]))
  (dash_names(opts)[['fig-cap']] %==% 'caption')
  (dash_names(opts)[['out-width']] %==% 300)
  rm(opts)
})

assert('dev and dpi are convert to fig-format and fig-dpi', {
  opts = opts_chunk$merge(list('dev' = 'svg', 'dpi' = 750))
  (is.null(dash_names(opts)[['dev']]))
  (is.null(dash_names(opts)[['dpi']]))
  (dash_names(opts)[['fig-format']] %==% 'svg')
  (dash_names(opts)[['fig-dpi']] %==% 750)
  rm(opts)
})

library(testit)

rm(list = ls())
z = 5
assert('a list can be used as the parent frame for knit()', {
  # evaluate in a new environment; should create an object in current envir
  (!has_error(with(list(y = 4:8), knit('knit-envir.Rmd', quiet = TRUE))))
})

env = new.env()
env$y = 1:3
knit('knit-envir.Rmd', envir = env, quiet = TRUE)
assert('knit() creates objects in its envir argument', {
  (!exists('asdfqwerzxcv'))
  (exists('asdfqwerzxcv', envir = env))
})

assert('undefined external objects should cause errors', {
  (suppressMessages(has_error(knit('knit-envir.Rmd', quiet = TRUE))))  # y is not found
  (!has_error(with(list(y = letters), knit('knit-envir.Rmd', quiet = TRUE))))
})

file.remove('knit-envir.md')

library(shiny)
addResourcePath('assets', system.file('shiny', 'assets', package = 'knitr'))

# div nbSrc takes source code from div notebook (via Ace editor), and div nbOut
# holds results from knitr
shinyUI(
  bootstrapPage(
    tags$head(
      tags$title('An R Notebook in Shiny'),
      tags$script(src = 'http://ace.ajax.org/build/src-min-noconflict/ace.js',
                  type = 'text/javascript', charset = 'utf-8'),
      tags$link(rel = 'stylesheet', type = 'text/css', href = 'assets/ace-shiny.css'),
      tags$script(src = 'assets/highlight.pack.js', type = 'text/javascript'),
      tags$link(rel = 'stylesheet', type = 'text/css', href = 'assets/tomorrow.css'),
      tags$script(src = 'https://c328740.ssl.cf1.rackcdn.com/mathjax/2.0-latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML',
                  type = 'text/javascript')
    ),
    div(id = 'notebook', title = 'Compile notebook: F4\nInsert chunk: Ctrl+Alt+I',
        paste(c('This is an example taken from the **knitr** package. Press `F4` or `Ctrl+Shift+H` to compile it, and `Ctrl+Alt+I` to insert a code chunk.', '',
                readLines(system.file('examples', 'knitr-minimal.Rmd', package = 'knitr'))), collapse = '\n')),
    tags$textarea(id = 'nbSrc', style = 'display: none;'),
    tags$script(src = 'assets/ace-shiny.js', type = 'text/javascript'),
    htmlOutput('nbOut'),
    div(id = 'proxy', submitButton('Knit Notebook'))
  )
)

#!/usr/bin/env Rscript

if (!nzchar(Sys.which('lyx')) || system('lyx -version') != 0L) q()

call_lyx = function(file) {
  res = sapply(sprintf('lyx -e %s %s', c('knitr', 'r', 'pdf2'), file), 
               system, USE.NAMES = FALSE)
  unlink(sub('\\.lyx$', '.R', file))
  stopifnot(identical(res, integer(3L)))
}

for (i in list.files(pattern = '\\.lyx$')) {
  message(i)
  call_lyx(i)
  flush.console()
}

call_knit = function(cmd) {
  stopifnot(identical(system(cmd), 0L))
}
for (i in list.files(pattern = '\\.Rmd')) {
  message(i)
  cmd = if (i == 'knitr-minimal.Rmd') {
    sprintf("Rscript -e 'library(knitr);opts_knit$set(base.url=\"http://animation.r-forge.r-project.org/ideas/\");knit(\"%s\")'", i)
  } else {
    sprintf('knit %s', i)
  }
  call_knit(cmd)
  flush.console()
}

call_knit(sprintf("Rscript -e 'library(knitr);spin(\"knitr-spin.R\")'"))

setwd('child')
for (i in c('knitr-main.Rnw', 'knitr-parent.Rnw')) {
  call_knit(sprintf('knit %s', i))
}
unlink('*.tex')
call_knit(sprintf('knit %s', 'knitr-main.Rmd'))
setwd('..')

file.remove(c('knitr-slides.html', 'child/knitr-main.html'))

#' A wrapper for rst2pdf
#'
#' Convert reST to PDF using \command{rst2pdf} (which converts from rst to PDF
#' using the ReportLab open-source library).
#' @param input The input rst file.
#' @param command Character string giving the path of the
#'   \command{rst2pdf} program. If the program is not in your PATH, the full path has to be
#'   given here.
#' @param options Extra command line options, e.g. \code{'-v'}.
#' @author Alex Zvoleff and Yihui Xie
#' @return An input file \file{*.rst} will produce \file{*.pdf} and this output
#'   filename is returned if the conversion was successful.
#' @export
#' @seealso \code{\link{knit2pdf}}
#' @references \url{https://github.com/rst2pdf/rst2pdf}
rst2pdf = function(input, command = 'rst2pdf', options = '') {
  out = with_ext(input, 'pdf')
  system2(command, paste(shQuote(input), '-o', shQuote(out), options))
  if (file.exists(out)) out else stop('conversion by rst2pdf failed!')
}

#' Convert various input files to various output files using \code{knit()} and
#' Pandoc
#'
#' Knits the input file and compiles to an output format using Pandoc.
#' @inheritParams knit
#' @param to Character string giving the Pandoc output format to use.
#' @param pandoc_wrapper An R function used to call Pandoc; by default, if
#'   \pkg{rmarkdown} is installed, this uses
#'   \code{rmarkdown::\link[rmarkdown]{pandoc_convert}()}, otherwise it
#'   \code{\link{pandoc}()}.
#' @param ... Options to be passed to the \code{pandoc_wrapper} function.
#' @param encoding Ignored (always assumes UTF-8).
#' @author Trevor L. Davis
#' @return Returns the output of the \code{pandoc_wrapper} function.
#' @export
knit2pandoc = function(
  input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE,
  envir = parent.frame(), to = 'html', pandoc_wrapper = NULL, ..., encoding = 'UTF-8'
) {
  knit_output = knit(input, output, tangle, text, quiet, envir)
  if (!is.null(pandoc_wrapper)) return(pandoc_wrapper(knit_output, to, ...))
  if (!has_package('rmarkdown')) return(pandoc(knit_output, to, ...))
  output = gsub(paste0(file_ext(knit_output), '$'), to, knit_output)
  rmarkdown::pandoc_convert(knit_output, to, output = output, ...)
}

#' Convert Rnw or Rrst files to PDF
#'
#' Knit the input Rnw or Rrst document, and compile to PDF using
#' \code{tinytex::\link[tinytex]{latexmk}()} or \code{\link{rst2pdf}()}.
#' @inheritParams knit
#' @param compiler A character string giving the LaTeX engine used to
#'   compile the tex document to PDF. For an Rrst file, setting \code{compiler} to
#'   \code{'rst2pdf'} will use \code{\link{rst2pdf}} to compile the rst file to
#'   PDF using the ReportLab open-source library.
#' @param ... Options to be passed to \code{tinytex::\link[tinytex]{latexmk}} or
#'   \code{\link{rst2pdf}}.
#' @author Ramnath Vaidyanathan, Alex Zvoleff and Yihui Xie
#' @return The filename of the PDF file.
#' @note The \code{output} argument specifies the output filename to be passed
#'   to the PDF compiler (e.g. a tex document) instead of the PDF filename.
#' @export
#' @examples #' compile with xelatex
#' ## knit2pdf(..., compiler = 'xelatex')
#'
#' #' compile a reST file with rst2pdf
#' ## knit2pdf(..., compiler = 'rst2pdf')
knit2pdf = function(
  input, output = NULL, compiler = NULL, envir = parent.frame(), quiet = FALSE, ...
) {
  out = knit(input, output = output, envir = envir, quiet = quiet)
  owd = setwd(dirname(out)); on.exit(setwd(owd))
  if (is.null(compiler)) {
    compiler = if (grepl('\\.rst$', out)) 'rst2pdf' else 'pdflatex'
  }
  if (identical(compiler, 'rst2pdf')) {
    if (tolower(file_ext(out)) != 'rst')
      stop('for rst2pdf compiler input must be a .rst file')
    rst2pdf(basename(out), ...)
  } else {
    tinytex::latexmk(basename(out), engine = compiler, ...)
  }
  with_ext(out, 'pdf')
}

#' Convert markdown to HTML using knit() and markdownToHTML()
#'
#' This is a convenience function to knit the input markdown source and call
#' \code{\link[markdown]{markdownToHTML}()} in the \pkg{markdown} package to
#' convert the result to HTML.
#' @inheritParams knit
#' @param ... Options passed to \code{\link[markdown]{markdownToHTML}}.
#' @param force_v1 Boolean; whether to force rendering the input document as an R
#'   Markdown v1 document, even if it is for v2.
#' @export
#' @seealso \code{\link{knit}}, \code{\link[markdown]{markdownToHTML}}
#' @return If the argument \code{text} is NULL, a character string (HTML code)
#'   is returned; otherwise the result is written into a file and the filename
#'   is returned.
#' @note The \pkg{markdown} package is for R Markdown v1, which is much less
#'   powerful than R Markdown v2, i.e. the \pkg{rmarkdown} package
#'   (\url{https://rmarkdown.rstudio.com}). To render R Markdown v2 documents to
#'   HTML, please use \code{rmarkdown::render()} instead.
#' @examples # a minimal example
#' writeLines(c("# hello markdown", '```{r hello-random, echo=TRUE}', 'rnorm(5)', '```'), 'test.Rmd')
#' knit2html('test.Rmd')
#' if (interactive()) browseURL('test.html')
#'
#' unlink(c('test.Rmd', 'test.html', 'test.md'))
knit2html = function(input, output = NULL, ..., envir = parent.frame(), text = NULL,
                     quiet = FALSE, encoding = 'UTF-8', force_v1 = FALSE) {
  if (!force_v1 && is.null(text)) {
    signal = if (is_R_CMD_check()) warning2 else stop2
    if (length(grep('^---\\s*$', head(read_utf8(input), 1)))) signal(
      'It seems you should call rmarkdown::render() instead of knitr::knit2html() ',
      'because ', input, ' appears to be an R Markdown v2 document.'
    )
  }
  out = knit(input, text = text, envir = envir, quiet = quiet)
  if (is.null(text)) {
    output = with_ext(if (is.null(output) || is.na(output)) out else output, 'html')
    markdown::markdownToHTML(out, output, encoding = 'UTF-8', ...)
    invisible(output)
  } else markdown::markdownToHTML(text = out, ...)
}

knit2html_v1 = function(...) knit2html(..., force_v1 = TRUE)

#' Knit an R Markdown document and post it to WordPress
#'
#' This function is a wrapper around the \pkg{RWordPress} package. It compiles
#' an R Markdown document to HTML and post the results to WordPress.
#' @param input Filename of the Rmd document.
#' @param title Title of the post.
#' @param ... Other meta information of the post, e.g. \code{categories = c('R',
#'   'Stats')} and \code{mt_keywords = c('knitr', 'wordpress')}, et cetera.
#' @param shortcode A length-2 logical vector: whether to use the shortcode
#'   \samp{[sourcecode lang='lang']}, which can be useful to WordPress.com users
#'   for syntax highlighting of source code and output. The first element
#'   applies to source code, and the second applies to text output. By default,
#'   both are \code{FALSE}.
#' @param action Whether to create a new post, update an existing post, or create a new
#'   page.
#' @param postid If \code{action} is \code{editPost}, the post id \code{postid} must be
#'   specified.
#' @param publish Boolean: publish the post immediately?
#' @inheritParams knit
#' @export
#' @references \url{https://yihui.org/knitr/demo/wordpress/}
#' @author William K. Morris, Yihui Xie, and Jared Lander
#' @note This function will convert the encoding of the post and the title to
#'   UTF-8 internally. If you have additional data to send to WordPress (e.g.
#'   keywords and categories), you may have to manually convert them to the
#'   UTF-8 encoding with the \code{\link{iconv}(x, to = 'UTF-8')} function
#'   (especially when using Windows).
#' @examples # see the reference
knit2wp = function(
  input, title = 'A post from knitr', ..., envir = parent.frame(), shortcode = FALSE,
  action = c('newPost', 'editPost', 'newPage'), postid, publish = TRUE
) {
  out = knit(input, envir = envir); on.exit(unlink(out))
  content = file_string(out)
  content = markdown::markdownToHTML(text = content, fragment.only = TRUE)
  shortcode = rep(shortcode, length.out = 2L)
  if (shortcode[1]) content = gsub(
    '<pre><code class="([[:alpha:]]+)">(.+?)</code></pre>',
    '[sourcecode language="\\1"]\\2[/sourcecode]', content
  )
  content = gsub(
    '<pre><code( class="no-highlight"|)>(.+?)</code></pre>',
    if (shortcode[2]) '[sourcecode]\\2[/sourcecode]' else '<pre>\\2</pre>', content
  )

  content = enc2utf8(content)
  title = enc2utf8(title)

  # figure out if we are making a newPost or overwriting an existing post
  action = match.arg(action)

  # build a list of arguments to be fed into either newPost or editPost
  # the first argument is the content, which itself is a list containing
  #     description
  #     title
  #     ...
  # then there is the publish argument
  WPargs = list(content = list(description = content, title = title, ...), publish = publish)

  # if we are editing the post, also include the argument for postid
  if (action == "editPost") WPargs = c(postid = postid, WPargs)

  do.call('library', list(package = 'RWordPress', character.only = TRUE))
  do.call(action, args = WPargs)
}

#' Watch an input file continuously and knit it when it is updated
#'
#' Check the modification time of an input file continously in an infinite loop.
#' Whenever the time indicates the file has been modified, call a function to
#' recompile the input file.
#'
#' This is actually a general function not necessarily restricted to
#' applications in \pkg{knitr}. You may specify any \code{compile} function to
#' process the \code{input} file. To stop the infinite loop, press the
#' \samp{Escape} key or \samp{Ctrl + C} (depending on your editing environment
#' and operating system).
#' @param input An input file path, or a character vector of mutiple input file paths.
#' @param compile A function to compile the \code{input} file. This could be e.g.
#'   \code{\link{knit}} or \code{\link{knit2pdf}}, depending on the input file
#'   and the output you want.
#' @param interval A time interval to pause in each cycle of the infinite loop.
#' @param ... Other arguments to be passed to the \code{compile} function.
#' @export
#' @examples # knit_watch('foo.Rnw', knit2pdf)
#'
#' # knit_watch('foo.Rmd', rmarkdown::render)
knit_watch = function(input, compile = knit, interval = 1, ...) {
  mtime = function(...) file.info(...)[, 'mtime']
  last_time = mtime(input)
  updated = function() {
    this_time = mtime(input)
    on.exit(last_time <<- this_time, add = TRUE)
    this_time > last_time
  }
  for (f in input) compile(f, ...)
  while (TRUE) {
    for (f in input[updated()]) compile(f, ...)
    Sys.sleep(interval)
  }
}

## adapted from Hadley's decumar: https://github.com/hadley/decumar

# split input document into groups containing chunks and other texts
# (may contain inline R code)
split_file = function(lines, set.preamble = TRUE, patterns = knit_patterns$get()) {
  n = length(lines)
  chunk.begin = patterns$chunk.begin; chunk.end = patterns$chunk.end
  if (is.null(chunk.begin) || is.null(chunk.end))
    return(list(parse_inline(lines, patterns)))

  if (!child_mode() && set.preamble) {
    set_preamble(lines, patterns)  # prepare for tikz option 'standAlone'
  }

  markdown_mode = identical(patterns, all_patterns$md)
  groups = divide_chunks(lines, chunk.begin, chunk.end, markdown_mode)

  if (set.preamble)
    knit_concord$set(inlines = sapply(groups, length)) # input line numbers for concordance

  # parse 'em all
  lapply(seq_along(groups), function(i) {
    knit_concord$set(block = i)
    g = groups[[i]]
    block = grepl(chunk.begin, g[1])
    if (!set.preamble && !parent_mode()) {
      return(if (block) '' else g) # only need to remove chunks to get pure preamble
    }
    if (block) {
      n = length(g)
      # remove the optional chunk footer
      if (n >= 2 && grepl(chunk.end, g[n])) g = g[-n]
      # remove the optional prefix % in code in Rtex mode
      g = strip_block(g, patterns$chunk.code)
      params.src = if (group_pattern(chunk.begin)) {
        extract_params_src(chunk.begin, g[1])
      } else ''
      parse_block(g[-1], g[1], params.src, markdown_mode)
    } else parse_inline(g, patterns)
  })
}

# divide lines of input into code/text chunks
divide_chunks = function(x, begin, end, md = TRUE) {
  i = group_indices(grepl(begin, x), grepl(end, x), x, md)
  unname(split(x, i))
}

extract_params_src = function(chunk.begin, line) {
  trimws(gsub(chunk.begin, '\\1', line))
}

#' The code manager to manage code in all chunks
#'
#' This object provides methods to manage code (as character vectors) in all
#' chunks in \pkg{knitr} source documents. For example,
#' \code{knitr::knit_code$get()} returns a named list of all code chunks (the
#' names are chunk labels), and \code{knitr::knit_code$get('foo')} returns the
#' character vector of the code in the chunk with the label \code{foo}.
#' @note The methods on this object include the \code{set()} method (i.e., you
#'   could do something like \code{knitr::knit_code$set(foo = "'my precious new
#'   code'")}), but we recommend that you do not use this method to modify the
#'   content of code chunks, unless you are
#'   \href{https://emitanaka.rbind.io/post/knitr-knitr-code/}{as creative as Emi
#'   Tanaka} and know what you are doing.
#' @export
knit_code = new_defaults()

# strip the pattern in code
strip_block = function(x, prefix = NULL) {
  if (!is.null(prefix) && (length(x) > 1)) {
    x[-1L] = sub(prefix, '', x[-1L])
    spaces = min(attr(regexpr("^ *", x[-1L]), "match.length"))
    if (spaces > 0) x[-1L] = substring(x[-1L], spaces + 1)
  }
  x
}

# an object to store chunk dependencies; dep_list$get() is of the form list(foo
# = c('chunk', 'labels', 'that', 'depend', 'on', 'chunk', 'foo'))
dep_list = new_defaults()

# separate params and R code in code chunks
parse_block = function(code, header, params.src, markdown_mode = out_format('markdown')) {
  params = params.src
  engine = 'r'
  # consider the syntax ```{engine, opt=val} for chunk headers
  if (markdown_mode) {
    engine = get_chunk_engine(params)
    params = get_chunk_params(params)
  }
  params = clean_empty_params(params) # rm empty options
  # turn ```{engine} into ```{r, engine="engine"}
  if (tolower(engine) != 'r') {
    params = sprintf('%s, engine="%s"', params, engine)
    params = gsub('^\\s*,\\s*', '', params)
  }

  # for quarto, preserve the actual original params.src and do not remove the engine
  if (!is_quarto() || opts_knit$get('tangle')) params.src = params
  params = xfun::csv_options(params)

  # remove indent (and possibly markdown blockquote >) from code
  if (nzchar(spaces <- get_chunk_indent(header))) {
    params$indent = spaces
    code = gsub(sprintf('^%s', spaces), '', code)
    # in case the trailing spaces of the indent string are trimmed on certain
    # lines (e.g. in blockquotes https://github.com/yihui/knitr/issues/1446)
    code = gsub(sprintf('^%s', gsub('\\s+$', '', spaces)), '', code)
  }

  # merge with possible chunk options written as (YAML or CSV) metadata in
  # chunk, and remove metadata from code body
  parts = partition_chunk(engine, code)
  dup = intersect(names(params), names(parts$options))
  params = merge_list(params, parts$options)
  if (is.null(params$label)) params$label = unnamed_chunk()
  label = params$label
  if (length(dup)) warning(
    "Duplicated chunk option(s) ", paste0("'", dup, "'", collapse = ', '),
    " in both chunk header and pipe comments of the chunk '", label, "'.", call. = FALSE
  )

  code = parts$code
  .knitEnv$labels = c(.knitEnv$labels, label)
  if (length(code) || length(params[['file']]) || length(params[['code']])) {
    if (label %in% names(knit_code$get())) {
      if (identical(getOption('knitr.duplicate.label'), 'allow')) {
        params$label = label = unnamed_chunk(label)
      } else stop(
        "Duplicate chunk label '", label, "', which has been used for the chunk:\n",
        one_string(knit_code$get(label))
      )
    }
    code = as.character(code)
    knit_code$set(setNames(list(structure(code, chunk_opts = params)), label))
  }

  # store dependencies
  if (!is.null(deps <- params$dependson)) {
    deps = sc_split(deps)
    if (is.numeric(deps)) {
      deps[deps < 0] = length(.knitEnv$labels) + deps[deps < 0]
      deps = .knitEnv$labels[deps[deps > 0]]
    }
    if (is.character(deps)) {
      for (i in deps) dep_list$set(setNames(list(c(dep_list$get(i), label)), i))
    } else warning2("Invalid chunk option 'dependson' (must be character or numeric)")
  }

  # for quarto only
  if (is_quarto()) {
    params$original.params.src = params.src
    params$chunk.echo = isTRUE(params[['echo']])
    params$yaml.code = parts$src
    attr(params, 'quarto_options') = c('original.params.src', 'chunk.echo', 'yaml.code')
    # alias 'warning' explicitly set in chunk metadata to the 'message' option
    if (!is.null(parts$options[['warning']])) {
      params$message = parts$options[['warning']]
    }
  }

  structure(class = 'block', list(
    params = params, params.src = params.src, params.chunk = parts$src)
  )
}

get_chunk_indent = function(header) {
  gsub('^([\t >]*).*', '\\1', header)
}

get_chunk_engine = function(params) {
  sub('^([a-zA-Z0-9_]+).*$', '\\1', params)
}

get_chunk_params = function(params) {
  sub('^([a-zA-Z0-9_]+)', '', params)
}

clean_empty_params = function(params) {
  gsub('^\\s*,*\\s*|\\s*,*\\s*$', '', params) # rm empty options
}

# autoname for unnamed chunk
unnamed_chunk = function(prefix = NULL, i = chunk_counter()) {
  if (is.null(prefix)) prefix = opts_knit$get('unnamed.chunk.label')
  paste(prefix, i, sep = '-')
}

# this internal function is still used in RStudio IDE, otherwise can be removed
parse_params = function(params, label = TRUE) {
  res = xfun::csv_options(params)
  if (label && (!is.character(res$label) || identical(res$label, '')))
    res$label = unnamed_chunk()
  res
}

#' Partition chunk options from the code chunk body
#'
#' This is a wrapper function calling \code{xfun::\link[xfun]{divide_chunk}()}
#' under the hood.
#' @export
#' @keywords internal
partition_chunk = function(engine, code) {
  opts = options(xfun.handle_error.loc_fun = get_loc)
  on.exit(options(opts))
  # the code has been moved to the xfun package
  xfun::divide_chunk(engine, code, strict = is_quarto())
}

print_block = function(x) {
  params = x$params
  if (opts_knit$get('verbose')) {
    code = knit_code$get(params$label)
    if (length(code) && !is_blank(code)) {
      cat('\n')
      cat(one_string('  |  ', code), '\n')
    }
  }
}

# extract inline R code fragments (as well as global options)
parse_inline = function(input, patterns) {
  inline.code = patterns$inline.code; inline.comment = patterns$inline.comment
  if (!is.null(inline.comment)) {
    idx = grepl(inline.comment, input)
    # strip off inline code
    input[idx] = gsub(inline.code, '\\1', input[idx])
  }
  input = one_string(input) # merge into one line

  loc = cbind(start = numeric(0), end = numeric(0))
  if (group_pattern(inline.code)) loc = str_locate(input, inline.code)[[1]]
  code1 = code2 = character()
  lines = integer()
  if (nrow(loc)) {
    code = t(str_match(input, inline.code))
    if (NCOL(code) >= 2L) {
      code1 = code[, 1L]
      code2 = apply(code[, -1L, drop = FALSE], 1, paste, collapse = '')
      nl = gregexpr('\n', input, fixed = TRUE)[[1]]
      lines = if (length(nl) > 1 || nl > -1) findInterval(loc, nl) else 0L
    }
  }

  structure(list(
    input = input, location = loc, code = code2, code.src = code1,
    lines = matrix(lines, ncol = 2, nrow = nrow(loc))
  ), class = 'inline')
}

print_inline = function(x) {
  if (opts_knit$get('verbose')) {
    cat('\n')
    if (nrow(x$location)) {
      cat(sprintf('  |  %s  #%s:%s', x$code, x$location[, 1], x$location[, 2]), sep = '\n')
    }
  }
}

#' Read chunks from an external script
#'
#' Chunks can be put in an external script, and this function reads chunks into
#' the current \pkg{knitr} session; \code{read_demo()} is a convenience function
#' to read a demo script from a package.
#'
#' There are two approaches to read external code into the current session: (1)
#' Use a special separator of the from \code{## ---- chunk-label} (at least four
#' dashes before the chunk label) in the script; (2) Manually specify the
#' labels, starting and ending positions of code chunks in the script.
#'
#' The second approach will be used only when \code{labels} is not \code{NULL}.
#' For this approach, if \code{from} is \code{NULL}, the starting position is 1;
#' if \code{to} is \code{NULL}, each of its element takes the next element of
#' \code{from} minus 1, and the last element of \code{to} will be the length of
#' \code{lines} (e.g. when \code{from = c(1, 3, 8)} and the script has 10 lines
#' in total, \code{to} will be \code{c(2, 7, 10)}). Alternatively, \code{from}
#' and \code{to} can be character vectors as regular expressions to specify the
#' positions; when their length is 1, the single regular expression will be
#' matched against the \code{lines} vector, otherwise each element of
#' \code{from}/\code{to} is matched against \code{lines} and the match is
#' supposed to be unique so that the numeric positions returned from
#' \code{grep()} will be of the same length of \code{from}/\code{to}. Note
#' \code{labels} always has to match the length of \code{from} and \code{to}.
#' @param path Path to the R script.
#' @param lines Character vector of lines of code. By default, this is read from
#'   \code{path}.
#' @param labels Character vector of chunk labels (default \code{NULL}).
#' @param from,to Numeric vector specifying the starting/ending line numbers of
#'   code chunks, or a character vector; see Details.
#' @param from.offset,to.offset Offsets to be added to \code{from}/\code{to}.
#' @param roxygen_comments Logical dictating whether to keep trailing
#'   roxygen-style comments from code chunks in addition to whitespace
#' @return As a side effect, code chunks are read into the current session so
#'   that future chunks can (re)use the code by chunk label references. If an
#'   external chunk has the same label as a chunk in the current session, chunk
#'   label references by future chunks will refer to the external chunk.
#' @references \url{https://yihui.org/knitr/demo/externalization/}
#' @note This function can only be used in a chunk which is \emph{not} cached
#'   (chunk option \code{cache = FALSE}), and the code is read and stored in the
#'   current session \emph{without} being executed (to actually run the code,
#'   you have to use a chunk with a corresponding label).
#' @author Yihui Xie; the idea of the second approach came from Peter
#'   Ruckdeschel (author of the \pkg{SweaveListingUtils} package)
#' @export
#' @examples ## put this in foo.R and read_chunk('foo.R')
#'
#' ## ---- my-label ----
#' 1+1
#' lm(y~x, data=data.frame(x=1:10,y=rnorm(10)))
#'
#' ## later you can use <<my-label>>= to reference this chunk
#'
#' ## the 2nd approach
#' code = c("#@@a", '1+1', "#@@b", "#@@a", 'rnorm(10)', "#@@b")
#' read_chunk(lines = code, labels = 'foo') # put all code into one chunk named foo
#' read_chunk(lines = code, labels = 'foo', from = 2, to = 2) # line 2 into chunk foo
#' read_chunk(lines = code, labels = c('foo', 'bar'), from = c(1, 4), to = c(3, 6))
#' # automatically figure out 'to'
#' read_chunk(lines = code, labels = c('foo', 'bar'), from = c(1, 4))
#' read_chunk(lines = code, labels = c('foo', 'bar'), from = "^#@@a", to = "^#@@b")
#' read_chunk(lines = code, labels = c('foo', 'bar'), from = "^#@@a", to = "^#@@b", from.offset = 1, to.offset = -1)
#'
#' ## later you can use, e.g., <<foo>>=
#' knitr::knit_code$get() # use this to check chunks in the current session
#' knitr::knit_code$restore() # clean up the session
read_chunk = function(
  path, lines = read_utf8(path), labels = NULL, from = NULL, to = NULL,
  from.offset = 0L, to.offset = 0L, roxygen_comments = TRUE
) {
  if (!length(lines)) {
    warning('code is empty')
    return(invisible())
  }
  lab = .sep.label
  if (is.null(labels)) {
    if (!group_pattern(lab)) return(invisible())
  } else {
    if (is.null(from)) from = 1L
    if (!is.numeric(from)) from = pattern_index(from, lines)
    if (is.null(to)) to = c(from[-1L] - 1L, length(lines))
    if (!is.numeric(to)) to = pattern_index(to, lines)
    stopifnot(length(labels) == length(from), length(from) == length(to))
    from = from + from.offset; to = to + to.offset
    code = list()
    for (i in seq_along(labels)) {
      code[[labels[i]]] = strip_white(lines[from[i]:to[i]])
    }
    knit_code$set(code)
    return(invisible())
  }
  idx = cumsum(grepl(lab, lines))
  if (idx[1] == 0) {
    idx = c(0, idx); lines = c('', lines)  # no chunk header in the beginning
  }
  groups = unname(split(lines, idx))
  labels = trimws(gsub(lab, '\\3', sapply(groups, `[`, 1)))
  labels = gsub(',.*', '', labels)  # strip off possible chunk options
  code = lapply(groups, strip_chunk, roxygen_comments)
  for (i in which(!nzchar(labels))) labels[i] = unnamed_chunk()
  knit_code$set(setNames(code, labels))
}

#' @rdname read_chunk
#' @param topic,package Name of the demo and the package. See
#'   \code{utils::\link{demo}}.
#' @param ... Arguments passed to \code{\link{read_chunk}}.
#' @export
read_demo = function(topic, package = NULL, ...) {
  paths = list.files(file.path(find.package(package), 'demo'), full.names = TRUE)
  read_chunk(paths[sans_ext(basename(paths)) == topic], ...)
}

# convert patterns to numeric indices in a character vector
pattern_index = function(pattern, text) {
  if (length(pattern) == 1L) {
    idx = grep(pattern, text)
    if (length(idx) == 0L) stop('pattern ', pattern, ' not found')
    return(idx)
  }
  sapply(pattern, function(p) {
    idx = grep(p, text)
    if (length(idx) != 1L) stop('non-unique matches of ', p)
    idx
  })
}

strip_chunk = function(x, roxygen_comments = TRUE) {
  x = x[-1]
  strip_white(x, if (roxygen_comments) is_blank else function(line) {
    is_blank(line) || grepl("^#+'[ ]?", line)
  })
}

# strip lines that are pure white spaces or
# that match the test_strip condition(s)
strip_white = function(x, test_strip = is_blank) {
  if (!length(x)) return(x)
  while (test_strip(x[1])) {
    x = x[-1]; if (!length(x)) return(x)
  }
  while (test_strip(x[(n <- length(x))])) {
    x = x[-n]; if (n < 2) return(x)
  }
  x
}

# (recursively) parse chunk references inside a chunk
parse_chunk = function(x, rc = knit_patterns$get('ref.chunk')) {
  if (length(x) == 0L) return(x)
  x = c(x)  # drop attributes of code (e.g. chunk_opts)
  if (!group_pattern(rc) || length(idx <- grep(rc, x)) == 0) return(x)

  labels = sub(rc, '\\1', x[idx])
  code = knit_code$get()
  i = labels %in% names(code)
  idx = idx[i]; code = code[labels[i]]
  indent = gsub('^(\\s*).*', '\\1', x[idx])
  code = mapply(indent_block, code, indent, SIMPLIFY = FALSE, USE.NAMES = FALSE)

  x = as.list(x)
  x[idx] = lapply(code, function(z) parse_chunk(z, rc))
  unlist(x, use.names = FALSE)
}

# split text lines into groups of code and text chunks
group_indices = function(chunk.begin, chunk.end, lines = NA, is.md = FALSE) {
  in.chunk = FALSE  # whether inside a chunk now
  pattern.end = NA  # the expected chunk end pattern (derived from header)
  b = NA  # the last found chunk header
  g = NA  # group index: odd - text; even - chunk
  fun = function(is.begin, is.end, line, i) {
    if (i == 1) {
      g <<- if (is.begin) {
        in.chunk <<- TRUE
        b <<- i
        0
      } else 1
      return(g)
    }
    # begin of another chunk is found while the previous chunk is not complete yet
    if (in.chunk && is.begin) {
      if (!is.md || match_chunk_begin(pattern.end, line)) {
        g <<- g + 2  # same amount of ` as previous chunk, so should be a new chunk
        if (is.md) b <<- i
      }  # otherwise ignore the chunk header
      return(g)
    }
    if (in.chunk && is.end && match_chunk_end(pattern.end, line, i, b, lines)) {
      in.chunk <<- FALSE
      g <<- g + 1
      return(g - 1)  # don't use incremented g yet; use it in the next step
    }
    if (!in.chunk && is.begin) {
      in.chunk <<- TRUE
      if (is.md) {
        pattern.end <<- sub('(^[\t >]*```+).*', '^\\1\\\\s*$', line)
        b <<- i
      }
      g <<- g + 2 - g%%2  # make sure g is even
    }
    g
  }
  mapply(fun, chunk.begin, chunk.end, lines, seq_along(chunk.begin))
}

match_chunk_begin = function(pattern.end, x, pattern = '^\\1\\\\{') {
  grepl(gsub('^([^`]*`+).*', pattern, pattern.end), x)
}

match_chunk_end = function(pattern, line, i, b, lines) {
  if (is.na(pattern) || grepl(pattern, line)) return(TRUE)
  n = length(lines)
  # if the exact match was not found, look ahead to see if there is another
  # chunk end that is an exact match before the next chunk begin
  if (i < n && length(k <- grep(pattern, lines[(i + 1):n]))) {
    k = k[1]
    if (k == 1) return(FALSE)  # the next line is real chunk end
    # no other chunk headers before the new next exact chunk end
    if (!any(match_chunk_begin(pattern, lines[i + 1:(k - 1)], '^\\1`*\\\\{')))
      return(FALSE)
  }
  stop2(
    'The closing fence on line ', i, ' ("', line, '") in ', current_input(),
    ' does not match the opening fence "',
    gsub('\\^(\\s*`+).*', '\\1', pattern), '" on line ', b, '. You are recommended to ',
    'fix either the opening or closing fence of the code chunk to use exactly ',
    'the same numbers of backticks and same level of indentation (or blockquote). ',
    'See https://yihui.org/en/2021/10/unbalanced-delimiters/ for more info.'
  )
}

#' Get all chunk labels in a document
#'
#' The function \code{all_labels()} returns all chunk labels as a character
#' vector. Optionally, you can specify a series of conditions to filter the
#' labels. The function `all_rcpp_labels()` is a wrapper function for
#' \code{all_labels(engine == 'Rcpp')}.
#'
#' For example, suppose the condition expression is \code{engine == 'Rcpp'}, the
#' object \code{engine} is the local chunk option \code{engine}. If an
#' expression fails to be evaluated (e.g. when a certain object does not exist),
#' \code{FALSE} is returned and the label for this chunk will be filtered out.
#' @param ... A vector of R expressions, each of which should return \code{TRUE}
#'   or \code{FALSE}. The expressions are evaluated using the \emph{local} chunk
#'   options of each code chunk as the environment, which means global chunk
#'   options are not considered when evaluating these expressions. For example,
#'   if you set the global chunk option \code{opts_chunk$set(purl = TRUE)},
#'   \code{all_labels(purl == TRUE)} will \emph{not} return the labels of all
#'   code chunks, but will only return the labels of those code chunks that have
#'   local chunk options \code{purl = TRUE}.
#' @note Empty code chunks are always ignored, including those chunks that are
#'   empty in the original document but filled with code using chunk options
#'   such as \code{ref.label} or \code{code}.
#' @return A character vector.
#' @export
#' @examples # the examples below are meaningless unless you put them in a knitr document
#' all_labels()
#' all_labels(engine == 'Rcpp')
#' all_labels(echo == FALSE && results != 'hide')
#' # or separate the two conditions
#' all_labels(echo == FALSE, results != 'hide')
all_labels = function(...) {
  cond = as.list(match.call())[-1]
  code = knit_code$get()
  labels = names(code)

  if (length(cond) == 0) return(labels)

  params = lapply(code, attr, 'chunk_opts')
  idx = rep_len(TRUE, length(labels))
  for (i in seq_along(cond)) {
    for (j in seq_along(params)) {
      # need tryCatch() because the expression cond[[i]] may trigger an error
      # when any variable is not found, e.g. not all chunks have the engine
      # option when the condition is engine == 'Rcpp'
      try_eval = function(expr) tryCatch(
        eval(expr, envir = params[[j]], enclos = knit_global()),
        error = function(e) FALSE
      )
      if (idx[j]) {
        res = try_eval(cond[[i]])
        # the condition could be evaluated to an expression; see all_rcpp_labels()
        if (is.expression(res)) res = try_eval(res)
        idx[j] = res
      }
    }
  }

  labels[idx]
}

#' @rdname all_labels
#' @export
all_rcpp_labels = function(...) all_labels(expression(engine == 'Rcpp'), ...)

#' Wrap code using the inline R expression syntax
#'
#' This is a convenience function to write the "source code" of inline R
#' expressions. For example, if you want to write \samp{`r 1+1`} literally in an
#' R Markdown document, you may write \samp{`` `r knitr::inline_expr('1+1')`
#' ``}; for Rnw documents, this may be
#' \samp{\verb|\Sexpr{knitr::inline_expr{'1+1'}}|}.
#' @param code Character string of the inline R source code.
#' @param syntax A character string to specify the syntax, e.g. \code{rnw},
#'   \code{html}, or \code{md}. If not specified, this will be guessed from
#'   the knitting context.
#' @return A character string marked up using the inline R code syntax.
#' @export
#' @examples library(knitr)
#' inline_expr('1+1', 'rnw'); inline_expr('1+1', 'html'); inline_expr('1+1', 'md')
inline_expr = function(code, syntax) {
  if (!is.character(code) || length(code) != 1)
    stop('The inline code must be a character string')
  if (!missing(syntax)) pat = syntax else {
    inline = knit_patterns$get('inline.code')
    if (is.null(inline)) stop('inline_expr() must be called in a knitting process')
    pat = NULL
      for (i in names(all_patterns)) {
        if (inline == all_patterns[[i]][['inline.code']]) {
          pat = i; break
        }
      }
  }
  if (is.null(pat)) stop('Unknown document format')
  sprintf(switch(
    pat, rnw = '\\Sexpr{%s}', tex = '\\rinline{%s}', html = '<!--rinline %s -->',
    md = '`r %s`', rst = ':r:`%s`', asciidoc = '`r %s`', textile = '@r %s@',
    stop('Unknown syntax ', pat)
  ), code)
}


#' Convert the in-header chunk option syntax to the in-body syntax
#'
#' This is a helper function for moving chunk options from the chunk header to
#' the chunk body using the new syntax.
#' @param input File path to the document with code chunks to convert.
#' @param output The default \code{NULL} will output to console. Other values
#'   can be a file path to write the converted content into or a function which
#'   takes \code{input} as argument and returns a file path to write into (e.g.,
#'   \code{output = identity} to overwrite the input file).
#' @param type This determines how the in-body options will be formatted.
#'   \code{"mutiline"} (the default, except for \file{qmd} documents, for which
#'   the default is \code{"yaml"}) will write each chunk option on a separate
#'   line. Long chunk option values will be wrapped onto several lines, and you
#'   can use \code{width = 0} to keep one line per option only. \code{"wrap"}
#'   will wrap all chunk options together using
#'   \code{\link[base:strwrap]{base::strwrap}()}. \code{"yaml"} will convert
#'   chunk options to YAML.
#' @param width An integer passed to \code{base::strwrap()} for \code{type =
#'   "wrap"} and \code{type = "multiline"}. If set to \code{0}, deactivate the
#'   wrapping (for \code{type = "multiline"} only).
#' @return A character vector of converted \code{input} when \code{output =
#'   NULL}. The output file path with converted content otherwise.
#' @note Learn more about the new chunk option syntax in
#'   \url{https://yihui.org/en/2022/01/knitr-news/}
#' @section About \pkg{knitr} option syntax:
#'
#' Historical chunk option syntax have chunk option in the chunk header using
#' valid R syntax. This is an example for \verb{.Rmd} document
#' \preformatted{
#' ```\{r, echo = FALSE, fig.width: 10\}
#' ```
#' }
#'
#' New syntax allows to pass option inside the chunk using several variants
#' \itemize{
#' \item Passing options one per line using valid R syntax. This corresponds to \code{convert_chunk_header(type = "multiline")}.
#' \preformatted{
#' ```\{r\}
#' #| echo = FALSE,
#' #| fig.width = 10
#' ```
#' }
#'
#' \item Passing option part from header in-chunk with several line if wrapping is
#' needed. This corresponds to \code{convert_chunk_header(type = "wrap")}
#' \preformatted{
#' ```\{r\}
#' #| echo = FALSE, fig.width = 10
#' ```
#' }
#' \item Passing options key value pairs in-chunk using YAML syntax. Values are no
#' more R expression but valid YAML syntax. This corresponds to
#' \code{convert_chunk_header(type = "yaml")} (not implement yet).
#' \preformatted{```\{r\}
#' #| echo: false,
#' #| fig.width: 10
#' ```
#' }
#' }
#' @examples
#' knitr_example = function(...) system.file('examples', ..., package = 'knitr')
#' # Convert a document for multiline type
#' convert_chunk_header(knitr_example('knitr-minimal.Rmd'))
#' # Convert a document for wrap type
#' convert_chunk_header(knitr_example('knitr-minimal.Rmd'), type = "wrap")
#' # Reduce default wrapping width
#' convert_chunk_header(knitr_example('knitr-minimal.Rmd'), type = "wrap", width = 0.6 * getOption('width'))
#' \dontrun{
#' # Explicitly name the output
#' convert_chunk_header('test.Rmd', output = 'test2.Rmd')
#' # Overwrite the input
#' convert_chunk_header('test.Rmd', output = identity)
#' # Use a custom function to name the output
#' convert_chunk_header('test.Rmd', output = \(f) sprintf('%s-new.%s', xfun::sans_ext(f), xfun::file_ext(f)))
#' }
#' @export
convert_chunk_header = function(
  input, output = NULL, type = c('multiline', 'wrap', 'yaml'),
  width = 0.9 * getOption('width')
) {

  # extract fenced header information
  text = xfun::read_utf8(input)
  ext  = xfun::file_ext(input)
  if (missing(type) && ext == 'qmd') type = 'yaml'  # default to yaml for Quarto
  type = match.arg(type)
  pattern = detect_pattern(text, ext)
  # no code chunk in brew file
  if (pattern == 'brew') return()
  markdown_mode = pattern == 'md'
  chunk_begin = all_patterns[[pattern]]$chunk.begin

  # counter for inserted lines
  nb_added = 0L
  new_text = text
  for (i in grep(chunk_begin, text)) {
    # transform each chunk one by one
    indent = get_chunk_indent(text[i])
    header = extract_params_src(chunk_begin, text[i])
    engine = if (markdown_mode) get_chunk_engine(header) else 'r'
    params = if (markdown_mode) get_chunk_params(header) else header
    # if no params nothing to format
    if (params == '') next
    params2 = clean_empty_params(params)
    params2 = trimws(clean_empty_params(params2))

    # select the correct prefix char (e.g `#|`)
    opt_chars = xfun:::get_option_comment(engine)
    prefix = paste0(indent, opt_chars$start)

    # clean old chunk keeping only engine
    new_text[i + nb_added] = gsub(params, '', text[i], fixed = TRUE)

    # format new chunk
    if (type == 'wrap') {
      # simple line wrapping of R code
      params3 = strwrap(params2, width, prefix = prefix)
    } else if (type == 'multiline') {
      # one option per line of the form `key = value,`
      res = xfun::csv_options(params2)
      params3 = sprintf('%s = %s,', names(res), deparsed_string(res))

      # remove trailing for last element
      last = length(params3)
      params3[last] = gsub(',$', '', params3[last])

      # wrap long single line and add prefix
      params3 = if (width <= 0) paste0(prefix, params3) else {
        strwrap(params3, width, prefix = prefix)
      }
    } else {
      params3 = xfun::csv_options(params2)

      # fix un-evaluated options for yaml by transforming to !expr val
      params3 = lapply(params3, function(x) {
        if (is.symbol(x) || is.language(x)) {
          x = deparse(x, 500L)
          attr(x, 'tag') = '!expr'
        }
        x
      })
      # transform dot option to dash option
      params3 = dash_names(params3)
      # convert to yaml and add prefix
      params3 = strsplit(yaml::as.yaml(
        params3, handlers = list(
          # true / false instead of no
          logical = function(x) {
            x = tolower(x)
            class(x) = 'verbatim'
            x
          },
          # use character with verbatim for no quotes
          # so that integers as kept unchanged (without changing precision)
          # fig.width = 10, should not be fig-width: 10.0
          numeric = function(x) {
            if (length(x) != 1) return(x)
            x2 = as.integer(x)
            if (x2 == x) x2 else x
          }), line.sep = '\n'), '\n')[[1]]
      params3 = paste0(prefix, params3)
    }

    if (nzchar(opt_chars$end)) params3 = paste0(params3, opt_chars$end)

    # insert new chunk header
    new_text = append(new_text, params3, after = i + nb_added)
    nb_added = nb_added + length(params3)
  }

  if (is.null(output)) return(new_text)
  # otherwise write to file
  if (is.function(output)) output = output(input)
  xfun::write_utf8(new_text, output)
  invisible(output)
}

# TODO: when R 4.0.0 is minimal version, switch to deparse1()
deparsed_string = function(exprs) {
  unlist(lapply(exprs, function(x) paste(deparse(x, 500), collapse = ' ')))
}

context('Parser')

test_that('parsing options', {
  ## new syntax
  expect_identical(parse_params('a-s-d,b=TRUE,c=def'), alist(label='a-s-d',b=TRUE,c=def))
  expect_error(parse_params('a,b'))
  expect_error(parse_params('a,b,c=qwer'))
  expect_identical(parse_params('a,opt=c(1,3,5)'),alist(label='a',opt=c(1,3,5)))
  expect_identical(parse_params('label="xx",opt=zz'),alist(label='xx',opt=zz))
  expect_identical(parse_params('label=foo'),alist(label='foo'))
  expect_identical(parse_params('a,b=2,c="qwer",asdf="efg"'),
                   alist(label='a', b=2, c='qwer',asdf='efg'))
  expect_identical(parse_params('2a'), alist(label='2a'))
  expect_identical(parse_params('abc-function,fig.path="foo/bar-"'),
                   alist(label='abc-function', fig.path="foo/bar-"))
})


test_that('parsing inline texts', {
  res = parse_inline(c('aaa \\Sexpr{x}', 'bbb \\Sexpr{NA} and \\Sexpr{1+2}',
                       'another expression \\Sexpr{rnorm(10)}'), all_patterns$rnw)
  expect_identical(res$code, c('x', 'NA', '1+2', 'rnorm(10)'))
  expect_identical(nchar(res$input), 81L)
  # empty inline code is not recognized
  expect_identical(parse_inline('\\Sexpr{}', all_patterns$rnw)$code,
                   character(0))
  # can use > in HTML inline code
  expect_identical(parse_inline('<!--rinline "<a>" -->', all_patterns$html)$code,
                   ' "<a>" ')
})

library(testit)

template = 'The value of a is {{a}}, so a + 1 is {{a + 1}}'
exp = knit_expand(text = template, a = 10)
act = 'The value of a is 10, so a + 1 is 11'
assert('templates use data specified', {
  (exp %==% act)
})

template = 'The value of a is <% a %>, so a + 1 is <% a + 1 %>'
exp = knit_expand(text = template, a = 10, delim = c("<%", "%>"))
act = 'The value of a is 10, so a + 1 is 11'
assert('templates respect custom delimiter pairs', {
  (exp %==% act)
})

template = 'hello $(LETTERS[24]) and $(pi)!'
exp = knit_expand(text = template, delim = c("$(", ")"))
act = "hello X and 3.14159265358979!"
assert('templates respect pypi delimiters', {
  (exp %==% act)
})

template = 'The value of a is <% a %>, so a + 1 is <% a + 1 %>'
assert('error is thrown when delimiter is not a pair', {
  (has_error(knit_expand(text = template, a = 10, delim = '<%')))
})

template = 'The value of a is {{a}}, and b + 1 is {{b + 1}}'
b = -1.21
exp = knit_expand(text = template, a = 10)
act = "The value of a is 10, and b + 1 is -0.21"
assert('templates use data from parent frame', {
  (exp %==% act)
})

library(knitr)
knit_expand(text = 'The value of pi is {{pi}}.')
knit_expand(text = 'The value of a is {{a}}, so a + 1 is {{a+1}}.', a = rnorm(1))
knit_expand(text = 'The area of a circle with radius {{r}} is {{pi*r^2}}', r = 5)

knit_expand(text = 'a is {{a}} and b is {{b}}, with my own pi being {{pi}} instead of {{base::pi}}', a=1, b=2, pi=3)

knit_expand(text = 'I do not like curly braces, so use % with <> instead: a is <% a %>.', a = 8, delim = c("<%", "%>"))

knit_expand(text = 'hello $(LETTERS[24]) and $(pi)!', delim = c("$(", ")"))

knit_expand(text = 'you cannot see the value of x {{x=rnorm(1)}}but it is indeed created: x = {{x}}')
res = knit_expand(text = c(' x | x^2', '{{x=1:5;paste(sprintf("%2d | %3d", x, x^2), collapse = "\n")}}'))
cat(res)

res = knit_expand(text = c('{{i=0;h2=function(x){i<<-i+1;sprintf("<h2>%d. %s</h2>", i, x)} }}<html>',
'{{h2("First Section")}}', '{{h2("Second Section")}}', '{{h2("Conclusion")}}', '</html>'))
cat(res)

src = lapply(names(mtcars)[2:5], function(i) {
knit_expand(text=c("# Regression on {{i}}", '```{r lm-{{i}}}', 'lm(mpg~{{i}}, data=mtcars)', '```', ''))
})
# knit the source
litedown::fuse(unlist(src), 'markdown')

library(testit)

pkgs = c(rownames(installed.packages(priority = 'high')), 'evaluate', 'knitr')
write_bib(pkgs, tempfile(), tweak = FALSE)

assert(
  '& is escaped in title when write_bib(tweak = TRUE)',
  length(grep(' & ', grep(
    '^  title =', capture.output(write_bib(pkgs, tweak = TRUE)), value = TRUE
  ))) == 0
)

#' Encode an image file to a data URI
#'
#' This function takes an image file and uses the \pkg{markdown} package to
#' encode it as a base64 string, which can be used in the \code{img} tag in
#' HTML.
#' @param f Path to the image file.
#' @return The data URI as a character string.
#' @author Wush Wu and Yihui Xie
#' @export
#' @references \url{http://en.wikipedia.org/wiki/Data_URI_scheme}
#' @examples uri = image_uri(file.path(R.home('doc'), 'html', 'logo.jpg'))
#' if (interactive()) {cat(sprintf('<img src="%s" />', uri), file = 'logo.html')
#' browseURL('logo.html') # you can check its HTML source
#' }
image_uri = function(f) markdown:::.b64EncodeFile(f)

# alternative approaches to base64 encoding
image_uri2 = function(f) {
  content = readBin(f, what = 'raw', n = file.info(f)$size)
  paste0('data:', mime_type(f), ';base64,', base64_encode(content))
}

base64_table = c(LETTERS, letters, 0:9, '+', '/')

# base64 encode a raw string
base64_encode = function(raw.string) {
  n = length(s <- as.integer(raw.string))
  res = rep(NA, (n + 2) / 3 * 4)
  i = 0L  # index of res vector
  j = 1L  # index of base64_table
  while (n > 2L) {
    res[i <- i + 1L] = base64_table[s[j] %/% 4L + 1L]
    res[i <- i + 1L] = base64_table[16 * (s[j] %% 4L) + s[j + 1L] %/% 16 + 1L]
    res[i <- i + 1L] = base64_table[4L * (s[j + 1L] %% 16) + s[j + 2L] %/% 64L + 1L]
    res[i <- i + 1L] = base64_table[s[j + 2L] %% 64L + 1L]
    j = j + 3L
    n = n - 3L
  }
  if (n) {
    res[i <- i + 1L] = base64_table[s[j] %/% 4L + 1L]
    if (n > 1L) {
      res[i <- i + 1L] = base64_table[16 * (s[j] %% 4L) + s[j + 1L] %/% 16 + 1L]
      res[i <- i + 1L] = base64_table[4L * (s[j + 1L] %% 16) + 1L]
      res[i <- i + 1L] = '='
    } else {
      res[i <- i + 1L] = base64_table[16 * (s[j] %% 4L) + 1L]
      res[i <- i + 1L] = '='
      res[i <- i + 1L] = '='
    }
  }
  paste(res[!is.na(res)], collapse = '')
}

# lazy man's mime function
mime_type = function(file) {
  ext = tolower(file_ext(file))
  switch(ext, svg = 'image/svg+xml', jpg = 'image/jpeg',
         paste('image', ext, sep = '/'))
}

library(shiny)
library(knitr)
options(device.ask.default = FALSE)

shinyServer(function(input, output) {

  output$nbOut = reactive({
    src = input$nbSrc
    if (length(src) == 0L || src == '') return('Nothing to show yet...')
    owd = setwd(tempdir()); on.exit(setwd(owd))
    opts_knit$set(root.dir = owd)

    paste(knit2html(text = src, fragment.only = TRUE, quiet = TRUE),
          '<script>',
          '// highlight code blocks',
          "$('#nbOut pre code').each(function(i, e) {hljs.highlightBlock(e)});",
          'MathJax.Hub.Queue(["Typeset", MathJax.Hub]); // update MathJax expressions',
          '</script>', sep = '\n')
  })
})

library(testit)

kable2 = function(...) as.character(kable(...))

assert('kable() works on data frames/matrices of one row', {
  (kable2(data.frame(x = 1, y = 1), format = 'pipe') %==%
     c('|  x|  y|', '|--:|--:|', '|  1|  1|'))
})

m = matrix(1:2, nrow = 1, dimnames = list('a', c('x', 'y')))
assert('kable() does not discard row names when there is only one row', {
  (kable2(m) %==% c('|   |  x|  y|', '|:--|--:|--:|', '|a  |  1|  2|'))
})

assert('kable() recycles the align argument correctly', {
  (kable2(m, align = 'c') %==%
     c('|   | x | y |', '|:--|:-:|:-:|', '|a  | 1 | 2 |'))
})

assert('kable() align with strings correctly', {
  (kable2(m, align = c('c', 'r')) %==% kable2(m, align = 'cr'))
})

assert('kable() works on character data frames', {
  (kable2(data.frame(x = 'a')) %==% c('|x  |', '|:--|', '|a  |'))
})

assert("kable() works on NA's", {
  (kable2(data.frame(x = c(NA, FALSE))) %==%
     c('|x     |', '|:-----|', '|NA    |', '|FALSE |'))
})

assert('kable() does not add extra spaces to character columns', {
  (kable2(data.frame(x = c(1.2, 4.87), y = c('fooooo', 'bar')), 'latex') %==% '
\\begin{tabular}{r|l}
\\hline
x & y\\\\
\\hline
1.20 & fooooo\\\\
\\hline
4.87 & bar\\\\
\\hline
\\end{tabular}')
})

assert('kable() escapes LaTeX special characters by default', {
  (kable2(data.frame(x = c('10%', '5%'), col_name = c('3_8', '40_6')), 'latex') %==% '
\\begin{tabular}{l|l}
\\hline
x & col\\_name\\\\
\\hline
10\\% & 3\\_8\\\\
\\hline
5\\% & 40\\_6\\\\
\\hline
\\end{tabular}')
})

assert("kable() doesn't escape LaTeX special characters when escape = FALSE", {
  (kable2(data.frame(x = c('10%', '5%'), col_name = c('3_8', '40_6')), 'latex', escape = FALSE) %==% '
\\begin{tabular}{l|l}
\\hline
x & col_name\\\\
\\hline
10% & 3_8\\\\
\\hline
5% & 40_6\\\\
\\hline
\\end{tabular}'
  )
})

assert('kable() adds {} before [] when booktabs = TRUE', {
  (kable2(data.frame(x = c('[0, 1]', '(1, 2]'), y = c(35, 62)), 'latex', booktabs = TRUE) %==% '
\\begin{tabular}{lr}
\\toprule
x & y\\\\
\\midrule
{}[0, 1] & 35\\\\
(1, 2] & 62\\\\
\\bottomrule
\\end{tabular}'
   )
})

assert('kable(format = "latex", linesep = ...) works', {
  (kable2(data.frame(x = 1:4), 'latex', linesep  = c('', '', '\\midrule')) %==% '
\\begin{tabular}{r}
\\hline
x\\\\
\\hline
1\\\\
2\\\\
3\\\\
\\midrule
4\\\\
\\hline
\\end{tabular}')
})

assert('kable() escapes HTML special characters by default', {
  (kable2(data.frame(x = c('10<>', '5&2'), y = c('3>8', '"40"')), 'html') %==%
     '<table>\n <thead>\n  <tr>\n   <th style="text-align:left;"> x </th>\n   <th style="text-align:left;"> y </th>\n  </tr>\n </thead>\n<tbody>\n  <tr>\n   <td style="text-align:left;"> 10&lt;&gt; </td>\n   <td style="text-align:left;"> 3&gt;8 </td>\n  </tr>\n  <tr>\n   <td style="text-align:left;"> 5&amp;2 </td>\n   <td style="text-align:left;"> &quot;40&quot; </td>\n  </tr>\n</tbody>\n</table>')
})

assert('kable() doesn\'t escape HTML special characters when escape = FALSE', {
  (kable2(data.frame(x = c('10<>', '5&2'), y = c('3>8', '"40"')), 'html', escape = FALSE) %==%
     '<table>\n <thead>\n  <tr>\n   <th style="text-align:left;"> x </th>\n   <th style="text-align:left;"> y </th>\n  </tr>\n </thead>\n<tbody>\n  <tr>\n   <td style="text-align:left;"> 10<> </td>\n   <td style="text-align:left;"> 3>8 </td>\n  </tr>\n  <tr>\n   <td style="text-align:left;"> 5&2 </td>\n   <td style="text-align:left;"> "40" </td>\n  </tr>\n</tbody>\n</table>')
})

assert('kable(digits = vector) works on numeric matrices', {
  (kable2(matrix(c(1.1, 1.2, 2.3, 2.4), 2, dimnames = list(NULL, c('a', 'b'))), digits = c(0, 1)) %==%
     c('|  a|   b|', '|--:|---:|', '|  1| 2.3|', '|  1| 2.4|'))
})

assert('kable() works on matrices with duplicate row names', {
  (kable2(matrix(c(1, 1, 1, 1), ncol = 2, dimnames = list(c('A', 'A'), c('B', 'B')))) %==%
     c('|   |  B|  B|', '|:--|--:|--:|', '|A  |  1|  1|', '|A  |  1|  1|'))
})

assert('kable() works on matrices with NA colname', {
  (kable2(matrix(c(1, 1, 1, 1), ncol = 2, dimnames = list(c('A', NA), c('B', NA)))) %==%
     c('|   |  B| NA|', '|:--|--:|--:|', '|A  |  1|  1|', '|NA |  1|  1|'))
})

# edge cases (should not error)
x1 = matrix(NA, 0, 0)
x2 = matrix(NA, 0, 1)
x3 = matrix(NA, 1, 0)
for (f in c('simple', 'html', 'latex', 'rst')) {
  kable(x1, f)
  kable(x2, f)
  kable(x3, f)
}

colnames(x2) = 'a'
assert('kable(, "pipe") works for a 0 row 1 column matrix', {
  (kable2(x2, 'pipe') %==% c('|a  |', '|:--|'))
})

assert('kable(, "pipe") works for a 1-column matrix without column names', {
  (kable2(matrix(1), 'pipe') %==% c('|   |', '|--:|', '|  1|'))
})

assert('kable(, "simple") generates a pipe table for a 0 row data.frame', {
  (kable2(data.frame(x = character(0), y = integer(0)), 'simple') %==%
     c('|x  |  y|', '|:--|--:|'))
})

assert('kable(, "simple") generates a simple table for a 1-column object', {
  # if the object has column names, indent the table by one space
  (kable2(data.frame(x = 1), 'simple') %==% c('   x', ' ---', '   1'))
  # if it doesn't have column names, generate the usual table (without indent)
  (kable2(matrix(1), 'simple') %==% c('---', '  1', '---'))
})

assert('kable(, "simple", caption = "Table Caption") works for a 1-column matrix', {
  x4 = matrix(1:2, ncol = 1, dimnames = list(NULL, 'a'))
  (kable2(x4, 'simple', caption = 'Table Caption') %==%
    c('Table: Table Caption', '', '   a', ' ---', '   1', '   2'))
})

assert('kable() works on an irregular matrix', {
  (kable2(matrix(list('a', 2, 3, 4), nrow = 2), col.names = c('a', 'b')) %==%
     c('|a  |b  |', '|:--|:--|', '|a  |3  |', '|2  |4  |'))
})

assert('has_rownames() works', {
  (!has_rownames(matrix(1:4, 2)))
  (!has_rownames(iris))
  (has_rownames(mtcars))
  (!has_rownames(as.data.frame(matrix(nrow = 0, ncol = 3))))
})

op = options(knitr.kable.NA = '')
assert('kable() can display NA as empty strings', {
  (kable2(matrix(c(1, NA, 3, 4), nrow = 2), col.names = c('a', 'b')) %==%
     c('|  a|  b|', '|--:|--:|', '|  1|  3|', '|   |  4|'))
})
options(op)

assert('kable() can apply formatting to custom objects', {
  d = tibble::tibble(x = structure(
    'print_me', .to_upper = TRUE, class = 'make_upper'
  ))

  format.make_upper = function(x, ...) {
    xout = unclass(x)
    if (isTRUE(attr(x, '.to_upper'))) {
      xout = toupper(xout)
    }
    as.character(xout)
  }
  registerS3method('format', 'make_upper', format.make_upper, environment(format))

  (kable2(d) %==% c('|x        |', '|:--------|', '|PRINT_ME |'))
})

library(testit)

op = options(device = function(file = NULL, ...) {
  pdf(file, ...)
  dev.control('enable')  # important! otherwise plots get discarded
})

evaluate = evaluate::evaluate
classes = function(x) vapply(x, function(x) class(x)[1], character(1))

# remove the blank plot
assert('blank plots are removed', {
  res = evaluate('layout(t(1:2))')
  (identical(classes(res), 'source'))
})

assert('plots generated by par(), palette() or layout() are removed', {
  res = evaluate('par(mfrow = c(1, 2))\npie(islands)\nbarplot(islands)')
  (identical(classes(res), rep(c('source', 'recordedplot'), c(3, 1))))
  res = evaluate('layout(t(1:2))\npie(islands)\nbarplot(islands)')
  (identical(classes(res), rep(c('source', 'recordedplot'), c(3, 1))))
  res = evaluate('pie(islands)\nbarplot(islands)\npar(mfrow = c(1, 2))')
  res = merge_low_plot(res)
  (identical(classes(res), rep(c('source', 'recordedplot'), length = 5)))
  res = evaluate('pie(islands)\npar(cex.main=1.2)\nbarplot(islands)')
  res = merge_low_plot(res)
  (identical(classes(res), c('source', 'recordedplot')[c(1, 2, 1, 1, 2)]))
  res = evaluate('par(cex.main=1.2)\npalette(c("red","black"))\nbarplot(islands)')
  (identical(classes(res), rep(c('source', 'recordedplot'), c(3, 1))))
})

assert('merge low-level changes', {
  res = evaluate('plot(1)\npoints(1.1, 1.1)')
  (classes(res) %==% rep(c('source', 'recordedplot'), 2))
  (classes(merge_low_plot(res)) %==% rep(c('source', 'recordedplot'), c(2, 1)))
})

assert('captures grid graphics', {
  res = evaluate('library(grid)
    grid.newpage()
    grid.rect(gp=gpar(fill="grey"))
    grid.rect(gp=gpar(fill="red"))')
  (classes(res) %==% c('source', 'recordedplot')[c(1, 1, 1, 2, 1, 2)])
  res = merge_low_plot(res)
  (identical(classes(res), rep(c('source', 'recordedplot'), c(4, 1))))
})

options(op)

# rmarkdown sets dev.args = list(pdf = list(useDingbats = FALSE)) when dev = 'pdf'
if (!has_error({png(); dev.off()})) {
  assert('chunk_device() correctly opens the png device with dev.args', {
    chunk_device(opts_chunk$merge(list(
      dev = 'png', dev.args = list(pdf = list(useDingbats = FALSE))
    )))
    plot(1:10)
    dev.off()
    TRUE
  })
}

if (requireNamespace("ragg", quietly = TRUE) &&
    !has_error({ragg::agg_png(); dev.off()})) {
  assert(
    'chunk_device() correctly opens the ragg::agg_png device with dev.args',
    {
      chunk_device(opts_chunk$merge(list(
        dev = 'ragg_png', dev.args = list(pdf = list(useDingbats = FALSE))
      )))
      plot(1:10)
      dev.off()
      TRUE
    }
  )
  assert(
    'ragg_png_dev correctly handles bg dev.arg into background arg',
    {
      chunk_device(opts_chunk$merge(list(
        dev = 'ragg_png', dev.args = list(bg = "grey")
      )))
      plot(1:10)
      dev.off()
      TRUE
    }
  )
}

# should not error (find `pdf` correctly in grDevices, instead of the one
# defined below)
pdf = function() {}
do.call(pdf_null, list(7, 7))
dev.off()


gen_source = function(x) structure(x, class = 'source')
gen_plotrc = function(x) structure(factor(x), class = c('factor', 'recordedplot'))

assert('fig_before_code() moves plots before code blocks', {
  res = list(
    gen_source(1), gen_plotrc('a'), gen_plotrc('b'), gen_source(2), gen_source(3),
    gen_plotrc('c'), gen_source(4), gen_plotrc('d')
  )
  (fig_before_code(res) %==% res[c(2, 3, 1, 4, 6, 5, 8, 7)])
})

assert('plots are rearrange based on fig.keep & fig.show options', {
  res = list(gen_source(1), gen_source(2))
  (rearrange_figs(res, 'high', NULL, 'asis') %==% res)
  # only one plot to keep
  res = c(evaluate('plot(1)'), list(gen_source(1)))
  (rearrange_figs(res, 'high', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'all', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'last', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'first', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'index', 2, 'asis') %==% res)
  # several plots
  res = c(list(gen_source(1)), evaluate('plot(1)\npoints(1.1, 1.1)'),
          list(gen_plotrc('b'), gen_source(2)))
  (rearrange_figs(res, 'high', NULL, 'asis') %==% res[-3])
  (rearrange_figs(res, 'all', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'all', NULL, 'hold') %==% res[c(1:2, 4, 7, 3, 5, 6)])
  (rearrange_figs(res, 'last', NULL, 'asis') %==% res[c(-3, -5)])
  (rearrange_figs(res, 'first', NULL, 'asis') %==% res[c(-5, -6)])
  (rearrange_figs(res, 'none', NULL, 'asis') %==% res[c(-3, -5, -6)])
  # correspond to options$fig.keep with numeric vector
  (rearrange_figs(res, 'index', 1, 'asis') %==% res[c(-5, -6)])
  (rearrange_figs(res, 'index', c(2, 3), 'asis') %==% res[c(-3)])
  (rearrange_figs(res, 'index', c(2, 3), 'hold') %==% res[c(1:2, 4, 7, 5, 6)])
  (rearrange_figs(res, 'index', c(1, 2, 3), 'asis') %==% res)
})

# should not error when a plot label contains special characters and sanitize=TRUE
if (xfun::loadable('tikzDevice') && Sys.which('pdflatex') != '' &&
    (!is.na(Sys.getenv('CI', NA)) || Sys.getenv('USER') == 'yihui' || !xfun::is_macos())) {
  knit('knit-tikzDevice.Rnw', quiet = TRUE)
  unlink(c('*-tikzDictionary', 'figure', 'knit-tikzDevice.tex'), recursive = TRUE)
}

# https://github.com/yihui/knitr/issues/1166
knit(text = "\\Sexpr{include_graphics('myfigure.pdf', error = FALSE)}", quiet = TRUE)

assert('include_graphics() expands ~', {
  path1 = "~/test.png"
  (!has_warning(include_graphics("img/test.png", error = FALSE)))
  (unclass(suppressWarnings(include_graphics(path1, error = FALSE))) %==% path.expand(path1))
})

with_par = function(expr, ...) {
  # set par
  op = graphics::par(...)
  # reset on exit
  on.exit(graphics::par(op))
  # save changed state
  global.pars = par(no.readonly = TRUE)
  # reset par
  graphics::par(op)
  # simulate what happens when global.par = TRUE by restoring pars
  par2(global.pars)
  # evaluate in this state
  force(expr)
}

assert("par2 correctly handles specific pars", {
  (par2(NULL) %==% NULL)
  # correctly changed
  (with_par(par("col") %==% "red", col = "red"))
  (with_par(par("cex") %==% 2, cex = 2))
  # unchanged
  old = par("fig")
  (with_par(par("fig") %==% old, fig = old / 2))
  old = par("fin")
  (with_par(par("fin") %==% old, fin = old / 2))
  old = par("pin")
  (with_par(par("pin") %==% old, pin = old / 2))
  old = par("usr")
  (with_par(par("usr") %==% old, usr = old / 2))
  old = par("ask")
  (with_par(par("ask") %==% old, ask = !old))
  # Does not work - something else is changing plt when setting everything
  # old = par("plt")
  # (with_par(par("plt") %==% old, plt = old / 2))
})

context('Capture plots')

options(device = function(file = NULL, ...) {
  pdf(file, ...)
  dev.control('enable')  # important! otherwise plots get discarded
})

test_that('remove blank plots', {
  # remove the a blank plot
  res = evaluate('layout(t(1:2))')
  expect_identical(sapply(rm_blank_plot(res), class), 'source')
})

test_that('plots generated by par(), palette() or layout() are removed', {

  res = evaluate('par(mfrow = c(1, 2))\npie(islands)\nbarplot(islands)')
  res = rm_blank_plot(res)
  expect_identical(sapply(res, class), rep(c('source', 'recordedplot'), c(3, 1)))

  res = evaluate('layout(t(1:2))\npie(islands)\nbarplot(islands)')
  res1 = rm_blank_plot(res)
  expect_identical(sapply(res1, class), rep(c('source', 'recordedplot'), c(3, 1)))

  res = evaluate('pie(islands)\nbarplot(islands)\npar(mfrow = c(1, 2))')
  res = merge_low_plot(res)
  expect_identical(sapply(res, class), rep(c('source', 'recordedplot'), length = 5))

  res = evaluate('pie(islands)\npar(cex.main=1.2)\nbarplot(islands)')
  res = merge_low_plot(res)
  expect_identical(sapply(res, class), c('source', 'recordedplot')[c(1, 2, 1, 1, 2)])

  res = evaluate('par(cex.main=1.2)\npalette(c("red","black"))\nbarplot(islands)')
  res = rm_blank_plot(res)
  expect_identical(sapply(res, class), rep(c('source', 'recordedplot'), c(3, 1)))

})

test_that('merge low-level changes', {
  res = evaluate('plot(1)\npoints(1.1, 1.1)')
  expect_identical(sapply(merge_low_plot(res), class),
                   rep(c('source', 'recordedplot'), c(2, 1)))
})

test_that('captures grid graphics', {
  res = evaluate('library(grid)
    grid.newpage()
    grid.rect(gp=gpar(fill="grey"))
    grid.rect(gp=gpar(fill="red"))')
  res = rm_blank_plot(res)
  expect_identical(sapply(res, class), c('source', 'recordedplot')[c(1, 1, 1, 2, 1, 2)])
  res = merge_low_plot(res)
  expect_identical(sapply(res, class), rep(c('source', 'recordedplot'), c(4, 1)))
})

#' All built-in patterns
#'
#' This object is a named list of all built-in patterns.
#' @references Usage: \url{https://yihui.org/knitr/patterns/}
#' @export
#' @seealso \code{\link{knit_patterns}}
#' @examples all_patterns$rnw; all_patterns$html
#'
#' str(all_patterns)
all_patterns = list(
  `rnw` = list(
    chunk.begin = '^\\s*<<(.*)>>=.*$', chunk.end = '^\\s*@\\s*(%+.*|)$',
    inline.code = '\\\\Sexpr\\{([^}]+)\\}', inline.comment = '^\\s*%.*',
    ref.chunk = '^\\s*<<(.+)>>\\s*$', header.begin = '(^|\n)\\s*\\\\documentclass[^}]+\\}',
    document.begin = '\\s*\\\\begin\\{document\\}'),

  `brew` = list(inline.code = '<%[=]{0,1}\\s+([^%]+)\\s+[-]*%>'),

  `tex` = list(
    chunk.begin = '^\\s*%+\\s*begin.rcode\\s*(.*)', chunk.end = '^\\s*%+\\s*end.rcode',
    chunk.code = '^\\s*%+', ref.chunk = '^%+\\s*<<(.+)>>\\s*$',
    inline.comment = '^\\s*%.*', inline.code = '\\\\rinline\\{([^}]+)\\}',
    header.begin = '(^|\n)\\s*\\\\documentclass[^}]+\\}',
    document.begin = '\\s*\\\\begin\\{document\\}'),

  `html` = list(
    chunk.begin = '^\\s*<!--\\s*begin.rcode\\s*(.*)',
    chunk.end = '^\\s*end.rcode\\s*-->', ref.chunk = '^\\s*<<(.+)>>\\s*$',
    inline.code = '<!--\\s*rinline(.+?)-->', header.begin = '\\s*<head>'),

  `md` = list(
    chunk.begin = '^[\t >]*```+\\s*\\{([a-zA-Z0-9_]+( *[ ,].*)?)\\}\\s*$',
    chunk.end = '^[\t >]*```+\\s*$',
    ref.chunk = '^\\s*<<(.+)>>\\s*$', inline.code = '(?<!(^|\n)``)`r[ #]([^`]+)\\s*`'),

  `rst` = list(
    chunk.begin = '^\\s*[.][.]\\s+\\{r(.*)\\}\\s*$',
    chunk.end = '^\\s*[.][.]\\s+[.][.]\\s*$', chunk.code = '^\\s*[.][.]',
    ref.chunk = '^\\.*\\s*<<(.+)>>\\s*$', inline.code = ':r:`([^`]+)`'),

  `asciidoc` = list(
    chunk.begin = '^//\\s*begin[.]rcode(.*)$', chunk.end = '^//\\s*end[.]rcode\\s*$',
    chunk.code = '^//+', ref.chunk = '^\\s*<<(.+)>>\\s*$',
    inline.code = '`r +([^`]+)\\s*`|[+]r +([^+]+)\\s*[+]',
    inline.comment = '^//.*'),

  `textile` = list(
    chunk.begin = '^###[.]\\s+begin[.]rcode(.*)$',
    chunk.end = '^###[.]\\s+end[.]rcode\\s*$',
    ref.chunk = '^\\s*<<(.+)>>\\s*$',
    inline.code = '@r +([^@]+)\\s*@',
    inline.comment = '^###[.].*')
)

.sep.label = '^(#|--)+\\s*(@knitr|----+)(.*?)-*\\s*$'  # pattern for code chunks in an R script

# initial pattern list
.pat.init = list(
  chunk.begin = NULL, chunk.end = NULL, chunk.code = NULL, inline.code = NULL,
  global.options = NULL, input.doc = NULL, ref.chunk = NULL,
  header.begin = NULL, document.begin = NULL
)

#' Patterns to match and extract R code in a document
#'
#' Patterns are regular expressions and will be used in functions like
#' \code{\link[base]{grep}} to extract R code and chunk options. The object
#' \code{knit_patterns} controls the patterns currently used; see the references
#' and examples for usage.  All built-in patterns are available in the list
#' \link{all_patterns}.
#'
#' @seealso \code{\link{all_patterns}}
#' @references Usage: \url{https://yihui.org/knitr/objects/}
#'
#' Components in \code{knit_patterns}: \url{https://yihui.org/knitr/patterns/}
#' @export
#' @examples library(knitr)
#' opat = knit_patterns$get() # old pattern list (to restore later)
#'
#' apats = all_patterns  # a list of all built-in patterns
#' str(apats)
#' knit_patterns$set(apats[['rnw']]) # set pattern list from apats
#'
#' knit_patterns$get(c('chunk.begin', 'chunk.end', 'inline.code'))
#'
#' # a customized pattern list; has to empty the original patterns first!
#' knit_patterns$restore()
#' # we may want to use this in an HTML document
#' knit_patterns$set(list(chunk.begin = '<!--helloR\\s+(.*)', chunk.end = '^byeR-->'))
#' str(knit_patterns$get())
#'
#' knit_patterns$set(opat)  # put the old patterns back
knit_patterns = new_defaults(.pat.init)

# in LaTeX, may need to put this \newcommand{\rinline}[1]{R output}

# convenience functions
set_pattern = function(type) {
  knit_patterns$restore(all_patterns[[type]])
}

#' Set regular expressions to read input documents
#'
#' These are convenience functions to set pre-defined pattern lists (the syntax
#' to read input documents). The function names are built from corresponding
#' file extensions, e.g. \code{pat_rnw()} can set the Sweave syntax to read Rnw
#' documents.
#' @rdname pat_fun
#' @return The patterns object \code{\link{knit_patterns}} is modified as a side
#'   effect.
#' @export pat_rnw pat_brew pat_tex pat_html pat_md pat_rst pat_asciidoc pat_textile
#' @examples # see how knit_patterns is modified
#' knit_patterns$get(); pat_rnw(); knit_patterns$get()
#'
#' knit_patterns$restore()  # empty the list
pat_rnw = function() set_pattern('rnw')
#' @rdname pat_fun
pat_brew = function() set_pattern('brew')
#' @rdname pat_fun
pat_tex = function() set_pattern('tex')
#' @rdname pat_fun
pat_html = function() set_pattern('html')
#' @rdname pat_fun
pat_md = function() set_pattern('md')
#' @rdname pat_fun
pat_rst = function() set_pattern('rst')
#' @rdname pat_fun
pat_asciidoc = function() set_pattern('asciidoc')
#' @rdname pat_fun
pat_textile = function() set_pattern('textile')


# helper functions

# is it a group pattern?
group_pattern = function(pattern) {
  !is.null(pattern) && grepl('\\(.+\\)', pattern)
}

# automatically detect the chunk patterns
detect_pattern = function(text, ext) {
  if (!missing(ext)) {
    if (ext %in% c('rnw', 'snw', 'stex')) return('rnw')
    if (ext == 'brew') return('brew')
    if (ext %in% c('htm', 'html', 'rhtm', 'rhtml')) return('html')
    if (ext %in% c('rmd', 'rmarkdown', 'markdown', 'md')) return('md')
    if (ext %in% c('rst', 'rrst')) return('rst')
    if (ext %in% c('asciidoc', 'rasciidoc', 'adoc', 'radoc')) return('asciidoc')
  }
  for (p in names(all_patterns)) {
    for (i in c('chunk.begin', 'inline.code')) {
      pat = all_patterns[[p]][[i]]
      if (length(pat) && any(stringr::str_detect(text, pat))) return(p)
    }
  }
  # *.Rtex indicates the tex syntax in knitr, but Rnw syntax in traditional
  # Sweave, which should have been detected in the above loop
  if (!missing(ext) && ext == 'rtex') return('rnw')
  NULL
}

context('Patterns')

test_that('auto detect patterns', {
  expect_identical(detect_pattern('<<>>='), 'rnw')
  expect_identical(detect_pattern('<<foo, bar=TRUE>>='), 'rnw')
  expect_identical(detect_pattern('% begin.rcode'), 'tex')
  expect_identical(detect_pattern('<!--begin.rcode'), 'html')
  expect_identical(detect_pattern('``` {r}'), 'md')
  expect_identical(detect_pattern('asdf', 'rnw'), 'rnw')
})

test_that('does a pattern contain a group?', {
  expect_true(group_pattern('(.*)'))
  expect_false(group_pattern('()'))
  expect_false(group_pattern('abc'))
  expect_false(group_pattern(NULL))
})

test_that('patterns for Rnw', {
  ce = all_patterns$rnw$chunk.end
  expect_identical(grep(ce, '  @'), 1L) # spaces before @
  expect_identical(grep(ce, '@  '), 1L) # spaces after @
  expect_identical(grep(ce, '@ %asdf'), 1L) # comments after %
  expect_identical(grep(ce, '@ asdf'), integer()) # only spaces/comments allowed
  expect_identical(grep(ce, ' @ a% sdf'), integer())
})

## ----show-off, tidy=TRUE------------------------------------------------------
options(digits=4)
rnorm(20)
fit=lm(dist~speed, data=cars)
b=coef(fit)

## ----results='asis', echo=FALSE-----------------------------------------------
knitr::kable(summary(fit)$coefficients, caption='Regression coefficients.')

## ----graphics, fig.cap='A scatterplot with a regression line.'----------------
par(mar=c(4, 4, 1, .1))
plot(cars, pch = 20)
abline(fit, col = 'red')

library(testit)

res = mapply(
  auto_out_name,
  c('abc.Rnw', 'abc.rnw', 'abc.rtex', 'abc.Rmd', 'abc.rhtm', 'abc.Rhtml', 'foo.abc.rhtml'),
  USE.NAMES = FALSE
)
assert('auto_out_name() converts .Rfoo to .foo', {
  (res %==% c('abc.tex', 'abc.tex', 'abc.tex', 'abc.md', 'abc.htm', 'abc.html', 'foo.abc.html'))
})

res = mapply(
  auto_out_name,
  c('abc.tex', '_knit_abc.tex', '_knit_abc.md', 'foo_knit_.html'),
  USE.NAMES = FALSE
)
assert('auto_out_name() converts .tex/.unknown to .txt, and removes _knit_', {
  (res %==% c('abc.txt', 'abc.tex', 'abc.md', 'foo.html'))
  (auto_out_name('foo.bar') %==% 'foo.txt')
})

assert('chunks with include=FALSE should stop on error', {
  (suppressMessages(
    has_error(knit(text = c('<<include=F>>=', '1+"a"', '@'), quiet = TRUE))
  ))
})

assert('tidy=FALSE + eval=numeric should work', {
  ('[1] 1\n[1] 2\n' %==% knit(
    text = c('<<tidy=FALSE, eval=1:2, echo=FALSE, results="asis">>=', '1', '1+', '1', '1', '@'),
    quiet = TRUE
  ))
})

assert('opts_template options are used', {
  ("\n[1] 2\n" %==% knit(
    text = c(
      "<<include=FALSE>>=","opts_template$set(quiet = list(echo=FALSE))", "@",
      "<<results='asis', opts.label='quiet'>>=", "1+1", "@"
    ),
    quiet = TRUE
  ))
})

assert('local chunk options override opts_template', {
  ("\n\\begin{kframe}\n\\begin{alltt}\n\\hlnum{1}\\hlopt{+}\\hlnum{1}\n\\end{alltt}\n\\end{kframe}[1] 2\n" %==% knit(
    text = c(
      "<<include=FALSE>>=","opts_template$set(quiet = list(echo=FALSE))", "@",
      "<<results='asis', opts.label='quiet', echo=TRUE, tidy=FALSE>>=", "1+1", "@"
    ),
    quiet = TRUE
  ))
})

# a shortcut
k = function(text) {
  on.exit(opts_chunk$restore())
  knit(text = c('```{r}', text, '```'), quiet = TRUE, envir = parent.frame())
}
k(c(
  'knit_child(text = "```{r}\nopts_chunk$set(echo=FALSE)\n```", options=list())',
  'x1 = opts_chunk$get("echo")'
))
k(c(
  'knit_child(text = "```{r}\nopts_chunk$set(dpi=200)\n```", options=list(dpi=100))',
  'x2 = opts_chunk$get("dpi")'
))

assert('using knit_child() does not reset global chunk options set in child documents', {
  (x1 %==% FALSE)
  (x2 %==% 200)
})

txt = '%\\documentclass{article}
\\documentclass{article}
\\begin{document}
\\Sexpr{pi}
\\end{document}'
res = strsplit(knit(text = txt, quiet = TRUE), '\n')[[1]]

assert('insert_header_latex() finds the correct \\documentclass{}', {
  (res[1] %==% '%\\documentclass{article}')
})

txt = '\\documentclass{article}
\\begin{document}
\\begin{verbatim}
\\documentclass{article}
\\end{verbatim}
\\Sexpr{pi}
\\end{document}'
res = strsplit(knit(text = txt, quiet = TRUE), '\n')[[1]]

assert('insert_header_latex() finds the correct \\documentclass{}', {
  (res[length(res) - 3] %==% '\\documentclass{article}')
})

assert('knit_meta_add() adds meta objects with the correct number of labels', {
  knit_meta(clean = TRUE)
  knit_meta_add(list(1, 2))
  m = attr(knit_meta_add(list(3, 4), label = c('a', 'b')), 'knit_meta_id')
  knit_meta(clean = TRUE)
  (m %==% c('', '', 'a', 'b'))
})

local({
  out = NULL
  hnd = function(cnd) out <<- cnd

  on.exit(file.remove('knit-handlers.md'))
  knit('knit-handlers.Rmd', quiet = TRUE)

  assert('knit() passes calling handlers to evaluate()', {
    (inherits(out, 'error'))
  })
})

library(testit)

op = options(device = function(file = NULL, ...) {
  pdf(file, ...)
  dev.control('enable')  # important! otherwise plots get discarded
})

evaluate = evaluate::evaluate
classes = evaluate:::classes

# remove the blank plot
assert('blank plots are removed', {
  res = evaluate('layout(t(1:2))')
  (identical(classes(res), 'source'))
})

assert('plots generated by par(), palette() or layout() are removed', {
  res = evaluate('par(mfrow = c(1, 2))\npie(islands)\nbarplot(islands)')
  (identical(classes(res), rep(c('source', 'recordedplot'), c(3, 1))))
  res = evaluate('layout(t(1:2))\npie(islands)\nbarplot(islands)')
  (identical(classes(res), rep(c('source', 'recordedplot'), c(3, 1))))
  res = evaluate('pie(islands)\nbarplot(islands)\npar(mfrow = c(1, 2))')
  res = merge_low_plot(res)
  (identical(classes(res), rep(c('source', 'recordedplot'), length = 5)))
  res = evaluate('pie(islands)\npar(cex.main=1.2)\nbarplot(islands)')
  res = merge_low_plot(res)
  (identical(classes(res), c('source', 'recordedplot')[c(1, 2, 1, 1, 2)]))
  res = evaluate('par(cex.main=1.2)\npalette(c("red","black"))\nbarplot(islands)')
  (identical(classes(res), rep(c('source', 'recordedplot'), c(3, 1))))
})

assert('merge low-level changes', {
  res = evaluate('plot(1)\npoints(1.1, 1.1)')
  (classes(res) %==% rep(c('source', 'recordedplot'), 2))
  (classes(merge_low_plot(res)) %==% rep(c('source', 'recordedplot'), c(2, 1)))
})

assert('captures grid graphics', {
  res = evaluate('library(grid)
    grid.newpage()
    grid.rect(gp=gpar(fill="grey"))
    grid.rect(gp=gpar(fill="red"))')
  (classes(res) %==% c('source', 'recordedplot')[c(1, 1, 1, 2, 1, 2)])
  res = merge_low_plot(res)
  (identical(classes(res), rep(c('source', 'recordedplot'), c(4, 1))))
})

options(op)

# rmarkdown sets dev.args = list(pdf = list(useDingbats = FALSE)) when dev = 'pdf'
if (!has_error({png(); dev.off()})) {
  assert('chunk_device() correctly opens the png device with dev.args', {
    chunk_device(opts_chunk$merge(list(
      dev = 'png', dev.args = list(pdf = list(useDingbats = FALSE))
    )))
    plot(1:10)
    dev.off()
    TRUE
  })
}

if (requireNamespace("ragg", quietly = TRUE) &&
    !has_error({ragg::agg_png(); dev.off()})) {
  assert(
    'chunk_device() correctly opens the ragg::agg_png device with dev.args',
    {
      chunk_device(opts_chunk$merge(list(
        dev = 'ragg_png', dev.args = list(pdf = list(useDingbats = FALSE))
      )))
      plot(1:10)
      dev.off()
      TRUE
    }
  )
  assert(
    'ragg_png_dev correctly handles bg dev.arg into background arg',
    {
      chunk_device(opts_chunk$merge(list(
        dev = 'ragg_png', dev.args = list(bg = "grey")
      )))
      plot(1:10)
      dev.off()
      TRUE
    }
  )
}

# should not error (find `pdf` correctly in grDevices, instead of the one
# defined below)
pdf = function() {}
do.call(pdf_null, list(7, 7))
dev.off()


gen_source = function(x) structure(x, class = 'source')
gen_plotrc = function(x) structure(factor(x), class = c('factor', 'recordedplot'))

assert('fig_before_code() moves plots before code blocks', {
  res = list(
    gen_source(1), gen_plotrc('a'), gen_plotrc('b'), gen_source(2), gen_source(3),
    gen_plotrc('c'), gen_source(4), gen_plotrc('d')
  )
  (fig_before_code(res) %==% res[c(2, 3, 1, 4, 6, 5, 8, 7)])
})

assert('plots are rearrange based on fig.keep & fig.show options', {
  res = list(gen_source(1), gen_source(2))
  (rearrange_figs(res, 'high', NULL, 'asis') %==% res)
  # only one plot to keep
  res = c(evaluate('plot(1)'), list(gen_source(1)))
  (rearrange_figs(res, 'high', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'all', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'last', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'first', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'index', 2, 'asis') %==% res)
  # several plots
  res = c(list(gen_source(1)), evaluate('plot(1)\npoints(1.1, 1.1)'),
          list(gen_plotrc('b'), gen_source(2)))
  (rearrange_figs(res, 'high', NULL, 'asis') %==% res[-3])
  (rearrange_figs(res, 'all', NULL, 'asis') %==% res)
  (rearrange_figs(res, 'all', NULL, 'hold') %==% res[c(1:2, 4, 7, 3, 5, 6)])
  (rearrange_figs(res, 'last', NULL, 'asis') %==% res[c(-3, -5)])
  (rearrange_figs(res, 'first', NULL, 'asis') %==% res[c(-5, -6)])
  (rearrange_figs(res, 'none', NULL, 'asis') %==% res[c(-3, -5, -6)])
  # correspond to options$fig.keep with numeric vector
  (rearrange_figs(res, 'index', 1, 'asis') %==% res[c(-5, -6)])
  (rearrange_figs(res, 'index', c(2, 3), 'asis') %==% res[c(-3)])
  (rearrange_figs(res, 'index', c(2, 3), 'hold') %==% res[c(1:2, 4, 7, 5, 6)])
  (rearrange_figs(res, 'index', c(1, 2, 3), 'asis') %==% res)
})

# should not error when a plot label contains special characters and sanitize=TRUE
if (xfun::loadable('tikzDevice') &&
    (!is.na(Sys.getenv('CI', NA)) || Sys.getenv('USER') == 'yihui' || !xfun::is_macos())) {
  knit('knit-tikzDevice.Rnw', quiet = TRUE)
  unlink(c('*-tikzDictionary', 'figure', 'knit-tikzDevice.tex'), recursive = TRUE)
}

# https://github.com/yihui/knitr/issues/1166
knit(text = "\\Sexpr{include_graphics('myfigure.pdf', error = FALSE)}", quiet = TRUE)

assert('include_graphics() expands ~', {
  path1 = "~/test.png"
  (!has_warning(include_graphics("img/test.png", error = FALSE)))
  (unclass(suppressWarnings(include_graphics(path1, error = FALSE))) %==% path.expand(path1))
})

with_par = function(expr, ...) {
  # set par
  op = graphics::par(...)
  # reset on exit
  on.exit(graphics::par(op))
  # save changed state
  global.pars = par(no.readonly = TRUE)
  # reset par
  graphics::par(op)
  # simulate what happens when global.par = TRUE by restoring pars
  par2(global.pars)
  # evaluate in this state
  force(expr)
}

assert("par2 correctly handles specific pars", {
  (par2(NULL) %==% NULL)
  # correctly changed
  (with_par(par("col") %==% "red", col = "red"))
  (with_par(par("cex") %==% 2, cex = 2))
  # unchanged
  old = par("fig")
  (with_par(par("fig") %==% old, fig = old / 2))
  old = par("fin")
  (with_par(par("fin") %==% old, fin = old / 2))
  old = par("pin")
  (with_par(par("pin") %==% old, pin = old / 2))
  old = par("usr")
  (with_par(par("usr") %==% old, usr = old / 2))
  old = par("ask")
  (with_par(par("ask") %==% old, ask = !old))
  # Does not work - something else is changing plt when setting everything
  # old = par("plt")
  # (with_par(par("plt") %==% old, plt = old / 2))
})

#' Package vignette engines
#'
#' Since R 3.0.0, package vignettes can use non-Sweave engines, and \pkg{knitr}
#' has provided a few engines to compile vignettes via \code{\link{knit}()} with
#' different templates. See \url{https://yihui.org/knitr/demo/vignette/} for
#' more information.
#' @name vignette_engines
#' @note If you use the \code{knitr::rmarkdown} engine, please make sure that
#'   you put \pkg{rmarkdown} in the \samp{Suggests} field of your
#'   \file{DESCRIPTION} file. Also make sure the executables \command{pandoc}
#'   and \command{pandoc-citeproc} can be found by \pkg{rmarkdown} during
#'   \command{R CMD build}. If you build your package from RStudio, this is
#'   normally not a problem. If you build the package outside RStudio, run
#'   \command{which pandoc} and \command{which pandoc-citeproc} in the terminal
#'   (or \code{Sys.which('pandoc')} and \code{Sys.which('pandoc-citeproc')} in
#'   R) to check if \command{pandoc} and \command{pandoc-citeproc} can be found.
#'   If you use Linux, you may make symlinks to the Pandoc binaries in RStudio:
#'   \url{https://rmarkdown.rstudio.com/docs/articles/pandoc.html}, or
#'   install \command{pandoc} and \command{pandoc-citeproc} separately.
#'
#'   When the \pkg{rmarkdown} package is not installed or not available, or
#'   \command{pandoc} or \command{pandoc-citeproc} cannot be found, the
#'   \code{knitr::rmarkdown} engine will fall back to the \code{knitr::knitr}
#'   engine, which uses R Markdown v1 based on the \pkg{markdown} package.
#' @examples library(knitr)
#' vig_list = tools::vignetteEngine(package = 'knitr')
#' str(vig_list)
#' vig_list[['knitr::knitr']][c('weave', 'tangle')]
#' vig_list[['knitr::knitr_notangle']][c('weave', 'tangle')]
#' vig_list[['knitr::docco_classic']][c('weave', 'tangle')]
NULL

vweave = function(file, driver, syntax, encoding = 'UTF-8', quiet = FALSE, ...) {
  {
    on.exit({opts_chunk$restore(); knit_hooks$restore()}, add = TRUE)
    oopts = options(markdown.HTML.header = NULL); on.exit(options(oopts), add = TRUE)
  }
  opts_chunk$set(error = FALSE)  # should not hide errors
  knit_hooks$set(purl = hook_purl)  # write out code while weaving
  (if (grepl('\\.[Rr]md$', file)) knit2html_v1 else if (grepl('\\.[Rr]rst$', file)) knit2pandoc else knit)(
    file, encoding = encoding, quiet = quiet, envir = globalenv(), ...
  )
}

vtangle = function(file, ..., encoding = 'UTF-8', quiet = FALSE) {
  if (is_R_CMD_check()) {
    file = with_ext(file, 'R')
    file.create(file)
    return(file)
  }
  purl(file, encoding = encoding, quiet = quiet, ...)
}

vweave_docco_linear = vweave
body(vweave_docco_linear)[5L] = expression(knit2html(
  file, encoding = encoding, quiet = quiet, envir = globalenv(),
  template = system.file('misc', 'docco-template.html', package = 'knitr'),
  ...
))

vweave_docco_classic = vweave
body(vweave_docco_classic)[5L] = expression(rocco(
  file, encoding = encoding, quiet = quiet, envir = globalenv(), ...
))

vweave_rmarkdown = vweave
body(vweave_rmarkdown)[5L] = expression(rmarkdown::render(
  file, encoding = encoding, quiet = quiet, envir = globalenv(),
  output_dir = getwd(), ...
))

# do not tangle R code from vignettes
untangle_weave = function(vig_list, eng) {
  weave = vig_list[[c(eng, 'weave')]]
  # remove the purl hook from the weave function, but the rmarkdown engine
  # function is different (not vweave_rmarkdown above, but the function(...)
  # defined below in vig_engine('rmarkdown'), and it is not straightforward to
  # remove the purl hook there)
  if (eng != 'knitr::rmarkdown') body(weave)[4L] = expression({})
  weave
}
vtangle_empty = function(file, ...) {
  unlink(with_ext(file, 'R'))
  return()
}

register_vignette_engines = function(pkg) {
  # the default engine
  vig_engine('knitr', vweave, '[.]([rRsS](nw|tex)|[Rr](md|html|rst))$')
  vig_engine('docco_linear', vweave_docco_linear, '[.][Rr](md|markdown)$')
  vig_engine('docco_classic', vweave_docco_classic, '[.][Rr]mk?d$')
  vig_engine('rmarkdown', function(...) if (has_package('rmarkdown')) {
    if (pandoc_available()) {
      vweave_rmarkdown(...)
    } else {
      warning(
        'Pandoc (>= 1.12.3) and/or pandoc-citeproc not available. ',
        'Falling back to R Markdown v1.'
      )
      vweave(...)
    }
  } else {
    warning('The vignette engine knitr::rmarkdown is not available, ',
            'because the rmarkdown package is not installed. Please install it.')
    vweave(...)
  }, '[.][Rr](md|markdown)$')
  # vignette engines that disable tangle
  vig_list = tools::vignetteEngine(package = 'knitr')
  engines  = grep('_notangle$', names(vig_list), value = TRUE, invert = TRUE)
  for (eng in engines) vig_engine(
    paste(sub('^knitr::', '', eng), 'notangle', sep = '_'),
    untangle_weave(vig_list, eng),
    tangle = vtangle_empty,
    pattern = vig_list[[c(eng, 'pattern')]]
  )
}
# all engines use the same tangle and package arguments, so factor them out
vig_engine = function(..., tangle = vtangle) {
  tools::vignetteEngine(..., tangle = tangle, package = 'knitr', aspell = list(
    filter = knit_filter
  ))
}

#' Spell check filter for source documents
#'
#' When performing spell checking on source documents, we may need to skip R
#' code chunks and inline R expressions, because many R functions and symbols
#' are likely to be identified as typos. This function is designed for the
#' \code{filter} argument of \code{\link{aspell}()} to filter out code chunks
#' and inline expressions.
#' @param ifile Filename of the source document.
#' @param encoding Ignored (the file \code{ifile} must be encoded in UTF-8).
#' @return A character vector of the file content, excluding code chunks and
#'   inline expressions.
#' @export
#' @examples library(knitr)
#' knitr_example = function(...) system.file('examples', ..., package = 'knitr')
#' \donttest{
#' if (Sys.which('aspell') != '') {
#' # -t means the TeX mode
#' utils::aspell(knitr_example('knitr-minimal.Rnw'), knit_filter, control = '-t')
#'
#' # -H is the HTML mode
#' utils::aspell(knitr_example('knitr-minimal.Rmd'), knit_filter, control = '-H -t')
#' }}
knit_filter = function(ifile, encoding = 'UTF-8') {
  x = read_utf8(ifile)
  n = length(x); if (n == 0) return(x)
  p = detect_pattern(x, tolower(file_ext(ifile)))
  if (is.null(p)) return(x)
  p = all_patterns[[p]]; p1 = p$chunk.begin; p2 = p$chunk.end
  i1 = grepl(p1, x)
  i2 = filter_chunk_end(i1, grepl(p2, x))
  m = numeric(n)
  m[i1] = 1; m[i2] = 2  # 1: code; 2: text
  if (m[1] == 0) m[1] = 2
  for (i in seq_len(n - 1)) if (m[i + 1] == 0) m[i + 1] = m[i]
  x[m == 1 | i2] = ''
  x[m == 2] = stringr::str_replace_all(x[m == 2], p$inline.code, '')
  structure(x, control = '-H -t')
}

pandoc_available = function() {
  # if you have this environment variable, chances are you are good to go
  if (Sys.getenv("RSTUDIO_PANDOC") != '') return(TRUE)
  if (Sys.which('pandoc-citeproc') == '') return(FALSE)
  rmarkdown::pandoc_available('1.12.3')
}

html_vignette = function(
  ..., fig_caption = TRUE, theme = NULL, highlight = "pygments",
  css = system.file('misc', 'vignette.css', package = 'knitr'),
  includes = list(
    after_body = system.file('misc', 'vignette.html', package = 'knitr')
  )
) {
  rmarkdown::html_document(
    ..., fig_caption = fig_caption, theme = theme, highlight = highlight,
    css = css, includes = includes
  )
}

#' Generate BibTeX bibliography databases for R packages
#'
#' A wrapper function of \code{xfun::pkg_bib()}.
#' @param ...,prefix Arguments passed to \code{xfun::\link[xfun]{pkg_bib}()}.
#' @export
write_bib = function(..., prefix = getOption('knitr.bib.prefix', 'R-')) {
  xfun::pkg_bib(..., prefix = prefix)
}

library(testit)

spin_w_tempfile = function(..., format = "Rmd") {
  tmp = tempfile(fileext = ".R")
  writeLines(c(...), tmp)
  spinned = spin(tmp, knit = FALSE, format = format)
  result = readLines(spinned)
  file.remove(c(tmp, spinned))
  result
}

assert("spin() detects lines for documentation", {
  (spin_w_tempfile("#' test", "1 * 1", "#' test") %==%
     c("test", "", "```{r}", "1 * 1", "```", "", "test"))
  # a multiline string literal contains the pattern of doc or inline
  (spin_w_tempfile("code <- \"", "#' test\"") %==%
    c("", "```{r}", "code <- \"", "#' test\"", "```", ""))
  (spin_w_tempfile("code <- \"", "{{ 1 + 1 }}", "\"") %==%
    c("", "```{r}", "code <- \"", "{{ 1 + 1 }}", "\"", "```", ""))
  # a multiline symbol contains the pattern of doc or inline
  (spin_w_tempfile("`", "#' test", "`") %==%
    c("", "```{r}", "`", "#' test", "`", "```", ""))
  (spin_w_tempfile("`", "{{ 1 + 1 }}", "`") %==%
    c("", "```{r}", "`", "{{ 1 + 1 }}", "`", "```", ""))
})

assert("spin() uses proper number of backticks", {
  (spin_w_tempfile("{{ '`' }}") %==% c("``r  '`'  ``"))
  (spin_w_tempfile("{{`x`}}") %==% c("``r `x` ``"))
  (spin_w_tempfile("x <- '", "```", "'") %==%
    c("", "````{r}", "x <- '", "```", "'", "````", ""))
})

has_rlang = FALSE

.onLoad = function(lib, pkg) {
  register_vignette_engines(pkg)

  default_handlers <<- evaluate::new_output_handler()

  otel_cache_tracer()

  has_rlang <<- requireNamespace("rlang", quietly = TRUE)

  if (has_rlang)
    rlang_entrace_handler <<- list(error = function(e) rlang::entrace(e))
}

library(testit)

template = 'The value of a is {{a}}, so a + 1 is {{a + 1}}'
exp = knit_expand(text = template, a = 10)
act = 'The value of a is 10, so a + 1 is 11'
assert('templates use data specified', exp == act)

template = 'The value of a is <% a %>, so a + 1 is <% a + 1 %>'
exp = knit_expand(text = template, a = 10, delim = c("<%", "%>"))
act = 'The value of a is 10, so a + 1 is 11'
assert('templates respect custom delimiter pairs', exp == act)

template = 'hello $(LETTERS[24]) and $(pi)!'
exp = knit_expand(text = template, delim = c("$(", ")"))
act = "hello X and 3.14159265358979!"
assert('templates respect pypi delimiters', exp == act)

template = 'The value of a is <% a %>, so a + 1 is <% a + 1 %>'
assert(
  'error is thrown when delimiter is not a pair',
  has_error(knit_expand(text = template, a = 10, delim = '<%'))
)

template = 'The value of a is {{a}}, and b + 1 is {{b + 1}}'
b = -1.21
exp = knit_expand(text = template, a = 10)
act = "The value of a is 10, and b + 1 is -0.21"
assert('templates use data from parent frame', exp == act)

library(testit)

z = new_defaults(list(a = 1))

assert(
  '$set() and $get() methods set/get values',
  identical(z$get(), list(a = 1)),
  identical({z$set(a = 2); z$get('a')}, 2),
  identical({z$set(b = 'f'); z$get(c('a', 'b'))}, list(a = 2, b = 'f'))
)

assert(
  '$merge() does not change the object but returns a list of new values',
  identical({z$merge(list(b = 'g')); z$get('b')}, 'f'),
  identical(z$merge(list(b = 'g')), list(a = 2, b = 'g'))
)

assert(
  '$get(default=TRUE) returns the initial value',
  identical(z$get(default = TRUE), list(a = 1))
)

assert(
  '$get(names) keeps the names if drop = FALSE and one or more names do not exist',
  identical(z$get(c('a', 'c')), list(a = 2, c = NULL)),
  identical(z$get('c'), NULL),
  identical(z$get('c', drop = FALSE), list(c = NULL))
)

assert('$delete() deletes keys from the list', {
  z$set(b1 = TRUE, b2 = FALSE); z$delete(c('b1', 'b2'))
  (!any(c('b1', 'b2') %in% z$get()))
})

assert(
  '$restore() restores to the initial value',
  identical({z$restore(); z$get()}, list(a = 1))
)

assert('$append() returns appended the chunk option', {
  z$set(d = 1)
  z$append(d = 2)
  identical(z$get('d'), c(1, 2))
})

z = new_defaults()

# named argument in set()
z$set(a = list(b = 2, c = 'qwer'))

assert(
  'a named argument of list in $set() method is not treated as a list of options',
  identical(length(z$get()), 1L),
  # unnamed argument in set()
  identical({z$restore(); z$set(list(b = 2, c = 'qwer')); z$get()}, list(b = 2, c = 'qwer'))
)

#' Create tables in LaTeX, HTML, Markdown and reStructuredText
#'
#' This is a very simple table generator. It is simple by design. It is not
#' intended to replace any other R packages for making tables.
#'
#' Missing values (\code{NA}) in the table are displayed as \code{NA} by
#' default. If you want to display them with other characters, you can set the
#' option \code{knitr.kable.NA}, e.g. \code{options(knitr.kable.NA = '')} to
#' hide \code{NA} values.
#' @param x An R object, typically a matrix or data frame.
#' @param format A character string. Possible values are \code{latex},
#'   \code{html}, \code{markdown}, \code{pandoc}, and \code{rst}; this will be
#'   automatically determined if the function is called within \pkg{knitr}; it
#'   can also be set in the global option \code{knitr.table.format}. If
#'   \code{format} is a function, it must return a character string.
#' @param digits Maximum number of digits for numeric columns, passed to
#'   \code{round()}. This can also be a vector of length \code{ncol(x)}, to set
#'   the number of digits for individual columns.
#' @param row.names Logical: whether to include row names. By default, row names
#'   are included if \code{rownames(x)} is neither \code{NULL} nor identical to
#'   \code{1:nrow(x)}.
#' @param col.names A character vector of column names to be used in the table.
#' @param align Column alignment: a character vector consisting of \code{'l'}
#'   (left), \code{'c'} (center) and/or \code{'r'} (right). By default or if
#'   \code{align = NULL}, numeric columns are right-aligned, and other columns
#'   are left-aligned. If \code{length(align) == 1L}, the string will be
#'   expanded to a vector of individual letters, e.g. \code{'clc'} becomes
#'   \code{c('c', 'l', 'c')}, unless the output format is LaTeX.
#' @param caption The table caption.
#' @param label The table reference label. By default, the label is obtained
#'   from \code{knitr::\link{opts_current}$get('label')}.
#' @param format.args A list of arguments to be passed to \code{\link{format}()}
#'   to format table values, e.g. \code{list(big.mark = ',')}.
#' @param escape Boolean; whether to escape special characters when producing
#'   HTML or LaTeX tables. When \code{escape = FALSE}, you have to make sure
#'   that special characters will not trigger syntax errors in LaTeX or HTML.
#' @param ... Other arguments (see Examples).
#' @return A character vector of the table source code.
#' @seealso Other R packages such as \pkg{huxtable}, \pkg{xtable},
#'   \pkg{kableExtra}, and \pkg{tables} for HTML and LaTeX tables, and
#'   \pkg{ascii} and \pkg{pander} for different flavors of markdown output and
#'   some advanced features and table styles.
#' @note The tables for \code{format = 'markdown'} also work for Pandoc when the
#'   \code{pipe_tables} extension is enabled (this is the default behavior for
#'   Pandoc >= 1.10).
#'
#'   When using \code{kable()} as a \emph{top-level} expression, you do not need
#'   to explicitly \code{print()} it due to R's automatic implicit printing.
#'   When it is wrapped inside other expressions (such as a \code{\link{for}}
#'   loop), you must explicitly \code{print(kable(...))}.
#' @references See
#'   \url{https://github.com/yihui/knitr-examples/blob/master/091-knitr-table.Rnw}
#'    for some examples in LaTeX, but they also apply to other document formats.
#' @export
#' @examples  kable(head(iris), format = 'latex')
#' kable(head(iris), format = 'html')
#' kable(head(iris), format = 'latex', caption = 'Title of the table')
#' kable(head(iris), format = 'html', caption = 'Title of the table')
#' # use the booktabs package
#' kable(mtcars, format = 'latex', booktabs = TRUE)
#' # use the longtable package
#' kable(matrix(1000, ncol=5), format = 'latex', digits = 2, longtable = TRUE)
#' # change LaTeX default table environment
#' kable(head(iris), format = "latex", caption = "My table", table.envir='table*')
#' # add some table attributes
#' kable(head(iris), format = 'html', table.attr = 'id="mytable"')
#' # reST output
#' kable(head(mtcars), format = 'rst')
#' # no row names
#' kable(head(mtcars), format = 'rst', row.names = FALSE)
#' # R Markdown/Github Markdown tables
#' kable(head(mtcars[, 1:5]), format = 'markdown')
#' # no inner padding
#' kable(head(mtcars), format = 'markdown', padding = 0)
#' # more padding
#' kable(head(mtcars), format = 'markdown', padding = 2)
#' # Pandoc tables
#' kable(head(mtcars), format = 'pandoc', caption = 'Title of the table')
#' # format numbers using , as decimal point, and ' as thousands separator
#' x = as.data.frame(matrix(rnorm(60, 1e6, 1e4), 10))
#' kable(x, format.args = list(decimal.mark = ',', big.mark = "'"))
#' # save the value
#' x = kable(mtcars, format = 'html')
#' cat(x, sep = '\n')
#' # can also set options(knitr.table.format = 'html') so that the output is HTML
kable = function(
  x, format, digits = getOption('digits'), row.names = NA, col.names = NA,
  align, caption = NULL, label = NULL, format.args = list(), escape = TRUE, ...
) {

  # determine the table format
  if (missing(format) || is.null(format)) format = getOption('knitr.table.format')
  if (is.null(format)) format = if (is.null(pandoc_to())) switch(
    out_format() %n% 'markdown',
    latex = 'latex', listings = 'latex', sweave = 'latex',
    html = 'html', markdown = 'markdown', rst = 'rst',
    stop('table format not implemented yet!')
  ) else if (isTRUE(opts_knit$get('kable.force.latex')) && is_latex_output()) {
    # force LaTeX table because Pandoc's longtable may not work well with floats
    # http://tex.stackexchange.com/q/276699/9128
    'latex'
  } else 'pandoc'
  if (is.function(format)) format = format()

  # expand align if applicable
  if (!missing(align) && length(align) == 1L && !grepl('[^lcr]', align))
    align = strsplit(align, '')[[1]]

  # create a label for bookdown if applicable
  if (is.null(label)) label = opts_current$get('label')
  if (!is.null(caption) && !is.na(caption)) caption = paste0(
    create_label('tab:', label, latex = (format == 'latex')), caption
  )
  if (inherits(x, 'list')) {
    # if the output is for Pandoc and we want multiple tabular in one table, we
    # should use the latex format instead, because Pandoc does not support
    # Markdown in LaTeX yet https://github.com/jgm/pandoc/issues/2453
    if (format == 'pandoc' && is_latex_output()) format = 'latex'
    res = lapply(
      x, kable, format = format, digits = digits, row.names = row.names,
      col.names = col.names, align = align, caption = NA,
      format.args = format.args, escape = escape, ...
    )
    res = unlist(lapply(res, one_string))
    res = if (format == 'latex') {
      kable_latex_caption(res, caption)
    } else if (format == 'html' || (format == 'pandoc' && is_html_output())) kable_html(
      matrix(paste0('\n\n', res, '\n\n'), 1), caption = caption, escape = FALSE,
      table.attr = 'class="kable_wrapper"'
    ) else {
      res = paste(res, collapse = '\n\n')
      if (format == 'pandoc') kable_pandoc_caption(res, caption) else res
    }
    return(structure(res, format = format, class = 'knitr_kable'))
  }
  if (!is.matrix(x)) x = as.data.frame(x)
  if (identical(col.names, NA)) col.names = colnames(x)
  m = ncol(x)
  # numeric columns
  isn = if (is.matrix(x)) rep(is.numeric(x), m) else sapply(x, is.numeric)
  if (missing(align) || (format == 'latex' && is.null(align)))
    align = ifelse(isn, 'r', 'l')
  # rounding
  digits = rep(digits, length.out = m)
  for (j in seq_len(m)) {
    if (is_numeric(x[, j])) x[, j] = round(x[, j], digits[j])
  }
  if (any(isn)) {
    if (is.matrix(x)) {
      if (is.table(x) && length(dim(x)) == 2) class(x) = 'matrix'
      x = format_matrix(x, format.args)
    } else x[, isn] = format_args(x[, isn], format.args)
  }
  if (is.na(row.names)) row.names = has_rownames(x)
  if (!is.null(align)) align = rep(align, length.out = m)
  if (row.names) {
    x = cbind(' ' = rownames(x), x)
    if (!is.null(col.names)) col.names = c(' ', col.names)
    if (!is.null(align)) align = c('l', align)  # left align row names
  }
  n = nrow(x)
  x = replace_na(to_character(as.matrix(x)), is.na(x))
  if (!is.matrix(x)) x = matrix(x, nrow = n)
  x = trimws(x)
  colnames(x) = col.names
  if (format != 'latex' && length(align) && !all(align %in% c('l', 'r', 'c')))
    stop("'align' must be a character vector of possible values 'l', 'r', and 'c'")
  attr(x, 'align') = align
  res = do.call(
    paste('kable', format, sep = '_'),
    list(x = x, caption = caption, escape = escape, ...)
  )
  structure(res, format = format, class = 'knitr_kable')
}

# convert to character while preserving dim/dimnames attributes
to_character = function(x) {
  if (is.character(x)) return(x)
  x2 = as.character(x); dim(x2) = dim(x); dimnames(x2) = dimnames(x)
  x2
}

# as.data.frame() does not allow duplicate row names (#898)
format_matrix = function(x, args) {
  nms = rownames(x)
  rownames(x) = NULL
  x = as.matrix(format_args(as.data.frame(x), args))
  rownames(x) = nms
  x
}

format_args = function(x, args = list()) {
  args$x = x
  args$trim = TRUE
  replace_na(do.call(format, args), is.na(x))
}

replace_na = function(x, which = is.na(x), to = getOption('knitr.kable.NA')) {
  if (is.null(to)) return(x)
  x[which] = to
  x
}

has_rownames = function(x) {
  !is.null(rownames(x)) && !identical(rownames(x), as.character(seq_len(NROW(x))))
}

#' @export
print.knitr_kable = function(x, ...) {
  if (!(attr(x, 'format') %in% c('html', 'latex'))) cat('\n\n')
  cat(x, sep = '\n')
}

#' @export
knit_print.knitr_kable = function(x, ...) {
  x = one_string(c(
    if (!(attr(x, 'format') %in% c('html', 'latex'))) c('', ''), x, '\n'
  ))
  asis_output(x)
}

kable_latex = function(
  x, booktabs = FALSE, longtable = FALSE, valign = 't', position = '', centering = TRUE,
  vline = getOption('knitr.table.vline', if (booktabs) '' else '|'),
  toprule = getOption('knitr.table.toprule', if (booktabs) '\\toprule' else '\\hline'),
  bottomrule = getOption('knitr.table.bottomrule', if (booktabs) '\\bottomrule' else '\\hline'),
  midrule = getOption('knitr.table.midrule', if (booktabs) '\\midrule' else '\\hline'),
  linesep = if (booktabs) c('', '', '', '', '\\addlinespace') else '\\hline',
  caption = NULL, caption.short = '', table.envir = if (!is.null(caption)) 'table',
  escape = TRUE
) {
  if (!is.null(align <- attr(x, 'align'))) {
    align = paste(align, collapse = vline)
    align = paste0('{', align, '}')
  }
  centering = if (centering && !is.null(caption)) '\n\\centering'
  # vertical align only if 'caption' is not NULL (may be NA) or 'valign' has
  # been explicitly specified
  valign = if ((!is.null(caption) || !missing(valign)) && valign != '') {
    sprintf('[%s]', valign)
  } else ''
  if (identical(caption, NA)) caption = NULL
  if (position != '') position = paste0('[', position, ']')
  env1 = sprintf('\\begin{%s}%s\n', table.envir, position)
  env2 = sprintf('\n\\end{%s}',   table.envir)
  if (caption.short != '') caption.short = paste0('[', caption.short, ']')
  cap = if (is.null(caption)) '' else sprintf('\n\\caption%s{%s}', caption.short, caption)

  if (nrow(x) == 0) midrule = ""

  linesep = if (nrow(x) > 1) {
    c(rep(linesep, length.out = nrow(x) - 1), '')
  } else rep('', nrow(x))
  linesep = ifelse(linesep == "", linesep, paste0('\n', linesep))

  if (escape) x = escape_latex(x)
  if (!is.character(toprule)) toprule = NULL
  if (!is.character(bottomrule)) bottomrule = NULL
  tabular = if (longtable) 'longtable' else 'tabular'

  paste(c(
    if (!longtable) c(env1, cap, centering),
    sprintf('\n\\begin{%s}%s', tabular, valign), align,
    if (longtable && cap != '') c(cap, '\\\\'),
    sprintf('\n%s', toprule), '\n',
    if (!is.null(cn <- colnames(x))) {
      if (escape) cn = escape_latex(cn)
      paste0(paste(cn, collapse = ' & '), sprintf('\\\\\n%s\n', midrule))
    },
    one_string(apply(x, 1, paste, collapse = ' & '), sprintf('\\\\%s', linesep), sep = ''),
    sprintf('\n%s', bottomrule),
    sprintf('\n\\end{%s}', tabular),
    if (!longtable) env2
  ), collapse = '')
}

kable_latex_caption = function(x, caption) {
  paste(c(
    '\\begin{table}\n', sprintf('\\caption{%s}\n', caption), x, '\n\\end{table}'
  ), collapse = '')
}

kable_html = function(x, table.attr = '', caption = NULL, escape = TRUE, ...) {
  table.attr = trimws(table.attr)
  # need a space between <table and attributes
  if (nzchar(table.attr)) table.attr = paste('', table.attr)
  align = if (is.null(align <- attr(x, 'align'))) '' else {
    sprintf(' style="text-align:%s;"', c(l = 'left', c = 'center', r = 'right')[align])
  }
  if (identical(caption, NA)) caption = NULL
  cap = if (length(caption)) sprintf('\n<caption>%s</caption>', caption) else ''
  if (escape) x = escape_html(x)
  one_string(c(
    sprintf('<table%s>%s', table.attr, cap),
    if (!is.null(cn <- colnames(x))) {
      if (escape) cn = escape_html(cn)
      c(' <thead>', '  <tr>', sprintf('   <th%s> %s </th>', align, cn), '  </tr>', ' </thead>')
    },
    '<tbody>',
    paste(
      '  <tr>',
      apply(x, 1, function(z) one_string(sprintf('   <td%s> %s </td>', align, z))),
      '  </tr>', sep = '\n'
    ),
    '</tbody>',
    '</table>'
  ))
}

#' Generate tables for Markdown and reST
#'
#' This function provides the basis for Markdown and reST tables.
#' @param x The data matrix.
#' @param sep.row A length-3 character vector, specifying separators to be printed
#'   before the header, after the header, and at the end of the table respectively.
#' @param sep.col The column separator.
#' @param padding Number of spaces for the table cell padding.
#' @param align.fun A function to process the separator under the header
#'   according to the alignment.
#' @return A character vector of the table content.
#' @noRd
kable_mark = function(x, sep.row = c('=', '=', '='), sep.col = '  ', padding = 0,
                      align.fun = function(s, a) s, rownames.name = '', ...) {
  # when the column separator is |, replace existing | with its HTML entity
  if (sep.col == '|') for (j in seq_len(ncol(x))) {
    x[, j] = gsub('\\|', '&#124;', x[, j])
  }
  l = if (prod(dim(x)) > 0) apply(x, 2, function(z) max(nchar(z, type = 'width'), na.rm = TRUE))
  cn = colnames(x)
  if (length(cn) > 0) {
    cn[is.na(cn)] = "NA"
    if (sep.col == '|') cn = gsub('\\|', '&#124;', cn)
    if (grepl('^\\s*$', cn[1L])) cn[1L] = rownames.name  # no empty cells for reST
    l = pmax(if (length(l) == 0) 0 else l, nchar(cn, type = 'width'))
  }
  align = attr(x, 'align')
  padding = padding * if (length(align) == 0) 2 else {
    ifelse(align == 'c', 2, 1)
  }
  l = pmax(l + padding, 3)  # at least of width 3 for Github Markdown
  s = unlist(lapply(l, function(i) paste(rep(sep.row[2], i), collapse = '')))
  res = rbind(if (!is.na(sep.row[1])) s, cn, align.fun(s, align),
              x, if (!is.na(sep.row[3])) s)
  apply(mat_pad(res, l, align), 1, paste, collapse = sep.col)
}

kable_rst = function(x, rownames.name = '\\', ...) {
  kable_mark(x, rownames.name = rownames.name)
}

# actually R Markdown
kable_markdown = function(x, padding = 1, ...) {
  if (is.null(colnames(x))) {
    warning('The table should have a header (column names)')
    colnames(x) = rep('', ncol(x))
  }
  res = kable_mark(x, c(NA, '-', NA), '|', padding, align.fun = function(s, a) {
    if (is.null(a)) return(s)
    r = c(l = '^.', c = '^.|.$', r = '.$')
    for (i in seq_along(s)) {
      s[i] = gsub(r[a[i]], ':', s[i])
    }
    s
  }, ...)
  sprintf('|%s|', res)
}

kable_pandoc = function(x, caption = NULL, padding = 1, ...) {
  # pandoc's table format cannot create 1-column or 0-row tables
  tab = if (ncol(x) == 1 || nrow(x) == 0) kable_markdown(
    x, padding = padding, ...
  ) else kable_mark(
    x, c(NA, '-', if (is_blank(colnames(x))) '-' else NA),
    padding = padding, ...
  )
  kable_pandoc_caption(tab, caption)
}

kable_pandoc_caption = function(x, caption) {
  if (identical(caption, NA)) caption = NULL
  if (length(caption)) c(paste('Table:', caption), "", x) else x
}

# pad a matrix
mat_pad = function(m, width, align = NULL) {
  n = nrow(m); p = ncol(m)
  res = matrix('', nrow = n, ncol = p)
  if (n * p == 0) return(res)
  stopifnot(p == length(width))
  side = rep('both', p)
  if (!is.null(align)) side = c(l = 'right', c = 'both', r = 'left')[align]
  apply(m, 2, function(x) max(nchar(x, 'width') - nchar(x, 'chars')))
  matrix(pad_width(c(m), rep(width, each = n), rep(side, each = n)), ncol = p)
}

# pad a character vector to width (instead of number of chars), considering the
# case of width > chars (e.g. CJK chars)
pad_width = function(x, width, side) {
  if (!all(side %in% c('left', 'right', 'both')))
    stop("'side' must be 'left', 'right', or 'both'")
  w = width - nchar(x, 'width')
  w1 = floor(w / 2)  # the left half of spaces when side = 'both'
  s1 = v_spaces(w * (side == 'left') + w1 * (side == 'both'))
  s2 = v_spaces(w * (side == 'right') + (w - w1) * (side == 'both'))
  paste0(s1, x, s2)
}

# vectorized over n to generate sequences of spaces
v_spaces = function(n) {
  unlist(lapply(n, highr:::spaces))
}

library(testit)

res = mapply(
  auto_out_name,
  c('abc.Rnw', 'abc.rnw', 'abc.rtex', 'abc.Rmd', 'abc.rhtm', 'abc.Rhtml', 'foo.abc.rhtml'),
  USE.NAMES = FALSE
)
assert(
  'auto_out_name() converts .Rfoo to .foo',
  identical(res, c('abc.tex', 'abc.tex', 'abc.tex', 'abc.md', 'abc.htm', 'abc.html', 'foo.abc.html'))
)

res = mapply(
  auto_out_name,
  c('abc.tex', '_knit_abc.tex', '_knit_abc.md', 'foo_knit_.html'),
  USE.NAMES = FALSE
)
assert(
  'auto_out_name() converts .tex/.unknown to .txt, and removes _knit_',
  identical(res, c('abc.txt', 'abc.tex', 'abc.md', 'foo.html')),
  identical(auto_out_name('foo.bar'), 'foo.txt')
)

assert(
  'chunks with include=FALSE should stop on error',
  suppressMessages(
    has_error(knit(text = c('<<include=F>>=', '1+"a"', '@'), quiet = TRUE))
  )
)

assert(
  'tidy=FALSE + eval=numeric should work',
  identical(
    '[1] 1\n[1] 2\n',
    knit(
      text = c('<<tidy=FALSE, eval=1:2, echo=FALSE, results="asis">>=', '1', '1+', '1', '1', '@'),
      quiet = TRUE
    )
  )
)

assert(
  'opts_template options are used',
  identical(
    "\n[1] 2\n",
    knit(
      text = c(
        "<<include=FALSE>>=","opts_template$set(quiet = list(echo=FALSE))", "@",
        "<<results='asis', opts.label='quiet'>>=", "1+1", "@"
      ),
      quiet = TRUE
    )
  )
)

assert(
  'local chunk options override opts_template',
  identical(
    "\n\\begin{kframe}\n\\begin{alltt}\n\\hlnum{1}\\hlopt{+}\\hlnum{1}\n\\end{alltt}\n\\end{kframe}[1] 2\n",
    knit(
      text = c(
        "<<include=FALSE>>=","opts_template$set(quiet = list(echo=FALSE))", "@",
        "<<results='asis', opts.label='quiet', echo=TRUE, tidy=FALSE>>=", "1+1", "@"
      ),
      quiet = TRUE
    )
  )
)

# a shortcut
k = function(text) {
  on.exit(opts_chunk$restore())
  knit(text = c('```{r}', text, '```'), quiet = TRUE, envir = parent.frame())
}
k(c(
  'knit_child(text = "```{r}\nopts_chunk$set(echo=FALSE)\n```", options=list())',
  'x1 = opts_chunk$get("echo")'
))
k(c(
  'knit_child(text = "```{r}\nopts_chunk$set(dpi=200)\n```", options=list(dpi=100))',
  'x2 = opts_chunk$get("dpi")'
))

assert(
  'using knit_child() does not reset global chunk options set in child documents',
  x1 == FALSE, x2 == 200
)

txt = '%\\documentclass{article}
\\documentclass{article}
\\begin{document}
\\Sexpr{pi}
\\end{document}'
res = strsplit(knit(text = txt, quiet = TRUE), '\n')[[1]]

assert(
  'insert_header_latex() finds the correct \\documentclass{}',
  identical(res[1], '%\\documentclass{article}')
)

txt = '\\documentclass{article}
\\begin{document}
\\begin{verbatim}
\\documentclass{article}
\\end{verbatim}
\\Sexpr{pi}
\\end{document}'
res = strsplit(knit(text = txt, quiet = TRUE), '\n')[[1]]

assert(
  'insert_header_latex() finds the correct \\documentclass{}',
  identical(res[length(res) - 3], '\\documentclass{article}')
)

assert('knit_meta_add() adds meta objects with the correct number of labels', {
  knit_meta(clean = TRUE)
  knit_meta_add(list(1, 2))
  m = attr(knit_meta_add(list(3, 4), label = c('a', 'b')), 'knit_meta_id')
  knit_meta(clean = TRUE)
  (m %==% c('', '', 'a', 'b'))
})

#' Convert Sweave to knitr documents
#'
#' This function converts an Sweave document to a \pkg{knitr}-compatible
#' document.
#'
#' The pseudo command \samp{\\SweaveInput{file.Rnw}} is converted to a code
#' chunk header \code{<<child='file.Rnw'>>=}.
#'
#' Similarly \samp{\\SweaveOpts{opt = value}} is converted to a code chunk
#' \samp{opts_chunk$set(opt = value)} with the chunk option \code{include =
#' FALSE}; the options are automatically fixed in the same way as local chunk
#' options (explained below).
#'
#' The Sweave package \samp{\\usepackage{Sweave}} in the preamble is removed
#' because it is not required.
#'
#' Chunk options are updated if necessary: option values \code{true} and
#' \code{false} are changed to \code{TRUE} and \code{FALSE} respectively;
#' \code{fig=TRUE} is removed because it is not necessary for \pkg{knitr} (plots
#' will be automatically generated); \code{fig=FALSE} is changed to
#' \code{fig.keep='none'}; the devices \code{pdf/jpeg/png/eps/tikz=TRUE} are
#' converted to \code{dev='pdf'/'jpeg'/'png'/'postscript'/'tikz'};
#' \code{pdf/jpeg/png/eps/tikz=FALSE} are removed;
#' \code{results=tex/verbatim/hide} are changed to
#' \code{results='asis'/'markup'/'hide'}; \code{width/height} are changed to
#' \code{fig.width/fig.height}; \code{prefix.string} is changed to
#' \code{fig.path}; \code{print/term/prefix=TRUE/FALSE} are removed; most of the
#' character options (e.g. \code{engine} and \code{out.width}) are quoted;
#' \code{keep.source=TRUE/FALSE} is changed to \code{tidy=FALSE/TRUE} (note the
#' order of values).
#'
#' If a line \code{@@} (it closes a chunk) directly follows a previous
#' \code{@@}, it is removed; if a line \code{@@} appears before a code chunk and
#' no chunk is before it, it is also removed, because \pkg{knitr} only uses one
#' \samp{@@} after \samp{<<>>=} by default (which is not the original Noweb
#' syntax but more natural).
#' @param file Path to the Rnw file (must be encoded in UTF-8).
#' @param output Output file path. By default, \file{file.Rnw} produces
#'   \file{file-knitr.Rnw}); if \code{text} is not NULL, no output file will be
#'   produced.
#' @param text An alternative way to provide the Sweave code as a character
#'   string. If \code{text} is provided, \code{file} will be ignored.
#' @return If \code{text} is \code{NULL}, the \code{output} file is written and
#'   \code{NULL} is returned. Otherwise, the converted text string is returned.
#' @note If \samp{\\SweaveOpts{}} spans across multiple lines, it will not be
#'   fixed, and you have to fix it manually. The LaTeX-style syntax of Sweave
#'   chunks are ignored (see \code{?SweaveSyntaxLatex}); only the Noweb syntax
#'   is supported.
#' @seealso \code{\link{Sweave}}, \code{\link{gsub}}
#' @references The motivation of the changes in the syntax:
#'   \url{https://yihui.org/knitr/demo/sweave/}
#' @export
#' @examples Sweave2knitr(text='<<echo=TRUE>>=') # this is valid
#' Sweave2knitr(text="<<png=true>>=") # dev='png'
#' Sweave2knitr(text="<<eps=TRUE, pdf=FALSE, results=tex, width=5, prefix.string=foo>>=")
#' Sweave2knitr(text="<<,png=false,fig=TRUE>>=")
#' Sweave2knitr(text="\\SweaveOpts{echo=false}")
#' Sweave2knitr(text="\\SweaveInput{hello.Rnw}")
#' # Sweave example in utils
#' testfile = system.file("Sweave", "Sweave-test-1.Rnw", package = "utils")
#' Sweave2knitr(testfile, output = 'Sweave-test-knitr.Rnw')
#' if (interactive()) knit('Sweave-test-knitr.Rnw') # or knit2pdf() directly
#' unlink('Sweave-test-knitr.Rnw')
Sweave2knitr = function(file, output = gsub('[.]([^.]+)$', '-knitr.\\1', file), text = NULL) {
  x = text
  if (is.null(x)) x = read_utf8(file)
  x = native_encode(x)
  x = gsub_msg('removing \\usepackage{Sweave}',
               '^\\s*\\\\usepackage(\\[.*\\])?\\{Sweave\\}', '', x)
  i = grep('^<<(.*)>>=\\s*$', x)
  if (length(i)) {
    opts = gsub('^<<(.*)>>=\\s*$', '\\1', x[i])
    x[i] = paste0('<<', fix_sweave(opts), '>>=')
  }
  x = gsub_msg("replacing \\SweaveInput{...} with <<child='...'>>=",
               '^\\s*\\\\SweaveInput\\{([^}]+)\\}', "\n<<'child-\\1', child='\\1'>>=\n@\n", x)
  s = '^\\s*\\\\SweaveOpts\\{([^}]*)\\}.*$'
  i = grep(s, x)
  if (length(i)) {
    opts = fix_sweave(gsub(s, '\\1', x[i]))
    x[i] = gsub_msg('changing \\SweaveOpts{} to opts_chunk$set()', s, '@_@_@', x[i])
    for (j in seq_along(i))
      x[i[j]] = gsub('@_@_@', one_string(c(
        '\n<<include=FALSE>>=', 'library(knitr)', 'opts_chunk$set(', opts[j], ')', '@\n'
      )), x[i[j]])
  }
  # remove the extra @
  i1 = grepl(all_patterns$rnw$chunk.begin, x)
  i2 = grepl(all_patterns$rnw$chunk.end, x)
  i = which(i2 & !filter_chunk_end(i1, i2))
  if (length(i)) {
    message('removing extra lines (#n shows line numbers):\n',
            one_string(formatUL(sprintf('(#%d) %s', i, x[i]), offset = 4)))
    x = x[-i]
  }
  if (is.null(text)) write_utf8(x, output) else x
}

gsub_msg = function(msg, pattern, replacement, x, ...) {
  if (length(i <- grep(pattern, x, ...))) {
    message(msg, ':\n', one_string(formatUL(x[i], offset = 4)))
    gsub(pattern, replacement, x, ...)
  } else x
}

fix_sweave = function(x) {
  x = gsub_msg('capitalizing true/false to TRUE/FALSE',
               '(echo|eval|split|include|fig|pdf|eps|jpeg|png|tikz|keep.source|print|term|strip.white|prefix)\\s*=\\s*(true|false)', '\\1=\\U\\2', x, perl = TRUE)

  x = gsub_msg('removing the unnecessary option fig=TRUE',
               'fig\\s*=\\s*TRUE', '', x)
  x = gsub_msg("fig=FALSE should be fig.keep='none'",
               'fig\\s*=\\s*FALSE', "fig.keep='none'", x)

  x = gsub_msg("replacing pdf/jpeg/png/tikz=TRUE with dev='pdf'/'jpeg'/'png'/'tikz'",
               '(pdf|jpeg|png|tikz)\\s*=\\s*TRUE', "dev='\\1'", x)
  x = gsub_msg('removing pdf/jpeg/png/eps/tikz=FALSE',
               '(pdf|jpeg|png|eps|tikz)\\s*=\\s*FALSE', '', x)
  x = gsub_msg("replacing eps=TRUE with dev='postscript'",
               'eps\\s*=\\s*TRUE', "dev='postscript'", x)

  x = gsub_msg('replacing results=tex with results=asis',
               'results\\s*=\\s*tex', 'results=asis', x)
  x = gsub_msg('replacing results=verbatim with results=markup',
               'results\\s*=\\s*verbatim', 'results=markup', x)
  x = gsub_msg('quoting the results option',
               'results\\s*=\\s*(asis|markup|hide)', "results='\\1'", x)

  x = gsub_msg('replacing width/height with fig.width/fig.height',
               '([^.])(width|height)\\s*=\\s*(\\d+)', '\\1fig.\\2=\\3', x)

  x = gsub_msg("replacing prefix.string=foo with fig.path='foo'",
               'prefix.string\\s*=\\s*([^,]+)', "fig.path='\\1'", x)

  x = gsub_msg("removing options 'print', 'term', 'prefix'",
               '(print|term|prefix)\\s*=\\s*(TRUE|FALSE|T|F)', '', x)

  x = gsub_msg('quoting options engine, fig.path, cache.path, fig.keep, fig.show, dev, out.width, out.height, fig.align',
               "(engine|fig\\.path|cache\\.path|fig\\.keep|fig\\.show|dev|out\\.width|out\\.height|fig\\.align)\\s*=\\s*([^,'\"]+)",
               "\\1='\\2'", x)

  x = gsub_msg('changing keep.source=TRUE to tidy=FALSE',
               'keep\\.source\\s*=\\s*(TRUE|T)', 'tidy=FALSE', x)
  x = gsub_msg('changing keep.source=FALSE to tidy=TRUE',
               'keep\\.source\\s*=\\s*(FALSE|F)', 'tidy=TRUE', x)

  x = gsub_msg('doubling backslashes', '\\', '\\\\', x, fixed = TRUE)
  # after we remove some options, there might be , ,
  while (length(grep(',\\s*,', x))) x = gsub(',\\s*,', ',', x)
  x = gsub('^(\\s*,\\s*)+|(\\s*,\\s*)+$', '', x)
  x
}

# check the source code to see if it is an Sweave document
which_sweave = function(x) {
  unique(c(
    grep('^\\s*\\\\(usepackage(\\[.*\\])?\\{Sweave|SweaveInput\\{|SweaveOpts\\{)', x),
    grep('^<<.*\\b(echo|eval|split|include)\\s*=\\s*(true|false)\\b.*>>=', x),
    grep('^<<.*\\bresults\\s*=\\s*(tex|verbatim|hide)\\b.*>>=', x),
    grep('^<<.*\\b(fig|pdf|eps|jpeg|png|tikz)\\s*=\\s*(true|false|T|F).*>>=', x),
    grep('^<<.*([, ])(width|height)\\s*=\\s*(\\d+).*>>=', x),
    grep('^<<.*\\b(keep.source|print|term|prefix)\\s*=\\s*(true|false|T|F).*>>=', x)
  ))
}

remind_sweave = function(file, sweave_lines) {
  msg = sprintf('It seems you are using the Sweave-specific syntax in line(s) %s; you may need Sweave2knitr("%s") to convert it to knitr',
                paste(sweave_lines, collapse = ', '), file)
  # throw a normal warning when R CMD check or tcltk not available
  warning(msg)
  if (!is_R_CMD_check() && Sys.getenv('CI') != 'true' && tcltk_available()) {
    do.call(
      getFromNamespace('tkmessageBox', 'tcltk'),
      list(title = 'Sweave Noweb syntax?', icon = 'info', message = msg)
    )
  }
}

tcltk_available = function() {
  capabilities('tcltk') && capabilities('X11') && has_package('tcltk') &&
    getFromNamespace('.TkUp', 'tcltk')
}

#' @rdname hook_plot
#' @export
hook_plot_textile = function(x, options) {
  cap = .img.cap(options); if (is.na(cap)) cap = ''

  tags = unlist(c(Map(
    sprintf, c('width: %s', 'height: %s'),
    options[c('out.width', 'out.height')]
  ), css_align(options$fig.align)))
  tags = if (length(tags)) sprintf('{%s}', paste(tags, collapse = ';')) else ''

  paste0(
    '!', tags, opts_knit$get('base.url'), .upload.url(x),
    if (nzchar(cap)) sprintf('(%s)', cap), '!\n\n',
    if (nzchar(cap)) sprintf('p(knitr plot caption#%s). %s', options$label, cap),
    '\n\n'
  )
}

#' @rdname output_hooks
#' @export
render_textile = function() {
  set_html_dev()
  opts_knit$set(out.format = 'textile')
  textile.hook = function(name) {
    force(name)
    function(x, options) {
      if (name == 'source') x = c(hilight_source(x, 'textile', options), '')
      x = one_string(x)
      sprintf('bc(knitr %s %s#%s).. %s\np(knitr_end). \n\n',
              tolower(options$engine), name, options$label, x)
    }
  }
  hook.inline = function(x) .inline.hook(format_sci(x, 'html'))
  z = list()
  for (i in c('source', 'warning', 'message', 'error'))
    z[[i]] = textile.hook(i)
  knit_hooks$set(z)
  knit_hooks$set(
    inline = hook.inline, output = textile.hook('output'), plot = hook_plot_textile
  )
}

library(testit)

assert(
  !is_sql_update_query('SELECT 1'),
  !is_sql_update_query('SELECT * FROM foo'),
  !is_sql_update_query(' SELECT 1'),
  !is_sql_update_query('\nSELECT 1'),
  !is_sql_update_query('\tSELECT 1')
)

assert(!is_sql_update_query(c('-- Some SQL', 'SELECT 1')))

assert(!is_sql_update_query(c('/* ', '   Some SQL', '*/', 'SELECT 1')))

assert(!is_sql_update_query(c('   /* ', '      Some SQL', '   */', 'SELECT 1')))

assert(is_sql_update_query('UPDATE foo SET a=1'))
assert(is_sql_update_query(' UPDATE foo SET a=1'))
assert(is_sql_update_query('\n\nUPDATE foo SET a=1'))
assert(is_sql_update_query('\tUPDATE foo SET a=1'))
assert(is_sql_update_query('DELETE FROM foo'))
assert(is_sql_update_query('INSERT INTO foo values(1)'))

assert(is_sql_update_query(c('-- SELECT 1', 'INSERT INTO foo values(1)')))

assert(is_sql_update_query(c('/*SELECT 1*/', '   INSERT INTO foo values(1)')))

assert(is_sql_update_query(c(
  '/*', '   Insert records into table', '*/', '', '   INSERT INTO foo values(1)'
)))

assert(is_sql_update_query('update foo set a=1'))
assert(is_sql_update_query('delete from foo'))
assert(is_sql_update_query('insert into foo values(1)'))

#' @rdname hook_plot
#' @export
hook_plot_md = function(x, options) {
  # if not using R Markdown v2 or output is HTML, just return v1 output
  if (is.null(to <- pandoc_to()) || is_html_output(to))
    return(hook_plot_md_base(x, options))
  if ((options$fig.show == 'animate' || is_tikz_dev(options)) && is_latex_output())
    return(hook_plot_tex(x, options))
  office_output = to %in% c('docx', 'pptx', 'rtf', 'odt')
  if (need_special_plot_hook(options)) {
    if (is_latex_output()) {
      # Pandoc < 1.13 does not support \caption[]{} so suppress short caption
      if (is.null(options$fig.scap)) options$fig.scap = NA
      return(hook_plot_tex(x, options))
    }
    if (office_output) {
      if (options$fig.align != 'default') {
        warn_options_unsupported('fig.align', to)
        options$fig.align = 'default'
      }
      if (!is.null(options$fig.alt)) {
        warn_options_unsupported('fig.alt', to)
        options$fig.alt = NULL
      }
      return(hook_plot_md_pandoc(x, options))
    }
  }
  hook_plot_md_base(x, options)
}

# decide if the markdown plot hook is not enough and needs special hooks like
# hook_plot_tex() to handle chunk options like out.width
need_special_plot_hook = function(options) {
  opts = opts_chunk$get(default = TRUE)
  for (i in c(
    'out.width', 'out.height', 'out.extra', 'fig.align', 'fig.subcap',
    'fig.env', 'fig.scap', 'fig.alt'
  )) if (!identical(options[[i]], opts[[i]])) return(TRUE)
  FALSE
}

hook_plot_md_base = function(x, options) {
  # when fig.id = TRUE, add an `id` attribute to images of the form fig:label-i
  if (isTRUE(options$fig.id)) options$fig.id = function(options) {
    id = sprintf('%s%s-%s', options$fig.lp, options$label, options$fig.cur)
    sprintf('id="%s"', xfun::alnum_id(id))
  }
  if (is.function(options$fig.id)) options$out.extra = c(
    options$out.extra, options$fig.id(options)
  )
  if (options$fig.show == 'animate') return(hook_plot_html(x, options))

  cap = .img.cap(options)
  alt = .img.cap(options, alt = TRUE, escape = TRUE)

  w = options[['out.width']]; h = options[['out.height']]
  s = options$out.extra; a = options$fig.align
  ai = options$fig.show == 'asis'
  # whether to use <object> to embed SVG graphics
  is_svg = grepl('[.]svg$', x, ignore.case = TRUE) && getOption('knitr.svg.object', FALSE)
  # self-contained mode?
  sc = any(c('--embed-resources', '--self-contained') %in% opts_knit$get('rmarkdown.pandoc.args'))
  lnk = options$fig.link
  pandoc_html = cap != '' && is_html_output()
  in_bookdown = isTRUE(opts_knit$get('bookdown.internal.label'))
  plot1 = ai || options$fig.cur <= 1L
  plot2 = ai || options$fig.cur == options$fig.num
  to = pandoc_to(); from = pandoc_from()
  if (is.null(w) && is.null(h) && is.null(s) && is.null(options$fig.alt) &&
      a == 'default' && !(pandoc_html && in_bookdown) && !is_svg) {
    # append <!-- --> to ![]() to prevent the figure environment in these cases
    nocap = cap == '' && !is.null(to) && !grepl('^markdown', to) &&
      (options$fig.num == 1 || ai) && !grepl('-implicit_figures', from)
    x2 = paste0(opts_knit$get('base.url'), .upload.url(x))
    res = sprintf('![%s](%s)', cap, x2)
    if (!is.null(lnk) && !is.na(lnk)) res = sprintf('[%s](%s)', res, lnk)
    res = paste0(res, if (nocap) '<!-- -->' else '', if (is_latex_output()) ' ' else '')
    return(res)
  }
  add_link = function(x) {
    if (is.null(lnk) || is.na(lnk)) return(x)
    sprintf('<a href="%s" target="_blank">%s</a>', lnk, x)
  }
  img_code = function(s2 = NULL) {
    img = if (is_svg && sc) svg_code(x, s) else .img.tag(x, w, h, alt, c(s, s2))
    add_link(img)
  }
  # use HTML syntax <img src=...>
  if (pandoc_html && !isTRUE(grepl('-implicit_figures', from))) {
    d1 = if (plot1) sprintf('<div class="figure"%s>\n', css_text_align(a))
    d2 = sprintf('<p class="caption">%s</p>', cap)
    img = img_code()
    # whether to place figure caption at the top or bottom of a figure
    if (isTRUE(options$fig.topcaption)) {
      paste0(d1, if (ai || options$fig.cur <= 1) d2, img, if (plot2) '</div>')
    } else {
      paste0(d1, img, if (plot2) paste0('\n', d2, '\n</div>'))
    }
  } else {
    img_code(sprintf('style="%s"', css_align(a)))
  }
}

# read svg, remove the xml/doctype declaration, and put the code in a raw html block
svg_code = function(file, extra = NULL) {
  x = read_utf8(file)
  while (length(x) > 0 && !grepl('^\\s*<svg .+', x[1])) x = x[-1]
  if (length(x) > 0 && length(extra) == 1) {
    if (grepl(r <- '\\s*>\\s*$', x[1])) {
      x[1] = paste0(gsub(r, ' ', x[1]), extra, '>')
    } else {
      x[1] = paste(x[1], extra)
    }
  }
  raw_html(x)
}

hook_plot_md_pandoc = function(x, options) {
  if (options$fig.show == 'animate') return(hook_plot_html(x, options))

  base = opts_knit$get('base.url') %n% ''
  cap = .img.cap(options)

  at = paste(
    c(
      sprintf('width=%s', options[['out.width']]),
      sprintf('height=%s', options[['out.height']]),
      options[['out.extra']]
    ),
    collapse = ' '
  )
  if (at != '') at = paste0('{', at, '}')

  sprintf('![%s](%s%s)%s', cap, base, .upload.url(x), at)
}

css_align = function(align) {
  sprintf('display: block; margin: %s;', switch(
    align, left = 'auto auto auto 0', center = 'auto', right = 'auto 0 auto auto'
  ))
}

css_text_align = function(align) {
  if (align == 'default') '' else sprintf(' style="text-align: %s"', align)
}

# turn a class string "a b" to c(".a", ".b") for Pandoc fenced code blocks
block_class = function(x) {
  if (length(x) > 0) gsub('^[.]*', '.', unlist(strsplit(x, '\\s+')))
}

# concatenate block attributes (including classes) for Pandoc fenced code blocks
block_attr = function(attr, class = NULL, lang = NULL) {
  x = c(block_class(class), attr)
  if (length(x) == 0) return(lang)
  x = c(sprintf('.%s', lang), x)
  paste0('{', paste0(x, collapse = ' '), '}')
}

#' @rdname output_hooks
#' @export
#' @param strict Boolean; whether to use strict markdown or reST syntax. For markdown, if
#'   \code{TRUE}, code blocks will be indented by 4 spaces, otherwise they are
#'   put in fences made by three backticks. For reST, if \code{TRUE}, code is
#'   put under two colons and indented by 4 spaces, otherwise it is put under the
#'   \samp{sourcecode} directive (this is useful for e.g. Sphinx).
#' @param fence_char A single character to be used in the code blocks fence.
#'   This can be e.g. a backtick or a tilde, depending on your Markdown rendering
#'   engine.
render_markdown = function(strict = FALSE, fence_char = '`') {
  set_html_dev()
  opts_knit$set(out.format = 'markdown')
  knit_hooks$set(hooks_markdown(strict, fence_char))
}

#' @rdname output_hooks
#' @export
hooks_markdown = function(strict = FALSE, fence_char = '`') {
  fence = paste(rep(fence_char, 3), collapse = '')
  # four spaces will generate a <pre> block
  hook.t = function(x, attr = NULL, class = NULL) {
    if (strict) {
      paste('\n', indent_block(x), '\n', sep = '\n')
    } else {
      fenced_block(x, attr, class, .char = fence_char)
    }
  }
  hook.o = function(class) {
    force(class)
    function(x, options) {
      hook.t(x, options[[paste0('attr.', class)]], options[[paste0('class.', class)]])
    }
  }
  hook.r = function(x, options) {
    lang = tolower(options$lang %n% eng2lang(options$engine))
    if (!options$highlight) lang = 'text'
    fenced_block(x, options$attr.source, options$class.source, lang, .char = fence_char)
  }
  list(
    source = function(x, options) {
      x = hilight_source(x, 'markdown', options)
      if (strict) hook.t(x) else hook.r(c(x, ''), options)
    },
    inline = function(x) {
      if (is_latex_output()) .inline.hook.tex(x) else {
        .inline.hook(format_sci(x, if (length(pandoc_to()) == 1L) 'latex' else 'html'))
      }
    },
    plot = hook_plot_md,
    chunk = function(x, options) {
      x = gsub(paste0('[\n]{2,}(', fence, '|    )'), '\n\n\\1', x)
      x = gsub('[\n]+$', '', x)
      x = gsub('^[\n]+', '\n', x)
      if (isTRUE(options$collapse)) {
        r = sprintf('\n([%s]{3,})\n+\\1((\\{[.])?%s[^\n]*)?\n', fence_char, tolower(options$engine))
        x = gsub(r, '\n', x)
        x = gsub(asis_token, '', x, fixed = TRUE)
      }
      x = pandoc_div(x, options[['attr.chunk']], options[['class.chunk']])
      if (is.null(s <- options$indent)) return(x)
      line_prompt(x, prompt = s, continue = s)
    },
    output = hook.o('output'), warning = hook.o('warning'),
    error = hook.o('error'), message = hook.o('message')
  )
}

pandoc_div = function(x, .attr = NULL, .class = NULL) {
  if (is.null(.attr) && is.null(.class)) return(x)
  fenced_block(c(x, ''), .attr, .class, .char = ':', .sep = ' ', .outer = '')
}

# add a fence around content (either fenced code block ``` or Div :::)
fenced_block = function(x, ..., .char = '`', .sep = '', .outer = '\n\n') {
  x = one_string(c('', x))
  f = create_fence(x, .char)
  paste0(.outer, paste(f, block_attr(...), sep = .sep), x, f, .outer)
}

create_fence = function(x, char = '`') {
  r = paste0('\n', char, '{3,}')
  l = max(if (grepl(r, x)) attr(gregexpr(r, x)[[1]], 'match.length'), 3)
  paste(rep(char, l), collapse = '')
}

# convert some engine names to language names
eng2lang = function(x) {
  d = c(
    asy = 'cpp', mysql = 'sql', node = 'javascript', psql = 'sql',
    rscript = 'r', rcpp = 'cpp', tikz = 'tex'
  )
  x = tolower(x)
  if (x %in% names(d)) d[x] else x
}

#' @param highlight Which code highlighting engine to use: if \code{pygments},
#'   the Liquid syntax is used (default approach Jekyll); if \code{prettify},
#'   the output is prepared for the JavaScript library \file{prettify.js}; if
#'   \code{none}, no highlighting engine will be used, and code blocks are simply
#'   indented by 4 spaces).
#' @param extra Extra tags for the highlighting engine. For \code{pygments}, this
#'   can be \code{'linenos'}; for \code{prettify}, it can be \code{'linenums'}.
#' @rdname output_hooks
#' @export
render_jekyll = function(highlight = c('pygments', 'prettify', 'none'), extra = '') {
  render_markdown(TRUE)
  knit_hooks$set(hooks_jekyll(highlight = highlight, extra = extra))
}

#' @rdname output_hooks
#' @export
hooks_jekyll = function(highlight = c('pygments', 'prettify', 'none'), extra = '') {
  hook.m = hooks_markdown(TRUE)
  hi = match.arg(highlight)
  if (hi == 'none') return(hook.m)
  switch(hi, pygments = {
    hook.r = function(x, options) {
      paste0(
        '\n\n{% highlight ', tolower(options$engine), if (extra != '') ' ', extra,
        ' %}\n', x, '\n{% endhighlight %}\n\n'
      )
    }
    hook.t = function(x, options) paste0(
      '\n\n{% highlight text %}\n', x, '{% endhighlight %}\n\n'
    )
  }, prettify = {
    hook.r = function(x, options) {
      paste0(
        '\n\n<pre><code class="prettyprint ', extra, '">', escape_html(x),
        '</code></pre>\n\n'
      )
    }
    hook.t = function(x, options) paste0(
      '\n\n<pre><code>', escape_html(x), '</code></pre>\n\n'
    )
  })
  source = function(x, options) {
    x = one_string(hilight_source(x, 'markdown', options))
    hook.r(x, options)
  }
  merge_list(hook.m, list(
    source = source, output = hook.t, warning = hook.t, message = hook.t, error = hook.t
  ))
}

library(gWidgetsWWW2)
w = gwindow('An R Notebook')
sb = gstatusbar('Powered by Rook, gWidgetsWWW2 and knitr', container = w)
g = gframe('An R Notebook based on knitr', use.scrollwindow = TRUE,
           horizontal = FALSE, container = w)
gbutton('Knit', container = g, handler = function(h, ...) {
  library(knitr)
  svalue(g3) = try(knit2html(text = svalue(g1), fragment.only = TRUE))
})
g0 = ggroup(container = g)
code = readLines(system.file('examples', 'knitr-minimal.Rmd', package = 'knitr'))
# g1 = gtext(code, container = g, use.codemirror = TRUE, width = 500)
g1 = gtext(code, container = g0, width = 500, height = 500)
g2 = ggroup(cont = g0, expand = TRUE)
g3 = ghtml('results here', container = g2)

library(testit)

rm(list = ls())
z = 5
assert(
  'a list can be used as the parent frame for knit()',
  # evaluate in a new environment; should create an object in current envir
  !has_error(with(list(y = 4:8), knit('knit-envir.Rmd', quiet = TRUE)))
)

env = new.env()
env$y = 1:3
knit('knit-envir.Rmd', envir = env, quiet = TRUE)
assert(
  'knit() creates objects in its envir argument',
  !exists('asdfqwerzxcv'), exists('asdfqwerzxcv', envir = env)
)

assert(
  'undefined external objects should cause errors',
  suppressMessages(has_error(knit('knit-envir.Rmd', quiet = TRUE))),  # y is not found
  !has_error(with(list(y = letters), knit('knit-envir.Rmd', quiet = TRUE)))
)

file.remove('knit-envir.md')

#' Knit package documentation
#'
#' Run examples in a package and insert output into the examples code;
#' \code{knit_rd_all()} is a wrapper around \code{knit_rd()} to build static
#' HTML help pages for all packages under the \file{html} directory of them.
#' @param pkg Package name.
#' @param links A character vector of links to be passed to
#'   \code{\link[tools]{Rd2HTML}}.
#' @param frame Boolean: whether to put a navigation frame on the left of the
#'   index page.
#' @return All HTML pages corresponding to topics in the package are written
#'   under the current working directory. An \file{index.html} is also written
#'   as a table of content.
#' @note Ideally the html pages should be put under the \file{html} directory of
#'   an installed package which can be found via \code{system.file('html',
#'   package = 'your_package_name')}, otherwise some links may not work (e.g.
#'   the link to the DESCRITION file).
#' @examples library(knitr)
#' \dontrun{
#'
#' knit_rd('maps')
#' knit_rd('rpart')
#' setwd(system.file('html', package = 'ggplot2'))
#' knit_rd('ggplot2') # time-consuming!
#'
#' knit_rd_all()  # this may take really long time if you have many packages installed
#' }
#' @export
knit_rd = function(pkg, links = tools::findHTMLlinks(), frame = TRUE) {
  library(pkg, character.only = TRUE)
  optc = opts_chunk$get(); on.exit(opts_chunk$set(optc))
  file.copy(system.file('misc', 'R.css', package = 'knitr'), './')
  pkgRdDB = getFromNamespace('fetchRdDB', 'tools')(file.path(find.package(pkg), 'help', pkg))
  force(links); topics = names(pkgRdDB)
  for (p in topics) {
    message('** knitting documentation of ', p)
    tools::Rd2HTML(pkgRdDB[[p]], f <- tempfile(),
            package = pkg, Links = links, no_links = is.null(links), stages = 'render')
    txt = read_utf8(f)
    unlink(f)
    if (length(i <- grep('<h3>Examples</h3>', txt)) == 1L &&
      length(grep('</pre>', txt[i:length(txt)]))) {
      i0 = grep('<pre>', txt); i0 = i0[i0 > i][1L] - 1L
      i1 = grep('</pre>', txt); i1 = i1[i1 > i0][1L] + 1L
      tools::Rd2ex(pkgRdDB[[p]], ef <- tempfile())
      ex = read_utf8(ef)
      unlink(ef)
      ex = ex[-(1L:grep('### ** Examples', ex, fixed = TRUE))]
      ex = c('```{r}', ex, '```')
      opts_chunk$set(fig.path = paste0('figure/', p, '-'), tidy = FALSE)
      res = try(knit2html(text = ex, envir = parent.frame(2), fragment.only = TRUE, quiet = TRUE))
      if (inherits(res, 'try-error')) {
        res = ex; res[1] = '<pre><code class="r">'; res[length(res)] = '</code></pre>'
      }
      txt = c(txt[1:i0], res, txt[i1:length(txt)])
      txt = sub('</head>', '
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/8.3/styles/github.min.css">
<script src="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/8.3/highlight.min.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/8.3/languages/r.min.js"></script>
<script>hljs.initHighlightingOnLoad();</script>
</head>', txt)
    } else message('no examples found for ', p)
    write_utf8(txt, paste0(p, '.html'))
  }
  unlink('figure/', recursive = TRUE)
  toc = sprintf('- <a href="%s" target="content">%s</a>', paste0(topics, '.html'), topics)
  toc = c(paste0('# ', pkg), '', toc, '',
          paste('Generated with [knitr](https://yihui.org/knitr) ', packageVersion('knitr')))
  markdown::markdownToHTML(text = one_string(toc), output = '00frame_toc.html',
                           title = paste('R Documentation of', pkg),
                           options = NULL, extensions = NULL, stylesheet = 'R.css')
  txt = read_utf8(file.path(find.package(pkg), 'html', '00Index.html'))
  unlink('00Index.html')
  # fix image links
  write_utf8(gsub('../../../doc/html/', 'http://stat.ethz.ch/R-manual/R-devel/doc/html/',
                  txt, fixed = TRUE), '00Index.html')
  if (!frame) {
    unlink(c('00frame_toc.html', 'index.html'))
    # do not need a navigation frame, so make 00Index the real homepage
    (if (is_windows()) file.copy else file.symlink)('00Index.html', 'index.html')
    return(invisible())
  }
  write_utf8(sprintf(
'<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN" "http://www.w3.org/TR/html4/frameset.dtd">
<html>
<head><title>Documentation of the %s package</title></head>
<frameset cols="15%%,*">
  <frame src="00frame_toc.html">
  <frame src="00Index.html" name="content">
</frameset>
</html>
', pkg), 'index.html')
}

#' @rdname knit_rd
#' @export
knit_rd_all = function() {
  owd = getwd(); on.exit(setwd(owd))
  links = tools::findHTMLlinks()
  for (p in .packages(TRUE)) {
    message('* Making static html help pages for ', p)
    setwd(system.file('html', package = p))
    knit_rd(p, links, frame = FALSE)
  }
}

library(testit)

assert(
  'find_globals() identifies global variables',
  # nothing from outside environment
  identical(find_globals('x=1'), character(0)),
  # qwer must be created outside somewhere
  identical(find_globals('a=1; b=a; d=qwer'), 'qwer'),
  identical(find_globals('a=function(){f=2;g}'), 'g'),
  # y was assigned locally in z, but there is another y outside from nowhere
  identical(find_globals('z=function(){y=1};y'), 'y'),
  # more complicated cases: operators, subscripts, ...
  identical(find_globals(c('a=1%*%1%o%2 %in% d', 'b=d%%10+3%/%2-z[1:3]')), c('d', 'z'))
)

assert(
  'find_symbols() identifies all symbols',
  find_symbols('x = x + 1; rnorm(1, std = z)') %==% c('x', 'rnorm', 'z')
)

knit_lazy = function(lazy = TRUE) {
  in_dir(tempdir(), {
    txt = c(sprintf('```{r test, cache=TRUE, cache.lazy=%s}', lazy),
            'x1 = Sys.time()', '```')
    knit(text = txt, quiet = TRUE)
    x2 = x1
    Sys.sleep(0.1)
    knit(text = txt, quiet = TRUE)
    x1 == x2  # x1 should not be updated
  })
}
assert(
  'cache.lazy = TRUE/FALSE works',
  knit_lazy(TRUE), knit_lazy(FALSE)
)

knit_code$set(a = 1, b = 2, c = 3)
assert(
  'dep_prev() sets dependencies on previous chunks',
  # dependency is empty now
  identical(dep_list$get(), list()),
  # b/c depend on a, c depends on b
  identical({dep_prev(); dep_list$get()}, list(a = c('b', 'c'), b = 'c'))
)
dep_list$restore()
knit_code$restore()

# S3 method to deal with chunks and inline text respectively
process_group = function(x) {
  UseMethod('process_group', x)
}
#' @export
process_group.block = function(x) call_block(x)
#' @export
process_group.inline = function(x) {
  x = call_inline(x)
  knit_hooks$get('text')(x)
}


call_block = function(block) {
  # now try eval all options except those in eval.after and their aliases
  af = opts_knit$get('eval.after'); al = opts_knit$get('aliases')
  if (!is.null(al) && !is.null(af)) af = c(af, names(al[af %in% al]))

  params = opts_chunk$merge(block$params)
  params = dot_names(params)
  for (o in setdiff(names(params), af)) {
    params[o] = list(eval_lang(params[[o]]))
    # also update original options before being merged with opts_chunk
    if (o %in% names(block$params)) block$params[o] = params[o]
  }

  label = ref.label = params$label
  if (!is.null(params$ref.label)) {
    ref.label = sc_split(params$ref.label)
    # ref.label = I() implies opts.label = ref.label
    if (inherits(params$ref.label, 'AsIs') && is.null(params$opts.label))
      params$opts.label = ref.label
  }
  params[['code']] = get_code(params, label, ref.label)

  # opts.label = TRUE means inheriting chunk options from ref.label
  if (isTRUE(params$opts.label)) params$opts.label = ref.label
  # expand chunk options defined via opts_template and reference chunks
  params2 = NULL
  for (lab in params$opts.label) {
    # referenced chunk options (if any) override template options
    params3 = merge_list(opts_template$get(lab), attr(knit_code$get(lab), 'chunk_opts'))
    params2 = merge_list(params2, params3)
  }
  if (length(params2)) {
    # local options override referenced options
    params2 = merge_list(params2, block$params)
    # then override previously merged opts_chunk options
    params  = merge_list(params, params2)
    # in case any options are not evaluated
    for (o in setdiff(names(params), af)) params[o] = list(eval_lang(params[[o]]))
  }

  # save current chunk options in opts_current
  optc = opts_current$get(); on.exit(opts_current$restore(optc), add = TRUE)
  opts_current$restore(params)

  if (opts_knit$get('progress')) print(block)

  params[['code']] = parse_chunk(params[['code']]) # parse sub-chunk references

  ohooks = opts_hooks$get()
  for (opt in names(ohooks)) {
    hook = ohooks[[opt]]
    if (!is.function(hook)) {
      warning("The option hook '", opt, "' should be a function")
      next
    }
    if (!is.null(params[[opt]])) params = as.strict_list(hook(params))
    if (!is.list(params))
      stop("The option hook '", opt, "' should return a list of chunk options")
  }

  params = fix_options(params)  # for compatibility

  if (!is.null(params$child)) {
    if (!is_blank(params[['code']]) && getOption('knitr.child.warning', TRUE)) warning(
      "The chunk '", params$label, "' has the 'child' option, ",
      "and this code chunk must be empty. Its code will be ignored."
    )
    if (!params$eval) return('')
    cmds = lapply(sc_split(params$child), knit_child, options = block$params)
    out = one_string(unlist(cmds))
    return(out)
  }

  # Check cache
  if (params$cache > 0) {
    content = c(
      params[if (params$cache < 3) cache1.opts else setdiff(names(params), cache0.opts)],
      75L, if (params$cache == 2) params[cache2.opts]
    )
    if (params$engine == 'R' && isFALSE(params$cache.comments)) {
      content[['code']] = parse_only(content[['code']])
    }
    hash = paste(valid_path(params$cache.path, label), digest(content), sep = '_')
    params$hash = hash
    if (cache$exists(hash, params$cache.lazy) &&
        isFALSE(params$cache.rebuild) &&
        params$engine != 'Rcpp') {
      if (opts_knit$get('verbose')) message('  loading cache from ', hash)
      cache$load(hash, lazy = params$cache.lazy)
      cache_engine(params)
      if (!params$include) return('')
      if (params$cache == 3) return(cache$output(hash))
    }
    if (params$engine == 'R')
      cache$library(params$cache.path, save = FALSE) # load packages
  } else if (label %in% names(dep_list$get()) && !isFALSE(opts_knit$get('warn.uncached.dep')))
    warning2('code chunks must not depend on the uncached chunk "', label, '"')

  params$params.src = block$params.src
  opts_current$restore(params)  # save current options
  # prevent users from modifying opts_current (#1798)
  opts_current$lock(); on.exit(opts_current$lock(FALSE), add = TRUE)

  # set local options() for the current R chunk
  if (is.list(params$R.options)) {
    op = options(params$R.options); on.exit(options(op), add = TRUE)
  }

  block_exec(params)
}

# if chunk option 'file' is provided, read the file(s) as the chunk body;
# otherwise if 'code' is provided, use it; if neither 'file' nor 'code' is
# provided, use the chunk body
get_code = function(params, label, ref.label) {
  code = params[['code']]; file = params[['file']]
  if (is.null(code) && is.null(file))
    return(unlist(knit_code$get(ref.label), use.names = FALSE))
  if (!is.null(file)) code = in_input_dir(xfun::read_all(file))
  set_code(label, code)
  code
}

# replace code in knit_code but preserve attributes
set_code = function(label, code) {
  res = knit_code$get(label)
  attributes(code) = attributes(res)
  knit_code$set(setNames(list(code), label))
}

# options that should affect cache when cache level = 1,2
cache1.opts = c('code', 'eval', 'cache', 'cache.path', 'cache.globals', 'message', 'warning', 'error')
# more options affecting cache level 2
cache2.opts = c('fig.keep', 'fig.path', 'fig.ext', 'dev', 'dpi', 'dev.args', 'fig.width', 'fig.height')
# options that should not affect cache
cache0.opts = c('include', 'out.width.px', 'out.height.px', 'cache.rebuild')

block_exec = function(options) {
  if (options$engine == 'R') return(eng_r(options))

  # when code is not R language
  res.before = run_hooks(before = TRUE, options)
  engine = get_engine(options$engine)
  output = in_input_dir(engine(options))
  if (is.list(output)) output = unlist(output)
  res.after = run_hooks(before = FALSE, options)
  output = paste(c(res.before, output, res.after), collapse = '')
  output = knit_hooks$get('chunk')(output, options)
  if (options$cache) {
    cache.exists = cache$exists(options$hash, options$cache.lazy)
    if (options$cache.rebuild || !cache.exists) block_cache(options, output, switch(
      options$engine,
      'stan' = options$output.var, 'sql' = options$output.var, character(0)
    ))
  }
  if (options$include) output else ''
}

#' Engine for R
#'
#' This function handles the execution of R code blocks (when the chunk option
#' \code{engine} is \code{'R'}) and generates the R output for each code block.
#'
#' This engine function has one argument \code{options}: the source code of the
#' current chunk is in \code{options$code}. It returns a processed output that
#' can consist of data frames (as tables), graphs, or character output. This
#' function is intended for advanced use to allow developers to extend R, and
#' customize the pipeline with which R code is executed and processed within
#' knitr.
#'
#' @param options A list of chunk options. Usually this is just the object
#'   \code{options} associated with the current code chunk.
#' @noRd
eng_r = function(options) {
  # eval chunks (in an empty envir if cache)
  env = knit_global()
  obj.before = ls(globalenv(), all.names = TRUE)  # global objects before chunk

  keep = options$fig.keep
  keep.idx = NULL
  if (is.logical(keep)) keep = which(keep)
  if (is.numeric(keep)) {
    keep.idx = keep
    keep = "index"
  }

  if (keep.pars <- opts_knit$get('global.par')) on.exit({
    opts_knit$set(global.pars = par(no.readonly = TRUE))
  }, add = TRUE)

  tmp.fig = tempfile(); on.exit(unlink(tmp.fig), add = TRUE)
  # open a device to record plots if not using a global device or no device is
  # open, and close this device if we don't want to use a global device
  if (!opts_knit$get('global.device') || is.null(dev.list())) {
    # reset current device if any is open (#2166)
    if (!is.null(dev.list())) {
      dv0 = dev.cur(); on.exit(dev.set(dv0), add = TRUE)
    }
    chunk_device(options, keep != 'none', tmp.fig)
    dv = dev.cur()
    if (!opts_knit$get('global.device')) on.exit(dev.off(dv), add = TRUE)
    showtext(options)  # showtext support
  }
  # preserve par() settings from the last code chunk
  if (keep.pars) par2(opts_knit$get('global.pars'))

  res.before = run_hooks(before = TRUE, options, env) # run 'before' hooks

  code = options$code
  echo = options$echo  # tidy code if echo
  if (!isFALSE(echo) && !isFALSE(options$tidy) && length(code)) {
    tidy.method = if (isTRUE(options$tidy)) 'formatR' else options$tidy
    if (is.character(tidy.method)) tidy.method = switch(
      tidy.method,
      formatR = function(code, ...) {
        if (!loadable('formatR')) stop2(
          'The formatR package is required by the chunk option tidy = TRUE but ',
          'not installed; tidy = TRUE will be ignored.'
        )
        formatR::tidy_source(text = code, output = FALSE, ...)$text.tidy
      },
      styler = function(code, ...) unclass(styler::style_text(text = code, ...))
    )
    res = try_silent(do.call(tidy.method, c(list(code), options$tidy.opts)))

    if (!inherits(res, 'try-error')) code = res else warning(
      "Failed to tidy R code in chunk '", options$label, "'. Reason:\n", res
    )
  }
  # only evaluate certain lines
  if (is.numeric(ev <- options$eval)) {
    # group source code into syntactically complete expressions
    if (isFALSE(options$tidy)) code = sapply(xfun::split_source(code), one_string)
    iss = seq_along(code)
    code = comment_out(code, '##', setdiff(iss, iss[ev]), newline = FALSE)
  }
  # guess plot file type if it is NULL
  if (keep != 'none') options$fig.ext = dev2ext(options)

  cache.exists = cache$exists(options$hash, options$cache.lazy)
  evaluate = knit_hooks$get('evaluate')
  # return code with class 'source' if not eval chunks
  res = if (is_blank(code)) list() else if (isFALSE(ev)) {
    as.source(code)
  } else if (cache.exists && isFALSE(options$cache.rebuild)) {
    fix_evaluate(cache$output(options$hash, 'list'), options$cache == 1)
  } else in_input_dir(
    evaluate(
      code, envir = env, new_device = FALSE,
      keep_warning = if (is.numeric(options$warning)) TRUE else options$warning,
      keep_message = if (is.numeric(options$message)) TRUE else options$message,
      stop_on_error = if (is.numeric(options$error)) options$error else {
        if (options$error && options$include) 0L else 2L
      },
      output_handler = knit_handlers(options$render, options)
    )
  )
  if (options$cache %in% 1:2 && (!cache.exists || isTRUE(options$cache.rebuild))) {
    # make a copy for cache=1,2; when cache=2, we do not really need plots
    res.orig = if (options$cache == 2) remove_plot(res, keep == 'high') else res
  }

  # eval other options after the chunk
  if (!isFALSE(ev))
    for (o in opts_knit$get('eval.after'))
      options[o] = list(eval_lang(options[[o]], env))

  # remove some components according options
  if (isFALSE(echo)) {
    res = Filter(Negate(evaluate::is.source), res)
  } else if (is.numeric(echo)) {
    # choose expressions to echo using a numeric vector
    res = if (isFALSE(ev)) {
      as.source(code[echo])
    } else {
      filter_evaluate(res, echo, evaluate::is.source)
    }
  }
  if (options$results == 'hide') res = Filter(Negate(is.character), res)
  if (options$results == 'hold') {
    i = vapply(res, is.character, logical(1))
    if (any(i)) res = c(res[!i], merge_character(res[i]))
  }
  res = filter_evaluate(res, options$warning, evaluate::is.warning)
  res = filter_evaluate(res, options$message, evaluate::is.message)

  # rearrange locations of figures
  res = rearrange_figs(res, keep, keep.idx, options$fig.show)

  # number of plots in this chunk
  if (is.null(options$fig.num))
    options$fig.num = if (length(res)) sum(sapply(res, function(x) {
      if (inherits(x, 'knit_image_paths')) return(length(x))
      if (is_plot_output(x)) return(1)
      0
    })) else 0L

  # merge neighbor elements of the same class into one element
  for (cls in c('source', 'message', 'warning')) res = merge_class(res, cls)

  if (isTRUE(options$fig.beforecode)) res = fig_before_code(res)

  on.exit({
    plot_counter(reset = TRUE)
    shot_counter(reset = TRUE)
    opts_knit$delete('plot_files')
  }, add = TRUE)  # restore plot number

  output = unlist(sew(res, options)) # wrap all results together
  res.after = run_hooks(before = FALSE, options, env) # run 'after' hooks

  output = paste(c(res.before, output, res.after), collapse = '')  # insert hook results
  output = knit_hooks$get('chunk')(output, options)

  if (options$cache > 0) {
    # if cache.vars has been specifically provided, only cache these vars and no
    # need to look for objects in globalenv()
    obj.new = if (is.null(options$cache.vars)) setdiff(ls(globalenv(), all.names = TRUE), obj.before)
    copy_env(globalenv(), env, obj.new)
    objs = if (isFALSE(ev) || length(code) == 0) character(0) else
      options$cache.vars %n% codetools::findLocalsList(parse_only(code))
    # make sure all objects to be saved exist in env
    objs = intersect(c(objs, obj.new), ls(env, all.names = TRUE))
    if (options$autodep) {
      # you shall manually specify global object names if find_symbols() is not reliable
      cache$objects(
        objs, cache_globals(options$cache.globals, code), options$label,
        options$cache.path
      )
      dep_auto()
    }
    if (options$cache < 3) {
      if (options$cache.rebuild || !cache.exists) block_cache(options, res.orig, objs)
    } else block_cache(options, output, objs)
  }

  if (options$include) output else if (is.null(s <- options$indent)) '' else s
}

block_cache = function(options, output, objects) {
  hash = options$hash
  outname = cache_output_name(hash)
  assign(outname, output, envir = knit_global())
  purge_cache(options)
  cache$library(options$cache.path, save = TRUE)
  cache$save(objects, outname, hash, lazy = options$cache.lazy)
}

purge_cache = function(options) {
  # purge my old cache and cache of chunks dependent on me
  cache$purge(paste0(valid_path(
    options$cache.path, c(options$label, dep_list$get(options$label))
  ), '_????????????????????????????????'))
}

cache_globals = function(option, code) {
  if (is.character(option)) option else {
    (if (isFALSE(option)) find_symbols else find_globals)(code)
  }
}

# open a graphical device for a chunk to record plots
chunk_device = function(options, record = TRUE, tmp = tempfile()) {
  width = options$fig.width[1L]
  height = options$fig.height[1L]
  dev = fallback_dev(options$dev)
  dev.args = options$dev.args
  dpi = options$dpi

  # actually I should adjust the recording device according to dev, but here I
  # have only considered devices like png and tikz (because the measurement
  # results can be very different especially with the latter, see #1066), the
  # cairo_pdf device (#1235), and svg (#1705)
  if (identical(dev, 'png')) {
    do.call(grDevices::png, c(list(
      filename = tmp, width = width, height = height, units = 'in', res = dpi
    ), get_dargs(dev.args, 'png')))
  } else if (identical(dev, 'ragg_png')) {
    do.call(ragg_png_dev, c(list(
      filename = tmp, width = width, height = height, units = 'in', res = dpi
    ), get_dargs(dev.args, 'ragg_png')))
  } else if (identical(dev, 'tikz')) {
    dargs = c(list(
      file = tmp, width = width, height = height
    ), get_dargs(dev.args, 'tikz'))
    dargs$sanitize = options$sanitize; dargs$standAlone = options$external
    if (is.null(dargs$verbose)) dargs$verbose = FALSE
    do.call(tikz_dev, dargs)
  } else if (identical(dev, 'cairo_pdf')) {
    do.call(grDevices::cairo_pdf, c(list(
      filename = tmp, width = width, height = height
    ), get_dargs(dev.args, 'cairo_pdf')))
  } else if (identical(dev, 'svg')) {
    do.call(grDevices::svg, c(list(
      filename = tmp, width = width, height = height
    ), get_dargs(dev.args, 'svg')))
  } else if (identical(dev, 'svglite')) {
    do.call(svglite::svglite, c(list(
      filename = tmp, width = width, height = height
    ), get_dargs(dev.args, 'svglite')))
  } else if (identical(getOption('device'), pdf_null)) {
    if (!is.null(dev.args)) {
      dev.args = get_dargs(dev.args, 'pdf')
      dev.args = dev.args[intersect(names(dev.args), c('pointsize', 'bg'))]
    }
    do.call(pdf_null, c(list(width = width, height = height), dev.args))
  } else dev.new(width = width, height = height)
  dev.control(displaylist = if (record) 'enable' else 'inhibit')
}

# fall back to a usable device (e.g., during R CMD check)
fallback_dev = function(dev) {
  if (length(dev) != 1 || !getOption('knitr.device.fallback', is_R_CMD_check()))
    return(dev)
  choices = list(
    svg = c('png', 'jpeg', 'bmp'), cairo_pdf = c('pdf'), cairo_ps = c('postscript'),
    png = c('jpeg', 'svg', 'bmp'), jpeg = c('png', 'svg', 'bmp')
  )
  # add choices provided by users
  choices = merge_list(choices, getOption('knitr.device.choices'))
  if (!dev %in% names(choices)) return(dev)  # no fallback devices available
  # first test if the specified device actually works
  if (dev_available(dev)) return(dev)
  for (d in choices[[dev]]) if (dev_available(d)) {
    warning2("The device '", dev, "' is not operational; falling back to '", d, "'.")
    return(d)
  }
  dev  # no fallback device found; you'll to run into an error soon
}

# filter out some results based on the numeric chunk option as indices
filter_evaluate = function(res, opt, test) {
  if (length(res) == 0 || !is.numeric(opt) || !any(idx <- sapply(res, test)))
    return(res)
  idx = which(idx)
  idx = setdiff(idx, na.omit(idx[opt]))  # indices of elements to remove
  if (length(idx) == 0) res else res[-idx]
}

# find recorded plots in the output of evaluate()
find_recordedplot = function(x) {
  vapply(x, is_plot_output, logical(1))
}

is_plot_output = function(x) {
  evaluate::is.recordedplot(x) ||
    inherits(x, c('knit_image_paths', 'html_screenshot', 'knit_other_plot'))
}

# move plots before source code
fig_before_code = function(x) {
  s = vapply(x, evaluate::is.source, logical(1))
  if (length(s) == 0 || !any(s)) return(x)
  s = which(s)
  f = which(find_recordedplot(x))
  f = f[f >= min(s)]  # only move those plots after the first code block
  for (i in f) {
    j = max(s[s < i])
    tmp = x[i]; x[[i]] = NULL; x = append(x, tmp, j - 1)
    s = which(vapply(x, evaluate::is.source, logical(1)))
  }
  x
}

rearrange_figs = function(res, keep, idx, show) {
  figs = find_recordedplot(res)
  if (!any(figs)) return(res) # no figures
  if (keep == 'none') return(res[!figs]) # remove all

  if (show == 'hold') {
    res = c(res[!figs], res[figs]) # move to the end
    figs = find_recordedplot(res)
  }
  if (sum(figs) <= 1) return(res) # return early if only 1 figure to keep
  switch(
    keep,
    first = res[-tail(which(figs), -1L)],
    last  = res[-head(which(figs), -1L)],
    high  = merge_low_plot(res, figs),  # merge low-level plotting changes
    index = {
      i = which(figs)[-idx]
      if (length(i) > 0) res[-i] else res  # keep only selected
    },
    res
  )
}

# merge neighbor elements of the same class in a list returned by evaluate()
merge_class = function(res, class = c('source', 'message', 'warning')) {

  class = match.arg(class)
  idx = if (length(res)) which(sapply(res, inherits, what = class))
  if ((n <- length(idx)) <= 1) return(res)

  k1 = idx[1]; k2 = NULL; res1 = res[[k1]]
  el = c(source = 'src', message = 'message', warning = 'message')[class]
  for (i in 1:(n - 1)) {
    idx2 = idx[i + 1]; idx1 = idx[i]
    if (idx2 - idx1 == 1) {
      res2 = res[[idx2]]
      # merge warnings/messages only if next one is identical to previous one
      if (class == 'source' || identical(res1, res2) ||
          (class == 'message' && !grepl('\n$', tail(res1[[el]], 1)))) {
        res[[k1]][[el]] = c(res[[k1]][[el]], res2[[el]])
        k2 = c(k2, idx2)
      } else {
        k1 = idx2
        res1 = res[[k1]]
      }
    } else k1 = idx2
  }
  if (length(k2)) res = res[-k2] # remove lines that have been merged back
  res

}

# merge character output for output='hold', if the subsequent character is of
# the same class(es) as the previous one (e.g. should not merge normal
# characters with asis_output())
merge_character = function(res) {
  if ((n <- length(res)) <= 1) return(res)
  k = NULL
  for (i in 1:(n - 1)) {
    cls = class(res[[i]])
    if (identical(cls, class(res[[i + 1]]))) {
      res[[i + 1]] = paste0(res[[i]], res[[i + 1]])
      class(res[[i + 1]]) = cls
      k = c(k, i)
    }
  }
  if (length(k)) res = res[-k]
  res
}

call_inline = function(block) {
  optc = opts_current$get(); on.exit(opts_current$restore(optc), add = TRUE)
  params = opts_chunk$merge(list(label = unnamed_chunk()))
  opts_current$restore(params)
  if (opts_knit$get('progress')) print(block)
  in_input_dir(inline_exec(block))
}

inline_exec = function(
  block, envir = knit_global(), hook = knit_hooks$get('inline'),
  hook_eval = knit_hooks$get('evaluate.inline')
) {

  # run inline code and substitute original texts
  code = block$code; input = block$input
  if ((n <- length(code)) == 0) return(input) # untouched if no code is found
  code.src = block$code.src

  ans = character(n)
  for (i in 1:n) {
    tryCatch(parse_only(code[i]), error = function(e) {
      stop2('Failed to parse the inline R code: ', code.src[i], '\nReason: ', e$message)
    })
    res = hook_eval(code[i], envir)
    if (inherits(res, c('knit_asis', 'knit_asis_url'))) res = sew(res, inline = TRUE)
    tryCatch(as.character(res), error = function(e) {
      stop2("The inline value cannot be coerced to character: ", code[i])
    })
    if (length(res)) ans[i] = paste(hook(res), collapse = '')
  }
  # replace with evaluated results
  str_replace(input, block$location, ans)
}

process_tangle = function(x) {
  UseMethod('process_tangle', x)
}
#' @export
process_tangle.block = function(x) {
  params = opts_chunk$merge(x$params)
  for (o in c('purl', 'eval', 'child')) {
    if (inherits(try(params[o] <- list(eval_lang(params[[o]]))), 'try-error')) {
      params[['purl']] = FALSE  # if any of these options cannot be determined, don't purl
    }
  }
  if (isFALSE(params$purl)) return('')
  label = params$label; ev = params$eval
  if (params$engine != 'R') return(
    one_string(comment_out(knit_code$get(label), params$comment, newline = FALSE))
  )
  code = if (!isFALSE(ev) && !is.null(params$child)) {
    cmds = lapply(sc_split(params$child), knit_child)
    one_string(unlist(cmds))
  } else knit_code$get(label)
  # read external code if exists
  if (!isFALSE(ev) && length(code) && any(grepl('read_chunk\\(.+\\)', code))) {
    eval(parse_only(unlist(str_extract(code, 'read_chunk\\(([^)]+)\\)'))))
  }
  code = parse_chunk(code)
  if (isFALSE(ev)) code = comment_out(code, params$comment, newline = FALSE)
  if (opts_knit$get('documentation') == 0L) return(one_string(code))
  # e.g. when documentation 1 or 2 with purl()
  label_code(code, x)
}
#' @export
process_tangle.inline = function(x) {

  output = if (opts_knit$get('documentation') == 2L) {
    output = paste("#'", gsub('\n', "\n#' ", x$input, fixed = TRUE))
  } else ''

  code = x$code
  if (length(code) == 0L) return(output)

  if (getOption('knitr.purl.inline', FALSE)) output = c(output, code)

  idx = grepl('knit_child\\(.+\\)', code)
  if (any(idx)) {
    cout = sapply(code[idx], function(z) eval(parse_only(z)))
    output = c(output, cout, '')
  }

  one_string(output)
}


# add a label [and extra chunk options] to a code chunk
label_code = function(code, options) {
  comments = paste0(
    '## ----', options$params.src,
    strrep('-', max(getOption('width') - 11L - nchar(options$params.src), 0L)),
    '----'
  )
  one_string(c(comments, options$params.chunk, code, ''))
}

as.source = function(code) {
  list(structure(list(src = code), class = 'source'))
}

# SET OF HOOKS FOR RESTRUCTURED TEXT ---

#' @rdname hook_plot
#' @export
hook_plot_rst = function(x, options) {
  if (options$fig.show == 'animate') return(hook_plot_html(x, options))

  cap = .img.cap(options)
  # TODO: add all options for figure
  # See http://docutils.sourceforge.net/docs/ref/rst/directives.html#image
  # http://docutils.sourceforge.net/docs/ref/rst/directives.html#figure
  make_directive(
    'figure',
    paste0(opts_knit$get('base.url'), .upload.url(x)),
    c(align = if (options$fig.align == 'default') NULL else options$fig.align,
      alt = cap, width = options$out.width, height = options$out.height),
    cap
  )
}

#' @rdname output_hooks
#' @export
render_rst = function(strict = FALSE) {
  set_html_dev()
  hook.s = function(x, options) {
    one_string(c('\n\n::\n', indent_block(x), ''))
  }
  hook.t = function(x, options) {
    make_directive('sourcecode', tolower(options$engine), '', content = x)
  }
  hook.i = function(x) .inline.hook(format_sci(x, 'rst'))
  knit_hooks$set(
    source = function(x, options) {
      x = one_string(c(hilight_source(x, 'rst', options), ''))
      (if (strict) hook.s else hook.t)(x, options)
    },
    warning = hook.s, error = hook.s, message = hook.s,
    output = hook.s, inline = hook.i, plot = hook_plot_rst)
}

# Insert a reStructuredText directive for sphinx
#
# A reSt directive consists of  a name, arguments, option and some content.
# A typical reSt directive looks like this:
# .. <name>:: <arguments>
#    :<option>: <option values>
#
#     content
#
# This function accepts these arguments and returns the correctly formatted
# reStructuredText directive
#  Input
#      make_directive('figure', 'fig.png', c(align = 'center', alt = 'cap'))
#  Output
#  .. figure:: fig.png
#      :align: center
#      :alt: cap
make_directive = function(name, arg, opt, content = '') {
  l1 = sprintf('\n.. %s:: %s\n', name, arg)
  l2 = one_string(sprintf(':%s: %s', names(opt), opt))
  paste0(l1, indent_block(l2), '\n\n', indent_block(content))
}

## ideas borrowed from cacheSweave but not as sophisticated

## closure adapted from Hadley's decumar: https://github.com/hadley/decumar
## but it is using .rdb and .rdx as 'hard cache' (instead of cache in memory)
new_cache = function() {

  cache_path = function(hash) {
    d = dirname(hash)
    if (!file.exists(d)) dir.create(d, showWarnings = FALSE, recursive = TRUE)
    file.path(d, basename(hash))
  }

  cache_purge = function(hash) {
    for (h in hash) unlink(paste(cache_path(h), c('rdb', 'rdx', 'RData'), sep = '.'))
  }

  cache_save = function(keys, outname, hash, lazy = TRUE) {
    meta_name = cache_meta_name(hash)
    if (exists(meta_name, envir = knit_global())) outname = c(outname, meta_name)
    out0 = outname
    on.exit(rm(list = out0, envir = knit_global()), add = TRUE)
    # keys are new variables created; outname is the text output of a chunk
    path = cache_path(hash)
    # add random seed to cache if exists
    if (exists('.Random.seed', envir = globalenv(), inherits = FALSE)) {
      copy_env(globalenv(), knit_global(), '.Random.seed')
      outname = c('.Random.seed', outname)
    }
    if (!lazy) outname = c(keys, outname)
    save(list = outname, file = paste(path, 'RData', sep = '.'), envir = knit_global())
    if (!lazy) return()  # everything has been saved; no need to make lazy db
    # random seed is always load()ed
    keys = setdiff(keys, '.Random.seed')
    getFromNamespace('makeLazyLoadDB', 'tools')(knit_global(), path, variables = keys)
  }

  save_objects = function(objs, label, path) {
    if (length(objs) == 0L) objs = ''
    # save object names
    x = paste(c(label, objs), collapse = '\t')
    if (file.exists(path)) {
      lines = read_utf8(path)
      lines = lines[lines != label] # knitr < 1.5 may have lines == label
      idx = substr(lines, 1L, nchar(label) + 1L) == paste0(label, '\t')
      if (any(idx)) {
        lines[idx] = x  # update old objects
      } else lines = c(lines, x)
    } else lines = x
    write_utf8(lines, path)
  }
  cache_objects = function(keys, globals, label, path) {
    save_objects(keys, label, valid_path(path, '__objects'))
    save_objects(globals, label, valid_path(path, '__globals'))
  }

  cache_load = function(hash, lazy = TRUE) {
    path = cache_path(hash)
    if (!is_abs_path(path)) path = file.path(getwd(), path)
    if (lazy) lazyLoad(path, envir = knit_global())
    # load output from last run if exists
    if (file.exists(path2 <- paste(path, 'RData', sep = '.'))) {
      load(path2, envir = knit_global())
      if (exists('.Random.seed', envir = knit_global(), inherits = FALSE))
        copy_env(knit_global(), globalenv(), '.Random.seed')
      name = cache_meta_name(hash)
      if (exists(name, envir = knit_global())) {
        .knitEnv$meta = c(
          .knitEnv$meta, get(name, envir = knit_global(), inherits = FALSE)
        )
        rm(list = name, envir = knit_global())
      }
    }
  }

  cache_library = function(path, save = TRUE) {
    # save or load R packages
    path = valid_path(path, '__packages')
    if (save) {
      x = rev(.packages())
      if (file.exists(path)) x = setdiff(c(read_utf8(path), x), .base.pkgs)
      write_utf8(x, path)
    } else {
      if (!file.exists(path)) return()
      for (p in read_utf8(path))
        suppressPackageStartupMessages(library(p, character.only = TRUE))
    }
  }

  cache_exists = function(hash, lazy = TRUE) {
    is.character(hash) &&
      all(file.exists(paste(
        cache_path(hash), if (lazy) c('rdb', 'rdx') else 'RData', sep = '.'
      )))
  }

  # when cache=3, code output is stored in .[hash], so cache=TRUE won't lose
  # output as cacheSweave does; for cache=1,2, output is the evaluate() list
  cache_output = function(hash, mode = 'character') {
    name = cache_output_name(hash)
    res = get(name, envir = knit_global(), mode = mode, inherits = FALSE)
    # clean up this hidden variable after we obtain its value
    if (mode == mode(res)) rm(list = name, envir = knit_global())
    res
  }

  list(purge = cache_purge, save = cache_save, load = cache_load, objects = cache_objects,
       exists = cache_exists, output = cache_output, library = cache_library)
}
# analyze code and find out global variables
find_globals = function(code) {
  fun = eval(parse_only(c('function(){', code, '}')))
  setdiff(codetools::findGlobals(fun), known_globals)
}
known_globals = c(
  '{', '[', '(', ':', '<-', '=', '+', '-', '*', '/', '%%', '%/%', '%*%', '%o%', '%in%'
)

# analyze code and find out all possible variables (not necessarily global variables)
find_symbols = function(code) {
  if (is.null(code) || length(p <- parse(text = code, keep.source = TRUE)) == 0) return()
  p = getParseData(p)
  p = p[p$terminal & p$token %in% c('SYMBOL', 'SYMBOL_FUNCTION_CALL', 'SPECIAL'), ]
  unique(p$text)
}

# a variable name to store the metadata object from code chunks
cache_meta_name = function(hash) sprintf('.%s_meta', hash)
# a variable name to store the text output of code chunks
cache_output_name = function(hash) sprintf('.%s', hash)

cache = new_cache()

# a regex for cache files
cache_rx = '_[abcdef0123456789]{32}[.](rdb|rdx|RData)$'

#' Build automatic dependencies among chunks
#'
#' When the chunk option \code{autodep = TRUE}, all names of objects created in
#' a chunk will be saved in a file named \file{__objects} and all global objects
#' used in a chunk will be saved to \file{__globals}. This function can analyze
#' object names in these files to automatically build cache dependencies, which
#' is similar to the effect of the \code{dependson} option. It is supposed to be
#' used in the first chunk of a document and this chunk must not be cached.
#' @param path Path to the dependency file.
#' @return \code{NULL}. The dependencies are built as a side effect.
#' @note Be cautious about \code{path}: because this function is used in a
#'   chunk, the working directory when the chunk is evaluated is the directory
#'   of the input document in \code{\link{knit}}, and if that directory differs
#'   from the working directory before calling \code{knit()}, you need to adjust
#'   the \code{path} argument here to make sure this function can find the cache
#'   files \file{__objects} and \file{__globals}.
#' @export
#' @seealso \code{\link{dep_prev}}
#' @references \url{https://yihui.org/knitr/demo/cache/}
dep_auto = function(path = opts_chunk$get('cache.path')) {
  # this function should be evaluated in the original working directory
  owd = setwd(opts_knit$get('output.dir')); on.exit(setwd(owd))
  paths = valid_path(path, c('__objects', '__globals'))
  locals = parse_objects(paths[1L]); globals = parse_objects(paths[2L])
  if (is.null(locals) || is.null(globals)) return(invisible(NULL))
  if (!identical(names(locals), names(globals))) {
    warning('corrupt dependency files? \ntry remove ', paste(paths, collapse = '; '))
    return(invisible(NULL))
  }
  nms = intersect(names(knit_code$get()), names(locals)) # guarantee correct order
  # locals may contain old chunk names; the intersection can be of length < 2
  if (length(nms) < 2) return(invisible(NULL))
  for (i in 2:length(nms)) {
    if (length(g <- globals[[nms[i]]]) == 0) next
    for (j in 1:(i - 1L)) {
      # check if current globals are in old locals
      if (any(g %in% locals[[nms[j]]]))
        dep_list$set(setNames(list(unique(c(dep_list$get(nms[j]), nms[i]))), nms[j]))
    }
  }
}
# parse objects in dependency files
parse_objects = function(path) {
  if (!file.exists(path)) {
    warning('file ', path, ' not found'); return()
  }
  lines = strsplit(read_utf8(path), '\t')
  if (length(lines) < 2L) return()  # impossible for dependson
  objs = lapply(lines, `[`, -1L)
  names(objs) = lapply(lines, `[`, 1L)
  objs
}

#' Load the cache database of a code chunk
#'
#' If a code chunk has turned on the chunk option \code{cache = TRUE}, a cache
#' database will be established after the document is compiled. You can use this
#' function to manually load the database anywhere in the document (even before
#' the code chunk). This makes it possible to use objects created later in the
#' document earlier, e.g. in an inline R expression before the cached code
#' chunk, which is normally not possible because \pkg{knitr} compiles the
#' document in a linear fashion, and objects created later cannot be used before
#' they are created.
#' @param label The chunk label of the code chunk that has a cache database.
#' @param object The name of the object to be fetched from the database. If it is
#'   missing, \code{NULL} is returned).
#' @param notfound A value to use when the \code{object} cannot be found.
#' @param path Path of the cache database (normally set in the global chunk
#'   option \code{cache.path}).
#' @param lazy Whether to \code{\link{lazyLoad}} the cache database (depending
#'   on the chunk option \code{cache.lazy = TRUE} or \code{FALSE} of that code
#'   chunk).
#' @note Apparently this function loads the value of the object from the
#'   \emph{previous} run of the document, which may be problematic when the
#'   value of the object becomes different the next time the document is
#'   compiled. Normally you must compile the document twice to make sure the
#'   cache database is created, and the object can be read from it. Please use
#'   this function with caution.
#' @references See the example #114 at
#'   \url{https://github.com/yihui/knitr-examples}.
#' @return Invisible \code{NULL} when \code{object} is not specified (the cache
#'   database will be loaded as a side effect), otherwise the value of the
#'   object if found.
#' @export
load_cache = function(
  label, object, notfound = 'NOT AVAILABLE', path = opts_chunk$get('cache.path'),
  lazy = TRUE
) {
  owd = setwd(opts_knit$get('output.dir')); on.exit(setwd(owd))
  path = valid_path(path, label)
  p0 = dirname(path); p1 = basename(path)
  p2 = list.files(p0, cache_rx)
  if (length(p2) == 0) return(notfound)
  p2 = p2[substr(p2, 1, nchar(p1)) == p1]
  if (length(p2) == 0) return(notfound)
  if (length(p2) > 3) stop(
    'Wrong cache databases for the chunk ', label,
    '. You need to remove redundant cache files. Found ', paste(p2, collapse = ', ')
  )
  p2 = unique(gsub('[.](rdb|rdx|RData)$', '', p2))
  if (length(p2) != 1) stop('Cannot identify the cache database for chunk ', label)
  cache$load(file.path(p0, p2), lazy)
  if (missing(object)) return(invisible(NULL))
  if (exists(object, envir = knit_global(), inherits = FALSE)) {
    get(object, envir = knit_global(), inherits = FALSE)
  } else notfound
}

#' Make later chunks depend on previous chunks
#'
#' This function can be used to build dependencies among chunks so that all
#' later chunks depend on previous chunks, i.e. whenever the cache of a previous
#' chunk is updated, the cache of all its later chunks will be updated.
#' @return \code{NULL}; the internal dependency structure is updated as a side
#'   effect.
#' @export
#' @seealso \code{\link{dep_auto}}
#' @references \url{https://yihui.org/knitr/demo/cache/}
dep_prev = function() {
  labs = names(knit_code$get())
  if ((n <- length(labs)) < 2L) return() # one chunk or less; no sense of deps
  opts_knit$set(warn.uncached.dep = FALSE)
  for (i in 1L:(n - 1L)) {
    dep_list$set(setNames(list(labs[(i + 1L):n]), labs[i]))
  }
}

#' An unevaluated expression to return .Random.seed if exists
#'
#' This expression returns \code{.Random.seed} when \code{eval(rand_seed)} and
#' \code{NULL} otherwise.
#'
#' It is designed to work with \code{opts_chunk$set(cache.extra = rand_seed)}
#' for reproducibility of chunks that involve with random number generation. See
#' references.
#' @export
#' @format NULL
#' @references \url{https://yihui.org/knitr/demo/cache/}
#' @examples eval(rand_seed)
#' rnorm(1) # .Random.seed is created (or modified)
#' eval(rand_seed)
rand_seed = quote({
  .GlobalEnv$.Random.seed
})

#' Clean cache files that are probably no longer needed
#'
#' If you remove or rename some cached code chunks, their original cache files
#' will not be automatically cleaned. You can use this function to identify
#' these possible files, and clean them if you are sure they are no longer
#' needed.
#' @param clean Boolean; whether to remove the files.
#' @param path Path to the cache.
#' @note  The identification is not guaranteed to be correct, especially when
#'   multiple documents share the same cache directory. You are recommended to
#'   call \code{clean_cache(FALSE)} and carefully check the list of files (if
#'   any) before you really delete them (\code{clean_cache(TRUE)}).
#'
#'   This function must be called within a code chunk in a source document,
#'   since it needs to know all chunk labels of the current document to
#'   determine which labels are no longer present, and delete cache
#'   corresponding to these labels.
#' @export
clean_cache = function(clean = FALSE, path = opts_chunk$get('cache.path')) {
  odir = opts_knit$get('output.dir')
  if (is.null(odir)) {
    warning('This function must be called inside a source document')
    return()
  }
  owd = setwd(odir); on.exit(setwd(owd))
  if (file_test('-d', path)) {
    p0 = path; p1 = ''
  } else {
    p0 = dirname(path); p1 = basename(path)
  }
  files = list.files(p0, cache_rx, full.names = TRUE)
  if (length(files) == 0) return()
  base = basename(files)
  labs = .knitEnv$labels
  if (length(labs) == 0) return()
  i = !(sub(cache_rx, '', base) %in% paste0(p1, labs))
  if (p1 != '') i = i & (substr(base, 1, nchar(p1)) == p1)
  if (!any(i)) return()
  if (clean) unlink(files[i]) else message(
    'Clean these cache files?\n\n', one_string(files[i]), '\n'
  )
}

# copy objects in one environment to the other
copy_env = function(from, to, keys = ls(envir = from, all.names = TRUE)) {
  if (identical(from, to)) return()
  for (i in keys) assign(i, get(i, envir = from, inherits = FALSE), envir = to)
}


knit_counter = function(init = 0L) {
  n = init
  function(reset = FALSE) {
    if (reset) return(n <<- init)
    n <<- n + 1L
    n - 1L
  }
}

plot_counter = knit_counter(1L)
shot_counter = knit_counter(1L)
chunk_counter = knit_counter(1L)

# a vectorized and better version than evaluate:::line_prompt
line_prompt = function(x, prompt = getOption('prompt'), continue = getOption('continue')) {
  # match a \n, then followed by any character (use zero width assertion)
  paste0(prompt, gsub('(?<=\n)(?=.|\n)', continue, x, perl = TRUE))
}

# add a prefix to output
comment_out = function(x, prefix = '##', which = TRUE, newline = TRUE) {
  x = gsub('[\n]{2,}$', '\n', x)
  if (newline) x = gsub('([^\n])$', '\\1\n', x)  # add \n if not exists
  if (is.null(prefix) || !nzchar(prefix) || is.na(prefix)) return(x)
  prefix = paste(prefix, '')
  x = gsub(' +([\n]*)$', '\\1', x)
  x[which] = line_prompt(x[which], prompt = prefix, continue = prefix)
  x
}

# assign string in comments to a global variable
comment_to_var = function(x, varname, pattern, envir) {
  if (grepl(pattern, x)) {
    assign(varname, sub(pattern, '', x), envir = envir)
    return(TRUE)
  }
  FALSE
}

is_blank = function(x) {
  if (length(x)) all(grepl('^\\s*$', x)) else TRUE
}
valid_path = function(prefix, label) {
  if (length(prefix) == 0L || is.na(prefix) || prefix == 'NA') prefix = ''
  paste0(prefix, label)
}

# define a color variable in TeX
color_def = function(col, variable = 'shadecolor') {
  if (is.na(col)) return('')  # no LaTeX code when color is NA
  x = if (length(col) == 1L) sc_split(col) else col
  if ((n <- length(x)) != 3L) {
    if (n == 1L) x = drop(col2rgb(x) / 255) else {
      x = switch(variable, shadecolor = rep(.97, 3), fgcolor = rep(0, 3))
      warning("the color '", col, "' is invalid;",
              'using default color...',
              'see https://yihui.org/knitr/options')
    }
  }
  if (length(x) != 3L) stop('invalid color:', col)
  if (is.numeric(x)) x = round(x, 3L)
  outdec = options(OutDec = '.'); on.exit(options(outdec))
  sprintf('\\definecolor{%s}{rgb}{%s, %s, %s}', variable, x[1], x[2], x[3])
}

# split by semicolon or colon
sc_split = function(string) {
  if (is.call(string)) string = eval(string)
  if (is.numeric(string) || length(string) != 1L) return(string)
  stringr::str_trim(stringr::str_split(string, ';|,')[[1]])
}

# extract LaTeX packages for tikzDevice
set_preamble = function(input, patterns = knit_patterns$get()) {
  if (!out_format('latex')) return()
  .knitEnv$tikzPackages = .knitEnv$bibliography = NULL
  if (length(db <- patterns$document.begin) != 1L) return()  # no \begin{document} pattern
  if (length(hb <- patterns$header.begin) != 1L) return()  # no \documentclass{} pattern
  idx2 = grep(db, input)[1]
  if (is.na(idx2) || idx2 < 2L) return()
  idx1 = grep(hb, input)[1]
  if (is.na(idx1) || idx1 >= idx2) return()
  txt = one_string(input[idx1:(idx2 - 1L)])  # rough preamble
  idx = stringr::str_locate(txt, hb)  # locate documentclass
  options(tikzDocumentDeclaration = stringr::str_sub(txt, idx[, 1L], idx[, 2L]))
  preamble = pure_preamble(split_lines(stringr::str_sub(txt, idx[, 2L] + 1L)), patterns)
  .knitEnv$tikzPackages = c(.header.sweave.cmd, preamble, '\n')
  .knitEnv$bibliography = grep('^\\\\bibliography.+', input, value = TRUE)
}
# filter out code chunks from preamble if they exist (they do in LyX/Sweave)
pure_preamble = function(preamble, patterns) {
  res = split_file(lines = preamble, set.preamble = FALSE, patterns) # should avoid recursion
  if (!parent_mode()) {
    # when not in parent mode, just return normal texts and skip code
    return(unlist(res))
  }
  owd = setwd(input_dir()); on.exit(setwd(owd))
  progress = opts_knit$get('progress')  # suppress printing of blocks and texts
  opts_knit$set(progress = FALSE); on.exit(opts_knit$set(progress = progress), add = TRUE)
  # run the code in the preamble
  sapply(res, if (opts_knit$get('tangle')) process_tangle else process_group)
}

#' Specify the parent document of child documents
#'
#' This function extracts the LaTeX preamble of the parent document to use for
#' the child document, so that the child document can be compiled as an
#' individual document.
#'
#' When the preamble of the parent document also contains code chunks and inline
#' R code, they will be evaluated as if they were in this child document. For
#' examples, when \pkg{knitr} hooks or other options are set in the preamble of
#' the parent document, it will apply to the child document as well.
#' @param parent Path to the parent document, relative to the current child
#'   document.
#' @return The preamble is extracted and stored to be used later when the
#'   complete output is written.
#' @note Obviously this function is only useful when the output format is LaTeX.
#'   This function only works when the child document is compiled in a
#'   standalone mode using \code{\link{knit}()} (instead of being called in
#'   \code{\link{knit_child}()}); when the parent document is compiled, this
#'   function in the child document will be ignored.
#' @references \url{https://yihui.org/knitr/demo/child/}
#' @export
#' @examples ## can use, e.g. \Sexpr{set_parent('parent_doc.Rnw')} or
#'
#' # <<setup-child, include=FALSE>>=
#'
#' # set_parent('parent_doc.Rnw')
#'
#' # @@
set_parent = function(parent) {
  if (child_mode()) return(invisible(NULL)) # quit if in child mode
  opts_knit$set(parent = TRUE)
  set_preamble(read_utf8(parent))
  invisible(NULL)
}

# whether to write results as-is?
output_asis = function(x, options) {
  is_blank(x) || options$results == 'asis'
}

# path relative to dir of the input file
input_dir = function(ignore_root = FALSE) {
  root = opts_knit$get('root.dir')
  # LyX is a special case: the input file is in tempdir, and we should use
  # root.dir as the real input dir (#809)
  if (is_lyx()) return(root)
  if (ignore_root) {
    .knitEnv$input.dir %n% '.'
  } else {
    root %n% (if (!getOption('knitr.use.cwd', FALSE)) .knitEnv$input.dir) %n% '.'
  }
}

is_lyx = function() {
  args = commandArgs(TRUE)
  if (length(args) < 4) return(FALSE)
  grepl('[.]Rnw$', args[1]) && !is.na(Sys.getenv('LyXDir', NA))
}

# round a number to getOption('digits') decimal places by default, and format()
# it using significant digits if the option knitr.digits.signif = TRUE
round_digits = function(x) {
  if (getOption('knitr.digits.signif', FALSE)) format(x) else {
    as.character(round(x, getOption('digits')))
  }
}

# scientific notation in TeX, HTML and reST
format_sci_one = function(
  x, format = 'latex', times = getOption('knitr.inline.times', '\\times ')
) {

  if (!(class(x)[1] == 'numeric') || is.na(x) || x == 0) return(as.character(x))

  if (is.infinite(x)) {
    return(
      switch(format, latex = {
        sprintf("%s\\infty{}", ifelse(x < 0, "-", ""))
      }, html = {
        sprintf("%s&infin;", ifelse(x < 0, "-", ""))
      }, as.character(x)))
  }

  if (abs(lx <- floor(log10(abs(x)))) < getOption('scipen') + 4L)
    return(round_digits(x)) # no need sci notation

  b = round_digits(x / 10^lx)
  b[b %in% c(1, -1)] = ''

  switch(format, latex = {
    sci_notation('%s%s10^{%s}', b, times, lx)
  },
  html = sci_notation('%s%s10<sup>%s</sup>', b, ' &times; ', lx),
  md   = sci_notation('%s%s10^%s^', b, '&times; ', lx),
  rst  = {
    # if AsIs, use the :math: directive
    if (inherits(x, 'AsIs')) {
      s = sci_notation('%s%s10^{%s}', b, times, lx)
      sprintf(':math:`%s`', s)
    } else {
      # This needs the following line at the top of the file to define |times|
      # .. include <isonum.txt>
      sci_notation('%s%s10 :sup:`%s`', b, ' |times| ', lx)
    }
  }, as.character(x))
}

sci_notation = function(format, base, times, power) {
  sprintf(format, base, ifelse(base == '', '', times), power)
}

# vectorized version of format_sci_one()
format_sci = function(x, ...) {
  if (inherits(x, 'roman')) return(as.character(x))
  vapply(x, format_sci_one, character(1L), ..., USE.NAMES = FALSE)
}

# absolute path?
is_abs_path = function(x) {
  if (is_windows()) grepl('^(\\\\|[A-Za-z]:)', x) else grepl('^[/~]', x)
}

# paths of web resources?
is_web_path = function(x) {
  grepl('^(f|ht)tps?://', x)
}

# is tikz device without externalization?
is_tikz_dev = function(options) {
  'tikz' %in% options$dev && !options$external
}

tikz_dict = function(path) {
  paste(sans_ext(basename(path)), 'tikzDictionary', sep = '-')
}

# compatibility with Sweave and old beta versions of knitr
fix_options = function(options) {
  options = as.strict_list(options)

  # if you want to use subfloats, fig.show must be 'hold'
  if (length(options$fig.subcap)) options$fig.show = 'hold'
  # if the animation hook has been set, fig.show must be 'animate'
  if (!is.null(options$animation.hook)) options$fig.show = 'animate'
  # the default device NULL is not valid; use pdf is not set
  if (is.null(options$dev)) options$dev = 'pdf'
  # FALSE means hide for options$results
  if (isFALSE(options$results)) options$results = 'hide'

  # the figure/cache filenames may contain UTF-8 chars, which won't work on
  # Windows, e.g. png() fails if filename contains UTF-8 chars (must use native
  # encoding); in rmarkdown::render(input), basename(input) may implicitly
  # convert the encoding of the filename `input` to UTF-8
  options$fig.path = native_encode(options$fig.path)
  options$cache.path = native_encode(options$cache.path)

  # cache=TRUE -> 3; FALSE -> 0
  if (is.logical(options$cache)) options$cache = options$cache * 3
  # non-R code should not use cache=1,2
  if (options$engine != 'R') options$cache = (options$cache > 0) * 3

  options$eval = unname(options$eval)

  # handle aspect ratio/figure dimensions; give priority to fig.dim, with
  # warning (seems relatively more likely as user's intention in case when both
  # present) if aspect ratio is specified, calculate figure height
  fix_asp = is.numeric(options$fig.asp)
  if (length(options$fig.dim) == 2L) {
    if (fix_asp) warning('The chunk option fig.asp is ignored since fig.dim is provided.')
    options$fig.width = options$fig.dim[1L]; options$fig.height = options$fig.dim[2L]
  } else {
    if (fix_asp) options$fig.height = options$fig.width * options$fig.asp
  }

  # out.[width|height].px: unit in pixels for sizes
  for (i in c('width', 'height')) {
    options[[sprintf('out.%s.px', i)]] = options[[o <- sprintf('out.%s', i)]] %n%
      (options[[sprintf('fig.%s', i)]] * options$dpi)
    # turn x% to x/100\linewidth or \textheight
    if (is_latex_output()) options[o] = list(latex_percent_size(options[[o]], i))
  }
  # for Retina displays, increase physical size, and decrease output size
  if (is.numeric(r <- options$fig.retina) && r != 1) {
    if (is.null(options[['out.width']])) {
      options$out.width = options$fig.width * options$dpi
    }
    options$dpi = options$dpi * r
  } else {
    options$fig.retina = 1
  }

  # deal with aliases: a1 is real option; a0 is alias
  if (length(a1 <- opts_knit$get('aliases')) && length(a0 <- names(a1))) {
    for (i in seq_along(a1)) {
      # use alias only if the name exists in options
      if (a0[i] %in% names(options)) options[[a1[i]]] = options[[a0[i]]]
    }
  }

  options
}

#' Check if the current output type is LaTeX or HTML
#'
#' The function \code{is_latex_output()} returns \code{TRUE} when the output
#' format is LaTeX; it works for both \file{.Rnw} and R Markdown documents (for
#' the latter, the two Pandoc formats \code{latex} and \code{beamer} are
#' considered LaTeX output). The function \code{is_html_output()} only works for
#' R Markdown documents.
#'
#' These functions may be useful for conditional output that depends on the
#' output format. For example, you may write out a LaTeX table in an R Markdown
#' document when the output format is LaTeX, and an HTML or Markdown table when
#' the output format is HTML.
#'
#' Internally, the Pandoc output format of the current R Markdown document is
#' stored in \code{knitr::\link{opts_knit}$get('rmarkdown.pandoc.to')}. By
#' default, these formats are considered as HTML formats: \code{c('markdown',
#' 'epub', 'html', 'html5', 'revealjs', 's5', 'slideous', 'slidy')}.
#' @rdname output_type
#' @export
#' @examples knitr::is_latex_output()
#' knitr::is_html_output()
#' knitr::is_html_output(excludes = c('markdown', 'epub'))
is_latex_output = function() {
  out_format('latex') || pandoc_to(c('latex', 'beamer'))
}

#' @param fmt A character vector of output formats to be checked. By default, this
#'   is the current Pandoc output format.
#' @param excludes A character vector of output formats that should not be
#'   considered as HTML format.
#' @rdname output_type
#' @export
is_html_output = function(fmt = pandoc_to(), excludes = NULL) {
  if (length(fmt) == 0) return(FALSE)
  if (grepl('^markdown', fmt)) fmt = 'markdown'
  if (fmt == 'epub3') fmt = 'epub'
  fmts = c('markdown', 'epub', 'html', 'html4', 'html5', 'revealjs', 's5', 'slideous', 'slidy', 'gfm')
  fmt %in% setdiff(fmts, excludes)
}


# turn percent width/height to LaTeX unit, e.g. out.width = 30% -> .3\linewidth
latex_percent_size = function(x, which = c('width', 'height')) {
  if (!is.character(x)) return(x)
  i = grep('^[0-9.]+%$', x)
  if (length(i) == 0) return(x)
  xi = as.numeric(sub('%$', '', x[i]))
  if (any(is.na(xi))) return(x)
  which = match.arg(which)
  x[i] = paste0(xi / 100, if (which == 'width') '\\linewidth' else '\\textheight')
  x
}

# eval options as symbol/language objects
eval_lang = function(x, envir = knit_global()) {
  if (!is.symbol(x) && !is.language(x)) return(x)
  eval(x, envir = envir)
}

# check latex packages; if not exist, copy them over to ./
test_latex_pkg = function(name, path) {
  res = try_silent(system(sprintf('%s %s.sty', kpsewhich(), name), intern = TRUE))
  if (inherits(res, 'try-error') || !length(res)) {
    warning("unable to find LaTeX package '", name, "'; will use a copy from knitr")
    file.copy(path, '.')
  }
}

# get child and parent mode
child_mode = function() opts_knit$get('child')
parent_mode = function() opts_knit$get('parent')

# return the output format, or if current format is in specified formats
out_format = function(x) {
  fmt = opts_knit$get('out.format')
  if (missing(x)) fmt else !is.null(fmt) && (fmt %in% x)
}

# tempfile under the current working directory
wd_tempfile = function(...) basename(tempfile(tmpdir = '.', ...))

# rmarkdown sets an option for the Pandoc output format from markdown
pandoc_to = function(x) {
  fmt = opts_knit$get('rmarkdown.pandoc.to')
  if (missing(x)) fmt else !is.null(fmt) && (fmt %in% x)
}

# rmarkdown's input format
pandoc_from = function() {
  opts_knit$get('rmarkdown.pandoc.from') %n% 'markdown'
}

pandoc_fragment = function(text, to = pandoc_to(), from = pandoc_from()) {
  if (length(text) == 0) return(text)
  f1 = wd_tempfile('pandoc', '.md'); f2 = wd_tempfile('pandoc')
  on.exit(unlink(c(f1, f2)), add = TRUE)
  write_utf8(text, f1)
  rmarkdown::pandoc_convert(f1, to, from, f2, options = if (is_html_output(to)) '--mathjax')
  file_string(f2)
}

#' Path for figure files
#'
#' The filename of figure files is the combination of options \code{fig.path}
#' and \code{label}. This function returns the path of figures for the current
#' chunk by default.
#' @param suffix A filename suffix; if it is non-empty and does not
#'   contain a dot \code{.}, it will be treated as the filename extension (e.g.
#'   \code{png} will be used as \code{.png})
#' @param options A list of options; by default the options of the current chunk.
#' @param number The current figure number. The default is the internal chunk option
#'   \code{fig.cur}, if this is available.
#' @return A character vector of the form \file{fig.path-label-i.suffix}.
#' @note When there are special characters (not alphanumeric or \samp{-} or
#'   \samp{_}) in the path, they will be automatically replaced with \samp{_}.
#'   For example, \file{a b/c.d-} will be sanitized to \file{a_b/c_d-}. This
#'   makes the filenames safe to LaTeX.
#' @export
#' @examples fig_path('.pdf', options = list(fig.path='figure/abc-', label='first-plot'))
#' fig_path('.png', list(fig.path='foo-', label='bar'), 1:10)
fig_path = function(suffix = '', options = opts_current$get(), number) {
  if (suffix != '' && !grepl('[.]', suffix)) suffix = paste0('.', suffix)
  if (missing(number)) number = options$fig.cur %n% 1L
  if (!is.null(number)) suffix = paste0('-', number, suffix)
  path = valid_path(options$fig.path, options$label)
  (if (out_format(c('latex', 'sweave', 'listings'))) sanitize_fn else
    paste0)(path, suffix)
}
# sanitize filename for LaTeX
sanitize_fn = function(path, suffix = '') {
  if (grepl('[^~:_./\\[:alnum:]-]', path)) {
    warning('replaced special characters in figure filename "', path, '" -> "',
            path <- gsub('[^~:_./\\[:alnum:]-]', '_', path), '"')
  }
  # replace . with _ except ../ and ./
  s = strsplit(path, '[/\\\\]')[[1L]]
  i = (s != '.') & (s != '..') & grepl('\\.', s)
  if (any(i)) {
    s[i] = gsub('\\.', '_', s[i])
    path = paste(s, collapse = '/')
    warning('dots in figure paths replaced with _ ("', path, '")')
  }
  paste0(path, suffix)
}

#' Obtain the figure filenames for a chunk
#'
#' Given a chunk label, the figure file extension, the figure number(s), and the
#' chunk option \code{fig.path}, return the filename(s).
#'
#' This function can be used in an inline R expression to write out the figure
#' filenames without hard-coding them. For example, if you created a plot in a
#' code chunk with the label \code{foo} and figure path \file{my-figure/}, you
#' are not recommended to use hard-coded figure paths like
#' \samp{\includegraphics{my-figure/foo-1.pdf}} (in \file{.Rnw} documents) or
#' \samp{![](my-figure/foo-1.png)} (R Markdown) in your document. Instead, you
#' should use \samp{\\Sexpr{fig_chunk('foo', 'pdf')}} or \samp{![](`r
#' fig_chunk('foo', 'png')`)}.
#'
#' You can generate plots in a code chunk but not show them inside the code
#' chunk by using the chunk option \code{fig.show = 'hide'}. Then you can use
#' this function if you want to show them elsewhere.
#' @param label The chunk label.
#' @param ext The figure file extension, e.g. \code{png} or \code{pdf}.
#' @param number The figure number (by default \code{1}).
#' @param fig.path Passed to \code{\link{fig_path}}. By default, the chunk
#'   option \code{fig.path} is used.
#' @return A character vector of filenames.
#' @export
#' @examples library(knitr)
#' fig_chunk('foo', 'png')
#' fig_chunk('foo', 'pdf')
#' fig_chunk('foo', 'svg', 2)  # the second plot of the chunk foo
#' fig_chunk('foo', 'png', 1:5)  # if the chunk foo produced 5 plots
fig_chunk = function(label, ext = '', number, fig.path = opts_chunk$get('fig.path')) {
  fig_path(ext, list(fig.path = fig.path, label = label), number)
}

#' The global environment in which code chunks are evaluated
#'
#' This function makes the environment of a code chunk accessible inside a
#' chunk.
#'
#' It returns the \code{envir} argument of \code{\link{knit}}, e.g. if we call
#' \code{\link{knit}()} in the global environment, \code{knit_global()} returns
#' R's global environment by default. You can call functions like
#' \code{\link{ls}()} on this environment.
#' @export
knit_global = function() {
  .knitEnv$knit_global %n% globalenv()
}

# Indents a Block
#  Input
#     "library(ggplot2)\nqplot(wt, mpg, data = mtcars)"
#  Output
#          library(ggplot2)
#          qplot(wt, mpg, data  = mtcars)
indent_block = function(block, spaces = '    ') {
  if (is.null(block) || !any(nzchar(block))) return(rep(spaces, length(block)))
  if (spaces == '') return(block)
  line_prompt(block, spaces, spaces)
}

# print knitr logs
print_knitlog = function() {
  if (!opts_knit$get('verbose') || child_mode() || !length(klog <- knit_log$get(drop = FALSE)))
    return()

  for (i in unlist(klog, use.names = FALSE)) {
    cat(sub('\n+$', '', i), '\n\n')
    if (length(code <- knit_code$get(sub('^Chunk ([^:]+):\n.*', '\\1', i))))
      cat(code, sep = '\n')
  }
  cat('\nNumber of messages:\n')
  print(sapply(klog, length))
}

# count the number of lines
line_count = function(x) stringr::str_count(x, '\n') + 1L

has_package = function(pkg) loadable(pkg, FALSE)

# if LHS is NULL, return the RHS
`%n%` = function(x, y) if (is.null(x)) y else x

# merge elements of y into x with the same names
merge_list = function(x, y) {
  x[names(y)] = y
  x
}

# paths of all figures
all_figs = function(options, ext = options$fig.ext, num = options$fig.num) {
  unlist(lapply(ext, fig_path, options = options, number = seq_len(num)))
}

# evaluate an expression in a diretory and restore wd after that
in_dir = function(dir, expr) {
  if (!is.null(dir)) {
    owd = setwd(dir); on.exit(setwd(owd))
  }
  wd1 = getwd()
  res = expr
  wd2 = getwd()
  if (wd1 != wd2) warning(
    'You changed the working directory to ', wd2, ' (probably via setwd()). ',
    'It will be restored to ', wd1, '. See the Note section in ?knitr::knit'
  )
  res
}

# evaluate under the base.dir
in_base_dir = function(expr) {
  d = opts_knit$get('base.dir')
  if (is.character(d) && !file_test('-d', d)) dir.create(d, recursive = TRUE)
  in_dir(d, expr)
}

# escape special LaTeX characters
escape_latex = function(x, newlines = FALSE, spaces = FALSE) {
  x = gsub('\\\\', '\\\\textbackslash', x)
  x = gsub('([#$%&_{}])', '\\\\\\1', x)
  x = gsub('\\\\textbackslash', '\\\\textbackslash{}', x)
  x = gsub('~', '\\\\textasciitilde{}', x)
  x = gsub('\\^', '\\\\textasciicircum{}', x)
  if (newlines) x = gsub('(?<!\n)\n(?!\n)', '\\\\\\\\', x, perl = TRUE)
  if (spaces) x = gsub('  ', '\\\\ \\\\ ', x)
  x
}

# escape special HTML chars
escape_html = highr:::escape_html

#' Read source code from R-Forge
#'
#' This function reads source code from the SVN repositories on R-Forge.
#' @param path Relative path to the source script on R-Forge.
#' @param project Name of the R-Forge project.
#' @param extra Extra parameters to be passed to the URL (e.g. \code{extra =
#'   '&revision=48'} to check out the source of revision 48).
#' @return A character vector of the source code.
#' @author Yihui Xie and Peter Ruckdeschel
#' @export
#' @examples library(knitr)
#' \donttest{# relies on r-forge.r-project.org being accessible
#' read_rforge('rgl/R/axes.R', project = 'rgl')
#' read_rforge('rgl/R/axes.R', project = 'rgl', extra='&revision=519')}
read_rforge = function(path, project, extra = '') {
  base = 'http://r-forge.r-project.org/scm/viewvc.php/*checkout*/pkg'
  read_utf8(sprintf('%s/%s?root=%s%s', base, path, project, extra))
}

# TODO: just import xfun::split_lines()
split_lines = function(x) {
  if (length(grep('\n', x)) == 0L) return(x)
  x = gsub('\n$', '\n\n', x)
  x[x == ''] = '\n'
  unlist(strsplit(x, '\n'))
}

# if a string is encoded in UTF-8, convert it to native encoding
native_encode = function(x, to = '') {
  from = if (any(Encoding(x) == 'UTF-8')) 'UTF-8' else ''
  x2 = iconv(x, from, to)
  if (!any(is.na(x2))) return(x2)  # use conversion only if it succeeds
  warning('some characters may not work under the current locale')
  x
}

is_native_enc = function(x) {
  x == 'native.enc' || x == ''
}

is_utf8 = function(x) {
  !any(is.na(iconv(x, 'UTF-8', 'UTF-8')))
}

#' Wrap long lines in Rmd files
#'
#' This function wraps long paragraphs in an R Markdown file. Other elements are
#' not wrapped: the YAML preamble, fenced code blocks, section headers and
#' indented elements. The main reason for wrapping long lines is to make it
#' easier to review differences in version control.
#' @param file The input Rmd file.
#' @param width The expected line width.
#' @param text A character vector of text lines, as an alternative to \code{file}. If
#'   \code{text} is not \code{NULL}, \code{file} is ignored.
#' @param backup Path to back up the original file in case anything goes
#'   wrong. If set to \code{NULL}, no backup is made. The default value is constructed
#'   from \code{file} by adding \code{__} before the base filename.
#' @return If \code{file} is provided, it is overwritten; if \code{text} is
#'   provided, a character vector is returned.
#' @note Currently it does not wrap blockquotes or lists (ordered or unordered).
#'   This feature may or may not be added in the future.
#' @export
#' @examples wrap_rmd(text = c('```', '1+1', '```', '- a list item', '> a quote', '',
#' paste(rep('this is a normal paragraph', 5), collapse = ' ')))
wrap_rmd = function(file, width = 80, text = NULL, backup) {
  x = if (is.null(text)) read_utf8(file) else split_lines(text)
  x = strip_white(x)  # strip blank lines in the beginning and end
  if ((n <- length(x)) <= 1L) return(x)  # are you kidding?
  idx = NULL  # collect the lines to exclude from wrapping
  i = grep('^---$', x)  # yaml preamble
  if (length(i) > 1 && i[1L] == 1L) idx = c(idx, i[1L]:i[2L])
  i = grep('^(```|\\{% (end|)highlight [a-z ]*%\\}|</?script.*>)', x)  # code blocks
  if (length(i)) {
    if (length(i) %% 2L != 0L) stop('markers for code blocks must be paired up')
    idx = c(idx, unlist(apply(matrix(i, ncol = 2L, byrow = TRUE), 1L,
                              function(z) z[1L]:z[2L])))
  }
  # section headers, indented code blocks and latex math
  idx = c(idx, grep('^(#|===|---|    |\t)', x))
  # blank lines
  idx = c(idx, grep('^\\s*$', x))
  # TODO: this is naive -- I treat a line as (a part of) a normal paragraph if
  # it does not start with a space, or > (blockquotes) or -, *, 1. (lists), and
  # only wrap paragraphs
  idx = c(idx, grep('^\\s*( |> |- |\\* |\\d+ )', x))
  idx = unique(idx)
  if (length(idx) == n) return(x)  # no need to wrap anything

  i = logical(n); i[idx] = TRUE; r = rle(i)
  n = length(r$lengths); txt = vector('list', n); j = c(0L, cumsum(r$lengths))
  for (i in seq_len(n)) {
    block = x[seq(j[i] + 1L, j[i + 1])]
    txt[[i]] = if (r$value[i]) {
      # those lines not to be wrapped
      gsub('\\s+$', '', block)  # strip pending spaces
    } else {
      strwrap(one_string(block), width)
    }
  }
  txt = unlist(txt)
  if (is.null(text)) {
    if (missing(backup)) backup = file.path(dirname(file), paste0('__', basename(file)))
    if (!is.null(backup)) file.copy(file, backup, overwrite = TRUE)
    write_utf8(txt, file)
  } else txt
}

# change the default device to an appropriate device when the output is html
# (e.g. markdown, reST, AsciiDoc)
set_html_dev = function() {
  # only change if device has not been set
  if (!is.null(opts_chunk$get('dev'))) return()
  # in some cases, png() does not work (e.g. options('bitmapType') == 'Xlib' on
  # headless servers); use svg then
  opts_chunk$set(dev = if (png_available()) 'png' else 'svg')
}

png_available = function() {
  !inherits(try_silent({
    f = tempfile(); on.exit(unlink(f)); grDevices::png(f); grDevices::dev.off()
  }), 'try-error')
}

# locate kpsewhich especially for Mac OS because /usr/texbin may not be in PATH
kpsewhich = function() {
  if (Sys.info()['sysname'] != 'Darwin' || !file.exists(x <- '/usr/texbin/kpsewhich')
      || nzchar(Sys.which('kpsewhich')))
    'kpsewhich' else x
}

# check if a utility exists; if it does, save its availability in opts_knit
has_utility = function(name, package = name) {
  name2 = paste('util', name, sep = '_')  # e.g. util_pdfcrop
  if (is.logical(yes <- opts_knit$get(name2))) return(yes)
  yes = nzchar(Sys.which(name))
  if (!yes) warning(package, ' not installed or not in PATH')
  opts_knit$set(setNames(list(yes), name2))
  yes
}

#' Query the current input filename
#'
#' Returns the name of the input file passed to \code{\link{knit}()}.
#' @param dir Boolean; whether to prepend the current working directory to the file path,
#'   i.e. whether to return an absolute path or a relative path.
#' @return A character string, if this function is called inside an input
#'   document. Otherwise \code{NULL}.
#' @export
current_input = function(dir = FALSE) {
  input = knit_concord$get('infile')
  outwd = opts_knit$get('output.dir')
  if (is.null(input)) return()
  if (dir) {
    if (is.null(outwd)) {
      warning('Cannot determine the directory of the input document')
      dir = FALSE
    }
  }
  if (!dir) return(basename(input))
  if (is_abs_path(input)) input else file.path(outwd, input)
}

# import output handlers from evaluate
default_handlers = evaluate:::default_output_handler
# change the value handler in evaluate default handlers
knit_handlers = function(fun, options) {
  if (!is.function(fun)) fun = function(x, ...) {
    res = withVisible(knit_print(x, ...))
    # indicate the htmlwidget result with a special class so we can attach
    # the figure caption to it later in wrap.knit_asis
    if (inherits(x, 'htmlwidget'))
      class(res$value) = c(class(res$value), 'knit_asis_htmlwidget')
    if (res$visible) res$value else invisible(res$value)
  }
  if (length(formals(fun)) < 2)
    stop("the chunk option 'render' must be a function of the form ",
         "function(x, options) or function(x, ...)")
  merge_list(default_handlers, list(value = function(x, visible) {
    if (visible) fun(x, options = options)
  }))
}

# conditionally disable some features during R CMD check
is_R_CMD_check = function() {
  ('CheckExEnv' %in% search()) ||
    any(c('_R_CHECK_TIMINGS_', '_R_CHECK_LICENSE_') %in% names(Sys.getenv()))
}

# is the inst dir under . or ..? differs in R CMD build/INSTALL and devtools/roxygen2
inst_dir = function(...) {
  p = file.path(c('..', '.'), 'inst', ...)
  p[file.exists(p)]
}

# a restricted version of is.numeric (e.g. do not treat chron::chron() as
# numeric since their behavior may be somewhat unpredictable, e.g. through
# round(), #1118); see #1396 for difftime
is_numeric = function(x) {
  class(x)[1] %in% c('numeric', 'integer', 'difftime', 'complex')
}

# create \label{x} or (\#x); the latter is current an internal hack for bookdown
create_label = function(..., latex = FALSE) {
  if (isTRUE(opts_knit$get('bookdown.internal.label'))) {
    lab1 = '(\\#'; lab2 = ')'
  } else if (latex) {
    lab1 = '\\label{'; lab2 = '}'
  } else {
    return('')  # we don't want the label at all
  }
  paste0(lab1, ..., lab2)
}

#' Combine multiple words into a single string
#'
#' When a value from an inline R expression is a character vector of multiple
#' elements, we may want to combine them into a phrase like \samp{a and b}, or
#' \code{a, b, and c}. That is what this a helper function does.
#'
#' If the length of the input \code{words} is smaller than or equal to 1,
#' \code{words} is returned. When \code{words} is of length 2, the first word
#' and second word are combined using the \code{and} string. When the length is
#' greater than 2, \code{sep} is used to separate all words, and the \code{and}
#' string is prepended to the last word.
#' @param words A character vector.
#' @param sep Separator to be inserted between words.
#' @param and Character string to be prepended to the last word.
#' @param before,after A character string to be added before/after each word.
#' @return A character string marked by \code{xfun::\link{raw_string}()}.
#' @export
#' @examples combine_words('a'); combine_words(c('a', 'b'))
#' combine_words(c('a', 'b', 'c'))
#' combine_words(c('a', 'b', 'c'), sep = ' / ', and = '')
#' combine_words(c('a', 'b', 'c'), and = '')
#' combine_words(c('a', 'b', 'c'), before = '"', after = '"')
combine_words = function(words, sep = ', ', and = ' and ', before = '', after = before) {
  n = length(words); rs = xfun::raw_string
  if (n == 0) return(words)
  words = paste0(before, words, after)
  if (n == 1) return(rs(words))
  if (n == 2) return(rs(paste(words, collapse = and)))
  if (grepl('^ ', and) && grepl(' $', sep)) and = gsub('^ ', '', and)
  words[n] = paste0(and, words[n])
  rs(paste(words, collapse = sep))
}

warning2 = function(...) warning(..., call. = FALSE)
stop2 = function(...) stop(..., call. = FALSE)

raw_markers = c('!!!!!RAW-KNITR-CONTENT', 'RAW-KNITR-CONTENT!!!!!')

#' @export
#' @rdname raw_output
extract_raw_output = function(text, markers = raw_markers) {
  r = sprintf('%s(.*?)%s', markers[1], markers[2])
  x = one_string(text)
  m = gregexpr(r, x)
  s = regmatches(x, m)
  n = length(s[[1]])
  if (n == 0) return(list(value = text, chunks = character()))

  chunks = tokens = character(n)
  for (i in seq_len(n)) {
    chunks[i] = sub(r, '\\1', s[[1]][i])
    tokens[i] = digest(chunks[i])
    s[[1]][i] = gsub(r, paste0(markers[1], tokens[i], markers[2]), s[[1]][i])
  }
  regmatches(x, m) = s

  list(value = x, chunks = setNames(chunks, tokens))
}

#' @export
#' @rdname raw_output
restore_raw_output = function(text, chunks, markers = raw_markers) {
  if ((n <- length(chunks)) == 0) return(text)
  text = enc2utf8(text); chunks = enc2utf8(chunks); tokens = names(chunks)
  for (i in seq_len(n)) {
    r = paste0(markers[1], tokens[i], markers[2])
    text = gsub(r, chunks[i], text, fixed = TRUE, useBytes = TRUE)
  }
  Encoding(text) = 'UTF-8'
  text
}

#' Mark character strings as raw output that should not be converted
#'
#' These functions provide a mechanism to protect the character output of R code
#' chunks. The output is annotated with special markers in \code{raw_output};
#' \code{extract_raw_output()} will extract raw output wrapped in the markers,
#' and replace the raw output with its MD5 digest; \code{restore_raw_output()}
#' will restore the MD5 digest with the original raw output.
#'
#' This mechanism is designed primarily for R Markdown pre/post-processors. In
#' an R code chunk, you generate \code{raw_output()} to the Markdown output. In
#' the pre-processor, you can \code{extract_raw_output()} from the Markdown
#' file, store the raw output and MD5 digests, and remove the actual raw output
#' from Markdown so Pandoc will never see it. In the post-processor, you can
#' read the Pandoc output (e.g., an HTML or RTF file), and restore the raw
#' output.
#' @param x The character vector to be protected.
#' @param markers A length-2 character vector to be used to wrap \code{x};
#'   see \code{knitr:::raw_markers} for the default value.
#' @param ... Arguments to be passed to \code{\link{asis_output}()}.
#' @param text For \code{extract_raw_output()}, the content of the input file
#'   (e.g. Markdown); for \code{restore_raw_output()}, the content of the output
#'   file (e.g. HTML generated by Pandoc from Markdown).
#' @param chunks A named character vector returned from
#'   \code{extract_raw_output()}.
#' @return For \code{extract_raw_output()}, a list of two components:
#'   \code{value} (the \code{text} with raw output replaced by MD5 digests) and
#'   \code{chunks} (a named character vector, of which the names are MD5 digests
#'   and values are the raw output). For \code{restore_raw_output()}, the
#'   restored \code{text}.
#' @export
#' @examples library(knitr)
#' out = c('*hello*', raw_output('<special>content</special> *protect* me!'), '*world*')
#' pre = extract_raw_output(out)
#' str(pre)
#' pre$value = gsub('[*]([^*]+)[*]', '<em>\\1</em>', pre$value)  # think this as Pandoc conversion
#' pre$value
#' # raw output was protected from the conversion (e.g. *protect* was not converted)
#' restore_raw_output(pre$value, pre$chunks)
raw_output = function(x, markers = raw_markers, ...) {
  asis_output(paste(c(markers[1], x, markers[2]), collapse = ''), ...)
}


#' Mark character strings as raw blocks in R Markdown
#'
#' Wraps content in a raw attribute block, which protects it from being escaped
#' by Pandoc. See \url{https://pandoc.org/MANUAL.html#generic-raw-attribute}.
#' Functions \code{raw_latex()} and \code{raw_html()} are shorthands of
#' \code{raw_block(x, 'latex')} and \code{raw_block(x, 'html')}, respectively.
#' @param x The character vector to be protected.
#' @param type The type of raw blocks (i.e., the Pandoc output format). If you
#'   are not sure about the Pandoc output format of your document, insert a code
#'   chunk \code{knitr:::pandoc_to()} and see what it returns after the document
#'   is compiled.
#' @param ... Arguments to be passed to \code{\link{asis_output}()}.
#' @export
#' @examples
#' knitr::raw_latex('\\emph{some text}')
raw_block = function(x, type = 'latex', ...) {
  if (!rmarkdown::pandoc_available('2.0.0')) warning('raw_block() requires Pandoc >= 2.0.0')
  x = c(sprintf('\n```{=%s}', type), x, '```\n')
  asis_output(one_string(x), ...)
}

#' @rdname raw_block
#' @export
raw_latex = function(x, ...) raw_block(x, 'latex', ...)

#' @rdname raw_block
#' @export
raw_html = function(x, ...) raw_block(x, 'html', ...)

trimws = function(x) gsub('^\\s+|\\s+$', '', x)

optipng = function(...) {
  warning2('knitr:::optipng() has been deprecated; please use xfun::optipng()')
  xfun::optipng(...)
}

digest = function(x) {
  if (loadable('digest')) digest::digest(x) else digest2(x)
}

# compatible with digest::digest() but definitely slower because of file I/O
digest2 = function(x) {
  s = serialize(x, NULL, ascii = FALSE)
  if (length(s) > 14) s = s[-(1:14)]  # https://d.cosx.org/d/419804
  writeBin(s, f <- tempfile())
  on.exit(unlink(f), add = TRUE)
  unname(tools::md5sum(f))
}

# not removing the serialize() header (first few bytes)
digest3 = function(x) {
  f = tempfile(); on.exit(unlink(f), add = TRUE)
  s = file(f, open = 'wb')
  serialize(x, s, ascii = FALSE)
  close(s)
  unname(tools::md5sum(f))
}

# collapse by \n
one_string = function(x, ...) paste(x, ..., collapse = '\n')

# c(1, 1, 1, 2, 3, 3) -> c(1a, 1b, 1c, 2a, 3a, 3b)
make_unique = function(x) {
  if (length(x) == 0) return(x)
  x2 = make.unique(x)
  if (all(i <- x2 == x)) return(x)
  x2[i] = paste0(x2[i], '.0')
  i = as.numeric(sub('.*[.]([0-9]+)$', '\\1', x2)) + 1
  s = letters[i]
  s = ifelse(is.na(s), i, s)
  paste0(x, s)
}

context('Utils')

test_that('abs_path() is working', {
  expect_false(is_abs_path('abc/def'))
  if (.Platform$OS.type == 'windows') {
    expect_true(is_abs_path('D:\\abc'))
    expect_true(is_abs_path('\\\\netdrive\\somewhere'))
  } else {
    expect_true(is_abs_path('/abc/def'))
  }
})

test_that('scientific notation with format_sci()',{
  options(digits = 4, scipen = 0)
  expect_identical(format_sci(1), '1')
  expect_identical(format_sci(0), '0')
  expect_identical(format_sci(c(1.84e8, 1e5, 2.34e3)),
                   c(sprintf("\\ensuremath{%s}", c("1.84\\times 10^{8}", "10^{5}")), "2340"))
  expect_identical(format_sci(1.23456789*10^-5), "\\ensuremath{1.2346\\times 10^{-5}}")
  expect_identical(format_sci(9.87654e6, 'html'), "9.8765 &times; 10<sup>6</sup>")
  expect_identical(format_sci(9.87654e6, 'rst'), "9.8765 |times| 10 :sup:`6`")
  expect_identical(format_sci(letters), letters)
  expect_identical(format_sci(NA_real_), NA_character_)
})

test_that('fig_path() sanitizes paths', {
  expect_identical(sanitize_fn('fig/foo', '.png'), 'fig/foo.png')
  expect_warning(sanitize_fn('figure/a b'))
  suppressWarnings({
    expect_identical(sanitize_fn('figure/a b'), 'figure/a_b')
    expect_identical(sanitize_fn('fig space/a.b'), 'fig_space/a_b')
    expect_identical(sanitize_fn('../c.d'), '../c_d')
    expect_identical(sanitize_fn('./../c..d'), './../c__d')
  })
  expect_identical(sanitize_fn('C:/foo/bar'), 'C:/foo/bar')
})

test_that('base64_encode() gets the same result as markdown:::.b64EncodeFile', {
  f = file.path(R.home('doc'), "html", "logo.jpg")
  if (require('markdown')) expect_identical(markdown:::.b64EncodeFile(f), image_uri(f))
})

test_that('escape special LaTeX characters', {
  expect_identical(escape_latex('# $ % & ~ _ ^ \\ { }'),
                   '\\# \\$ \\% \\& \\textasciitilde{} \\_ \\textasciicircum{} \\textbackslash{} \\{ \\}')
})

#' Spin goat's hair into wool
#'
#' This function takes a specially formatted R script and converts it to a
#' literate programming document. By default normal text (documentation) should
#' be written after the roxygen comment (\code{#'}) and code chunk options are
#' written after \code{#+} or \code{#-} or \code{# ----} or any of these
#' combinations replacing \code{#} with \code{--}.
#'
#' Obviously the goat's hair is the original R script, and the wool is the
#' literate programming document (ready to be knitted).
#' @param hair Path to the R script. The script must be encoded in UTF-8 if it
#'   contains multibyte characters.
#' @param knit Logical; whether to compile the document after conversion.
#' @param report Logical; whether to generate a report for \file{Rmd},
#'   \file{Rnw} and \file{Rtex} output. Ignored if \code{knit = FALSE}.
#' @param text A character vector of code, as an alternative way to provide the
#'   R source. If \code{text} is not \code{NULL}, \code{hair} will be ignored.
#' @param envir Environment for \code{\link{knit}()} to evaluate the code.
#' @param format Character; the output format. The default is R Markdown.
#' @param doc A regular expression to identify the documentation lines; by
#'   default it follows the roxygen convention, but it can be customized, e.g.
#'   if you want to use \code{##} to denote documentation, you can use
#'   \code{'^##\\\\s*'}.
#' @param inline A regular expression to identify inline R expressions; by
#'   default, code of the form \code{\{\{code\}\}} on its own line is treated as
#'   an inline expression.
#' @param comment A pair of regular expressions for the start and end delimiters
#'   of comments; the lines between a start and an end delimiter will be
#'   ignored. By default, the delimiters are \verb{/*} at the beginning of a
#'   line, and \verb{*/} at the end, following the convention of C comments.
#' @param precious logical: whether intermediate files (e.g., \code{.Rmd} files
#'   when \code{format} is \code{"Rmd"}) should be preserved. The default is
#'   \code{FALSE} if \code{knit} is \code{TRUE} and the input is a file.
#' @author Yihui Xie, with the original idea from Richard FitzJohn (who named it
#'   as \code{sowsear()} which meant to make a silk purse out of a sow's ear)
#' @return If \code{text} is \code{NULL}, the path of the final output document,
#'   otherwise the content of the output.
#' @note If the output format is Rnw and no document class is specified in
#'   roxygen comments, this function will automatically add the \code{article}
#'   class to the LaTeX document so that it is complete and can be compiled. You
#'   can always specify the document class and other LaTeX settings in roxygen
#'   comments manually.
#'
#'   When the output format is Rmd, it is compiled to HTML via
#'   \code{\link{knit2html}()}, which uses R Markdown v1 instead of v2. If you
#'   want to use the latter, you should call
#'   \code{rmarkdown::\link[rmarkdown]{render}()} instead.
#' @export
#' @seealso \code{\link{stitch}} (feed a template with an R script)
#' @references \url{https://yihui.org/knitr/demo/stitch/}
spin = function(
  hair, knit = TRUE, report = TRUE, text = NULL, envir = parent.frame(),
  format = c('Rmd', 'Rnw', 'Rhtml', 'Rtex', 'Rrst'),
  doc = "^#+'[ ]?", inline = '^[{][{](.+)[}][}][ ]*$',
  comment = c("^[# ]*/[*]", "^.*[*]/ *$"), precious = !knit && is.null(text)
) {

  format = match.arg(format)
  x = if (nosrc <- is.null(text)) read_utf8(hair) else split_lines(text)
  stopifnot(length(comment) == 2L)
  c1 = grep(comment[1], x); c2 = grep(comment[2], x)
  if (length(c1) != length(c2))
    stop('comments must be put in pairs of start and end delimiters')
  # remove comments
  if (length(c1)) x = x[-unique(unlist(mapply(seq, c1, c2, SIMPLIFY = FALSE)))]

  # remove multiline string literals and symbols (note that this ignores lines with spaces at their
  # beginnings, assuming doc and inline regex don't match these lines anyway)
  parsed_data = getParseData(parse(text = x, keep.source = TRUE))
  is_matchable = seq_along(x) %in% unique(parsed_data[parsed_data$col1 == 1, 'line1'])

  # .Rmd needs to be treated specially
  p = if (identical(tolower(format), 'rmd')) .fmt.rmd(x) else .fmt.pat[[tolower(format)]]

  # turn {{expr}} into inline expressions, e.g. `r expr` or \Sexpr{expr}
  if (any(i <- is_matchable & grepl(inline, x))) x[i] = gsub(inline, p[4], x[i])

  r = rle((is_matchable & grepl(doc, x)) | i)  # inline expressions are treated as doc instead of code
  n = length(r$lengths); txt = vector('list', n); idx = c(0L, cumsum(r$lengths))
  p1 = gsub('\\{', '\\\\{', paste0('^', p[1L], '.*', p[2L], '$'))

  for (i in seq_len(n)) {
    block = x[seq(idx[i] + 1L, idx[i + 1])]
    txt[[i]] = if (r$values[i]) {
      # normal text; just strip #'
      sub(doc, '', block)
    } else {
      # R code; #+/- indicates chunk options
      block = strip_white(block) # rm white lines in beginning and end
      if (!length(block)) next
      if (length(opt <- grep(rc <- '^(#|--)+(\\+|-| ----+| @knitr)', block))) {
        block[opt] = paste0(p[1L], gsub(paste0(rc, '\\s*|-*\\s*$'), '', block[opt]), p[2L])
        # close each chunk if there are multiple chunks in this block
        if (any(opt > 1)) {
          j = opt[opt > 1]
          block[j] = paste(p[3L], block[j], sep = '\n')
        }
      }
      if (!grepl(p1, block[1L])) {
        block = c(paste0(p[1L], p[2L]), block)
      }
      c('', block, p[3L], '')
    }
  }

  txt = unlist(txt)
  # make it a complete TeX document if document class not specified
  if (report && format %in% c('Rnw', 'Rtex') && !grepl('^\\s*\\\\documentclass', txt)) {
    txt = c('\\documentclass{article}', '\\begin{document}', txt, '\\end{document}')
  }
  if (nosrc) {
    outsrc = with_ext(hair, format)
    write_utf8(txt, outsrc)
    txt = NULL
  } else outsrc = NULL
  if (!knit) return(txt %n% outsrc)

  out = if (report) {
    if (format == 'Rmd') {
      knit2html(outsrc, text = txt, envir = envir)
    } else if (!is.null(outsrc) && (format %in% c('Rnw', 'Rtex'))) {
      knit2pdf(outsrc, envir = envir)
    }
  } else knit(outsrc, text = txt, envir = envir)

  if (!precious && !is.null(outsrc)) file.remove(outsrc)
  invisible(out)
}

.fmt.pat = list(
  rnw = c('<<', '>>=', '@', '\\\\Sexpr{\\1}'),
  rhtml = c('<!--begin.rcode ', '', 'end.rcode-->', '<!--rinline \\1 -->'),
  rtex = c('% begin.rcode ', '', '% end.rcode', '\\\\rinline{\\1}'),
  rrst = c('.. {r ', '}', '.. ..', ':r:`\\1`')
)

# determine how many backticks we need to wrap code blocks and inline code
.fmt.rmd = function(x) {
  x = one_string(x)
  l = attr(gregexpr('`+', x)[[1]], 'match.length')
  l = max(l, 0)
  if (length(l) > 0) {
    i = highr:::spaces(l + 1, '`')
    b = highr:::spaces(max(l + 1, 3), '`')
  } else {
    i = '`'
    b = '```'
  }
  c(paste0(b, '{r '), '}', b, paste0(i, 'r \\1 ', i))
}

#' Spin a child R script
#'
#' This function is similar to \code{\link{knit_child}()} but is used in R
#' scripts instead. When the main R script is not called via
#' \code{\link{spin}()}, this function simply executes the child script via
#' \code{\link{sys.source}()}, otherwise it calls \code{\link{spin}()} to spin
#' the child script into a source document, and uses \code{\link{knit_child}()}
#' to compile it. You can call this function in R code, or using the syntax of
#' inline R expressions in \code{\link{spin}()} (e.g.
#' \code{{{knitr::spin_child('script.R')}}}).
#' @param input Filename of the input R script.
#' @param format Passed to \code{format} in \code{spin()}. If not
#'   provided, it will be guessed from the current knitting process.
#' @return A character string of the knitted R script.
#' @export
spin_child = function(input, format) {
  if (!isTRUE(getOption('knitr.in.progress')))
    return(sys.source(input, parent.frame()))
  fmt = if (missing(format)) {
    if (is.null(fmt <- out_format()))
      stop('spin_child() must be called in a knitting process')
    .spin.fmt = c(
      'latex' = 'Rnw', 'sweave' = 'Rnw', 'listings' = 'Rnw',
      'html' = 'Rhtml', 'markdown' = 'Rmd'
    )
    if (is.na(fmt <- .spin.fmt[fmt]))
      stop('the document format ', fmt, ' is not supported yet')
    fmt
  } else format
  asis_output(knit_child(
    text = spin(text = read_utf8(input), knit = FALSE, report = FALSE, format = fmt),
    quiet = TRUE
  ))
}

library(testit)

# see http://stackoverflow.com/q/18992260/559676 for the bug
assert('inline_exec only accept character result', {
  block = list(code = "function() 1", input = "inline `r function() 1`")
  res = xfun::try_silent(inline_exec(block, new.env()))
  (inherits(res, 'try-error'))
  block = list(code = "(function() 1)()", input = "inline `r (function() 1)()`")
  block$location = matrix(c(8,27), ncol = 2, byrow = TRUE)
  res = inline_exec(block, new.env())
  (res %==% "inline 1")
  block = list(code = character(0), input = "no inline")
  res = inline_exec(block, new.env())
  (res %==% "no inline")
})

# replace parts of a string with new values; `pos` is a matrix of positions and
# each row is a pair of [start, end]
str_replace = function(x, pos, value) {
  if (length(x) != 1) stop("Only a character scalar is supported.")
  # extract parts of the string that are outside [start, end]
  m = rbind(pos[, 1] - 1, pos[, 2] + 1)
  m = matrix(c(1, m, nchar(x)), nrow = 2)
  y = substring(x, m[1, ], m[2, ])
  paste(rbind(y, c(value, '')), collapse = '')
}

# insert a value after i-th character in the string x
str_insert = function(x, i, value) {
  if (i <= 0) return(paste0(value, x))
  n = nchar(x)
  if (n == 0 || i >= n) return(paste0(x, value))
  paste0(substr(x, 1, i), value, substr(x, i + 1, n))
}

# a wrapper function to make strwrap() return a character vector of the same
# length as the input vector; each element of the output vector is a string
# formed by concatenating wrapped strings by \n
str_wrap = function(...) {
  res = strwrap(..., simplify = FALSE)
  unlist(lapply(res, one_string))
}

# a simplified replacement for stringr::str_locate_all() that returns a list
# having an element for every element of 'string'; every list element is an
# integer matrix having a row per match, and two columns: 'start' and 'end'.
str_locate = function(x, pattern, all = TRUE) {
  out = (if (all) gregexpr else regexpr)(pattern, x, perl = TRUE)
  if (all) lapply(out, location) else location(out)
}

location = function(x) {
  len = attr(x, 'match.length')
  if (length(x) == 1 && x == -1) x = integer()
  cbind(start = x, end = x + len - 1L)
}

# a replacement for stringr::str_extract_all()
str_extract = function(x, pattern) {
  m = gregexpr(pattern, x, perl = TRUE)
  regmatches(x, m)
}

str_match = function(x, pattern) {
  # gregexec() was added in R 4.1.0; for lower versions of R, use fallback
  if (is.function(gregexec <- baseenv()[['gregexec']])) {
    m = gregexec(pattern, x, perl = TRUE)
  } else {
    x = unlist(str_extract(x, pattern))
    m = regexec(pattern, x, perl = TRUE)
  }
  do.call(cbind, regmatches(x, m))
}

## @knitr cars-demo
summary(cars)
fit=lm(dist~speed, data=cars)
summary(fit)


## @knitr cars-plot, fig.width=7, fig.height=6, fig.align='center'
par(mar=c(4,4,.1,.1))
plot(cars, pch=19)


## @knitr setup, eval=FALSE
## library(knitr)
## # to base64 encode images
## opts_knit$set(upload.fun = image_uri)

## @knitr eval=FALSE
## help(package = 'YourPackage', help_type = 'html')


## @knitr hello, results='asis'
cat('_hello_ **markdown**!', '\n')


## @knitr 
1+1
10:1
rnorm(5)^2
strsplit('hello, markdown vignettes', '')


## @knitr 
n=300; set.seed(123)
par(mar=c(4,4,.1,.1))
plot(rnorm(n), rnorm(n), pch=21, cex=5*runif(n), col='white', bg='gray')


## @knitr css
options(markdown.HTML.stylesheet = system.file('misc', 'vignette.css', package='knitr'))

#' @rdname hook_plot
#' @export
hook_plot_html = function(x, options) {
  # pull out all the relevant plot options
  fig.num = options$fig.num = options$fig.num %n% 1L
  fig.cur = options$fig.cur %n% 1L

  if (options$fig.show == 'animate') {
    # Don't print out intermediate plots if we're animating
    return(if (fig.cur < fig.num) '' else hook_animation(options)(x, options))
  }
  ai = options$fig.show == 'asis'
  plot1 = ai || fig.cur <= 1L; plot2 = ai || fig.cur == fig.num
  d1 = if (plot1) paste0(if (out_format('html')) '</div>',
                        sprintf('<div class="rimage %s">', options$fig.align))
  d2 = if (plot2) paste0('</div>', if (out_format('html')) '<div class="rcode">')
  paste0(
    d1, .img.tag(
      .upload.url(x), options$out.width, options$out.height, .img.cap(options),
      paste(c(options$out.extra, 'class="plot"'), collapse = ' ')
    ), d2, '\n'
  )
}

hook_animation = function(options) {
  if (is.function(fun <- options$animation.hook)) return(fun)
  if (is.character(fun)) return(switch(
    fun, ffmpeg = hook_ffmpeg_html, gifski = hook_gifski,
    scianimator = hook_scianimator, r2swf = hook_r2swf,
    stop2('Invalid value for the chunk option animation.hook: ', fun)
  ))
  if (is.function(fun <- opts_knit$get('animation.fun'))) return(fun)
  hook_ffmpeg_html
}

.img.attr = function(w, h, extra) {
  paste(c(sprintf('width="%s"', w), sprintf('height="%s"', h), extra), collapse = ' ')
}

.img.tag = function(src, w, h, caption, extra) {
  caption = if (length(caption) == 1 && caption != '') {
    paste0('title="', caption, '" alt="', caption, '" ')
  }
  tag = if (grepl('[.]pdf$', src, ignore.case = TRUE)) {
    extra = c(extra, 'type="application/pdf"')
    'embed'
  } else 'img'
  paste0(
    '<', tag, ' src="', opts_knit$get('base.url'), src, '" ', caption,
    .img.attr(w, h, extra), ' />'
  )
}

.img.cap = function(options, alt = FALSE) {
  cap = options$fig.cap %n% {
    if (is.null(pandoc_to())) sprintf('plot of chunk %s', options$label) else ''
  }
  if (length(cap) == 0) cap = ''
  if (is_blank(cap)) return(cap)
  if (alt) return(escape_html(cap))
  paste0(create_label(
    options$fig.lp, options$label,
    if (options$fig.num > 1L && options$fig.show == 'asis') options$fig.cur
  ), cap)
}

# a wrapper to upload an image and return the URL
.upload.url = function(x) {
  opts_knit$get('upload.fun')(x)
}

.chunk.hook.html = function(x, options) {
  if (output_asis(x, options)) return(x)
  x = sprintf('<div class="chunk" id="%s"><div class="rcode">%s</div></div>',
              options$label, x)
  x = gsub('<div class="rcode">\\s*</div>', '', x) # rm empty rcode layers
  if (options$split) {
    name = fig_path('.html', options, NULL)
    if (!file.exists(dirname(name)))
      dir.create(dirname(name))
    write_utf8(x, name)
    sprintf('<iframe src="%s" class="knitr" width="100%%"></iframe>', name)
  } else x
}

#' Hooks to create animations in HTML output
#'
#' \code{hook_ffmpeg_html()} uses FFmpeg to convert images to a video;
#' \code{hook_gifski()} uses the \pkg{gifski} to convert images to a GIF
#' animation; \code{hook_scianimator()} uses the JavaScript library SciAnimator
#' to create animations; \code{hook_r2swf()} uses the \pkg{R2SWF} package.
#'
#' These hooks are mainly for the package option \code{animation.fun}, e.g. you
#' can set \code{opts_knit$set(animation.fun = hook_scianimator)}.
#' @inheritParams hook_plot_tex
#' @rdname hook_animation
#' @export
hook_ffmpeg_html = function(x, options) {
  hook_ffmpeg(x, options, options$ffmpeg.format %n% 'webm')
}

hook_ffmpeg = function(x, options, format = 'webm') {
  x = c(sans_ext(x), file_ext(x))
  fig.num = options$fig.num
  format = sub('^[.]', '', format)
  # set up the ffmpeg run
  base = sub(paste0(fig.num, '$'), '', x[1])
  fig.fname = paste0(base, '%d', '.', x[2])
  mov.fname = paste0(sub('-$', '', base), '.', format)

  extra = switch(
    format,
    webm = paste('-b:v', options$ffmpeg.bitrate %n% '1M', '-crf 10'),
    mp4  = '-pix_fmt yuv420p'  # enables Safari support of .mp4
  )
  ffmpeg.cmd = paste(
    'ffmpeg', '-y', '-r', 1 / options$interval, '-i', fig.fname, extra, mov.fname
  )

  if (Sys.which('ffmpeg') == '') stop2(
    'Could not find ffmpeg command. You should either change the animation.fun ',
    'hook option or install ffmpeg with libvpx enabled.'
  )
  message('executing: ', ffmpeg.cmd)
  system(ffmpeg.cmd, ignore.stdout = TRUE)

  # use a normal plot hook if the output is GIF
  if (format == 'gif') {
    options$fig.show = 'hold'
    return((if (out_format('markdown')) hook_plot_md else hook_plot_html)(mov.fname, options))
  }

  # controls,loop --> controls loop
  opts = paste(sc_split(options$aniopts), collapse = ' ')
  opts = paste(
    sprintf('width="%s"', options$out.width),
    sprintf('height="%s"', options$out.height), opts
  )
  cap = .img.cap(options, alt = TRUE)
  if (cap != '') cap = sprintf('<p>%s</p>', cap)
  sprintf(
    '<video %s><source src="%s" />%s</video>', trimws(opts),
    paste0(opts_knit$get('base.url'), mov.fname), cap
  )
}

# use gifski to create gif's
#' @rdname hook_animation
#' @export
hook_gifski = function(x, options) {
  x = c(sans_ext(x), file_ext(x))
  if (tolower(x[2]) != 'png') stop(
    "To use hook_gifski(), the code chunk must generate 'png' images instead of '", x[2], "'."
  )
  fig.num = options$fig.num
  base = sub(paste0(fig.num, '$'), '', x[1])
  frames = paste0(base, format(seq_len(fig.num), trim = TRUE), '.', x[2])
  gif = paste0(base, '.gif')
  dpi = options$dpi
  gifski::gifski(
    frames, gif, width = options$fig.width * dpi, height = options$fig.height * dpi,
    delay = options$interval, loop = isTRUE(grepl('\\bloop\\b', options$aniopts)),
    progress = opts_knit$get('progress')
  )
  unlink(frames)
  # pretend it is a single image (gif) generated from the code chunk
  options$fig.show = 'asis'; options$fig.cur = 1; options$fig.num = 1
  knit_hooks$get('plot')(gif, options)
}


# use SciAnimator to create animations
#' @rdname hook_animation
#' @export
hook_scianimator = function(x, options) {
  x = c(sans_ext(x), file_ext(x))
  fig.num = options$fig.num
  base = opts_knit$get('base.url') %n% ''

  # write the div and js code here
  id = gsub('[^[:alnum:]]', '_', options$label)
  sprintf('
<div class="scianimator">
<div id="%s" style="display: inline-block;">
</div>
</div>
<script type="text/javascript">
  (function($) {
    $(document).ready(function() {
      var imgs = Array(%s);
      for (i = 0; ; i++) {
        if (i == imgs.length) break;
        imgs[i] = "%s%s" + (i + 1) + ".%s";
      }
      $("#%s").scianimator({
          "images": imgs,
          "delay": %s,
          "controls": ["first", "previous", "play", "next", "last", "loop", "speed"],
      });
      $("#%s").scianimator("play");
    });
  })(jQuery);
</script>
',
          id, fig.num, base, sub(paste0(fig.num, '$'), '', x[1]), x[2], id,
          options$interval * 1000, id)
}


# use the R2SWF package to create Flash animations
#' @rdname hook_animation
#' @export
hook_r2swf = function(x, options) {
  x = c(sans_ext(x), file_ext(x))
  fig.num = options$fig.num
  # set up the R2SWF run
  fig.name = paste0(sub(paste0(fig.num, '$'), '', x[1]), 1:fig.num, '.', x[2])
  swf.name = fig_path('.swf', options, NULL)

  w = options$out.width %n% (options$fig.width * options$dpi)
  h = options$out.height %n% (options$fig.height * options$dpi)

  swf2html = getFromNamespace('swf2html', 'R2SWF')
  file2swf = getFromNamespace('file2swf', 'R2SWF')
  swfhtml = swf2html(file2swf(files = fig.name, swf.name, interval = options$interval),
                     output = FALSE, fragment = TRUE,  width = w, height = h)
  if (options$fig.align == 'default') return(swfhtml)
  sprintf(paste('<div align = "%s">\n', swfhtml, '\n</div>'), options$fig.align)
}

#' @rdname output_hooks
#' @export
render_html = function() {
  set_html_dev()
  opts_knit$set(out.format = 'html')
  # use div with different classes
  html.hook = function(name) {
    force(name)
    function(x, options) {
      x = if (name == 'source') {
        c(hilight_source(x, 'html', options), '')
      } else escape_html(x)
      x = one_string(x)
      sprintf('<div class="%s"><pre class="knitr %s">%s</pre></div>\n', name, tolower(options$engine), x)
    }
  }
  h = opts_knit$get('header')
  if (!nzchar(h['highlight'])) set_header(highlight = .header.hi.html)
  z = list()
  for (i in c('source', 'warning', 'message', 'error'))
    z[[i]] = html.hook(i)
  knit_hooks$set(z)
  knit_hooks$set(inline = function(x) {
    sprintf(if (inherits(x, 'AsIs')) '%s' else '<code class="knitr inline">%s</code>',
            .inline.hook(format_sci(x, 'html')))
  }, output = html.hook('output'), plot = hook_plot_html, chunk = .chunk.hook.html)
}

## stolen from Romain's highlight package (v0.3.2)

# http://www.w3schools.com/css/css_colornames.asp
w3c.colors = c(
  aqua = '#00FFFF', black = '#000000', blue = '#0000FF', fuchsia = '#FF00FF',
  gray = '#808080', green = '#008000', lime = '#00FF00', maroon = '#800000',
  navy = '#000080', olive = '#808000', purple = '#800080', red = '#FF0000',
  silver = '#C0C0C0', teal = '#008080', white = '#FFFFFF', yellow = '#FFFF00'
)

css.parse.color = function(txt, default = '#000000') {
  txt = gsub('\\s+', '', tolower(txt))
  if (is.hex(txt)) return(txt)

  # css specs are from 0 to 255
  rgb = function(...) grDevices::rgb(..., maxColorValue = 255)

  # first we try to match against w3c standard colors
  if (!grepl('[^a-z]', txt) && txt %in% names(w3c.colors))
    return(w3c.colors[txt])

  # now we try R colors
  if (!grepl('[^a-z0-9]', txt)) {
    R.colors = colors()
    res = R.colors %in% txt
    if (any(res)) {
      return(rgb(t(col2rgb(R.colors[res]))))
    }
  }

  # next we try an rgb() specification
  if (grepl('rgb', txt)) {
    p = try(parse(text = txt), silent = TRUE)
    if (!inherits(p, 'try-error')) {
      res = try(eval(p), silent = TRUE)
      if (!inherits(res, 'try-error')) return(res)
    }
  }

  # fall back on the default color
  default
}

is.hex = function(x) grepl('^#[0-9a-f]{6}$', x)

# minimal css parser
css.parser = function(file, lines = readLines(file)) {

  rx = '^\\.(.*?) *\\{.*$'
  dec.lines = grep(rx, lines)
  dec.names = sub(rx, '\\1', lines[dec.lines])
  if (any(grepl('[0-9]', dec.names))) warning('use of numbers in style names')

  end.lines = grep('^\\s*\\}', lines)

  # find the closing brace of each declaration
  dec.close = end.lines[sapply(dec.lines, function(x) which.min(end.lines < x))]

  pos = matrix(c(dec.lines, dec.close), ncol = 2)
  styles = apply(pos, 1, function(x) {
    data = lines[(x[1] + 1):(x[2] - 1)]
    settings.rx = '^\\s*(.*?)\\s*:\\s*(.*?)\\s*;\\s*$'
    settings = sub(settings.rx, '\\1', data, perl = TRUE)
    contents = sub(settings.rx, '\\2', data, perl = TRUE)
    out = list()
    for (i in 1:length(settings)) {
      setting = settings[i]
      content = contents[i]
      out[[setting]] = switch(
        setting,
        color = css.parse.color(content, '#000000'),
        background = css.parse.color(content, '#FFFFFF'),
        content
      )
    }
    out
  })
  names(styles) = dec.names
  styles
}

#' Engines of other languages
#'
#' This object controls how to execute the code from languages other than R
#' (when the chunk option \code{engine} is not \code{'R'}). Each component in
#' this object is a function that takes a list of current chunk options
#' (including the source code) and returns a character string to be written into
#' the output.
#'
#' The engine function has one argument \code{options}: the source code of the
#' current chunk is in \code{options$code}. Usually we can call external
#' programs to run the code via \code{\link{system2}}. Other chunk options are
#' also contained in this argument, e.g. \code{options$echo} and
#' \code{options$eval}, etc.
#'
#' In most cases, \code{options$engine} can be directly used in command line to
#' execute the code, e.g. \code{python} or \code{ruby}, but sometimes we may
#' want to specify the path of the engine program, in which case we can pass it
#' through the \code{engine.path} option. For example, \code{engine='ruby',
#' engine.path='/usr/bin/ruby1.9.1'}. Additional command line arguments can be
#' passed through \code{options$engine.opts}, e.g. \code{engine='ruby',
#' engine.opts='-v'}.
#'
#' See \code{str(knitr::knit_engines$get())} for a list of built-in language
#' engines.
#' @export
#' @note The Leiningen engine \code{lein} requires lein-exec plugin; see
#'   \url{https://github.com/yihui/knitr/issues/1176} for details.
#' @references Usage: \url{https://yihui.org/knitr/objects/}; examples:
#'   \url{https://yihui.org/knitr/demo/engines/}
#' @examples knit_engines$get('python'); knit_engines$get('awk')
#' names(knit_engines$get())
knit_engines = new_defaults()


#' Cache engines of other languages
#'
#' This object controls how to load cached environments from languages other
#' than R (when the chunk option \code{engine} is not \code{'R'}). Each
#' component in this object is a function that takes the current path to the
#' chunk cache and loads it into the language environment.
#'
#' The cache engine function has one argument \code{options}, a list containing
#' all chunk options. Note that \code{options$hash} is the path to the current
#' chunk cache with the chunk's hash, but without any file extension, and the
#' language engine may write a cache database to this path (with an extension).
#'
#' The cache engine function should load the cache environment and should know
#' the extension appropriate for the language.
#' @references See \url{https://github.com/rstudio/reticulate/pull/167} for an
#'   implementation of a cache engine for Python.
#' @export
cache_engines = new_defaults()

#' An output wrapper for language engine output
#'
#' If you have designed a language engine, you may call this function in the end
#' to format and return the text output from your engine.
#'
#' For expert users, an advanced usage of this function is
#' \code{engine_output(options, out = LIST)} where \code{LIST} is a list that
#' has the same structure as the output of \code{evaluate::evaluate()}. In this
#' case, the arguments \code{code} and \code{extra} are ignored, and the list is
#' passed to an internal function \code{knitr:::wrap()} to return a character
#' vector of final output.
#' @param options A list of chunk options. Usually this is just the object
#'   \code{options} passed to the engine function; see
#'   \code{\link{knit_engines}}.
#' @param code Source code of the chunk, to which the output hook
#'   \code{source} is applied, unless the chunk option \code{echo} is \code{FALSE}.
#' @param out Text output from the engine, to which the hook \code{output}
#'   is applied, unless the chunk option \code{results} is \code{'hide'}
#' @param extra Any additional text output that you want to include.
#' @return A character string generated from the source code and output using
#'   the appropriate output hooks.
#' @export
#' @examples library(knitr)
#' engine_output(opts_chunk$merge(list(engine = 'Rscript')), code = '1 + 1', out = '[1] 2')
#' engine_output(opts_chunk$merge(list(echo = FALSE, engine = 'Rscript')), code = '1 + 1', out = '[1] 2')
#'
#' # expert use only
#' engine_output(opts_chunk$merge(list(engine = 'python')), out = list(structure(list(src = '1 + 1'), class = 'source'), '2'))
engine_output = function(options, code, out, extra = NULL) {
  if (missing(code) && is.list(out)) return(unlist(wrap(out, options)))
  if (!is.logical(options$echo)) code = code[options$echo]
  if (length(code) != 1L) code = one_string(code)
  if (options$engine == 'sas' && length(out) > 1L && !grepl('[[:alnum:]]', out[2]))
    out = tail(out, -3L)
  if (length(out) != 1L) out = one_string(out)
  out = sub('([^\n]+)$', '\\1\n', out)
  # replace the engine names for markup later, e.g. ```Rscript should be ```r
  options$engine = switch(
    options$engine, mysql = 'sql', node = 'javascript', psql = 'sql', Rscript = 'r',
    options$engine
  )
  if (options$engine == 'stata') {
    out = gsub('\n+running.*profile.do', '', out)
    out = sub('...\n+', '', out)
    out = sub('\n. \nend of do-file\n', '', out)
  }
  one_string(c(
    if (length(options$echo) > 1L || options$echo) knit_hooks$get('source')(code, options),
    if (options$results != 'hide' && !is_blank(out)) {
      if (options$engine == 'highlight') out else wrap.character(out, options)
    },
    extra
  ))
}

## command-line tools
eng_interpreted = function(options) {
  engine = options$engine
  code = if (engine %in% c('highlight', 'Rscript', 'sas', 'haskell', 'stata')) {
    f = wd_tempfile(engine, switch(engine, sas = '.sas', Rscript = '.R', stata = '.do', '.txt'))
    write_utf8(c(switch(
      engine,
      sas = "OPTIONS NONUMBER NODATE PAGESIZE = MAX FORMCHAR = '|----|+|---+=|-/<>*' FORMDLIM=' ';title;",
      NULL
    ), options$code), f)
    on.exit(unlink(f), add = TRUE)
    switch(
      engine,
      haskell = paste('-e', shQuote(paste(':script', f))),
      sas = {
        logf = sub('[.]sas$', '.lst', f)
        on.exit(unlink(c(logf, sub('[.]sas$', '.log', f))), add = TRUE)
        f
      },
      stata = {
        logf = sub('[.]do$', '.log', f)
        on.exit(unlink(c(logf)), add = TRUE)
        sprintf(switch(
          Sys.info()[['sysname']],
          Windows = '/q /e do %s',
          Darwin = paste('-q < %s >', shQuote(xfun::normalize_path(logf))),
          Linux = '-q -e do %s',
          '-q -b do %s'
        ), shQuote(normalizePath(f)))
      },
      f
    )
  } else paste(switch(
    engine, bash = '-c', coffee = '-e', groovy = '-e', lein = 'exec -ep',
    mysql = '-e', node = '-e', octave = '--eval', perl = '-E', psql = '-c',
    python = '-c', ruby = '-e', scala = '-e', sh = '-c', zsh = '-c', NULL
  ), shQuote(one_string(options$code)))

  opts = get_engine_opts(options$engine.opts, engine)
  # FIXME: for these engines, the correct order is options + code + file
  code = if (engine %in% c('awk', 'gawk', 'sed', 'sas'))
    paste(code, opts) else paste(opts, code)
  cmd = get_engine_path(options$engine.path, engine)
  out = if (options$eval) {
    if (options$message) message('running: ', cmd, ' ', code)
    tryCatch(
      system2(cmd, code, stdout = TRUE, stderr = TRUE, env = options$engine.env),
      error = function(e) {
        if (!options$error) stop(e)
        paste('Error in running command', cmd)
      }
    )
  } else ''
  # chunk option error=FALSE means we need to signal the error
  if (!options$error && !is.null(attr(out, 'status'))) stop(one_string(out))
  if (options$eval && engine %in% c('sas', 'stata') && file.exists(logf))
    out = c(read_utf8(logf), out)
  engine_output(options, options$code, out)
}

# options$engine.path can be list(name1 = path1, name2 = path2, ...); similarly,
# options$engine.opts can be list(name1 = opts1, ...)
get_engine_opts = function(opts, engine, fallback = '') {
  if (is.list(opts)) opts = opts[[engine]]
  opts %n% fallback
}

get_engine_path = function(path, engine) get_engine_opts(path, engine, engine)

## C and Fortran (via R CMD SHLIB)
eng_shlib = function(options) {
  n = switch(options$engine, c = 'c', fortran = 'f', fortran95 = 'f95')
  f = wd_tempfile(n, paste0('.', n))
  write_utf8(options$code, f)
  on.exit(unlink(c(f, with_ext(f, c('o', 'so', 'dll')))), add = TRUE)
  if (options$eval) {
    out = system(paste('R CMD SHLIB', f), intern = TRUE)
    dyn.load(sub(sprintf('[.]%s$', n), .Platform$dynlib.ext, f))
  } else out = ''
  engine_output(options, options$code, out)
}

## Python
eng_python = function(options) {
  if (isFALSE(options$python.reticulate)) {
    eng_interpreted(options)
  } else {
    if (!loadable('reticulate')) warning2(
      "The 'python' engine in knitr requires the reticulate package. ",
      "If you do not want to use the reticulate package, set the chunk option ",
      "python.reticulate = FALSE."
    )
    reticulate::eng_python(options)
  }
}

cache_eng_python = function(options) {
  if (isFALSE(options$python.reticulate)) return()
  # TODO: change this hack to reticulate::cache_eng_python(options) after
  # https://github.com/rstudio/reticulate/pull/167 is merged and released
  if (!'cache_eng_python' %in% ls(asNamespace('reticulate'))) return()
  fun = getFromNamespace('cache_eng_python', 'reticulate')
  fun(options)
}

## Java
#  e.g. see http://cran.rstudio.com/package=jvmr

## Rcpp
eng_Rcpp = function(options) {

  sourceCpp = getFromNamespace('sourceCpp', 'Rcpp')

  code = one_string(options$code)
  # engine.opts is a list of arguments to be passed to Rcpp function, e.g.
  # engine.opts=list(plugin='RcppArmadillo')
  opts = options$engine.opts

  # use custom cacheDir for sourceCpp if it's supported
  cache = options$cache && ('cacheDir' %in% names(formals(sourceCpp)))
  if (cache) {
    opts$cacheDir = paste(valid_path(options$cache.path, options$label), 'sourceCpp', sep = '_')
    opts$cleanupCacheDir = TRUE
  }

  if (!is.environment(opts$env)) opts$env = knit_global() # default env is knit_global()
  if (options$eval) {
    message('Building shared library for Rcpp code chunk...')
    do.call(sourceCpp, c(list(code = code), opts))
  }

  options$engine = 'cpp' # wrap up source code in cpp syntax instead of Rcpp
  engine_output(options, code, '')
}

## Julia
eng_julia = function(options) {
  JuliaCall::eng_juliacall(options)
}

## Stan
## Compiles Stan model in the code chunk, creates a stanmodel object,
## and assigns it to a variable with the name given in engine.opts$x.
eng_stan = function(options) {
  code = one_string(options$code)
  opts = options$engine.opts
  ## name of the modelfit object returned by stan_model
  if (is.null(x <- options$output.var)) {
    warning("the option engine.opts$x is deprecated; use the chunk option output.var instead")
    x = opts$x
  }
  if (!is.character(x) || length(x) != 1L) stop(
    "the chunk option output.var must be a character string ",
    "providing a name for the returned `stanmodel` object."
  )
  opts$x = NULL
  if (options$eval) {
    message("Creating a 'stanmodel' object ", x)
    assign(
      x,
      do.call(getFromNamespace('stan_model', 'rstan'), c(list(model_code = code), opts)),
      envir = knit_global()
    )
  }
  engine_output(options, code, '')
}

## convert tikz string to PDF
eng_tikz = function(options) {
  if (!options$eval) return(engine_output(options, options$code, ''))

  lines = read_utf8(options$engine.opts$template %n%
                    system.file('misc', 'tikz2pdf.tex', package = 'knitr'))
  i = grep('%% TIKZ_CODE %%', lines)
  if (length(i) != 1L)
    stop("Couldn't find replacement string; or the are multiple of them.")

  s = append(lines, options$code, i)  # insert tikz into tex-template
  write_utf8(s, texf <- wd_tempfile('tikz', '.tex'))
  on.exit(unlink(texf), add = TRUE)

  ext = tolower(options$fig.ext %n% dev2ext(options$dev))

  to_svg = ext == 'svg'
  outf = if (to_svg) tinytex::latexmk(texf, 'latex') else tinytex::latexmk(texf)

  fig = fig_path(if (to_svg) '.dvi' else '.pdf', options)
  dir.create(dirname(fig), recursive = TRUE, showWarnings = FALSE)
  file.rename(outf, fig)

  fig2 = with_ext(fig, ext)
  if (to_svg) {
    # dvisvgm needs to be on the path
    # dvisvgm for windows needs ghostscript bin dir on the path also
    if (Sys.which('dvisvgm') == '') tinytex::tlmgr_install('dvisvgm')
    if (system2('dvisvgm', c('-o', shQuote(fig2), fig)) != 0)
      stop('Failed to compile ', fig, ' to ', fig2)
  } else {
    # convert to the desired output-format using magick
    if (ext != 'pdf') magick::image_write(do.call(magick::image_convert, c(
      list(magick::image_read_pdf(fig), ext), options$engine.opts$convert.opts
    )), fig2)
  }
  fig = fig2

  options$fig.num = 1L; options$fig.cur = 1L
  extra = knit_hooks$get('plot')(fig, options)
  options$engine = 'tex'  # for output hooks to use the correct language class
  engine_output(options, options$code, '', extra)
}

## GraphViz (dot) and Asymptote are similar
eng_dot = function(options) {

  # create temporary file
  f = wd_tempfile('code')
  write_utf8(code <- options$code, f)
  on.exit(unlink(f), add = TRUE)

  # adapt command to either graphviz or asymptote
  if (options$engine == 'dot') {
    command_string = '%s %s -T%s -o%s'
    syntax         = 'dot'
  } else if (options$engine == 'asy') {
    command_string = '%s %s -f %s -o %s'
    syntax         = 'cpp'  # use cpp syntax for syntax highlighting
  }

  # prepare system command
  cmd = sprintf(
    command_string, shQuote(get_engine_path(options$engine.path, options$engine)),
    shQuote(f), ext <- options$fig.ext %n% dev2ext(options$dev),
    shQuote(paste0(fig <- fig_path(), '.', ext))
  )

  # generate output
  dir.create(dirname(fig), recursive = TRUE, showWarnings = FALSE)
  outf = paste(fig, ext, sep = '.')
  unlink(outf)
  extra = if (options$eval) {
    message('running: ', cmd)
    system(cmd)
    if (!file.exists(outf)) stop('failed to compile content');
    options$fig.num = 1L; options$fig.cur = 1L
    knit_hooks$get('plot')(outf, options)
  }

  # wrap
  options$engine = syntax
  engine_output(options, code, '', extra)
}

## Andre Simon's highlight
eng_highlight = function(options) {
  # e.g. engine.opts can be '-S matlab -O latex'
  if (is.null(options$engine.opts)) options$engine.opts = '-S text'
  options$engine.opts[1L] = paste('-f', options$engine.opts[1L])
  options$echo = FALSE; options$results = 'asis'  # do not echo source code
  res = eng_interpreted(options)
  if (out_format('latex')) {
    highlight_header()
    sub('(.*)\\\\\\\\(.*)', '\\1\\2', res)
  } else res
}

## save the code
eng_cat = function(options) {
  cat2 = function(..., file = '', lang = NULL) {
    # do not write to stdout like the default behavior of cat()
    if (!identical(file, '')) cat(..., file = file)
  }
  if (options$eval)
    do.call(cat2, c(list(options$code, sep = '\n'), options$engine.opts))

  if (is.null(lang <- options$engine.opts$lang) && is.null(lang <- options$class.source))
    return('')
  options$engine = lang
  engine_output(options, options$code, NULL)
}

## output the code without processing it
eng_asis = function(options) {
  if (options$echo && options$eval) one_string(options$code)
}

# write a block environment according to the output format
eng_block = function(options) {
  if (isFALSE(options$echo)) return()
  code = one_string(options$code)
  to = pandoc_to()
  is_pandoc = !is.null(to)
  if (!is_pandoc) {
    # not in R Markdown v2
    to = out_format()
    if (!(to %in% c('latex', 'html', 'markdown'))) to = NULL
  }
  if (is.null(to)) return(code)
  if (to == 'beamer') to = 'latex'
  if (is_html_output(to)) to = 'html'
  type = options$type
  if (is.null(type)) return(code)
  # convert the chunk content to HTML or LaTeX (ideally I only need to specify
  # the markdown extension, but it is not implemented yet for LaTeX:
  # https://github.com/jgm/pandoc/issues/2453)
  if (is_pandoc) code = pandoc_fragment(code, if (to == 'html') 'html4' else to)
  l1 = options$latex.options
  if (is.null(l1)) l1 = ''
  h2 = options$html.tag %n% 'div'
  h3 = options$html.before %n% ''
  h4 = options$html.after %n% ''
  # e.g. type = c(latex = 'marginfigure', html = 'marginnote')
  if (to %in% names(type)) type = type[to]
  # block level tags? this is an incomplete list, but should work for most cases
  if (to == 'html') if (h2 %in% c('div', 'p', 'blockquote')) {
    code = paste0('\n', code, '\n')
  } else {
    code = gsub('<p>', '<span style="display: block;">', code)
    code = gsub('</p>', '</span>', code)
  }
  switch(
    to,
    latex = sprintf('\\begin{%s}%s\n%s\n\\end{%s}', type, l1, code, type),
    html =  sprintf('%s<%s class="%s">%s</%s>%s', h3, h2, type, code, h2, h4),
    code
  )
}

eng_block2 = function(options) {
  if (isFALSE(options$echo)) return()

  code = one_string(options$code); type = options$type
  if (is.null(type)) return(code)

  if (is.null(pandoc_to())) stop('The engine "block2" is for R Markdown only')

  l1 = options$latex.options
  if (is.null(l1)) l1 = ''
  # protect environment options because Pandoc may escape the characters like
  # {}; when encoded in integers, they won't be escaped, but will need to
  # restore them later; see bookdown:::restore_block2
  if (l1 != '') l1 = paste(
    c('\\iffalse{', utf8ToInt(enc2utf8(l1)), '}\\fi{}'), collapse = '-'
  )
  h2 = options$html.tag %n% 'div'
  h3 = options$html.before %n% ''
  h4 = options$html.after %n% ''
  h5 = options$html.before2 %n% ''
  h6 = options$html.after2 %n% ''

  sprintf(
    '\\BeginKnitrBlock{%s}%s%s<%s class="%s">%s%s%s</%s>%s\\EndKnitrBlock{%s}',
    type, l1, h3, h2, type, h5, code, h6, h2, h4, type
  )
}

# helper to create engines the wrap embedded html assets (e.g. css,js)
eng_html_asset = function(prefix, postfix) {
  function(options) {
    out = if (options$eval && is_html_output(excludes = 'markdown')) {
      one_string(c(prefix, options$code, postfix))
    }
    options$results = 'asis'
    engine_output(options, options$code, out)
  }
}

# include js in a script tag (ignore if not html output)
eng_js = eng_html_asset('<script type="text/javascript">', '</script>')

# include css in a style tag (ignore if not html output)
eng_css = eng_html_asset('<style type="text/css">', '</style>')

# perform basic sql parsing to determine if a sql query is an update query
is_sql_update_query = function(query) {
  query = one_string(query)
  # remove line comments
  query = gsub('^\\s*--.*\n', '', query)
  # remove multi-line comments
  if (grepl('^\\s*\\/\\*.*', query)) query = gsub('.*\\*\\/', '', query)
  grepl('^\\s*(INSERT|UPDATE|DELETE|CREATE|DROP).*', query, ignore.case = TRUE)
}

# sql engine
eng_sql = function(options) {
  # return chunk before interpolation eagerly to avoid connection option check
  if (isFALSE(options$eval) && !isTRUE(options$sql.show_interpolated)) {
    return(engine_output(options, options$code, ''))
  }

  # Return char vector of sql interpolation param names
  varnames_from_sql = function(conn, sql) {
    varPos = DBI::sqlParseVariables(conn, sql)
    if (length(varPos$start) > 0) {
      varNames = substring(sql, varPos$start, varPos$end)
      sub('^\\?', '', varNames)
    }
  }

  # Vectorized version of exists
  mexists = function(x, env = knit_global(), inherits = TRUE) {
    vapply(x, exists, logical(1), where = env, inherits = inherits)
  }

  # Interpolate a sql query based on the variables in an environment
  interpolate_from_env = function(conn, sql, env = knit_global(), inherits = TRUE) {
    names = unique(varnames_from_sql(conn, sql))
    names_missing = names[!mexists(names, env, inherits)]
    if (length(names_missing) > 0) {
      stop("Object(s) not found: ", paste('"', names_missing, '"', collapse = ", "))
    }

    args = if (length(names) > 0) setNames(
      mget(names, envir = env, inherits = inherits), names
    )

    do.call(DBI::sqlInterpolate, c(list(conn, sql), args))
  }

  # extract options
  conn = options$connection
  if (is.character(conn)) conn = get(conn, envir = knit_global())
  if (is.null(conn)) stop2(
    "The 'connection' option (DBI connection) is required for sql chunks."
  )
  varname = options$output.var
  max.print = options$max.print %n% (opts_knit$get('sql.max.print') %n% 10)
  if (is.na(max.print) || is.null(max.print))
    max.print = -1
  sql = one_string(options$code)

  query = interpolate_from_env(conn, sql)
  if (isFALSE(options$eval)) return(engine_output(options, query, ''))

  if (is_sql_update_query(query)) {
    DBI::dbExecute(conn, query)
    data = NULL
  } else if (is.null(varname) && max.print > 0) {
    # execute query -- when we are printing with an enforced max.print we
    # use dbFetch so as to only pull down the required number of records
    res = DBI::dbSendQuery(conn, query)
    data = DBI::dbFetch(res, n = max.print)
    DBI::dbClearResult(res)
  } else {
    data = DBI::dbGetQuery(conn, query)
  }

  # create output if needed (we have data and we aren't assigning it to a variable)
  output = if (length(dim(data)) == 2 && ncol(data) > 0 && is.null(varname)) capture.output({

    # apply max.print to data
    display_data = if (max.print == -1) data else head(data, n = max.print)

    # get custom sql print function
    sql.print = opts_knit$get('sql.print')

    # use kable for markdown
    if (!is.null(sql.print)) {
      options$results = 'asis'
      cat(sql.print(data))
    } else if (out_format('markdown')) {

      # we are going to output raw markdown so set results = 'asis'
      options$results = 'asis'

      # force left alignment if the first column is an incremental id column
      first_column = display_data[[1]]
      if (is.numeric(first_column) && all(diff(first_column) == 1))
        display_data[[1]] = as.character(first_column)

      # wrap html output in a div so special styling can be applied
      if (is_html_output()) cat('<div class="knitsql-table">\n')

      # determine records caption
      caption = options$tab.cap
      if (is.null(caption)) {
        rows = nrow(data)
        rows_formatted = formatC(rows, format = "d", big.mark = ',')
        caption = if (max.print == -1 || rows < max.print) {
          paste(rows_formatted, "records")
        } else {
          paste("Displaying records 1 -", rows_formatted)
        }
      }
      # disable caption
      if (identical(caption, NA)) caption = NULL

      # print using kable
      print(kable(display_data, caption = caption))

      # terminate div
      if (is_html_output()) cat("\n</div>\n")

      # otherwise use tibble if it's available
    } else if (loadable('tibble')) {
      print(tibble::as_tibble(display_data), n = max.print)

    } else print(display_data) # fallback to standard print
  })
  if (options$results == 'hide') output = NULL

  # assign varname if requested
  if (!is.null(varname)) assign(varname, data, envir = knit_global())

  # reset query to pre-interpolated if not expanding
  if (!isTRUE(options$sql.show_interpolated)) query <- options$code

  # return output
  engine_output(options, query, output)
}

# go engine, added by @hodgesds https://github.com/yihui/knitr/pull/1330
eng_go = function(options) {
  f = wd_tempfile('code', '.go')
  write_utf8(code <- options$code, f)
  on.exit(unlink(f), add = TRUE)
  cmd = get_engine_path(options$engine.path, options$engine)

  fmt_args = sprintf('fmt %s', f)

  tryCatch(
    system2(cmd, fmt_args, stdout = TRUE, stderr = TRUE, env = options$engine.env),
    error = function(e) {
      if (!options$error) stop(e)
    }
  )

  run_args = sprintf(" run %s", f)

  extra = if (options$eval) {
    message('running: ', cmd, run_args)
    tryCatch(
      system2(cmd, run_args, stdout = TRUE, stderr = TRUE, env = options$engine.env),
      error = function(e) {
        if (!options$error) stop(e)
        'Error in executing go code'
      }
    )
  }

  if (options$results == 'hide') extra = NULL

  engine_output(options, code, extra)
}

# SASS / SCSS engine (contributed via https://github.com/yihui/knitr/pull/1666)
#
# Converts SASS / SCSS -> CSS (with same treatments as CSS engine) using either:
# LibSass sass R package (https://github.com/rstudio/sass) when
#   + the package is installed
#   + engine.opts does not set package = FALSE (e.g. engine.opts = list(package = FALSE))
#   + an explicit path to the executable is not provided through engine.path, or
# dart-sass standalone executable (https://sass-lang.com/install) otherwise
#
# CSS output is compressed by default but formatting can be set through style in engine.opts
#  For the sass R package, valid styles are "compressed","expanded", "nested", and "compact"
#  For the executable, valid styles are "compressed" and "expanded"
#  Please refer to respective package / executable documentation for more details
eng_sxss = function(options) {

  # early exit if evaluated output not requested
  options$results = 'asis'
  if (!options$eval) return(engine_output(options, options$code, ''))

  # create temporary file with input code
  f = wd_tempfile('code', paste0('.', options$engine))
  xfun::write_utf8(options$code , f)
  on.exit(unlink(f), add = TRUE)

  # process provided engine options
  package = options$engine.opts$package %n% TRUE
  style = options$engine.opts$style %n% "compressed"
  cmd = get_engine_path(options$engine.path, "sass")

  use_package = loadable("sass") && package && cmd == "sass"

  style = match.arg(style, c("compressed", "expanded", if (use_package) c("compact", "nested")))
  # convert sass/sxss -> css
  if (use_package) {
    message("Converting with the R package sass")

    sass_fun = options$engine.opts$sass_fun %n% sass::sass
    out = tryCatch(
      sass_fun(sass::sass_file(f), options = sass::sass_options(output_style = style)),
      error = function(e) {
        if (!options$error) stop(e)
        warning2(paste('Error in converting to CSS using sass R package:', e, sep = "\n"))
        NULL
      }
    )

    # remove final newline chars from output
    if (!is.null(out)) out = sub("\\n$", "", out)
  } else {
    message("Converting sass with ", cmd)
    style = paste0("--style=", style)

    # attempt execution of sass
    out = tryCatch(
      system2(cmd, args = c(f, style), stdout = TRUE, stderr = TRUE),
      error = function(e) {
        if (!options$error) stop2(e)
        warning2(paste('Error in converting to CSS using executable:', e, sep = "\n"))
        NULL
      }
    )

    # handle execution errors (status codes)
    if (!is.null(attr(out, 'status')) && !options$error) stop2(one_string(out))
  }

  # wrap final output for correct rendering
  final_out = if (!is.null(out) && is_html_output(excludes = 'markdown')) {
    one_string(c('<style type="text/css">', out, '</style>'))
  }

  engine_output(options, options$code, final_out)

}

# set engines for interpreted languages
local({
  for (i in c(
    'awk', 'bash', 'coffee', 'gawk', 'groovy', 'haskell', 'lein', 'mysql',
    'node', 'octave', 'perl', 'psql', 'Rscript', 'ruby', 'sas',
    'scala', 'sed', 'sh', 'stata', 'zsh'
  )) knit_engines$set(setNames(list(eng_interpreted), i))
})

# additional engines
knit_engines$set(
  highlight = eng_highlight, Rcpp = eng_Rcpp, tikz = eng_tikz, dot = eng_dot,
  c = eng_shlib, fortran = eng_shlib, fortran95 = eng_shlib, asy = eng_dot,
  cat = eng_cat, asis = eng_asis, stan = eng_stan, block = eng_block,
  block2 = eng_block2, js = eng_js, css = eng_css, sql = eng_sql, go = eng_go,
  python = eng_python, julia = eng_julia, sass = eng_sxss, scss = eng_sxss
)

cache_engines$set(python = cache_eng_python)

get_engine = function(name) {
  fun = knit_engines$get(name)
  if (is.function(fun)) return(fun)
  warning(
    "Unknown language engine '", name,
    "' (must be registered via knit_engines$set())."
  )
  function(options) {
    engine_output(options, options$code, '')
  }
}

cache_engine = function(options) {
  cache_fun = cache_engines$get(options$engine)
  if (!is.function(cache_fun)) return()
  cache_fun(options)
}

# possible values for engines (for auto-completion in RStudio)
opts_chunk_attr$engine = as.list(sort(c('R', names(knit_engines$get()))))
opts_chunk_attr[c('engine.path', 'engine.opts')] = list('character', 'character')

otel_tracer_name = 'org.yihui.knitr'

# generic otel helpers:

otel_cache_tracer = NULL
otel_active_span = NULL

local({
  otel_tracer = NULL
  otel_is_tracing = FALSE

  otel_cache_tracer <<- function() {
    requireNamespace('otel', quietly = TRUE) || return()
    otel_tracer <<- otel::get_tracer(otel_tracer_name)
    otel_is_tracing <<- tracer_enabled(otel_tracer)
  }

  # - without specifying `scope`, the span ends when this function returns;
  #   to make this a local span (last as long as the function it is called from),
  #   specify `scope = environment()`
  # - arguments remain unevaluated on early return
  otel_active_span <<- function(
    name,
    label,
    attributes = list(),
    scope = environment()
  ) {
    otel_is_tracing || return()
    otel::start_local_active_span(
      name = sprintf('%s %s', name, label),
      attributes = otel::as_attributes(attributes),
      tracer = otel_tracer,
      activation_scope = scope
    )
  }
})

tracer_enabled = function(tracer) {
  .subset2(tracer, 'is_enabled')()
}

with_otel_record = function(expr) {
  on.exit(otel_cache_tracer())
  otelsdk::with_otel_record({
    otel_cache_tracer()
    expr
  })
}

# knitr-specific helpers:

make_chunk_attributes = function(options) {
  list(
    knitr.chunk.device = options$dev,
    knitr.chunk.echo = options$echo,
    knitr.chunk.engine = options$engine,
    knitr.chunk.eval = options$eval,
    knitr.chunk.label = options$label
  )
}

make_knitr_attributes = function() {
  list(
    knitr.format = out_format(),
    knitr.input = get_knitr_concord('infile'),
    knitr.output = get_knitr_concord('outfile')
  )
}

# safe version that always returns a string
get_knitr_concord = function(name) {
  knit_concord$get(name) %n% ''
}

#' A general-purpose tool for dynamic report generation in R
#'
#' This is an alternative tool to Sweave with a more flexible design and new
#' features like caching and finer control of graphics. It is not limited to
#' LaTeX and is ready to be customized to process other file formats. See the
#' package website in the references for more information and examples.
#' @docType package
#' @name knitr-package
#' @aliases knitr
#' @import graphics grDevices methods stats utils
#' @author Yihui Xie <\url{https://yihui.org}>
#' @seealso The core function in this package: \code{\link{knit}}. If you are an
#'   Sweave user, see \code{\link{Sweave2knitr}} on how to convert Sweave files
#'   to \pkg{knitr}.
#' @note The pronunciation of \pkg{knitr} is similar to \emph{neater} (neater
#'   than what?) or you can think of \emph{knitter} (but it is \emph{single t}).
#'   The name comes from \code{knit} + \code{R} (while \code{Sweave} = \code{S}
#'   + \code{weave}).
#' @references Full documentation and demos: \url{https://yihui.org/knitr/};
#'   FAQ's: \url{https://yihui.org/knitr/faq/}
#' @importFrom xfun attr file_ext isFALSE is_windows loadable parse_only
#'   sans_ext try_silent with_ext read_utf8 write_utf8 file_string
NULL

.knitEnv = new.env()

.knitEnv$meta = list()

# no partial matching for lists!!
#' @export
`$.knitr_strict_list` = function(x, name) x[[name]]

as.strict_list = function(x) {
  if (!is.list(x)) stop("'x' is not a list")
  class(x) = 'knitr_strict_list'
  x
}

#' Default plot hooks for different output formats
#'
#' These hook functions define how to mark up graphics output in different
#' output formats.
#'
#' Depending on the options passed over, \code{hook_plot_tex} may return the
#' normal \samp{\\includegraphics{}} command, or \samp{\\input{}} (for tikz
#' files), or \samp{\\animategraphics{}} (for animations); it also takes many
#' other options into consideration to align plots and set figure sizes, etc.
#' Similarly, \code{hook_plot_html}, \code{hook_plot_md} and
#' \code{hook_plot_rst} return character strings which are HTML, Markdown, reST
#' code.
#'
#' In most cases we do not need to call these hooks explicitly, and they were
#' designed to be used internally. Sometimes we may not be able to record R
#' plots using \code{\link[grDevices]{recordPlot}}, and we can make use of these
#' hooks to insert graphics output in the output document; see
#' \code{\link{hook_plot_custom}} for details.
#' @param x Filename for the plot (a character string).
#' @param options A list of the current chunk options.
#' @rdname hook_plot
#' @return A character string of code, with plot filenames wrapped.
#' @references \url{https://yihui.org/knitr/hooks/}
#' @seealso \code{\link{hook_plot_custom}}
#' @export
#' @examples # this is what happens for a chunk like this
#'
#' # <<foo-bar-plot, dev='pdf', fig.align='right'>>=
#' hook_plot_tex('foo-bar-plot.pdf', opts_chunk$merge(list(fig.align='right')))
#'
#' # <<bar, dev='tikz'>>=
#' hook_plot_tex('bar.tikz', opts_chunk$merge(list(dev='tikz')))
#'
#' # <<foo, dev='pdf', fig.show='animate', interval=.1>>=
#'
#' # 5 plots are generated in this chunk
#' hook_plot_tex('foo5.pdf', opts_chunk$merge(list(fig.show='animate',interval=.1,fig.cur=5, fig.num=5)))
hook_plot_tex = function(x, options) {
  # This function produces the image inclusion code for LaTeX.
  # optionally wrapped in code that resizes it, aligns it, handles it
  # as a subfigure, and/or wraps it in a float. Here is a road map of
  # the intermediate variables this function fills in (or leaves empty,
  # as needed), and an impression of their (possible) contents.
  #
  #     fig1,                   # \begin{...}[...]
  #       align1,               #   {\centering
  #         sub1,               #     \subfloat[...]{
  #           resize1,          #       \resizebox{...}{...}{
  #             tikz code       #         '\\input{chunkname.tikz}'
  #             or animate code #         or '\\animategraphics[size]{1/interval}{chunkname}{1}{fig.num}'
  #             or plain code   #         or '\\includegraphics[size]{chunkname}'
  #           resize2,          #       }
  #         sub2,               #     }
  #       align2,               #   }
  #     fig2                    #   \caption[...]{...\label{...}}
  #                             # \end{...}  % still fig2

  rw = options$resize.width
  rh = options$resize.height
  resize1 = resize2 = ''
  if (!is.null(rw) || !is.null(rh)) {
    resize1 = sprintf('\\resizebox{%s}{%s}{', rw %n% '!', rh %n% '!')
    resize2 = '} '
  }

  tikz = is_tikz_dev(options)

  a = options$fig.align
  fig.cur = options$fig.cur %n% 1L
  fig.num = options$fig.num %n% 1L
  animate = options$fig.show == 'animate'
  fig.ncol = options$fig.ncol %n% fig.num
  if (is.null(fig.sep <- options$fig.sep)) {
    fig.sep = character(fig.num)
    if (fig.ncol < fig.num) fig.sep[seq(fig.ncol, fig.num - 1L, fig.ncol)] = '\\newline'
  }
  sep.cur = NULL

  # If this is a non-tikz animation, skip to the last fig.
  if (!tikz && animate && fig.cur < fig.num) return('')

  usesub = length(subcap <- options$fig.subcap) && fig.num > 1
  # multiple plots: begin at 1, end at fig.num
  ai = options$fig.show != 'hold'

  # TRUE if this picture is standalone or first in set
  plot1 = ai || fig.cur <= 1L
  # TRUE if this picture is standalone or last in set
  plot2 = ai || fig.cur == fig.num

  # open align code if this picture is standalone/first in set
  align1 = if (plot1)
    switch(a, left = '\n\n', center = '\n\n{\\centering ', right = '\n\n\\hfill{}', '\n')
  # close align code if this picture is standalone/last in set
  align2 = if (plot2)
    switch(a, left = '\\hfill{}\n\n', center = '\n\n}\n\n', right = '\n\n', '')

  # figure environment: caption, short caption, label
  cap = options$fig.cap
  scap = options$fig.scap
  fig1 = fig2 = ''
  mcap = fig.num > 1L && options$fig.show == 'asis' && !length(subcap)
  # initialize subfloat strings
  sub1 = sub2 = ''

  # Wrap in figure environment only if user specifies a caption
  if (length(cap) && !is.na(cap)) {
    lab = paste0(options$fig.lp, options$label)
    # If pic is standalone/first in set: open figure environment
    if (plot1) {
      pos = options$fig.pos
      if (pos != '' && !grepl('^[[{]', pos)) pos = sprintf('[%s]', pos)
      fig1 = sprintf('\\begin{%s}%s', options$fig.env, pos)
    }
    # Add subfloat code if needed
    if (usesub) {
      sub1 = sprintf('\\subfloat[%s%s]{', subcap, create_label(lab, fig.cur, latex = TRUE))
      sub2 = '}'
      sep.cur = fig.sep[fig.cur]; if (is.na(sep.cur)) sep.cur = NULL
    }

    # If pic is standalone/last in set:
    # * place caption with label
    # * close figure environment
    if (plot2) {
      if (is.null(scap) && !grepl('[{].*?[:.;].*?[}]', cap)) {
        scap = strsplit(cap, '[:.;]( |\\\\|$)')[[1L]][1L]
      }
      scap = if (is.null(scap) || is.na(scap)) '' else sprintf('[%s]', scap)
      cap = if (cap == '') '' else sprintf(
        '\\caption%s{%s}%s\n', scap, cap,
        create_label(lab, if (mcap) fig.cur, latex = TRUE)
      )
      fig2 = sprintf('%s\\end{%s}\n', cap, options$fig.env)
    }
  } else if (pandoc_to(c('latex', 'beamer'))) {
    # use alignment environments for R Markdown latex output (\centering won't work)
    align.env = switch(a, left = 'flushleft', center = 'center', right = 'flushright')
    align1 = if (plot1) if (a == 'default') '\n' else sprintf('\n\n\\begin{%s}', align.env)
    align2 = if (plot2) if (a == 'default') '' else sprintf('\\end{%s}\n\n', align.env)
  }

  ow = options$out.width
  # maxwidth does not work with animations
  if (animate && identical(ow, '\\maxwidth')) ow = NULL
  if (is.numeric(ow)) ow = paste0(ow, 'px')
  size = paste(c(sprintf('width=%s', ow),
                 sprintf('height=%s', options$out.height),
                 options$out.extra), collapse = ',')

  paste0(
    fig1, align1, sub1, resize1,
    if (tikz) {
      sprintf('\\input{%s}', x)
    } else if (animate) {
      # \animategraphics{} should be inserted only *once*!
      aniopts = options$aniopts
      aniopts = if (is.na(aniopts)) NULL else gsub(';', ',', aniopts)
      size = paste(c(size, sprintf('%s', aniopts)), collapse = ',')
      if (nzchar(size)) size = sprintf('[%s]', size)
      sprintf('\\animategraphics%s{%s}{%s}{%s}{%s}', size, 1 / options$interval,
              sub(sprintf('%d$', fig.num), '', sans_ext(x)), 1L, fig.num)
    } else {
      if (nzchar(size)) size = sprintf('[%s]', size)
      res = sprintf(
        '\\includegraphics%s{%s} ', size,
        if (getOption('knitr.include_graphics.ext', FALSE)) x else sans_ext(x)
      )
      lnk = options$fig.link
      if (is.null(lnk) || is.na(lnk)) res else sprintf('\\href{%s}{%s}', lnk, res)
    },

    resize2, sub2, sep.cur, align2, fig2
  )
}

.chunk.hook.tex = function(x, options) {
  ai = output_asis(x, options)
  col = if (!ai) paste0(
    color_def(options$background), if (!is_tikz_dev(options)) '\\color{fgcolor}'
  )
  k1 = paste0(col, '\\begin{kframe}\n')
  k2 = '\\end{kframe}'
  x = .rm.empty.envir(paste0(k1, x, k2))
  size = if (options$size == 'normalsize') '' else sprintf('\\%s', options$size)
  if (!ai) x = sprintf('\\begin{knitrout}%s\n%s\n\\end{knitrout}', size, x)
  if (options$split) {
    name = fig_path('.tex', options, NULL)
    if (!file.exists(dirname(name)))
      dir.create(dirname(name))
    write_utf8(x, name)
    sprintf('\\input{%s}', name)
  } else x
}

# rm empty kframe and verbatim environments
.rm.empty.envir = function(x) {
  x = gsub('\\\\begin\\{(kframe)\\}\\s*\\\\end\\{\\1\\}', '', x)
  gsub('\\\\end\\{(verbatim|alltt)\\}\\s*\\\\begin\\{\\1\\}[\n]?', '', x)
}

# inline hook for tex
.inline.hook.tex = function(x) {
  if (is.numeric(x)) {
    x = format_sci(x, 'latex')
    i = grep('[}]', x)
    x[i] = sprintf('\\ensuremath{%s}', x[i])
    # why \text{}: https://github.com/yihui/knitr/issues/348
    if (getOption('OutDec') != '.') x = sprintf('\\text{%s}', x)
  }
  .inline.hook(x)
}

.verb.hook = function(x)
  one_string(c('\\begin{verbatim}', sub('\n$', '', x), '\\end{verbatim}', ''))
.color.block = function(color1 = '', color2 = '') {
  function(x, options) {
    x = gsub('\n*$', '', x)
    x = escape_latex(x, newlines = TRUE, spaces = TRUE)
    # babel might have problems with "; see http://stackoverflow.com/q/18125539/559676
    x = gsub('"', '"{}', x)
    sprintf('\n\n{\\ttfamily\\noindent%s%s%s}', color1, x, color2)
  }
}

#' Set output hooks for different output formats
#'
#' These functions set built-in output hooks for LaTeX, HTML, Markdown,
#' reStructuredText, AsciiDoc and Textile.
#'
#' There are three variants of markdown documents: ordinary markdown
#' (\code{render_markdown(strict = TRUE)}), extended markdown (e.g. GitHub
#' Flavored Markdown and pandoc; \code{render_markdown(strict = FALSE)}), and
#' Jekyll (a blogging system on GitHub; \code{render_jekyll()}). For LaTeX
#' output, there are three variants as well: \pkg{knitr}'s default style
#' (\code{render_latex()}; use the LaTeX \pkg{framed} package), Sweave style
#' (\code{render_sweave()}; use \file{Sweave.sty}) and listings style
#' (\code{render_listings()}; use LaTeX \pkg{listings} package). Default HTML
#' output hooks are set by \code{render_html()}; \code{render_rst()} and
#' \code{render_asciidoc()} are for reStructuredText and AsciiDoc respectively.
#'
#' These functions can be used before \code{knit()} or in the first chunk of the
#' input document (ideally this chunk has options \code{include = FALSE} and
#' \code{cache = FALSE}) so that all the following chunks will be formatted as
#' expected.
#'
#' You can use \code{\link{knit_hooks}} to further customize output hooks; see
#' references.
#' @rdname output_hooks
#' @return \code{NULL}; corresponding hooks are set as a side effect
#' @export
#' @references See output hooks in \url{https://yihui.org/knitr/hooks/}.
#'
#'   Jekyll and Liquid:
#'   \url{https://github.com/jekyll/jekyll/wiki/Liquid-Extensions};
#'   prettify.js: \url{http://code.google.com/p/google-code-prettify/}
render_latex = function() {
  opts_chunk$set(out.width = '\\maxwidth', dev = 'pdf')
  opts_knit$set(out.format = 'latex')
  h = opts_knit$get('header')
  if (!nzchar(h['framed'])) set_header(framed = .header.framed)
  if (!nzchar(h['highlight'])) set_header(highlight = .header.hi.tex)
  knit_hooks$set(
    source = function(x, options) {
      x = hilight_source(x, 'latex', options)
      if (options$highlight) {
        if (options$engine == 'R' || x[1] != '\\noindent') {
          one_string(c('\\begin{alltt}', x, '\\end{alltt}', ''))
        } else {
          if ((n <- length(x)) > 4) x[n - 2] = sub('\\\\\\\\$', '', x[n - 2])
          one_string(c(x, ''))
        }
      } else .verb.hook(x)
    },
    output = function(x, options) {
      if (output_asis(x, options)) {
        paste0('\\end{kframe}', x, '\\begin{kframe}')
      } else .verb.hook(x)
    },
    warning = .color.block('\\color{warningcolor}{', '}'),
    message = .color.block('\\itshape\\color{messagecolor}{', '}'),
    error = .color.block('\\bfseries\\color{errorcolor}{', '}'),
    inline = .inline.hook.tex, chunk = .chunk.hook.tex,
    plot = function(x, options) {
      # escape plot environments from kframe
      paste0('\\end{kframe}', hook_plot_tex(x, options), '\n\\begin{kframe}')
    }
  )
}
#' @rdname output_hooks
#' @export
render_sweave = function() {
  opts_chunk$set(highlight = FALSE, comment = NA, prompt = TRUE) # mimic Sweave settings
  opts_knit$set(out.format = 'sweave')
  test_latex_pkg('Sweave', file.path(R.home('share'), 'texmf', 'tex', 'latex', 'Sweave.sty'))
  set_header(framed = '', highlight = '\\usepackage{Sweave}')
  # wrap source code in the Sinput environment, output in Soutput
  hook.i = function(x, options)
    one_string(c('\\begin{Sinput}', hilight_source(x, 'sweave', options), '\\end{Sinput}', ''))
  hook.s = function(x, options) {
    if (output_asis(x, options)) return(x)
    paste0('\\begin{Soutput}\n', x, '\\end{Soutput}\n')
  }
  hook.c = function(x, options) {
    if (output_asis(x, options)) return(x)
    paste0('\\begin{Schunk}\n', x, '\\end{Schunk}')
  }
  knit_hooks$set(source = hook.i, output = hook.s, warning = hook.s,
                 message = hook.s, error = hook.s, inline = .inline.hook.tex,
                 plot = hook_plot_tex, chunk = hook.c)
}
#' @rdname output_hooks
#' @export
render_listings = function() {
  render_sweave()
  opts_chunk$set(prompt = FALSE)
  opts_knit$set(out.format = 'listings')
  test_latex_pkg('Sweavel', system.file('misc', 'Sweavel.sty', package = 'knitr'))
  set_header(framed = '', highlight = '\\usepackage{Sweavel}')
  invisible(NULL)
}

# may add textile, and many other markup languages

#' Some potentially useful document hooks
#'
#' A document hook is a function to post-process the output document.
#'
#' \code{hook_movecode()} is a document hook to move code chunks out of LaTeX
#' floating environments like \samp{figure} and \samp{table} when the chunks
#' were actually written inside the floats. This function is primarily designed
#' for LyX: we often insert code chunks into floats to generate figures or
#' tables, but in the final output we do not want the code to float with the
#' environments, so we use regular expressions to find out the floating
#' environments, extract the code chunks and move them out. To disable this
#' behavior, use a comment \code{\% knitr_do_not_move} in the floating
#' environment.
#' @rdname hook_document
#' @param x A character string (the whole output document).
#' @return The post-processed document as a character string.
#' @note These functions are hackish. Also note \code{hook_movecode()} assumes
#'   you to use the default output hooks for LaTeX (not Sweave or listings), and
#'   every figure/table environment must have a label.
#' @export
#' @references \url{https://yihui.org/knitr/hooks/}
#' @examples \dontrun{knit_hooks$set(document = hook_movecode)}
#' # see example 103 at https://github.com/yihui/knitr-examples
hook_movecode = function(x) {
  x = split_lines(x)
  res = split(x, cumsum(grepl('^\\\\(begin|end)\\{figure\\}', x)))
  x = split_lines(unlist(lapply(res, function(p) {
    if (length(p) <= 4 || !grepl('^\\\\begin\\{figure\\}', p[1]) ||
          length(grep('% knitr_do_not_move', p)) ||
          !any(grepl('\\\\begin\\{(alltt|kframe)\\}', p))) return(p)
    idx = c(1, grep('\\\\includegraphics', p))
    if (length(idx) <= 1) return(p) # no graphics
    if (length(i <- grep('\\{\\\\centering.*\\\\includegraphics', p))) {
      idx = c(idx, i - 1, j2 <- i + 1)
      for (j in j2) {
        while (p[j] != '}') idx = c(idx, j <- j + 1) # find } for {\\centering
      }
    }
    if (length(i <- grep('\\\\hfill\\{\\}.*\\\\includegraphics', p)))
      idx = c(idx, i - 1, i + 1)
    if (length(i <- grep('\\\\includegraphics.*\\\\hfill\\{\\}', p)))
      idx = c(idx, i - 1, i + 1)
    idx = sort(c(idx, seq(grep('\\\\caption', p), grep('\\\\label', p))))
    idx = unique(idx)
    p = one_string(c(p[-idx], p[idx]))
    gsub('\\\\end\\{(kframe)\\}\\s*\\\\begin\\{\\1\\}', '', p)
  }), use.names = FALSE))

  res = split(x, cumsum(grepl('^\\\\(begin|end)\\{table\\}', x)))
  res = one_string(unlist(lapply(res, function(p) {
    if (length(p) <= 4 || !grepl('^\\\\begin\\{table\\}', p[1]) ||
          length(grep('% knitr_do_not_move', p)) ||
          !any(grepl('\\\\begin\\{(alltt|kframe)\\}', p))) return(p)
    if (!any(grepl('\\\\label\\{.*\\}', p))) return(p)
    idx = c(1, seq(grep('\\\\caption', p), grep('\\\\label', p)))
    i0 = grep('\\\\begin\\{tabular\\}', p); i1 = grep('\\\\end\\{tabular\\}', p)
    for (i in seq_along(i0)) idx = c(idx, i0[i]:i1[i])
    idx = sort(idx)
    p = one_string(c(p[-idx], p[idx]))
    gsub('\\\\end\\{(kframe)\\}\\s*\\\\begin\\{\\1\\}', '', p)
  }), use.names = FALSE))
  .rm.empty.envir(res)
}

#' @include themes.R
#' @include highlight.R

# doc is the output of processed document
insert_header = function(doc) {
  if (is.null(b <- knit_patterns$get('header.begin'))) return(doc)

  if (out_format('html'))
    return(insert_header_html(doc, b))
  if (out_format(c('latex', 'listings', 'sweave')))
    return(insert_header_latex(doc, b))
  doc
}

# Makes latex header with macros required for highlighting, tikz and framed
make_header_latex = function() {
  h = one_string(c(
    sprintf('\\usepackage[%s]{graphicx}\\usepackage[%s]{color}',
            opts_knit$get('latex.options.graphicx') %n% '',
            opts_knit$get('latex.options.color') %n% ''),
    .header.maxwidth, opts_knit$get('header'),
    if (getOption('OutDec') != '.') '\\usepackage{amsmath}',
    if (out_format('latex')) '\\usepackage{alltt}'
  ))
  if (opts_knit$get('self.contained')) h else {
    write_utf8(h, 'knitr.sty')
    '\\usepackage{knitr}'
  }
}

insert_header_latex = function(doc, b) {
  i = grep(b, doc)
  if (length(i) >= 1L) {
    # it is safer to add usepackage{upquote} before begin{document} than after
    # documentclass{article} because it must appear after usepackage{fontenc};
    # see this weird problem: http://stackoverflow.com/q/12448507/559676
    if (!out_format('listings') && length(j <- grep(p <- '(\\s*)(\\\\begin\\{document\\})', doc)[1L])) {
      doc[j] = sub(p, '\n\\\\IfFileExists{upquote.sty}{\\\\usepackage{upquote}}{}\n\\2', doc[j])
    }
    i = i[1L]; l = stringr::str_locate(doc[i], b)
    tmp = stringr::str_sub(doc[i], l[, 1], l[, 2])
    stringr::str_sub(doc[i], l[,1], l[,2]) = paste0(tmp, make_header_latex())
  } else if (parent_mode() && !child_mode()) {
    # in parent mode, we fill doc to be a complete document
    doc[1L] = one_string(c(
      getOption('tikzDocumentDeclaration'), make_header_latex(),
      .knitEnv$tikzPackages, '\\begin{document}', doc[1L]
    ))
    doc[length(doc)] = one_string(
      c(doc[length(doc)], .knitEnv$bibliography, '\\end{document}')
    )
  }
  doc
}

make_header_html = function() {
  h = opts_knit$get('header')
  h = h[setdiff(names(h), c('tikz', 'framed'))]
  if (opts_knit$get('self.contained')) one_string(c(
    '<style type="text/css">', h[['highlight']], '</style>',
    unlist(h[setdiff(names(h), 'highlight')])
  )) else {
    write_utf8(h, 'knitr.css')
    '<link rel="stylesheet" href="knitr.css" type="text/css" />'
  }
}

insert_header_html = function(doc, b) {
  i = grep(b, doc)
  if (length(i) == 1L) {
    l = stringr::str_locate(doc[i], b)
    tmp = stringr::str_sub(doc[i], l[, 1], l[, 2])
    stringr::str_sub(doc[i], l[,1], l[,2]) = paste0(tmp, '\n', make_header_html())
  }
  doc
}

#' Set the header information
#'
#' Some output documents may need appropriate header information. For example,
#' for LaTeX output, we need to write \samp{\\usepackage{tikz}} into the
#' preamble if we use tikz graphics; this function sets the header information
#' to be written into the output.
#'
#' By default, \pkg{knitr} will set up the header automatically. For example, if
#' the tikz device is used, \pkg{knitr} will add \samp{\\usepackage{tikz}} to
#' the LaTeX preamble, and this is done by setting the header component
#' \code{tikz} to be a character string: \code{set_header(tikz =
#' '\\usepackage{tikz}')}. Similary, when we highlight R code using the
#' \pkg{highlight} package (i.e. the chunk option \code{highlight = TRUE}),
#' \pkg{knitr} will set the \code{highlight} component of the header vector
#' automatically; if the output type is HTML, this component will be different
#' -- instead of LaTeX commands, it contains CSS definitions.
#'
#' For power users, all the components can be modified to adapt to a customized
#' type of output. For instance, we can change \code{highlight} to LaTeX
#' definitions of the \pkg{listings} package (and modify the output hooks
#' accordingly), so we can decorate R code using the \pkg{listings} package.
#' @param ... Header components; currently possible components are
#'   \code{highlight}, \code{tikz} and \code{framed}, which contain the
#'   necessary commands to be used in the HTML header or LaTeX preamble. Note that
#'   HTML output does not use the \code{tikz} and \code{framed} components, since
#'   they do not make sense in the context of HTML.
#' @return The header vector in \code{opts_knit} is set.
#' @export
#' @examples set_header(tikz = '\\usepackage{tikz}')
#' opts_knit$get('header')
set_header = function(...) {
  opts_knit$set(header = merge_list(opts_knit$get('header'), c(...)))
}

.default.sty = inst_dir('themes', 'default.css')
# header for Latex Syntax Highlighting
.header.hi.tex = theme_to_header_latex(.default.sty)$highlight
.knitr.sty = inst_dir('misc', 'knitr.sty')
.header.framed = one_string(read_utf8(.knitr.sty))
# CSS for html syntax highlighting
.header.hi.html = theme_to_header_html(.default.sty)$highlight
rm(list = c('.default.sty', '.knitr.sty')) # do not need them any more

.header.sweave.cmd =
'\\newcommand{\\SweaveOpts}[1]{}  % do not interfere with LaTeX
\\newcommand{\\SweaveInput}[1]{} % because they are not real TeX commands
\\newcommand{\\Sexpr}[1]{}       % will only be parsed by R
'

.header.maxwidth =
'% maxwidth is the original width if it is less than linewidth
% otherwise use linewidth (to make sure the graphics do not exceed the margin)
\\makeatletter
\\def\\maxwidth{ %
  \\ifdim\\Gin@nat@width>\\linewidth
    \\linewidth
  \\else
    \\Gin@nat@width
  \\fi
}
\\makeatother
'

library(testit)

render_jekyll('prettify')
options(prompt = '> ', continue = '+ ')

# see http://stackoverflow.com/q/18992260/559676 for the bug
assert('the source hook pastes source code into one character string', {
  (knit_hooks$get('source')(c('1+1', '2-2'), opts_chunk$get()) %==%
    "\n\n<pre><code class=\"prettyprint \">1+1\n2-2</code></pre>\n\n")
  (knit_hooks$get('source')(c('1+1+\n1', '2-2'), opts_chunk$merge(list(prompt = TRUE))) %==%
    "\n\n<pre><code class=\"prettyprint \">&gt; 1+1+\n+ 1\n&gt; 2-2</code></pre>\n\n")
})

render_markdown()

img_output = function(path, opts = list()) {
  opts = opts_chunk$merge(opts)
  sew(knit_print(include_graphics(path, error = FALSE)), opts)
}

assert('include_graphics() includes custom images correctly', {
  (img_output('a.png') %==% '![](a.png)')
  (img_output(c('a.png', 'b.png'), list(fig.show = 'hold')) %==% '![](a.png)![](b.png)')
  (img_output('a.png', list(fig.cap = 'foo bar')) %==% '![foo bar](a.png)')
  (img_output('a.png', list(out.width = '50%')) %==% '<img src="a.png" alt="" width="50%" />')
  (img_output('a.pdf', list(out.width = '300px')) %==% '<embed src="a.pdf" width="300px" type="application/pdf" />')
})

hook_src = knit_hooks$get("source")
options_ = list(engine = "r", prompt = FALSE, highlight = TRUE)

assert('Length of fences are satisfied', {
  (hook_src("", options_) %==% "\n\n``` r\n\n```\n\n")
  (hook_src("```", options_) %==% "\n\n```` r\n```\n````\n\n")
})

assert('Attributes for source can be specified class.source and attr.source', {
  (hook_src("1", c(options_, class.source = "a b")) %==% "\n\n```{.r .a .b}\n1\n```\n\n")
  (hook_src("1", c(options_, attr.source = ".a .b")) %==% "\n\n```{.r .a .b}\n1\n```\n\n")
  (hook_src("1", c(options_, class.source = "a", attr.source = "b='1'")) %==%
    "\n\n```{.r .a b='1'}\n1\n```\n\n")
  (hook_src("1", c(options_, attr.source = ".a b='1'")) %==%
    "\n\n```{.r .a b='1'}\n1\n```\n\n")
})

assert('class.source and attr.source works also with collapse = TRUE', {
  hook_chunk = hooks_markdown()$chunk
  (hook_chunk("```{.r .a b=1}\n1\n```", c(options_, collapse = TRUE)) %==%
      "```{.r .a b=1}\n1\n```")
  (hook_chunk("```{.r .a b=1}\n1\n```\n```{.r .a b=1}\n1\n```",
              c(options_, collapse = TRUE)) %==%
      "```{.r .a b=1}\n1\n1\n```")
})

hook_out = knit_hooks$get("output")

assert('Attributes for source can be specified class.source and attr.source', {
  (hook_out("1\n", c(options_, class.output = "a b")) %==%
    "\n\n```{.a .b}\n1\n```\n\n")
  (hook_out("1\n", c(options_, attr.output = ".a .b")) %==%
    "\n\n```{.a .b}\n1\n```\n\n")
  (hook_out("1\n", c(options_, class.output = "a", attr.output = "b='1'")) %==%
    "\n\n```{.a b='1'}\n1\n```\n\n")
  (hook_out("1\n", c(options_, attr.output = ".a b='1'")) %==%
    "\n\n```{.a b='1'}\n1\n```\n\n")
})

hook_chunk = knit_hooks$get("chunk")

assert('Chunks are enclosed by fenced divs when needed.', {
  (hook_chunk('', list(class.chunk=NULL)) %==% '')
  (hook_chunk('', list(class.chunk="")) %==% '::: \n\n:::')
  (hook_chunk('', list(class.chunk="foo")) %==% '::: foo\n\n:::')
  (hook_chunk(':::', list(class.chunk="foo")) %==% ':::: foo\n:::\n::::')
})

knit_hooks$restore()


x = "1.png"
w = h = 1
ex = "style='margin: 0;'"
cap = "foo"
opt = function(
  w = NULL, h = NULL, ex = NULL, cap = NULL, show = 'asis',
  fig.align = 'default', ...
) {
  list(
    out.width = w, out.height = h, out.extra = ex,
    fig.cap = cap, fig.show = show, fig.align = fig.align, ...
  )
}

assert("Include a plot by pandoc md", {
  (hook_plot_md_pandoc(x, opt()) %==% "![](1.png)")
  (hook_plot_md_pandoc(x, opt(w = w)) %==% sprintf("![](1.png){width=%s}", w))
  (hook_plot_md_pandoc(x, opt(h = h)) %==% sprintf("![](1.png){height=%s}", h))
  (hook_plot_md_pandoc(x, opt(cap = cap)) %==% sprintf("![%s](1.png)", cap))
  (hook_plot_md_pandoc(x, opt(ex = ex)) %==% sprintf("![](1.png){%s}", ex))
  (hook_plot_md_pandoc(x, opt(w = w, cap = cap, ex = ex)) %==%
    sprintf("![%s](1.png){width=%s %s}", cap, w, ex))
})

assert('empty alt text is preserved and NA alt is discarded', {
  (hook_plot_md(x, opts_chunk$merge(list(fig.alt = ''))) %==% '<img src="1.png" alt=""  />')
  (hook_plot_md(x, opts_chunk$merge(list(fig.alt = NA, out.width = '100'))) %==% '<img src="1.png" width="100" />')
})

assert("fig.alt does not break office document", {
  old = opts_knit$get('rmarkdown.pandoc.to')
  opts_knit$set(rmarkdown.pandoc.to = "docx")
  (suppressWarnings(hook_plot_md(x, opt())) %==% "![](1.png)")
  (suppressWarnings(hook_plot_md(x, opt(fig.alt = "bar"))) %==% "![](1.png)")
  opts_knit$set('rmarkdown.pandoc.to' = old)
})

#' Knit a document
#'
#' This function takes an input file, extracts the R code in it according to a
#' list of patterns, evaluates the code and writes the output in another file.
#' It can also tangle R source code from the input document (\code{purl()} is a
#' wrapper to \code{knit(..., tangle = TRUE)}). The \code{knitr.purl.inline}
#' option can be used to also tangle the code of inline expressions (disabled
#' by default).
#'
#' For most of the time, it is not necessary to set any options outside the
#' input document; in other words, a single call like
#' \code{knit('my_input.Rnw')} is usually enough. This function will try to
#' determine many internal settings automatically. For the sake of
#' reproducibility, it is better practice to include the options inside the
#' input document (to be self-contained), instead of setting them before
#' knitting the document.
#'
#' First the filename of the output document is determined in this way:
#' \file{foo.Rnw} generates \file{foo.tex}, and other filename extensions like
#' \file{.Rtex}, \file{.Rhtml} (\file{.Rhtm}) and \file{.Rmd}
#' (\file{.Rmarkdown}) will generate \file{.tex}, \file{.html} and \file{.md}
#' respectively. For other types of files, if the filename contains
#' \samp{_knit_}, this part will be removed in the output file, e.g.,
#' \file{foo_knit_.html} creates the output \file{foo.html}; if \samp{_knit_} is
#' not found in the filename, \file{foo.ext} will produce \file{foo.txt} if
#' \code{ext} is not \code{txt}, otherwise the output is \file{foo-out.txt}. If
#' \code{tangle = TRUE}, \file{foo.ext} generates an R script \file{foo.R}.
#'
#' We need a set of syntax to identify special markups for R code chunks and R
#' options, etc. The syntax is defined in a pattern list. All built-in pattern
#' lists can be found in \code{all_patterns} (call it \code{apat}). First
#' \pkg{knitr} will try to decide the pattern list based on the filename
#' extension of the input document, e.g. \samp{Rnw} files use the list
#' \code{apat$rnw}, \samp{tex} uses the list \code{apat$tex}, \samp{brew} uses
#' \code{apat$brew} and HTML files use \code{apat$html}; for unkown extensions,
#' the content of the input document is matched against all pattern lists to
#' automatically determine which pattern list is being used. You can also manually
#' set the pattern list using the \code{\link{knit_patterns}} object or the
#' \code{\link{pat_rnw}} series functions in advance and \pkg{knitr} will
#' respect the setting.
#'
#' According to the output format (\code{opts_knit$get('out.format')}), a set of
#' output hooks will be set to mark up results from R (see
#' \code{\link{render_latex}}). The output format can be LaTeX, Sweave and HTML,
#' etc. The output hooks decide how to mark up the results (you can customize
#' the hooks).
#'
#' The name \code{knit} comes from its counterpart \samp{weave} (as in Sweave),
#' and the name \code{purl} (as \samp{tangle} in Stangle) comes from a knitting
#' method `knit one, purl one'.
#'
#' If the input document has child documents, they will also be compiled
#' recursively. See \code{\link{knit_child}}.
#'
#' See the package website and manuals in the references to know more about
#' \pkg{knitr}, including the full documentation of chunk options and demos,
#' etc.
#' @param input Path to the input file.
#' @param output Path to the output file for \code{knit()}. If \code{NULL}, this
#'   function will try to guess a default, which will be under the current working
#'   directory.
#' @param tangle Boolean; whether to tangle the R code from the input file (like
#'   \code{\link[utils]{Stangle}}).
#' @param text A character vector. This is an alternative way to provide the input
#'   file.
#' @param quiet Boolean; suppress the progress bar and messages?
#' @param envir Environment in which code chunks are to be evaluated,
#'   for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
#'   \code{\link{globalenv}()}).
#' @param encoding Encoding of the input file; always assumed to be UTF-8 (i.e.,
#'   this argument is effectively ignored).
#' @return The compiled document is written into the output file, and the path
#'   of the output file is returned. If the \code{text} argument is not
#'   \code{NULL}, the compiled output is returned as a character vector. In
#'   other words, if you provide a file input, you get an output filename; if
#'   you provide a character vector input, you get a character vector output.
#' @note The working directory when evaluating R code chunks is the directory of
#'   the input document by default, so if the R code involves external files
#'   (like \code{read.table()}), it is better to put these files under the same
#'   directory of the input document so that we can use relative paths. However,
#'   it is possible to change this directory with the package option
#'   \code{\link{opts_knit}$set(root.dir = ...)} so all paths in code chunks are
#'   relative to this \code{root.dir}. It is not recommended to change the
#'   working directory via \code{\link{setwd}()} in a code chunk, because it may
#'   lead to terrible consequences (e.g. figure and cache files may be written to
#'   wrong places). If you do use \code{setwd()}, please note that \pkg{knitr}
#'   will always restore the working directory to the original one. Whenever you
#'   feel confused, print \code{getwd()} in a code chunk to see what the working
#'   directory really is.
#'
#'   If the \code{output} argument is a file path, it is strongly recommended to
#'   be in the current working directory (e.g. \file{foo.tex} instead of
#'   \file{somewhere/foo.tex}), especially when the output has external
#'   dependencies such as figure files. If you want to write the output to a
#'   different directory, it is recommended to set the working directory to
#'   that directory before you knit a document. For example, if the source
#'   document is \file{foo.Rmd} and the expected output is \file{out/foo.md},
#'   you can write \code{setwd('out/'); knit('../foo.Rmd')} instead of
#'   \code{knit('foo.Rmd', 'out/foo.md')}.
#'
#'   N.B. There is no guarantee that the R script generated by \code{purl()} can
#'   reproduce the computation done in \code{knit()}. The \code{knit()} process
#'   can be fairly complicated (special values for chunk options, custom chunk
#'   hooks, computing engines besides R, and the \code{envir} argument, etc). If
#'   you want to reproduce the computation in a report generated by
#'   \code{knit()}, be sure to use \code{knit()}, instead of merely executing
#'   the R script generated by \code{purl()}. This seems to be obvious, but some
#'   people \href{http://bit.ly/SnLi6h}{just do not get it}.
#' @export
#' @references Package homepage: \url{https://yihui.org/knitr/}. The \pkg{knitr}
#'   \href{https://yihui.org/knitr/demo/manual/}{main manual}: and
#'   \href{https://yihui.org/knitr/demo/graphics/}{graphics manual}.
#'
#'   See \code{citation('knitr')} for the citation information.
#' @examples library(knitr)
#' (f = system.file('examples', 'knitr-minimal.Rnw', package = 'knitr'))
#' knit(f)  # compile to tex
#'
#' purl(f)  # tangle R code
#' purl(f, documentation = 0)  # extract R code only
#' purl(f, documentation = 2)  # also include documentation
#'
#' unlink(c('knitr-minimal.tex', 'knitr-minimal.R', 'figure'), recursive = TRUE)
knit = function(
  input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE,
  envir = parent.frame(), encoding = 'UTF-8'
) {

  in.file = !missing(input) && is.character(input)  # is input provided?
  oconc = knit_concord$get(); on.exit(knit_concord$set(oconc), add = TRUE)

  if (child_mode()) {
    setwd(opts_knit$get('output.dir')) # always restore original working dir
    # in child mode, input path needs to be adjusted
    if (in.file && !is_abs_path(input)) {
      input = paste0(opts_knit$get('child.path'), input)
      input = file.path(input_dir(TRUE), input)
    }
    # respect the quiet argument in child mode (#741)
    optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
    opts_knit$set(progress = opts_knit$get('progress') && !quiet)
    quiet = !opts_knit$get('progress')
  } else {
    opts_knit$set(output.dir = getwd()) # record working directory in 1st run
    knit_log$restore()
    on.exit(chunk_counter(reset = TRUE), add = TRUE) # restore counter
    adjust_opts_knit()
    # turn off fancy quotes, use a null pdf device to record graphics
    oopts = options(
      useFancyQuotes = FALSE, device = pdf_null, knitr.in.progress = TRUE
    )
    on.exit(options(oopts), add = TRUE)
    # restore chunk options after parent exits
    optc = opts_chunk$get(); on.exit(opts_chunk$restore(optc), add = TRUE)
    ocode = knit_code$get(); on.exit(knit_code$restore(ocode), add = TRUE)
    on.exit(opts_current$restore(), add = TRUE)
    optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
    opts_knit$set(tangle = tangle, progress = opts_knit$get('progress') && !quiet)
  }
  # store the evaluation environment and restore on exit
  oenvir = .knitEnv$knit_global; .knitEnv$knit_global = envir
  on.exit({.knitEnv$knit_global = oenvir}, add = TRUE)

  ext = 'unknown'
  if (in.file) {
    input.dir = .knitEnv$input.dir; on.exit({.knitEnv$input.dir = input.dir}, add = TRUE)
    .knitEnv$input.dir = dirname(input) # record input dir
    ext = tolower(file_ext(input))
    if ((is.null(output) || is.na(output)) && !child_mode())
      output = basename(auto_out_name(input, ext))
    # do not run purl() when the output is newer than input (the output might
    # have been generated by hook_purl)
    if (is.character(output) && !child_mode()) {
      out.purl = with_ext(input, 'R')
      if (xfun::same_path(output, out.purl) && tangle && file_test('-nt', out.purl, input))
        return(out.purl)
      otangle = .knitEnv$tangle.file  # the tangled R script
      .knitEnv$tangle.file = normalizePath(out.purl, mustWork = FALSE)
      .knitEnv$tangle.start = FALSE
      on.exit({.knitEnv$tangle.file = otangle; .knitEnv$tangle.start = NULL}, add = TRUE)
    }
    if (is.null(getOption('tikzMetricsDictionary'))) {
      options(tikzMetricsDictionary = tikz_dict(input)) # cache tikz dictionary
      on.exit(options(tikzMetricsDictionary = NULL), add = TRUE)
    }
    knit_concord$set(infile = input, outfile = output)
  }

  if (is.null(text)) {
    text = readLines(input, encoding = 'UTF-8', warn = FALSE)
    if (!is_utf8(text)) warning(
      'The file "', input, '" must be encoded in UTF-8. Please see ',
      'https://yihui.org/en/2018/11/biggest-regret-knitr/ for more info.'
    )
  } else text = split_lines(text) # make sure each element is one line
  if (!length(text)) {
    if (is.character(output)) file.create(output)
    return(output) # a trivial case: create an empty file and exit
  }

  apat = all_patterns; opat = knit_patterns$get()
  on.exit(knit_patterns$restore(opat), add = TRUE)
  if (length(opat) == 0 || all(vapply(opat, is.null, logical(1)))) {
    # use ext if cannot auto detect pattern
    if (is.null(pattern <- detect_pattern(text, ext))) {
      # nothing to be executed; just return original input
      if (is.null(output)) {
        return(if (tangle) '' else one_string(text))
      } else {
        write_utf8(if (tangle) '' else text, output)
        return(output)
      }
    }
    if (!(pattern %in% names(apat))) stop(
      "a pattern list cannot be automatically found for the file extension '",
      ext, "' in built-in pattern lists; ",
      'see ?knit_patterns on how to set up customized patterns'
    )
    set_pattern(pattern)
    if (pattern == 'rnw' && length(sweave_lines <- which_sweave(text)) > 0)
      remind_sweave(if (in.file) input, sweave_lines)
    opts_knit$set(out.format = switch(
      pattern, rnw = 'latex', tex = 'latex', html = 'html', md = 'markdown',
      rst = 'rst', brew = 'brew', asciidoc = 'asciidoc', textile = 'textile'
    ))
  }

  if (is.null(out_format())) auto_format(ext)

  params = NULL  # the params field from YAML
  if (out_format('markdown')) {
    if (child_mode()) {
      # in child mode, strip off the YAML metadata in Markdown if exists
      if (grepl('^---\\s*$', text[1])) {
        i = grep('^---\\s*$', text)
        if (length(i) >= 2) text = text[-(1:i[2])]
      }
    } else {
      params = knit_params(text)
      params = if (length(params))
        c('params <-', capture.output(dput(flatten_params(params), '')), '')
      .knitEnv$tangle.params = params  # for hook_purl()
    }
  }
  # change output hooks only if they are not set beforehand
  if (identical(knit_hooks$get(names(.default.hooks)), .default.hooks) && !child_mode()) {
    getFromNamespace(paste('render', out_format(), sep = '_'), 'knitr')()
    on.exit(knit_hooks$set(.default.hooks), add = TRUE)
  }

  progress = opts_knit$get('progress')
  if (in.file && !quiet) message(ifelse(progress, '\n\n', ''), 'processing file: ', input)
  res = process_file(text, output)
  res = one_string(knit_hooks$get('document')(res))
  if (tangle) res = c(params, res)
  if (!is.null(output)) write_utf8(res, output)
  if (!child_mode()) {
    dep_list$restore()  # empty dependency list
    .knitEnv$labels = NULL
  }

  if (in.file && is.character(output) && file.exists(output)) {
    concord_gen(input, output)
    if (!quiet) message('output file: ', output, ifelse(progress, '\n', ''))
  }

  output %n% res
}
#' @rdname knit
#' @param documentation An integer specifying the level of documentation to add to
#'   the tangled script. \code{0} means to output pure code, discarding all text chunks);
#'   \code{1} (the default) means to add the chunk headers to the code; \code{2} means to
#'   add all text chunks to code as roxygen comments.
#' @param ... arguments passed to \code{\link{knit}()} from \code{purl()}
#' @export
purl = function(..., documentation = 1L) {
  doc = opts_knit$get('documentation'); on.exit(opts_knit$set(documentation = doc))
  opts_knit$set(documentation = documentation)
  knit(..., tangle = TRUE)
}

process_file = function(text, output) {
  groups = split_file(lines = text)
  n = length(groups); res = character(n)
  tangle = opts_knit$get('tangle')

  # when in R CMD check, turn off the progress bar (R-exts said the progress bar
  # was not appropriate for non-interactive mode, and I don't want to argue)
  progress = opts_knit$get('progress') && !is_R_CMD_check()
  if (progress) {
    pb = txtProgressBar(0, n, char = '.', style = 3)
    on.exit(close(pb), add = TRUE)
  }
  wd = getwd()
  for (i in 1:n) {
    if (!is.null(.knitEnv$terminate)) {
      res[i] = one_string(.knitEnv$terminate)
      knit_exit(NULL)
      break  # must have called knit_exit(), so exit early
    }
    if (progress) {
      setTxtProgressBar(pb, i)
      if (!tangle) cat('\n')  # under tangle mode, only show one progress bar
      flush.console()
    }
    group = groups[[i]]
    res[i] = withCallingHandlers(
      if (tangle) process_tangle(group) else process_group(group),
      error = function(e) {
        setwd(wd)
        cat(res, sep = '\n', file = output %n% '')
        message(
          'Quitting from lines ', paste(current_lines(i), collapse = '-'),
          ' (', knit_concord$get('infile'), ') '
        )
      }
    )
  }

  if (!tangle) res = insert_header(res)  # insert header
  # output line numbers
  if (concord_mode()) knit_concord$set(outlines = line_count(res))
  print_knitlog()
  if (tangle) res = strip_white(res)

  res
}

auto_out_name = function(input, ext = tolower(file_ext(input))) {
  base = sans_ext(input)
  name = if (opts_knit$get('tangle')) c(base, '.R') else
    if (ext %in% c('rnw', 'snw')) c(base, '.tex') else
      if (ext %in% c('rmd', 'rmarkdown', 'rhtml', 'rhtm', 'rtex', 'stex', 'rrst', 'rtextile'))
        c(base, '.', substring(ext, 2L)) else
          if (grepl('_knit_', input)) sub('_knit_', '', input) else
            if (ext != 'txt') c(base, '.txt') else c(base, '-out.', ext)
  paste(name, collapse = '')
}

# determine output format based on file extension
ext2fmt = c(
  rnw = 'latex', snw = 'latex', tex = 'latex', rtex = 'latex', stex = 'latex',
  htm = 'html', html = 'html', rhtml = 'html', rhtm = 'html',
  md = 'markdown', markdown = 'markdown', rmd = 'markdown', rmarkdown = 'markdown',
  brew = 'brew', rst = 'rst', rrst = 'rst'
)

auto_format = function(ext) {
  fmt = ext2fmt[ext]
  if (is.na(fmt)) fmt = {
    warning('cannot automatically decide the output format')
    'unknown'
  }
  opts_knit$set(out.format = fmt)
  invisible(fmt)
}

#' Knit a child document
#'
#' This function knits a child document and returns a character string to input
#' the result into the main document. It is designed to be used in the chunk
#' option \code{child} and serves as the alternative to the
#' \command{SweaveInput} command in Sweave.
#' @param ... Arguments passed to \code{\link{knit}}.
#' @param options A list of chunk options to be used as global options inside
#'   the child document. When one uses the \code{child}
#'   option in a parent chunk, the chunk options of the parent chunk will be
#'   passed to the \code{options} argument here.  Ignored if not a list.
#' @inheritParams knit
#' @return A character string of the content of the compiled child document is
#'   returned as a character string so it can be written back to the parent
#'   document directly.
#' @references \url{https://yihui.org/knitr/demo/child/}
#' @note This function is not supposed be called directly like
#'   \code{\link{knit}()}; instead it must be placed in a parent document to let
#'   \code{\link{knit}()} call it indirectly.
#'
#'   The path of the child document is determined relative to the parent document.
#' @export
#' @examples # you can write \Sexpr{knit_child('child-doc.Rnw')} in an Rnw file 'main.Rnw'
#' # to input results from child-doc.Rnw in main.tex
#'
#' # comment out the child doc by \Sexpr{knit_child('child-doc.Rnw', eval = FALSE)}
knit_child = function(..., options = NULL, envir = knit_global()) {
  child = child_mode()
  opts_knit$set(child = TRUE) # yes, in child mode now
  on.exit(opts_knit$set(child = child)) # restore child status
  if (is.list(options)) {
    options$label = options$child = NULL  # do not need to pass the parent label on
    if (length(options)) {
      optc = opts_chunk$get(names(options), drop = FALSE); opts_chunk$set(options)
      # if user did not touch opts_chunk$set() in child, restore the chunk option
      on.exit({
        for (i in names(options)) if (identical(options[[i]], opts_chunk$get(i)))
          opts_chunk$set(optc[i])
      }, add = TRUE)
    }
  }
  res = knit(..., tangle = opts_knit$get('tangle'), envir = envir)
  one_string(c('', res))
}

#' Exit knitting early
#'
#' Sometimes we may want to exit the knitting process early, and completely
#' ignore the rest of the document. This function provides a mechanism to
#' terminate \code{\link{knit}()}.
#' @param append A character vector to be appended to the results from
#'   \code{knit()} so far. By default, this is \samp{\end{document}} for LaTeX
#'   output, and \samp{</body></html>} for HTML output, to make the output
#'   document complete. For other types of output, it is an empty string.
#' @return Invisible \code{NULL}. An internal signal is set up (as a side
#'   effect) to notify \code{knit()} to quit as if it had reached the end of the
#'   document.
#' @export
#' @examples # see https://github.com/yihui/knitr-examples/blob/master/096-knit-exit.Rmd
knit_exit = function(append) {
  if (missing(append)) append = if (out_format(c('latex', 'sweave', 'listings')))
    '\\end{document}' else if (out_format('html')) '</body>\n</html>' else ''
  .knitEnv$terminate = append # use this terminate variable to notify knit()
  invisible()
}

knit_log = new_defaults()  # knitr log for errors, warnings and messages

#' Wrap evaluated results for output
#'
#' @param x output from \code{\link[evaluate]{evaluate}}
#' @param options List of options used to control output
#' @noRd
wrap = function(x, options = list(), ...) {
  UseMethod('wrap', x)
}

#' @export
wrap.list = function(x, options = list()) {
  if (length(x) == 0L) return(x)
  lapply(x, wrap, options)
}

# ignore unknown classes
#' @export
wrap.default = function(x, options) return()

#' @export
wrap.character = function(x, options) {
  if (options$results == 'hide') return()
  if (output_asis(x, options)) {
    if (!out_format('latex')) return(x)  # latex output still need a tweak
  } else x = comment_out(x, options$comment)
  knit_hooks$get('output')(x, options)
}

# If you provide a custom print function that returns a character object of
# class 'knit_asis', it will be written as is.
#' @export
wrap.knit_asis = function(x, options, inline = FALSE) {
  m = attr(x, 'knit_meta')
  knit_meta_add(m, if (missing(options)) '' else options$label)
  if (!missing(options)) {
    if (options$cache > 0 && isFALSE(attr(x, 'knit_cacheable'))) stop(
      "The code chunk '", options$label, "' is not cacheable; ",
      "please use the chunk option cache=FALSE on this chunk"
    )
    # store metadata in an object named of the form .hash_meta when cache=TRUE
    if (length(m) && options$cache == 3)
      assign(cache_meta_name(options$hash), m, envir = knit_global())
    if (inherits(x, 'knit_asis_htmlwidget')) {
      options$fig.cur = plot_counter()
      options = reduce_plot_opts(options)
      return(add_html_caption(options, x))
    }
  }
  x = as.character(x)
  if (!out_format('latex') || inline) return(x)
  # latex output need the \end{kframe} trick
  options$results = 'asis'
  knit_hooks$get('output')(x, options)
}

#' @export
wrap.source = function(x, options) {
  if (isFALSE(options$echo)) return()
  src = sub('\n$', '', x$src)
  if (!options$collapse && options$strip.white) src = strip_white(src)
  if (is_blank(src)) return()  # an empty chunk
  knit_hooks$get('source')(src, options)
}

msg_wrap = function(message, type, options) {
  # when the output format is LaTeX, do not wrap messages (let LaTeX deal with wrapping)
  if (!length(grep('\n', message)) && !out_format(c('latex', 'listings', 'sweave')))
    message = stringr::str_wrap(message, width = getOption('width'))
  knit_log$set(setNames(
    list(c(knit_log$get(type), paste0('Chunk ', options$label, ':\n  ', message))),
    type
  ))
  message = msg_sanitize(message, type)
  knit_hooks$get(type)(comment_out(message, options$comment), options)
}

# set options(knitr.sanitize.errors = TRUE) to hide error messages, etc
msg_sanitize = function(message, type) {
  type = match.arg(type, c('error', 'warning', 'message'))
  opt = getOption(sprintf('knitr.sanitize.%ss', type), FALSE)
  if (isTRUE(opt)) message = switch(
    type, error = 'An error occurred', warning = 'A warning was emitted',
    message = 'A message was emitted'
  ) else if (is.character(opt)) message = opt
  message
}

#' @export
wrap.warning = function(x, options) {
  call = if (is.null(x$call)) '' else {
    call = deparse(x$call)[1]
    if (call == 'eval(expr, envir, enclos)') '' else paste(' in', call)
  }
  msg_wrap(sprintf('Warning%s: %s', call, x$message), 'warning', options)
}

#' @export
wrap.message = function(x, options) {
  msg_wrap(paste(x$message, collapse = ''), 'message', options)
}

#' @export
wrap.error = function(x, options) {
  msg_wrap(as.character(x), 'error', options)
}

#' @export
wrap.recordedplot = function(x, options) {
  # figure number sequence for multiple plots
  fig.cur = plot_counter()
  options$fig.cur = fig.cur # put fig num in options
  name = fig_path('', options, number = fig.cur)
  in_base_dir(
    # automatically creates dir for plots
    if (!file_test('-d', dirname(name)))
      dir.create(dirname(name), recursive = TRUE)
  )
  # vectorize over dev, ext and dpi: save multiple versions of the plot
  files = mapply(
    save_plot, width = options$fig.width, height = options$fig.height,
    dev = options$dev, ext = options$fig.ext, dpi = options$dpi,
    MoreArgs = list(plot = x, name = name, options = options), SIMPLIFY = FALSE
  )
  opts_knit$append(plot_files = unlist(files))
  if (options$fig.show == 'hide') return('')
  in_base_dir(run_hook_plot(files[[1]], reduce_plot_opts(options)))
}

#' @export
wrap.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE) {
  if (options$fig.show == 'hide') return('')
  # remove the automatically set out.width when fig.retina is set, otherwise the
  # size of external images embedded via include_graphics() will be set to
  # fig.width * dpi in fix_options()
  if (is.numeric(r <- options$fig.retina)) {
    w1 = options$out.width
    w2 = options$fig.width * options$dpi / r
    if (length(w1) * length(w2) == 1 && is.numeric(w1) && w1 == w2)
      options['out.width'] = list(NULL)
  }
  dpi = attr(x, 'dpi') %n% options$dpi
  hook = knit_hooks$get('plot')
  paste(unlist(lapply(seq_along(x), function(i) {
    options$fig.cur = plot_counter()
    if (is.null(options[['out.width']]))
      options['out.width'] = list(raster_dpi_width(x[i], dpi))
    hook(x[i], reduce_plot_opts(options))
  })), collapse = '')
}

#' @export
wrap.html_screenshot = function(x, options = opts_chunk$get(), inline = FALSE) {
  ext = x$extension
  in_base_dir({
    i = plot_counter()
    if (is.null(f <- x$file)) {
      f = fig_path(ext, options, i)
      dir.create(dirname(f), recursive = TRUE, showWarnings = FALSE)
      writeBin(x$image, f, useBytes = TRUE)
    }
    options$fig.cur = i
    options = reduce_plot_opts(options)
    if (!is.null(x$url) && is.null(options$fig.link)) options$fig.link = x$url
    run_hook_plot(f, options)
  })
}

# record plot filenames in opts_knit$get('plot_files'), including those from R
# code and auto screenshots of HTML widgets, etc. Then run the plot hook.
run_hook_plot = function(x, options) {
  opts_knit$append(plot_files = x)
  hook = knit_hooks$get('plot')
  hook(x, options)
}

#' @export
wrap.knit_embed_url = function(x, options = opts_chunk$get(), inline = FALSE) {
  options$fig.cur = plot_counter()
  options = reduce_plot_opts(options)
  if (length(extra <- options$out.extra)) extra = paste('', extra, collapse = '')
  add_html_caption(options, sprintf(
    '<iframe src="%s" width="%s" height="%s"%s></iframe>',
    escape_html(x$url), options$out.width %n% '100%', x$height %n% '400px',
    extra %n% ''
  ))
}

add_html_caption = function(options, code) {
  cap = .img.cap(options)
  if (cap == '') return(code)
  sprintf(
    '<div class="figure"%s>\n%s\n<p class="caption">%s</p>\n</div>',
    css_text_align(options$fig.align), code, cap
  )
}

#' A custom printing function
#'
#' The S3 generic function \code{knit_print} is the default printing function in
#' \pkg{knitr}. The chunk option \code{render} uses this function by default.
#' The main purpose of this S3 generic function is to customize printing of R
#' objects in code chunks. We can fall back to the normal printing behavior by
#' setting the chunk option \code{render = normal_print}.
#'
#' Users can write custom methods based on this generic function. For example,
#' if we want to print all data frames as tables in the output, we can define a
#' method \code{knit_print.data.frame} that turns a data.frame into a table (the
#' implementation may use other R packages or functions, e.g. \pkg{xtable} or
#' \code{\link{kable}()}).
#' @param x An R object to be printed
#' @param ... Additional arguments passed to the S3 method. Currently ignored,
#'   except two optional arguments \code{options} and \code{inline}; see
#'   the references below.
#' @return The value returned from the print method should be a character vector
#'   or can be converted to a character value. You can wrap the value in
#'   \code{\link{asis_output}()} so that \pkg{knitr} writes the character value
#'   as is in the output.
#' @note It is recommended to leave a \code{...} argument in your method, to
#'   allow future changes of the \code{knit_print()} API without breaking your
#'   method.
#' @references See \code{vignette('knit_print', package = 'knitr')}.
#' @export
#' @examples library(knitr)
#' # write tables for data frames
#' knit_print.data.frame = function(x, ...) {
#'   res = paste(c('', '', kable(x, output = FALSE)), collapse = '\n')
#'   asis_output(res)
#' }
#' # register the method
#' registerS3method("knit_print", "data.frame", knit_print.data.frame)
#' # after you define and register the above method, data frames will be printed
#' # as tables in knitr, which is different with the default print() behavior
knit_print = function(x, ...) {
  if (need_screenshot(x, ...)) {
    html_screenshot(x)
  } else {
    UseMethod('knit_print')
  }
}

#" the default print method is just print()/show()
#' @export
knit_print.default = function(x, ..., inline = FALSE) {
  if (inline) x else normal_print(x)
}

#' @export
knit_print.knit_asis = function(x, ...) x

#' @rdname knit_print
#' @export
normal_print = default_handlers$value
formals(normal_print) = alist(x = , ... = )

#' Mark an R object with a special class
#'
#' This is a convenience function that assigns the input object a class named
#' \code{knit_asis}, so that \pkg{knitr} will treat it as is (the effect is the
#' same as the chunk option \code{results = 'asis'}) when it is written to the
#' output.
#'
#' This function is normally used in a custom S3 method based on the printing
#' function \code{\link{knit_print}()}.
#'
#' For the \code{cacheable} argument, you need to be careful when printing the
#' object involves non-trivial side effects, in which case it is strongly
#' recommended to use \code{cacheable = FALSE} to instruct \pkg{knitr} that this
#' object should not be cached using the chunk option \code{cache = TRUE},
#' otherwise the side effects will be lost the next time the chunk is knitted.
#' For example, printing a \pkg{shiny} input element or an HTML widget in an R
#' Markdown document may involve registering metadata about some JavaScript
#' libraries or stylesheets, and the metadata may be lost if we cache the code
#' chunk, because the code evaluation will be skipped the next time. This
#' particular issue has been solved in \pkg{knitr} after v1.13 (the metadata
#' will be saved and loaded automatically when caching is enabled), but not all
#' metadata can be saved and loaded next time and still works in the new R
#' session.
#' @param x An R object. Typically a character string, or an object which can
#'    be converted to a character string via \code{\link{as.character}()}.
#' @param meta Additional metadata of the object to be printed. The metadata
#'   will be collected when the object is printed, and accessible via
#'   \code{knit_meta()}.
#' @param cacheable Boolean indicating whether this object is cacheable. If
#'   \code{FALSE}, \pkg{knitr} will stop when caching is enabled on code chunks
#'   that contain \code{asis_output()}.
#' @note This function only works in top-level R expressions, and it will not
#'   work when it is called inside another expression, such as a for-loop. See
#'   \url{https://github.com/yihui/knitr/issues/1137} for a discussion.
#' @export
#' @examples  # see ?knit_print
asis_output = function(x, meta = NULL, cacheable = NA) {
  structure(x, class = 'knit_asis', knit_meta = meta, knit_cacheable = cacheable)
}

#' Metadata about objects to be printed
#'
#' As an object is printed, \pkg{knitr} will collect metadata about it (if
#' available). After knitting is done, all the metadata is accessible via this
#' function. You can manually add metadata to the \pkg{knitr} session via
#' \code{knit_meta_add()}.
#' @param class Optionally return only metadata entries that inherit from the
#'   specified class. The default, \code{NULL}, returns all entries.
#' @param clean Whether to clean the collected metadata. By default, the
#'   metadata stored in \pkg{knitr} is cleaned up once retrieved, because we may
#'   not want the metadata to be passed to the next \code{knit()} call; to be
#'   defensive (i.e. not to have carryover metadata), you can call
#'   \code{knit_meta()} before \code{knit()}.
#' @export
#' @return \code{knit_meta()} returns the matched metadata specified by
#'   \code{class}; \code{knit_meta_add()} returns all current metadata.
knit_meta = function(class = NULL, clean = TRUE) {
  if (is.null(class)) {
    if (clean) on.exit({.knitEnv$meta = list()}, add = TRUE)
    return(.knitEnv$meta)
  }
  # if a class was specified, match the items belonging to the class
  matches = if (length(.knitEnv$meta)) {
    vapply(.knitEnv$meta, inherits, logical(1), what = class)
  }
  if (!any(matches)) return(list())
  if (clean) on.exit({
    .knitEnv$meta[matches] = NULL
    id = attr(.knitEnv$meta, 'knit_meta_id')
    if (length(id)) attr(.knitEnv$meta, 'knit_meta_id') = id[!matches]
  }, add = TRUE)
  .knitEnv$meta[matches]
}

#' @param meta A metadata object to be added to the session.
#' @param label A chunk label to indicate which chunk the metadata belongs to.
#' @rdname knit_meta
#' @export
knit_meta_add = function(meta, label = '') {
  if (length(meta)) {
    meta_id = attr(.knitEnv$meta, 'knit_meta_id')
    .knitEnv$meta = c(.knitEnv$meta, meta)
    attr(.knitEnv$meta, 'knit_meta_id') = c(meta_id, rep_len(label, length(meta)))
  }
  .knitEnv$meta
}

context('Output functions')

test_that('automatic output file name works as expected', {

  res = unlist(lapply(c('abc.Rnw', 'abc.rnw', 'abc.tex', '_knit_abc.tex', '_knit_abc.md', 'foo_knit_.html'),
                      auto_out_name))
  expect_identical(res, c('abc.tex', 'abc.tex', 'abc.txt', 'abc.tex', 'abc.md', 'foo.html'))

  res = unlist(lapply(c('abc.rtex', 'abc.Rmd', 'abc.rhtm', 'abc.Rhtml', 'foo.abc.rhtml'),
                      auto_out_name))
  expect_identical(res, c('abc.tex', 'abc.md', 'abc.htm', 'abc.html', 'foo.abc.html'))

  expect_identical(auto_out_name('foo.bar'), 'foo.txt')
})

test_that('chunks with include=FALSE should stop on error', {
  expect_error(knit(text = c('<<include=F>>=', '1+"a"', '@')))
})

context('knitr environments')

test_that('use a list as parent frame', {
  opts_knit$set(progress = FALSE)
  rm(list = ls())
  z = 5
  # evaluate in a new environment; should create an object in current envir
  with(list(y = 4:8), knit('knit-envir.Rmd', envir = new.env()))
  expect_false(exists('asdfqwerzxcv'))
  
  expect_error(knit('knit-envir.Rmd'))  # y is not found
  with(list(y = letters), knit('knit-envir.Rmd'))
  opts_knit$set(progress = TRUE)
})

#' Automatically create a report based on an R script and a template
#'
#' This is a convenience function for small-scale automatic reporting based on
#' an R script and a template. The default template is an Rnw file (LaTeX);
#' \code{stitch_rhtml()} and \code{stitch_rmd()} are wrappers on top of
#' \code{stitch()} using the R HTML and R Markdown templates respectively.
#'
#' The first two lines of the R script can contain the title and author of the
#' report in comments of the form \samp{## title:} and \samp{## author:}. The
#' template must have a token \samp{\%sCHUNK_LABEL_HERE}, which will be used to
#' input all the R code from the script. See the examples below.
#'
#' The R script may contain chunk headers of the form \samp{## ---- label,
#' opt1=val1, opt2=val2}, which will be copied to the template; if no chunk
#' headers are found, the whole R script will be inserted into the template as
#' one code chunk.
#' @param script Path to the R script.
#' @param template Path of the template to use. By default, the Rnw template in
#'   this package; there is also an HTML template in \pkg{knitr}.
#' @param output Output filename, passed to \code{\link{knit}}). By default,
#'   the base filename of the script is used.
#' @inheritParams knit
#' @return path of the output document
#' @export
#' @seealso \code{\link{spin}} (turn a specially formatted R script to a report)
#' @examples s = system.file('misc', 'stitch-test.R', package = 'knitr')
#' if (interactive()) stitch(s)  # compile to PDF
#'
#' # HTML report
#' stitch(s, system.file('misc', 'knitr-template.Rhtml', package = 'knitr'))
#'
#' # or convert markdown to HTML
#' stitch(s, system.file('misc', 'knitr-template.Rmd', package = 'knitr'))
#'
#' unlink(c('stitch-test.html', 'stitch-test.md', 'figure'), recursive = TRUE)
stitch = function(script,
                  template = system.file('misc', 'knitr-template.Rnw', package = 'knitr'),
                  output = NULL, text = NULL, envir = parent.frame()) {
  lines = if (nosrc <- is.null(text)) read_utf8(script) else split_lines(text)
  # extract title and author from first two lines
  if (comment_to_var(lines[1L], '.knitr.title', '^#+ *title:', envir)) lines = lines[-1L]
  if (comment_to_var(lines[1L], '.knitr.author', '^#+ *author:', envir)) lines = lines[-1L]
  input = basename(template)
  input = with_ext(basename(if (nosrc) script else tempfile()), file_ext(input))
  txt = read_utf8(template)
  i = grep('%sCHUNK_LABEL_HERE', txt)
  if (length(i) != 1L) stop('Wrong template for stitch: ', template)
  h = sub('CHUNK_LABEL_HERE', '', txt[i])
  j = grep(.sep.label, lines)
  if (length(j) == 0) {
    lines = c(sprintf(h, 'auto-report'), lines)
  } else {
    lines[j] = sprintf(h, gsub(.sep.label, '\\3', lines[j]))
    if (j[1] != 1L) lines = c(sprintf(h, ''), lines)
  }
  txt[i] = one_string(lines)
  opts_chunk$set(
    fig.align = 'center', par = TRUE, fig.width = 6, fig.height = 6,
    fig.path = paste('figure', gsub('[^[:alnum:]]', '-', input), sep = '/')
  )
  on.exit(opts_chunk$restore(), add = TRUE)
  knit_hooks$set(par = function(before, options, envir) {
    if (before) par(mar = c(4, 4, .1, .1), cex.lab = .95, cex.axis = .9,
                    mgp = c(2, .7, 0), tcl = -.3, las = 1)
  })
  on.exit(knit_hooks$restore(), add = TRUE)

  out = knit(input, output, envir = envir, text = txt)
  switch(file_ext(out), tex = {
    tinytex::latexmk(out)
    message('PDF output at: ', with_ext(out, 'pdf'))
  }, md = {
    out.html = with_ext(out, 'html')
    markdown::markdownToHTML(out, out.html)
    message('HTML output at: ', out.html)
  })
  out
}
#' @rdname stitch
#' @param ... Arguments passed to \code{stitch()}.
#' @export
stitch_rhtml = function(..., envir = parent.frame()) stitch(
  ..., envir = envir,
  template = system.file('misc', 'knitr-template.Rhtml', package = 'knitr')
)

#' @rdname stitch
#' @export
stitch_rmd = function(..., envir = parent.frame()) stitch(
  ..., envir = envir,
  template = system.file('misc', 'knitr-template.Rmd', package = 'knitr')
)

#' A simple macro preprocessor for templating purposes
#'
#' This function expands a template based on the R expressions in \code{{{}}}
#' (this tag can be customized by the \code{delim} argument). These expressions
#' are extracted, evaluated and replaced by their values in the original
#' template.
#' @param file The template file.
#' @param ... A list of variables to be used for the code in the template; note that
#'   the variables will be searched for in the parent frame as well.
#' @param text Character vector of lines of code. An alternative way to specify
#'   the template code directly. If \code{text} is provided, \code{file} will be ignored.
#' @param delim A pair of opening and closing delimiters for the templating tags.
#' @return A character vector, with the tags evaluated and replaced by their
#'   values.
#' @references This function was inspired by the pyexpander and m4
#'   (\url{http://www.gnu.org/software/m4/}), thanks to Frank Harrell.
#' @export
#' @examples # see the knit_expand vignette
#' if (interactive()) browseVignettes(package='knitr')
knit_expand = function(file, ..., text = read_utf8(file), delim = c('{{', '}}') ){

  # check if delim is a pair, escaping regex as necessary
  if (length(delim) != 2L) stop('"delim" must be of length 2')
  delim = gsub('([.|()\\^{}+$*?]|\\[|\\])', '\\\\\\1', delim)
  delim = paste0(delim[1L], '((.|\n)+?)', delim[2L])

  txt = one_string(text)
  loc = stringr::str_locate_all(txt, delim)[[1L]]
  if (nrow(loc) == 0L) return(txt) # no match
  mat = stringr::str_extract_all(txt, delim)[[1L]]
  mat = sub(delim, '\\1', mat)
  env = list(...)
  env = if (length(env)) list2env(env, parent = parent.frame()) else parent.frame()
  inline_exec(list(code = mat, input = txt, location = loc),
              envir = env, hook = identity)
}

context('Templates')

test_that('templates use data specified', {
  template = 'The value of a is {{a}}, so a + 1 is {{a + 1}}'
  exp = knit_expand(text = template, a = 10)
  act = 'The value of a is 10, so a + 1 is 11'
  expect_equal(exp, act)
})

test_that('templates respect custom delimiter pairs', {
  template = 'The value of a is <% a %>, so a + 1 is <% a + 1 %>'
  exp = knit_expand(text = template, a = 10, delim = c("<%", "%>"))
  act = 'The value of a is 10, so a + 1 is 11'
  expect_equal(exp, act)
})

test_that('templates respect pypi delimiters', {
  template = 'hello $(LETTERS[24]) and $(pi)!'
  exp = knit_expand(text = template, delim = c("$(", ")"))
  act = "hello X and 3.14159265358979!"
  expect_equal(exp, act)
})

test_that('error is thrown when delimiter is not a pair', {
  template = 'The value of a is <% a %>, so a + 1 is <% a + 1 %>'
  expect_error(knit_expand(text = template, a = 10, delim = c("<%")))
})

test_that('templates use data from parent frame', {
  template = 'The value of a is {{a}}, and b + 1 is {{b + 1}}'
  b = -1.21
  exp = knit_expand(text = template, a = 10)
  act = "The value of a is 10, and b + 1 is -0.21"
  expect_equal(exp, act)
})

#' Knit R Markdown using the classic Docco style
#'
#' The classic Docco style is a two-column layout, with text in the left and
#' code in the right column.
#'
#' The output HTML page supports resizing and hiding/showing the two columns.
#' Move the cursor to the center of the page, and it will change to a
#' bidirectional resize cursor; drag the cursor to resize the two columns. Press
#' the key \code{t} to hide the code column (show the text column only), and
#' press again to hide the text column (show code).
#' @param input Path of the input R Markdown file.
#' @param ... Arguments to be passed to \code{\link{knit2html}}
#' @return An HTML file is written, and its name is returned.
#' @author Weicheng Zhu and Yihui Xie
#' @references The Docco package by Jeremy Ashkenas:
#'   \url{https://github.com/jashkenas/docco}
#' @export
#' @examples rocco_view=function(input) {
#' owd = setwd(tempdir()); on.exit(setwd(owd))
#' if (!file.exists(input)) return()
#' o=rocco(input, quiet=TRUE)
#' if (interactive()) browseURL(o)}
#' # knit these two vignettes using the docco style
#' rocco_view(system.file('doc', 'docco-classic.Rmd', package = 'knitr'))
#' rocco_view(system.file('doc', 'knit_expand.Rmd', package = 'knitr'))
rocco = function(input, ...) {
  knit2html(
    input, ...,
    meta = list(
      css = c('@npm/@xiee/utils/css/docco-classic.min.css', '@prism-xcode'),
      js = c(
        '@npm/jquery@3.7.1/dist/jquery.min.js',
        '@npm/@xiee/utils/js/docco-classic.min.js,docco-resize.js',
        '@npm/@xiee/utils/js/center-img.min.js'
      )
    )
  )
}

# the Docco linear style
docco_linear = function(...) {
  knit2html(..., meta = list(
    css = c(
      'https://ashkenas.com/docco/resources/linear/public/stylesheets/normalize.css',
      'https://ashkenas.com/docco/resources/linear/docco.css', '@prism-xcode'
    ),
    js = '@npm/@xiee/utils/js/center-img.min.js',
    `header-includes` = '<style type="text/css">.container{width:auto;max-width:920px;}.page{width:auto;max-width:800px;}.page pre{width:100%;max-width:768px;}pre, code{font-size:90%;}</style>',
    `include-before` = '<div class="container"><div class="page">',
    `include-after` = '<div class="fleur">h</div></div></div>'
  ))
}

#' Built-in chunk hooks to extend knitr
#'
#' Hook functions are called when the corresponding chunk options are not
#' \code{NULL} to do additional jobs beside the R code in chunks. This package
#' provides a few useful hooks, which can also serve as examples of how to
#' define chunk hooks in \pkg{knitr}.
#'
#' The function \code{hook_pdfcrop()} can use the program \command{pdfcrop} to
#' crop the extra white margin when the plot format is PDF to make better use of
#' the space in the output document, otherwise we often have to struggle with
#' \code{\link[graphics]{par}} to set appropriate margins. Note
#' \command{pdfcrop} often comes with a LaTeX distribution such as MiKTeX or
#' TeXLive, and you may not need to install it separately (use
#' \code{Sys.which('pdfcrop')} to check it; if it not empty, you are able to use
#' it). Similarly, when the plot format is not PDF (e.g. PNG), the \pkg{magick}
#' package is used to crop the plot.
#'
#' The function \code{hook_optipng()} calls the program \command{optipng} to
#' optimize PNG images. Note the chunk option \code{optipng} can be used to
#' provide additional parameters to the program \command{optipng}, e.g.
#' \code{optipng = '-o7'}.
#'
#' The function \code{hook_pngquant()} calls the program \command{pngquant} to
#' optimize PNG images. Note the chunk option \code{pngquant} can be used to
#' provide additional parameters to the program \command{pngquant}, e.g.
#' \code{pngquant = '--speed=1 --quality=0-50'}.
#'
#' The function \code{hook_mogrify()} calls the program \command{mogrify}.  Note
#' the chunk option \code{mogrify} can be used to provide additional parameters
#' to the program \command{mogrify} (with default \code{-trim} to trim PNG
#' files).
#'
#' When the plots are not recordable via \code{\link[grDevices]{recordPlot}} and
#' we save the plots to files manually via other functions (e.g. \pkg{rgl}
#' plots), we can use the chunk hook \code{hook_plot_custom} to help write code
#' for graphics output into the output document.
#'
#' The hook \code{hook_purl()} can be used to write the code chunks to an R
#' script. It is an alternative approach to \code{\link{purl}}, and can be more
#' reliable when the code chunks depend on the execution of them (e.g.
#' \code{\link{read_chunk}()}, or \code{\link{opts_chunk}$set(eval = FALSE)}).
#' To enable this hook, it is recommended to associate it with the chunk option
#' \code{purl}, i.e. \code{knit_hooks$set(purl = hook_purl)}. When this hook is
#' enabled, an R script will be written while the input document is being
#' \code{\link{knit}}. Currently the code chunks that are not R code or have the
#' chunk option \code{purl=FALSE} are ignored. Please note when the cache is
#' turned on (the chunk option \code{cache = TRUE}), no chunk hooks will be
#' executed, hence \code{hook_purl()} will not work, either. To solve this
#' problem, we need \code{cache = 2} instead of \code{TRUE} (see
#' \url{https://yihui.org/knitr/demo/cache/} for the meaning of \code{cache =
#' 2}).
#' @rdname chunk_hook
#' @param before,options,envir See \emph{References} below.
#' @references \url{https://yihui.org/knitr/hooks/#chunk_hooks}
#' @seealso \code{\link[rgl]{rgl.snapshot}}, \code{\link[rgl]{rgl.postscript}},
#'   \code{\link[rgl]{hook_rgl}}, \code{\link[rgl]{hook_webgl}}
#' @note The two hook functions \code{hook_rgl()} and \code{hook_webgl()} were
#'   moved from \pkg{knitr} to the \pkg{rgl} package (>= v0.95.1247) after
#'   \pkg{knitr} v1.10.5, and you can \code{library(rgl)} to get them.
#' @export
#' @examples if (require('rgl') && exists('hook_rgl')) knit_hooks$set(rgl = hook_rgl)
#' # then in code chunks, use the option rgl=TRUE
hook_pdfcrop = function(before, options, envir) {
  # crops plots after a chunk is evaluated and plot files produced
  if (before) return()
  in_base_dir(for (f in get_plot_files()) plot_crop(f))
}

get_plot_files = function() {
  unique(opts_knit$get('plot_files'))
}

#' @export
#' @rdname chunk_hook
hook_optipng = function(before, options, envir) {
  hook_png(before, options, envir, 'optipng')
}

hook_png = function(
  before, options, envir, cmd = c('optipng', 'pngquant', 'mogrify'), post_process = identity
) {
  if (before) return()
  cmd = match.arg(cmd)
  if (!nzchar(Sys.which(cmd))) {
    warning('cannot find ', cmd, '; please install and put it in PATH'); return()
  }
  paths = get_plot_files()
  paths = grep('[.]png$', paths, ignore.case = TRUE, value = TRUE)

  in_base_dir(
    lapply(paths, function(x) {
      message('optimizing ', x)
      cmd = paste(cmd, if (is.character(options[[cmd]])) options[[cmd]], shQuote(x))
      (if (is_windows()) shell else system)(cmd)
      post_process(x)
    })
  )
  return()
}

#' @export
#' @rdname chunk_hook
hook_pngquant = function(before, options, envir) {
  if (is.null(options[['pngquant']])) options$pngquant = '--skip-if-larger'
  options[['pngquant']] = paste(options[['pngquant']], '--ext -fs8.png')
  hook_png(before, options, envir, 'pngquant', function(x) {
    # pngquant creates an output file with '-fs8.png' as the extension.
    x2 = sub("\\.png$", "-fs8.png", x)
    if (file.exists(x2)) file.rename(x2, x)
  })
}

#' @export
#' @rdname chunk_hook
hook_mogrify = function(before, options, envir) {
  if (is.null(options[['mogrify']])) options$mogrify = '-trim'
  hook_png(before, options, envir, cmd = 'mogrify', identity)
}

#' @export
#' @rdname chunk_hook
hook_plot_custom = function(before, options, envir){
  if (before) return() # run hook after the chunk
  if (options$fig.show == 'hide') return() # do not show figures

  ext = options$fig.ext %n% dev2ext(options$dev)
  hook = knit_hooks$get('plot')

  n = options$fig.num
  if (n == 0L) n = options$fig.num = 1L # make sure fig.num is at least 1
  res = unlist(lapply(seq_len(n), function(i) {
    options$fig.cur = i
    hook(fig_path(ext, options, i), reduce_plot_opts(options))
  }), use.names = FALSE)
  paste(res, collapse = '')
}

#" a hook function to write out code from chunks
#' @export
#' @rdname chunk_hook
hook_purl = function(before, options, envir) {
  # at the moment, non-R chunks are ignored; it is unclear what I should do
  if (before || !options$purl || options$engine != 'R') return()

  output = .knitEnv$tangle.file
  if (isFALSE(.knitEnv$tangle.start)) {
    .knitEnv$tangle.start = TRUE
    unlink(output)
    # write out knit_params() data from YAML
    params = .knitEnv$tangle.params
    if (length(params)) write_utf8(params, output)
    .knitEnv$tangle.params = NULL
  }

  code = options$code
  if (isFALSE(options$eval)) code = comment_out(code, '# ', newline = FALSE)
  if (is.character(output)) {
    code = c(
      if (file.exists(output)) read_utf8(output),
      label_code(code, options$params.src)
    )
    write_utf8(code, output)
  }
}

library(testit)

if (requireNamespace('otelsdk', quietly = TRUE)) {

  record = with_otel_record({
    knit(
      text = c('<<tidy=FALSE, eval=1:2, echo=FALSE, results="asis">>=', '1', '1+', '1', '1', '@'),
      quiet = TRUE
    )
  })

  traces = record$traces

  assert('otel tracing works on text input', {
    (length(traces) %==% 3L)
    (startsWith(traces[[1L]]$name, 'knitr processing'))
    (traces[[1L]]$attributes$knitr.format %==% 'latex')
    (traces[[1L]]$attributes$knitr.input %==% '')
    (traces[[1L]]$attributes$knitr.output %==% '')
    (traces[[2L]]$name %==% 'knit unnamed-chunk-1')
    (traces[[2L]]$attributes$knitr.chunk.device %==% 'pdf')
    (traces[[2L]]$attributes$knitr.chunk.echo %==% FALSE)
    (traces[[2L]]$attributes$knitr.chunk.engine %==% 'R')
    (traces[[2L]]$attributes$knitr.chunk.eval %==% c(1, 2))
    (traces[[2L]]$attributes$knitr.chunk.label %==% 'unnamed-chunk-1')
    (startsWith(traces[[3L]]$name, 'knitr output'))
    (traces[[3L]]$attributes$knitr.format %==% 'latex')
    (traces[[3L]]$attributes$knitr.input %==% '')
    (traces[[3L]]$attributes$knitr.output %==% '')
  })

  record = with_otel_record({
    local({
      env = new.env()
      env$y = 1:3
      z = 5
      on.exit(file.remove('knit-envir.md'))
      knit('knit-envir.Rmd', envir = env, quiet = TRUE)
    })
  })

  traces = record$traces

  assert('otel tracing works when knitting files', {
    (length(traces) %==% 3L)
    (traces[[1L]]$name %==% 'knitr processing knit-envir.Rmd')
    (traces[[1L]]$attributes$knitr.format %==% 'markdown')
    (traces[[1L]]$attributes$knitr.input %==% 'knit-envir.Rmd')
    (traces[[1L]]$attributes$knitr.output %==% 'knit-envir.md')
    (traces[[2L]]$name %==% 'knit test')
    (traces[[2L]]$attributes$knitr.chunk.device %==% 'png')
    (traces[[2L]]$attributes$knitr.chunk.echo %==% TRUE)
    (traces[[2L]]$attributes$knitr.chunk.engine %==% 'R')
    (traces[[2L]]$attributes$knitr.chunk.eval %==% TRUE)
    (traces[[2L]]$attributes$knitr.chunk.label %==% 'test')
    (traces[[3L]]$name %==% 'knitr output knit-envir.md')
    (traces[[3L]]$attributes$knitr.format %==% 'markdown')
    (traces[[3L]]$attributes$knitr.input %==% 'knit-envir.Rmd')
    (traces[[3L]]$attributes$knitr.output %==% 'knit-envir.md')
  })

}

#' @include utils.R

new_defaults = function(value = list()) {
  defaults = value

  get = function(name, default = FALSE, drop = TRUE) {
    if (default) defaults = value  # this is only a local version
    if (missing(name)) defaults else {
      if (drop && length(name) == 1) defaults[[name]] else {
        setNames(defaults[name], name)
      }
    }
  }
  resolve = function(...) {
    dots = list(...)
    if (length(dots) == 0) return()
    if (is.null(names(dots)) && length(dots) == 1 && is.list(dots[[1]]))
      if (length(dots <- dots[[1]]) == 0) return()
    dots
  }
  set = function(...) {
    dots = resolve(...)
    if (length(dots)) defaults <<- merge(dots)
    invisible(NULL)
  }
  delete = function(keys) {
    for (k in keys) defaults[[k]] <<- NULL
  }
  merge = function(values) merge_list(defaults, values)
  restore = function(target = value) defaults <<- target
  append = function(...) {
    dots = resolve(...)
    for (i in names(dots)) dots[[i]] <- c(defaults[[i]], dots[[i]])
    if (length(dots)) defaults <<- merge(dots)
    invisible(NULL)
  }

  list(
    get = get, set = set, delete = delete,
    append = append, merge = merge, restore = restore
  )
}

#' Default and current chunk options
#'
#' Options for R code chunks. When running R code, the object \code{opts_chunk}
#' (default options) is not modified by chunk headers (local chunk options are
#' merged with default options), whereas \code{opts_current} (current options)
#' changes with different chunk headers and it always reflects the options for
#' the current chunk.
#'
#' Normally we set up the global options once in the first code chunk in a
#' document using \code{opts_chunk$set()}, so that all \emph{latter} chunks will
#' use these options. Note the global options set in one chunk will not affect
#' the options in this chunk itself, and that is why we often need to set global
#' options in a separate chunk.
#'
#' See \code{str(knitr::opts_chunk$get())} for a list of default chunk options.
#' @references Usage: \url{https://yihui.org/knitr/objects/}
#'
#'   A list of available options:
#'   \url{https://yihui.org/knitr/options/#chunk_options}
#' @note \code{opts_current} should be treated as read-only and you are supposed
#'   to only query its values via \code{opts_current$get()}. Technically you
#'   could also call \code{opts_current$set()} to change the values, but you are
#'   not recommended to do so unless you understand the consequences.
#' @export
#' @examples opts_chunk$get('prompt'); opts_chunk$get('fig.keep')
opts_chunk = new_defaults(list(

  eval = TRUE, echo = TRUE, results = 'markup', tidy = FALSE, tidy.opts = NULL,
  collapse = FALSE, prompt = FALSE, comment = '##', highlight = TRUE,
  strip.white = TRUE, size = 'normalsize', background = '#F7F7F7',

  cache = FALSE, cache.path = 'cache/', cache.vars = NULL, cache.lazy = TRUE,
  dependson = NULL, autodep = FALSE, cache.rebuild = FALSE,

  fig.keep = 'high', fig.show = 'asis', fig.align = 'default', fig.path = 'figure/',
  dev = NULL, dev.args = NULL, dpi = 72, fig.ext = NULL, fig.width = 7, fig.height = 7,
  fig.env = 'figure', fig.cap = NULL, fig.scap = NULL, fig.lp = 'fig:', fig.subcap = NULL,
  fig.pos = '', out.width = NULL, out.height = NULL, out.extra = NULL, fig.retina = 1,
  external = TRUE, sanitize = FALSE, interval = 1, aniopts = 'controls,loop',

  warning = TRUE, error = TRUE, message = TRUE,

  render = NULL,

  ref.label = NULL, child = NULL, engine = 'R', split = FALSE, include = TRUE, purl = TRUE

))

#' @rdname opts_chunk
#' @export
opts_current = new_defaults()

#' @include plot.R

# a list of options attributes for RStudio
opts_chunk_attr = local({
  opts = lapply(opts_chunk$get(), class)
  opts[opts == 'NULL'] = 'character'
  opts$results = list('markup', 'asis', 'hold', 'hide')
  opts$fig.show = list('asis', 'hold', 'animate', 'hide')
  opts$fig.keep = list('high', 'none', 'all', 'first', 'last')
  opts$fig.align = list('default', 'left', 'right', 'center')
  opts$fig.showtext = 'logical'
  # quartz_ devices are for OS X only
  opts$dev = names(auto_exts)
  if (Sys.info()[['sysname']] != 'Darwin')
    opts$dev = grep('^quartz_', opts$dev, value = TRUE, invert = TRUE)
  if (.Platform$OS.type != 'windows')
    opts$dev = setdiff(opts$dev, 'win.metafile')
  opts$dev = as.list(opts$dev)
  opts$fig.ext = as.list(unique(auto_exts))
  opts$external = opts$sanitize = NULL  # hide these two rare options
  opts$fig.process = 'function'
  opts$fig.asp = 'numeric'
  opts$fig.dim = 'list'
  opts$R.options = 'list'
  opts$cache.comments = 'logical'
  opts$animation.hook = list('ffmpeg', 'gifski')
  opts
})

#' Set aliases for chunk options
#'
#' We do not have to use the chunk option names given in \pkg{knitr}; we can set
#' aliases for them. The aliases are a named character vector; the names are
#' aliases and the elements in this vector are the real option names.
#' @param ... Named arguments. Argument names are aliases, and argument values
#'   are real option names.
#' @return \code{NULL}. \code{opts_knit$get('aliases')} is modified as the side effect.
#' @export
#' @examples set_alias(w = 'fig.width', h = 'fig.height')
#' # then we can use options w and h in chunk headers instead of fig.width and fig.height
set_alias = function(...) {
  opts_knit$set(aliases = c(...))
}

#' Options for the knitr package
#'
#' Options including whether to use a progress bar when knitting a document, and
#' the base directory of images, etc.
#'
#' Besides the standard usage (\code{opts_knit$set()}), we can also set package
#' options prior to loading \code{knitr} or calling \code{knit()} using
#' \code{\link{options}()} in base R. A global option \code{knitr.package.foo}
#' in \code{options()} will be set as an option \code{foo} in \code{opts_knit},
#' i.e. global options in base R with the prefix \code{knitr.package.}
#' correspond to options in \code{opts_knit}. This can be useful to set package
#' options in \file{~/.Rprofile} without loading \pkg{knitr}.
#'
#' See \code{str(knitr::opts_knit$get())} for a list of default package options.
#' @references Usage: \url{https://yihui.org/knitr/objects/}
#'
#'   A list of available options:
#'   \url{https://yihui.org/knitr/options/#package_options}
#' @export
#' @examples opts_knit$get('verbose'); opts_knit$set(verbose = TRUE)  # change it
#' if (interactive()) {
#' # for unnamed chunks, use 'fig' as the figure prefix
#' opts_knit$set(unnamed.chunk.label='fig')
#' knit('001-minimal.Rmd') # from https://github.com/yihui/knitr-examples
#' }
#' @include hooks-html.R
opts_knit = new_defaults(list(
  progress = TRUE, verbose = FALSE, eval.after = 'fig.cap',
  base.dir = NULL, base.url = NULL, root.dir = NULL, child.path = '',
  upload.fun = identity, global.device = FALSE, global.par = FALSE,
  concordance = FALSE, documentation = 1L, self.contained = TRUE,
  unnamed.chunk.label = 'unnamed-chunk', highr.opts = NULL,

  # internal options; users should not touch them
  out.format = NULL, child = FALSE, parent = FALSE, tangle = FALSE, aliases = NULL,
  header = c(highlight = '', tikz = '', framed = ''), global.pars = NULL
))
# tangle: whether I'm in tangle mode; child: whether I'm in child document mode;
# parent: whether I need to add parent preamble to the child output

# you may modify these options in options(knitr.package.foo)
opts_knit_names = c(
  'progress', 'verbose', 'upload.fun', 'animation.fun', 'global.device',
  'eval.after', 'concordance', 'documentation', 'aliases', 'self.contained',
  'unnamed.chunk.label'
)
# adjust opts_chunk and opts_knit according to options(), e.g.
# options(knitr.package.progress = FALSE) --> opts_knit$set(progress = FALSE),
# and options(knitr.chunk.tidy) --> opts_chunk$set(tidy = TRUE); this makes it
# possible to set options in ~/.Rprofile without loading knitr
adjust_opts_knit = function() {
  # begin_hack: R CMD build does not evaluate .Rprofile, but I need a way to
  # modify opts_chunk just for myself
  if (nzchar(opts <- Sys.getenv('R_KNITR_OPTIONS')))
    eval(parse_only(sprintf('base::options(%s)', opts)), envir = globalenv())
  # end_hack
  opts = options()
  nms = names(opts)
  if (length(nms <- grep('^knitr[.]', nms, value = TRUE)) == 0) return()
  opts = opts[nms]
  # for backward compatibility
  i = grep('^knitr[.](package|chunk)[.]', nms, invert = TRUE)
  i = intersect(i, which(nms[i] %in% paste('knitr', opts_knit_names, sep = '.')))
  if (length(i)) {
    nms.pkg = sub('^knitr.', 'knitr.package.', nms[i])
    warning2(
      'These options must be renamed (from left to right):\n',
      formatUL(sprintf('%s => %s', nms[i], nms.pkg)), immediate. = TRUE
    )
    Sys.sleep(10)
    names(opts)[i] = nms[i] = nms.pkg
  }
  # strip off knitr.chunk from option names and set chunk options
  i = grep('^knitr[.]chunk[.]', nms)
  opts_chunk$set(setNames(opts[i], sub('^knitr[.]chunk[.]', '', nms[i])))
  # similarly for knitr.package.options
  i = grep('^knitr[.]package[.]', nms)
  opts_knit$set(setNames(opts[i], sub('^knitr[.]package[.]', '', nms[i])))
}

#' Template for creating reusable chunk options
#'
#' Creates a template binding a label to a set of chunk options. Every chunk
#' that references the template label will have the specificed set of options
#' applied to it.
#' @export
#' @examples opts_template$set(myfigures = list(fig.height = 4, fig.width = 4))
#' # later you can reuse these chunk options by 'opts.label', e.g.
#'
#' # <<foo, opts.label='myfigures'>>=
#'
#' # the above is equivalent to <<foo, fig.height=4, fig.width=4>>=
opts_template = new_defaults()

context('Cache')

test_that('find_globals() identifies global variables', {
  # nothing from outside environment
  expect_identical(find_globals('x=1'), character(0))
  # qwer must be created outside somewhere
  expect_identical(find_globals('a=1; b=a; d=qwer'), 'qwer')
  expect_identical(find_globals('a=function(){f=2;g}'), 'g')
  # y was assigned locally in z, but there is another y outside from nowhere
  expect_identical(find_globals('z=function(){y=1};y'), 'y')
  # more complicated cases: operators, subscripts, ...
  expect_identical(find_globals(c('a=1%*%1%o%2 %in% d','b=d%%10+3%/%2-z[1:3]')), c('d', 'z'))
})

test_that('dep_prev() sets dependencies on previous chunks', {
  knit_code$set(a=1, b=2, c=3)
  # dependency is empty now
  expect_identical(dep_list$get(), list())
  dep_prev()
  # b/c depend on a, c depends on b
  expect_identical(dep_list$get(), list(a=c('b', 'c'), b='c'))
  dep_list$restore()
  knit_code$restore()
})

context('new_defaults()')

test_that('set(), get(), merge() and restore() methods', {
  z = new_defaults(list(a = 1))
  expect_identical(z$get(), list(a = 1))
  z$set(a = 2)
  expect_identical(z$get('a'), 2)
  z$set(b = 'f')
  expect_identical(z$get(c('a', 'b')), list(a=2, b='f'))
  # merge() does not change z
  z$merge(list(b = 'g'))
  expect_identical(z$get('b'), 'f')
  # but the returned value contains the merge
  expect_identical(z$merge(list(b = 'g'))[['b']], 'g')
  expect_identical(z$get(default = TRUE), list(a=1))
  z$restore()
  expect_identical(z$get(), list(a=1))
})

test_that('a named argument of list in $set() method is not treated as a list of options', {
  z = new_defaults()
  # named argument in set()
  z$set(a = list(b=2, c='qwer'))
  expect_identical(length(z$get()), 1L)
  z$restore()
  # unnamed argument in set()
  z$set(list(b=2, c='qwer'))
  expect_identical(z$get(), list(b=2, c='qwer'))
})

library(testit)

# helper function to convert raw src to params list
read_params <- function(src, evaluate = TRUE) {
  lines <- strsplit(src, "\n")[[1]]
  knit_params(lines, evaluate = evaluate)
}

# helper function to convert raw src yaml to params list
read_params_yaml <- function(src, evaluate = TRUE) {
  knit_params_yaml(src, evaluate = evaluate)
}


## test basic parameter parsing --------------------------------------------

params <- read_params('
---
params:
  a: 10
  b: 20
---
'
)
assert(params[[1]]$name == 'a')
assert(params[[1]]$value == 10)
assert(params[[2]]$name == 'b')
assert(params[[2]]$value == 20)

assert(identical(flatten_params(params), list(a = 10L, b = 20L)))

## test date custom type (these deprecated and here for backwards compt) --

params <- read_params('
---
params:
  start: !date 2015-01-01
  end: !datetime 2015-01-01 12:30:00
---
'
)
assert(params[[1]]$name == 'start')
assert('Date' %in% class(params[[1]]$value))
assert(params[[1]]$value == as.Date("2015-01-01"))
assert(params[[2]]$name == 'end')
assert('POSIXct' %in% class(params[[2]]$value))
assert(params[[2]]$value == as.POSIXct("2015-01-01 12:30:00", tz = "GMT"))


## test specifying value in sub-object and type at object level ------------

params <- read_params('
---
params:
  file1:
    value: data1.csv
---
'
)
assert(params[[1]]$name == 'file1')
assert(params[[1]]$value == 'data1.csv')


## test parameters with length(value) > 1 ----------------------------------

params <- read_params('
---
params:
  regions:
    value: [North, South]
---
'
)
assert(length(params[[1]]$value) == 2)
assert(params[[1]]$value[[2]] == 'South')


## test including additional parameter attributes --------------------------

params <- read_params('
---
params:
  regions:
    value: [North, South]
    choices: [North, South, East, West]
    label: "Select Regions"
---
'
)
assert(identical(params[[1]]$choices, c('North', 'South', 'East', 'West')))
assert(params[[1]]$label == "Select Regions")


## test y/Y/n/N ------------------------------------------------------------

params <- read_params('
---
params:
  x: 1
  y: 2
  z: 3
  n: 4
  Y: 5
  N: 6
---
'
)

assert(
  'y/Y/n/N are not converted to booleans',
  identical(unname(unlist(lapply(params, `[[`, 'name'))), c('x', 'y', 'z', 'n', 'Y', 'N'))
)


## test handling of expressions --------------------------------------------

params <- read_params('
---
params:
  today: !r Sys.Date()
  now: !expr Sys.time()
  x: 10
---
'
)
assert(!is.null(params[[1]]$expr))
assert('Date' %in% class(params[[1]]$value))
assert(!is.null(params[[2]]$expr))
assert('POSIXct' %in% class(params[[2]]$value))
assert(is.null(params[[3]]$expr))

## test handling of unevaluated expressions --------------------------------------------

params <- read_params('
---
params:
  today: !r Sys.Date()
---
', evaluate = FALSE)
assert(identical(params$today$expr, "Sys.Date()"))
assert(identical(class(params$today$value), "expression"))

## test handling of yaml parameters --------------------------------------------

params <- read_params_yaml('
params:
  x: 1
  today: !r Sys.Date()
  posixlt: !r strptime("2015-01-01", format = "%Y-%m-%d")
  list: [1,2,3]
#  map: { a: 1, b: 2 }
  map: { value: { a: 1, b: 2 } }
')
# The direct map value is not supported; an explicit value field is necessary
assert(params$x$value == 1)
assert(identical(class(params$x$value), "integer"))
assert(identical(params$today$expr, "Sys.Date()"))
assert('Date' %in% class(params$today$value))
assert(identical(params$posixlt$expr, 'strptime("2015-01-01", format = "%Y-%m-%d")'))
assert('POSIXlt' %in% class(params$posixlt$value))

## test handling of unevaluated yaml parameters --------------------------------------------

params <- read_params_yaml('
params:
  today: !r Sys.Date()
', evaluate = FALSE)
assert(identical(params$today$expr, "Sys.Date()"))
assert(identical(class(params$today$value), "expression"))

#' Set or get theme to use for syntax highlighting
#' @noRd
#' @author Ramnath Vaidyanathan
set_theme = function(theme) {
  header = if (is.list(theme)) theme else theme_to_header(theme)
  opts_chunk$set(background = header$background)
  set_header(highlight = header$highlight)
  # par(col = theme$foreground)
}
get_theme = function(theme = NULL) {
  if (is.null(theme)) {
    theme_dir = system.file('themes', package = 'knitr')
    theme_files = list.files(theme_dir,  pattern = '\\.css$')
    gsub('\\.css$', '', basename(theme_files))
  } else {
    theme_to_header(theme)
  }
}

#' Syntax highlighting themes
#'
#' This object can be used to set or get themes in \pkg{knitr} for syntax
#' highlighting.
#'
#' We can use \code{knit_theme$set(theme)} to set the theme, and
#' \code{knit_theme$get(theme)} to get a theme. The \code{theme} is a character
#' string for both methods (either the name of the theme, or the path to the CSS
#' file of a theme), and for the \code{set()} method, it can also be a list
#' returned by the \code{get()} method. See examples below.
#' @author Ramnath Vaidyanathan and Yihui Xie
#' @references For a preview of all themes, see
#'   \url{https://gist.github.com/yihui/3422133}.
#' @note The syntax highlighting here only applies to \file{.Rnw} (LaTeX) and
#'   \file{.Rhtml} (HTML) documents, and it does not work for other types of
#'   documents, such as \file{.Rmd} (R Markdown, which has its own syntax
#'   highlighting themes; see \url{http://rmarkdown.rstudio.com}).
#' @export
#' @examples opts_knit$set(out.format='latex'); knit_theme$set('edit-vim')
#'
#' knit_theme$get()  # names of all available themes
#'
#' thm = knit_theme$get('acid')  # parse the theme to a list
#' knit_theme$set(thm)
#'
#' opts_knit$set(out.format=NULL) # restore option
knit_theme = list(set = set_theme, get = get_theme)


#' Generates header based on a theme and output format of document
#' @author Ramnath Vaidyanathan
#' @noRd
theme_to_header = function(theme, format = out_format()){
  if (format == 'latex') {
    theme_to_header_latex(theme)
  } else theme_to_header_html(theme)
}

#' Generates latex header based on a theme
#' @author Ramnath Vaidyanathan
#' @noRd
theme_to_header_latex = function(theme) {
  css_file = if (file.exists(theme)) theme else {
    system.file('themes', sprintf('%s.css', theme), package = 'knitr', mustWork = TRUE)
  }
  css_out = css.parser(css_file)

  # get background and foreground colors
  background = css_out$background$color
  foreground = css_out$std$color

  # write latex highlight header
  fgheader = color_def(foreground, 'fgcolor')
  highlight = one_string(c(fgheader, styler_assistant_latex(css_out[-1])))
  list(highlight = highlight, background = background, foreground = foreground)
}

#' Generates css header based on a theme
#' @author Ramnath Vaidyanathan
#' @noRd
theme_to_header_html = function(theme){
  css_file = if (file.exists(theme)) theme else {
    system.file('themes', sprintf('%s.css', theme), package = 'knitr')
  }
  css = css.parser(css_file)
  bgcolor = css$background$color
  css_knitr = read_utf8(system.file('misc', 'knitr.css', package = 'knitr'))
  css_knitr[-2] = sub('^(\\s+background-color:\\s+)(.*)$', sprintf('\\1%s;', bgcolor), css_knitr[-2])
  css = c(css_knitr, gsub('^([.][a-z]{3} )', '.hl\\1', read_utf8(css_file)[-(1:3)]))
  list(highlight = one_string(css))
}

# parse a theme file from Highlight v3.x by Andre Simon to an R list of the form
# list(Colour = hex, Bold = TRUE, Italic = TRUE)
theme2list = function(theme.file) {
  y = read_utf8(theme.file)
  i = grep('^\\s*Description', y)
  if (i > 1) y = y[-seq_len(i - 1)]
  y = gsub('[{]', 'list(', y)
  y = gsub('[}]', ')', y)
  y = gsub(';', '', y)
  y = gsub('true', 'TRUE', y)
  y = one_string(y)
  y = gsub(',\\s*)', ')', y)
  env = new.env()
  #cat(y, sep = '\n')
  eval(parse(text = y), envir = env)
  y = as.list(env)
  for (i in seq_along(y$Keywords)) {
    y[[paste0('Keyword', i)]] = y$Keywords[[i]]
  }
  y$Keywords = NULL
  y
}

# mapping between CSS class and Highlight theme elements
cls2thm = c(
  background = 'Canvas', num = 'Number', str = 'String', com = 'BlockComment',
  opt = 'Operator', std = 'Default',
  kwa = 'Keyword1', kwb = 'Keyword2', kwc = 'Keyword3', kwd = 'Keyword4'
)

# turn a list from theme2list() to CSS code
list2css = function(lst) {
  css = character(length(cls2thm))
  for (i in seq_along(cls2thm)) {
    m = cls2thm[i]; l = lst[[m]]
    # if not found, use the default style
    if (!is.list(l)) l = lst[['Default']]
    css[i] = one_string(c(
      sprintf('.%s {', names(m)), sprintf('  color: %s;', l$Colour),
      sprintf('  font-weight: %s;', if (isTRUE(l$Bold)) 'bold'),
      sprintf('  font-style: %s;', if (isTRUE(l$Italic)) 'italic'), '}'
    ))
  }
  css
}

# generate CSS files for all themes in Andre Simon's Highlight package, e.g.
# themes2css('~/tmp/highlight/themes', '~/downloads/knitr/inst/themes')
themes2css = function(theme.path, css.path) {
  for (f in list.files(theme.path, pattern = '[.]theme$', full.names = TRUE)) {
    theme.name = sub('[.]theme$', '', basename(f))
    css.file = file.path(css.path, sprintf('%s.css', theme.name))
    write_utf8(list2css(theme2list(f)), css.file)
    message('theme ', theme.name, ' saved to ', css.file)
  }
}

#' @include defaults.R

# record input/output lines numbers in Rnw/tex and filenames
knit_concord = new_defaults(list(
  inlines = NULL, outlines = NULL, infile = NULL, outfile = NULL
))

# do not consider child mode for concordance
concord_mode = function() {
  opts_knit$get('concordance') && !child_mode() &&
    out_format(c('latex', 'sweave', 'listings'))
}

current_lines = function(i) {
  # a helpr function to return line numbers for block i
  n = knit_concord$get('inlines')
  n1 = sum(head(n, i)); n0 = n1 - n[i] + 2
  c(min(n0, n1), n1)
}

# generate concordance for RStudio
concord_gen = function(infile, outfile) {
  if (!concord_mode()) return()
  i = knit_concord$get('inlines'); o = knit_concord$get('outlines')
  if (is.null(i) || is.null(o)) {
    warning('cannot generate concordance due to incomplete line numbers')
    return()
  }
  stopifnot(length(i) == length(o))

  steps = NULL # how many steps to jump forward to match output line numbers
  for (k in seq_along(i)) {
    steps = c(steps, if (o[k] >= i[k]) {
      rep(c(1L, 0L), c(i[k], o[k] - i[k]))
    } else {
      c(rep(1L, o[k] - 1L), i[k] - o[k] + 1L)
    })
  }

  # generate data structure
  vals = rle(steps)
  vals = c(1L, as.numeric(rbind(vals$lengths, vals$values)))
  concordance = paste(strwrap(paste(vals, collapse = ' ')), collapse = ' %\n')

  confile = paste(sans_ext(outfile), 'concordance.tex', sep = '-')
  # write to file
  cat('\\Sconcordance{concordance:', outfile, ':', infile, ':%\n',
      concordance, '}\n', sep = '', file = confile)
}

# hooks that return character values will be inserted into final output
run_hooks = function(before, options, envir = knit_global()) {
  # default and user-defined new hooks
  hooks.n = knit_hooks$get()
  hooks.a = hooks.n[setdiff(names(hooks.n), names(.default.hooks))] # a list of hooks to run
  out = NULL
  nms = intersect(names(options), names(hooks.a))
  if (!before) nms = rev(nms)
  for (i in nms) {
    # run only when option is not NULL, and hook is not NULL
    if (is.null(options[[i]]) || is.null(hook <- hooks.a[[i]])) next
    if (is.character(hook)) hook = get(hook, envir = envir, mode = 'function')
    args = list(before = before, options = options, envir = envir, name = i)
    args = args[names(formals(hook))]
    res  = do.call(hook, args, envir = envir)
    if (is.character(res)) out = c(out, res)
  }
  out
}

#' Upload an image to imgur.com
#'
#' This function is an alias to \code{xfun::upload_imgur()}. It is kept in
#' \pkg{knitr} only for backward-compatibility reasons. You are recommended to
#' use \code{xfun::upload_imgur()} directly instead.
#' @param file,key,... See \code{xfun::\link[xfun]{upload_imgur}()}.
#' @export
#' @keywords internal
imgur_upload = function(file, key = xfun::env_option('knitr.imgur.key'), ...) {
  xfun::upload_imgur(file, key, ..., include_xml = TRUE)
}

library(testit)

op = options(digits = 3, scipen = 0, knitr.digits.signif = TRUE)

assert('format_sci() uses correct number of significant digits', {
  (format_sci(1) %==% '1')
  (format_sci(0) %==% '0')
  (format_sci(3.1415e2) %==% '314')
  (format_sci(3.1415) %==% '3.14')
})

options(op)

op = options(digits = 14, scipen = 0, knitr.digits.signif = TRUE)
assert('format_sci() prints numerics at maximum number of significant digits', {
  (format_sci(3.14159265358979) %==% '3.1415926535898')
})
options(op)

op = options(digits = 4, scipen = 0)

assert('format_sci() turns numbers into scientific notations', {
  (format_sci(c(1.84e8, 1e5, 2.34e3)) %==% c('1.84\\times 10^{8}', '10^{5}', '2340'))
  (format_sci(1.23456789 * 10^-5) %==% '1.2346\\times 10^{-5}')
  (format_sci(10^-5) %==% '10^{-5}')
  (format_sci(10^5) %==% '10^{5}')
  (format_sci(-10^-5) %==% '-10^{-5}')
  (format_sci(-10^5) %==% '-10^{5}')
  (format_sci(9.87654e6, 'html') %==% '9.8765 &times; 10<sup>6</sup>')
  (format_sci(9.87654e6, 'rst') %==% '9.8765 |times| 10 :sup:`6`')
  (format_sci(letters) %==% letters)
  (format_sci(NA_real_) %==% NA_character_)
})

assert('format_sci() coerces non-numeric and non-double values to characters', {
  (format_sci(Sys.Date()) == as.character(Sys.Date()))
  (format_sci(1000000L) == '1000000')
})

# https://github.com/yihui/knitr/issues/1625
assert('format_sci() does not convert roman numerals to arabic numerals', {
  format_sci(as.roman(c(1, 4, 7, 33, 100))) %==% c('I', 'IV', 'VII', 'XXXIII', 'C')
})

assert('format_sci() for Rnw does not add \\ensuremath{} at all', {
  (!grepl('[\\]ensuremath', format_sci(c(1e4, 1.2345e10, 2 * pnorm(-(3:4)), -Inf))))
})

assert('the inline hook for Rnw applies \\ensuremath{} correctly', {
  (.inline.hook.tex(1e4) == '\\ensuremath{10^{4}}')
  (.inline.hook.tex(-Inf) == '\\ensuremath{-\\infty{}}')
  (.inline.hook.tex(-1.23) == '-1.23')
  (.inline.hook.tex(c(1.2345e10, 2 * pnorm(-(3:4)))) ==
    "\\ensuremath{1.2345\\times 10^{10}}, 0.0027, \\ensuremath{6.3342\\times 10^{-5}}")
})

assert('Infinity and NaN are formatted correctly', {
  (format_sci(-Inf) %==% '-\\infty{}')
  (format_sci(-Inf, 'html') %==% '-&infin;')
  (format_sci(-Inf, 'rst') %==% '-Inf')
  (format_sci(NaN) %==% 'NaN')
})

assert('sanitize_fn() warns against spaces in filenames', {
  (has_warning(sanitize_fn('figure/a b')))
})

options(op)

assert('fig_path() sanitizes paths', {
  (sanitize_fn('fig/foo') %==% 'fig/foo')
  (sanitize_fn('figure/a b  c', FALSE) %==% 'figure/a_b__c')
  (sanitize_fn('fig space/a.b') %==% 'fig_space/a.b')
  (sanitize_fn('../c.d') %==% '../c.d')
  (sanitize_fn('./../c..d') %==% './../c..d')
  (sanitize_fn('C:/foo/bar') %==% 'C:/foo/bar')
})

assert('fig_chunk() generates figure filenames for a code chunk', {
  (fig_chunk('foo') %==% 'figure/foo-1')
  (fig_chunk('foo', 'pdf') %==% 'figure/foo-1.pdf')
  (fig_chunk('foo', 'png', 2) %==% 'figure/foo-2.png')
  (fig_chunk('foo', 'svg', 1:5) %==% sprintf('figure/foo-%d.svg', 1:5))
  (fig_chunk('foo', fig.path = 'my_figure/') %==% 'my_figure/foo-1')
  (fig_chunk('foo', '.pdf') %==%'figure/foo-1.pdf')
})

assert('all_figs() generates all figure paths for a code chunk', {
  opts = list(fig.path = 'abc/', label = 'foo', fig.num = 3)
  (all_figs(opts, '.svg') %==% sprintf('abc/foo-%d.svg', 1:3))
  (all_figs(opts, c('png', 'pdf'))  %==% apply(
    expand.grid(1:3, c('.png', '.pdf')), 1, function(x) {
      paste0(c('abc/foo-', x), collapse = '')
    }
  ))
})

assert('escape_latex() escapes special LaTeX characters', {
  (escape_latex('# $ % & ~ _ ^ \\ { }') %==%
     '\\# \\$ \\% \\& \\textasciitilde{} \\_ \\textasciicircum{} \\textbackslash{} \\{ \\}'
  )
  (escape_latex('a b', spaces = TRUE) %==% 'a b')
  (escape_latex('a   b', spaces = TRUE) %==% 'a \\ \\ b')
})

assert('indent_block() works when the first element is empty (#790)', {
  (indent_block(c('', 'a')) %==% c('    ', '    a'))
  (indent_block(c('', '')) %==% c('    ', '    '))
})

assert('current_input() returns NULL by default', {
  (is.null(current_input()))
  (suppressWarnings(is.null(current_input(TRUE))))
})

assert('color_def() generates LaTeX code to define a color variable', {
  (color_def(NA) %==% '')
  (color_def('red') %==% '\\definecolor{shadecolor}{rgb}{1, 0, 0}')
  (color_def('#00ff00') %==% '\\definecolor{shadecolor}{rgb}{0, 1, 0}')
  (color_def('.5,.6,.7', 'fgcolor') %==% '\\definecolor{fgcolor}{rgb}{.5, .6, .7}')
})

cw = function(...) unclass(combine_words(...))
assert('combine_words() combines multiple words into a single string', {
  (cw(NULL) %==% NULL)
  (cw(c('a')) %==% 'a')
  (cw(c('a', 'b')) %==% 'a and b')
  (cw(c('a', 'b'), and = "") %==% 'a, b')
  (cw(c('a', 'b', 'c')) %==% 'a, b, and c')
  (cw(c('a', 'b', 'c'), and = '') %==% 'a, b, c')
  (cw(c('a', 'b', 'c'), ' / ', '') %==% 'a / b / c')
  (cw(c('a', 'b', 'c'), before = '"') %==% '"a", "b", and "c"')
  (cw(c('a', 'b', 'c'), before = '``', after = "''") %==% "``a'', ``b'', and ``c''")
  (cw(c('a', 'b', 'c'), before = '``', after = "''", oxford_comma = FALSE) %==% "``a'', ``b'' and ``c''")
})
rm(list = 'cw')

opts = list(
  fig.cap = 'Figure "caption" <>.', fig.lp = 'Fig:', label = 'foo'
)
assert('.img.cap() generates the figure caption and alt attribute', {
  (.img.cap(list(fig.cap = NULL), FALSE) %==% "")
  (.img.cap(opts, FALSE) %==% opts$fig.cap)
  (.img.cap(opts, TRUE, TRUE)  %==% 'Figure &quot;caption&quot; &lt;&gt;.')

  opts$fig.alt = 'Figure "alternative text" <>.'

  (.img.cap(opts, TRUE, TRUE)  %==% 'Figure &quot;alternative text&quot; &lt;&gt;.')
  (.img.cap(opts, FALSE)  %==% opts$fig.cap)

  (.img.cap(list(fig.cap = '', fig.alt = "alt"), FALSE) %==% "")
  (.img.cap(list(fig.cap = '', fig.alt = "alt"), TRUE) %==% "alt")
})

z = as.strict_list(list(a = 1, aa = 2, bbb = 3))
assert('as.strict_list() does not allow partial matching', {
  (z$b %==% NULL)
  (z$bbb %==% 3)
})

out = c('*hello*', raw_output('<special>content</special> *protect* me!'), '*world*')
pre = extract_raw_output(out)
pre$value = gsub('[*]([^*]+)[*]', '<em>\\1</em>', pre$value)  # think this as Pandoc conversion
# raw output was protected from the conversion (e.g. *protect* was not converted)
assert('restore_raw_output() restores raw output', {
  (restore_raw_output(pre$value, pre$chunks) %==%
    '<em>hello</em>\n<special>content</special> *protect* me!\n<em>world</em>')
})

assert('raw_block() returns a raw attribute block for Pandoc', {
  (raw_latex('\\emph{x}') == '\n```{=latex}\n\\emph{x}\n```\n')
  (raw_html('<i>foo</i>') == '\n```{=html}\n<i>foo</i>\n```\n')
})

assert('block_class() turns a character vector into Pandoc attributes for code block classes', {
  (block_class(NULL) %==% NULL)
  (block_class('a') %==% '.a')
  (block_class('a b') %==% c('.a', '.b'))
  (block_class(c('a', 'b')) %==% c('.a', '.b'))
})

assert('block_attr(x) turns a character vector into Pandoc attributes', {
  (block_attr(NULL) %==% NULL)
  (block_attr(NULL, lang = 'r') %==% 'r')
  (block_attr('.a') %==% '{.a}')
  (block_attr('.a b="11"') %==% '{.a b="11"}')
  (block_attr(c('.a', 'b="11"')) %==% '{.a b="11"}')
})


assert('when collapse is TRUE, class.* and attr.* become NULL except for class.source and attr.source', {
  keys = unlist(lapply(
    c('class.', 'attr.'), paste0, c('source', 'output', 'message', 'warning', 'error')
  ))
  keys_source = c('class.source', 'attr.source')
  opts = fix_options(opts_chunk$merge(c(setNames(as.list(keys), keys), list(collapse = TRUE))))
  (opts[keys_source] %==% as.list(setNames(keys_source, keys_source)))
  (!any(names(opts) %in% setdiff(keys, keys_source)))
  rm(keys, keys_source, opts)
})

assert('default strip.white is conditional to collapse', {
  opts = opts_chunk$get(default = TRUE)
  (fix_options(opts)$strip.white %==% TRUE)
  opts$collapse = TRUE
  (fix_options(opts)$strip.white %==% FALSE)
  opts$strip.white = TRUE
  (fix_options(opts)$strip.white %==% TRUE)
  rm(opts)
})

assert('pandoc_to gets the current Pandoc format', {
  opts = opts_knit$get(c('rmarkdown.pandoc.to', 'rmarkdown.pandoc.from'))
  opts_knit$delete(c("rmarkdown.pandoc.from", "rmarkdown.pandoc.to"))
  (pandoc_from() %==% 'markdown')
  (pandoc_to() %==% NULL)
  opts_knit$set(rmarkdown.pandoc.from = 'gfm', rmarkdown.pandoc.to = 'docx')
  (pandoc_from() %==% 'gfm')
  (pandoc_to() %==% 'docx')
  (pandoc_to('docx'))
  (!pandoc_to('pptx'))
  (pandoc_to(c('docx', 'pptx')))
  opts_knit$set(opts)
})

assert('comment_out() add prefix and newlines if asked', {
  (comment_out("a") %==% "## a\n")
  (comment_out("ab cd") %==% "## ab cd\n")
  (comment_out("ab cd\n") %==% "## ab cd\n")
  (comment_out("") %==% "## \n")
  (comment_out("\n") %==% "## \n")
  (comment_out(c("a", "b")) %==% c("## a\n", "## b\n"))
  (comment_out(c("a", "b"), which = 2) %==% c("a\n", "## b\n"))
  (comment_out("a", newline = FALSE) %==% "## a")
  (comment_out("a", prefix = "$") %==% "$ a\n")
  (comment_out("a", prefix = NULL) %==% "a\n")
})

assert('remove_urls() removes the link', {
  (remove_urls(c('[a](b)', '[a b](c)')) %==% c('a', 'a b'))
  (remove_urls('a [b](c) d.') %==% 'a b d.')
  (remove_urls('a [b](c) d [e f+g](h) i.') %==% 'a b d e f+g i.')
  (remove_urls('a [b](c) `[d](e)` f.') %==% 'a b `[d](e)` f.')
})

assert('options using `-` are converted to `.` and default value replaced', {
  opts = opts_chunk$merge(list('fig-cap' = 'caption', 'out-width' = 300))
  (is.null(dot_names(opts)[['fig-cap']]))
  (is.null(dot_names(opts)[['out-width']]))
  (dot_names(opts)[['fig.cap']] %==% 'caption')
  (dot_names(opts)[['out.width']] %==% 300)
  rm(opts)
})

assert('fig.format and fig.dpi', {
  opts = opts_chunk$merge(list('fig-format' = 'svg', 'fig-dpi' = 750))
  (is.null(dot_names(opts)[['fig-format']]))
  (is.null(dot_names(opts)[['fig-dpi']]))
  (dot_names(opts)[['dev']] %==% 'svg')
  (dot_names(opts)[['dpi']] %==% 750)
  rm(opts)
})

assert('options using `.` are converted to `-` and default value replaced', {
  opts = opts_chunk$merge(list('fig.cap' = 'caption', 'out.width' = 300))
  (is.null(dash_names(opts)[['fig.cap']]))
  (is.null(dash_names(opts)[['out.width']]))
  (dash_names(opts)[['fig-cap']] %==% 'caption')
  (dash_names(opts)[['out-width']] %==% 300)
  rm(opts)
})

assert('dev and dpi are convert to fig-format and fig-dpi', {
  opts = opts_chunk$merge(list('dev' = 'svg', 'dpi' = 750))
  (is.null(dash_names(opts)[['dev']]))
  (is.null(dash_names(opts)[['dpi']]))
  (dash_names(opts)[['fig-format']] %==% 'svg')
  (dash_names(opts)[['fig-dpi']] %==% 750)
  rm(opts)
})

library(testit)

assert(
  'parse_params() parses chunk options to a list',
  identical(parse_params('a-s-d,b=TRUE,c=def'), alist(label = 'a-s-d', b = TRUE, c = def)),
  has_error(parse_params('a,b')),
  has_error(parse_params('a,b,c=qwer')),
  identical(parse_params('a,opt=c(1,3,5)'), alist(label = 'a', opt = c(1, 3, 5))),
  identical(parse_params('label="xx",opt=zz'), alist(label = 'xx', opt = zz)),
  identical(parse_params('label=foo'), alist(label = 'foo')),
  identical(parse_params('a,b=2,c="qwer",asdf="efg"'),
            alist(label = 'a', b = 2, c = 'qwer', asdf = 'efg')),
  identical(parse_params('2a'), alist(label = '2a')),
  identical(parse_params('abc-function,fig.path="foo/bar-"'),
            alist(label = 'abc-function', fig.path = "foo/bar-"))
)

opts_knit$set(out.format = 'markdown')
assert(
  'parse_params() parses the language engine from ```{lang}',
  identical(
    parse_block(NULL, '', 'r, foo, a=1,')$params,
    alist(label = 'foo', a = 1)
  ),
  identical(
    parse_block(NULL, '', 'Rcpp, foo, a=1,')$params,
    alist(label = 'foo', a = 1, engine = 'Rcpp')
  )
)

res = split_file(
  c('abc', '```{r foo}', '1+1', '```{r bar}', '2+2', '```', 'def'),
  patterns = all_patterns$md
)
assert(
  'split_file() treats ``` as part of code chunk instead of beginning of text chunk',
  # the foo chunk does not have a closing mark
  knit_code$get('foo') == '1+1',
  knit_code$get('bar') == '2+2',
  # before knitr v1.6, the text chunk was c('', 'def')
  identical(res[[4]][['input']], 'def')
)
opts_knit$restore()
knit_code$restore(); knit_concord$restore()

res = parse_inline(c('aaa \\Sexpr{x}', 'bbb \\Sexpr{NA} and \\Sexpr{1+2}',
                     'another expression \\Sexpr{rnorm(10)}'), all_patterns$rnw)
assert(
  'parse_inline() parses inline text',
  identical(res$code, c('x', 'NA', '1+2', 'rnorm(10)')),
  identical(nchar(res$input), 81L),
  # empty inline code is not recognized
  identical(parse_inline('\\Sexpr{}', all_patterns$rnw)$code, character(0)),
  # can use > in HTML inline code
  identical(parse_inline('<!--rinline "<a>" -->', all_patterns$html)$code, ' "<a>" ')
)

res = parse_inline('inline expressions `r pi+1`, +r cos(0)+ in AsciiDoc',
                   all_patterns$asciidoc)
assert(
  'both `r expression` and +r expression+ work for AsciiDoc',
  identical(res$code, c('pi+1', 'cos(0)'))
)

knit_code$restore()

read_chunk(lines = c('1+1'))
assert(
  'read_chunk() does not discard code without chunk headers',
  identical(knit_code$get(), list('unnamed-chunk-1' = '1+1'))
)

knit_code$restore()

read_chunk(lines = c('# ---- foo ----', '1+1'))
assert(
  'read_chunk() can identify chunk labels',
   identical(knit_code$get(), list(foo = '1+1'))
)

knit_code$restore()

# chunk references with <<>> --------------------------------------------------

knit_code$restore(list(
  a = '1+1', b = '2-2', c = c('if (T)', '  <<a>>'), d = c('function() {', '  <<c>>', '}')
))
pc = function(x) parse_chunk(x, all_patterns$rnw$ref.chunk)

assert(
  'parse_chunk() preserves indentation',
  identical(pc(c('3*3', '<<a>>', ' <<b>>', 'if (T)', '  <<a>>')),
            c("3*3", "1+1", " 2-2", "if (T)", "  1+1" )),
  identical(pc('<<d>>'), c('function() {', '  if (T)', '    1+1', '}'))
)

knit_code$restore()

# graphics devices in base R, plus those in Cairo, cairoDevice, tikzDevice
auto_exts = c(
  bmp = 'bmp', postscript = 'eps', pdf = 'pdf', png = 'png', svg = 'svg',
  jpeg = 'jpeg', pictex = 'tex', tiff = 'tiff', win.metafile = 'wmf',
  cairo_pdf = 'pdf', cairo_ps = 'eps',

  quartz_pdf = 'pdf', quartz_png = 'png', quartz_jpeg = 'jpeg',
  quartz_tiff = 'tiff', quartz_gif = 'gif', quartz_psd = 'psd',
  quartz_bmp = 'bmp',

  CairoJPEG = 'jpeg', CairoPNG = 'png', CairoPS = 'eps', CairoPDF = 'pdf',
  CairoSVG = 'svg', CairoTIFF = 'tiff',

  Cairo_pdf = 'pdf', Cairo_png = 'png', Cairo_ps = 'eps', Cairo_svg = 'svg',

  svglite = 'svg',

  tikz = 'tex'
)

dev2ext = function(x) {
  res = auto_exts[x]
  if (any(idx <- is.na(res))) {
    for (i in x[idx]) check_dev(i)
    stop2(
      'cannot find appropriate filename extensions for device ', x[idx], '; ',
      "please use chunk option 'fig.ext' (https://yihui.org/knitr/options)"
    )
  }
  unname(res)
}

check_dev = function(dev) {
  if (exists(dev, mode = 'function', envir = knit_global()))
    get(dev, mode = 'function', envir = knit_global()) else
      stop('the graphical device', sQuote(dev), 'does not exist (as a function)')
}

# quartiz devices under Mac
quartz_dev = function(type, dpi) {
  force(type); force(dpi)
  function(file, width, height, ...) {
    grDevices::quartz(file = file, width = width, height = height, type = type, dpi = dpi, ...)
  }
}

# a wrapper of the tikzDevice::tikz device
tikz_dev = function(...) {
  loadNamespace('tikzDevice')
  packages = switch(
    getOption('tikzDefaultEngine'),
    pdftex = getOption('tikzLatexPackages'),
    xetex = getOption('tikzXelatexPackages'),
    luatex = getOption('tikzLualatexPackages')
  )
  tikzDevice::tikz(..., packages = c('\n\\nonstopmode\n', packages, .knitEnv$tikzPackages))
}

# save a recorded plot
save_plot = function(plot, name, dev, width, height, ext, dpi, options) {

  path = paste(name, ext, sep = '.')
  # when cache=2 and plot file exists, just return the filename
  if (options$cache == 2 && cache$exists(options$hash, options$cache.lazy)) {
    if (in_base_dir(!file.exists(path))) {
      purge_cache(options)
      stop('cannot find ', path, '; the cache has been purged; please re-compile')
    }
    return(paste(name, if (dev == 'tikz' && options$external) 'pdf' else ext, sep = '.'))
  }

  # built-in devices
  device = switch(
    dev,
    bmp = function(...) bmp(...,  res = dpi, units = 'in'),
    postscript = function(...) {
      postscript(..., onefile = FALSE, horizontal = FALSE, paper = 'special')
    },
    jpeg = function(...) jpeg(..., res = dpi, units = 'in'),
    pdf = grDevices::pdf,
    png = function(...) png(..., res = dpi, units = 'in'),
    svg = grDevices::svg,
    pictex = grDevices::pictex,
    tiff = function(...) tiff(..., res = dpi, units = 'in'),
    win.metafile = grDevices::win.metafile,
    cairo_pdf = grDevices::cairo_pdf,
    cairo_ps = grDevices::cairo_ps,

    quartz_pdf = quartz_dev('pdf', dpi),
    quartz_png = quartz_dev('png', dpi),
    quartz_jpeg = quartz_dev('jpeg', dpi),
    quartz_tiff = quartz_dev('tiff', dpi),
    quartz_gif = quartz_dev('gif', dpi),
    quartz_psd = quartz_dev('psd', dpi),
    quartz_bmp = quartz_dev('bmp', dpi),

    CairoJPEG = load_device('CairoJPEG', 'Cairo', dpi = dpi),
    CairoPNG = load_device('CairoPNG', 'Cairo', dpi = dpi),
    CairoTIFF = load_device('CairoTIFF', 'Cairo', dpi = dpi),
    CairoPS = load_device('CairoPS', 'Cairo'),
    CairoPDF = load_device('CairoPDF', 'Cairo'),
    CairoSVG = load_device('CairoSVG', 'Cairo'),

    Cairo_pdf = load_device('Cairo_pdf', 'cairoDevice'),
    Cairo_png = load_device('Cairo_png', 'cairoDevice'),
    Cairo_ps = load_device('Cairo_ps', 'cairoDevice'),
    Cairo_svg = load_device('Cairo_svg', 'cairoDevice'),

    svglite = load_device('svglite', 'svglite'),

    tikz = function(...) {
      tikz_dev(..., sanitize = options$sanitize, standAlone = options$external)
    },

    check_dev(dev)
  )
  in_base_dir(plot2dev(plot, name, dev, device, path, width, height, options))
}

plot2dev = function(plot, name, dev, device, path, width, height, options) {
  dargs = get_dargs(options$dev.args, dev)
  # re-plot the recorded plot to an off-screen device
  do.call(device, c(list(path, width = width, height = height), dargs))
  showtext(options$fig.showtext)  # showtext support
  print(plot)
  dev.off()

  # Cairo::CairoPS always adds the extension .ps, even if you have specified an
  # extension like .eps (https://github.com/yihui/knitr/issues/1364)
  if (dev == 'CairoPS') {
    path2 = paste0(path, '.ps')
    if (file.exists(path2)) file.rename(path2, path)
  }

  # compile tikz to pdf
  if (dev == 'tikz') {
    patch_tikz_tex(path)
    if (options$external) path = tinytex::latexmk(path, getOption('tikzDefaultEngine'))
  }

  fig_process(options$fig.process, path, options)
}

# the tikz plot may contain raster legends, in which case we need to adjust the
# paths of these PNG files: https://stackoverflow.com/a/58410965/559676
patch_tikz_tex = function(path) {
  d = dirname(path)
  p = list.files(d, r <- '_ras[0-9]+[.]png$')
  b = sans_ext(basename(path)); bs = sub(r, '', p)
  # for foo.tex, there must be foo_ras[N].png, otherwise no need to patch foo.tex
  if (!(b %in% bs)) return()
  x = read_utf8(path)
  r = '^(\\s*\\\\pgfimage\\[.+?]\\{)(.+?_ras[0-9]+}};\\s*)$'
  i = grep(r, x)
  i = i[grep(b, x[i], fixed = TRUE)]
  x1 = sub(r, '\\1', x[i])
  x2 = sub(r, '\\2', x[i])
  if (is_windows()) d = gsub('\\\\', '/', d)
  x[i] = paste0(x1, d, '/', x2)
  write_utf8(x, path)
}

# filter the dev.args option
get_dargs = function(dargs, dev) {
  if (length(dargs) == 0) return()
  if (is.list(dargs) && all(sapply(dargs, is.list))) {
    # dev.args is list(dev1 = list(arg1 = val1, ...), dev2 = list(arg2, ...))
    dargs = dargs[[dev]]
  }
  dargs
}

# this is mainly for Cairo and cairoDevice
load_device = function(name, package, dpi = NULL) {
  dev = getFromNamespace(name, package)
  # dpi is for bitmap devices; units must be inches!
  if (is.null(dpi)) dev else function(...) dev(..., dpi = dpi, units = 'in')
}


# merge low-level plotting changes
merge_low_plot = function(x, idx = sapply(x, evaluate::is.recordedplot)) {
  idx = which(idx); n = length(idx); m = NULL # store indices that will be removed
  if (n <= 1) return(x)
  i1 = idx[1]; i2 = idx[2]  # compare plots sequentially
  for (i in 1:(n - 1)) {
    # remove the previous plot and move its index to the next plot
    if (is_low_change(x[[i1]], x[[i2]])) m = c(m, i1)
    i1 = idx[i + 1]
    i2 = idx[i + 2]
  }
  if (is.null(m)) x else x[-m]
}

# compare two recorded plots
is_low_change = function(p1, p2) {
  p1 = p1[[1]]; p2 = p2[[1]]  # real plot info is in [[1]]
  if (length(p2) < (n1 <- length(p1))) return(FALSE)  # length must increase
  identical(p1[1:n1], p2[1:n1])
}

# recycle some plot options such as fig.cap, out.width/height, etc when there
# are multiple plots per chunk
.recyle.opts = c('fig.cap', 'fig.scap', 'fig.env', 'fig.pos', 'fig.subcap',
                 'out.width', 'out.height', 'out.extra', 'fig.link')

# when passing options to plot hooks, reduce the recycled options to scalars
reduce_plot_opts = function(options) {
  i = options$fig.cur %n% 1L
  for (o in .recyle.opts) {
    v = options[[o]]
    if ((n <- length(v)) == 0) next
    if ((j <- i %% n) == 0) j = n
    options[o] = list(v[j])
  }
  options
}

# the memory address of a NativeSymbolInfo object will be lost if it is saved to
# disk; see http://markmail.org/message/zat2r2pfsvhrsfqz for the full
# discussion; the hack below was stolen (with permission) from RStudio:
# https://github.com/rstudio/rstudio/blob/master/src/cpp/r/R/Tools.R
fix_recordedPlot = function(plot) {
  # restore native symbols for R >= 3.0
  for (i in seq_along(plot[[1]])) {
    # get the symbol then test if it's a native symbol
    symbol = plot[[1]][[i]][[2]][[1]]
    if (inherits(symbol, 'NativeSymbolInfo')) {
      # determine the dll that the symbol lives in
      name = symbol[[if (is.null(symbol$package)) 'dll' else 'package']][['name']]
      pkgDLL = getLoadedDLLs()[[name]]
      # reconstruct the native symbol and assign it into the plot
      nativeSymbol = getNativeSymbolInfo(
        name = symbol$name, PACKAGE = pkgDLL, withRegistrationInfo = TRUE
      )
      plot[[1]][[i]][[2]][[1]] <- nativeSymbol
    }
  }
  attr(plot, 'pid') = Sys.getpid()
  plot
}

# fix plots in evaluate() results
fix_evaluate = function(list, fix = TRUE) {
  if (!fix) return(list)
  lapply(list, function(x) {
    if (evaluate::is.recordedplot(x)) fix_recordedPlot(x) else x
  })
}

# remove the plots from the evaluate results for the case of cache=2; if we only
# want to keep high-level plots, we need MD5 digests of the plot components so
# that we will be able to filter out low-level changes later
remove_plot = function(list, keep.high = TRUE) {
  lapply(list, function(x) {
    if (evaluate::is.recordedplot(x)) structure(
      if (keep.high) digest_plot(x) else NULL, class = 'recordedplot'
    ) else x
  })
}
# replace the content of the recorded plot with MD5 digests so that merge_plot()
# will still work, and this will also save disk space for the case of cache=2
digest_plot = function(x, level = 1) {
  if (!is.list(x) || level >= 3) return(digest(x))
  lapply(x, digest_plot, level = level + 1)
}

# a null device
pdf_null = function(width = 7, height = 7, ...) {
  grDevices::pdf(NULL, width, height, ...)
}

fig_process = function(FUN, path, options) {
  if (is.function(FUN)) {
    ARG = intersect(c("options", names(options)), names(formals(FUN)))
    path2 = do.call(FUN, c(path, c(options = list(options), options)[ARG]))
    if (!is.character(path2) || length(path2) != 1L)
      stop("'fig.process' must be a function that returns a character string")
    path = path2
  }
  path
}

#' Crop a plot (remove the edges) using PDFCrop or ImageMagick
#'
#' The command \command{pdfcrop} (often shipped with a LaTeX distribution) is
#' executed on a PDF plot file, and \code{magick::\link[magick]{image_trim}()}
#' is executed for other types of plot files.
#' @param x Filename of the plot.
#' @param quiet Whether to suppress standard output from the command.
#' @export
#' @references PDFCrop: \url{https://www.ctan.org/pkg/pdfcrop}. If you use
#'   TinyTeX, you may install \command{pdfcrop} with
#'   \code{tinytex::tlmgr_install('pdfcrop')}.
#' @return The original filename.
plot_crop = function(x, quiet = TRUE) {
  is_pdf = grepl('[.]pdf$', x, ignore.case = TRUE)
  x2 = x
  x = path.expand(x)
  if (is_pdf && !has_utility('pdfcrop')) return(x2)

  if (!quiet) message('cropping ', x)
  if (is_pdf) {
    system2('pdfcrop', shQuote(c(x, x)), stdout = if (quiet) FALSE else "")
  } else if (loadable('magick')) {
    img = magick::image_read(x)
    magick::image_write(magick::image_trim(img), x)
  } else message(
    'The magick package is required to crop "', x2, '" but not available.'
  )
  x2
}

showtext = function(show) if (isTRUE(show)) showtext::showtext_begin()

# handle some special cases of par()
par2 = function(x) {
  if (length(x) == 0) return()
  # this may not be correct, but there is no way to tell if the user set mfrow
  # or mfcol in par() (either setting will change mfrow/mfcol simultaneously),
  # and I just assume it was mfrow
  if (!is.null(x$mfrow)) {
    # do this before the rest of pars because setting mfrow/mfcol will reset cex
    par(mfrow = x$mfrow)
    x$mfrow = x$mfcol = NULL
    # do not restore mfg (#1215); this is unlikely to make sense, e.g. for
    # par(mfrow = c(1, 2)), mfg will be c(1, 2, 1, 2) when the second plot is
    # completed, but when the user generate a new plot, it should typically be
    # drawn at (1, 1) instead of (1, 2)
    x$mfg = NULL
  }
  # you are unlikely to want to reset these pars
  x$fig = x$fin = x$pin = x$plt = x$usr = NULL
  x$ask = NULL  # does not make sense for typical non-interactive R sessions
  par(x)
}

#' Embed external images in \pkg{knitr} documents
#'
#' When plots are not generated from R code, there is no way for \pkg{knitr} to
#' capture plots automatically. In this case, you may generate the images
#' manually and pass their file paths to this function to include them in the
#' output. The major advantage of using this function is that it is portable in
#' the sense that it works for all document formats that \pkg{knitr} supports,
#' so you do not need to think if you have to use, for example, LaTeX or
#' Markdown syntax, to embed an external image. Chunk options related to
#' graphics output that work for normal R plots also work for these images, such
#' as \code{out.width} and \code{out.height}.
#' @param path A character vector of image paths.
#' @param auto_pdf Whether to use PDF images automatically when the output
#'   format is LaTeX. If \code{TRUE}, then e.g. \file{foo/bar.png} will be
#'   replaced by \file{foo/bar.pdf} if the latter exists. This can be useful
#'   since normally PDF images are of higher quality than raster images like
#'   PNG, when the output is LaTeX/PDF.
#' @param dpi DPI (dots per inch) value. Used to calculate the output width (in
#'   inches) of the images. This will be their actual width in pixels, divided
#'   by \code{dpi}. If not provided, the chunk option \code{dpi} is used; if
#'   \code{NA}, the output width will not be calculated.
#' @param error Whether to signal an error if any files specified in the
#'   \code{path} argument do not exist and are not web resources.
#' @note This function is supposed to be used in R code chunks or inline R code
#'   expressions. You are recommended to use forward slashes (\verb{/}) as path
#'   separators instead of backslashes in the image paths.
#'
#'   The automatic calculation of the output width requires the \pkg{png}
#'   package (for PNG images) or the \pkg{jpeg} package (for JPEG images). The
#'   width will not be calculated if the chunk option \code{out.width} is
#'   already provided or \code{dpi = NA}.
#' @return The same as the input character vector \code{path} but it is marked
#'   with special internal S3 classes so that \pkg{knitr} will convert the file
#'   paths to proper output code according to the output format.
#' @export
include_graphics = function(
  path, auto_pdf = getOption('knitr.graphics.auto_pdf', FALSE), dpi = NULL,
  error = TRUE
) {
  path = native_encode(path)  # https://d.cosx.org/d/420524
  if (auto_pdf && is_latex_output()) {
    path2 = with_ext(path, 'pdf')
    i = file.exists(path2)
    path[i] = path2[i]
  }
  if (error && length(p <- path[!is_web_path(path) & !file.exists(path)])) stop(
    'Cannot find the file(s): ', paste0('"', p, '"', collapse = '; ')
  )
  structure(path, class = c('knit_image_paths', 'knit_asis'), dpi = dpi)
}

# calculate the width in inches for PNG/JPEG images given a DPI
raster_dpi_width = function(path, dpi) {
  if (!file.exists(path) || is.na(dpi)) return()
  is_png = grepl('[.]png$', path, ignore.case = TRUE)
  is_jpg = grepl('[.]jpe?g$', path, ignore.case = TRUE)
  if (!is_png && !is_jpg) return()
  if (is_png) {
    if (!loadable('png')) return()
    meta = attr(png::readPNG(path, native = TRUE, info = TRUE), 'info')
    w = meta$dim[1]
    if (!is.numeric(dpi)) dpi = meta$dpi[1]
    if (!is.numeric(dpi)) return()  # cannot calculate the desired width
  } else if (is_jpg) {
    if (!loadable('jpeg')) return()
    if (!is.numeric(dpi)) return()  # there is no dpi info in JPEG
    w = ncol(jpeg::readJPEG(path, native = TRUE))
  }
  if (is_latex_output()) {
    paste0(round(w / dpi, 2), 'in')
  } else if (is_html_output()) {
    round(w / (dpi / 96))
  }
}

#' Embed a URL as an HTML iframe or a screenshot in \pkg{knitr} documents
#'
#' When the output format is HTML, \code{include_url()} inserts an iframe in the
#' output; otherwise it takes a screenshot of the URL and insert the image in
#' the output. \code{include_app()} takes the URL of a Shiny app and adds
#' \samp{?showcase=0} to it (to disable the showcase mode), then passes the URL
#' to \code{include_url()}.
#' @param url Character string containing a URL.
#' @param height Character string with the height of the iframe.
#' @return An R object with a special class that \pkg{knitr} recognizes
#'   internally to generate the iframe or screenshot.
#' @seealso \code{\link{include_graphics}}
#' @export
include_url = function(url, height = '400px') {
  include_url2(url, height)
}

include_url2 = function(url, height = '400px', orig = url) {
  structure(
    list(url = url, height = height, url.orig = orig),
    class = c('knit_embed_url', 'knit_asis')
  )
}

#' @rdname include_url
#' @export
include_app = function(url, height = '400px') {
  orig = url  # store the original URL
  if (!grepl('?', url, fixed = TRUE)) url = paste0(url, '?showcase=0')
  include_url2(url, height, orig)
}

need_screenshot = function(x, ...) {
  options = list(...)[['options']]
  # user may say 'I know the consequence; just let me render HTML'
  if (isFALSE(options$screenshot.force)) return(FALSE)
  # force screenshotting even if the output format support HTML
  force = is.list(options) && isTRUE(options$screenshot.force)
  fmt = pandoc_to()
  i1 = inherits(x, 'htmlwidget')
  i2 = inherits(x, 'shiny.appobj')
  i3 = inherits(x, 'knit_embed_url')
  # not R Markdown v2, always screenshot htmlwidgets and shiny apps
  if (length(fmt) == 0 || force) return(i1 || i2 || i3)
  html_format = fmt %in% c('html', 'html4', 'html5', 'revealjs', 's5', 'slideous', 'slidy')
  res = ((i1 || i3) && !html_format) || (i2 && !(html_format && runtime_shiny()))
  res && webshot_available()
}

runtime_shiny = function() {
  identical(opts_knit$get('rmarkdown.runtime'), 'shiny')
}

webshot_available = local({
  res = NULL  # cache the availablity of webshot/PhantomJS
  function() {
    if (is.null(res))
      res <<- loadable('webshot') && !is.null(getFromNamespace('find_phantom', 'webshot')())
    res
  }
})

html_screenshot = function(x, options = opts_current$get(), ...) {
  i1 = inherits(x, 'htmlwidget')
  i2 = inherits(x, 'shiny.appobj')
  i3 = inherits(x, 'knit_embed_url')
  if (!(i1 || i2 || i3))
    stop('Screenshotting for the class ', class(x)[1], ' is not supported.')

  # if user has specified the screenshot image, just use it
  if (!is.null(shots <- options$screenshot.alt)) {
    i = shot_counter()
    if (length(shots) < i) stop('Not enough number of screenshots provided')
    return(structure(list(file = shots[i]), class = 'html_screenshot'))
  }

  ext = if (length(options$dev)) {
    switch(options$dev[1], pdf = '.pdf', jpeg = '.jpeg', '.png')
  } else '.png'
  wargs = options$screenshot.opts %n% list()
  if (is.null(wargs$vwidth)) wargs$vwidth = options$out.width.px
  if (is.null(wargs$vheight)) wargs$vheight = options$out.height.px
  if (is.null(wargs$delay)) wargs$delay = if (i1) 0.2 else 1
  d = tempfile()
  dir.create(d); on.exit(unlink(d, recursive = TRUE), add = TRUE)
  f = in_dir(d, {
    if (i1 || i3) {
      if (i1) {
        f1 = wd_tempfile('widget', '.html')
        save_widget(x, f1, FALSE, options = options)
      } else f1 = x$url
      f2 = wd_tempfile('webshot', ext)
      do.call(webshot::webshot, c(list(f1, f2), wargs))
      normalizePath(f2)
    } else if (i2) {
      f = wd_tempfile('webshot', ext)
      do.call(webshot::appshot, c(list(x, f), wargs))
      normalizePath(f)
    }
  })
  res = readBin(f, 'raw', file.info(f)[, 'size'])
  structure(
    list(image = res, extension = ext, url = if (i3) x$url.orig),
    class = 'html_screenshot'
  )
}

save_widget = function(..., options) {
  FUN = htmlwidgets::saveWidget
  if ('knitrOptions' %in% names(formals(FUN))) {
    FUN(..., knitrOptions = options)
  } else FUN(...)
}

#' Extract knit parameters from a document
#'
#' This function reads the YAML front-matter section of a document and returns a
#' list of any parameters declared there. This function exists primarily to
#' support the parameterized reports feature of the \pkg{rmarkdown} package,
#' however is also used by the knitr \code{\link{purl}} function to include
#' the default parameter values in the R code it emits.
#'
#' @param text Character vector containing the document text.
#' @param evaluate Boolean. If \code{TRUE} (the default), expression values embedded
#' within the YAML will be evaluated. If \code{FALSE}, parameters defined with an
#' expression will have the parsed but unevaluated expression in their \code{value}
#' field.
#'
#' @return List of objects of class \code{knit_param} that correspond to the
#'   parameters declared in the \code{params} section of the YAML front matter.
#'   These objects have the following fields:
#'
#'   \describe{
#'     \item{\code{name}}{The parameter name.}
#'     \item{\code{value}}{The default value for the parameter.}
#'     \item{\code{expr}}{The R expression (if any) that yielded the default value.}
#'   }
#'
#'   In addition, other fields included in the YAML may also be present
#'   alongside the name, type, and value fields (e.g. a \code{label} field
#'   that provides front-ends with a human readable name for the parameter).
#'
#' @details
#'
#' Parameters are included in YAML front matter using the \code{params} key.
#' This key can have any number of subkeys each of which represents a
#' parameter. For example:
#'
#' \preformatted{
#' ---
#' title: My Document
#' output: html_document
#' params:
#'   frequency: 10
#'   show_details: true
#' ---
#' }
#'
#' Parameter values can be provided inline as illustrated above or can be
#' included in a \code{value} sub-key. For example:
#'
#' \preformatted{
#' ---
#' title: My Document
#' output: html_document
#' params:
#'   frequency:
#'     value: 10
#' ---
#' }
#'
#' This second form is useful when you need to provide additional details
#' about the parameter (e.g. a \code{label} field as describe above).
#'
#' You can also use R code to yield the value of a parameter by prefacing the value
#' with \code{!r}, for example:
#'
#' \preformatted{
#' ---
#' title: My Document
#' output: html_document
#' params:
#'   start: !r Sys.Date()
#' ---
#' }
#'
#' @export
knit_params = function(text, evaluate = TRUE) {

  # make sure each element is on one line
  text = split_lines(text)

  # read the yaml front matter and see if there is a params element in it
  yaml = yaml_front_matter(text)
  if (is.null(yaml)) return(list())

  knit_params_yaml(enc2utf8(yaml), evaluate = evaluate)
}

#' Extract knit parameters from YAML text
#'
#' This function reads the YAML front-matter that has already been extracted
#' from a document and returns a list of any parameters declared there.
#'
#' @param yaml Character vector containing the YAML text.
#' @param evaluate If \code{TRUE} (the default) expression values
#' embedded within the YAML will be evaluated. If \code{FALSE}, parameters
#' defined with an expression will have the parsed but unevaluated expression
#' in their \code{value} field.
#'
#' @return List of objects of class \code{knit_param} that correspond to the
#' parameters declared in the \code{params} section of the YAML. See
#' \code{\link{knit_params}} for a full description of these objects.
#'
#' @seealso \code{\link{knit_params}}
#'
#' @export
knit_params_yaml = function(yaml, evaluate = TRUE) {
  # parse the yaml using our handlers
  parsed_yaml = yaml::yaml.load(
    yaml, handlers = knit_params_handlers(evaluate = evaluate), eval.expr = TRUE
  )

  # if we found paramters then resolve and return them
  if (is.list(parsed_yaml) && !is.null(parsed_yaml$params)) {
    resolve_params(parsed_yaml$params, evaluate = evaluate)
  } else {
    list()
  }
}

# turn params into a named list of values
flatten_params = function(params) {
  res = list()
  for (param in params) res[[param$name]] = param$value
  res
}

# Extract the yaml front matter (if any) from the passed lines. The front
# matter is returned as a single-element character vector (with newlines
# delimited by \n) suitable for passing to yaml::load. This code is based on
# the partition_yaml_front_matter and parse_yaml_front_matter functions here:
# https://github.com/rstudio/rmarkdown/blob/master/R/output_format.R
yaml_front_matter = function(lines) {

  # verify that the first two front matter delimiters (---) are not preceded
  # by other content
  has_front_matter = function(delimiters) {
    length(delimiters) >= 2 && (delimiters[2] - delimiters[1] > 1) &&
      (delimiters[1] == 1 || is_blank(head(lines, delimiters[1] - 1))) &&
      grepl("^---\\s*$", lines[delimiters[1]])
  }

  # find delimiters in the document
  delimiters = grep("^(---|\\.\\.\\.)\\s*$", lines)

  # if it's valid then return front matter as a text block suitable for passing
  # to yaml::load
  if (!has_front_matter(delimiters)) return()

  # return the yaml as a single-element character vector if
  # appears to be valid yaml
  front_matter_lines = lines[(delimiters[1]):(delimiters[2])]
  if (length(front_matter_lines) <= 2) return()

  front_matter = front_matter_lines
  front_matter = front_matter[2:(length(front_matter) - 1)]
  if (length(grep('^params:', front_matter)) == 0) return()  # no params in YAML
  front_matter = paste(front_matter, collapse = "\n")

  # ensure that the front-matter doesn't terminate with ':', so it won't cause a
  # crash when passed to yaml::load
  if (!grepl(":\\s*$", front_matter)) front_matter
}


# define custom handlers for knitr_params
knit_params_handlers = function(evaluate = TRUE) {

  # generic handler for r expressions where we want to preserve both the original
  # code and the fact that it was an expression.
  expr_handler = function(value) {
    expression = parse_only(value)
    transformed_value = if (evaluate) {
      eval(expression)
    } else {
      # When we are not evaluating, provide the parsed expression as the transformed value
      expression
    }

    wrapped = list(
        value = transformed_value,
        expr = value)
    wrapped = structure(wrapped, class = "knit_param_expr")
    wrapped
  }

  list(

    # r expressions where we want to preserve both the original code
    # and the fact that it was an expression.
    r = expr_handler,
    expr = expr_handler,

    # date and datetime (for backward compatibility with previous syntax)
    date = function(value) {
      value = as.Date(value)
      value
    },
    datetime = function(value) {
      value = as.POSIXct(value, tz = "GMT")
      value
    },

    # workaround default yaml parsing behavior to allow keys named 'y' and 'n'
    `bool#yes` = function(value) {
      if (tolower(value) == "y") value else TRUE
    },
    `bool#no` = function(value) {
      if (tolower(value) == "n") value else FALSE
    }
  )
}


# resolve the raw params list into the full params data structure (with name,
# type, value, and other optional fields included)
resolve_params = function(params, evaluate = TRUE) {

  # params we will return
  resolved_params = list()

  # iterate over names
  for (name in names(params)) {

    # get the parameter
    param = params[[name]]

    if (inherits(param, "knit_param_expr")) {
      # We have a key: !r expr
      param = list(expr = param$expr, value = param$value)
    } else if (is.list(param)) {
      if ("value" %in% names(param)) {
        # This looks like a complex parameter configuration.
        value = param$value
        if (inherits(value, "knit_param_expr")) {
          # We have a key: { value: !r expr }
          param$expr  = value$expr
          param$value = value$value
        }
      } else {
        stop2("no value field specified for YAML parameter '", name, "'")
      }
    } else {
      # A simple key: value
      param = list(value = param)
    }

    # param is now always a named list. record name and add knit_param class.
    param$name = name
    param = structure(param, class = "knit_param")

    # add the parameter
    resolved_params[[name]] = param
  }

  # return params
  resolved_params
}

#' @rdname hook_plot
#' @export
hook_plot_asciidoc = function(x, options) {
  base = opts_knit$get('base.url') %n% ''
  cap = .img.cap(options)

  width = sprintf('width=%s', options$out.width)
  height = sprintf('height=%s', options$out.height)
  align = sprintf('align=%s', options$fig.align)
  tags = paste(c(cap, width, height, align), collapse = ',')

  sprintf('.%s\nimage::%s%s[%s]\n', cap, base, .upload.url(x), tags)
}

#' @rdname output_hooks
#' @export
render_asciidoc = function() {
  set_html_dev()
  opts_knit$set(out.format = 'asciidoc')
  hook.source = function(x, options) {
    x = one_string(c(hilight_source(x, 'asciidoc', options), ''))
    sprintf('\n[source,%s]\n----\n%s----\n', tolower(options$engine), x)
  }
  hook.message = function(x, options) {
    sprintf('\n[NOTE]\n====\n.Message\n%s\n====\n', substring(x, comment_length(options$comment)))
  }
  hook.warning = function(x, options) {
    sprintf('\n[WARNING]\n====\n.Warning\n%s\n====\n', gsub('^.*Warning: ', '', x))
  }
  hook.error = function(x, options) {
    sprintf('\n[CAUTION]\n====\n.Error\n%s\n====\n', gsub('^.*Error: ', '', x))
  }
  hook.output = function(x, options) sprintf('\n----\n%s----\n', x)
  knit_hooks$set(
    source = hook.source, output = hook.output, message = hook.message,
    warning = hook.warning, error = hook.error, plot = hook_plot_asciidoc
  )
}

comment_length = function(x) {
  (if (is.null(x) || !nzchar(x) || is.na(x)) 0L else nchar(x)) + 1L
}

#' A Pandoc wrapper to convert documents to other formats
#'
#' This function calls Pandoc to convert documents to other formats such as
#' HTML, LaTeX/PDF and Word, etc, (optionally) based on a configuration file or
#' in-file configurations which specify the options to use for Pandoc.
#'
#' There are two ways to input the Pandoc configurations -- through a config
#' file, or embed the configurations in the input file as special comments
#' between \verb{<!--pandoc} and \verb{-->}.
#'
#' The configuration file is a DCF file (see \code{\link{read.dcf}}). This file
#' must contain a field named \code{t} which means the output format. The
#' configurations are written in the form of \code{tag:value} and passed to
#' Pandoc (if no value is needed, just leave it empty, e.g. the option
#' \code{standalone} or \code{s} for short). If there are multiple output
#' formats, write each format and relevant configurations in a block, and
#' separate blocks with blank lines.
#'
#' If there are multiple records of the \code{t} field in the configuration, the
#' input markdown file will be converted to all these formats by default, unless
#' the \code{format} argument is specified as one single format.
#' @param input A character vector of Markdown filenames (must be encoded in
#'   UTF-8).
#' @param format Name of the output format (see References). This can be a
#'   character vector of multiple formats; by default, it is obtained from the
#'   \code{t} field in the configuration. If the configuration is empty or the
#'   \code{t} field is not found, the default output format will be
#'   \code{'html'}.
#' @param config Path to the Pandoc configuration file. If missing, it is
#'   assumed to be a file with the same base name as the \code{input} file and
#'   an extension \code{.pandoc} (e.g. for \file{foo.md} it looks for
#'   \file{foo.pandoc})
#' @param ext Filename extensions. By default, the extension is inferred from
#'   the \code{format}, e.g. \code{latex} creates \code{pdf}, \code{dzslides}
#'   creates \code{html}, and so on
#' @return The output filename(s) (or an error if the conversion failed).
#' @references Pandoc: \url{https://pandoc.org}; Examples and rules of the
#'   configurations: \url{https://yihui.org/knitr/demo/pandoc/}
#'
#'   Also see R Markdown (v2) at \url{http://rmarkdown.rstudio.com}. The
#'   \pkg{rmarkdown} package has several convenience functions and templates
#'   that make it very easy to use Pandoc. The RStudio IDE also has
#'   comprehensive support for it, so I'd recommend users who are not familiar
#'   with command-line tools to use the \pkg{rmarkdown} package instead.
#' @seealso \code{\link{read.dcf}}
#' @export
#' @examples system('pandoc -h') # see possible output formats
pandoc = function(input, format, config = getOption('config.pandoc'), ext = NA) {
  exec = tryCatch(rmarkdown::pandoc_exec(), error = function(e) Sys.which('pandoc'))
  if (length(exec) != 1 || exec == '')
    stop('Please install either RStudio or Pandoc (https://pandoc.org)')
  cfg = if (is.null(config)) with_ext(input[1L], 'pandoc') else config
  txt = pandoc_cfg(read_utf8(input[1]))
  if (file.exists(cfg)) txt = c(txt, '', read_utf8(cfg))
  con = textConnection(txt, encoding = 'UTF-8'); on.exit(close(con), add = TRUE)
  cfg = read.dcf(con)
  nms = colnames(cfg)
  if (length(nms) && 'format' %in% nms) {
    warning('the "format" field in the configuration must be renamed to "t"')
    colnames(cfg)[nms == 'format'] = 't'  # for backward compatibility
  }
  if (missing(format)) format = pandoc_fmt(cfg)
  mapply(
    pandoc_one, input, format, ext, exec, MoreArgs = list(cfg = cfg),
    USE.NAMES = FALSE
  )
}
# format is a scalar
pandoc_one = function(input, format, ext, exec, cfg) {
  cmn = NULL  # common arguments
  if (nrow(cfg) == 0L) cfg = character(0) else if (nrow(cfg) == 1L) {
    if ('t' %in% colnames(cfg)) {
      cfg = if (cfg[1L, 't'] == format) drop(cfg) else NA
    } else {cmn = drop(cfg); cfg = NA}
  } else {
    if (!('t' %in% colnames(cfg)))
      stop('for a config file with multiple output formats, there must be a field named "t"')
    if (sum(idx <- is.na(cfg[, 't'])) > 1L)
      stop('at most one "t" field can be NA')
    if (sum(idx) == 1L) cmn = cfg[idx, ]
    cfg = cfg[!idx, , drop = FALSE]
    cfg = cfg[cfg[, 't'] == format, ]
    if (!is.null(dim(cfg))) {
      if (nrow(cfg) > 1) stop('the output format is not unique in config')
      cfg = character(0) # nrow(cfg) == 0; format not found in cfg
    }
  }
  out = unname(if (!is.na(cfg['o'])) cfg['o'] else {
    if (!is.na(cfg['output'])) cfg['output'] else {
      with_ext(input, if (is.na(ext)) pandoc_ext(format) else ext)
    }
  })
  cfg = cfg[setdiff(names(cfg), c('o', 'output', 't'))]
  cmd = c(
    pandoc_arg(cfg), pandoc_arg(cmn), '-t', format, '-o', shQuote(out), shQuote(input)
  )
  message('Executing: pandoc ', paste(cmd, collapse = ' '))
  if (system2(exec, cmd) == 0L) out else stop('conversion failed')
}

# detect output format from config
pandoc_fmt = function(config) {
  fields = colnames(config)
  if (prod(dim(config)) == 0 || !('t' %in% fields)) return('html')
  na.omit(config[, 't'])
}

# infer output extension from format
pandoc_ext = function(format) {
  if (grepl('^html', format)) return('html')
  if (grepl('^latex|beamer|context|texinfo', format)) return('pdf')
  if (format %in% c('s5', 'slidy', 'slideous', 'dzslides')) return('html')
  if (grepl('^rst', format)) return('rst')
  if (format == 'opendocument') return('xml')
  format
}

# give me a vector of arguments, I turn them into command-line
pandoc_arg = function(x) {
  if (length(x) == 0L || all(is.na(x))) return()
  x = x[!is.na(x)]  # options not provided
  nms = names(x)
  if (any(grepl('\n', x))) {
    # one argument used multiple times, e.g. --bibliography
    x = stringr::str_split(x, '\n')
    nms = rep(nms, sapply(x, length))
    x = unlist(x)
  }
  a1 = nchar(nms) == 1L
  paste0(ifelse(a1, '-', '--'), nms, ifelse(x == '', '', ifelse(a1, ' ', '=')), x)
}

# identify pandoc config in markdown comments
pandoc_cfg = function(x) {
  if (length(i1 <- grep('^<!--pandoc', x)) == 0L ||
        length(i2 <- grep('-->\\s*$', x)) == 0L) return(character(0))
  i1 = i1[1L]; if (all(i2 < i1)) return(character(0))
  i2 = i2[i2 >= i1][1L]
  cfg = x[i1:i2]
  cfg[1L] = gsub('^<!--pandoc\\s*', '', cfg[1L])
  cfg[length(cfg)] = gsub('-->\\s*$', '', cfg[length(cfg)])
  cfg
}

hilight_source = function(x, format, options) {
  if ((format %in% c('latex', 'html')) && options$highlight) {
    if (options$engine == 'R') {
      opts = opts_knit$get('highr.opts')
      highr::hilight(x, format, prompt = options$prompt, markup = opts$markup)
    } else {
      res = try(highr::hi_andre(x, options$engine, format))
      if (inherits(res, 'try-error')) {
        if (format == 'html') highr:::escape_html(x) else highr:::escape_latex(x)
      } else {
        highlight_header()
        n = length(res)
        # do not touch font size
        if (res[n] == '\\normalsize') res = res[-n]
        res
      }
    }
  } else if (options$prompt) {
    # if you did not reformat or evaluate the code, I have to figure out which
    # lines belong to one complete expression first (#779)
    if (options$engine == 'R' && isFALSE(options$tidy) && isFALSE(options$eval))
      x = vapply(highr:::group_src(x), one_string, character(1))
    line_prompt(x)
  } else x
}

highlight_header = function() {
  set_header(highlight.extra = paste(c(
    sprintf('\\let\\hl%s\\hlstd', c('esc', 'pps', 'lin')),
    sprintf('\\let\\hl%s\\hlcom', c('slc', 'ppc'))
  ), collapse = ' '))
}

# stolen from Romain's highlight package (v0.3.2)

# http://www.w3schools.com/css/css_colornames.asp
w3c.colors = c(
  aqua = '#00FFFF', black = '#000000', blue = '#0000FF', fuchsia = '#FF00FF',
  gray = '#808080', green = '#008000', lime = '#00FF00', maroon = '#800000',
  navy = '#000080', olive = '#808000', purple = '#800080', red = '#FF0000',
  silver = '#C0C0C0', teal = '#008080', white = '#FFFFFF', yellow = '#FFFF00'
)

css.parse.color = function(txt, default = '#000000') {
  txt = gsub('\\s+', '', tolower(txt))
  if (is.hex(txt)) return(txt)

  # css specs are from 0 to 255
  rgb = function(...) grDevices::rgb(..., maxColorValue = 255)

  # first we try to match against w3c standard colors
  if (!grepl('[^a-z]', txt) && txt %in% names(w3c.colors))
    return(w3c.colors[txt])

  # now we try R colors
  if (!grepl('[^a-z0-9]', txt)) {
    R.colors = colors()
    res = R.colors %in% txt
    if (any(res)) {
      return(rgb(t(col2rgb(R.colors[res]))))
    }
  }

  # next we try an rgb() specification
  if (grepl('rgb', txt)) {
    p = try_silent(parse(text = txt))
    if (!inherits(p, 'try-error')) {
      res = try_silent(eval(p))
      if (!inherits(res, 'try-error')) return(res)
    }
  }

  # fall back on the default color
  default
}

is.hex = function(x) grepl('^#[0-9a-f]{6}$', x)

# minimal css parser
css.parser = function(file, lines = read_utf8(file)) {

  rx = '^\\.(.*?) *\\{.*$'
  dec.lines = grep(rx, lines)
  dec.names = sub(rx, '\\1', lines[dec.lines])
  if (any(grepl('[0-9]', dec.names))) warning('use of numbers in style names')

  end.lines = grep('^\\s*\\}', lines)

  # find the closing brace of each declaration
  dec.close = end.lines[vapply(dec.lines, function(x) which.min(end.lines < x), integer(1))]

  pos = matrix(c(dec.lines, dec.close), ncol = 2)
  styles = apply(pos, 1, function(x) {
    data = lines[(x[1] + 1):(x[2] - 1)]
    settings.rx = '^\\s*(.*?)\\s*:\\s*(.*?)\\s*;\\s*$'
    settings = sub(settings.rx, '\\1', data, perl = TRUE)
    contents = sub(settings.rx, '\\2', data, perl = TRUE)
    out = list()
    for (i in seq_along(settings)) {
      setting = settings[i]
      content = contents[i]
      out[[setting]] = switch(
        setting,
        color = css.parse.color(content, '#000000'),
        background = css.parse.color(content, '#FFFFFF'),
        content
      )
    }
    out
  })
  names(styles) = dec.names
  styles
}

# styler assistant for latex
styler_assistant_latex = function(x) {

  styles = sapply(x, function(item) {
    settings = names(item)
    has = function(s, value) {
      s %in% settings && grepl(value, item[[s]])
    }
    start = end = ''
    if ('color' %in% settings) {
      start = paste0(start, '\\textcolor[rgb]{', col2latexrgb(item[['color']]), '}{')
      end = paste0(end, '}')
    }
    if (has('font-weight', 'bold')) {
      start = paste0(start, '\\textbf{')
      end = paste0('}', end)
    }
    if (has('font-style', 'italic')) {
      start = paste0(start, '\\textit{')
      end = paste0('}', end)
    }
    sprintf('%s#1%s', start, end)
  })
  res = sprintf('\\newcommand{\\hl%s}[1]{%s}%%', names(x), styles)
  c(res, '\\let\\hlipl\\hlkwb')
}

col2latexrgb = function(hex) {
  # as.character(0.123) -> 0,123 when "OutDec = ,", so make sure . is used
  outdec = options(OutDec = '.'); on.exit(options(outdec))
  col = col2rgb(hex)[, 1] / 255
  paste(round(col, 3), collapse = ',')
}

# format a single inline object
.inline.hook = function(x) {
  if (is.numeric(x)) x = round_digits(x)
  paste(as.character(x), collapse = ', ')
}
.out.hook = function(x, options) x
.plot.hook = function(x, options) paste(x, collapse = '.')

.default.hooks = list(
  source = .out.hook, output = .out.hook, warning = .out.hook,
  message = .out.hook, error = .out.hook, plot = .plot.hook,
  inline = .inline.hook, chunk = .out.hook, text = identity,
  evaluate.inline = function(code, envir = knit_global()) {
    v = withVisible(eval(parse_only(code), envir = envir))
    if (v$visible) knit_print(v$value, inline = TRUE, options = opts_chunk$get())
  },
  evaluate = function(...) evaluate::evaluate(...), document = identity
)

#' Hooks for R code chunks, inline R code and output
#'
#' A hook is a function of a pre-defined form (arguments) that takes values of
#' arguments and returns desired output. The object \code{knit_hooks} is used to
#' access or set hooks in this package.
#' @export
#' @references Usage: \url{https://yihui.org/knitr/objects/}
#'
#' Components in \code{knit_hooks}: \url{https://yihui.org/knitr/hooks/}
#' @examples knit_hooks$get('source'); knit_hooks$get('inline')
knit_hooks = new_defaults(.default.hooks)

render_brew = function() NULL

# the chunk option out.lines = n (first n rows), -n (last n rows), or c(n1, n2)
# (first n1 and last n2 rows)
hook_suppress = function(x, options) {
  n = options$out.lines
  if (length(n) == 0 || !is.numeric(n) || length(n) > 2) return(x)
  x = split_lines(x)
  m = length(x)
  if (length(n) == 1) {
    if (m > abs(n)) {
      x = if (n >= 0) c(head(x, n), '....') else c('....', tail(x, -n))
    }
  } else {
    if (m > sum(n)) x = c(head(x, n[1]), '....', tail(x, n[2]))
  }
  one_string(x)
}

#' Hooks for code chunk options
#'
#' Like \code{\link{knit_hooks}}, this object can be used to set hook functions
#' to manipulate chunk options.
#'
#' For every code chunk, if the chunk option named, say, \code{FOO}, is not
#' \code{NULL}, and a hook function with the same name has been set via
#' \code{opts_hooks$set(FOO = function(options) { options })} (you can manipuate
#' the \code{options} argument in the function and return it), the hook function
#' will be called to update the chunk options.
#' @references \url{https://yihui.org/knitr/hooks/}
#' @export
#' @examples # make sure the figure width is no smaller than fig.height
#' opts_hooks$set(fig.width = function(options) {
#'   if (options$fig.width < options$fig.height) {
#'     options$fig.width = options$fig.height
#'   }
#'   options
#' })
#' # remove all hooks
#' opts_hooks$restore()
opts_hooks = new_defaults(list())

---
title: R Markdown Vignettes with litedown
author: Yihui Xie
date: "`{r} Sys.Date()`"
output:
  html:
    meta:
      css: ["@default"]
---

<!--
%\VignetteEngine{litedown::vignette}
%\VignetteIndexEntry{R Markdown Vignettes with litedown}
-->

Before R 3.0.0, only Sweave/PDF vignettes were supported in R. Markdown is gaining popularity over the years due to its simplicity, and R 3.0.0 starts to support package vignettes written in R Markdown.

> Please note this example is for [litedown](https://cran.r-project.org/package=litedown). If you use [R Markdown v2](https://rmarkdown.rstudio.com), you should use the vignette engine `knitr::rmarkdown` instead of `litedown::vignette`.

## Package vignettes

To enable Markdown vignettes in an R package, you need to

- add `*.Rmd` files under the `vignettes` directory
- add `VignetteBuilder: litedown` to the `DESCRIPTION` file
- specify the vignette engine `\VignetteEngine{litedown::vignette}` in the `Rmd` files (inside HTML comments)

## View vignettes

And R will load the **litedown** package to build these vignettes to HTML files, and you can see them when you open the HTML help:

```{r eval=FALSE}
help(package = 'YourPackage', help_type = 'html')
# or see a standalone list of vignettes
browseVignettes('YourPackage')
```

## Examples

Below are some code chunks as examples.

```{r hello, results='asis'}
if (TRUE) cat('_hello_ **markdown**!', '\n')
```

Normally you do not need any chunk options.

```{r test, collapse=TRUE}
1+1
10:1
rnorm(5)^2
strsplit('hello, markdown vignettes', '')
```

Feel free to draw beautiful plots and write math $P(X>x)=\alpha/2$.

```{r}
n=300; set.seed(123)
par(mar=c(4,4,.1,.1))
plot(rnorm(n), rnorm(n), pch=21, cex=5*runif(n), col='white', bg='gray')
```

---
title: R Markdown with the Docco Linear Style
---

<!--
%\VignetteEngine{knitr::docco_linear_notangle}
%\VignetteIndexEntry{R Markdown with the Docco Linear Style}
-->

This is an example of Markdown vignettes using the [Docco style](http://ashkenas.com/docco/).

## Docco

To use the Docco style for Markdown vignettes in an R package, you need to

- add `*.Rmd` files under the `vignettes` directory
- add `Suggests: knitr` and `VignetteBuilder: knitr` to the `DESCRIPTION` file
- specify the vignette engine `\VignetteEngine{knitr::docco_linear}` in the `Rmd` files (inside HTML comments)

After building and installing the package, you can view vignettes via

```r
browseVignettes(package = 'Your_Package')
```

## Examples

Below are some code chunks as examples.

```{r hello, results='asis'}
cat('_hello_ **markdown**!', '\n')
```

Normally you do not need any chunk options.

```{r}
1+1
10:1
rnorm(5)^2
strsplit('hello, markdown vignettes', '')
```

Feel free to draw beautiful plots and write math $P(X>x)=\alpha/2$.

```{r}
n=300; set.seed(123)
par(mar=c(4,4,.1,.1))
plot(rnorm(n), rnorm(n), pch=21, cex=5*runif(n), col='white', bg='gray')
```

## How does it work

Custom CSS and JS files are passed to `markdown::mark_html()` to style the HTML page:

```r
knitr:::docco_linear
```

That is it.

---
title: R Markdown with the Docco Classic Style
author: Yihui Xie
date: "`r Sys.Date()`"
---

<!--
%\VignetteEngine{knitr::docco_classic_notangle}
%\VignetteIndexEntry{R Markdown with the Docco Classic Style}
-->

```{r setup, echo=FALSE, results='asis'}
x = readLines('docco-linear.Rmd')[-(1:11)]
x = gsub('linear', 'classic', x)
i = grep('^knitr:::docco_classic', x)
x[i - 1] = '```{r}'
x[i] = 'knitr::rocco'
library(knitr)
cat(knit_child(text = x, quiet = TRUE), sep = '\n')
```

You probably have noticed that you can adjust the widths of the two columns
using your cursor. What is more, press `T` on your keyboard, and see what
happens.

---
title: Templating with knit_expand()
author: Yihui Xie
date: "`{r} Sys.Date()`"
output:
  html:
    meta:
      css: ["@default"]
---

<!--
%\VignetteEngine{litedown::vignette}
%\VignetteIndexEntry{Templating with knit_expand()}
-->

A few simple examples:

```{r}
library(knitr)
knit_expand(text = 'The value of pi is {{pi}}.')
knit_expand(text = 'The value of a is {{a}}, so a + 1 is {{a+1}}.', a = rnorm(1))
knit_expand(text = 'The area of a circle with radius {{r}} is {{pi*r^2}}', r = 5)
```

Any number of variables:

```{r}
knit_expand(text = 'a is {{a}} and b is {{b}}, with my own pi being {{pi}} instead of {{base::pi}}', a=1, b=2, pi=3)
```

Custom delimiter `<% %>`:

```{r}
knit_expand(text = 'I do not like curly braces, so use % with <> instead: a is <% a %>.', a = 8, delim = c("<%", "%>"))
```

The pyexpander delimiter:

```{r}
knit_expand(text = 'hello $(LETTERS[24]) and $(pi)!', delim = c("$(", ")"))
```

Arbitrary R code:

```{r}
knit_expand(text = 'you cannot see the value of x {{x=rnorm(1)}}but it is indeed created: x = {{x}}')
res = knit_expand(text = c(' x | x^2', '{{x=1:5;paste(sprintf("%2d | %3d", x, x^2), collapse = "\n")}}'))
cat(res)
```

The m4 example: <https://en.wikipedia.org/wiki/M4_(computer_language)>

```{r}
res = knit_expand(text = c('{{i=0;h2=function(x){i<<-i+1;sprintf("<h2>%d. %s</h2>", i, x)} }}<html>',
'{{h2("First Section")}}', '{{h2("Second Section")}}', '{{h2("Conclusion")}}', '</html>'))
cat(res)
```

Build regression models based on a template; loop through some variables in `mtcars`:

```{r, comment=''}
src = lapply(names(mtcars)[2:5], function(i) {
knit_expand(text=c("# Regression on {{i}}", '```{r lm-{{i}}}', 'lm(mpg~{{i}}, data=mtcars)', '```', ''))
})
# knit the source
litedown::fuse(unlist(src), 'markdown')
```

`r if (exists('.knitr.title')) I(paste('#', .knitr.title, sep = ''))`

`r if (exists('.knitr.author')) I(.knitr.author)`

This report was automatically generated with the R package **knitr**
(version `r packageVersion('knitr')`).

```{r %sCHUNK_LABEL_HERE}
```

The R session information (including the OS info, R version and all
packages used):

```{r session-info, cache=FALSE}
sessionInfo()
Sys.time()
```
```{r clean-up, include=FALSE}
if (exists('.knitr.title')) rm(.knitr.author)
if (exists('.knitr.author')) rm(.knitr.author)
```

---
title: "Custom Print Methods"
author: "Yihui Xie"
date: '`r Sys.Date()`'
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Custom Print Methods}
output:
  knitr:::html_vignette:
    toc: yes
---

Before **knitr** v1.6, printing objects in R code chunks basically emulates the R console. For example, a data frame is printed like this^[Note R prints an object without an explicit `print()` call when it is _visible_; see `?invisible`]:

```{r normal-print, comment=''}
head(mtcars)
```

The text representation of the data frame above may look very familiar with most R users, but for reporting purposes, it may not be satisfactory -- often times we want to see a table representation instead. That is the problem that the chunk option `render` and the S3 generic function `knit_print()` try to solve.

## Customize Printing

After we evaluate each R expression in a code chunk, there is an object returned. For example, `1 + 1` returns `2`. This object is passed to the chunk option `render`, which is a function with two arguments, `x` and `options`, or `x` and `...`. The default value for the `render` option is `knit_print`, an S3 function in **knitr**:

```{r}
library(knitr)
knit_print  # an S3 generic function
methods(knit_print)
getS3method('knit_print', 'default')  # the default method
normal_print
```

As we can see, `knit_print()` has a `default` method, which is basically `print()` or `show()`, depending on whether the object is an S4 object. This means it does nothing special when printing R objects:

```{r collapse=TRUE}
knit_print(1:10)
knit_print(head(mtcars))
```

S3 generic functions are extensible in the sense that we can define custom methods for them. A method `knit_print.foo()` will be applied to the object that has the class `foo`. Here is quick example of how we can print data frames as tables:

```{r}
library(knitr)
# define a method for objects of the class data.frame
knit_print.data.frame = function(x, ...) {
  res = paste(c('', '', kable(x)), collapse = '\n')
  asis_output(res)
}
# register the method
registerS3method("knit_print", "data.frame", knit_print.data.frame)
```

If you define a method in a code chunk in a **knitr** document, the call to `registerS3method()` will be necessary for R >= 3.5.0, because the S3 dispatch mechanism has changed since R 3.5.0. If you are developing an R package, see the section [For package authors] below.

We expect the print method to return a character vector, or an object that can be coerced into a character vector. In the example above, the `kable()` function returns a character vector, which we pass to the `asis_output()` function so that later **knitr** knows that this result needs no special treatment (just write it as is), otherwise it depends on the chunk option `results` (`= 'asis'` / `'markup'` / `'hide'`) how a normal character vector should be written. The function `asis_output()` has the same effect as `results = 'asis'`, but saves us the effort to provide this chunk option explicitly. Now we check how the printing behavior is changed. We print a number, a character vector, a list, a data frame, and write a character value using `cat()` in the chunk below:

```{r knit-print}
1 + 1
head(letters)
list(a = 1, b = 9:4)
head(mtcars)
cat('This is cool.')
```

We see all objects except the data frame were printed "normally"^[The two hashes `##` were from the chunk option `comment`; you can turn them off by `comment = ''`.]. The data frame was printed as a real table. Note you do not have to use `kable()` to create tables -- there are many other options such as **xtable**. Just make sure the print method returns a character string.

The [**printr**](https://github.com/yihui/printr) package is a companion to **knitr** containing printing methods for some common objects like matrices and data frames. Users only need to load this package to get attractive printed results. A major factor to consider (which has been considered in **printr**) when defining a printing method is the output format. For example, the table syntax can be entirely different when the output is LaTeX vs when it is Markdown.

It is strongly recommended that your S3 method has a `...` argument, so that your method can safely ignore arguments that are passed to `knit_print()` but not defined in your method. At the moment, a `knit_print()` method can have two optional arguments:

- the `options` argument takes a list of the current chunk options;
- the `inline` argument indicates if the method is called in code chunks or inline R code;

Depending on your application, you may optionally use these arguments. Here are some examples:

```{r eval=FALSE, tidy=FALSE}
knit_print.classA = function(x, ...) {
  # ignore options and inline
}
knit_print.classB = function(x, options, ...) {
  # use the chunk option out.height
  asis_output(paste0(
    '<iframe src="https://yihui.org" height="', options$out.height, '"></iframe>',
  ))
}
knit_print.classC = function(x, inline = FALSE, ...) {
  # different output according to inline=TRUE/FALSE
  if (inline) {
    'inline output for classC'
  } else {
    'chunk output for classC'
  }
}
knit_print.classD = function(x, options, inline = FALSE, ...) {
  # use both options and inline
}
```

Note that when *using* your (or another) `knit_print()` method *inline* (if it supports that), you must not call `knit_print()` on the object, but just have it return. For example, your inline code should read `` `r
c("foo")` `` and *not* `` `r
knit_print(c("foo"))` ``. The latter inline code would yield the methods' result for *in-chunk* (not inline), because, as set up in the above, `knit_print()` methods default to `inline = FALSE`. This default gets overwritten depending on the context in which `knit_print()` is called (inline or in-chunk), only when `knit_print()` is called by **knitr** (not you) via the `render` option (see below). You can, of course, always manually set the inline option `` `r
knit_print(c("foo"), inline = TRUE)` ``, but that's a lot of typing.

## A Low-level Explanation

You can skip this section if you do not care about the low-level implementation details.

### The `render` option

As mentioned before, the chunk option `render` is a function that defaults to `knit_print()`. We can certainly use other render functions. For example, we create a dummy function that always says "I do not know what to print" no matter what objects it receives:

```{r}
dummy_print = function(x, ...) {
  cat("I do not know what to print!")
  # this function implicitly returns an invisible NULL
}
```

Now we use the chunk option `render = dummy_print`:

```{r knit-print, render=dummy_print, collapse=TRUE}
```

Note the `render` function is only applied to visible objects. There are cases in which the objects returned are invisible, e.g. those wrapped in `invisible()`.

```{r}
1 + 1
invisible(1 + 1)
invisible(head(mtcars))
x = 1:10  # invisibly returns 1:10
```

### Metadata

The print function can have a side effect of passing "metadata" about objects to **knitr**, and **knitr** will collect this information as it prints objects. The motivation of collecting metadata is to store external dependencies of the objects to be printed. Normally we print an object only to obtain a text representation, but there are cases that can be more complicated. For example, a [**ggvis** ](http://ggvis.rstudio.com/) graph requires external JavaScript and CSS dependencies such as `ggvis.js`. The graph itself is basically a fragment of JavaScript code, which will not work unless the required libraries are loaded (in the HTML header). Therefore we need to collect the dependencies of an object beside printing the object itself.

One way to specify the dependencies is through the `meta` argument of `asis_output()`. Here is a pseudo example:

```{r eval=FALSE, tidy=FALSE}
# pseudo code
knit_print.ggvis = function(x, ...) {
  res = ggvis::print_this_object(x)
  knitr::asis_output(res, meta = list(
    ggvis = list(
      version = '0.1.0',
      js  = system.file('www', 'js',  'ggvis.js',  package = 'ggvis'),
      css = system.file('www', 'www', 'ggvis.css', package = 'ggvis')
    )
  ))
}
```

Then when **knitr** prints a **ggvis** object, the `meta` information will be collected and stored. After knitting is done, we can obtain a list of all the dependencies via `knit_meta()`. It is very likely that there are duplicate entries in the list, and it is up to the package authors to clean them up, and process the metadata list in their own way (e.g. write the dependencies into the HTML header). We give a few more quick and dirty examples below to see how `knit_meta()` works.

Now we define a print method for `foo` objects:

```{r tidy=FALSE}
library(knitr)
knit_print.foo = function(x, ...) {
  res = paste('> **This is a `foo` object**:', x)
  asis_output(res, meta = list(
    js  = system.file('www', 'shared', 'shiny.js',  package = 'shiny'),
    css = system.file('www', 'shared', 'shiny.css', package = 'shiny')
  ))
}
```

See what happens when we print `foo` objects:

```{r}
new_foo = function(x) structure(x, class = 'foo')
new_foo('hello')
```

Check the metadata now:

```{r}
str(knit_meta(clean = FALSE))
```

Another `foo` object:

```{r}
new_foo('world')
```

Similarly for `bar` objects:

```{r}
knit_print.bar = function(x, ...) {
  asis_output(x, meta = list(head = '<script>console.log("bar!")</script>'))
}
new_bar = function(x) structure(x, class = 'bar')
new_bar('> **hello** world!')
new_bar('> hello **world**!')
```

The final version of the metadata, and clean it up:

```{r}
str(knit_meta())
str(knit_meta()) # empty now, because clean = TRUE by default
```

### For package authors

If you are implementing a custom print method in your own package, here are two tips:

1. Normally you need to import `knit_print` in your package namespace via `importFrom(knitr, knit_print)` (or roxygen: `#' @importFrom knitr knit_print`) (see [the **printr** package](https://github.com/yihui/printr) for an example). If you don't want to import **knitr**, you can call `registerS3method()` in your `.onLoad()` hook (see [rstudio/htmltools#108](https://github.com/rstudio/htmltools/pull/108/files) for an example);
1. `asis_output()` is simply a function that marks an object with the class `knit_asis`, and you do not have to import this function to your package, either -- just let your print method return `structure(x, class = 'knit_asis')`, and if there are additional metadata, just put it in the `knit_meta` attribute; here is the source code of this function:
    ```{r}
    knitr::asis_output
    ```

    Note that you can actually put **knitr** in the `Suggests` field in DESCRIPTION, and use `knitr::asis_output()`, so that you can avoid the "hard" dependency on **knitr**.

```{r clean-up, include=FALSE}
# R compiles all vignettes in the same session, which can be bad
rm(list = ls(all = TRUE))
```

---
title: "Not An Introduction to knitr"
author: "Yihui Xie"
date: "`{r} Sys.Date()`"
output:
  html:
    meta:
      css: ["@default"]
vignette: >
  %\VignetteEngine{litedown::vignette}
  %\VignetteIndexEntry{Not an Introduction to knitr}
---

The **knitr** package is an alternative tool to Sweave based on a different
design with more features. This document is not an introduction, but only serves
as a placeholder to guide you to the real manuals, which are available on the
package website <https://yihui.org/knitr/> (e.g. the [main
manual](https://yihui.org/knitr/demo/manual/) and the [graphics
manual](https://yihui.org/knitr/demo/graphics/) ), and remember to read the help
pages of functions in this package. There is a book "Dynamic Docuemnts with R
and knitr" for this package, too.

Below are code chunk examples:

```{r show-off}
options(digits = 4)
rnorm(20)
fit = lm(dist ~ speed, data = cars)
b = coef(fit)
```

```{r echo=FALSE}
summary(fit)$coefficients
```

The fitted regression equation is $Y=`{r} b[1]`+`{r} b[2]`x$.

```{r graphics, fig.cap='A scatterplot with a regression line.'}
par(mar=c(4, 4, 1, .1))
plot(cars, pch = 20)
abline(fit, col = 'red')
```

## References

```{r, echo=FALSE, results='asis', warning=FALSE}
print(citation('knitr'), style = 'html')
```

---
title: "knitr Reference Card"
author: "Yihui Xie"
date: "`{r} Sys.Date()`"
output:
  html:
    meta:
      css: ["@default"]
    options:
      number_sections: true
---

```{=html}
<!--
%\VignetteEngine{litedown::vignette}
%\VignetteIndexEntry{knitr Reference Card}
-->
```

## Syntax

| format    | start               | end             | inline             | output   |
|-----------|---------------------|-----------------|--------------------|----------|
| Rnw       | `<<*>>=`            | `@`             | `\Sexpr{x}`        | TeX      |
| Rmd       | ```` ```{r *} ````  | ```` ``` ````   | `` `r x` ``        | Markdown |
| Rhtml     | `<!--begin.rcode *` | `end.rcode-->`  | `<!--rinline x-->` | HTML     |
| Rrst      | `.. {r *}`          | `.. ..`         | `` :r:`x` ``       | reST     |
| Rtex      | `% begin.rcode *`   | `% end.rcode`   | `\rinline{x}`      | TeX      |
| Rasciidoc | `// begin.rcode *`  | `// end.rcode`  | `+r x+`            | AsciiDoc |
| Rtextile  | `### begin.rcode *` | `### end.rcode` | `@r x@`            | Textile  |
| brew      |                     |                 | `<% x %>`          | text     |

`*` denotes local chunk options, e.g., `<<label, eval=FALSE>>=`; `x` denotes
inline R code, e.g., `` `r 1+2` ``.

## Minimal Examples

::: col-2
### Sweave (\*.Rnw)

``` tex
\documentclass{article}
\begin{document}

Below is a code chunk.

<<foo, echo=TRUE>>=
z = 1 + 1
plot(cars)
@

The value of z is \Sexpr{z}.
\end{document}
```

### R Markdown (\*.Rmd)

```` md
---
title: "An R Markdown document"
---

Hi _Markdown_!

```{r foo, echo=TRUE}
z = 1 + 1
plot(cars)
```

The value of z is `r z`.
````
:::

## Chunk Options

`opts_chunk` controls global chunk options, e.g.,
`knitr::opts_chunk$set(tidy = FALSE)`, which can be overridden by local chunk
options. See all options at <https://yihui.org/knitr/options/>. Some frequently
used options are:

-   `eval`: whether to evaluate the chunk
-   `echo`: whether to echo source code
-   `results`: `'markup'`, `'asis'`, `'hold'`, `'hide'`
-   `tidy`: whether to reformat R code
-   `cache`: whether to cache results
-   `fig.width`, `fig.height`, `out.width`, `out.height`: device and output size
    of figures
-   `include`: whether to include the chunk results in output
-   `child`: path to child documents
-   `engine`: language name (R, python, ...)

## Functions

-   `knit()`: the main function in this package; knit input document and write
    output
-   `purl()`: extract R code from an input document
-   `spin()`: spin goat's hair (an R script with roxygen comments) into wool (a
    literate programming document to be passed to `knit()`)
-   `stitch()`: insert an R script into a template and compile the document
-   `knit_hooks$set()`: set or reset chunk and output
    [hooks](https://yihui.org/knitr/hooks/)

## Resources

-   Homepage: <https://yihui.org/knitr/>
-   Github: <https://github.com/yihui/knitr>
    ([CRAN](https://cran.r-project.org/package=knitr))
-   Examples: <https://github.com/yihui/knitr-examples>
-   Stack Overflow: <https://stackoverflow.com/tags/knitr/>

```{css, echo=FALSE}
body {
  max-width: none;
}
table {
  width: 100%;
}
.body h2 {
  border-bottom: none;
  font-size: 1.3em;
}
.body, .col-2 {
  columns: 2;
  & > :first-child {
    margin-top: 0;
  }
}
```

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\begin_preamble
% \VignetteIndexEntry{An Introduction to knitr}
% \VignetteEngine{knitr}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package default
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans lmss
\font_typewriter lmtt
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref false
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine natbib_authoryear
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 0
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 2.5cm
\topmargin 2.5cm
\rightmargin 2.5cm
\bottommargin 2.5cm
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Title
An Introduction to knitr
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-knitr"

\end_inset

 is an alternative tool to Sweave based on a different design with more
 features.
 This document is not the real vignette, but only serves as a placeholder
 to guide you to the real manuals of this package.
 You can find them in the package website: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://yihui.name/knitr
\end_layout

\end_inset


\begin_inset Foot
status open

\begin_layout Plain Layout
e.g.
 the main manual: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://bitbucket.org/stat/knitr/downloads/knitr-manual.pdf
\end_layout

\end_inset

 and the graphics manual: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://bitbucket.org/stat/knitr/downloads/knitr-graphics.pdf
\end_layout

\end_inset


\end_layout

\end_inset

, and remember to read the help pages of functions in this package.
\end_layout

\begin_layout Standard
Anyway, here is a code chunk that shows you can compile vignettes with 
\series bold
knitr
\series default
 as well by using a proper 
\family sans
Makefile
\family default
 (see 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://yihui.name/knitr/demo/vignette/
\end_layout

\end_inset

):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<show-off>>=
\end_layout

\begin_layout Plain Layout

rnorm(5)
\end_layout

\begin_layout Plain Layout

df=data.frame(y=rnorm(100), x=1:100)
\end_layout

\begin_layout Plain Layout

summary(lm(y~x, data=df))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
bibfiles "../examples/knitr-packages"
options "jss"

\end_inset


\end_layout

\end_body
\end_document

`r if (exists('.knitr.title')) I(paste('#', .knitr.title, sep = ''))`

`r if (exists('.knitr.author')) I(.knitr.author)`

This report was automatically generated with the R package **knitr**
(version `r packageVersion('knitr')`).

```{r %sCHUNK_LABEL_HERE}
```

The R session information (including the OS info, R version and all
packages used):

```{r session-info, cache=FALSE}
sessionInfo()
Sys.time()
```{r clean-up, include=FALSE}
if (exists('.knitr.title')) rm(.knitr.author)
if (exists('.knitr.author')) rm(.knitr.author)
```

---
title: "Custom Print Methods"
author: "Yihui Xie"
date: '`r Sys.Date()`'
output:
  html:
    options:
      toc: true
    meta:
      css: ["@default"]
vignette: >
  %\VignetteEngine{knitr::knitr_notangle}
  %\VignetteIndexEntry{Custom Print Methods}
---

Before **knitr** v1.6, printing objects in R code chunks basically emulates the
R console. For example, a data frame is printed like this[^1]:

[^1]: Note R prints an object without an explicit `print()` call when it is
    *visible*; see `?invisible`

```{r normal-print, comment=''}
head(mtcars)
```

The text representation of the data frame above may look very familiar with most
R users, but for reporting purposes, it may not be satisfactory -- often times
we want to see a table representation instead. That is the problem that the
chunk option `render` and the S3 generic function `knit_print()` try to solve.

## Customize Printing

After we evaluate each R expression in a code chunk, there is an object
returned. For example, `1 + 1` returns `2`. This object is passed to the chunk
option `render`, which is a function with two arguments, `x` and `options`, or
`x` and `...`. The default value for the `render` option is `knit_print`, an S3
function in **knitr**:

```{r}
library(knitr)
knit_print  # an S3 generic function
methods(knit_print)
getS3method('knit_print', 'default')  # the default method
normal_print
```

As we can see, `knit_print()` has a `default` method, which is basically
`print()` or `show()`, depending on whether the object is an S4 object. This
means it does nothing special when printing R objects:

```{r collapse=TRUE}
knit_print(1:10)
knit_print(head(mtcars))
```

S3 generic functions are extensible in the sense that we can define custom
methods for them. A method `knit_print.foo()` will be applied to the object that
has the class `foo`. Here is quick example of how we can print data frames as
tables:

```{r}
library(knitr)
# define a method for objects of the class data.frame
knit_print.data.frame = function(x, ...) {
  res = paste(c('', '', kable(x)), collapse = '\n')
  asis_output(res)
}
# register the method
registerS3method("knit_print", "data.frame", knit_print.data.frame)
```

If you define a method in a code chunk in a **knitr** document, the call to
`registerS3method()` will be necessary for R \>= 3.5.0, because the S3 dispatch
mechanism has changed since R 3.5.0. If you are developing an R package, see the
section [For package authors] below.

We expect the print method to return a character vector, or an object that can
be coerced into a character vector. In the example above, the `kable()` function
returns a character vector, which we pass to the `asis_output()` function so
that later **knitr** knows that this result needs no special treatment (just
write it as is), otherwise it depends on the chunk option `results` (`= 'asis'`
/ `'markup'` / `'hide'`) how a normal character vector should be written. The
function `asis_output()` has the same effect as `results = 'asis'`, but saves us
the effort to provide this chunk option explicitly. Now we check how the
printing behavior is changed. We print a number, a character vector, a list, a
data frame, and write a character value using `cat()` in the chunk below:

```{r knit-print}
1 + 1
head(letters)
list(a = 1, b = 9:4)
head(mtcars)
cat('This is cool.')
```

We see all objects except the data frame were printed "normally"[^2]. The data
frame was printed as a real table. Note you do not have to use `kable()` to
create tables -- there are many other options such as **xtable**. Just make sure
the print method returns a character string.

[^2]: The two hashes `##` were from the chunk option `comment`; you can turn
    them off by `comment = ''`.

The [**printr**](https://github.com/yihui/printr) package is a companion to
**knitr** containing printing methods for some common objects like matrices and
data frames. Users only need to load this package to get attractive printed
results. A major factor to consider (which has been considered in **printr**)
when defining a printing method is the output format. For example, the table
syntax can be entirely different when the output is LaTeX vs when it is
Markdown.

It is strongly recommended that your S3 method has a `...` argument, so that
your method can safely ignore arguments that are passed to `knit_print()` but
not defined in your method. At the moment, a `knit_print()` method can have two
optional arguments:

-   the `options` argument takes a list of the current chunk options;
-   the `inline` argument indicates if the method is called in code chunks or
    inline R code;

Depending on your application, you may optionally use these arguments. Here are
some examples:

```{r eval=FALSE, tidy=FALSE}
knit_print.classA = function(x, ...) {
  # ignore options and inline
}
knit_print.classB = function(x, options, ...) {
  # use the chunk option out.height
  asis_output(paste0(
    '<iframe src="https://yihui.org" height="', options$out.height, '"></iframe>',
  ))
}
knit_print.classC = function(x, inline = FALSE, ...) {
  # different output according to inline=TRUE/FALSE
  if (inline) {
    'inline output for classC'
  } else {
    'chunk output for classC'
  }
}
knit_print.classD = function(x, options, inline = FALSE, ...) {
  # use both options and inline
}
```

Note that when *using* your (or another) `knit_print()` method *inline* (if it
supports that), you must not call `knit_print()` on the object, but just have it
return. For example, your inline code should read `` `r c("foo")` `` and *not*
`` `r knit_print(c("foo"))` ``. The latter inline code would yield the methods'
result for *in-chunk* (not inline), because, as set up in the above,
`knit_print()` methods default to `inline = FALSE`. This default gets
overwritten depending on the context in which `knit_print()` is called (inline
or in-chunk), only when `knit_print()` is called by **knitr** (not you) via the
`render` option (see below). You can, of course, always manually set the inline
option `` `r knit_print(c("foo"), inline = TRUE)` ``, but that's a lot of
typing.

## A Low-level Explanation

You can skip this section if you do not care about the low-level implementation
details.

### The `render` option

As mentioned before, the chunk option `render` is a function that defaults to
`knit_print()`. We can certainly use other render functions. For example, we
create a dummy function that always says "I do not know what to print" no matter
what objects it receives:

```{r}
dummy_print = function(x, ...) {
  cat("I do not know what to print!")
  # this function implicitly returns an invisible NULL
}
```

Now we use the chunk option `render = dummy_print`:

```{r knit-print, render=dummy_print, collapse=TRUE}
```

Note the `render` function is only applied to visible objects. There are cases
in which the objects returned are invisible, e.g. those wrapped in
`invisible()`.

```{r}
1 + 1
invisible(1 + 1)
invisible(head(mtcars))
x = 1:10  # invisibly returns 1:10
```

### Metadata

The print function can have a side effect of passing "metadata" about objects to
**knitr**, and **knitr** will collect this information as it prints objects. The
motivation of collecting metadata is to store external dependencies of the
objects to be printed. Normally we print an object only to obtain a text
representation, but there are cases that can be more complicated. For example, a
[**ggvis**](https://ggvis.rstudio.com/) graph requires external JavaScript and
CSS dependencies such as `ggvis.js`. The graph itself is basically a fragment of
JavaScript code, which will not work unless the required libraries are loaded
(in the HTML header). Therefore we need to collect the dependencies of an object
beside printing the object itself.

One way to specify the dependencies is through the `meta` argument of
`asis_output()`. Here is a pseudo example:

```{r eval=FALSE, tidy=FALSE}
# pseudo code
knit_print.ggvis = function(x, ...) {
  res = ggvis::print_this_object(x)
  knitr::asis_output(res, meta = list(
    ggvis = list(
      version = '0.1.0',
      js  = system.file('www', 'js',  'ggvis.js',  package = 'ggvis'),
      css = system.file('www', 'www', 'ggvis.css', package = 'ggvis')
    )
  ))
}
```

Then when **knitr** prints a **ggvis** object, the `meta` information will be
collected and stored. After knitting is done, we can obtain a list of all the
dependencies via `knit_meta()`. It is very likely that there are duplicate
entries in the list, and it is up to the package authors to clean them up, and
process the metadata list in their own way (e.g. write the dependencies into the
HTML header). We give a few more quick and dirty examples below to see how
`knit_meta()` works.

Now we define a print method for `foo` objects:

```{r tidy=FALSE}
library(knitr)
knit_print.foo = function(x, ...) {
  res = paste('> **This is a `foo` object**:', x)
  asis_output(res, meta = list(
    js  = system.file('www', 'shared', 'shiny.js',  package = 'shiny'),
    css = system.file('www', 'shared', 'shiny.css', package = 'shiny')
  ))
}
```

See what happens when we print `foo` objects:

```{r}
new_foo = function(x) structure(x, class = 'foo')
new_foo('hello')
```

Check the metadata now:

```{r}
str(knit_meta(clean = FALSE))
```

Another `foo` object:

```{r}
new_foo('world')
```

Similarly for `bar` objects:

```{r}
knit_print.bar = function(x, ...) {
  asis_output(x, meta = list(head = '<script>console.log("bar!")</script>'))
}
new_bar = function(x) structure(x, class = 'bar')
new_bar('> **hello** world!')
new_bar('> hello **world**!')
```

The final version of the metadata, and clean it up:

```{r}
str(knit_meta())
str(knit_meta()) # empty now, because clean = TRUE by default
```

### For package authors

If you are implementing a custom print method in your own package, here are two
tips:

1.  With R \>= 3.6.0 (2019-04-26), you can declare the S3 method in the package
    `NAMESPACE` with `S3method(knitr::knit_print, class)`. If you use
    **roxygen2** package, that means a roxygen comment like this:

    ``` r
    #' @exportS3Method knitr::knit_print
    knit_print.class <- 
    ```

    With this method, you do not need to import **knitr** to your package, i.e.,
    **knitr** can be listed in `Suggests` and not necessarily `Imports` in the
    package `DESCRIPTION`. The S3 methods will be automatically registered when
    **knitr** is actually loaded.

    For R \< 3.6.0, you need to import `knit_print` in your package namespace
    via `importFrom(knitr, knit_print)` (or roxygen:
    `#' @importFrom knitr knit_print`) (see [the **printr**
    package](https://github.com/yihui/printr) for an example).

2.  `asis_output()` is simply a function that marks an object with the class
    `knit_asis`, and you do not have to import this function to your package,
    either---just let your print method return
    `structure(x, class = 'knit_asis')`, and if there are additional metadata,
    just put it in the `knit_meta` attribute; here is the source code of this
    function:

    ```{r}
    knitr::asis_output
    ```

    You may put **knitr** in the `Suggests` field in `DESCRIPTION`, and use
    `knitr::asis_output()`, so that you can avoid the "hard" dependency on
    **knitr**.

```{r clean-up, include=FALSE}
# R compiles all vignettes in the same session, which can be bad
rm(list = ls(all = TRUE))
```

---
title: Display Tables with the JavaScript Library simple-datatables
author: Yihui Xie
date: "`{r} Sys.Date()`"
output:
  html:
    meta:
      css: ["@default", "@npm/simple-datatables/dist/style.min.css"]
      js: ["@npm/simple-datatables/dist/umd/simple-datatables.min.js"]
---

<!--
%\VignetteEngine{litedown::vignette}
%\VignetteIndexEntry{Display Tables with the JavaScript Library simple-datatables}
-->

We can use the JavaScript library [**simple-datatables**](https://github.com/fiduswriter/simple-datatables/) to generate enhanced tables in HTML. In the example below, we create a table for the `mtcars` data:

::: {#mtcars-table}
```{r cool, print.args=list(data.frame=list(limit=NULL))}
mtcars
```
:::

Note we assigned an `id` to the table, and next we use the **DataTables** library to initialize the table and you will get an interactive table.

```{js}
window.addEventListener('load', () => {
  new simpleDatatables.DataTable('#mtcars-table > table');
});
```

By comparison, below is an ordinary table:

```{r boring}
mtcars
```

<!--
%\VignetteEngine{knitr}
%\VignetteIndexEntry{An R Markdown Vignette with knitr}
-->

This is an example of Markdown vignettes in R. Before R 3.0.0, only Sweave/PDF vignettes were supported in R. Markdown is gaining popularity over the years due to its simplicity, and R 3.0.0 starts to support package vignettes written in [R Markdown](http://www.rstudio.com/ide/docs/authoring/using_markdown).

## Package vignettes

To enable Markdown vignettes in an R package, you need to

- add `*.Rmd` files under the `inst/doc` or `vignettes` directory
- add `VignetteBuilder: knitr` to the `DESCRIPTION` file
- specify the vignette engine `\VignetteEngine{knitr}` in the `Rmd` files (inside HTML comments)

## View vignettes

And R will load the **knitr** package to build these vignettes to HTML files, and you can see them when you open the HTML help:

```{r eval=FALSE}
help(package = 'YourPackage', help_type = 'html')
```

## Examples

Below are some code chunks as examples.

```{r hello, results='asis'}
cat('_hello_ **markdown**!', '\n')
```

Normally you do not need any chunk options.

```{r}
1+1
10:1
rnorm(5)^2
strsplit('hello, markdown vignettes', '')
```

Feel free to draw beautiful plots and write math $P(X>x)=\alpha/2$.

```{r}
n=300; set.seed(123)
par(mar=c(4,4,.1,.1))
plot(rnorm(n), rnorm(n), pch=21, cex=5*runif(n), col='white', bg='gray')
```

You can use your own CSS file instead of the built-in style in the **markdown** package -- just set the option `markdown.HTML.stylesheet`, e.g.

```{r css}
options(markdown.HTML.stylesheet = system.file('misc', 'vignette.css', package='knitr'))
```

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[T1]{fontenc}
\usepackage[landscape]{geometry}
\geometry{verbose,tmargin=0cm,bmargin=0cm,lmargin=1cm,rmargin=1cm}
\setlength{\parskip}{0in}
\setlength{\parindent}{0pt}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\usepackage{breakurl}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
%% Because html converters don't know tabularnewline
\providecommand{\tabularnewline}{\\}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\usepackage{multicol, upquote}
% \VignetteIndexEntry{knitr Reference Card}
% \VignetteEngine{knitr}

\makeatother

\begin{document}


\title{knitr Reference Card}


\author{Yihui Xie}

\maketitle
\begin{multicols}{2}


\section{Syntax}

\noindent \begin{center}
\begin{tabular}{l|l|l|l|l}
\hline 
format & start & end & inline & output\tabularnewline
\hline 
Rnw & \verb|<<*>>=| & \texttt{@} & \verb|\|\verb|Sexpr{x}| & \TeX{}\tabularnewline
\hline 
Rmd & \verb|```{r *}| & \verb|```| & \verb|`r x`| & MD\tabularnewline
\hline 
Rhtml & \verb|<!--begin.rcode *| & \verb|end.rcode-->| & \verb|<!--rinline x-->| & HTML\tabularnewline
\hline 
Rrst & \texttt{..~\{r {*}\}} & \texttt{..~..} & \verb|:r:`x`| & reST\tabularnewline
\hline 
Rtex & \texttt{\% begin.rcode {*}} & \texttt{\% end.rcode} & \verb|\rinline{x}| & \TeX{}\tabularnewline
\hline 
brew &  &  & \verb|<% x %>| & text\tabularnewline
\hline 
\end{tabular}
\par\end{center}

\texttt{{*}} denotes local chunk options, e.g. \verb|<<label, eval=FALSE>>=|;
\texttt{x} denotes inline R code, e.g. \verb|`r 1+2`| (MD stands
for \href{http://en.wikipedia.org/wiki/Markdown}{Markdown})


\section{Minimal Examples}

\begin{multicols}{2}


\subsection{Sweave ({*}.Rnw)}

<<echo=FALSE>>=
x=paste('\\Sexpr', '{', 'z}', sep = '')
@
\begin{verbatim}
\document\Sexpr{'c'}lass{article}
\be\Sexpr{'g'}in{document}

Below is a code chunk.
\Sexpr{''}<<foo, echo=TRUE>>=
z = 1+1
plot(cars)
\Sexpr{''}@

The value of z is \Sexpr{x}.
\end{document}
\end{verbatim}


\subsection{R Markdown ({*}.Rmd)}

\begin{verbatim}
Hi _markdown_!

```{r foo, echo=TRUE}
z = 1+1
plot(cars)
```

The value of z is `r z`.
\end{verbatim}


\subsection{Brew ({*}.brew)}

\begin{verbatim}
The value of pi is <% pi %>.
\end{verbatim}

\end{multicols}


\section{Chunk Options}

\texttt{opts\_chunk} controls global chunk options, e.g. \texttt{opts\_chunk\$set(tidy
= FALSE)}, which can be overridden by local chunk options. See all
options at \url{http://yihui.name/knitr/options}; some frequently
used options:
\begin{description}
\item [{eval}] whether to evaluate the chunk
\item [{echo}] whether to echo source code
\item [{results}] \texttt{'markup'}, \texttt{'asis'}, \texttt{'hide'}
\item [{tidy}] whether to reformat R code
\item [{cache}] whether to cache results
\item [{fig.width,\ fig.height,\ out.width,\ out.height}] device and
output size of figures
\item [{include}] whether to include the chunk results in output
\item [{child}] filenames of child documents
\item [{engine}] language name (R, python, ...)
\end{description}

\section{Functions}
\begin{description}
\item [{knit()}] the main function in this package; knit input document
and write output
\item [{purl()}] extract R code from an input document
\item [{spin()}] spin goat's hair (an R script with roxygen comments) into
wool (a literate programming document to be passed to \texttt{knit()})
\item [{stitch()}] insert an R script into a template and compile the document
\item [{knit\_hooks\$set()}] set or reset chunk and output \href{http://yihui.name/knitr/hooks}{hooks}
\end{description}

\section{Resources}
\begin{itemize}
\item homepage: \url{http://yihui.name/knitr} 
\item development repository: \url{https://github.com/yihui/knitr} (\href{http://cran.r-project.org/package=knitr}{CRAN},
\href{http://rforge.net/knitr}{Rforge})
\item examples: \url{https://github.com/yihui/knitr-examples}
\item stackoverflow: \url{http://stackoverflow.com/questions/tagged/knitr}
\item mailing list: \url{https://groups.google.com/group/knitr}
\end{itemize}
\end{multicols}

\end{document}

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package default
\inputencoding auto
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref false
\papersize default
\use_geometry false
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/subfloats-')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
Using subfloats in LyX with knitr
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
If you insert a float figure inside a float figure in LyX, you will get
 subfloats.
 To make sure the two subfloats can stand side by side, you have to:
\end_layout

\begin_layout Enumerate
remove new lines between the two subfloat environments in LyX (if you are
 not sure if there are blank lines between them, check the LaTeX source);
\end_layout

\begin_layout Enumerate
use the chunk option 
\family typewriter
echo=FALSE
\family default
 to hide the R source code in the output;
\end_layout

\begin_layout Enumerate
set 
\family typewriter
out.width
\family default
 to be less than half of the linewidth if you do not want the sub figures
 to exceed the page margin;
\end_layout

\begin_layout Enumerate
most importantly, leave a blank line in the beginning of the subfloat environmen
t, so that 
\family typewriter
<<>>=
\family default
 starts in a new line and 
\series bold
knitr
\series default
 can recognize it;
\end_layout

\begin_layout Standard
Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:knitr-subfloat"

\end_inset

 is an example.
\end_layout

\begin_layout Standard
This document was compiled with 
\series bold
knitr
\series default
 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
Sexpr{packageVersion('knitr')}
\end_layout

\end_inset

.
 Rnw source at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/examples/knitr-subfloats.Rnw
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

<<test-subfloat, echo=FALSE, out.width='.45
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

plot(1)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
this is a subfloat
\end_layout

\end_inset


\end_layout

\end_inset


\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

<<test-another, echo=FALSE, out.width='.45
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

plot(rnorm(100))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
another float
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
one figure with 2 sub figures
\begin_inset CommandInset label
LatexCommand label
name "fig:knitr-subfloat"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\end_body
\end_document

#LyX 2.1 created this file. For more info see http://www.lyx.org/
\lyxformat 474
\begin_document
\begin_header
\textclass beamer
\begin_preamble
\usetheme{PaloAlto}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_math auto
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize 10
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered false
\pdf_bookmarksopen false
\pdf_bookmarksopenlevel 1
\pdf_breaklinks false
\pdf_pdfborder true
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 1
\use_package mathtools 1
\use_package mhchem 1
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine basic
\cite_engine_type default
\biblio_style plain
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/beamer-',fig.align='center',fig.show='hold',size='f
ootnotesize')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
\begin_inset Argument 1
status collapsed

\begin_layout Plain Layout
knitr, Beamer, and FragileFrame
\end_layout

\end_inset

A Minimal Demo of knitr with Beamer and Fragile Frames
\end_layout

\begin_layout Author
Yihui Xie
\begin_inset Foot
status open

\begin_layout Plain Layout
I thank Richard E.
 Goldberg for providing this demo.
\end_layout

\end_inset


\end_layout

\begin_layout Frame
\begin_inset Argument 4
status open

\begin_layout Plain Layout
Background
\end_layout

\end_inset


\end_layout

\begin_deeper
\begin_layout Itemize
The 
\series bold
knitr
\emph on
 
\series default
\emph default
package allows you to embed R code and figures in LaTeX documents
\end_layout

\begin_deeper
\begin_layout Itemize
It has functionality similar to Sweave but looks nicer and gives you more
 control
\end_layout

\end_deeper
\begin_layout Itemize
If you already have Sweave working in LyX, getting 
\series bold
knitr
\series default
 to work is trivial
\end_layout

\begin_deeper
\begin_layout Enumerate
Install the 
\series bold
knitr
\series default
 package in 
\emph on
R
\end_layout

\begin_layout Enumerate
Read 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/lyx/
\end_layout

\end_inset


\end_layout

\end_deeper
\begin_layout Itemize
If you use Sweave or 
\series bold
knitr
\series default
 with Beamer in LyX, you must use the
\emph on
 FragileFrame
\emph default
 environment for the frames that contain R code chunks.
 Let's see if 
\series bold
knitr
\series default
 works with Beamer in this small demo.
 
\end_layout

\end_deeper
\begin_layout Section
First Test
\end_layout

\begin_layout FragileFrame
\begin_inset Argument 4
status open

\begin_layout Plain Layout
First Test
\end_layout

\end_inset


\end_layout

\begin_deeper
\begin_layout Standard
OK, let's get started with just some text:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=FALSE,results='hide'>>=
\end_layout

\begin_layout Plain Layout

# some setup
\end_layout

\begin_layout Plain Layout

options(width=60)  # make the printing fit on the page
\end_layout

\begin_layout Plain Layout

set.seed(1121)   # make the results repeatable
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

<<>>= 
\end_layout

\begin_layout Plain Layout

# create some random numbers
\end_layout

\begin_layout Plain Layout

(x=rnorm(20))  
\end_layout

\begin_layout Plain Layout

mean(x);var(x)  
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
BTW, the first element of 
\family typewriter
x
\family default
 is 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
Sexpr{x[1]}
\end_layout

\end_inset

.
 (Did you notice the use of
\family typewriter
 
\backslash
Sexpr{}
\family default
?)
\end_layout

\end_deeper
\begin_layout Section
Second Test
\end_layout

\begin_layout FragileFrame
\begin_inset Argument 4
status open

\begin_layout Plain Layout
Second Test
\end_layout

\end_inset


\end_layout

\begin_deeper
\begin_layout Standard
Text is nice but let's see what happens if we make a couple of plots in
 our chunk:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-plots,fig.width=4,fig.height=4,out.width='.45
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

par(las=1,mar=c(4,4,.1,.1))  # tick labels direction
\end_layout

\begin_layout Plain Layout

boxplot(x) 
\end_layout

\begin_layout Plain Layout

hist(x,main='',col="blue",probability=TRUE) 
\end_layout

\begin_layout Plain Layout

lines(density(x),col="red")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_deeper
\begin_layout Section
The Big Question
\end_layout

\begin_layout Frame
\begin_inset Argument 4
status open

\begin_layout Plain Layout
The Big Question
\end_layout

\end_inset


\end_layout

\begin_deeper
\begin_layout Standard
Do the above chunks work? You should be able to compile the LyX document
 and get a nice-looking PDF slide presentation.
 If not, time to double-check everything...
\end_layout

\end_deeper
\end_body
\end_document

#LyX 2.1 created this file. For more info see http://www.lyx.org/
\lyxformat 474
\begin_document
\begin_header
\textclass article
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[buttonsize=1em]{animate}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans lmss
\font_typewriter lmtt
\font_math auto
\font_default_family default
\use_non_tex_fonts false
\font_sc true
\font_osf false
\font_sf_scale 100
\font_tt_scale 100
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview={XYZ null null 1}"
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 1
\use_package mathtools 1
\use_package mhchem 1
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine natbib
\cite_engine_type authoryear
\biblio_style plainnat
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 2.5cm
\topmargin 2.5cm
\rightmargin 2.5cm
\bottommargin 2.5cm
\secnumdepth 2
\tocdepth 2
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

## set global chunk options
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/manual-', cache.path='cache/manual-', fig.align='ce
nter', fig.show='hold', par=TRUE)
\end_layout

\begin_layout Plain Layout

## I use = but I can replace it with <-; set code/output width to be 68
\end_layout

\begin_layout Plain Layout

options(formatR.arrow=TRUE, width=68, digits=4)
\end_layout

\begin_layout Plain Layout

## tune details of base graphics (https://yihui.org/knitr/hooks)
\end_layout

\begin_layout Plain Layout

knit_hooks$set(par=function(before, options, envir){
\end_layout

\begin_layout Plain Layout

if (before && options$fig.show!='none') par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mg
p=c(2,.7,0),tcl=-.3)
\end_layout

\begin_layout Plain Layout

})
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
knitr: A General-Purpose Tool for Dynamic Report Generation in R
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
The original paradigm of literate programming was brought forward mainly
 for software development, or specifically, to mix source code (for computer)
 and documentation (for human) together.
 Early systems include 
\begin_inset CommandInset href
LatexCommand href
name "WEB"
target "http://www.literateprogramming.com/web.pdf"

\end_inset

 and 
\begin_inset CommandInset href
LatexCommand href
name "Noweb"
target "http://www.cs.tufts.edu/~nr/noweb/"

\end_inset

; Sweave 
\begin_inset CommandInset citation
LatexCommand citep
key "leisch2002"

\end_inset

 was derived from the latter, but it is less focused on documenting software,
 instead it is mainly used for reproducible data analysis and generating
 statistical reports.
 The 
\series bold
knitr
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-knitr"

\end_inset

 is following the steps of Sweave.
 For this manual, I assume readers have some background knowledge of Sweave
 to understand the technical details; for a reference of available options,
 hooks and demos, see the package homepage 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/
\end_layout

\end_inset

.
\end_layout

\begin_layout Section
Hello World
\end_layout

\begin_layout Standard
A natural question is why to reinvent the wheel.
 The short answer is that extending Sweave by hacking 
\family sans
SweaveDrivers.R
\family default
 in the 
\series bold
utils
\series default
 package is a difficult job to me.
 Many features in 
\series bold
knitr
\series default
 come naturally as users would have expected.
 Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:cars-demo"

\end_inset

 is a simple demo of some features of 
\series bold
knitr
\series default
.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cars-demo,dev='tikz',fig.width=4,fig.height=2.8,out.width='.45
\backslash

\backslash
textwidth',message=FALSE,cache=TRUE>>=
\end_layout

\begin_layout Plain Layout

fit=lm(dist~speed,data=cars) # linear regression
\end_layout

\begin_layout Plain Layout

par(mar=c(4, 4, 1, .1), mgp=c(2,1,0))
\end_layout

\begin_layout Plain Layout

with(cars,plot(speed,dist,panel.last=abline(fit)))
\end_layout

\begin_layout Plain Layout

text(10,100,'$Y = 
\backslash

\backslash
beta_0 + 
\backslash

\backslash
beta_1x + 
\backslash

\backslash
epsilon$')
\end_layout

\begin_layout Plain Layout

library(ggplot2)
\end_layout

\begin_layout Plain Layout

qplot(speed, dist, data=cars)+geom_smooth()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
\begin_inset CommandInset label
LatexCommand label
name "fig:cars-demo"

\end_inset

A simple demo of possible output in 
\series bold
knitr
\series default
: (1) multiple plots per chunk; (2) no need to 
\emph on
print()
\emph default
 objects in 
\series bold
ggplot2
\series default
; (3) device size is 
\begin_inset Formula $4\times2.8$
\end_inset

 (inches) but output size is adjusted to 
\family typewriter
.45
\backslash
textwidth
\family default
 in chunk options; (4) base graphics and 
\series bold
ggplot2
\series default
 can sit side by side; (5) use the 
\emph on
tikz()
\emph default
 device in 
\series bold
tikzDevice
\series default
 by setting chunk option 
\family typewriter
dev='tikz'
\family default
 (hence can write native LaTeX expressions in R plots); (6) code highlighting.
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
I would have chosen to hide the R code if this were a real report, but here
 I show the code just for the sake of demonstration.
 If we type 
\emph on
qplot()
\emph default
 in R, we get a plot, and the same thing happens in 
\series bold
knitr
\series default
.
 If we draw two plots in the code, 
\series bold
knitr
\series default
 will show two plots and we do not need to tell it how many plots are there
 in the code in advance.
 If we set 
\family typewriter
out.width='.49
\backslash

\backslash
textwidth'
\family default
 in chunk options, we get it in the final output document.
 If we say 
\family typewriter
fig.align='center'
\family default
, the plots are centered.
 That's it.
 Many enhancements and new features will be introduced later.
 If you come from the Sweave land, you can take a look at the page of transition
 first: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/sweave/
\end_layout

\end_inset

.
\end_layout

\begin_layout Section
Design
\end_layout

\begin_layout Standard
The flow of processing an input file is similar to Sweave, and two major
 differences are that 
\series bold
knitr
\series default
 provides more flexibility to the users to customize the processing, and
 has many built-in options such as the support to a wide range of graphics
 devices and cache.
 Below is a brief description of the process:
\end_layout

\begin_layout Enumerate

\series bold
knitr
\series default
 takes an input file and automatically determines an appropriate set of
 
\begin_inset CommandInset href
LatexCommand href
name "patterns"
target "https://yihui.org/knitr/patterns/"

\end_inset

 to use if they are not provided in advance (e.g.
 
\family sans
file.Rnw
\family default
 will use 
\family typewriter
knit_patterns$get('rnw')
\family default
);
\end_layout

\begin_layout Enumerate
a set of output 
\begin_inset CommandInset href
LatexCommand href
name "hooks"
target "https://yihui.org/knitr/hooks/"

\end_inset

 will also be set up automatically according to the filename extension (e.g.
 use LaTeX environments or HTML elements to wrap up R results);
\end_layout

\begin_layout Enumerate
the input file is read in and split into pieces consisting of R code chunks
 and normal texts; the former will be executed one after the other, and
 the latter may contain global chunk options or inline R code;
\end_layout

\begin_layout Enumerate
for each chunk, the code is evaluated using the 
\series bold
evaluate
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-evaluate"

\end_inset

, and the results may be filtered according to chunk options (e.g.
 
\family typewriter
echo=FALSE
\family default
 will remove the R source code)
\end_layout

\begin_deeper
\begin_layout Enumerate
if 
\family typewriter
cache=TRUE
\family default
 for this chunk, 
\series bold
knitr
\series default
 will first check if there are previously cached results under the cache
 directory before really evaluating the chunk; if cached results exist and
 this code chunk has not been changed since last run (use MD5 sum to verify),
 the cached results will be (lazy-) loaded, otherwise new cache will be
 built; if a cached chunk depends on other chunks (see the 
\family typewriter
dependson
\family default
 
\begin_inset CommandInset href
LatexCommand href
name "option"
target "https://yihui.org/knitr/options/"

\end_inset

) and any one of these chunks has changed, this chunk must be forcibly updated
 (old cache will be purged)
\end_layout

\begin_layout Enumerate
there are six types of possible output from 
\series bold
evaluate
\series default
, and their classes are 
\family typewriter
character
\family default
 (normal text output), 
\family typewriter
source
\family default
 (source code), 
\family typewriter
warning
\family default
, 
\family typewriter
message
\family default
, 
\family typewriter
error
\family default
 and 
\family typewriter
recordedplot
\family default
; an internal S3 generic function 
\emph on
wrap()
\emph default
 is used to deal with different types of output, using output hooks defined
 in the object 
\family typewriter
knit_hooks
\end_layout

\begin_layout Enumerate
note plots are recorded as R objects before they are really saved to files,
 so graphics devices will not be opened unless plots have really been produced
 in a chunk
\end_layout

\begin_layout Enumerate
a code chunk is evaluated in a separate empty environment with the global
 environment as its parent, and all the objects in this environment after
 the evaluation will be saved if 
\family typewriter
cache=TRUE
\end_layout

\begin_layout Enumerate
chunk hooks can be run before and/or after a chunk
\end_layout

\end_deeper
\begin_layout Enumerate
for normal texts, 
\series bold
knitr
\series default
 will find inline R code (e.g.
 in 
\family typewriter

\backslash
Sexpr{}
\family default
) and evaluate it; the output is wrapped by the 
\family typewriter
inline
\family default
 hook;
\end_layout

\begin_layout Standard
The hooks play important roles in 
\series bold
knitr
\series default
: this package makes almost everything accessible to the users.
 Consider the following extremely simple example which may demonstrate this
 freedom:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<simple-example>>=
\end_layout

\begin_layout Plain Layout

1+1
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
There are two parts in the final output: the source code 
\family typewriter
1 + 1
\family default
 and the output 
\family typewriter
[1] 2
\family default
; the comment characters 
\family typewriter
##
\family default
 are from the default chunk option 
\family typewriter
comment
\family default
.
 Users may define a hook function for the source code like this to use the
 
\family typewriter
lstlisting
\family default
 environment:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<hook-source, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

knit_hooks$set(source = function(x, options) {
\end_layout

\begin_layout Plain Layout

paste('
\backslash

\backslash
begin{lstlisting}
\backslash
n', x, '
\backslash

\backslash
end{lstlisting}
\backslash
n', sep = '')
\end_layout

\begin_layout Plain Layout

})
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Similarly we can put other types of output into other environments.
 There is no need to hack at 
\family sans
Sweave.sty
\family default
 for 
\series bold
knitr
\series default
 and you can put the output in any environments.
 What is more, the output hooks make 
\series bold
knitr
\series default
 ready for other types of output, and a typical one is HTML (there are built-in
 hooks).
 The website has provided many examples demonstrating the flexibility of
 the output.
\end_layout

\begin_layout Section
Features
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package borrowed features such as tikz graphics and cache from 
\series bold
pgfSweave
\series default
 and 
\series bold
cacheSweave
\series default
 respectively, but the implementations are different.
 New features like code reference from an external R script as well as output
 customization are also introduced.
 The feature of hook functions in Sweave is re-implemented and hooks have
 new usage now.
 There are several other small features which are motivated from my everyday
 use of Sweave.
 For example, a progress bar is provided when knitting a file so we roughly
 know how long we still need to wait; output from inline R code (e.g.
 
\family typewriter

\backslash
Sexpr{x[1]}
\family default
) is automatically formatted in TeX math notation (like 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
Sexpr{123456789}
\end_layout

\end_inset

) if the result is numeric.
 You may check out a number of specific manuals dedicated to specific features
 such as graphics in the website: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/
\end_layout

\end_inset

.
\end_layout

\begin_layout Subsection
Code Decoration
\end_layout

\begin_layout Standard
The 
\series bold
highr
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-highr"

\end_inset

 is used to highlight R code, and the 
\series bold
formatR
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-formatR"

\end_inset

 is used to reformat R code (like 
\family typewriter
keep.source=FALSE
\family default
 in Sweave but will also try to retain comments).
 For LaTeX output, the 
\series bold
framed
\series default
 package is used to decorate code chunks with a light gray background.
 If this LaTeX package is not found in the system, a version will be copied
 directly from 
\series bold
knitr
\series default
.
 The prompt characters are removed by default because they mangle the R
 source code in the output and make it difficult to copy R code.
 The R output is masked in comments by default based on the same rationale.
 It is easy to revert to the output with prompts (set option 
\family typewriter
prompt=TRUE
\family default
), and you will quickly realize the inconvenience to the readers if they
 want to copy and run the code in the output document:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<stupid-prompts, prompt=TRUE, comment=NA, highlight=FALSE>>=
\end_layout

\begin_layout Plain Layout

x=rnorm(5)
\end_layout

\begin_layout Plain Layout

x
\end_layout

\begin_layout Plain Layout

var(x)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The example below shows the effect of 
\family typewriter
tidy=TRUE/FALSE
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<tidy-no, eval=FALSE, tidy=FALSE>>=
\end_layout

\begin_layout Plain Layout

## option tidy=FALSE
\end_layout

\begin_layout Plain Layout

for(k in 1:10){j=cos(sin(k)*k^2)+3;print(j-5)}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout

<<tidy-yes, eval=FALSE, tidy=TRUE>>=
\end_layout

\begin_layout Plain Layout

## option tidy=TRUE
\end_layout

\begin_layout Plain Layout

for(k in 1:10){j=cos(sin(k)*k^2)+3;print(j-5)}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Note 
\family typewriter
=
\family default
 is replaced by 
\family typewriter
<-
\family default
 because 
\family typewriter
options('formatR.arrow')
\family default
 was set to be 
\family typewriter
TRUE
\family default
 in this document; see the documentation of 
\emph on
tidy.source()
\emph default
 in 
\series bold
formatR
\series default
 for details.
\end_layout

\begin_layout Standard
Many highlighting themes can be used in 
\series bold
knitr
\series default
, which are borrowed from the 
\series bold
highlight
\series default
 package by 
\begin_inset CommandInset href
LatexCommand href
name "Andre Simon"
target "http://www.andre-simon.de/"

\end_inset


\begin_inset Foot
status open

\begin_layout Plain Layout
not the R package mentioned before; for a preview of these themes, see 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.andre-simon.de/dokuwiki/doku.php?id=theme_examples
\end_layout

\end_inset


\end_layout

\end_inset

; it is also possible to use themes from 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.eclipsecolorthemes.org/
\end_layout

\end_inset

 by providing a theme id to 
\series bold
knitr
\series default

\begin_inset Foot
status open

\begin_layout Plain Layout
many thanks to 
\begin_inset CommandInset href
LatexCommand href
name "Ramnath Vaidyanathan"
target "https://github.com/ramnathv"

\end_inset

 for the work on themes
\end_layout

\end_inset

.
 See 
\family typewriter
?knit_theme
\family default
 for details.
\end_layout

\begin_layout Subsection
Graphics
\end_layout

\begin_layout Standard
Graphics is an important part of reports, and several enhancements have
 been made in 
\series bold
knitr
\series default
.
 For example, grid graphics may not need to be explicitly printed as long
 as the same code can produce plots in R (in some cases, however, they have
 to be printed, e.g.
 in a loop, because you have to do so in an R terminal).
\end_layout

\begin_layout Subsubsection
Graphical Devices
\end_layout

\begin_layout Standard
Over a long time, a frequently requested feature for Sweave was the support
 for other graphics devices, which has been implemented since R 2.13.0.
 Instead of using logical options like 
\family typewriter
png
\family default
 or 
\family typewriter
jpeg
\family default
 (this list can go on and on), 
\series bold
knitr
\series default
 uses a single option 
\family typewriter
dev
\family default
 (like 
\family typewriter
grdevice
\family default
 in Sweave) which has support for more than 20 devices.
 For instance, 
\family typewriter
dev='png'
\family default
 will use the 
\emph on
png()
\emph default
 device, and 
\family typewriter
dev='CairoJPEG'
\family default
 uses the 
\emph on
CairoJPEG()
\emph default
 device in the 
\series bold
Cairo
\series default
 package (it has to be installed first, of course).
 If none of these devices is satisfactory, you can provide the name of a
 customized device function, which must have been defined before it is called.
\end_layout

\begin_layout Subsubsection
Plot Recording
\end_layout

\begin_layout Standard
As mentioned before, all the plots in a code chunk are first recorded as
 R objects and then 
\begin_inset Quotes eld
\end_inset

replayed
\begin_inset Quotes erd
\end_inset

 inside a graphical device to generate plot files.
 The 
\series bold
evaluate
\series default
 package will record plots per 
\emph on
expression
\emph default
 basis, in other words, the source code is split into individual complete
 expressions and 
\series bold
evaluate
\series default
 will examine possible plot changes in snapshots after each single expression
 has been evaluated.
 For example, the code below consists of three expressions, out of which
 two are related to drawing plots, therefore 
\series bold
evaluate
\series default
 will produce two plots by default:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<low-level-plots, fig.keep='all', dev='tikz', fig.width=2.5, fig.height=2.5,
 out.width='.3
\backslash

\backslash
textwidth', cache=TRUE>>=
\end_layout

\begin_layout Plain Layout

par(mar=c(3,3,.1,.1))
\end_layout

\begin_layout Plain Layout

plot(1:10, ann=FALSE,las=1)
\end_layout

\begin_layout Plain Layout

text(5,9,'mass $
\backslash

\backslash
rightarrow$ energy
\backslash
n$E=mc^2$')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This brings a significant difference with traditional tools in R for dynamic
 report generation, since low-level plotting changes can also be recorded.
 The option 
\family typewriter
fig.keep
\family default
 controls which plots to keep in the output; 
\family typewriter
fig.keep='all'
\family default
 will keep low-level changes as separate plots; by default (
\family typewriter
fig.keep='high'
\family default
), 
\series bold
knitr
\series default
 will merge low-level plot changes into the previous high-level plot, like
 most graphics devices do.
 This feature may be useful for teaching R graphics step by step.
 Note, however, low-level plotting commands in a single expression (a typical
 case is a loop) will not be recorded accumulatively, but high-level plotting
 commands, regardless of where they are, will always be recorded.
 For example, this chunk will only produce 2 plots instead of 21 plots because
 there are 2 complete expressions:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<low-plot-loop, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

plot(0,0,type='n',ann=FALSE)
\end_layout

\begin_layout Plain Layout

for(i in seq(0, 2*pi,length=20)) points(cos(i),sin(i))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
But this will produce 20 plots as expected:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<high-plot-loop, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

for(i in seq(0, 2*pi,length=20)) {plot(cos(i),sin(i),xlim=c(-1,1),ylim=c(-1,1))}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
As I showed in the beginning of this manual, it is straightforward to let
 
\series bold
knitr
\series default
 keep all the plots in a chunk and insert them into the output document,
 so we no longer need the 
\family typewriter
cat('
\backslash

\backslash
includegraphics{}')
\family default
 trick.
\end_layout

\begin_layout Standard
We can discard all previous plots and keep the last one only by 
\family typewriter
fig.keep='last'
\family default
, or keep only the first plot by 
\family typewriter
fig.keep='first'
\family default
, or discard all plots by 
\family typewriter
fig.keep='none'
\family default
.
\end_layout

\begin_layout Subsubsection
Plot Rearrangement
\end_layout

\begin_layout Standard
The option 
\family typewriter
fig.show
\family default
 can decide whether to hold all plots while evaluating the code and 
\begin_inset Quotes eld
\end_inset

flush
\begin_inset Quotes erd
\end_inset

 all of them to the end of a chunk (
\family typewriter
fig.show='hold'
\family default
), or just insert them to the place where they were created (by default
 
\family typewriter
fig.show='asis'
\family default
).
 Here is an example of 
\family typewriter
fig.show='asis'
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fig-hold, fig.show='asis', dev='pdf', fig.width=6, fig.height=4, out.width='.35
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

contour(volcano)  # contour lines
\end_layout

\begin_layout Plain Layout

filled.contour(volcano)  # fill contour plot with colors
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Beside 
\family typewriter
hold
\family default
 and 
\family typewriter
asis
\family default
, the option 
\family typewriter
fig.show
\family default
 can take a third value: 
\family typewriter
animate
\family default
, which makes it possible to insert animations into the output document.
 In LaTeX, the package 
\series bold
animate
\series default
 is used to put together image frames as an animation.
 For animations to work, there must be more than one plot produced in a
 chunk.
 The option 
\family typewriter
interval
\family default
 controls the time interval between animation frames; by default it is 1
 second.
 Note you have to add 
\family typewriter

\backslash
usepackage{animate}
\family default
 in the LaTeX preamble, because 
\series bold
knitr
\series default
 will not add it automatically.
 Animations in the PDF output can only be viewed in Adobe Reader.
\end_layout

\begin_layout Standard
As a simple demonstration, here is a 
\begin_inset CommandInset href
LatexCommand href
name "Mandelbrot animation"
target "http://en.wikipedia.org/wiki/Mandelbrot_set"

\end_inset

 taken from the 
\series bold
animation
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-animation"

\end_inset

; note the PNG device is used because PDF files are too large.
 You should be able to see the animation immediately with Acrobat Reader
 since it was set to play automatically:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<animate-demo, fig.show='animate', dev='png', out.width='.45
\backslash

\backslash
linewidth', interval=.5, aniopts='controls,loop,autoplay',  cache=TRUE>>=
\end_layout

\begin_layout Plain Layout

library(animation)
\end_layout

\begin_layout Plain Layout

demo('Mandelbrot', echo = FALSE, package = 'animation')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsubsection
Plot Size
\end_layout

\begin_layout Standard
The 
\family typewriter
fig.width
\family default
 and 
\family typewriter
fig.height
\family default
 options specify the size of plots in the graphics device, and the real
 size in the output document can be different (see 
\family typewriter
out.width
\family default
 and 
\family typewriter
out.height
\family default
).
 When there are multiple plots per chunk, it is possible to arrange more
 than one plot per line in LaTeX -- just specify 
\family typewriter
out.width
\family default
 to be less than half of the current line width, e.g.
 
\family typewriter
out.width='.49
\backslash

\backslash
linewidth'
\family default
.
\end_layout

\begin_layout Subsubsection
The tikz Device
\end_layout

\begin_layout Standard
Beside PDF, PNG and other traditional R graphical devices, 
\series bold
knitr
\series default
 has special support to tikz graphics via the 
\series bold
tikzDevice
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-tikzDevice"

\end_inset

, which is similar to 
\series bold
pgfSweave
\series default
.
 If we set the chunk option 
\family typewriter
dev='tikz'
\family default
, the 
\emph on
tikz()
\emph default
 device in 
\series bold
tikzDevice
\series default
 will be used to save plots.
 Options 
\family typewriter
sanitize
\family default
 and 
\family typewriter
external
\family default
 are related to the tikz device: see the documentation of 
\emph on
tikz()
\emph default
 for details.
 Note 
\family typewriter
external=TRUE
\family default
 in 
\series bold
knitr
\series default
 has a different meaning with 
\series bold
pgfSweave
\series default
 -- it means 
\family typewriter
standAlone=TRUE
\family default
 in 
\emph on
tikz()
\emph default
, and the tikz graphics output will be compiled to PDF 
\emph on
immediately
\emph default
 after it is created, so the 
\begin_inset Quotes eld
\end_inset

externalization
\begin_inset Quotes erd
\end_inset

 does not depend on the 
\series bold
tikz
\series default
 package; to maintain consistency in (font) styles, 
\series bold
knitr
\series default
 will read the preamble of the input document and use it in the tikz device.
 At the moment, I'm not sure if this is a faithful way to externalize tikz
 graphics, but I have not seen any problems so far.
 The assumption to make, however, is that you declare all the styles in
 the preamble; 
\series bold
knitr
\series default
 is agnostic of 
\emph on
local
\emph default
 style changes in the body of the document.
\end_layout

\begin_layout Standard
Below is an example taken from StackOverflow
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://stackoverflow.com/q/8190087/559676
\end_layout

\end_inset


\end_layout

\end_inset

; we usually have to write R code like this to obtain a math expression
 
\begin_inset Formula $\mathrm{d}\mathbf{x}_{t}=\alpha[(\theta-\mathbf{x}_{t})\mathrm{d}t+4]\mathrm{d}B_{t}$
\end_inset

 in R graphics:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<math-expr-R, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

qplot(1:10, 1:10) + opts(title = substitute(paste(d * 
\end_layout

\begin_layout Plain Layout

    bolditalic(x)[italic(t)] == alpha * (theta - bolditalic(x)[italic(t)])
 * 
\end_layout

\begin_layout Plain Layout

    d * italic(t) + lambda * d * italic(B)[italic(t)]), list(lambda = 4)))
 
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
With the tikz device, it is both straightforward and more beautiful: 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<math-expr-tikz, dev='tikz', fig.width=5, fig.height=3, out.width='.55
\backslash

\backslash
linewidth', cache=TRUE, message=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(ggplot2)
\end_layout

\begin_layout Plain Layout

qplot(1:10, 1:10) + 
\end_layout

\begin_layout Plain Layout

labs(title = sprintf('$
\backslash

\backslash
mathrm{d}
\backslash

\backslash
mathbf{x}_{t} = 
\backslash

\backslash
alpha[(
\backslash

\backslash
theta - 
\backslash

\backslash
mathbf{x}_{t})
\backslash

\backslash
mathrm{d}t + %d]
\backslash

\backslash
mathrm{d}B_{t}$', 4))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The advantage of tikz graphics is the consistency of styles
\begin_inset Foot
status collapsed

\begin_layout Plain Layout
Users are encouraged to read the vignette of 
\series bold
tikzDevice
\series default
, which is the most beautiful vignette I have ever seen in R packages: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://cran.r-project.org/web/packages/tikzDevice/vignettes/tikzDevice.pdf
\end_layout

\end_inset


\end_layout

\end_inset

, and one disadvantage is that LaTeX may not be able to handle too large
 tikz files (it can run out of memory).
 For example, an R plot with tens of thousands of graphical elements may
 fail to compile in LaTeX if we use the tikz device.
 In such cases, we can switch to the PDF or PNG device, or reconsider our
 decision on the type of plots, e.g., a scatter plot with millions of points
 is usually difficult to read, and a contour plot or a hexagon plot showing
 the 2D density can be a better alternative (they are smaller in size).
\end_layout

\begin_layout Standard
The graphics manual contains more detailed information and you can check
 it out in the 
\begin_inset CommandInset href
LatexCommand href
name "website"
target "https://yihui.org/knitr/demo/graphics/"

\end_inset

.
\end_layout

\begin_layout Subsection
Cache
\end_layout

\begin_layout Standard
The feature of cache is not a new idea -- both 
\series bold
cacheSweave
\series default
 and 
\series bold
weaver
\series default
 have implemented it based on Sweave, with the former using 
\series bold
filehash
\series default
 and the latter using 
\family sans
.RData
\family default
 images; 
\series bold
cacheSweave
\series default
 also supports lazy-loading of objects based on 
\series bold
filehash
\series default
.
 The 
\series bold
knitr
\series default
 package directly uses internal base R functions to save (
\emph on
tools:::makeLazyLoadDB()
\emph default
) and lazy-load objects (
\emph on
lazyLoad()
\emph default
).
 These functions are either undocumented or marked as internal, but as far
 as I understand, they are the tools to implement lazy-loading for packages.
 The 
\series bold
cacheSweave
\series default
 vignette has clearly explained lazy-loading, and roughly speaking, lazy-loading
 means an object will not be really loaded into memory unless it is really
 used somewhere.
 This is very useful for cache; sometimes we read a large object and cache
 it, then take a subset for analysis and this subset is also cached; in
 the future, the initial large object will not be loaded into R if our computati
on is only based on the object of its subset.
\end_layout

\begin_layout Standard
The paths of cache files are determined by the chunk option 
\family typewriter
cache.path
\family default
; by default all cache files are created under a directory 
\family sans
cache
\family default
 relative to the current working directory, and if the option value contains
 a directory (e.g.
 
\family typewriter
cache.path='cache/abc-'
\family default
), cache files will be stored under that directory (automatically created
 if it does not exist).
 The cache is invalidated and purged on any changes to the code chunk, including
 both the R code and chunk options
\begin_inset Foot
status open

\begin_layout Plain Layout
One exception is the 
\family typewriter
include
\family default
 option, which is not cached because 
\family typewriter
include=TRUE/FALSE
\family default
 does not affect code evaluation; meanwhile, the value 
\family typewriter
getOption('width')
\family default
 is also cached, so if you change this option, the cache will also be invalidate
d (this option affects the width of text output)
\end_layout

\end_inset

; this means previous cache files of this chunk are removed (filenames are
 identified by the chunk label).
 Unlike 
\series bold
pgfSweave
\series default
, cache files will never accumulate since old cache files will always be
 removed in 
\series bold
knitr
\series default
.
 Unlike 
\series bold
weaver
\series default
 or 
\series bold
cacheSweave
\series default
, 
\series bold
knitr
\series default
 will try to preserve these side-effects:
\end_layout

\begin_layout Enumerate
printed results: meaning that any output of a code chunk will be loaded
 into the output document for a cached chunk, although it is not really
 evaluated.
 The reason is 
\series bold
knitr
\series default
 also cache the output of a chunk as a character string.
 Note this means graphics output is also cached since it is part of the
 output.
 It has been a pain for me for a long time to have to lose output to gain
 cache;
\end_layout

\begin_layout Enumerate
loaded packages: after the evaluation of each cached chunk, the list of
 packages used in the current R session is written to a file under the cache
 path named 
\family sans
__packages
\family default
; next time if a cached chunk needs to be rebuilt, these packages will be
 loaded first.
 The reasons for caching package names are, it can be slow to load some
 packages, and a package might be loaded in a previous cached chunk which
 is not available to the next cached chunk when only the latter needs to
 be rebuilt.
 Note this only applies to cached chunks, and for uncached chunks, you must
 always use 
\emph on
library()
\emph default
 to load packages explicitly;
\end_layout

\begin_layout Standard
Although 
\series bold
knitr
\series default
 tries to keep some side-effects, there are still other types of side-effects
 like setting 
\emph on
par()
\emph default
 or 
\emph on
options()
\emph default
 which are not cached.
 Users should be aware of these special cases, and make sure to clearly
 separate the code which is not meant to be cached to other chunks which
 are not cached, e.g., set all global options in the first chunk of a document
 and do not cache that chunk.
\end_layout

\begin_layout Standard
Sometimes a cached chunk may need to use objects from other cached chunks,
 which can bring a serious problem -- if objects in previous chunks have
 changed, this chunk will not be aware of the changes and will still use
 old cached results, unless there is a way to detect such changes from other
 chunks.
 There is an option called 
\family typewriter
dependson
\family default
 in 
\series bold
cacheSweave
\series default
 which does this job.
 We can explicitly specify which other chunks this chunk depends on by setting
 an option like 
\family typewriter
dependson='chunkA;chunkB'
\family default
 or equivalently 
\family typewriter
dependson=c('chunkA', 'chunkB')
\family default
.
 Each time the cache of a chunk is rebuilt, all other chunks which depend
 on this chunk will lose cache, hence their cache will be rebuilt as well.
\end_layout

\begin_layout Standard
Another way to specify the dependencies among chunks is to use the chunk
 option 
\family typewriter
autodep
\family default
 and the function 
\emph on
dep_auto()
\emph default
.
 This is an experimental feature borrowed from 
\series bold
weaver
\series default
 which frees us from setting chunk dependencies manually.
 The basic idea is, if a latter chunk uses any objects created from a previous
 chunk, the latter chunk is said to depend on the previous one.
 The function 
\emph on
findGlobals()
\emph default
 in the 
\series bold
codetools
\series default
 package is used to find out all global objects in a chunk, and according
 to its documentation, the result is an approximation.
 Global objects roughly mean the ones which are not created locally, e.g.
 in the expression 
\family typewriter
function() {y <- x}
\family default
, 
\family typewriter
x
\family default
 should be a global object, whereas 
\family typewriter
y
\family default
 is local.
 Meanwhile, we also need to save the list of objects created in each cached
 chunk, so that we can compare them to the global objects in latter chunks.
 For example, if chunk A created an object 
\family typewriter
x
\family default
 and chunk B uses this object, chunk B must depend on A, i.e.
 whenever A changes, B must also be updated.
 When 
\family typewriter
autodep=TRUE
\family default
, 
\series bold
knitr
\series default
 will write out the names of objects created in a cached chunk as well as
 those global objects in two files named 
\family sans
__objects
\family default
 and 
\family sans
__globals
\family default
 respectively; later we can use the function 
\emph on
dep_auto()
\emph default
 to analyze the object names to figure out the dependencies automatically.
 See 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/cache/
\end_layout

\end_inset

 for examples.
\end_layout

\begin_layout Standard
Yet another way to specify dependencies is 
\emph on
dep_prev()
\emph default
: this is a conservative approach which sets the dependencies so that a
 cached chunk will depend on all its previous chunks, i.e.
 whenever a previous chunk is updated, all later chunks will be updated
 accordingly.
\end_layout

\begin_layout Subsection
Code Externalization
\end_layout

\begin_layout Standard
It can be more convenient to write R code in a separate file, rather than
 mixing it into a LaTeX document; for example, we can run R code successively
 in a pure R script from one chunk to the other without jumping through
 other texts.
 Since I prefer using LyX to write reports, Sweave is even more inconvenient
 because I have to recompile the whole document each time, even if I only
 want to know the results of a single chunk.
 Therefore 
\series bold
knitr
\series default
 introduced the feature of code externalization to a separate R script.
 Currently the setting is like this: the R script also uses chunk labels
 (marked in the form 
\family typewriter
## ---- chunk-label
\family default
 by default); if the code chunk in the input document is empty, 
\series bold
knitr
\series default
 will match its label with the label in the R script to input external R
 code.
 For example, suppose this is a code chunk labelled as 
\family typewriter
Q1
\family default
 in an R script named 
\family sans
homework1-xie.R
\family default
 which is under the same directory as the Rnw document:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ext-r-code, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

## ---- Q1 ---------------------
\end_layout

\begin_layout Plain Layout

gcd = function(m, n) {
\end_layout

\begin_layout Plain Layout

    while ((r <- m %% n) != 0) {
\end_layout

\begin_layout Plain Layout

       m = n; n = r
\end_layout

\begin_layout Plain Layout

    }
\end_layout

\begin_layout Plain Layout

    n
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In the Rnw document, we can first read the script using the function 
\emph on
read_chunk()
\emph default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<read-chunk, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

read_chunk('homework1-xie.R')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This is usually done in an early chunk, and we can use the chunk 
\family typewriter
Q1
\family default
 later in the Rnw document:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<use-ext-chunk, echo=FALSE, comment=NA>>=
\end_layout

\begin_layout Plain Layout

cat('<<Q1, echo=TRUE, tidy=TRUE>>=','@',sep='
\backslash
n')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Different documents can read the same R script, so the R code can be reusable
 across different input documents.
\end_layout

\begin_layout Subsection
Evaluation of Chunk Options
\begin_inset CommandInset label
LatexCommand label
name "sub:conditional"

\end_inset


\end_layout

\begin_layout Standard
By default 
\series bold
knitr
\series default
 uses a new syntax to parse chunk options: it treats them as function arguments
 instead of a text string to be split to obtain option values.
 This gives the user much more power than the old syntax; we can pass arbitrary
 R objects to chunk options besides simple ones like 
\family typewriter
TRUE
\family default
/
\family typewriter
FALSE
\family default
, numbers and character strings.
 The page 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/sweave/
\end_layout

\end_inset

 has given two examples to show the advantages of the new syntax.
 Here we show yet another useful application.
\end_layout

\begin_layout Standard
Before 
\series bold
knitr
\series default
 0.3, there was a feature named 
\begin_inset Quotes eld
\end_inset

conditional evaluation
\begin_inset Quotes erd
\end_inset


\begin_inset Foot
status open

\begin_layout Plain Layout
request from 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://plus.google.com/u/0/116405544829727492615/posts/43WrRUffjzK
\end_layout

\end_inset


\end_layout

\end_inset

.
 The idea is, instead of setting chunk options 
\family typewriter
eval
\family default
 and 
\family typewriter
echo
\family default
 to be 
\family typewriter
TRUE
\family default
 or 
\family typewriter
FALSE
\family default
 (constants), their values can be controlled by global variables in the
 current R session.
 This enables 
\series bold
knitr
\series default
 to conditionally evaluate code chunks according to variables.
 For example, here we assign 
\family typewriter
TRUE
\family default
 to a variable 
\family typewriter
dothis
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cond-variable>>=
\end_layout

\begin_layout Plain Layout

dothis=TRUE
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In the next chunk, we set chunk options 
\family typewriter
eval=dothis
\family default
 and 
\family typewriter
echo=!dothis
\family default
, both are valid R expressions since the variable 
\family typewriter
dothis
\family default
 exists.
 As we can see, the source code is hidden, but it was indeed evaluated:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cond-out1, eval=dothis, echo=!dothis>>=
\end_layout

\begin_layout Plain Layout

print('you cannot see my source because !dothis is FALSE')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Then we set 
\family typewriter
eval=dothis
\family default
 and 
\family typewriter
echo=dothis
\family default
 for another chunk:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cond-out2,eval=dothis,echo=dothis>>=
\end_layout

\begin_layout Plain Layout

dothis
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
If we change the value of 
\family typewriter
dothis
\family default
 to 
\family typewriter
FALSE
\family default
, neither of the above chunks will be evaluated any more.
 Therefore we can control many chunks with a single variable, and present
 results selectively.
\end_layout

\begin_layout Standard
This old feature requires 
\series bold
knitr
\series default
 to treat 
\family typewriter
eval
\family default
 and 
\family typewriter
echo
\family default
 specially, and we can easily see that it is no longer necessary with the
 new syntax: 
\family typewriter
eval=dothis
\family default
 will tell R to find the variable 
\family typewriter
dothis
\family default
 automatically just like we call a function 
\family typewriter
foobar(eval = dothis)
\family default
.
 What is more, all options will be evaluated as R expressions unless they
 are already constants which do not need to be evaluated, so this old feature
 has been generalized to all other options naturally.
\end_layout

\begin_layout Subsection
Customization
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package is ready for customization.
 Both the patterns and hooks can be customized; see the package website
 for details.
 Here I show an example on how to save 
\series bold
rgl
\series default
 plots 
\begin_inset CommandInset citation
LatexCommand citep
key "R-rgl"

\end_inset

 using a customized hook function.
 First we define a hook named 
\family typewriter
rgl
\family default
 using the function 
\emph on
hook_rgl()
\emph default
 in 
\series bold
rgl
\series default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<rgl-demo>>=
\end_layout

\begin_layout Plain Layout

library(rgl)
\end_layout

\begin_layout Plain Layout

knit_hooks$set(rgl = hook_rgl)
\end_layout

\begin_layout Plain Layout

head(hook_rgl)  # the hook function is defined as this
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Then we only have to set the chunk option 
\family typewriter
rgl=TRUE
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fancy-rgl, rgl=TRUE, dev='png', fig.width=5, fig.height=5, out.width='2in',
 message=FALSE, warning=FALSE, cache=TRUE>>=
\end_layout

\begin_layout Plain Layout

library(rgl)
\end_layout

\begin_layout Plain Layout

demo('bivar', package='rgl', echo=FALSE) 
\end_layout

\begin_layout Plain Layout

par3d(zoom=.7)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Due to the flexibility of output hooks, 
\series bold
knitr
\series default
 supports several different output formats.
 The implementation is fairly easy, e.g., for LaTeX we put R output in 
\family typewriter
verbatim
\family default
 environments, and in HTML, it is only a matter of putting output in 
\family typewriter
div
\family default
 layers.
 These are simply character string operations.
 Many demos in 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/
\end_layout

\end_inset

 show this idea clearly.
 This manual did not cover all the features of 
\series bold
knitr
\series default
, and users are encouraged to thumb through the website to know more possible
 features.
\end_layout

\begin_layout Section
Editors
\end_layout

\begin_layout Standard
You can use any text editors to write the source documents, but some have
 built-in support for 
\series bold
knitr
\series default
.
 Both RStudio (
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.rstudio.org
\end_layout

\end_inset

) and LyX (
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.lyx.org
\end_layout

\end_inset

) have full support for 
\series bold
knitr
\series default
, and you can compile the document to PDF with just one click.
 See 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/rstudio/
\end_layout

\end_inset

 and 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/lyx/
\end_layout

\end_inset

 respectively.
 It is also possible to support other editors like 
\begin_inset CommandInset href
LatexCommand href
name "Eclipse"
target "https://yihui.org/knitr/demo/eclipse/"

\end_inset

, 
\begin_inset CommandInset href
LatexCommand href
name "Texmaker and WinEdt"
target "https://yihui.org/knitr/demo/editors/"

\end_inset

; see the demo list in the website for configuration instructions.
\end_layout

\begin_layout Section*
About This Document
\end_layout

\begin_layout Standard
This manual was written in LyX and compiled with 
\series bold
knitr
\series default
 (version 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
Sexpr{packageVersion('knitr')}
\end_layout

\end_inset

).
 The LyX source and the Rnw document exported from LyX can be found under
 these directories:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<source-location, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

system.file('examples', 'knitr-manual.lyx', package='knitr')  # lyx source
\end_layout

\begin_layout Plain Layout

system.file('examples', 'knitr-manual.Rnw', package='knitr')  # Rnw source
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
You can use the function 
\emph on
knit()
\emph default
 to knit the Rnw document (remember to put the two 
\family sans
.bib
\family default
 files under the same directory), and you need to make sure all the R packages
 used in this document are installed:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<required-packages, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

install.packages(c('animation', 'rgl', 'tikzDevice', 'ggplot2'))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Feedback and comments on this manual and the package are always welcome.
 Bug reports and feature requests can be sent to 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/issues
\end_layout

\end_inset

, and questions can be delivered to the 
\begin_inset CommandInset href
LatexCommand href
name "mailing list"
target "knitr@googlegroups.com"
type "mailto:"

\end_inset

 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://groups.google.com/group/knitr
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% when knitr is updated, this chunk will be updated; why?
\end_layout

\begin_layout Plain Layout

<<auto-bib, version=packageVersion('knitr'), echo=FALSE, cache=TRUE, message=FAL
SE, warning=FALSE>>=
\end_layout

\begin_layout Plain Layout

# write all packages in the current session to a bib file
\end_layout

\begin_layout Plain Layout

write_bib(c(.packages(), 'evaluate', 'formatR', 'highr'), file = 'knitr-packages.b
ib')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
bibfiles "knitr-manual,knitr-packages"
options "jss"

\end_inset


\end_layout

\end_body
\end_document

This is a special R script which can be used to generate a report. You can
write normal text in roxygen comments.

First we set up some options (you do not have to do this):

```{r setup, include=FALSE}
library(knitr)
opts_chunk$set(fig.path = 'figure/silk-')
```

The report begins here.

```{r test-a, cache=FALSE}
# boring examples as usual
set.seed(123)
x = rnorm(5)
mean(x)
```

You can use the special syntax {{code}} to embed inline expressions, e.g.
`r mean(x) + 2`
is the mean of x plus 2.
The code itself may contain braces, but these are not checked.  Thus,
perfectly valid (though very strange) R code such as `{{2 + 3}} - {{4 - 5}}`
can lead to errors because `2 + 3}} - {{4 - 5` will be treated as inline code.

Now we continue writing the report. We can draw plots as well.

```{r test-b, fig.width=5, fig.height=5}
par(mar = c(4, 4, .1, .1)); plot(x)
```

Actually you do not have to write chunk options, in which case knitr will use
default options. For example, the code below has no options attached:

```{r }
var(x)
quantile(x)
```

And you can also write two chunks successively like this:

```{r test-chisq5}
sum(x^2) # chi-square distribution with df 5
```
```{r test-chisq4}
sum((x - mean(x))^2) # df is 4 now
```

Done. Call spin('knitr-spin.R') to make silk from sow's ear now and knit a
lovely purse.

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc true
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview={XYZ null null 1}"
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 2.5cm
\topmargin 2.5cm
\rightmargin 2.5cm
\bottommargin 2.5cm
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\listings_params "language=R"
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path = 'figure/listings-')
\end_layout

\begin_layout Plain Layout

options(formatR.arrow = TRUE)
\end_layout

\begin_layout Plain Layout

render_listings()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
Using listings with knitr
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
To use the 
\series bold
listings
\series default
 package with 
\series bold
knitr
\series default
, all you need to do is to call a function in your first setup chunk (that
 chunk should be hidden from the output with 
\family typewriter
include=FALSE
\family default
 and should not be cached):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% I just want to echo the 2nd line
\end_layout

\begin_layout Plain Layout

<<setup, echo=2>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This function modifies the output hooks and header information so that the
 output is written in 
\series bold
listings
\series default
 environments, which are kindly provided by Frank Harrell and can be found
 in 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/misc/Sweavel.sty
\end_layout

\end_inset

.
 Of course you have to install the 
\series bold
listings
\series default
 package if your LaTeX toolset does not include this package.
\end_layout

\begin_layout Standard
Now we can see how the results look like with our new settings:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-random>>=
\end_layout

\begin_layout Plain Layout

set.seed(1121)  # for reproducibility
\end_layout

\begin_layout Plain Layout

options(width=85)
\end_layout

\begin_layout Plain Layout

x=rnorm(20)
\end_layout

\begin_layout Plain Layout

x
\end_layout

\begin_layout Plain Layout

mean(x)
\end_layout

\begin_layout Plain Layout

sqrt(-1)  # this will give you a warning message
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Another chunk:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<rgl-hook-demo,eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

hook_rgl = function(before, options, envir) {
\end_layout

\begin_layout Plain Layout

    ## after a chunk has been evaluated
\end_layout

\begin_layout Plain Layout

    if (before || !require('rgl') || rgl.cur() == 0) return()  # no active
 device
\end_layout

\begin_layout Plain Layout

    name = paste(valid_prefix(options$fig.path), options$label, sep = '')
\end_layout

\begin_layout Plain Layout

    par3d(windowRect = 100 + options$dpi * c(0, 0, options$width, options$height
))
\end_layout

\begin_layout Plain Layout

    Sys.sleep(.05) # need time to respond to window size change
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

    fmt = opts_knit$get('out.format')
\end_layout

\begin_layout Plain Layout

    if (fmt %in% c('html', 'markdown', 'gfm', 'jekyll')) options$dev = 'png'
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

    ## support 3 formats: eps, pdf and png (default)
\end_layout

\begin_layout Plain Layout

    switch(options$dev,
\end_layout

\begin_layout Plain Layout

           postscript = rgl.postscript(paste(name, '.eps', sep = ''), fmt
 = 'eps'),
\end_layout

\begin_layout Plain Layout

           pdf = rgl.postscript(paste(name, '.pdf', sep = ''), fmt = 'pdf'),
\end_layout

\begin_layout Plain Layout

           rgl.snapshot(paste(name, '.png', sep = ''), fmt = 'png'))
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

    if (fmt == 'html') return(.plot.hook.html(c(name, 'png'), options))
\end_layout

\begin_layout Plain Layout

    if (fmt %in% c('markdown', 'gfm', 'jekyll'))
\end_layout

\begin_layout Plain Layout

        return(.plot.hook.markdown(c(name, 'png'), options))
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

    paste(ifelse(options$align == 'center', '
\backslash

\backslash
centering{}', ''), '
\backslash

\backslash
includegraphics[',
\end_layout

\begin_layout Plain Layout

          sprintf('width=%s', options$out.width), ']{', name, '}
\backslash
n', sep = '')
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Well, we can do crazier things with 
\series bold
knitr
\series default
.
 Here we use the 
\family typewriter
fig.cap
\family default
 option to write plots into the 
\family typewriter
figure
\family default
 environment automatically, and the caption is generated from data dynamically
 (see Figure 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
ref{fig:boxplot-ex}
\end_layout

\end_inset

):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% set an option first
\end_layout

\begin_layout Plain Layout

<<eval-opts, echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

opts_knit$set(eval.after = 'fig.cap') # evaluate fig.cap after the chunk
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

<<boxplot-ex, fig.height=2, out.width='.8
\backslash

\backslash
linewidth', fig.cap=sprintf('This is a boxplot; the median is %.2f.', median(x))>>=
\end_layout

\begin_layout Plain Layout

par(mar=c(4,4,.1,.1))
\end_layout

\begin_layout Plain Layout

boxplot(x, horizontal=TRUE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
As we know, 
\family typewriter
figure
\family default
 is a float environment, so it has floated away from the R code to the top
 of this page.
 This should not be surprising to LaTeX users.
\end_layout

\begin_layout Standard
You should be able to compile the TeX document and get a PDF file like this
 one: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/releases/download/doc/knitr-listings.pdf
\end_layout

\end_inset

.
 For more information about out hooks, see 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/hooks/
\end_layout

\end_inset

.
\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc true
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview={XYZ null null 1}"
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 2.5cm
\topmargin 2.5cm
\rightmargin 2.5cm
\bottommargin 2.5cm
\secnumdepth 2
\tocdepth 2
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

# set global chunk options
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/minimal-', fig.align='center', fig.show='hold')
\end_layout

\begin_layout Plain Layout

options(formatR.arrow=TRUE,width=90)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
A Minimal Demo of knitr
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
You can test if 
\series bold
knitr
\series default
 works with this minimal demo.
 OK, let's get started with some boring random numbers:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-random>>=
\end_layout

\begin_layout Plain Layout

set.seed(1121)
\end_layout

\begin_layout Plain Layout

(x=rnorm(20))
\end_layout

\begin_layout Plain Layout

mean(x);var(x)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The first element of 
\family typewriter
x
\family default
 is 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
Sexpr{x[1]}
\end_layout

\end_inset

.
 Boring boxplots and histograms recorded by the PDF device:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-plots, fig.width=4, fig.height=4, out.width='.4
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

## two plots side by side (option fig.show='hold')
\end_layout

\begin_layout Plain Layout

par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3,las=1)
\end_layout

\begin_layout Plain Layout

boxplot(x)
\end_layout

\begin_layout Plain Layout

hist(x,main='')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Do the above chunks work? You should be able to compile the TeX document
 and get a PDF file like this one: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/releases/download/doc/knitr-minimal.pdf
\end_layout

\end_inset

.
 The Rnw source of this document is at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/examples/knitr-minimal.Rnw
\end_layout

\end_inset

.
\end_layout

\end_body
\end_document

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage{mathpazo}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\usepackage{url}
\usepackage[authoryear]{natbib}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
% \VignetteIndexEntry{An Introduction to knitr}
% \VignetteEngine{knitr}

\makeatother

\begin{document}


\title{An Introduction to knitr}


\author{Yihui Xie}

\maketitle
The \textbf{knitr} package \citep{R-knitr} is an alternative tool
to Sweave based on a different design with more features. This document
is not the real vignette, but only serves as a placeholder to guide
you to the real manuals of this package. You can find them in the
package website: \url{http://yihui.name/knitr}%
\footnote{e.g. the main manual: \url{https://bitbucket.org/stat/knitr/downloads/knitr-manual.pdf}
and the graphics manual: \url{https://bitbucket.org/stat/knitr/downloads/knitr-graphics.pdf}%
}, and remember to read the help pages of functions in this package.

Anyway, here is a code chunk that shows you can compile vignettes
with \textbf{knitr} as well by using a proper \textsf{Makefile} (see
\url{http://yihui.name/knitr/demo/vignette/}):

<<show-off>>=
rnorm(5)
df=data.frame(y=rnorm(100), x=1:100)
summary(lm(y~x, data=df))
@

\bibliographystyle{jss}
\bibliography{../examples/knitr-packages}


\end{document}

#LyX 2.1 created this file. For more info see http://www.lyx.org/
\lyxformat 474
\begin_document
\begin_header
\textclass tufte-handout
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[buttonsize=1em]{animate}
\end_preamble
\options justified
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_math auto
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks true
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview=FitH"
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 1
\use_package mathtools 1
\use_package mhchem 1
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine basic
\cite_engine_type default
\biblio_style plain
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\secnumdepth 2
\tocdepth 2
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

options(formatR.arrow=TRUE,width=50)
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/graphics-', cache.path='cache/graphics-',
 fig.align='center', dev='tikz', fig.width=5, fig.height=5, fig.show='hold',
 cache=TRUE, par=TRUE)
\end_layout

\begin_layout Plain Layout

knit_hooks$set(par=function(before, options, envir){
\end_layout

\begin_layout Plain Layout

if (before && options$fig.show!='none') par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mg
p=c(2,.7,0),tcl=-.3)
\end_layout

\begin_layout Plain Layout

}, crop=hook_pdfcrop)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
knitr Graphics Manual
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Abstract
This manual shows features of graphics in the 
\series bold
knitr
\series default
 package (version 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
Sexpr{packageVersion('knitr')}
\end_layout

\end_inset

) in detail, including the graphical devices, plot recording, plot rearrangement
, control of plot sizes, the tikz device, figure captions, animations and
 other types of plots such as 
\series bold
rgl
\series default
 or GGobi plots.
\end_layout

\begin_layout Standard
Before reading this specific manual
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://bit.ly/knitr-graphics-src
\end_layout

\end_inset

 (Rnw source)
\end_layout

\end_inset

, you must have finished the main manual
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://bit.ly/knitr-main
\end_layout

\end_inset


\end_layout

\end_inset

.
\end_layout

\begin_layout Section
Graphical Devices
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package comes with more than 20 built-in graphical devices, and you can
 specify them through the 
\family typewriter
dev
\family default
 option.
 This document uses the global option 
\family typewriter
dev='tikz'
\family default
, i.e., the plots are recorded by the tikz device by default, but we can change
 the device locally.
 Since tikz will be used extensively throughout this manual and you will
 see plenty of tikz graphics later, now we first show a few other devices.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<test-plot, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

with(trees, symbols(Height, Volume, circles = Girth/16, inches = FALSE,
 bg = 'deeppink', fg = 'gray30'))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float marginfigure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<pdf-dev, ref.label='test-plot', dev='pdf', out.width='
\backslash

\backslash
linewidth', echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The default PDF device.
\begin_inset CommandInset label
LatexCommand label
name "mar:pdf-dev"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float marginfigure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<png-dev, ref.label='test-plot', dev='png', out.width='
\backslash

\backslash
linewidth', echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The PNG device.
\begin_inset CommandInset label
LatexCommand label
name "mar:png-dev"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "mar:pdf-dev"

\end_inset

 and 
\begin_inset CommandInset ref
LatexCommand ref
reference "mar:png-dev"

\end_inset

 show two standard devices in the 
\series bold
grDevices
\series default
 package.
 We can also use devices in the 
\series bold
Cairo
\series default
 or 
\series bold
cairoDevice
\series default
 package, e.g., the chunk below uses the 
\emph on
CairoPNG()
\emph default
 device in the 
\series bold
Cairo
\series default
 package.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cairo-png-dev, ref.label='test-plot', dev='CairoPNG', out.width='.5
\backslash

\backslash
linewidth', echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Plot Recording
\end_layout

\begin_layout Standard
As mentioned in the main manual, 
\series bold
knitr
\series default
 uses the 
\series bold
evaluate
\series default
 package to record plots.
 There are two sources of plots: first, whenever 
\emph on
plot.new()
\emph default
 or 
\emph on
grid.newpage()
\emph default
 is called, 
\series bold
evaluate
\series default
 will try to save a snapshot of the current plot
\begin_inset Foot
status open

\begin_layout Plain Layout
For technical details, see 
\family typewriter
?setHook
\family default
 and 
\family typewriter
?recordPlot
\end_layout

\end_inset

; second, after each complete expression is evaluated, a snapshot is also
 saved.
 To speed up recording, the null graphical device 
\family typewriter
pdf(file = NULL)
\family default
 is used.
 Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:two-high"

\end_inset

 shows two expressions producing two high-level plots.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<two-high, fig.width=3, fig.height=2.5, out.width='.49
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

plot(cars)
\end_layout

\begin_layout Plain Layout

boxplot(cars$dist,xlab='dist')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Two high-level plots are captured.
 The key to arrange two plots side by side is to specify the 
\family typewriter
out.width
\family default
 option so that each plot takes less than half of the line width.
 We do not have to use the 
\family typewriter
par(mfrow)
\family default
 trick, and it may not work in some cases (e.g.
 to put base graphics and 
\series bold
ggplot2
\series default
 side by side; recall Figure 1 in the main manual).
\begin_inset CommandInset label
LatexCommand label
name "fig:two-high"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:low-plot-loop"

\end_inset

 shows another example of two R expressions, but the second expression only
 involves with low-level plotting changes.
 By default, low-level plot changes are discarded, but you can retain them
 with the option 
\family typewriter
fig.keep='all'
\family default
.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<low-plot-loop, fig.width=3, fig.height=2.5, out.width='.49
\backslash

\backslash
linewidth', fig.keep='all'>>=
\end_layout

\begin_layout Plain Layout

plot(0,0,type='n',ann=FALSE)
\end_layout

\begin_layout Plain Layout

for(i in seq(0, 2*pi,length=20)) {points(cos(i),sin(i))}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Two complete R expressions will produce at most two plots, as long as there
 are not multiple high-level plotting calls in each expression; option 
\family typewriter
fig.keep='all'
\family default
 here.
\begin_inset CommandInset label
LatexCommand label
name "fig:low-plot-loop"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Together with 
\family typewriter
fig.show='asis'
\family default
, we can show the process of plotting step by step like Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:high-with-low"

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<high-with-low, fig.width=3, fig.height=2.5, out.width='.5
\backslash

\backslash
linewidth', fig.keep='all', fig.show='asis'>>=
\end_layout

\begin_layout Plain Layout

plot(cars)
\end_layout

\begin_layout Plain Layout

abline(lm(dist~speed, data=cars))  # a regression line
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Low-level plot changes in base graphics can be recorded separately, and
 plots can be put in the places where they were produced.
\begin_inset CommandInset label
LatexCommand label
name "fig:high-with-low"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
A further note on plot recording: 
\series bold
knitr
\series default
 examines all recorded plots (as R objects) and compares them sequentially;
 if the previous plot is a 
\begin_inset Quotes eld
\end_inset

subset
\begin_inset Quotes erd
\end_inset

 of the next plot (= previous plot + low-level changes), the previous plot
 will be removed when 
\family typewriter
fig.keep='high'
\family default
.
 If two successive plots are identical, the second one will be removed by
 default, so it might be a little bit surprising that the following chunk
 will only produce one plot by default
\begin_inset Foot
status open

\begin_layout Plain Layout
adapted from 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/issues/41
\end_layout

\end_inset


\end_layout

\end_inset

:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<identical-plots,eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

m = matrix(1:100, ncol = 10)
\end_layout

\begin_layout Plain Layout

image(m)
\end_layout

\begin_layout Plain Layout

image(m*2)  # exactly the same as previous plot
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Plot Rearrangement
\end_layout

\begin_layout Standard
We can rearrange the plots in chunks in several ways.
 They can be inserted right after the line(s) of R code which produced them,
 or accumulated till the end of the chunk.
 There is an example in the main manual demonstrating 
\family typewriter
fig.show='asis'
\family default
 for two high-level plots, and Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:high-with-low"

\end_inset

 in this manual also demonstrates this option for a high-level plot followed
 by a low-level change.
\end_layout

\begin_layout Standard
Here is an example demonstrating the option 
\family typewriter
fig.keep='last'
\family default
 (only the last plot is kept):
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fig-last, fig.width=5.2, fig.height=3, out.width='.7
\backslash

\backslash
linewidth', fig.keep='last', message=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(ggplot2)
\end_layout

\begin_layout Plain Layout

pie <- ggplot(diamonds, aes(x = factor(1), fill = cut)) + xlab('cut') +
 geom_bar(width = 1) 
\end_layout

\begin_layout Plain Layout

pie + coord_polar(theta = "y") # a pie chart
\end_layout

\begin_layout Plain Layout

pie + coord_polar() # the bullseye chart 
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Two plots were produced in this chunk, but only the last one is kept.
 This can be useful when we experiment with many plots, but only want the
 last result.
 (Adapted from the 
\series bold
ggplot2
\series default
 website)
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
When multiple plots are produced by a code chunk, we may want to show them
 as an animation with the option 
\family typewriter
fig.show='animate'
\family default
.
 Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:clock-animation"

\end_inset

 shows a simple clock animation; you may compare the code to Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:high-with-low"

\end_inset

 to understand that high-level plots are always recorded, regardless of
 where they appeared.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<clock-animation, fig.width=3, fig.height=3, out.width='.4
\backslash

\backslash
linewidth', fig.show='animate', tidy=FALSE, crop=TRUE>>=
\end_layout

\begin_layout Plain Layout

par(mar = rep(3, 4))
\end_layout

\begin_layout Plain Layout

for (i in seq(pi/2, -4/3 * pi, length = 12)) {
\end_layout

\begin_layout Plain Layout

    plot(0, 0, pch = 20, ann = FALSE, axes = FALSE)
\end_layout

\begin_layout Plain Layout

    arrows(0, 0, cos(i), sin(i))
\end_layout

\begin_layout Plain Layout

    axis(1, 0, "VI"); axis(2, 0, "IX")
\end_layout

\begin_layout Plain Layout

    axis(3, 0, "XII"); axis(4, 0, "III"); box()
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
A clock animation.
 You have to view it in Adobe Reader: click to play/pause; there are also
 buttons to speed up or slow down the animation.
\begin_inset CommandInset label
LatexCommand label
name "fig:clock-animation"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
We can also set the alignment of plots easily with the 
\family typewriter
fig.align
\family default
 option; this document uses 
\family typewriter
fig.align='center'
\family default
 as a global option, and we can also set plots to be left/right-aligned.
 Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:maruko-plot"

\end_inset

 is an example of a left-aligned plot.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<maruko-plot, fig.width=7, fig.height=4, out.width='.45
\backslash

\backslash
linewidth', fig.align='left', crop=TRUE>>=
\end_layout

\begin_layout Plain Layout

stars(cbind(1:16,10*(16:1)),draw.segments=TRUE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
A left-aligned plot adapted from 
\family typewriter
?stars
\family default
 (I call this the 
\begin_inset Quotes eld
\end_inset

Maruko
\begin_inset Quotes erd
\end_inset

 plot, and it is one of my favorites; see 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://en.wikipedia.org/wiki/Chibi_Maruko-chan
\end_layout

\end_inset

).
\begin_inset CommandInset label
LatexCommand label
name "fig:maruko-plot"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Section
Plot Size
\end_layout

\begin_layout Standard
We have seen several examples in which two or more plots can be put side
 by side, and this is because the plots were resized in the output document;
 with the chunk option 
\family typewriter
out.width
\family default
 less than half of the line width, LaTeX will arrange two plots in one line;
 if it is less than 
\begin_inset Formula $1/3$
\end_inset

 of the line width, three plots can be put in one line.
 Of course we can also set it to be an absolute width like 
\family typewriter
3in
\family default
 (3 inches).
 This option is used extensively in this document to control the size of
 plots in the output document.
\end_layout

\begin_layout Section
The tikz Device
\end_layout

\begin_layout Standard
The main advantage of using tikz graphics is the consistency of styles between
 texts in plots and those in the main document.
 Since we can use native LaTeX commands in plots, the styles of texts in
 plots can be very sophisticated (see Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:math-formula-tt"

\end_inset

 for an example).
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<math-formula-tt, fig.width=6, fig.height=2, out.width='
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

plot(0:1,0:1,type='n', ylab='origin of statistics', xlab='statistical presentati
on rocks with 
\backslash

\backslash
LaTeX{}')
\end_layout

\begin_layout Plain Layout

text(.5,c(.8,.5,.2), c('
\backslash

\backslash
texttt{lm(y 
\backslash

\backslash
textasciitilde{} x)}', '$
\backslash

\backslash
hat{
\backslash

\backslash
beta}=(X^{
\backslash

\backslash
prime}X)^{-1}X^{
\backslash

\backslash
prime}y$',  '$(
\backslash

\backslash
Omega,
\backslash

\backslash
mathcal{F},
\backslash

\backslash
mu)$'))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
A plot created by 
\series bold
tikzDevice
\series default
 with math expressions and typewriter fonts.
 Note the font style consistency -- we write the same expressions in LaTeX
 here: 
\begin_inset Formula $\hat{\beta}=(X^{\prime}X)^{-1}X^{\prime}y$
\end_inset

 and 
\begin_inset Formula $(\Omega,\mathcal{F},\mu)$
\end_inset

; also 
\family typewriter
lm(y ~ x)
\family default
.
\begin_inset CommandInset label
LatexCommand label
name "fig:math-formula-tt"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
When using XeLaTeX instead of PDFLaTeX to compile the document, we need
 to tell the 
\series bold
tikzDevice
\series default
 package by setting the 
\family typewriter
tikzDefaultEngine
\family default
 option before all plot chunks (preferably in the first chunk):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<xetex-tikz, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

options(tikzDefaultEngine='xetex')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This is useful and often necessary to compile tikz plots which contain (UTF8)
 multi-byte characters.
\end_layout

\begin_layout Section
Figure Caption
\end_layout

\begin_layout Standard
If the chunk option 
\family typewriter
fig.cap
\family default
 is not 
\family typewriter
NULL
\family default
 or 
\family typewriter
NA
\family default
, the plots will be put in a 
\family typewriter
figure
\family default
 environment when the output format is LaTeX, and this option is used to
 write a caption in this environment using 
\family typewriter

\backslash
caption{}
\family default
.
 The other two related options are 
\family typewriter
fig.scap
\family default
 and 
\family typewriter
fig.lp
\family default
 which set the short caption and a prefix string for the figure label.
 The default short caption is extracted from the caption by truncating it
 at the first period or colon or semi-colon.
 The label is a combination of 
\family typewriter
fig.lp
\family default
 and the chunk label.
 Because 
\family typewriter
figure
\family default
 is a float environment, it can float away from the chunk output to other
 places such as the top or bottom of a page when the TeX document is compiled.
\end_layout

\begin_layout Section
Other Features
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package can be extended with hook functions, and here I give a few examples
 illustrating the flexibility.
\end_layout

\begin_layout Subsection
Cropping PDF Graphics
\end_layout

\begin_layout Standard
Some R users may have been suffering from the extra margins in R plots,
 especially in base graphics (
\series bold
ggplot2
\series default
 is much better in this aspect).
 The default graphical option 
\family typewriter
mar
\family default
 is about 
\family typewriter
c(5, 4, 4, 2)
\family default
 (see 
\family typewriter
?par
\family default
), which is often too big.
 Instead of endlessly setting 
\family typewriter
par(mar)
\family default
, you may consider the program 
\family typewriter
pdfcrop
\family default
, which can crop the white margin automatically
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.ctan.org/pkg/pdfcrop
\end_layout

\end_inset


\end_layout

\end_inset

.
 In 
\series bold
knitr
\series default
, we can set up the hook 
\emph on
hook_pdfcrop()
\emph default
 to work with a chunk option, say, 
\family typewriter
crop
\family default
.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<crop-hook, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

knit_hooks$set(crop=hook_pdfcrop)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Now we compare two plots below.
 The first one is not cropped (Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pdf-nocrop"

\end_inset

); then the same plot is produced but with a chunk option 
\family typewriter
crop=TRUE
\family default
 which will call the cropping hook (Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pdf-crop"

\end_inset

).
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{kframe}
\end_layout

\begin_layout Plain Layout

<<pdf-nocrop, fig.width=3.2, fig.height=3.2, out.width='.5
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

par(mar=c(5,4,4,2),bg='white')  # large margin
\end_layout

\begin_layout Plain Layout

plot(lat~long,data=quakes,pch=20,col=rgb(0,0,0,.2))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout


\backslash
end{kframe}
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The original plot produced in R, with a large margin.
\begin_inset CommandInset label
LatexCommand label
name "fig:pdf-nocrop"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{kframe}
\end_layout

\begin_layout Plain Layout

<<pdf-crop, ref.label='pdf-nocrop', echo=FALSE, fig.width=3.2, fig.height=3.2,
 out.width='.5
\backslash

\backslash
linewidth', crop=TRUE>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout


\backslash
end{kframe}
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The cropped plot; it fits better in the document.
\begin_inset CommandInset label
LatexCommand label
name "fig:pdf-crop"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
As we can see, the white margins are gone.
 If we use 
\emph on
par()
\emph default
, it might be hard and tedious to figure out a reasonable amount of margin
 in order that neither is any label cropped nor do we get a too large margin.
 My experience is that 
\family typewriter
pdfcrop
\family default
 works well with base graphics, but barely works with 
\series bold
grid
\series default
 graphics (therefore 
\series bold
lattice
\series default
 and 
\series bold
ggplot2
\series default
 are ruled out).
\end_layout

\begin_layout Subsection
Manually Saved Plots
\end_layout

\begin_layout Standard
We have explained how R plots are recorded before.
 In some cases, it is not possible to capture plots by 
\emph on
recordPlot()
\emph default
 (such as 
\series bold
rgl
\series default
 plots), but we can save them using other functions.
 To insert these plots into the output, we need to set up a hook first like
 this (see 
\family typewriter
?hook_plot_custom
\family default
 for details):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<custom-plot-hook, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

knit_hooks$set(custom.plot = hook_plot_custom)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Then we set the chunk option 
\family typewriter
custom.plot=TRUE
\family default
, and manually write plot files in the chunk.
 Here we show an example of capturing GGobi plots using the function 
\emph on
ggobi_display_save_picture()
\emph default
 in the 
\series bold
rggobi
\series default
 package:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ggobi-plot, custom.plot=TRUE, fig.ext='png', out.width='2.5in', results='hide',
 warning=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(rggobi)
\end_layout

\begin_layout Plain Layout

ggobi(ggobi_find_file('data', 'flea.csv'))
\end_layout

\begin_layout Plain Layout

Sys.sleep(1)  # wait for snapshot
\end_layout

\begin_layout Plain Layout

ggobi_display_save_picture(path=fig_path('.png'))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
One thing to note here is we have to make sure the plot filename is from
 
\emph on
fig_path()
\emph default
, which is a convenience function to return the figure path for the current
 chunk.
\end_layout

\begin_layout Standard
We can do whatever normal R plots can do with this hook, and we give another
 example below to show how to work with animations.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<rgl-animation, custom.plot=TRUE, fig.ext='png', out.width='2.5in', fig.show='animat
e', fig.num=20, interval=.4,  aniopts='controls,autoplay', results='hide',
 warning=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(animation) # adapted from demo('rgl_animation')
\end_layout

\begin_layout Plain Layout

data(pollen)
\end_layout

\begin_layout Plain Layout

uM = matrix(c(-0.37, -0.51, -0.77, 0, -0.73, 0.67, -0.1, 0, 0.57, 0.53, -0.63, 0,
 0, 0, 0, 1), 4, 4)
\end_layout

\begin_layout Plain Layout

library(rgl)
\end_layout

\begin_layout Plain Layout

open3d(userMatrix = uM, windowRect = c(0, 0, 400, 400))
\end_layout

\begin_layout Plain Layout

plot3d(pollen[, 1:3])
\end_layout

\begin_layout Plain Layout

zm = seq(1, 0.05, length = 20)
\end_layout

\begin_layout Plain Layout

par3d(zoom = 1)  # change the zoom factor gradually later
\end_layout

\begin_layout Plain Layout

for (i in 1:length(zm)) {
\end_layout

\begin_layout Plain Layout

    par3d(zoom = zm[i]); Sys.sleep(.05)
\end_layout

\begin_layout Plain Layout

    rgl.snapshot(fig_path('png', number = i))
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
rgl Plots
\end_layout

\begin_layout Standard
With the hook 
\emph on
hook_rgl()
\emph default
, we can easily save snapshots from the 
\series bold
rgl
\series default
 package.
 We have shown an example in the main manual, and here we add some details.
 The rgl hook is a good example of taking care of details by carefully using
 the 
\family typewriter
options
\family default
 argument in the hook; for example, we cannot directly set the width and
 height of rgl plots in 
\emph on
rgl.snapshot()
\emph default
 or 
\emph on
rgl.postscript()
\emph default
, so we make use of the options 
\family typewriter
fig.width
\family default
, 
\family typewriter
fig.height
\family default
 and 
\family typewriter
dpi
\family default
 to calculate the expected size of the window, then resize the current window
 by 
\emph on
par3d()
\emph default
, and finally save the plot.
\end_layout

\begin_layout Standard
This hook is actually built upon 
\emph on
hook_plot_custom()
\emph default
 -- first it saves the 
\series bold
rgl
\series default
 snapshot, then it calls 
\emph on
hook_plot_custom()
\emph default
 to write the output code.
\end_layout

\begin_layout Section
\start_of_appendix
How to Compile This Manual
\end_layout

\begin_layout Standard
This manual has a long chain of dependencies, so it may not be easy to compile.
 These packages are required (all of them are free software):
\end_layout

\begin_layout Description
R Cairo, ggplot2, tikzDevice, rgl, rggobi, animation (all available on CRAN
 except tikzDevice which is on R-Forge for the time being)
\end_layout

\begin_layout Description
LaTeX animate, hyperref and the tufte-handout class
\end_layout

\begin_layout Description
Other GGobi, pdfcrop
\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\begin_preamble
\usepackage{multicol, upquote}
% \VignetteIndexEntry{knitr Reference Card}
% \VignetteEngine{knitr}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered false
\pdf_bookmarksopen false
\pdf_bookmarksopenlevel 1
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation landscape
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1cm
\topmargin 0cm
\rightmargin 1cm
\bottommargin 0cm
\secnumdepth 3
\tocdepth 3
\paragraph_separation skip
\defskip 0in
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Title
knitr Reference Card
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
begin{multicols}{2}
\end_layout

\end_inset


\end_layout

\begin_layout Section
Syntax
\end_layout

\begin_layout Standard
\noindent
\align center
\begin_inset Tabular
<lyxtabular version="3" rows="7" columns="5">
<features tabularvalignment="middle">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0">
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
format
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
start
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
end
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
inline
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
output
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rnw
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<<*>>=|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
@
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|
\backslash
|
\backslash
verb|Sexpr{x}|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
TeX
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rmd
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|```{r *}|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|```|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|`r x`|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
MD
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rhtml
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<!--begin.rcode *|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|end.rcode-->|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<!--rinline x-->|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
HTML
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rrst
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
..
\begin_inset space ~
\end_inset

{r *}
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
..
\begin_inset space ~
\end_inset

..
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|:r:`x`|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
reST
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rtex
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
% begin.rcode *
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
% end.rcode
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|
\backslash
rinline{x}|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
TeX
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
brew
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<% x %>|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
text
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\begin_layout Standard

\family typewriter
*
\family default
 denotes local chunk options, e.g.
 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<<label, eval=FALSE>>=|
\end_layout

\end_inset

; 
\family typewriter
x
\family default
 denotes inline R code, e.g.
 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|`r 1+2`|
\end_layout

\end_inset

 (MD stands for 
\begin_inset CommandInset href
LatexCommand href
name "Markdown"
target "http://en.wikipedia.org/wiki/Markdown"

\end_inset

)
\end_layout

\begin_layout Section
Minimal Examples
\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
begin{multicols}{2}
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Sweave (*.Rnw)
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

x=paste('
\backslash

\backslash
Sexpr', '{', 'z}', sep = '')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout


\backslash
begin{verbatim}
\end_layout

\begin_layout Plain Layout


\backslash
document
\backslash
Sexpr{'c'}lass{article}
\end_layout

\begin_layout Plain Layout


\backslash
be
\backslash
Sexpr{'g'}in{document}
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

Below is a code chunk.
\end_layout

\begin_layout Plain Layout


\backslash
Sexpr{''}<<foo, echo=TRUE>>=
\end_layout

\begin_layout Plain Layout

z = 1+1
\end_layout

\begin_layout Plain Layout

plot(cars)
\end_layout

\begin_layout Plain Layout


\backslash
Sexpr{''}@
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

The value of z is 
\backslash
Sexpr{x}.
\end_layout

\begin_layout Plain Layout


\backslash
end{document}
\end_layout

\begin_layout Plain Layout


\backslash
end{verbatim}
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
R Markdown (*.Rmd)
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{verbatim}
\end_layout

\begin_layout Plain Layout

Hi _markdown_!
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

```{r foo, echo=TRUE}
\end_layout

\begin_layout Plain Layout

z = 1+1
\end_layout

\begin_layout Plain Layout

plot(cars)
\end_layout

\begin_layout Plain Layout

```
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

The value of z is `r z`.
\end_layout

\begin_layout Plain Layout


\backslash
end{verbatim}
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Brew (*.brew)
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{verbatim}
\end_layout

\begin_layout Plain Layout

The value of pi is <% pi %>.
\end_layout

\begin_layout Plain Layout


\backslash
end{verbatim}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
end{multicols}
\end_layout

\end_inset


\end_layout

\begin_layout Section
Chunk Options
\end_layout

\begin_layout Standard

\family typewriter
opts_chunk
\family default
 controls global chunk options, e.g.
 
\family typewriter
opts_chunk$set(tidy = FALSE)
\family default
, which can be overridden by local chunk options.
 See all options at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://yihui.name/knitr/options
\end_layout

\end_inset

; some frequently used options:
\end_layout

\begin_layout Description
eval whether to evaluate the chunk
\end_layout

\begin_layout Description
echo whether to echo source code
\end_layout

\begin_layout Description
results 
\family typewriter
'markup'
\family default
, 
\family typewriter
'asis'
\family default
, 
\family typewriter
'hide'
\end_layout

\begin_layout Description
tidy whether to reformat R code
\end_layout

\begin_layout Description
cache whether to cache results
\end_layout

\begin_layout Description
fig.width,
\begin_inset space \space{}
\end_inset

fig.height,
\begin_inset space \space{}
\end_inset

out.width,
\begin_inset space \space{}
\end_inset

out.height device and output size of figures
\end_layout

\begin_layout Description
include whether to include the chunk results in output
\end_layout

\begin_layout Description
child filenames of child documents
\end_layout

\begin_layout Description
engine language name (R, python, ...)
\end_layout

\begin_layout Section
Functions
\end_layout

\begin_layout Description
knit() the main function in this package; knit input document and write
 output
\end_layout

\begin_layout Description
purl() extract R code from an input document
\end_layout

\begin_layout Description
spin() spin goat's hair (an R script with roxygen comments) into wool (a
 literate programming document to be passed to 
\family typewriter
knit()
\family default
)
\end_layout

\begin_layout Description
stitch() insert an R script into a template and compile the document
\end_layout

\begin_layout Description
knit_hooks$set() set or reset chunk and output 
\begin_inset CommandInset href
LatexCommand href
name "hooks"
target "http://yihui.name/knitr/hooks"

\end_inset


\end_layout

\begin_layout Section
Resources
\end_layout

\begin_layout Itemize
homepage: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://yihui.name/knitr
\end_layout

\end_inset

 
\end_layout

\begin_layout Itemize
development repository: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr
\end_layout

\end_inset

 (
\begin_inset CommandInset href
LatexCommand href
name "CRAN"
target "http://cran.r-project.org/package=knitr"

\end_inset

, 
\begin_inset CommandInset href
LatexCommand href
name "Rforge"
target "http://rforge.net/knitr"

\end_inset

)
\end_layout

\begin_layout Itemize
examples: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr-examples
\end_layout

\end_inset


\end_layout

\begin_layout Itemize
stackoverflow: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://stackoverflow.com/questions/tagged/knitr
\end_layout

\end_inset


\end_layout

\begin_layout Itemize
mailing list: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://groups.google.com/group/knitr
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
end{multicols}
\end_layout

\end_inset


\end_layout

\end_body
\end_document

#LyX 2.1 created this file. For more info see http://www.lyx.org/
\lyxformat 474
\begin_document
\begin_header
\textclass tufte-handout
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[breaklinks=true,pdfstartview=FitH]{hyperref}
\end_preamble
\options justified
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_math auto
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 88
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref false
\pdf_author "Ramnath Vaidyanathan"
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 1
\pdf_breaklinks false
\pdf_pdfborder true
\pdf_colorlinks true
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "citecolor = blue, linkcolor = magenta, urlcolor = green"
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 1
\use_package mathtools 1
\use_package mhchem 1
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine basic
\cite_engine_type default
\biblio_style plain
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 0
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1in
\topmargin 1in
\rightmargin 1in
\bottommargin 1in
\secnumdepth 2
\tocdepth 2
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/theme-', cache.path='cache/theme-', cache=TRUE)
\end_layout

\begin_layout Plain Layout

options(formatR.arrow=TRUE,width=78)
\end_layout

\begin_layout Plain Layout

knit_hooks$set(par=function(before, options, envir){if (before) par(mar=c(4,4,.1,.
1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3)})
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout

<<solarized-theme, cache=FALSE, echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

thm = knit_theme$get("solarized-dark")
\end_layout

\begin_layout Plain Layout

knit_theme$set(thm)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
Customizing Syntax Highlighting Themes
\end_layout

\begin_layout Author
Yihui Xie & Ramnath Vaidyanathan
\end_layout

\begin_layout Abstract
This manual
\begin_inset Foot
status open

\begin_layout Plain Layout
source code at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/examples/knitr-themes.Rnw
\end_layout

\end_inset


\end_layout

\end_inset

 shows how to customize syntax highlighting of source code using themes.
 It walks the user through the basics of syntax highlighting in 
\series bold
knitr
\series default
, and the use of built-in themes.
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package uses the 
\series bold
highr
\series default
 package to highlight source code in a document.
 In short, the 
\series bold
highr
\series default
 package parses the source code, tokenizes it into grammar symbols, and
 formats their appearance using CSS or LaTeX commands.
 
\end_layout

\begin_layout Section
Usage
\end_layout

\begin_layout Standard
We can use the object 
\family typewriter
knit_theme
\family default
 to set / get a theme.
 See 
\family typewriter
?knit_theme
\family default
 for the usage.
 For example, we set the theme of this document to be 
\family typewriter
solarized-dark
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<solarized-theme>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Built-in Themes
\end_layout

\begin_layout Standard
\begin_inset Float margintable
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eclipse-css, comment=NA, echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

cat(c(readLines('../themes/edit-eclipse.css', n=30),'...'),sep='
\backslash
n')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The listing on the right shows the CSS file for one of the themes, 
\family typewriter
edit-eclipse
\family default
, which was adapted from Andre Simon's excellent highlighter
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.andre-simon.de/
\end_layout

\end_inset


\end_layout

\end_inset

.
 The 
\series bold
knitr
\series default
 package comes pre-loaded with a number of themes based on this highlighter.
 Here is list of all available code themes
\begin_inset Foot
status open

\begin_layout Plain Layout
For a preview of all themes, see 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://gist.github.com/yihui/3422133
\end_layout

\end_inset


\end_layout

\end_inset

:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<all-themes>>=
\end_layout

\begin_layout Plain Layout

knit_theme$get()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Shown below is how the 
\family typewriter
solarized-dark
\family default
 theme looks like when applied to R code:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<demo-code, eval = FALSE>>=
\end_layout

\begin_layout Plain Layout

library(XML)
\end_layout

\begin_layout Plain Layout

library(plyr)
\end_layout

\begin_layout Plain Layout

library(reshape)
\end_layout

\begin_layout Plain Layout

# SCRAPE THE DATA FROM WEB 
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

base_url = "http://www.mlsoccer.com/stats/%s/reg"
\end_layout

\begin_layout Plain Layout

years    = 1996:2010
\end_layout

\begin_layout Plain Layout

options(width = 40)
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

#' Function to save data for each year
\end_layout

\begin_layout Plain Layout

save_data = function(y){
\end_layout

\begin_layout Plain Layout

  url  = sprintf(base_url, y)
\end_layout

\begin_layout Plain Layout

  tab  = readHTMLTable(url, header = FALSE, stringsAsFactors = FALSE);
\end_layout

\begin_layout Plain Layout

  pos  = max(grep("United", tab));
\end_layout

\begin_layout Plain Layout

  tab  = tab[[pos]];
\end_layout

\begin_layout Plain Layout

  tab$year   = y;
\end_layout

\begin_layout Plain Layout

  return(tab)            
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

	
\end_layout

\begin_layout Plain Layout

team.list  = llply(years, save_data);
\end_layout

\begin_layout Plain Layout

mls        = merge_recurse(team.list);
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Misc
\end_layout

\begin_layout Standard
One thing to consider is the foreground color of plots when we use dark
 themes; we need to make it lighter, otherwise the graphical elements will
 be difficult to be read.
 We can access the foreground color of the theme in the list returned by
 
\family typewriter
knit_theme$get(theme)
\family default
, e.g., for this document:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fg-color>>=
\end_layout

\begin_layout Plain Layout

## the object thm is from the first chunk
\end_layout

\begin_layout Plain Layout

thm$foreground  # foreground color
\end_layout

\begin_layout Plain Layout

thm$background  # background color
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
When we make plots, we may use these colors, e.g.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<plot-color, fig.width=6, fig.height=4, out.width='.8
\backslash

\backslash
linewidth', par=TRUE>>=
\end_layout

\begin_layout Plain Layout

## can design a chunk hook to set foreground color automatically
\end_layout

\begin_layout Plain Layout

fgcolor=thm$foreground
\end_layout

\begin_layout Plain Layout

par(fg=fgcolor, col.axis=fgcolor, col.lab=fgcolor)
\end_layout

\begin_layout Plain Layout

plot(rnorm(100),pch=19)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Of course, we do not need to worry about these colors when we use a white
 background for the plots.
\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc true
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview={XYZ null null 1}"
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 3cm
\topmargin 3cm
\rightmargin 3cm
\bottommargin 3cm
\secnumdepth 2
\tocdepth 2
\paragraph_separation skip
\defskip bigskip
\quotes_language english
\papercolumns 2
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/twocolumn-', fig.align='center', fig.show='hold')
\end_layout

\begin_layout Plain Layout

render_listings()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
Using knitr in A Two-column Document
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
We can use the 
\family typewriter
figure*
\family default
 environment in LaTeX so that figures can span over two columns.
 Because the output hooks use the 
\series bold
framed
\series default
 package which does not work well with the 
\family typewriter
figure*
\family default
 environment, we can either modify the output hooks or use other packages
 like 
\series bold
listings
\series default
.
 For example,
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<chunk-hook, echo=2, tidy=TRUE, tidy.opts=list(width.cutoff=30)>>=
\end_layout

\begin_layout Plain Layout

<<setup>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float figure
wide true
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-plots, fig.width=5, fig.height=5, out.width='.45
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

x=rnorm(100)
\end_layout

\begin_layout Plain Layout

par(mar=c(4,4,.1,.1))
\end_layout

\begin_layout Plain Layout

boxplot(x)
\end_layout

\begin_layout Plain Layout

hist(x,main='')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
Two plots spanning over two columns.
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The output PDF file will be like: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/releases/download/doc/knitr-twocolumn.pdf
\end_layout

\end_inset

.
 The Rnw source of this document is at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/examples/knitr-twocolumn.Rnw
\end_layout

\end_inset

.
\end_layout

\end_body
\end_document

You can also use the `child` option to include child documents in markdown.

```{r test-main, child='knitr-child.Rmd'}
```

You can continue your main document below, of course.

```{r test-another}
pmax(1:10, 5)
```

If you refer to a child document in a chunk, only the child document will be evaluated. 
Other code in the chunk will not be evaluated when you `render` the document.

```{r test-warning-option, include=FALSE}
options(knitr.child.warning = FALSE)
```

```{r test-warning, child='knitr-child2.Rmd'}
x <- "this code is not evaluated"
x
```

## Frequently Asked Questions

This FAQ is compiled from the [issues](https://github.com/yihui/knitr/issues) and messages I received from blog posts and emails, etc.

1. **knitr** does not work...
  - Please first update all your R packages (use `update.packages()`) and probably R itself ([what is the current R version?](http://cran.r-project.org/)), then see if it works; if not, please file an [issue](https://github.com/yihui/knitr/issues) to me.
1. What is the best place to ask questions when the [package website](http://yihui.name/knitr) is not helpful?
  - Depending what you want to ask, you may use these tools (I keep track of the first three more frequently):
  - (Recommended) [StackOverflow](http://stackoverflow.com/questions/tagged/knitr): general questions (more experts and quicker answers there).
  - [Github issues](https://github.com/yihui/knitr/issues): bug reports and feature requests.
  - [knitr mailing list](https://groups.google.com/group/knitr) or [R-help](http://www.r-project.org/mail.html) list: general questions and feedback via email in public.
  - Private email is the least recommended way unless there are really private issues.
  - Twitter ([`@xieyihui`](http://twitter.com/xieyihui)) or Google+ ([`+Yihui Xie`](https://plus.google.com/u/0/109653178371807724268/posts)).
1. How should I get started?
  - Watch the [5-min video](http://www.screenr.com/qcv8) on the [homepage](http://yihui.name/knitr).
1. What is the best editor for writing **knitr** source documents?
  - For beginners, perhaps [RStudio](http://www.rstudio.com/ide/); **knitr** is also supported in [LyX](http://yihui.name/knitr/demo/lyx/), [Emacs/ESS](http://ess.r-project.org/), WinEdt, Tinn-R and a variety of [editors](http://yihui.name/knitr/demo/editors/).
1. Where are those prompt characters `>` and `+`? I feel uncomfortable reading R output without them.
  - They are removed by default, because [I believe they make no sense](http://yihui.name/en/2013/01/code-pollution-with-command-prompts/). This is the reason why I dislike books on R which used `>` and `+`; they twist my mind and make my eyes bleed when I read the R code in the books. For those who really want to read R code like `> 1+1` instead of `1 + 1`, you have the [chunk option](http://yihui.name/knitr/options) `prompt`.
1. What is the working directory? Can I change my working directory in my code chunks?
  - You'd better not do this. Your working directory is always `getwd()` (all output files will be written here), but the code chunks are evaluated under the directory where your input document comes from. Changing working directories while running R code is a bad practice in general. See [#38](https://github.com/yihui/knitr/issues/38) for a discussion. You should also try to avoid absolute directories whenever possible (use relative directories instead), because it makes things less reproducible.
1. The gray (shading) box is too narrow for my output.
  - No, it is not because the box is too narrow (the box uses the current line width); it is because your output is too wide. Use a smaller `width` option to avoid text output exceeding the page margin, e.g. `options(width = 60)` (see [#44](https://github.com/yihui/knitr/issues/44)).
1. How to comment out inline R code like in `\Sexpr{code}`?
  - see issue [#110](https://github.com/yihui/knitr/issues/110): you can destroy `\Sexpr` by `%\%Sexpr{code}` or comment out R code like `\Sexpr{#code}`, or just comment out the whole paragraph with `%` in the beginning of lines.
1. How can I write a literal code chunk? i.e. write a code chunk which is not parsed and useful for tutorials?
  - You need to destroy the chunk header, e.g. add an empty string before the chunk header: `\Sexpr{''}<<label, what=ever>>=`, or ```` `r ''` ```{r label, what=ever} ```` ([#443](https://github.com/yihui/knitr/issues/443)); see [example 062](https://github.com/yihui/knitr-examples)
1. I have done something cool with **knitr**; could you add a link in your website?
  - Sure! I'd love to; just let me know.
1. What can I do for you?
  - Many things, e.g. [donate zillions to me](https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=ZRJDEBSEJEUJY&lc=US&item_name=Donate%20to%20Yihui%20Xie&currency_code=USD&bn=PP%2dDonationsBF%3abtn_donateCC_LG%2egif%3aNonHosted) (well, I'm kidding), buy me a book from my [Amazon wishlist](http://amzn.com/w/2S7M0GLEC32SB), [tweet](https://twitter.com/xieyihui) my [links](http://yihui.name/knitr), mention **knitr** on [Google+](https://plus.google.com/u/0/109653178371807724268/posts) or Facebook, or fork this repository and contribute code, or just say hello to me somewhere.
  - Cite the package and the [knitr book](http://www.crcpress.com/product/isbn/9781482203530); see `citation('knitr')` in R.

---
output:
  pdf_document:
    highlight: null
    number_sections: yes
    template: assets/template-refcard.tex
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{knitr Reference Card}
-->

# Chunk Options

`opts_chunk` controls global chunk options, e.g.
`opts_chunk$set(tidy = FALSE)`, which can be overridden by local chunk
options. See all options at <https://yihui.org/knitr/options/>; some
frequently used options:

eval
:   whether to evaluate the chunk

echo
:   whether to echo source code

results
:   `'markup'`, `'asis'`, `'hold'`, `'hide'`

tidy
:   whether to reformat R code

cache
:   whether to cache results

fig.width, fig.height, out.width, out.height
:   device and output size of figures

include
:   whether to include the chunk results in output

child
:   filenames of child documents

engine
:   language name (R, python, ...)


# Functions

`knit()`
:   the main function in this package; knit input document and write
    output

`purl()`
:   extract R code from an input document

`spin()`
:   spin goat's hair (an R script with roxygen comments) into wool (a
    literate programming document to be passed to `knit()`)

`stitch()`
:   insert an R script into a template and compile the document

`knit_hooks$set()`
:   set or reset chunk and output [hooks](https://yihui.org/knitr/hooks/)

# Resources

-   homepage: <https://yihui.org/knitr/>
-   development repository: <https://github.com/yihui/knitr>
    ([CRAN](http://cran.rstudio.com/package=knitr),
-   examples: <https://github.com/yihui/knitr-examples>
-   stackoverflow: <http://stackoverflow.com/tags/knitr/>
-   mailing list: <https://groups.google.com/group/knitr>

---
title: "Not An Introduction to knitr"
author: "Yihui Xie"
date: "`r Sys.Date()`"
bibliography:
  - ../inst/examples/knitr-packages.bib
  - ../inst/examples/knitr-manual.bib
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Not an Introduction to knitr}
output: knitr:::html_vignette
---

The **knitr** package [@R-knitr] is an alternative tool to Sweave based on a
different design with more features. This document is not an introduction,
but only serves as a placeholder to guide you to the real manuals, which are
available on the package website <https://yihui.org/knitr/>^[e.g. the [main
manual](https://yihui.org/knitr/demo/manual/) and the [graphics
manual](https://yihui.org/knitr/demo/graphics/)], and remember to read the help pages of
functions in this package. There is a book [@xie2013] for this package, but
it may not be useful to those who prefer digging out information on the web.

Anyway, here is a code chunk that shows you can compile vignettes with
**knitr** as well using R 3.0.x, which supports non-Sweave vignettes:

```{r show-off, tidy=TRUE}
options(digits=4)
rnorm(20)
fit=lm(dist~speed, data=cars)
b=coef(fit)
```{r results='asis', echo=FALSE}
knitr::kable(summary(fit)$coefficients, caption='Regression coefficients.')
```

The fitted regression equation is $Y=`r b[1]`+`r b[2]`x$.

```{r graphics, fig.cap='A scatterplot with a regression line.'}
par(mar=c(4, 4, 1, .1))
plot(cars, pch = 20)
abline(fit, col = 'red')
```

## References

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc true
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview={XYZ null null 1}"
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 3cm
\topmargin 3cm
\rightmargin 3cm
\bottommargin 3cm
\secnumdepth 2
\tocdepth 2
\paragraph_separation skip
\defskip bigskip
\quotes_language english
\papercolumns 2
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/twocolumn-', fig.align='center', fig.show='hold')
\end_layout

\begin_layout Plain Layout

render_listings()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
Using knitr in A Two-column Document
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
We can use the 
\family typewriter
figure*
\family default
 environment in LaTeX so that figures can span over two columns.
 Because the output hooks use the 
\series bold
framed
\series default
 package which does not work well with the 
\family typewriter
figure*
\family default
 environment, we can either modify the output hooks or use other packages
 like 
\series bold
listings
\series default
.
 For example,
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<chunk-hook, echo=2>>=
\end_layout

\begin_layout Plain Layout

<<setup>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float figure
wide true
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-plots, fig.width=5, fig.height=5, out.width='.45
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

x=rnorm(100)
\end_layout

\begin_layout Plain Layout

par(mar=c(4,4,.1,.1))
\end_layout

\begin_layout Plain Layout

boxplot(x)
\end_layout

\begin_layout Plain Layout

hist(x,main='')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
Two plots spanning over two columns.
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The output PDF file will be like: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://bitbucket.org/stat/knitr/downloads/knitr-twocolumn.pdf
\end_layout

\end_inset

.
 The Rnw source of this document is at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/examples/knitr-twocolumn.Rnw
\end_layout

\end_inset

.
\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass tufte-handout
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[breaklinks=true,pdfstartview=FitH]{hyperref}
\end_preamble
\options justified
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 88

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref false
\pdf_author "Ramnath Vaidyanathan"
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 1
\pdf_breaklinks false
\pdf_pdfborder true
\pdf_colorlinks true
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "citecolor = blue, linkcolor = magenta, urlcolor = green"
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 0
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1in
\topmargin 1in
\rightmargin 1in
\bottommargin 1in
\secnumdepth 2
\tocdepth 2
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/theme-', cache.path='cache/theme-', cache=TRUE)
\end_layout

\begin_layout Plain Layout

options(replace.assign=TRUE,width=78)
\end_layout

\begin_layout Plain Layout

knit_hooks$set(par=function(before, options, envir){if (before) par(mar=c(4,4,.1,.
1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3)})
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout

<<solarized-theme, cache=FALSE, echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

thm = knit_theme$get("solarized-dark")
\end_layout

\begin_layout Plain Layout

knit_theme$set(thm)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
Customizing Syntax Highlighting Themes
\end_layout

\begin_layout Author
Yihui Xie & Ramnath Vaidyanathan
\end_layout

\begin_layout Abstract
This manual
\begin_inset Foot
status open

\begin_layout Plain Layout
source code at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/examples/knitr-themes.Rnw
\end_layout

\end_inset


\end_layout

\end_inset

 shows how to customize syntax highlighting of source code using themes.
 It walks the user through the basics of syntax highlighting in 
\series bold
knitr
\series default
, the use of built-in themes as well as those from 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.eclipsecolorthemes.org/
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package uses the 
\series bold
highlight
\series default
 package by Romain Francois to highlight source code in a document.
 In short, the 
\series bold
highlight
\series default
 package parses the source code using the 
\noun on
bison
\noun default
 parser, tokenizes it into grammar symbols, and formats their appearance
 based on a CSS style file.
 
\end_layout

\begin_layout Section
Usage
\end_layout

\begin_layout Standard
We can use the object 
\family typewriter
knit_theme
\family default
 to set / get a theme.
 See 
\family typewriter
?knit_theme
\family default
 for the usage.
 For example, we set the theme of this document to be 
\family typewriter
solarized-dark
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<solarized-theme>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Built-in Themes
\end_layout

\begin_layout Standard
\begin_inset Float margintable
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eclipse-css, comment=NA, echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

cat(c(readLines('../themes/edit-eclipse.css', n=30),'...'),sep='
\backslash
n')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The listing on the right shows the CSS file for one of the themes, 
\family typewriter
edit-eclipse
\family default
, which was adapted from Andre Simon's excellent highlighter
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.andre-simon.de/
\end_layout

\end_inset


\end_layout

\end_inset

.
 The 
\series bold
knitr
\series default
 package comes pre-loaded with a number of themes based on this highlighter.
 Here is list of all available code themes:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<all-themes>>=
\end_layout

\begin_layout Plain Layout

knit_theme$get()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Shown below is how the 
\family typewriter
solarized-dark
\family default
 theme looks like when applied to R code:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<demo-code, eval = FALSE>>=
\end_layout

\begin_layout Plain Layout

# SCRAPE THE DATA FROM WEB 
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

base_url = "http://www.mlsoccer.com/stats/%s/reg"
\end_layout

\begin_layout Plain Layout

years    = 1996:2010
\end_layout

\begin_layout Plain Layout

options(width = 40)
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

#' Function to save data for each year
\end_layout

\begin_layout Plain Layout

save_data = function(y){
\end_layout

\begin_layout Plain Layout

  url  = sprintf(base_url, y)
\end_layout

\begin_layout Plain Layout

  tab  = XML::readHTMLTable(url, header = FALSE, stringsAsFactors = FALSE);
\end_layout

\begin_layout Plain Layout

  pos  = max(grep("United", tab));
\end_layout

\begin_layout Plain Layout

  tab  = tab[[pos]];
\end_layout

\begin_layout Plain Layout

  tab$year   = y;
\end_layout

\begin_layout Plain Layout

  return(tab)            
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

	
\end_layout

\begin_layout Plain Layout

team.list  = plyr::llply(years, save_data);
\end_layout

\begin_layout Plain Layout

mls        = reshape::merge_recurse(team.list);
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Eclipse Color Themes
\end_layout

\begin_layout Standard
A rich repository of syntax highlighting themes for use with Eclipse IDE
 is available at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.eclipsecolorthemes.org/
\end_layout

\end_inset

.
 In order to use one of these themes with 
\series bold
knitr
\series default
, it needs to be converted into a CSS file that can be parsed by 
\series bold
highlight
\series default
.
 This is achieved by mapping language tokens across the two formats (Table
 
\begin_inset CommandInset ref
LatexCommand ref
reference "mar:Token-Mapping"

\end_inset

) to create a CSS file.
\end_layout

\begin_layout Standard
\begin_inset Float margintable
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
renewcommand
\backslash
arraystretch{1.3}
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
Token mapping
\begin_inset CommandInset label
LatexCommand label
name "mar:Token-Mapping"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\noindent
\begin_inset Tabular
<lyxtabular version="3" rows="16" columns="2">
<features booktabs="true" tabularvalignment="middle">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0pt">
<row>
<cell alignment="left" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\series bold
Highlight (CSS)
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\series bold
Eclipse (XML)
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
background
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
background
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
number
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
number
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
functioncall
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
method
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
string
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
string
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
keyword
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
keyword
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
argument
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
localVariable
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
comment
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
singleLineComment
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
roxygencomment
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
javadoc
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
formalargs
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
localVariable
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
eqformalargs
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
foreground
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
assignement
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
foreground
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
package
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
class
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
slot
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
foreground
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
symbol
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
localVariableDeclaration
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
prompt
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family roman
\series medium
\shape up
\size normal
\emph off
\bar no
\strikeout off
\uuline off
\uwave off
\noun off
\color none
foreground
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
If you want to use an eclipse theme that is not shipped with 
\series bold
knitr
\series default
, you can use the utility function 
\emph on
eclipse_theme()
\emph default
 to automatically download, map and save the theme as a CSS file.
 For example, if you want to use the theme Oughsumm
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.eclipsecolorthemes.org/?view=theme&id=1885
\end_layout

\end_inset

 (theme id is 1885)
\end_layout

\end_inset

, you can use the code below in your first chunk:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eclipse-theme, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

css = eclipse_theme(1885)
\end_layout

\begin_layout Plain Layout

thm = knit_theme$get(css)
\end_layout

\begin_layout Plain Layout

knit_theme$set(thm)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Note this requires Internet connection, and we can put the code to download
 the theme in a cached chunk, so it will be downloaded only once, then we
 set the theme in an uncached chunk using the parsed theme object.
 For example, download the theme:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

%% please use cache=TRUE and echo=FALSE for this chunk
\end_layout

\begin_layout Plain Layout

<<download-theme, eval=FALSE, message=FALSE>>=
\end_layout

\begin_layout Plain Layout

(css = eclipse_theme(1885))
\end_layout

\begin_layout Plain Layout

thm = knit_theme$get(css)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Then apply the theme:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

%% cache=FALSE for this chunk
\end_layout

\begin_layout Plain Layout

<<apply-theme, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

knit_theme$set(thm)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Even we do not have Internet connection the next time, we can still use
 the theme.
\end_layout

\begin_layout Section
Misc
\end_layout

\begin_layout Standard
One thing to consider is the foreground color of plots when we use dark
 themes; we need to make it lighter, otherwise the graphical elements will
 be difficult to be read.
 We can access the foreground color of the theme in the list returned by
 
\family typewriter
knit_theme$get(theme)
\family default
, e.g., for this document:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fg-color>>=
\end_layout

\begin_layout Plain Layout

## the object thm is from the first chunk
\end_layout

\begin_layout Plain Layout

thm$foreground  # foreground color
\end_layout

\begin_layout Plain Layout

thm$background  # background color
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
When we make plots, we may use these colors, e.g.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<plot-color, fig.width=6, fig.height=4, out.width='.8
\backslash

\backslash
linewidth', par=TRUE>>=
\end_layout

\begin_layout Plain Layout

## can design a chunk hook to set foreground color automatically
\end_layout

\begin_layout Plain Layout

fgcolor=thm$foreground
\end_layout

\begin_layout Plain Layout

par(fg=fgcolor, col.axis=fgcolor, col.lab=fgcolor)
\end_layout

\begin_layout Plain Layout

plot(rnorm(100),pch=19)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Of course, we do not need to worry about these colors when we use a white
 background for the plots.
\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass beamer
\begin_preamble
\usetheme{PaloAlto}
\end_preamble
\use_default_options true
\begin_modules
knitr
beamer-fragile
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize 10
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered false
\pdf_bookmarksopen false
\pdf_bookmarksopenlevel 1
\pdf_breaklinks false
\pdf_pdfborder true
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE>>=
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/beamer-',fig.align='center',fig.show='hold',size='f
ootnotesize')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
\begin_inset Argument
status collapsed

\begin_layout Plain Layout
knitr, Beamer, and FragileFrame
\end_layout

\end_inset

A Minimal Demo of knitr with Beamer and Fragile Frames
\end_layout

\begin_layout Author
Yihui Xie
\begin_inset Foot
status open

\begin_layout Plain Layout
I thank Richard E.
 Goldberg for providing this demo.
\end_layout

\end_inset


\end_layout

\begin_layout EndFrame

\end_layout

\begin_layout BeginFrame
Background
\end_layout

\begin_layout Itemize
The 
\series bold
knitr
\emph on
 
\series default
\emph default
package allows you to embed R code and figures in LaTeX documents
\end_layout

\begin_deeper
\begin_layout Itemize
It has functionality similar to Sweave but looks nicer and gives you more
 control
\end_layout

\end_deeper
\begin_layout Itemize
If you already have Sweave working in LyX, getting 
\series bold
knitr
\series default
 to work is trivial
\end_layout

\begin_deeper
\begin_layout Enumerate
Install the 
\series bold
knitr
\series default
 package in 
\emph on
R
\end_layout

\begin_layout Enumerate
Read 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://yihui.name/knitr/demo/lyx/
\end_layout

\end_inset


\end_layout

\end_deeper
\begin_layout Itemize
If you use Sweave or 
\series bold
knitr
\series default
 with Beamer in LyX, you probably use the 
\emph on
Beamer Fragile
\emph default
 module
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.lyx.org/trac/ticket/7273
\end_layout

\end_inset


\end_layout

\end_inset

 too.
 Let's see if 
\series bold
knitr
\series default
 works with Beamer in this small demo.
 
\end_layout

\begin_layout Section
First Test
\end_layout

\begin_layout Standard
\begin_inset Flex FragileFrame
status open

\begin_layout FrameTitle
First Test
\end_layout

\begin_layout Plain Layout
OK, let's get started with just some text:
\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=FALSE,results='hide'>>=
\end_layout

\begin_layout Plain Layout

# some setup
\end_layout

\begin_layout Plain Layout

options(width=60)  # make the printing fit on the page
\end_layout

\begin_layout Plain Layout

set.seed(1121)   # make the results repeatable
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

<<>>= 
\end_layout

\begin_layout Plain Layout

# create some random numbers
\end_layout

\begin_layout Plain Layout

(x=rnorm(20))  
\end_layout

\begin_layout Plain Layout

mean(x);var(x)  
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
BTW, the first element of 
\family typewriter
x
\family default
 is 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
Sexpr{x[1]}
\end_layout

\end_inset

.
 (Did you notice the use of
\family typewriter
 
\backslash
Sexpr{}
\family default
.)
\end_layout

\end_inset


\end_layout

\begin_layout Section
Second Test
\end_layout

\begin_layout Standard
\begin_inset Flex FragileFrame
status open

\begin_layout FrameTitle
Second Test
\end_layout

\begin_layout Plain Layout
Text is nice but let's see what happens if we make a couple of plots in
 our chunk:
\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-plots,fig.width=4,fig.height=4,out.width='.45
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

par(las=1,mar=c(4,4,.1,.1))  # tick labels direction
\end_layout

\begin_layout Plain Layout

boxplot(x) 
\end_layout

\begin_layout Plain Layout

hist(x,main='',col="blue",probability=TRUE) 
\end_layout

\begin_layout Plain Layout

lines(density(x),col="red")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Section
The Big Question
\end_layout

\begin_layout BeginFrame
The Big Question
\end_layout

\begin_layout Standard
Do the above chunks work? You should be able to compile the LyX document
 and get a nice-looking PDF slide presentation.
 If not, time to double-check everything...
\end_layout

\begin_layout EndFrame

\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package default
\inputencoding auto
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref false
\papersize default
\use_geometry false
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE>>=
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/subfloats-')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
Using subfloats in LyX with knitr
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
If you insert a float figure inside a float figure in LyX, you will get
 subfloats.
 To make sure the two subfloats can stand side by side, you have to:
\end_layout

\begin_layout Enumerate
remove new lines between the two subfloat environments in LyX (if you are
 not sure if there are blank lines between them, check the LaTeX source);
\end_layout

\begin_layout Enumerate
use the chunk option 
\family typewriter
echo=FALSE
\family default
 to hide the R source code in the output;
\end_layout

\begin_layout Enumerate
set 
\family typewriter
out.width
\family default
 to be less than half of the linewidth if you do not want the sub figures
 to exceed the page margin;
\end_layout

\begin_layout Enumerate
most importantly, leave a blank line in the beginning of the subfloat environmen
t, so that 
\family typewriter
<<>>=
\family default
 starts in a new line and 
\series bold
knitr
\series default
 can recognize it;
\end_layout

\begin_layout Standard
Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:knitr-subfloat"

\end_inset

 is an example.
\end_layout

\begin_layout Standard
This document was compiled with 
\series bold
knitr
\series default
 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
Sexpr{packageVersion('knitr')}
\end_layout

\end_inset

.
 Rnw source at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/examples/knitr-subfloats.Rnw
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

<<test-subfloat, echo=FALSE, out.width='.45
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

plot(1)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
this is a subfloat
\end_layout

\end_inset


\end_layout

\end_inset


\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

<<test-another, echo=FALSE, out.width='.45
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

plot(rnorm(100))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
another float
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
one figure with 2 sub figures
\begin_inset CommandInset label
LatexCommand label
name "fig:knitr-subfloat"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\end_body
\end_document

This is a special R script which can be used to generate a report. You can
write normal text in roxygen comments.

First we set up some options (you do not have to do this):

```{r setup, include=FALSE}
library(knitr)
opts_chunk$set(fig.path='figure/silk-')
```

The report begins here.

```{r test-a, cache=FALSE}
# boring examples as usual
set.seed(123)
x = rnorm(5)
mean(x)
```

Now we continue writing the report. We can draw plots as well.

```{r test-b, fig.width=5, fig.height=5}
par(mar = c(4, 4, .1, .1)); plot(x)
```

Actually you do not have to write chunk options, in which case knitr will use
default options. For example, the code below has no options attached:

```{r }
var(x)
quantile(x)
```

And you can also write two chunks successively like this:

```{r test-chisq5}
sum(x^2) # chi-square distribution with df 5
```{r test-chisq4}
sum((x - mean(x))^2) # df is 4 now
```

Done. Call spin('knitr-spin.R') to make silk from sow's ear now and knit a
lovely purse.

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\begin_preamble
\usepackage{multicol}
\IfFileExists{upquote.sty}{\usepackage{upquote}}{}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered false
\pdf_bookmarksopen false
\pdf_bookmarksopenlevel 1
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation landscape
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1cm
\topmargin 0cm
\rightmargin 1cm
\bottommargin 0cm
\secnumdepth 3
\tocdepth 3
\paragraph_separation skip
\defskip 0in
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Title
knitr Reference Card
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
begin{multicols}{2}
\end_layout

\end_inset


\end_layout

\begin_layout Section
Syntax
\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
begin{small}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
\align center
\begin_inset Tabular
<lyxtabular version="3" rows="9" columns="5">
<features tabularvalignment="middle">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0">
<column alignment="left" valignment="top" width="0">
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
format
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
start
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
end
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
inline
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
output
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rnw
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<<*>>=|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
@
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|
\backslash
|
\backslash
verb|Sexpr{x}|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
TeX
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rmd
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|```{r *}|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|```|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|`r x`|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
MD
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rhtml
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<!--begin.rcode *|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|end.rcode-->|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<!--rinline x-->|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
HTML
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rrst
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
..
\begin_inset space ~
\end_inset

{r *}
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
..
\begin_inset space ~
\end_inset

..
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|:r:`x`|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
reST
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="left" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rtex
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
% begin.rcode *
\end_layout

\end_inset
</cell>
<cell alignment="left" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
% end.rcode
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|
\backslash
rinline{x}|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
TeX
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rasciidoc
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
// begin.rcode *
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
// end.rcode
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
+r x+
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
AsciiDoc
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Rtextile
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
### begin.rcode *
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
### end.rcode
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\family typewriter
@r x@
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Textile
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
brew
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout

\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<% x %>|
\end_layout

\end_inset


\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
text
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
end{small}
\end_layout

\end_inset


\end_layout

\begin_layout Standard

\family typewriter
*
\family default
 denotes local chunk options, e.g.
 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|<<label, eval=FALSE>>=|
\end_layout

\end_inset

; 
\family typewriter
x
\family default
 denotes inline R code, e.g.
 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
verb|`r 1+2`|
\end_layout

\end_inset

 (MD stands for 
\begin_inset CommandInset href
LatexCommand href
name "Markdown"
target "http://en.wikipedia.org/wiki/Markdown"

\end_inset

)
\end_layout

\begin_layout Section
Minimal Examples
\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
begin{multicols}{2}
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Sweave (*.Rnw)
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{verbatim}
\end_layout

\begin_layout Plain Layout


\backslash
documentclass{article}
\end_layout

\begin_layout Plain Layout


\backslash
begin{document}
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

Below is a code chunk.
\end_layout

\begin_layout Plain Layout

<<foo, echo=TRUE>>=
\end_layout

\begin_layout Plain Layout

z = 1+1
\end_layout

\begin_layout Plain Layout

plot(cars)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

The value of z is 
\backslash
Sexpr{z}.
\end_layout

\begin_layout Plain Layout


\backslash
end{document}
\end_layout

\begin_layout Plain Layout


\backslash
end{verbatim}
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
R Markdown (*.Rmd)
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{verbatim}
\end_layout

\begin_layout Plain Layout

Hi _markdown_!
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

```{r foo, echo=TRUE}
\end_layout

\begin_layout Plain Layout

z = 1+1
\end_layout

\begin_layout Plain Layout

plot(cars)
\end_layout

\begin_layout Plain Layout

```
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

The value of z is `r z`.
\end_layout

\begin_layout Plain Layout


\backslash
end{verbatim}
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Brew (*.brew)
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{verbatim}
\end_layout

\begin_layout Plain Layout

The value of pi is <% pi %>.
\end_layout

\begin_layout Plain Layout


\backslash
end{verbatim}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
end{multicols}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout

$body$
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
end{multicols}
\end_layout

\end_inset


\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc true
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview={XYZ null null 1}"
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 2.5cm
\topmargin 2.5cm
\rightmargin 2.5cm
\bottommargin 2.5cm
\secnumdepth 2
\tocdepth 2
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

# set global chunk options
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/minimal-', fig.align='center', fig.show='hold')
\end_layout

\begin_layout Plain Layout

options(replace.assign=TRUE,width=90)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
A Minimal Demo of knitr
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
You can test if 
\series bold
knitr
\series default
 works with this minimal demo.
 OK, let's get started with some boring random numbers:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-random>>=
\end_layout

\begin_layout Plain Layout

set.seed(1121)
\end_layout

\begin_layout Plain Layout

(x=rnorm(20))
\end_layout

\begin_layout Plain Layout

mean(x);var(x)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The first element of 
\family typewriter
x
\family default
 is 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
Sexpr{x[1]}
\end_layout

\end_inset

.
 Boring boxplots and histograms recorded by the PDF device:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-plots, fig.width=4, fig.height=4, out.width='.4
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

## two plots side by side (option fig.show='hold')
\end_layout

\begin_layout Plain Layout

par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3,las=1)
\end_layout

\begin_layout Plain Layout

boxplot(x)
\end_layout

\begin_layout Plain Layout

hist(x,main='')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Do the above chunks work? You should be able to compile the TeX document
 and get a PDF file like this one: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://bitbucket.org/stat/knitr/downloads/knitr-minimal.pdf
\end_layout

\end_inset

.
 The Rnw source of this document is at 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/examples/knitr-minimal.Rnw
\end_layout

\end_inset

.
\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass tufte-handout
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[buttonsize=1em]{animate}
\end_preamble
\options justified
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman default
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks true
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview=FitH"
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\secnumdepth 2
\tocdepth 2
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

options(replace.assign=TRUE,width=50)
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/graphics-', cache.path='cache/graphics-',
 fig.align='center', dev='tikz', fig.width=5, fig.height=5, fig.show='hold',
 cache=TRUE, par=TRUE)
\end_layout

\begin_layout Plain Layout

knit_hooks$set(par=function(before, options, envir){
\end_layout

\begin_layout Plain Layout

if (before && options$fig.show!='none') par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mg
p=c(2,.7,0),tcl=-.3)
\end_layout

\begin_layout Plain Layout

}, crop=hook_pdfcrop)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
knitr Graphics Manual
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Abstract
This manual shows features of graphics in the 
\series bold
knitr
\series default
 package (version 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
Sexpr{packageVersion('knitr')}
\end_layout

\end_inset

) in detail, including the graphical devices, plot recording, plot rearrangement
, control of plot sizes, the tikz device, figure captions, animations and
 other types of plots such as 
\series bold
rgl
\series default
 or GGobi plots.
\end_layout

\begin_layout Standard
Before reading this specific manual
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://bit.ly/knitr-graphics-src
\end_layout

\end_inset

 (Rnw source)
\end_layout

\end_inset

, you must have finished the main manual
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://bit.ly/knitr-main-pdf
\end_layout

\end_inset


\end_layout

\end_inset

.
\end_layout

\begin_layout Section
Graphical Devices
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package comes with more than 20 built-in graphical devices, and you can
 specify them through the 
\family typewriter
dev
\family default
 option.
 This document uses the global option 
\family typewriter
dev='tikz'
\family default
, i.e., the plots are recorded by the tikz device by default, but we can change
 the device locally.
 Since tikz will be used extensively throughout this manual and you will
 see plenty of tikz graphics later, now we first show a few other devices.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<test-plot, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

with(trees, symbols(Height, Volume, circles = Girth/16, inches = FALSE,
 bg = 'deeppink', fg = 'gray30'))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float marginfigure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<pdf-dev, ref.label='test-plot', dev='pdf', out.width='
\backslash

\backslash
linewidth', echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
The default PDF device.
\begin_inset CommandInset label
LatexCommand label
name "mar:pdf-dev"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float marginfigure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<png-dev, ref.label='test-plot', dev='png', out.width='
\backslash

\backslash
linewidth', echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
The PNG device.
\begin_inset CommandInset label
LatexCommand label
name "mar:png-dev"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "mar:pdf-dev"

\end_inset

 and 
\begin_inset CommandInset ref
LatexCommand ref
reference "mar:png-dev"

\end_inset

 show two standard devices in the 
\series bold
grDevices
\series default
 package.
 We can also use devices in the 
\series bold
Cairo
\series default
 or 
\series bold
cairoDevice
\series default
 package, e.g., the chunk below uses the 
\emph on
CairoPNG()
\emph default
 device in the 
\series bold
Cairo
\series default
 package.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cairo-png-dev, ref.label='test-plot', dev='CairoPNG', out.width='.5
\backslash

\backslash
linewidth', echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Plot Recording
\end_layout

\begin_layout Standard
As mentioned in the main manual, 
\series bold
knitr
\series default
 uses the 
\series bold
evaluate
\series default
 package to record plots.
 There are two sources of plots: first, whenever 
\emph on
plot.new()
\emph default
 or 
\emph on
grid.newpage()
\emph default
 is called, 
\series bold
evaluate
\series default
 will try to save a snapshot of the current plot
\begin_inset Foot
status open

\begin_layout Plain Layout
For technical details, see 
\family typewriter
?setHook
\family default
 and 
\family typewriter
?recordPlot
\end_layout

\end_inset

; second, after each complete expression is evaluated, a snapshot is also
 saved.
 To speed up recording, the null graphical device 
\family typewriter
pdf(file = NULL)
\family default
 is used.
 Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:two-high"

\end_inset

 shows two expressions producing two high-level plots.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<two-high, fig.width=3, fig.height=2.5, out.width='.49
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

plot(cars)
\end_layout

\begin_layout Plain Layout

boxplot(cars$dist,xlab='dist')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
Two high-level plots are captured.
 The key to arrange two plots side by side is to specify the 
\family typewriter
out.width
\family default
 option so that each plot takes less than half of the line width.
 We do not have to use the 
\family typewriter
par(mfrow)
\family default
 trick, and it may not work in some cases (e.g.
 to put base graphics and 
\series bold
ggplot2
\series default
 side by side; recall Figure 1 in the main manual).
\begin_inset CommandInset label
LatexCommand label
name "fig:two-high"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:low-plot-loop"

\end_inset

 shows another example of two R expressions, but the second expression only
 involves with low-level plotting changes.
 By default, low-level plot changes are discarded, but you can retain them
 with the option 
\family typewriter
fig.keep='all'
\family default
.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<low-plot-loop, fig.width=3, fig.height=2.5, out.width='.49
\backslash

\backslash
linewidth', fig.keep='all'>>=
\end_layout

\begin_layout Plain Layout

plot(0,0,type='n',ann=FALSE)
\end_layout

\begin_layout Plain Layout

for(i in seq(0, 2*pi,length=20)) {points(cos(i),sin(i))}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
Two complete R expressions will produce at most two plots, as long as there
 are not multiple high-level plotting calls in each expression; option 
\family typewriter
fig.keep='all'
\family default
 here.
\begin_inset CommandInset label
LatexCommand label
name "fig:low-plot-loop"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Together with 
\family typewriter
fig.show='asis'
\family default
, we can show the process of plotting step by step like Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:high-with-low"

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<high-with-low, fig.width=3, fig.height=2.5, out.width='.5
\backslash

\backslash
linewidth', fig.keep='all', fig.show='asis'>>=
\end_layout

\begin_layout Plain Layout

plot(cars)
\end_layout

\begin_layout Plain Layout

abline(lm(dist~speed, data=cars))  # a regression line
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
Low-level plot changes in base graphics can be recorded separately, and
 plots can be put in the places where they were produced.
\begin_inset CommandInset label
LatexCommand label
name "fig:high-with-low"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
A further note on plot recording: 
\series bold
knitr
\series default
 examines all recorded plots (as R objects) and compares them sequentially;
 if the previous plot is a 
\begin_inset Quotes eld
\end_inset

subset
\begin_inset Quotes erd
\end_inset

 of the next plot (= previous plot + low-level changes), the previous plot
 will be removed when 
\family typewriter
fig.keep='high'
\family default
.
 If two successive plots are identical, the second one will be removed by
 default, so it might be a little bit surprising that the following chunk
 will only produce one plot by default
\begin_inset Foot
status open

\begin_layout Plain Layout
adapted from 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/issues/41
\end_layout

\end_inset


\end_layout

\end_inset

:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<identical-plots,eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

m = matrix(1:100, ncol = 10)
\end_layout

\begin_layout Plain Layout

image(m)
\end_layout

\begin_layout Plain Layout

image(m*2)  # exactly the same as previous plot
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Plot Rearrangement
\end_layout

\begin_layout Standard
We can rearrange the plots in chunks in several ways.
 They can be inserted right after the line(s) of R code which produced them,
 or accumulated till the end of the chunk.
 There is an example in the main manual demonstrating 
\family typewriter
fig.show='asis'
\family default
 for two high-level plots, and Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:high-with-low"

\end_inset

 in this manual also demonstrates this option for a high-level plot followed
 by a low-level change.
\end_layout

\begin_layout Standard
Here is an example demonstrating the option 
\family typewriter
fig.keep='last'
\family default
 (only the last plot is kept):
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fig-last, fig.width=5.2, fig.height=3, out.width='.7
\backslash

\backslash
linewidth', fig.keep='last', message=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(ggplot2)
\end_layout

\begin_layout Plain Layout

pie <- ggplot(diamonds, aes(x = factor(1), fill = cut)) + xlab('cut') +
 geom_bar(width = 1) 
\end_layout

\begin_layout Plain Layout

pie + coord_polar(theta = "y") # a pie chart
\end_layout

\begin_layout Plain Layout

pie + coord_polar() # the bullseye chart 
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
Two plots were produced in this chunk, but only the last one is kept.
 This can be useful when we experiment with many plots, but only want the
 last result.
 (Adapted from the 
\series bold
ggplot2
\series default
 website)
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
When multiple plots are produced by a code chunk, we may want to show them
 as an animation with the option 
\family typewriter
fig.show='animate'
\family default
.
 Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:clock-animation"

\end_inset

 shows a simple clock animation; you may compare the code to Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:high-with-low"

\end_inset

 to understand that high-level plots are always recorded, regardless of
 where they appeared.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<clock-animation, fig.width=3, fig.height=3, out.width='.4
\backslash

\backslash
linewidth', fig.show='animate', tidy=FALSE, crop=TRUE>>=
\end_layout

\begin_layout Plain Layout

par(mar = rep(3, 4))
\end_layout

\begin_layout Plain Layout

for (i in seq(pi/2, -4/3 * pi, length = 12)) {
\end_layout

\begin_layout Plain Layout

    plot(0, 0, pch = 20, ann = FALSE, axes = FALSE)
\end_layout

\begin_layout Plain Layout

    arrows(0, 0, cos(i), sin(i))
\end_layout

\begin_layout Plain Layout

    axis(1, 0, "VI"); axis(2, 0, "IX")
\end_layout

\begin_layout Plain Layout

    axis(3, 0, "XII"); axis(4, 0, "III"); box()
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
A clock animation.
 You have to view it in Adobe Reader: click to play/pause; there are also
 buttons to speed up or slow down the animation.
\begin_inset CommandInset label
LatexCommand label
name "fig:clock-animation"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
We can also set the alignment of plots easily with the 
\family typewriter
fig.align
\family default
 option; this document uses 
\family typewriter
fig.align='center'
\family default
 as a global option, and we can also set plots to be left/right-aligned.
 Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:maruko-plot"

\end_inset

 is an example of a left-aligned plot.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<maruko-plot, fig.width=7, fig.height=4, out.width='.45
\backslash

\backslash
linewidth', fig.align='left', crop=TRUE>>=
\end_layout

\begin_layout Plain Layout

stars(cbind(1:16,10*(16:1)),draw.segments=TRUE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
A left-aligned plot adapted from 
\family typewriter
?stars
\family default
 (I call this the 
\begin_inset Quotes eld
\end_inset

Maruko
\begin_inset Quotes erd
\end_inset

 plot, and it is one of my favorites; see 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://en.wikipedia.org/wiki/Chibi_Maruko-chan
\end_layout

\end_inset

).
\begin_inset CommandInset label
LatexCommand label
name "fig:maruko-plot"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Section
Plot Size
\end_layout

\begin_layout Standard
We have seen several examples in which two or more plots can be put side
 by side, and this is because the plots were resized in the output document;
 with the chunk option 
\family typewriter
out.width
\family default
 less than half of the line width, LaTeX will arrange two plots in one line;
 if it is less than 
\begin_inset Formula $1/3$
\end_inset

 of the line width, three plots can be put in one line.
 Of course we can also set it to be an absolute width like 
\family typewriter
3in
\family default
 (3 inches).
 This option is used extensively in this document to control the size of
 plots in the output document.
\end_layout

\begin_layout Section
The tikz Device
\end_layout

\begin_layout Standard
The main advantage of using tikz graphics is the consistency of styles between
 texts in plots and those in the main document.
 Since we can use native LaTeX commands in plots, the styles of texts in
 plots can be very sophisticated (see Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:math-formula-tt"

\end_inset

 for an example).
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<math-formula-tt, fig.width=6, fig.height=2, out.width='
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

plot(0:1,0:1,type='n', ylab='origin of statistics', xlab='statistical presentati
on rocks with 
\backslash

\backslash
LaTeX{}')
\end_layout

\begin_layout Plain Layout

text(.5,c(.8,.5,.2), c('
\backslash

\backslash
texttt{lm(y 
\backslash

\backslash
textasciitilde{} x)}', '$
\backslash

\backslash
hat{
\backslash

\backslash
beta}=(X^{
\backslash

\backslash
prime}X)^{-1}X^{
\backslash

\backslash
prime}y$',  '$(
\backslash

\backslash
Omega,
\backslash

\backslash
mathcal{F},
\backslash

\backslash
mu)$'))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
A plot created by 
\series bold
tikzDevice
\series default
 with math expressions and typewriter fonts.
 Note the font style consistency -- we write the same expressions in LaTeX
 here: 
\begin_inset Formula $\hat{\beta}=(X^{\prime}X)^{-1}X^{\prime}y$
\end_inset

 and 
\begin_inset Formula $(\Omega,\mathcal{F},\mu)$
\end_inset

; also 
\family typewriter
lm(y ~ x)
\family default
.
\begin_inset CommandInset label
LatexCommand label
name "fig:math-formula-tt"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
When using XeLaTeX instead of PDFLaTeX to compile the document, we need
 to tell the 
\series bold
tikzDevice
\series default
 package by setting the 
\family typewriter
tikzDefaultEngine
\family default
 option before all plot chunks (preferably in the first chunk):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<xetex-tikz, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

options(tikzDefaultEngine='xetex')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This is useful and often necessary to compile tikz plots which contain (UTF8)
 multi-byte characters.
\end_layout

\begin_layout Section
Figure Caption
\end_layout

\begin_layout Standard
If the chunk option 
\family typewriter
fig.cap
\family default
 is not 
\family typewriter
NULL
\family default
 or 
\family typewriter
NA
\family default
, the plots will be put in a 
\family typewriter
figure
\family default
 environment when the output format is LaTeX, and this option is used to
 write a caption in this environment using 
\family typewriter

\backslash
caption{}
\family default
.
 The other two related options are 
\family typewriter
fig.scap
\family default
 and 
\family typewriter
fig.lp
\family default
 which set the short caption and a prefix string for the figure label.
 The default short caption is extracted from the caption by truncating it
 at the first period or colon or semi-colon.
 The label is a combination of 
\family typewriter
fig.lp
\family default
 and the chunk label.
 Because 
\family typewriter
figure
\family default
 is a float environment, it can float away from the chunk output to other
 places such as the top or bottom of a page when the TeX document is compiled.
\end_layout

\begin_layout Section
Other Features
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package can be extended with hook functions, and here I give a few examples
 illustrating the flexibility.
\end_layout

\begin_layout Subsection
Cropping PDF Graphics
\end_layout

\begin_layout Standard
Some R users may have been suffering from the extra margins in R plots,
 especially in base graphics (
\series bold
ggplot2
\series default
 is much better in this aspect).
 The default graphical option 
\family typewriter
mar
\family default
 is about 
\family typewriter
c(5, 4, 4, 2)
\family default
 (see 
\family typewriter
?par
\family default
), which is often too big.
 Instead of endlessly setting 
\family typewriter
par(mar)
\family default
, you may consider the program 
\family typewriter
pdfcrop
\family default
, which can crop the white margin automatically
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.ctan.org/pkg/pdfcrop
\end_layout

\end_inset


\end_layout

\end_inset

.
 In 
\series bold
knitr
\series default
, we can set up the hook 
\emph on
hook_pdfcrop()
\emph default
 to work with a chunk option, say, 
\family typewriter
crop
\family default
.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<crop-hook, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

knit_hooks$set(crop=hook_pdfcrop)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Now we compare two plots below.
 The first one is not cropped (Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pdf-nocrop"

\end_inset

); then the same plot is produced but with a chunk option 
\family typewriter
crop=TRUE
\family default
 which will call the cropping hook (Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pdf-crop"

\end_inset

).
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{kframe}
\end_layout

\begin_layout Plain Layout

<<pdf-nocrop, fig.width=3.2, fig.height=3.2, out.width='.5
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

par(mar=c(5,4,4,2),bg='white')  # large margin
\end_layout

\begin_layout Plain Layout

plot(lat~long,data=quakes,pch=20,col=rgb(0,0,0,.2))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout


\backslash
end{kframe}
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
The original plot produced in R, with a large margin.
\begin_inset CommandInset label
LatexCommand label
name "fig:pdf-nocrop"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{kframe}
\end_layout

\begin_layout Plain Layout

<<pdf-crop, ref.label='pdf-nocrop', echo=FALSE, fig.width=3.2, fig.height=3.2,
 out.width='.5
\backslash

\backslash
linewidth', crop=TRUE>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout


\backslash
end{kframe}
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption

\begin_layout Plain Layout
The cropped plot; it fits better in the document.
\begin_inset CommandInset label
LatexCommand label
name "fig:pdf-crop"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
As we can see, the white margins are gone.
 If we use 
\emph on
par()
\emph default
, it might be hard and tedious to figure out a reasonable amount of margin
 in order that neither is any label cropped nor do we get a too large margin.
 My experience is that 
\family typewriter
pdfcrop
\family default
 works well with base graphics, but barely works with 
\series bold
grid
\series default
 graphics (therefore 
\series bold
lattice
\series default
 and 
\series bold
ggplot2
\series default
 are ruled out).
\end_layout

\begin_layout Subsection
Manually Saved Plots
\end_layout

\begin_layout Standard
We have explained how R plots are recorded before.
 In some cases, it is not possible to capture plots by 
\emph on
recordPlot()
\emph default
 (such as 
\series bold
rgl
\series default
 plots), but we can save them using other functions.
 To insert these plots into the output, we need to set up a hook first like
 this (see 
\family typewriter
?hook_plot_custom
\family default
 for details):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<custom-plot-hook, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

knit_hooks$set(custom.plot = hook_plot_custom)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Then we set the chunk option 
\family typewriter
custom.plot=TRUE
\family default
, and manually write plot files in the chunk.
 Here we show an example of capturing GGobi plots using the function 
\emph on
ggobi_display_save_picture()
\emph default
 in the 
\series bold
rggobi
\series default
 package:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ggobi-plot, custom.plot=TRUE, fig.ext='png', out.width='2.5in', results='hide',
 warning=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(rggobi)
\end_layout

\begin_layout Plain Layout

ggobi(ggobi_find_file('data', 'flea.csv'))
\end_layout

\begin_layout Plain Layout

Sys.sleep(1)  # wait for snapshot
\end_layout

\begin_layout Plain Layout

ggobi_display_save_picture(path=fig_path('.png'))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
One thing to note here is we have to make sure the plot filename is from
 
\emph on
fig_path()
\emph default
, which is a convenience function to return the figure path for the current
 chunk.
\end_layout

\begin_layout Standard
We can do whatever normal R plots can do with this hook, and we give another
 example below to show how to work with animations.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<rgl-animation, custom.plot=TRUE, fig.ext='png', out.width='2.5in', fig.show='animat
e', fig.num=20, interval=.4,  aniopts='controls,autoplay', results='hide'>>=
\end_layout

\begin_layout Plain Layout

library(animation) # adapted from demo('rgl_animation')
\end_layout

\begin_layout Plain Layout

data(pollen)
\end_layout

\begin_layout Plain Layout

uM = matrix(c(-0.37, -0.51, -0.77, 0, -0.73, 0.67, -0.1, 0, 0.57, 0.53, -0.63, 0,
 0, 0, 0, 1), 4, 4)
\end_layout

\begin_layout Plain Layout

library(rgl)
\end_layout

\begin_layout Plain Layout

open3d(userMatrix = uM, windowRect = c(0, 0, 400, 400))
\end_layout

\begin_layout Plain Layout

plot3d(pollen[, 1:3])
\end_layout

\begin_layout Plain Layout

zm = seq(1, 0.05, length = 20)
\end_layout

\begin_layout Plain Layout

par3d(zoom = 1)  # change the zoom factor gradually later
\end_layout

\begin_layout Plain Layout

for (i in 1:length(zm)) {
\end_layout

\begin_layout Plain Layout

    par3d(zoom = zm[i]); Sys.sleep(.05)
\end_layout

\begin_layout Plain Layout

    rgl.snapshot(paste(fig_path(i), 'png', sep = '.'))
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
rgl Plots
\end_layout

\begin_layout Standard
With the hook 
\emph on
hook_rgl()
\emph default
, we can easily save snapshots from the 
\series bold
rgl
\series default
 package.
 We have shown an example in the main manual, and here we add some details.
 The rgl hook is a good example of taking care of details by carefully using
 the 
\family typewriter
options
\family default
 argument in the hook; for example, we cannot directly set the width and
 height of rgl plots in 
\emph on
rgl.snapshot()
\emph default
 or 
\emph on
rgl.postscript()
\emph default
, so we make use of the options 
\family typewriter
fig.width
\family default
, 
\family typewriter
fig.height
\family default
 and 
\family typewriter
dpi
\family default
 to calculate the expected size of the window, then resize the current window
 by 
\emph on
par3d()
\emph default
, and finally save the plot.
\end_layout

\begin_layout Standard
This hook is actually built upon 
\emph on
hook_plot_custom()
\emph default
 -- first it saves the 
\series bold
rgl
\series default
 snapshot, then it calls 
\emph on
hook_plot_custom()
\emph default
 to write the output code.
\end_layout

\begin_layout Section
\start_of_appendix
How to Compile This Manual
\end_layout

\begin_layout Standard
This manual has a long chain of dependencies, so it may not be easy to compile.
 These packages are required (all of them are free software):
\end_layout

\begin_layout Description
R Cairo, ggplot2, tikzDevice, rgl, rggobi, animation (all available on CRAN
 except tikzDevice which is on R-Forge for the time being)
\end_layout

\begin_layout Description
LaTeX animate, hyperref and the tufte-handout class
\end_layout

\begin_layout Description
Other GGobi, pdfcrop
\end_layout

\end_body
\end_document

<!--
%\VignetteEngine{knitr::docco_classic}
%\VignetteIndexEntry{R Markdown with the Docco Classic Style}
-->

# R Markdown with the Docco Classic Style

```{r setup, echo=FALSE, results='asis'}
x = readLines('docco-linear.Rmd')[-(1:7)]
x = gsub('linear', 'classic', x)
i = grep('^knit2html[(][.]{3}', x)
x[i - 1] = '```{r}'
x[i] = 'head(knitr::rocco, 5)'
library(knitr)
cat(knit_child(text = x, quiet = TRUE), sep = '\n')
```

You probably have noticed that you can adjust the widths of the two columns
using your cursor. What is more, press `T` on your keyboard, and see what
happens.

%% LyX 2.1.3 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[T1]{fontenc}
\usepackage[landscape]{geometry}
\geometry{verbose,tmargin=0cm,bmargin=0cm,lmargin=1cm,rmargin=1cm}
\setlength{\parskip}{0in}
\setlength{\parindent}{0pt}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
%% Because html converters don't know tabularnewline
\providecommand{\tabularnewline}{\\}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\usepackage{multicol}
\IfFileExists{upquote.sty}{\usepackage{upquote}}{}
\providecommand{\tightlist}{%
  \setlength{\itemsep}{0pt}\setlength{\parskip}{0pt}}

\makeatother

\begin{document}

\title{knitr Reference Card}


\author{Yihui Xie}

\maketitle
\begin{multicols}{2}


\section{Syntax}

\begin{small}

\noindent \begin{center}
\begin{tabular}{l|l|l|l|l}
\hline 
format & start & end & inline & output\tabularnewline
\hline 
Rnw & \verb|<<*>>=| & \texttt{@} & \verb|\|\verb|Sexpr{x}| & \TeX{}\tabularnewline
\hline 
Rmd & \verb|```{r *}| & \verb|```| & \verb|`r x`| & MD\tabularnewline
\hline 
Rhtml & \verb|<!--begin.rcode *| & \verb|end.rcode-->| & \verb|<!--rinline x-->| & HTML\tabularnewline
\hline 
Rrst & \texttt{..~\{r {*}\}} & \texttt{..~..} & \verb|:r:`x`| & reST\tabularnewline
\hline 
Rtex & \texttt{\% begin.rcode {*}} & \texttt{\% end.rcode} & \verb|\rinline{x}| & \TeX{}\tabularnewline
\hline 
Rasciidoc & \texttt{// begin.rcode {*}} & \texttt{// end.rcode} & \texttt{+r x+} & AsciiDoc\tabularnewline
\hline 
Rtextile & \texttt{\#\#\# begin.rcode {*}} & \texttt{\#\#\# end.rcode} & \texttt{@r x@} & Textile\tabularnewline
\hline 
brew &  &  & \verb|<% x %>| & text\tabularnewline
\hline 
\end{tabular}
\par\end{center}

\end{small}

\texttt{{*}} denotes local chunk options, e.g. \verb|<<label, eval=FALSE>>=|;
\texttt{x} denotes inline R code, e.g. \verb|`r 1+2`| (MD stands
for \href{http://en.wikipedia.org/wiki/Markdown}{Markdown})


\section{Minimal Examples}

\begin{multicols}{2}


\subsection{Sweave ({*}.Rnw)}

\begin{verbatim}
\documentclass{article}
\begin{document}

Below is a code chunk.
<<foo, echo=TRUE>>=
z = 1+1
plot(cars)
@

The value of z is \Sexpr{z}.
\end{document}
\end{verbatim}


\subsection{R Markdown ({*}.Rmd)}

\begin{verbatim}
Hi _markdown_!

```{r foo, echo=TRUE}
z = 1+1
plot(cars)
```

The value of z is `r z`.
\end{verbatim}


\subsection{Brew ({*}.brew)}

\begin{verbatim}
The value of pi is <% pi %>.
\end{verbatim}

\end{multicols}

$body$

\end{multicols}
\end{document}

# A Minimal Example for Markdown

This is a minimal example of using **knitr** to produce an _HTML_ page from _Markdown_.

## R code chunks

```{r setup}
# set global chunk options: images will be 7x5 inches
knitr::opts_chunk$set(fig.width=7, fig.height=5)
options(digits = 4)
```

Now we write some code chunks in this markdown file:

```{r computing}
x <- 1+1 # a simple calculator
set.seed(123)
rnorm(5)  # boring random numbers
```

We can also produce plots:

```{r graphics}
par(mar = c(4, 4, .1, .1))
with(mtcars, {
  plot(mpg~hp, pch=20, col='darkgray')
  lines(lowess(hp, mpg))
})
```

## Inline code

Inline R code is also supported, e.g. the value of `x` is `r x`, and 2 &times; &pi;
= `r 2*pi`.

## Math

LaTeX math as usual: $f(\alpha, \beta) \propto x^{\alpha-1}(1-x)^{\beta-1}$.

## Misc

You can indent code chunks so they can nest within other environments such as lists.

1. the area of a circle with radius x
    ```{r foo}
    pi * x^2
    ```
2. OK, that is great

To compile me, use

```{r compile, eval=FALSE}
library(knitr)
knit('knitr-minimal.Rmd')
```

## Conclusion

Markdown is super easy to write. Go to **knitr** [homepage](https://yihui.org/knitr/) for details.

#LyX 2.1 created this file. For more info see http://www.lyx.org/
\lyxformat 474
\begin_document
\begin_header
\textclass article
\begin_preamble
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[buttonsize=1em]{animate}
\end_preamble
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans lmss
\font_typewriter lmtt
\font_math auto
\font_default_family default
\use_non_tex_fonts false
\font_sc true
\font_osf false
\font_sf_scale 100
\font_tt_scale 100
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview={XYZ null null 1}"
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 1
\use_package mathtools 1
\use_package mhchem 1
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine natbib
\cite_engine_type authoryear
\biblio_style plainnat
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 2.5cm
\topmargin 2.5cm
\rightmargin 2.5cm
\bottommargin 2.5cm
\secnumdepth 2
\tocdepth 2
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

## set global chunk options
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path='figure/manual-', cache.path='cache/manual-', fig.align='ce
nter', fig.show='hold', par=TRUE)
\end_layout

\begin_layout Plain Layout

## I use = but I can replace it with <-; set code/output width to be 68
\end_layout

\begin_layout Plain Layout

options(formatR.arrow=TRUE, width=68, digits=4)
\end_layout

\begin_layout Plain Layout

## tune details of base graphics (https://yihui.org/knitr/hooks)
\end_layout

\begin_layout Plain Layout

knit_hooks$set(par=function(before, options, envir){
\end_layout

\begin_layout Plain Layout

if (before && options$fig.show!='none') par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mg
p=c(2,.7,0),tcl=-.3)
\end_layout

\begin_layout Plain Layout

})
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
knitr: A General-Purpose Tool for Dynamic Report Generation in R
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
The original paradigm of literate programming was brought forward mainly
 for software development, or specifically, to mix source code (for computer)
 and documentation (for human) together.
 Early systems include 
\begin_inset CommandInset href
LatexCommand href
name "WEB"
target "http://www.literateprogramming.com/web.pdf"

\end_inset

 and 
\begin_inset CommandInset href
LatexCommand href
name "Noweb"
target "http://www.cs.tufts.edu/~nr/noweb/"

\end_inset

; Sweave 
\begin_inset CommandInset citation
LatexCommand citep
key "leisch2002"

\end_inset

 was derived from the latter, but it is less focused on documenting software,
 instead it is mainly used for reproducible data analysis and generating
 statistical reports.
 The 
\series bold
knitr
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-knitr"

\end_inset

 is following the steps of Sweave.
 For this manual, I assume readers have some background knowledge of Sweave
 to understand the technical details; for a reference of available options,
 hooks and demos, see the package homepage 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/
\end_layout

\end_inset

.
\end_layout

\begin_layout Section
Hello World
\end_layout

\begin_layout Standard
A natural question is why to reinvent the wheel.
 The short answer is that extending Sweave by hacking 
\family sans
SweaveDrivers.R
\family default
 in the 
\series bold
utils
\series default
 package is a difficult job to me.
 Many features in 
\series bold
knitr
\series default
 come naturally as users would have expected.
 Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:cars-demo"

\end_inset

 is a simple demo of some features of 
\series bold
knitr
\series default
.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cars-demo,dev='tikz',fig.width=4,fig.height=2.8,out.width='.45
\backslash

\backslash
textwidth',message=FALSE,cache=TRUE>>=
\end_layout

\begin_layout Plain Layout

fit=lm(dist~speed,data=cars) # linear regression
\end_layout

\begin_layout Plain Layout

par(mar=c(4, 4, 1, .1), mgp=c(2,1,0))
\end_layout

\begin_layout Plain Layout

with(cars,plot(speed,dist,panel.last=abline(fit)))
\end_layout

\begin_layout Plain Layout

text(10,100,'$Y = 
\backslash

\backslash
beta_0 + 
\backslash

\backslash
beta_1x + 
\backslash

\backslash
epsilon$')
\end_layout

\begin_layout Plain Layout

library(ggplot2)
\end_layout

\begin_layout Plain Layout

qplot(speed, dist, data=cars)+geom_smooth()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
\begin_inset CommandInset label
LatexCommand label
name "fig:cars-demo"

\end_inset

A simple demo of possible output in 
\series bold
knitr
\series default
: (1) multiple plots per chunk; (2) no need to 
\emph on
print()
\emph default
 objects in 
\series bold
ggplot2
\series default
; (3) device size is 
\begin_inset Formula $4\times2.8$
\end_inset

 (inches) but output size is adjusted to 
\family typewriter
.45
\backslash
textwidth
\family default
 in chunk options; (4) base graphics and 
\series bold
ggplot2
\series default
 can sit side by side; (5) use the 
\emph on
tikz()
\emph default
 device in 
\series bold
tikzDevice
\series default
 by setting chunk option 
\family typewriter
dev='tikz'
\family default
 (hence can write native LaTeX expressions in R plots); (6) code highlighting.
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
I would have chosen to hide the R code if this were a real report, but here
 I show the code just for the sake of demonstration.
 If we type 
\emph on
qplot()
\emph default
 in R, we get a plot, and the same thing happens in 
\series bold
knitr
\series default
.
 If we draw two plots in the code, 
\series bold
knitr
\series default
 will show two plots and we do not need to tell it how many plots are there
 in the code in advance.
 If we set 
\family typewriter
out.width='.49
\backslash

\backslash
textwidth'
\family default
 in chunk options, we get it in the final output document.
 If we say 
\family typewriter
fig.align='center'
\family default
, the plots are centered.
 That's it.
 Many enhancements and new features will be introduced later.
 If you come from the Sweave land, you can take a look at the page of transition
 first: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/sweave/
\end_layout

\end_inset

.
\end_layout

\begin_layout Section
Design
\end_layout

\begin_layout Standard
The flow of processing an input file is similar to Sweave, and two major
 differences are that 
\series bold
knitr
\series default
 provides more flexibility to the users to customize the processing, and
 has many built-in options such as the support to a wide range of graphics
 devices and cache.
 Below is a brief description of the process:
\end_layout

\begin_layout Enumerate

\series bold
knitr
\series default
 takes an input file and automatically determines an appropriate set of
 
\begin_inset CommandInset href
LatexCommand href
name "patterns"
target "https://yihui.org/knitr/patterns"

\end_inset

 to use if they are not provided in advance (e.g.
 
\family sans
file.Rnw
\family default
 will use 
\family typewriter
knit_patterns$get('rnw')
\family default
);
\end_layout

\begin_layout Enumerate
a set of output 
\begin_inset CommandInset href
LatexCommand href
name "hooks"
target "https://yihui.org/knitr/hooks"

\end_inset

 will also be set up automatically according to the filename extension (e.g.
 use LaTeX environments or HTML elements to wrap up R results);
\end_layout

\begin_layout Enumerate
the input file is read in and split into pieces consisting of R code chunks
 and normal texts; the former will be executed one after the other, and
 the latter may contain global chunk options or inline R code;
\end_layout

\begin_layout Enumerate
for each chunk, the code is evaluated using the 
\series bold
evaluate
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-evaluate"

\end_inset

, and the results may be filtered according to chunk options (e.g.
 
\family typewriter
echo=FALSE
\family default
 will remove the R source code)
\end_layout

\begin_deeper
\begin_layout Enumerate
if 
\family typewriter
cache=TRUE
\family default
 for this chunk, 
\series bold
knitr
\series default
 will first check if there are previously cached results under the cache
 directory before really evaluating the chunk; if cached results exist and
 this code chunk has not been changed since last run (use MD5 sum to verify),
 the cached results will be (lazy-) loaded, otherwise new cache will be
 built; if a cached chunk depends on other chunks (see the 
\family typewriter
dependson
\family default
 
\begin_inset CommandInset href
LatexCommand href
name "option"
target "https://yihui.org/knitr/options"

\end_inset

) and any one of these chunks has changed, this chunk must be forcibly updated
 (old cache will be purged)
\end_layout

\begin_layout Enumerate
there are six types of possible output from 
\series bold
evaluate
\series default
, and their classes are 
\family typewriter
character
\family default
 (normal text output), 
\family typewriter
source
\family default
 (source code), 
\family typewriter
warning
\family default
, 
\family typewriter
message
\family default
, 
\family typewriter
error
\family default
 and 
\family typewriter
recordedplot
\family default
; an internal S3 generic function 
\emph on
wrap()
\emph default
 is used to deal with different types of output, using output hooks defined
 in the object 
\family typewriter
knit_hooks
\end_layout

\begin_layout Enumerate
note plots are recorded as R objects before they are really saved to files,
 so graphics devices will not be opened unless plots have really been produced
 in a chunk
\end_layout

\begin_layout Enumerate
a code chunk is evaluated in a separate empty environment with the global
 environment as its parent, and all the objects in this environment after
 the evaluation will be saved if 
\family typewriter
cache=TRUE
\end_layout

\begin_layout Enumerate
chunk hooks can be run before and/or after a chunk
\end_layout

\end_deeper
\begin_layout Enumerate
for normal texts, 
\series bold
knitr
\series default
 will find inline R code (e.g.
 in 
\family typewriter

\backslash
Sexpr{}
\family default
) and evaluate it; the output is wrapped by the 
\family typewriter
inline
\family default
 hook;
\end_layout

\begin_layout Standard
The hooks play important roles in 
\series bold
knitr
\series default
: this package makes almost everything accessible to the users.
 Consider the following extremely simple example which may demonstrate this
 freedom:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<simple-example>>=
\end_layout

\begin_layout Plain Layout

1+1
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
There are two parts in the final output: the source code 
\family typewriter
1 + 1
\family default
 and the output 
\family typewriter
[1] 2
\family default
; the comment characters 
\family typewriter
##
\family default
 are from the default chunk option 
\family typewriter
comment
\family default
.
 Users may define a hook function for the source code like this to use the
 
\family typewriter
lstlisting
\family default
 environment:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<hook-source, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

knit_hooks$set(source = function(x, options) {
\end_layout

\begin_layout Plain Layout

paste('
\backslash

\backslash
begin{lstlisting}
\backslash
n', x, '
\backslash

\backslash
end{lstlisting}
\backslash
n', sep = '')
\end_layout

\begin_layout Plain Layout

})
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Similarly we can put other types of output into other environments.
 There is no need to hack at 
\family sans
Sweave.sty
\family default
 for 
\series bold
knitr
\series default
 and you can put the output in any environments.
 What is more, the output hooks make 
\series bold
knitr
\series default
 ready for other types of output, and a typical one is HTML (there are built-in
 hooks).
 The website has provided many examples demonstrating the flexibility of
 the output.
\end_layout

\begin_layout Section
Features
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package borrowed features such as tikz graphics and cache from 
\series bold
pgfSweave
\series default
 and 
\series bold
cacheSweave
\series default
 respectively, but the implementations are different.
 New features like code reference from an external R script as well as output
 customization are also introduced.
 The feature of hook functions in Sweave is re-implemented and hooks have
 new usage now.
 There are several other small features which are motivated from my everyday
 use of Sweave.
 For example, a progress bar is provided when knitting a file so we roughly
 know how long we still need to wait; output from inline R code (e.g.
 
\family typewriter

\backslash
Sexpr{x[1]}
\family default
) is automatically formatted in TeX math notation (like 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
Sexpr{123456789}
\end_layout

\end_inset

) if the result is numeric.
 You may check out a number of specific manuals dedicated to specific features
 such as graphics in the website: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demos
\end_layout

\end_inset

.
\end_layout

\begin_layout Subsection
Code Decoration
\end_layout

\begin_layout Standard
The 
\series bold
highr
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-highr"

\end_inset

 is used to highlight R code, and the 
\series bold
formatR
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-formatR"

\end_inset

 is used to reformat R code (like 
\family typewriter
keep.source=FALSE
\family default
 in Sweave but will also try to retain comments).
 For LaTeX output, the 
\series bold
framed
\series default
 package is used to decorate code chunks with a light gray background.
 If this LaTeX package is not found in the system, a version will be copied
 directly from 
\series bold
knitr
\series default
.
 The prompt characters are removed by default because they mangle the R
 source code in the output and make it difficult to copy R code.
 The R output is masked in comments by default based on the same rationale.
 It is easy to revert to the output with prompts (set option 
\family typewriter
prompt=TRUE
\family default
), and you will quickly realize the inconvenience to the readers if they
 want to copy and run the code in the output document:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<stupid-prompts, prompt=TRUE, comment=NA, highlight=FALSE>>=
\end_layout

\begin_layout Plain Layout

x=rnorm(5)
\end_layout

\begin_layout Plain Layout

x
\end_layout

\begin_layout Plain Layout

var(x)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The example below shows the effect of 
\family typewriter
tidy=TRUE/FALSE
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<tidy-no, eval=FALSE, tidy=FALSE>>=
\end_layout

\begin_layout Plain Layout

## option tidy=FALSE
\end_layout

\begin_layout Plain Layout

for(k in 1:10){j=cos(sin(k)*k^2)+3;print(j-5)}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\begin_layout Plain Layout

<<tidy-yes, eval=FALSE, tidy=TRUE>>=
\end_layout

\begin_layout Plain Layout

## option tidy=TRUE
\end_layout

\begin_layout Plain Layout

for(k in 1:10){j=cos(sin(k)*k^2)+3;print(j-5)}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Note 
\family typewriter
=
\family default
 is replaced by 
\family typewriter
<-
\family default
 because 
\family typewriter
options('formatR.arrow')
\family default
 was set to be 
\family typewriter
TRUE
\family default
 in this document; see the documentation of 
\emph on
tidy.source()
\emph default
 in 
\series bold
formatR
\series default
 for details.
\end_layout

\begin_layout Standard
Many highlighting themes can be used in 
\series bold
knitr
\series default
, which are borrowed from the 
\series bold
highlight
\series default
 package by 
\begin_inset CommandInset href
LatexCommand href
name "Andre Simon"
target "http://www.andre-simon.de/"

\end_inset


\begin_inset Foot
status open

\begin_layout Plain Layout
not the R package mentioned before; for a preview of these themes, see 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.andre-simon.de/dokuwiki/doku.php?id=theme_examples
\end_layout

\end_inset


\end_layout

\end_inset

; it is also possible to use themes from 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.eclipsecolorthemes.org/
\end_layout

\end_inset

 by providing a theme id to 
\series bold
knitr
\series default

\begin_inset Foot
status open

\begin_layout Plain Layout
many thanks to 
\begin_inset CommandInset href
LatexCommand href
name "Ramnath Vaidyanathan"
target "https://github.com/ramnathv"

\end_inset

 for the work on themes
\end_layout

\end_inset

.
 See 
\family typewriter
?knit_theme
\family default
 for details.
\end_layout

\begin_layout Subsection
Graphics
\end_layout

\begin_layout Standard
Graphics is an important part of reports, and several enhancements have
 been made in 
\series bold
knitr
\series default
.
 For example, grid graphics may not need to be explicitly printed as long
 as the same code can produce plots in R (in some cases, however, they have
 to be printed, e.g.
 in a loop, because you have to do so in an R terminal).
\end_layout

\begin_layout Subsubsection
Graphical Devices
\end_layout

\begin_layout Standard
Over a long time, a frequently requested feature for Sweave was the support
 for other graphics devices, which has been implemented since R 2.13.0.
 Instead of using logical options like 
\family typewriter
png
\family default
 or 
\family typewriter
jpeg
\family default
 (this list can go on and on), 
\series bold
knitr
\series default
 uses a single option 
\family typewriter
dev
\family default
 (like 
\family typewriter
grdevice
\family default
 in Sweave) which has support for more than 20 devices.
 For instance, 
\family typewriter
dev='png'
\family default
 will use the 
\emph on
png()
\emph default
 device, and 
\family typewriter
dev='CairoJPEG'
\family default
 uses the 
\emph on
CairoJPEG()
\emph default
 device in the 
\series bold
Cairo
\series default
 package (it has to be installed first, of course).
 If none of these devices is satisfactory, you can provide the name of a
 customized device function, which must have been defined before it is called.
\end_layout

\begin_layout Subsubsection
Plot Recording
\end_layout

\begin_layout Standard
As mentioned before, all the plots in a code chunk are first recorded as
 R objects and then 
\begin_inset Quotes eld
\end_inset

replayed
\begin_inset Quotes erd
\end_inset

 inside a graphical device to generate plot files.
 The 
\series bold
evaluate
\series default
 package will record plots per 
\emph on
expression
\emph default
 basis, in other words, the source code is split into individual complete
 expressions and 
\series bold
evaluate
\series default
 will examine possible plot changes in snapshots after each single expression
 has been evaluated.
 For example, the code below consists of three expressions, out of which
 two are related to drawing plots, therefore 
\series bold
evaluate
\series default
 will produce two plots by default:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<low-level-plots, fig.keep='all', dev='tikz', fig.width=2.5, fig.height=2.5,
 out.width='.3
\backslash

\backslash
textwidth', cache=TRUE>>=
\end_layout

\begin_layout Plain Layout

par(mar=c(3,3,.1,.1))
\end_layout

\begin_layout Plain Layout

plot(1:10, ann=FALSE,las=1)
\end_layout

\begin_layout Plain Layout

text(5,9,'mass $
\backslash

\backslash
rightarrow$ energy
\backslash
n$E=mc^2$')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This brings a significant difference with traditional tools in R for dynamic
 report generation, since low-level plotting changes can also be recorded.
 The option 
\family typewriter
fig.keep
\family default
 controls which plots to keep in the output; 
\family typewriter
fig.keep='all'
\family default
 will keep low-level changes as separate plots; by default (
\family typewriter
fig.keep='high'
\family default
), 
\series bold
knitr
\series default
 will merge low-level plot changes into the previous high-level plot, like
 most graphics devices do.
 This feature may be useful for teaching R graphics step by step.
 Note, however, low-level plotting commands in a single expression (a typical
 case is a loop) will not be recorded accumulatively, but high-level plotting
 commands, regardless of where they are, will always be recorded.
 For example, this chunk will only produce 2 plots instead of 21 plots because
 there are 2 complete expressions:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<low-plot-loop, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

plot(0,0,type='n',ann=FALSE)
\end_layout

\begin_layout Plain Layout

for(i in seq(0, 2*pi,length=20)) points(cos(i),sin(i))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
But this will produce 20 plots as expected:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<high-plot-loop, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

for(i in seq(0, 2*pi,length=20)) {plot(cos(i),sin(i),xlim=c(-1,1),ylim=c(-1,1))}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
As I showed in the beginning of this manual, it is straightforward to let
 
\series bold
knitr
\series default
 keep all the plots in a chunk and insert them into the output document,
 so we no longer need the 
\family typewriter
cat('
\backslash

\backslash
includegraphics{}')
\family default
 trick.
\end_layout

\begin_layout Standard
We can discard all previous plots and keep the last one only by 
\family typewriter
fig.keep='last'
\family default
, or keep only the first plot by 
\family typewriter
fig.keep='first'
\family default
, or discard all plots by 
\family typewriter
fig.keep='none'
\family default
.
\end_layout

\begin_layout Subsubsection
Plot Rearrangement
\end_layout

\begin_layout Standard
The option 
\family typewriter
fig.show
\family default
 can decide whether to hold all plots while evaluating the code and 
\begin_inset Quotes eld
\end_inset

flush
\begin_inset Quotes erd
\end_inset

 all of them to the end of a chunk (
\family typewriter
fig.show='hold'
\family default
), or just insert them to the place where they were created (by default
 
\family typewriter
fig.show='asis'
\family default
).
 Here is an example of 
\family typewriter
fig.show='asis'
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fig-hold, fig.show='asis', dev='pdf', fig.width=6, fig.height=4, out.width='.35
\backslash

\backslash
linewidth'>>=
\end_layout

\begin_layout Plain Layout

contour(volcano)  # contour lines
\end_layout

\begin_layout Plain Layout

filled.contour(volcano)  # fill contour plot with colors
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Beside 
\family typewriter
hold
\family default
 and 
\family typewriter
asis
\family default
, the option 
\family typewriter
fig.show
\family default
 can take a third value: 
\family typewriter
animate
\family default
, which makes it possible to insert animations into the output document.
 In LaTeX, the package 
\series bold
animate
\series default
 is used to put together image frames as an animation.
 For animations to work, there must be more than one plot produced in a
 chunk.
 The option 
\family typewriter
interval
\family default
 controls the time interval between animation frames; by default it is 1
 second.
 Note you have to add 
\family typewriter

\backslash
usepackage{animate}
\family default
 in the LaTeX preamble, because 
\series bold
knitr
\series default
 will not add it automatically.
 Animations in the PDF output can only be viewed in Adobe Reader.
\end_layout

\begin_layout Standard
As a simple demonstration, here is a 
\begin_inset CommandInset href
LatexCommand href
name "Mandelbrot animation"
target "http://en.wikipedia.org/wiki/Mandelbrot_set"

\end_inset

 taken from the 
\series bold
animation
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-animation"

\end_inset

; note the PNG device is used because PDF files are too large.
 You should be able to see the animation immediately with Acrobat Reader
 since it was set to play automatically:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<animate-demo, fig.show='animate', dev='png', out.width='.45
\backslash

\backslash
linewidth', interval=.5, aniopts='controls,loop,autoplay',  cache=TRUE>>=
\end_layout

\begin_layout Plain Layout

library(animation)
\end_layout

\begin_layout Plain Layout

demo('Mandelbrot', echo = FALSE, package = 'animation')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsubsection
Plot Size
\end_layout

\begin_layout Standard
The 
\family typewriter
fig.width
\family default
 and 
\family typewriter
fig.height
\family default
 options specify the size of plots in the graphics device, and the real
 size in the output document can be different (see 
\family typewriter
out.width
\family default
 and 
\family typewriter
out.height
\family default
).
 When there are multiple plots per chunk, it is possible to arrange more
 than one plot per line in LaTeX -- just specify 
\family typewriter
out.width
\family default
 to be less than half of the current line width, e.g.
 
\family typewriter
out.width='.49
\backslash

\backslash
linewidth'
\family default
.
\end_layout

\begin_layout Subsubsection
The tikz Device
\end_layout

\begin_layout Standard
Beside PDF, PNG and other traditional R graphical devices, 
\series bold
knitr
\series default
 has special support to tikz graphics via the 
\series bold
tikzDevice
\series default
 package 
\begin_inset CommandInset citation
LatexCommand citep
key "R-tikzDevice"

\end_inset

, which is similar to 
\series bold
pgfSweave
\series default
.
 If we set the chunk option 
\family typewriter
dev='tikz'
\family default
, the 
\emph on
tikz()
\emph default
 device in 
\series bold
tikzDevice
\series default
 will be used to save plots.
 Options 
\family typewriter
sanitize
\family default
 and 
\family typewriter
external
\family default
 are related to the tikz device: see the documentation of 
\emph on
tikz()
\emph default
 for details.
 Note 
\family typewriter
external=TRUE
\family default
 in 
\series bold
knitr
\series default
 has a different meaning with 
\series bold
pgfSweave
\series default
 -- it means 
\family typewriter
standAlone=TRUE
\family default
 in 
\emph on
tikz()
\emph default
, and the tikz graphics output will be compiled to PDF 
\emph on
immediately
\emph default
 after it is created, so the 
\begin_inset Quotes eld
\end_inset

externalization
\begin_inset Quotes erd
\end_inset

 does not depend on the 
\series bold
tikz
\series default
 package; to maintain consistency in (font) styles, 
\series bold
knitr
\series default
 will read the preamble of the input document and use it in the tikz device.
 At the moment, I'm not sure if this is a faithful way to externalize tikz
 graphics, but I have not seen any problems so far.
 The assumption to make, however, is that you declare all the styles in
 the preamble; 
\series bold
knitr
\series default
 is agnostic of 
\emph on
local
\emph default
 style changes in the body of the document.
\end_layout

\begin_layout Standard
Below is an example taken from StackOverflow
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://stackoverflow.com/q/8190087/559676
\end_layout

\end_inset


\end_layout

\end_inset

; we usually have to write R code like this to obtain a math expression
 
\begin_inset Formula $\mathrm{d}\mathbf{x}_{t}=\alpha[(\theta-\mathbf{x}_{t})\mathrm{d}t+4]\mathrm{d}B_{t}$
\end_inset

 in R graphics:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<math-expr-R, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

qplot(1:10, 1:10) + opts(title = substitute(paste(d * 
\end_layout

\begin_layout Plain Layout

    bolditalic(x)[italic(t)] == alpha * (theta - bolditalic(x)[italic(t)])
 * 
\end_layout

\begin_layout Plain Layout

    d * italic(t) + lambda * d * italic(B)[italic(t)]), list(lambda = 4)))
 
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
With the tikz device, it is both straightforward and more beautiful: 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<math-expr-tikz, dev='tikz', fig.width=5, fig.height=3, out.width='.55
\backslash

\backslash
linewidth', cache=TRUE, message=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(ggplot2)
\end_layout

\begin_layout Plain Layout

qplot(1:10, 1:10) + 
\end_layout

\begin_layout Plain Layout

labs(title = sprintf('$
\backslash

\backslash
mathrm{d}
\backslash

\backslash
mathbf{x}_{t} = 
\backslash

\backslash
alpha[(
\backslash

\backslash
theta - 
\backslash

\backslash
mathbf{x}_{t})
\backslash

\backslash
mathrm{d}t + %d]
\backslash

\backslash
mathrm{d}B_{t}$', 4))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The advantage of tikz graphics is the consistency of styles
\begin_inset Foot
status collapsed

\begin_layout Plain Layout
Users are encouraged to read the vignette of 
\series bold
tikzDevice
\series default
, which is the most beautiful vignette I have ever seen in R packages: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://cran.r-project.org/web/packages/tikzDevice/vignettes/tikzDevice.pdf
\end_layout

\end_inset


\end_layout

\end_inset

, and one disadvantage is that LaTeX may not be able to handle too large
 tikz files (it can run out of memory).
 For example, an R plot with tens of thousands of graphical elements may
 fail to compile in LaTeX if we use the tikz device.
 In such cases, we can switch to the PDF or PNG device, or reconsider our
 decision on the type of plots, e.g., a scatter plot with millions of points
 is usually difficult to read, and a contour plot or a hexagon plot showing
 the 2D density can be a better alternative (they are smaller in size).
\end_layout

\begin_layout Standard
The graphics manual contains more detailed information and you can check
 it out in the 
\begin_inset CommandInset href
LatexCommand href
name "website"
target "https://yihui.org/knitr/demo/graphics/"

\end_inset

.
\end_layout

\begin_layout Subsection
Cache
\end_layout

\begin_layout Standard
The feature of cache is not a new idea -- both 
\series bold
cacheSweave
\series default
 and 
\series bold
weaver
\series default
 have implemented it based on Sweave, with the former using 
\series bold
filehash
\series default
 and the latter using 
\family sans
.RData
\family default
 images; 
\series bold
cacheSweave
\series default
 also supports lazy-loading of objects based on 
\series bold
filehash
\series default
.
 The 
\series bold
knitr
\series default
 package directly uses internal base R functions to save (
\emph on
tools:::makeLazyLoadDB()
\emph default
) and lazy-load objects (
\emph on
lazyLoad()
\emph default
).
 These functions are either undocumented or marked as internal, but as far
 as I understand, they are the tools to implement lazy-loading for packages.
 The 
\series bold
cacheSweave
\series default
 vignette has clearly explained lazy-loading, and roughly speaking, lazy-loading
 means an object will not be really loaded into memory unless it is really
 used somewhere.
 This is very useful for cache; sometimes we read a large object and cache
 it, then take a subset for analysis and this subset is also cached; in
 the future, the initial large object will not be loaded into R if our computati
on is only based on the object of its subset.
\end_layout

\begin_layout Standard
The paths of cache files are determined by the chunk option 
\family typewriter
cache.path
\family default
; by default all cache files are created under a directory 
\family sans
cache
\family default
 relative to the current working directory, and if the option value contains
 a directory (e.g.
 
\family typewriter
cache.path='cache/abc-'
\family default
), cache files will be stored under that directory (automatically created
 if it does not exist).
 The cache is invalidated and purged on any changes to the code chunk, including
 both the R code and chunk options
\begin_inset Foot
status open

\begin_layout Plain Layout
One exception is the 
\family typewriter
include
\family default
 option, which is not cached because 
\family typewriter
include=TRUE/FALSE
\family default
 does not affect code evaluation; meanwhile, the value 
\family typewriter
getOption('width')
\family default
 is also cached, so if you change this option, the cache will also be invalidate
d (this option affects the width of text output)
\end_layout

\end_inset

; this means previous cache files of this chunk are removed (filenames are
 identified by the chunk label).
 Unlike 
\series bold
pgfSweave
\series default
, cache files will never accumulate since old cache files will always be
 removed in 
\series bold
knitr
\series default
.
 Unlike 
\series bold
weaver
\series default
 or 
\series bold
cacheSweave
\series default
, 
\series bold
knitr
\series default
 will try to preserve these side-effects:
\end_layout

\begin_layout Enumerate
printed results: meaning that any output of a code chunk will be loaded
 into the output document for a cached chunk, although it is not really
 evaluated.
 The reason is 
\series bold
knitr
\series default
 also cache the output of a chunk as a character string.
 Note this means graphics output is also cached since it is part of the
 output.
 It has been a pain for me for a long time to have to lose output to gain
 cache;
\end_layout

\begin_layout Enumerate
loaded packages: after the evaluation of each cached chunk, the list of
 packages used in the current R session is written to a file under the cache
 path named 
\family sans
__packages
\family default
; next time if a cached chunk needs to be rebuilt, these packages will be
 loaded first.
 The reasons for caching package names are, it can be slow to load some
 packages, and a package might be loaded in a previous cached chunk which
 is not available to the next cached chunk when only the latter needs to
 be rebuilt.
 Note this only applies to cached chunks, and for uncached chunks, you must
 always use 
\emph on
library()
\emph default
 to load packages explicitly;
\end_layout

\begin_layout Standard
Although 
\series bold
knitr
\series default
 tries to keep some side-effects, there are still other types of side-effects
 like setting 
\emph on
par()
\emph default
 or 
\emph on
options()
\emph default
 which are not cached.
 Users should be aware of these special cases, and make sure to clearly
 separate the code which is not meant to be cached to other chunks which
 are not cached, e.g., set all global options in the first chunk of a document
 and do not cache that chunk.
\end_layout

\begin_layout Standard
Sometimes a cached chunk may need to use objects from other cached chunks,
 which can bring a serious problem -- if objects in previous chunks have
 changed, this chunk will not be aware of the changes and will still use
 old cached results, unless there is a way to detect such changes from other
 chunks.
 There is an option called 
\family typewriter
dependson
\family default
 in 
\series bold
cacheSweave
\series default
 which does this job.
 We can explicitly specify which other chunks this chunk depends on by setting
 an option like 
\family typewriter
dependson='chunkA;chunkB'
\family default
 or equivalently 
\family typewriter
dependson=c('chunkA', 'chunkB')
\family default
.
 Each time the cache of a chunk is rebuilt, all other chunks which depend
 on this chunk will lose cache, hence their cache will be rebuilt as well.
\end_layout

\begin_layout Standard
Another way to specify the dependencies among chunks is to use the chunk
 option 
\family typewriter
autodep
\family default
 and the function 
\emph on
dep_auto()
\emph default
.
 This is an experimental feature borrowed from 
\series bold
weaver
\series default
 which frees us from setting chunk dependencies manually.
 The basic idea is, if a latter chunk uses any objects created from a previous
 chunk, the latter chunk is said to depend on the previous one.
 The function 
\emph on
findGlobals()
\emph default
 in the 
\series bold
codetools
\series default
 package is used to find out all global objects in a chunk, and according
 to its documentation, the result is an approximation.
 Global objects roughly mean the ones which are not created locally, e.g.
 in the expression 
\family typewriter
function() {y <- x}
\family default
, 
\family typewriter
x
\family default
 should be a global object, whereas 
\family typewriter
y
\family default
 is local.
 Meanwhile, we also need to save the list of objects created in each cached
 chunk, so that we can compare them to the global objects in latter chunks.
 For example, if chunk A created an object 
\family typewriter
x
\family default
 and chunk B uses this object, chunk B must depend on A, i.e.
 whenever A changes, B must also be updated.
 When 
\family typewriter
autodep=TRUE
\family default
, 
\series bold
knitr
\series default
 will write out the names of objects created in a cached chunk as well as
 those global objects in two files named 
\family sans
__objects
\family default
 and 
\family sans
__globals
\family default
 respectively; later we can use the function 
\emph on
dep_auto()
\emph default
 to analyze the object names to figure out the dependencies automatically.
 See 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/cache/
\end_layout

\end_inset

 for examples.
\end_layout

\begin_layout Standard
Yet another way to specify dependencies is 
\emph on
dep_prev()
\emph default
: this is a conservative approach which sets the dependencies so that a
 cached chunk will depend on all its previous chunks, i.e.
 whenever a previous chunk is updated, all later chunks will be updated
 accordingly.
\end_layout

\begin_layout Subsection
Code Externalization
\end_layout

\begin_layout Standard
It can be more convenient to write R code in a separate file, rather than
 mixing it into a LaTeX document; for example, we can run R code successively
 in a pure R script from one chunk to the other without jumping through
 other texts.
 Since I prefer using LyX to write reports, Sweave is even more inconvenient
 because I have to recompile the whole document each time, even if I only
 want to know the results of a single chunk.
 Therefore 
\series bold
knitr
\series default
 introduced the feature of code externalization to a separate R script.
 Currently the setting is like this: the R script also uses chunk labels
 (marked in the form 
\family typewriter
## ---- chunk-label
\family default
 by default); if the code chunk in the input document is empty, 
\series bold
knitr
\series default
 will match its label with the label in the R script to input external R
 code.
 For example, suppose this is a code chunk labelled as 
\family typewriter
Q1
\family default
 in an R script named 
\family sans
homework1-xie.R
\family default
 which is under the same directory as the Rnw document:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ext-r-code, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

## ---- Q1 ---------------------
\end_layout

\begin_layout Plain Layout

gcd = function(m, n) {
\end_layout

\begin_layout Plain Layout

    while ((r <- m %% n) != 0) {
\end_layout

\begin_layout Plain Layout

       m = n; n = r
\end_layout

\begin_layout Plain Layout

    }
\end_layout

\begin_layout Plain Layout

    n
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In the Rnw document, we can first read the script using the function 
\emph on
read_chunk()
\emph default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<read-chunk, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

read_chunk('homework1-xie.R')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This is usually done in an early chunk, and we can use the chunk 
\family typewriter
Q1
\family default
 later in the Rnw document:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<use-ext-chunk, echo=FALSE, comment=NA>>=
\end_layout

\begin_layout Plain Layout

cat('<<Q1, echo=TRUE, tidy=TRUE>>=','@',sep='
\backslash
n')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Different documents can read the same R script, so the R code can be reusable
 across different input documents.
\end_layout

\begin_layout Subsection
Evaluation of Chunk Options
\begin_inset CommandInset label
LatexCommand label
name "sub:conditional"

\end_inset


\end_layout

\begin_layout Standard
By default 
\series bold
knitr
\series default
 uses a new syntax to parse chunk options: it treats them as function arguments
 instead of a text string to be split to obtain option values.
 This gives the user much more power than the old syntax; we can pass arbitrary
 R objects to chunk options besides simple ones like 
\family typewriter
TRUE
\family default
/
\family typewriter
FALSE
\family default
, numbers and character strings.
 The page 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/sweave/
\end_layout

\end_inset

 has given two examples to show the advantages of the new syntax.
 Here we show yet another useful application.
\end_layout

\begin_layout Standard
Before 
\series bold
knitr
\series default
 0.3, there was a feature named 
\begin_inset Quotes eld
\end_inset

conditional evaluation
\begin_inset Quotes erd
\end_inset


\begin_inset Foot
status open

\begin_layout Plain Layout
request from 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://plus.google.com/u/0/116405544829727492615/posts/43WrRUffjzK
\end_layout

\end_inset


\end_layout

\end_inset

.
 The idea is, instead of setting chunk options 
\family typewriter
eval
\family default
 and 
\family typewriter
echo
\family default
 to be 
\family typewriter
TRUE
\family default
 or 
\family typewriter
FALSE
\family default
 (constants), their values can be controlled by global variables in the
 current R session.
 This enables 
\series bold
knitr
\series default
 to conditionally evaluate code chunks according to variables.
 For example, here we assign 
\family typewriter
TRUE
\family default
 to a variable 
\family typewriter
dothis
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cond-variable>>=
\end_layout

\begin_layout Plain Layout

dothis=TRUE
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In the next chunk, we set chunk options 
\family typewriter
eval=dothis
\family default
 and 
\family typewriter
echo=!dothis
\family default
, both are valid R expressions since the variable 
\family typewriter
dothis
\family default
 exists.
 As we can see, the source code is hidden, but it was indeed evaluated:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cond-out1, eval=dothis, echo=!dothis>>=
\end_layout

\begin_layout Plain Layout

print('you cannot see my source because !dothis is FALSE')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Then we set 
\family typewriter
eval=dothis
\family default
 and 
\family typewriter
echo=dothis
\family default
 for another chunk:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<cond-out2,eval=dothis,echo=dothis>>=
\end_layout

\begin_layout Plain Layout

dothis
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
If we change the value of 
\family typewriter
dothis
\family default
 to 
\family typewriter
FALSE
\family default
, neither of the above chunks will be evaluated any more.
 Therefore we can control many chunks with a single variable, and present
 results selectively.
\end_layout

\begin_layout Standard
This old feature requires 
\series bold
knitr
\series default
 to treat 
\family typewriter
eval
\family default
 and 
\family typewriter
echo
\family default
 specially, and we can easily see that it is no longer necessary with the
 new syntax: 
\family typewriter
eval=dothis
\family default
 will tell R to find the variable 
\family typewriter
dothis
\family default
 automatically just like we call a function 
\family typewriter
foobar(eval = dothis)
\family default
.
 What is more, all options will be evaluated as R expressions unless they
 are already constants which do not need to be evaluated, so this old feature
 has been generalized to all other options naturally.
\end_layout

\begin_layout Subsection
Customization
\end_layout

\begin_layout Standard
The 
\series bold
knitr
\series default
 package is ready for customization.
 Both the patterns and hooks can be customized; see the package website
 for details.
 Here I show an example on how to save 
\series bold
rgl
\series default
 plots 
\begin_inset CommandInset citation
LatexCommand citep
key "R-rgl"

\end_inset

 using a customized hook function.
 First we define a hook named 
\family typewriter
rgl
\family default
 using the function 
\emph on
hook_rgl()
\emph default
 in 
\series bold
rgl
\series default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<rgl-demo>>=
\end_layout

\begin_layout Plain Layout

library(rgl)
\end_layout

\begin_layout Plain Layout

knit_hooks$set(rgl = hook_rgl)
\end_layout

\begin_layout Plain Layout

head(hook_rgl)  # the hook function is defined as this
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Then we only have to set the chunk option 
\family typewriter
rgl=TRUE
\family default
:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fancy-rgl, rgl=TRUE, dev='png', fig.width=5, fig.height=5, out.width='2in',
 message=FALSE, warning=FALSE, cache=TRUE>>=
\end_layout

\begin_layout Plain Layout

library(rgl)
\end_layout

\begin_layout Plain Layout

demo('bivar', package='rgl', echo=FALSE) 
\end_layout

\begin_layout Plain Layout

par3d(zoom=.7)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Due to the flexibility of output hooks, 
\series bold
knitr
\series default
 supports several different output formats.
 The implementation is fairly easy, e.g., for LaTeX we put R output in 
\family typewriter
verbatim
\family default
 environments, and in HTML, it is only a matter of putting output in 
\family typewriter
div
\family default
 layers.
 These are simply character string operations.
 Many demos in 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demos
\end_layout

\end_inset

 show this idea clearly.
 This manual did not cover all the features of 
\series bold
knitr
\series default
, and users are encouraged to thumb through the website to know more possible
 features.
\end_layout

\begin_layout Section
Editors
\end_layout

\begin_layout Standard
You can use any text editors to write the source documents, but some have
 built-in support for 
\series bold
knitr
\series default
.
 Both RStudio (
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.rstudio.org
\end_layout

\end_inset

) and LyX (
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.lyx.org
\end_layout

\end_inset

) have full support for 
\series bold
knitr
\series default
, and you can compile the document to PDF with just one click.
 See 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/rstudio/
\end_layout

\end_inset

 and 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/demo/lyx/
\end_layout

\end_inset

 respectively.
 It is also possible to support other editors like 
\begin_inset CommandInset href
LatexCommand href
name "Eclipse"
target "https://yihui.org/knitr/demo/eclipse/"

\end_inset

, 
\begin_inset CommandInset href
LatexCommand href
name "Texmaker and WinEdt"
target "https://yihui.org/knitr/demo/editors/"

\end_inset

; see the demo list in the website for configuration instructions.
\end_layout

\begin_layout Section*
About This Document
\end_layout

\begin_layout Standard
This manual was written in LyX and compiled with 
\series bold
knitr
\series default
 (version 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
Sexpr{packageVersion('knitr')}
\end_layout

\end_inset

).
 The LyX source and the Rnw document exported from LyX can be found under
 these directories:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<source-location, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

system.file('examples', 'knitr-manual.lyx', package='knitr')  # lyx source
\end_layout

\begin_layout Plain Layout

system.file('examples', 'knitr-manual.Rnw', package='knitr')  # Rnw source
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
You can use the function 
\emph on
knit()
\emph default
 to knit the Rnw document (remember to put the two 
\family sans
.bib
\family default
 files under the same directory), and you need to make sure all the R packages
 used in this document are installed:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<required-packages, eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

install.packages(c('animation', 'rgl', 'tikzDevice', 'ggplot2'))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Feedback and comments on this manual and the package are always welcome.
 Bug reports and feature requests can be sent to 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/issues
\end_layout

\end_inset

, and questions can be delivered to the 
\begin_inset CommandInset href
LatexCommand href
name "mailing list"
target "knitr@googlegroups.com"
type "mailto:"

\end_inset

 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://groups.google.com/group/knitr
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% when knitr is updated, this chunk will be updated; why?
\end_layout

\begin_layout Plain Layout

<<auto-bib, version=packageVersion('knitr'), echo=FALSE, cache=TRUE, message=FAL
SE, warning=FALSE>>=
\end_layout

\begin_layout Plain Layout

# write all packages in the current session to a bib file
\end_layout

\begin_layout Plain Layout

write_bib(c(.packages(), 'evaluate', 'formatR', 'highr'), file = 'knitr-packages.b
ib')
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
bibfiles "knitr-manual,knitr-packages"
options "jss"

\end_inset


\end_layout

\end_body
\end_document

#LyX 2.0 created this file. For more info see http://www.lyx.org/
\lyxformat 413
\begin_document
\begin_header
\textclass article
\use_default_options true
\begin_modules
knitr
\end_modules
\maintain_unincluded_children false
\language english
\language_package none
\inputencoding default
\fontencoding global
\font_roman palatino
\font_sans default
\font_typewriter default
\font_default_family default
\use_non_tex_fonts false
\font_sc true
\font_osf false
\font_sf_scale 100
\font_tt_scale 100

\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered true
\pdf_bookmarksopen true
\pdf_bookmarksopenlevel 2
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\pdf_quoted_options "pdfstartview={XYZ null null 1}"
\papersize default
\use_geometry true
\use_amsmath 1
\use_esint 1
\use_mhchem 1
\use_mathdots 1
\cite_engine basic
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 2.5cm
\topmargin 2.5cm
\rightmargin 2.5cm
\bottommargin 2.5cm
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\listings_params "language=R"
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<setup, include=FALSE, cache=FALSE>>=
\end_layout

\begin_layout Plain Layout

library(knitr)
\end_layout

\begin_layout Plain Layout

opts_chunk$set(fig.path = 'figure/listings-')
\end_layout

\begin_layout Plain Layout

options(formatR.arrow = TRUE)
\end_layout

\begin_layout Plain Layout

render_listings()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Title
Using listings with knitr
\end_layout

\begin_layout Author
Yihui Xie
\end_layout

\begin_layout Standard
To use the 
\series bold
listings
\series default
 package with 
\series bold
knitr
\series default
, all you need to do is to call a function in your first setup chunk (that
 chunk should be hidden from the output with 
\family typewriter
include=FALSE
\family default
 and should not be cached):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% I just want to echo the 2nd line
\end_layout

\begin_layout Plain Layout

<<setup, echo=2>>=
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This function modifies the output hooks and header information so that the
 output is written in 
\series bold
listings
\series default
 environments, which are kindly provided by Frank Harrell and can be found
 in 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/blob/master/inst/misc/Sweavel.sty
\end_layout

\end_inset

.
 Of course you have to install the 
\series bold
listings
\series default
 package if your LaTeX toolset does not include this package.
\end_layout

\begin_layout Standard
Now we can see how the results look like with our new settings:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<boring-random>>=
\end_layout

\begin_layout Plain Layout

set.seed(1121)  # for reproducibility
\end_layout

\begin_layout Plain Layout

options(width=85)
\end_layout

\begin_layout Plain Layout

x=rnorm(20)
\end_layout

\begin_layout Plain Layout

x
\end_layout

\begin_layout Plain Layout

mean(x)
\end_layout

\begin_layout Plain Layout

sqrt(-1)  # this will give you a warning message
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Another chunk:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<rgl-hook-demo,eval=FALSE>>=
\end_layout

\begin_layout Plain Layout

hook_rgl = function(before, options, envir) {
\end_layout

\begin_layout Plain Layout

    ## after a chunk has been evaluated
\end_layout

\begin_layout Plain Layout

    if (before || !require('rgl') || rgl.cur() == 0) return()  # no active
 device
\end_layout

\begin_layout Plain Layout

    name = paste(valid_prefix(options$fig.path), options$label, sep = '')
\end_layout

\begin_layout Plain Layout

    par3d(windowRect = 100 + options$dpi * c(0, 0, options$width, options$height
))
\end_layout

\begin_layout Plain Layout

    Sys.sleep(.05) # need time to respond to window size change
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

    fmt = opts_knit$get('out.format')
\end_layout

\begin_layout Plain Layout

    if (fmt %in% c('html', 'markdown', 'gfm', 'jekyll')) options$dev = 'png'
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

    ## support 3 formats: eps, pdf and png (default)
\end_layout

\begin_layout Plain Layout

    switch(options$dev,
\end_layout

\begin_layout Plain Layout

           postscript = rgl.postscript(paste(name, '.eps', sep = ''), fmt
 = 'eps'),
\end_layout

\begin_layout Plain Layout

           pdf = rgl.postscript(paste(name, '.pdf', sep = ''), fmt = 'pdf'),
\end_layout

\begin_layout Plain Layout

           rgl.snapshot(paste(name, '.png', sep = ''), fmt = 'png'))
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

    if (fmt == 'html') return(.plot.hook.html(c(name, 'png'), options))
\end_layout

\begin_layout Plain Layout

    if (fmt %in% c('markdown', 'gfm', 'jekyll'))
\end_layout

\begin_layout Plain Layout

        return(.plot.hook.markdown(c(name, 'png'), options))
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

    paste(ifelse(options$align == 'center', '
\backslash

\backslash
centering{}', ''), '
\backslash

\backslash
includegraphics[',
\end_layout

\begin_layout Plain Layout

          sprintf('width=%s', options$out.width), ']{', name, '}
\backslash
n', sep = '')
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Well, we can do crazier things with 
\series bold
knitr
\series default
.
 Here we use the 
\family typewriter
fig.cap
\family default
 option to write plots into the 
\family typewriter
figure
\family default
 environment automatically, and the caption is generated from data dynamically
 (see Figure 
\begin_inset ERT
status collapsed

\begin_layout Plain Layout


\backslash
ref{fig:boxplot-ex}
\end_layout

\end_inset

):
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% set an option first
\end_layout

\begin_layout Plain Layout

<<eval-opts, echo=FALSE>>=
\end_layout

\begin_layout Plain Layout

opts_knit$set(eval.after = 'fig.cap') # evaluate fig.cap after the chunk
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

<<boxplot-ex, fig.height=2, out.width='.8
\backslash

\backslash
linewidth', fig.cap=sprintf('This is a boxplot; the median is %.2f.', median(x))>>=
\end_layout

\begin_layout Plain Layout

par(mar=c(4,4,.1,.1))
\end_layout

\begin_layout Plain Layout

boxplot(x, horizontal=TRUE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
As we know, 
\family typewriter
figure
\family default
 is a float environment, so it has floated away from the R code to the top
 of this page.
 This should not be surprising to LaTeX users.
\end_layout

\begin_layout Standard
You should be able to compile the TeX document and get a PDF file like this
 one: 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://github.com/yihui/knitr/releases/download/doc/knitr-listings.pdf
\end_layout

\end_inset

.
 For more information about out hooks, see 
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

https://yihui.org/knitr/hooks
\end_layout

\end_inset

.
\end_layout

\end_body
\end_document

%% LyX 2.2.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[10pt]{beamer}
\usepackage[T1]{fontenc}
\setcounter{secnumdepth}{3}
\setcounter{tocdepth}{3}
\usepackage{url}
\ifx\hypersetup\undefined
  \AtBeginDocument{%
    \hypersetup{unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 0},pdfborderstyle={},backref=false,colorlinks=false}
  }
\else
  \hypersetup{unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 0},pdfborderstyle={},backref=false,colorlinks=false}
\fi
\usepackage{breakurl}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
\providecommand{\LyX}{\texorpdfstring%
  {L\kern-.1667em\lower.25em\hbox{Y}\kern-.125emX\@}
  {LyX}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
 % this default might be overridden by plain title style
 \newcommand\makebeamertitle{\frame{\maketitle}}%
 % (ERT) argument for the TOC
 \AtBeginDocument{%
   \let\origtableofcontents=\tableofcontents
   \def\tableofcontents{\@ifnextchar[{\origtableofcontents}{\gobbletableofcontents}}
   \def\gobbletableofcontents#1{\origtableofcontents}
 }

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\usetheme{PaloAlto}

\makeatother

\begin{document}
<<setup, include=FALSE>>=
library(knitr)
opts_chunk$set(fig.path='figure/beamer-',fig.align='center',fig.show='hold',size='footnotesize')
@

\title[knitr, Beamer, and FragileFrame]{A Minimal Demo of knitr with Beamer and Fragile Frames}

\author{Yihui Xie\thanks{I thank Richard E. Goldberg for providing this demo.}}
\makebeamertitle
\begin{frame}{Background}

\begin{itemize}
\item The \textbf{knitr}\textbf{\emph{ }}package allows you to embed R code
and figures in \LaTeX{} documents

\begin{itemize}
\item It has functionality similar to Sweave but looks nicer and gives you
more control
\end{itemize}
\item If you already have Sweave working in \LyX{}, getting \textbf{knitr}
to work is trivial

\begin{enumerate}
\item Install the \textbf{knitr} package in \emph{R}
\item Read \url{https://yihui.org/knitr/demo/lyx/}
\end{enumerate}
\item If you use Sweave or \textbf{knitr} with Beamer in \LyX{}, you must
use the\emph{ FragileFrame} environment for the frames that contain
R code chunks. Let's see if \textbf{knitr} works with Beamer in this
small demo. 
\end{itemize}
\end{frame}

\section{First Test}
\begin{frame}[fragile]{First Test}

OK, let's get started with just some text:

<<echo=FALSE,results='hide'>>=
# some setup
options(width=60)  # make the printing fit on the page
set.seed(1121)   # make the results repeatable
@

<<>>= 
# create some random numbers
(x=rnorm(20))  
mean(x);var(x)  
@

BTW, the first element of \texttt{x} is \Sexpr{x[1]}. (Did you notice
the use of\texttt{ \textbackslash{}Sexpr\{\}}?)
\end{frame}

\section{Second Test}
\begin{frame}[fragile]{Second Test}

Text is nice but let's see what happens if we make a couple of plots
in our chunk:

<<boring-plots,fig.width=4,fig.height=4,out.width='.45\\linewidth'>>=
par(las=1,mar=c(4,4,.1,.1))  # tick labels direction
boxplot(x) 
hist(x,main='',col="blue",probability=TRUE) 
lines(density(x),col="red")
@
\end{frame}

\section{The Big Question}
\begin{frame}{The Big Question}

Do the above chunks work? You should be able to compile the \LyX{}
document and get a nice-looking PDF slide presentation. If not, time
to double-check everything...
\end{frame}

\end{document}

\documentclass{article}
\usepackage[sc]{mathpazo}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\setcounter{secnumdepth}{2}
\setcounter{tocdepth}{2}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\makeatother

\begin{document}

<<setup,include=FALSE,cache=FALSE>>=
options(width=90)
knitr::opts_chunk$set(out.width = '.6\\linewidth')
.knitr.title = if (exists('.knitr.title')) paste('\\title{', .knitr.title, '}', sep = '') else ''
.knitr.author = if (nzchar(.knitr.title) && exists('.knitr.author')) {
  paste('\\author{', .knitr.author, '%\n',
        '\\thanks{This report is automatically generated with the R package \\textbf{knitr}
        (version ', packageVersion('knitr'), ').}}', sep = '')
} else ''
@

\Sexpr{.knitr.title}

\Sexpr{.knitr.author}

\Sexpr{if (nzchar(.knitr.title)) '\\maketitle'}
The results below are generated from an R script.

<<%sCHUNK_LABEL_HERE>>=
@

The R session information (including the OS info, R version and all
packages used):

<<session-info, cache=FALSE>>=
sessionInfo()
Sys.time()
@
<<clean-up, include=FALSE>>=
rm(.knitr.author); rm(.knitr.author)
@

\end{document}

\section{Child can specify its parents}

<<setup, include=FALSE>>=
set_parent('knitr-main.Rnw')
@

My parent is \textsf{knitr-main.Rnw}.

<<test-e>>=
1+1
@

I'm not a complete LaTeX document, but you can compile me nonetheless. I'll
borrow the preamble from my parent.

%% LyX 2.2.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[twocolumn]{article}
\usepackage[sc]{mathpazo}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=3cm,bmargin=3cm,lmargin=3cm,rmargin=3cm}
\setcounter{secnumdepth}{2}
\setcounter{tocdepth}{2}
\setlength{\parskip}{\bigskipamount}
\setlength{\parindent}{0pt}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\makeatother

\begin{document}
<<setup, include=FALSE, cache=FALSE>>=
library(knitr)
opts_chunk$set(fig.path='figure/twocolumn-', fig.align='center', fig.show='hold')
render_listings()
@

\title{Using knitr in A Two-column Document}

\author{Yihui Xie}

\maketitle
We can use the \texttt{figure{*}} environment in \LaTeX{} so that
figures can span over two columns. Because the output hooks use the
\textbf{framed} package which does not work well with the \texttt{figure{*}}
environment, we can either modify the output hooks or use other packages
like \textbf{listings}. For example,

<<chunk-hook, echo=2, tidy=TRUE, tidy.opts=list(width.cutoff=30)>>=
<<setup>>
@

\begin{figure*}
<<boring-plots, fig.width=5, fig.height=5, out.width='.45\\linewidth'>>=
x=rnorm(100)
par(mar=c(4,4,.1,.1))
boxplot(x)
hist(x,main='')
@

\caption{Two plots spanning over two columns.}
\end{figure*}

The output PDF file will be like: \url{https://github.com/yihui/knitr/releases/download/doc/knitr-twocolumn.pdf}.
The Rnw source of this document is at \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-twocolumn.Rnw}.
\end{document}

%% LyX 2.2.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[sc]{mathpazo}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\setcounter{secnumdepth}{2}
\setcounter{tocdepth}{2}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}
\begin{document}
<<setup, include=FALSE, cache=FALSE>>=
library(knitr)
# set global chunk options
opts_chunk$set(fig.path='figure/minimal-', fig.align='center', fig.show='hold')
options(formatR.arrow=TRUE,width=90)
@

\title{A Minimal Demo of knitr}

\author{Yihui Xie}

\maketitle
You can test if \textbf{knitr} works with this minimal demo. OK, let's
get started with some boring random numbers:

<<boring-random>>=
set.seed(1121)
(x=rnorm(20))
mean(x);var(x)
@

The first element of \texttt{x} is \Sexpr{x[1]}. Boring boxplots
and histograms recorded by the PDF device:

<<boring-plots, fig.width=4, fig.height=4, out.width='.4\\linewidth'>>=
## two plots side by side (option fig.show='hold')
par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3,las=1)
boxplot(x)
hist(x,main='')
@

Do the above chunks work? You should be able to compile the \TeX{}
document and get a PDF file like this one: \url{https://github.com/yihui/knitr/releases/download/doc/knitr-minimal.pdf}.
The Rnw source of this document is at \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-minimal.Rnw}.
\end{document}

\documentclass{article}

\begin{document}

<<test-main>>=
options(width = 60)
summary(iris)
@

Let's see how to work with child documents in knitr. Below we input
the file \textsf{knitr-input-child.Rnw}:

<<child-demo, child='knitr-input-child.Rnw'>>=
@
% Or alternatively, use knit_child('knitr-input-child.Rnw') in \Sexpr

Done!

\end{document}

\documentclass{article}
\usepackage{mathpazo}
\begin{document}
This main document has several children.

<<test-main>>=
1+1
@
<<test-child, child=c('knitr-child-a.Rnw', 'knitr-child-b.Rnw')>>=
@

<<test-child-dir, child='sub/knitr-child-c.Rnw'>>=
@

\end{document}

%% LyX 2.2.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[nohyper,justified]{tufte-handout}
\usepackage[T1]{fontenc}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=true,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview=FitH}
\usepackage{breakurl}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.

\title{knitr Graphics Manual}
\author{Yihui Xie}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[buttonsize=1em]{animate}

\makeatother

\begin{document}
<<setup, include=FALSE, cache=FALSE>>=
library(knitr)
options(formatR.arrow=TRUE,width=50)
opts_chunk$set(fig.path='figure/graphics-', cache.path='cache/graphics-', fig.align='center', dev='tikz', fig.width=5, fig.height=5, fig.show='hold', cache=TRUE, par=TRUE)
knit_hooks$set(par=function(before, options, envir){
if (before && options$fig.show!='none') par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3)
}, crop=hook_pdfcrop)
@
\maketitle
\begin{abstract}
This manual shows features of graphics in the \textbf{knitr} package
(version \Sexpr{packageVersion('knitr')}) in detail, including the
graphical devices, plot recording, plot rearrangement, control of
plot sizes, the tikz device, figure captions, animations and other
types of plots such as \textbf{rgl} or GGobi plots.
\end{abstract}
Before reading this specific manual\footnote{\url{http://bit.ly/knitr-graphics-src} (Rnw source)},
you must have finished the main manual\footnote{\url{http://bit.ly/knitr-main}}.

\section{Graphical Devices}

The \textbf{knitr} package comes with more than 20 built-in graphical
devices, and you can specify them through the \texttt{dev} option.
This document uses the global option \texttt{dev='tikz'}, i.e., the
plots are recorded by the tikz device by default, but we can change
the device locally. Since tikz will be used extensively throughout
this manual and you will see plenty of tikz graphics later, now we
first show a few other devices.

<<test-plot, eval=FALSE>>=
with(trees, symbols(Height, Volume, circles = Girth/16, inches = FALSE, bg = 'deeppink', fg = 'gray30'))
@

\begin{marginfigure}
<<pdf-dev, ref.label='test-plot', dev='pdf', out.width='\\linewidth', echo=FALSE>>=
@

\caption{The default PDF device.\label{mar:pdf-dev}}
\end{marginfigure}

\begin{marginfigure}
<<png-dev, ref.label='test-plot', dev='png', out.width='\\linewidth', echo=FALSE>>=
@

\caption{The PNG device.\label{mar:png-dev}}
\end{marginfigure}

Figure \ref{mar:pdf-dev} and \ref{mar:png-dev} show two standard
devices in the \textbf{grDevices} package. We can also use devices
in the \textbf{Cairo} or \textbf{cairoDevice} package, e.g., the chunk
below uses the \emph{CairoPNG()} device in the \textbf{Cairo} package.

<<cairo-png-dev, ref.label='test-plot', dev='CairoPNG', out.width='.5\\linewidth', echo=FALSE>>=
@

\section{Plot Recording}

As mentioned in the main manual, \textbf{knitr} uses the \textbf{evaluate}
package to record plots. There are two sources of plots: first, whenever
\emph{plot.new()} or \emph{grid.newpage()} is called, \textbf{evaluate}
will try to save a snapshot of the current plot\footnote{For technical details, see \texttt{?setHook} and \texttt{?recordPlot}};
second, after each complete expression is evaluated, a snapshot is
also saved. To speed up recording, the null graphical device \texttt{pdf(file
= NULL)} is used. Figure \ref{fig:two-high} shows two expressions
producing two high-level plots.

\begin{figure}
<<two-high, fig.width=3, fig.height=2.5, out.width='.49\\linewidth'>>=
plot(cars)
boxplot(cars$dist,xlab='dist')
@

\caption{Two high-level plots are captured. The key to arrange two plots side
by side is to specify the \texttt{out.width} option so that each plot
takes less than half of the line width. We do not have to use the
\texttt{par(mfrow)} trick, and it may not work in some cases (e.g.
to put base graphics and \textbf{ggplot2} side by side; recall Figure
1 in the main manual).\label{fig:two-high}}
\end{figure}

Figure \ref{fig:low-plot-loop} shows another example of two R expressions,
but the second expression only involves with low-level plotting changes.
By default, low-level plot changes are discarded, but you can retain
them with the option \texttt{fig.keep='all'}.

\begin{figure}
<<low-plot-loop, fig.width=3, fig.height=2.5, out.width='.49\\linewidth', fig.keep='all'>>=
plot(0,0,type='n',ann=FALSE)
for(i in seq(0, 2*pi,length=20)) {points(cos(i),sin(i))}
@

\caption{Two complete R expressions will produce at most two plots, as long
as there are not multiple high-level plotting calls in each expression;
option \texttt{fig.keep='all'} here.\label{fig:low-plot-loop}}
\end{figure}

Together with \texttt{fig.show='asis'}, we can show the process of
plotting step by step like Figure \ref{fig:high-with-low}.

\begin{figure}
<<high-with-low, fig.width=3, fig.height=2.5, out.width='.5\\linewidth', fig.keep='all', fig.show='asis'>>=
plot(cars)
abline(lm(dist~speed, data=cars))  # a regression line
@

\caption{Low-level plot changes in base graphics can be recorded separately,
and plots can be put in the places where they were produced.\label{fig:high-with-low}}
\end{figure}

A further note on plot recording: \textbf{knitr} examines all recorded
plots (as R objects) and compares them sequentially; if the previous
plot is a ``subset'' of the next plot (= previous plot + low-level
changes), the previous plot will be removed when \texttt{fig.keep='high'}.
If two successive plots are identical, the second one will be removed
by default, so it might be a little bit surprising that the following
chunk will only produce one plot by default\footnote{adapted from \url{https://github.com/yihui/knitr/issues/41}}:

<<identical-plots,eval=FALSE>>=
m = matrix(1:100, ncol = 10)
image(m)
image(m*2)  # exactly the same as previous plot
@

\section{Plot Rearrangement}

We can rearrange the plots in chunks in several ways. They can be
inserted right after the line(s) of R code which produced them, or
accumulated till the end of the chunk. There is an example in the
main manual demonstrating \texttt{fig.show='asis'} for two high-level
plots, and Figure \ref{fig:high-with-low} in this manual also demonstrates
this option for a high-level plot followed by a low-level change.

Here is an example demonstrating the option \texttt{fig.keep='last'}
(only the last plot is kept):

\begin{figure}
<<fig-last, fig.width=5.2, fig.height=3, out.width='.7\\linewidth', fig.keep='last', message=FALSE>>=
library(ggplot2)
pie <- ggplot(diamonds, aes(x = factor(1), fill = cut)) + xlab('cut') + geom_bar(width = 1) 
pie + coord_polar(theta = "y") # a pie chart
pie + coord_polar() # the bullseye chart 
@

\caption{Two plots were produced in this chunk, but only the last one is kept.
This can be useful when we experiment with many plots, but only want
the last result. (Adapted from the \textbf{ggplot2} website)}
\end{figure}

When multiple plots are produced by a code chunk, we may want to show
them as an animation with the option \texttt{fig.show='animate'}.
Figure \ref{fig:clock-animation} shows a simple clock animation;
you may compare the code to Figure \ref{fig:high-with-low} to understand
that high-level plots are always recorded, regardless of where they
appeared.

\begin{figure}
<<clock-animation, fig.width=3, fig.height=3, out.width='.4\\linewidth', fig.show='animate', tidy=FALSE, crop=TRUE>>=
par(mar = rep(3, 4))
for (i in seq(pi/2, -4/3 * pi, length = 12)) {
    plot(0, 0, pch = 20, ann = FALSE, axes = FALSE)
    arrows(0, 0, cos(i), sin(i))
    axis(1, 0, "VI"); axis(2, 0, "IX")
    axis(3, 0, "XII"); axis(4, 0, "III"); box()
}
@

\caption{A clock animation. You have to view it in Adobe Reader: click to play/pause;
there are also buttons to speed up or slow down the animation.\label{fig:clock-animation}}
\end{figure}

We can also set the alignment of plots easily with the \texttt{fig.align}
option; this document uses \texttt{fig.align='center'} as a global
option, and we can also set plots to be left/right-aligned. Figure
\ref{fig:maruko-plot} is an example of a left-aligned plot.

\begin{figure}
<<maruko-plot, fig.width=7, fig.height=4, out.width='.45\\linewidth', fig.align='left', crop=TRUE>>=
stars(cbind(1:16,10*(16:1)),draw.segments=TRUE)
@

\caption{A left-aligned plot adapted from \texttt{?stars} (I call this the
``Maruko'' plot, and it is one of my favorites; see \protect\url{http://en.wikipedia.org/wiki/Chibi_Maruko-chan}).\label{fig:maruko-plot}}
\end{figure}


\section{Plot Size}

We have seen several examples in which two or more plots can be put
side by side, and this is because the plots were resized in the output
document; with the chunk option \texttt{out.width} less than half
of the line width, \LaTeX{} will arrange two plots in one line; if
it is less than $1/3$ of the line width, three plots can be put in
one line. Of course we can also set it to be an absolute width like
\texttt{3in} (3 inches). This option is used extensively in this document
to control the size of plots in the output document.

\section{The tikz Device}

The main advantage of using tikz graphics is the consistency of styles
between texts in plots and those in the main document. Since we can
use native \LaTeX{} commands in plots, the styles of texts in plots
can be very sophisticated (see Figure \ref{fig:math-formula-tt} for
an example).

\begin{figure}
<<math-formula-tt, fig.width=6, fig.height=2, out.width='\\linewidth'>>=
plot(0:1,0:1,type='n', ylab='origin of statistics', xlab='statistical presentation rocks with \\LaTeX{}')
text(.5,c(.8,.5,.2), c('\\texttt{lm(y \\textasciitilde{} x)}', '$\\hat{\\beta}=(X^{\\prime}X)^{-1}X^{\\prime}y$',  '$(\\Omega,\\mathcal{F},\\mu)$'))
@

\caption{A plot created by \textbf{tikzDevice} with math expressions and typewriter
fonts. Note the font style consistency \textendash{} we write the
same expressions in \protect\LaTeX{} here: $\hat{\beta}=(X^{\prime}X)^{-1}X^{\prime}y$
and $(\Omega,\mathcal{F},\mu)$; also \texttt{lm(y \textasciitilde{}
x)}.\label{fig:math-formula-tt}}
\end{figure}

When using Xe\LaTeX{} instead of PDF\LaTeX{} to compile the document,
we need to tell the \textbf{tikzDevice} package by setting the \texttt{tikzDefaultEngine}
option before all plot chunks (preferably in the first chunk):

<<xetex-tikz, eval=FALSE>>=
options(tikzDefaultEngine='xetex')
@

This is useful and often necessary to compile tikz plots which contain
(UTF8) multi-byte characters.

\section{Figure Caption}

If the chunk option \texttt{fig.cap} is not \texttt{NULL} or \texttt{NA},
the plots will be put in a \texttt{figure} environment when the output
format is \LaTeX{}, and this option is used to write a caption in
this environment using \texttt{\textbackslash{}caption\{\}}. The other
two related options are \texttt{fig.scap} and \texttt{fig.lp} which
set the short caption and a prefix string for the figure label. The
default short caption is extracted from the caption by truncating
it at the first period or colon or semi-colon. The label is a combination
of \texttt{fig.lp} and the chunk label. Because \texttt{figure} is
a float environment, it can float away from the chunk output to other
places such as the top or bottom of a page when the \TeX{} document
is compiled.

\section{Other Features}

The \textbf{knitr} package can be extended with hook functions, and
here I give a few examples illustrating the flexibility.

\subsection{Cropping PDF Graphics}

Some R users may have been suffering from the extra margins in R plots,
especially in base graphics (\textbf{ggplot2} is much better in this
aspect). The default graphical option \texttt{mar} is about \texttt{c(5,
4, 4, 2)} (see \texttt{?par}), which is often too big. Instead of
endlessly setting \texttt{par(mar)}, you may consider the program
\texttt{pdfcrop}, which can crop the white margin automatically\footnote{\url{http://www.ctan.org/pkg/pdfcrop}}.
In \textbf{knitr}, we can set up the hook \emph{hook\_pdfcrop()} to
work with a chunk option, say, \texttt{crop}.

<<crop-hook, cache=FALSE>>=
knit_hooks$set(crop=hook_pdfcrop)
@

Now we compare two plots below. The first one is not cropped (Figure
\ref{fig:pdf-nocrop}); then the same plot is produced but with a
chunk option \texttt{crop=TRUE} which will call the cropping hook
(Figure \ref{fig:pdf-crop}).

\begin{figure}
\begin{kframe}
<<pdf-nocrop, fig.width=3.2, fig.height=3.2, out.width='.5\\linewidth'>>=
par(mar=c(5,4,4,2),bg='white')  # large margin
plot(lat~long,data=quakes,pch=20,col=rgb(0,0,0,.2))
@
\end{kframe}

\caption{The original plot produced in R, with a large margin.\label{fig:pdf-nocrop}}
\end{figure}

\begin{figure}
\begin{kframe}
<<pdf-crop, ref.label='pdf-nocrop', echo=FALSE, fig.width=3.2, fig.height=3.2, out.width='.5\\linewidth', crop=TRUE>>=
@
\end{kframe}

\caption{The cropped plot; it fits better in the document.\label{fig:pdf-crop}}
\end{figure}

As we can see, the white margins are gone. If we use \emph{par()},
it might be hard and tedious to figure out a reasonable amount of
margin in order that neither is any label cropped nor do we get a
too large margin. My experience is that \texttt{pdfcrop} works well
with base graphics, but barely works with \textbf{grid} graphics (therefore
\textbf{lattice} and \textbf{ggplot2} are ruled out).

\subsection{Manually Saved Plots}

We have explained how R plots are recorded before. In some cases,
it is not possible to capture plots by \emph{recordPlot()} (such as
\textbf{rgl} plots), but we can save them using other functions. To
insert these plots into the output, we need to set up a hook first
like this (see \texttt{?hook\_plot\_custom} for details):

<<custom-plot-hook, cache=FALSE>>=
knit_hooks$set(custom.plot = hook_plot_custom)
@

Then we set the chunk option \texttt{custom.plot=TRUE}, and manually
write plot files in the chunk. Here we show an example of capturing
GGobi plots using the function \emph{ggobi\_display\_save\_picture()}
in the \textbf{rggobi} package:

<<ggobi-plot, custom.plot=TRUE, fig.ext='png', out.width='2.5in', results='hide', warning=FALSE>>=
library(rggobi)
ggobi(ggobi_find_file('data', 'flea.csv'))
Sys.sleep(1)  # wait for snapshot
ggobi_display_save_picture(path=fig_path('.png'))
@

One thing to note here is we have to make sure the plot filename is
from \emph{fig\_path()}, which is a convenience function to return
the figure path for the current chunk.

We can do whatever normal R plots can do with this hook, and we give
another example below to show how to work with animations.

<<rgl-animation, custom.plot=TRUE, fig.ext='png', out.width='2.5in', fig.show='animate', fig.num=20, interval=.4,  aniopts='controls,autoplay', results='hide', warning=FALSE>>=
library(animation) # adapted from demo('rgl_animation')
data(pollen)
uM = matrix(c(-0.37, -0.51, -0.77, 0, -0.73, 0.67, -0.1, 0, 0.57, 0.53, -0.63, 0, 0, 0, 0, 1), 4, 4)
library(rgl)
open3d(userMatrix = uM, windowRect = c(0, 0, 400, 400))
plot3d(pollen[, 1:3])
zm = seq(1, 0.05, length = 20)
par3d(zoom = 1)  # change the zoom factor gradually later
for (i in 1:length(zm)) {
    par3d(zoom = zm[i]); Sys.sleep(.05)
    rgl.snapshot(fig_path('png', number = i))
}
@

\subsection{rgl Plots}

With the hook \emph{hook\_rgl()}, we can easily save snapshots from
the \textbf{rgl} package. We have shown an example in the main manual,
and here we add some details. The rgl hook is a good example of taking
care of details by carefully using the \texttt{options} argument in
the hook; for example, we cannot directly set the width and height
of rgl plots in \emph{rgl.snapshot()} or \emph{rgl.postscript()},
so we make use of the options \texttt{fig.width}, \texttt{fig.height}
and \texttt{dpi} to calculate the expected size of the window, then
resize the current window by \emph{par3d()}, and finally save the
plot.

This hook is actually built upon \emph{hook\_plot\_custom()} \textendash{}
first it saves the \textbf{rgl} snapshot, then it calls \emph{hook\_plot\_custom()}
to write the output code.

\appendix

\section{How to Compile This Manual}

This manual has a long chain of dependencies, so it may not be easy
to compile. These packages are required (all of them are free software):
\begin{description}
\item [{R}] Cairo, ggplot2, tikzDevice, rgl, rggobi, animation (all available
on CRAN except tikzDevice which is on R-Forge for the time being)
\item [{\LaTeX{}}] animate, hyperref and the tufte-handout class
\item [{Other}] GGobi, pdfcrop
\end{description}

\end{document}

%% LyX 2.2.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[sc]{mathpazo}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}
\usepackage{listings}
\lstset{language=R}
\begin{document}
<<setup, include=FALSE, cache=FALSE>>=
library(knitr)
opts_chunk$set(fig.path = 'figure/listings-')
options(formatR.arrow = TRUE)
render_listings()
@

\title{Using listings with knitr}

\author{Yihui Xie}

\maketitle
To use the \textbf{listings} package with \textbf{knitr}, all you
need to do is to call a function in your first setup chunk (that chunk
should be hidden from the output with \texttt{include=FALSE} and should
not be cached):

% I just want to echo the 2nd line
<<setup, echo=2>>=
@

This function modifies the output hooks and header information so
that the output is written in \textbf{listings} environments, which
are kindly provided by Frank Harrell and can be found in \url{https://github.com/yihui/knitr/blob/master/inst/misc/Sweavel.sty}.
Of course you have to install the \textbf{listings} package if your
\LaTeX{} toolset does not include this package.

Now we can see how the results look like with our new settings:

<<boring-random>>=
set.seed(1121)  # for reproducibility
options(width=85)
x=rnorm(20)
x
mean(x)
sqrt(-1)  # this will give you a warning message
@

Another chunk:

<<rgl-hook-demo,eval=FALSE>>=
hook_rgl = function(before, options, envir) {
    ## after a chunk has been evaluated
    if (before || !require('rgl') || rgl.cur() == 0) return()  # no active device
    name = paste(valid_prefix(options$fig.path), options$label, sep = '')
    par3d(windowRect = 100 + options$dpi * c(0, 0, options$width, options$height))
    Sys.sleep(.05) # need time to respond to window size change

    fmt = opts_knit$get('out.format')
    if (fmt %in% c('html', 'markdown', 'gfm', 'jekyll')) options$dev = 'png'

    ## support 3 formats: eps, pdf and png (default)
    switch(options$dev,
           postscript = rgl.postscript(paste(name, '.eps', sep = ''), fmt = 'eps'),
           pdf = rgl.postscript(paste(name, '.pdf', sep = ''), fmt = 'pdf'),
           rgl.snapshot(paste(name, '.png', sep = ''), fmt = 'png'))

    if (fmt == 'html') return(.plot.hook.html(c(name, 'png'), options))
    if (fmt %in% c('markdown', 'gfm', 'jekyll'))
        return(.plot.hook.markdown(c(name, 'png'), options))

    paste(ifelse(options$align == 'center', '\\centering{}', ''), '\\includegraphics[',
          sprintf('width=%s', options$out.width), ']{', name, '}\n', sep = '')
}
@

Well, we can do crazier things with \textbf{knitr}. Here we use the
\texttt{fig.cap} option to write plots into the \texttt{figure} environment
automatically, and the caption is generated from data dynamically
(see Figure \ref{fig:boxplot-ex}):

% set an option first
<<eval-opts, echo=FALSE>>=
opts_knit$set(eval.after = 'fig.cap') # evaluate fig.cap after the chunk

<<boxplot-ex, fig.height=2, out.width='.8\\linewidth', fig.cap=sprintf('This is a boxplot; the median is %.2f.', median(x))>>=
par(mar=c(4,4,.1,.1))
boxplot(x, horizontal=TRUE)
@

As we know, \texttt{figure} is a float environment, so it has floated
away from the R code to the top of this page. This should not be surprising
to \LaTeX{} users.

You should be able to compile the \TeX{} document and get a PDF file
like this one: \url{https://github.com/yihui/knitr/releases/download/doc/knitr-listings.pdf}.
For more information about out hooks, see \url{https://yihui.org/knitr/hooks/}.
\end{document}

%% LyX 2.2.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[sc]{mathpazo}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\setcounter{secnumdepth}{2}
\setcounter{tocdepth}{2}
\usepackage{url}
\usepackage[authoryear]{natbib}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
\providecommand{\LyX}{\texorpdfstring%
  {L\kern-.1667em\lower.25em\hbox{Y}\kern-.125emX\@}
  {LyX}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[buttonsize=1em]{animate}

\makeatother

\begin{document}
<<setup, include=FALSE, cache=FALSE>>=
library(knitr)
## set global chunk options
opts_chunk$set(fig.path='figure/manual-', cache.path='cache/manual-', fig.align='center', fig.show='hold', par=TRUE)
## I use = but I can replace it with <-; set code/output width to be 68
options(formatR.arrow=TRUE, width=68, digits=4)
## tune details of base graphics (https://yihui.org/knitr/hooks/)
knit_hooks$set(par=function(before, options, envir){
if (before && options$fig.show!='none') par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3)
})
@

\title{knitr: A General-Purpose Tool for Dynamic Report Generation in R}

\author{Yihui Xie}

\maketitle
The original paradigm of literate programming was brought forward
mainly for software development, or specifically, to mix source code
(for computer) and documentation (for human) together. Early systems
include \href{http://www.literateprogramming.com/web.pdf}{WEB} and
\href{http://www.cs.tufts.edu/~nr/noweb/}{Noweb}; Sweave \citep{leisch2002}
was derived from the latter, but it is less focused on documenting
software, instead it is mainly used for reproducible data analysis
and generating statistical reports. The \textbf{knitr} package \citep{R-knitr}
is following the steps of Sweave. For this manual, I assume readers
have some background knowledge of Sweave to understand the technical
details; for a reference of available options, hooks and demos, see
the package homepage \url{https://yihui.org/knitr/}.

\section{Hello World}

A natural question is why to reinvent the wheel. The short answer
is that extending Sweave by hacking \textsf{SweaveDrivers.R} in the
\textbf{utils} package is a difficult job to me. Many features in
\textbf{knitr} come naturally as users would have expected. Figure
\ref{fig:cars-demo} is a simple demo of some features of \textbf{knitr}.

\begin{figure}
<<cars-demo,dev='tikz',fig.width=4,fig.height=2.8,out.width='.45\\textwidth',message=FALSE,cache=TRUE>>=
fit=lm(dist~speed,data=cars) # linear regression
par(mar=c(4, 4, 1, .1), mgp=c(2,1,0))
with(cars,plot(speed,dist,panel.last=abline(fit)))
text(10,100,'$Y = \\beta_0 + \\beta_1x + \\epsilon$')
library(ggplot2)
qplot(speed, dist, data=cars)+geom_smooth()
@

\caption{\label{fig:cars-demo}A simple demo of possible output in \textbf{knitr}:
(1) multiple plots per chunk; (2) no need to \emph{print()} objects
in \textbf{ggplot2}; (3) device size is $4\times2.8$ (inches) but
output size is adjusted to \texttt{.45\textbackslash{}textwidth} in
chunk options; (4) base graphics and \textbf{ggplot2} can sit side
by side; (5) use the \emph{tikz()} device in \textbf{tikzDevice} by
setting chunk option \texttt{dev='tikz'} (hence can write native \protect\LaTeX{}
expressions in R plots); (6) code highlighting.}
\end{figure}

I would have chosen to hide the R code if this were a real report,
but here I show the code just for the sake of demonstration. If we
type \emph{qplot()} in R, we get a plot, and the same thing happens
in \textbf{knitr}. If we draw two plots in the code, \textbf{knitr}
will show two plots and we do not need to tell it how many plots are
there in the code in advance. If we set \texttt{out.width='.49\textbackslash{}\textbackslash{}textwidth'}
in chunk options, we get it in the final output document. If we say
\texttt{fig.align='center'}, the plots are centered. That's it. Many
enhancements and new features will be introduced later. If you come
from the Sweave land, you can take a look at the page of transition
first: \url{https://yihui.org/knitr/demo/sweave/}.

\section{Design}

The flow of processing an input file is similar to Sweave, and two
major differences are that \textbf{knitr} provides more flexibility
to the users to customize the processing, and has many built-in options
such as the support to a wide range of graphics devices and cache.
Below is a brief description of the process:
\begin{enumerate}
\item \textbf{knitr} takes an input file and automatically determines an
appropriate set of \href{https://yihui.org/knitr/patterns/}{patterns}
to use if they are not provided in advance (e.g. \textsf{file.Rnw}
will use \texttt{knit\_patterns\$get('rnw')});
\item a set of output \href{https://yihui.org/knitr/hooks/}{hooks} will
also be set up automatically according to the filename extension (e.g.
use \LaTeX{} environments or HTML elements to wrap up R results);
\item the input file is read in and split into pieces consisting of R code
chunks and normal texts; the former will be executed one after the
other, and the latter may contain global chunk options or inline R
code;
\item for each chunk, the code is evaluated using the \textbf{evaluate}
package \citep{R-evaluate}, and the results may be filtered according
to chunk options (e.g. \texttt{echo=FALSE} will remove the R source
code)

\begin{enumerate}
\item if \texttt{cache=TRUE} for this chunk, \textbf{knitr} will first check
if there are previously cached results under the cache directory before
really evaluating the chunk; if cached results exist and this code
chunk has not been changed since last run (use MD5 sum to verify),
the cached results will be (lazy-) loaded, otherwise new cache will
be built; if a cached chunk depends on other chunks (see the \texttt{dependson}
\href{https://yihui.org/knitr/options/}{option}) and any one of these
chunks has changed, this chunk must be forcibly updated (old cache
will be purged)
\item there are six types of possible output from \textbf{evaluate}, and
their classes are \texttt{character} (normal text output), \texttt{source}
(source code), \texttt{warning}, \texttt{message}, \texttt{error}
and \texttt{recordedplot}; an internal S3 generic function \emph{wrap()}
is used to deal with different types of output, using output hooks
defined in the object \texttt{knit\_hooks}
\item note plots are recorded as R objects before they are really saved
to files, so graphics devices will not be opened unless plots have
really been produced in a chunk
\item a code chunk is evaluated in a separate empty environment with the
global environment as its parent, and all the objects in this environment
after the evaluation will be saved if \texttt{cache=TRUE}
\item chunk hooks can be run before and/or after a chunk
\end{enumerate}
\item for normal texts, \textbf{knitr} will find inline R code (e.g. in
\texttt{\textbackslash{}Sexpr\{\}}) and evaluate it; the output is
wrapped by the \texttt{inline} hook;
\end{enumerate}
The hooks play important roles in \textbf{knitr}: this package makes
almost everything accessible to the users. Consider the following
extremely simple example which may demonstrate this freedom:

<<simple-example>>=
1+1
@

There are two parts in the final output: the source code \texttt{1
+ 1} and the output \texttt{{[}1{]} 2}; the comment characters \texttt{\#\#}
are from the default chunk option \texttt{comment}. Users may define
a hook function for the source code like this to use the \texttt{lstlisting}
environment:

<<hook-source, eval=FALSE>>=
knit_hooks$set(source = function(x, options) {
paste('\\begin{lstlisting}\n', x, '\\end{lstlisting}\n', sep = '')
})
@

Similarly we can put other types of output into other environments.
There is no need to hack at \textsf{Sweave.sty} for \textbf{knitr}
and you can put the output in any environments. What is more, the
output hooks make \textbf{knitr} ready for other types of output,
and a typical one is HTML (there are built-in hooks). The website
has provided many examples demonstrating the flexibility of the output.

\section{Features}

The \textbf{knitr} package borrowed features such as tikz graphics
and cache from \textbf{pgfSweave} and \textbf{cacheSweave} respectively,
but the implementations are different. New features like code reference
from an external R script as well as output customization are also
introduced. The feature of hook functions in Sweave is re-implemented
and hooks have new usage now. There are several other small features
which are motivated from my everyday use of Sweave. For example, a
progress bar is provided when knitting a file so we roughly know how
long we still need to wait; output from inline R code (e.g. \texttt{\textbackslash{}Sexpr\{x{[}1{]}\}})
is automatically formatted in \TeX{} math notation (like \Sexpr{123456789})
if the result is numeric. You may check out a number of specific manuals
dedicated to specific features such as graphics in the website: \url{https://yihui.org/knitr/demo/}.

\subsection{Code Decoration}

The \textbf{highr} package \citep{R-highr} is used to highlight R
code, and the \textbf{formatR} package \citep{R-formatR} is used
to reformat R code (like \texttt{keep.source=FALSE} in Sweave but
will also try to retain comments). For \LaTeX{} output, the \textbf{framed}
package is used to decorate code chunks with a light gray background.
If this \LaTeX{} package is not found in the system, a version will
be copied directly from \textbf{knitr}. The prompt characters are
removed by default because they mangle the R source code in the output
and make it difficult to copy R code. The R output is masked in comments
by default based on the same rationale. It is easy to revert to the
output with prompts (set option \texttt{prompt=TRUE}), and you will
quickly realize the inconvenience to the readers if they want to copy
and run the code in the output document:

<<stupid-prompts, prompt=TRUE, comment=NA, highlight=FALSE>>=
x=rnorm(5)
x
var(x)
@

The example below shows the effect of \texttt{tidy=TRUE/FALSE}:

<<tidy-no, eval=FALSE, tidy=FALSE>>=
## option tidy=FALSE
for(k in 1:10){j=cos(sin(k)*k^2)+3;print(j-5)}
@
<<tidy-yes, eval=FALSE, tidy=TRUE>>=
## option tidy=TRUE
for(k in 1:10){j=cos(sin(k)*k^2)+3;print(j-5)}
@

Note \texttt{=} is replaced by \texttt{<-} because \texttt{options('formatR.arrow')}
was set to be \texttt{TRUE} in this document; see the documentation
of \emph{tidy.source()} in \textbf{formatR} for details.

Many highlighting themes can be used in \textbf{knitr}, which are
borrowed from the \textbf{highlight} package by \href{http://www.andre-simon.de/}{Andre Simon}\footnote{not the R package mentioned before; for a preview of these themes,
see \url{http://www.andre-simon.de/dokuwiki/doku.php?id=theme_examples}}; it is also possible to use themes from \url{http://www.eclipsecolorthemes.org/}
by providing a theme id to \textbf{knitr}\footnote{many thanks to \href{https://github.com/ramnathv}{Ramnath Vaidyanathan}
for the work on themes}. See \texttt{?knit\_theme} for details.

\subsection{Graphics}

Graphics is an important part of reports, and several enhancements
have been made in \textbf{knitr}. For example, grid graphics may not
need to be explicitly printed as long as the same code can produce
plots in R (in some cases, however, they have to be printed, e.g.
in a loop, because you have to do so in an R terminal).

\subsubsection{Graphical Devices}

Over a long time, a frequently requested feature for Sweave was the
support for other graphics devices, which has been implemented since
R 2.13.0. Instead of using logical options like \texttt{png} or \texttt{jpeg}
(this list can go on and on), \textbf{knitr} uses a single option
\texttt{dev} (like \texttt{grdevice} in Sweave) which has support
for more than 20 devices. For instance, \texttt{dev='png'} will use
the \emph{png()} device, and \texttt{dev='CairoJPEG'} uses the \emph{CairoJPEG()}
device in the \textbf{Cairo} package (it has to be installed first,
of course). If none of these devices is satisfactory, you can provide
the name of a customized device function, which must have been defined
before it is called.

\subsubsection{Plot Recording}

As mentioned before, all the plots in a code chunk are first recorded
as R objects and then ``replayed'' inside a graphical device to
generate plot files. The \textbf{evaluate} package will record plots
per \emph{expression} basis, in other words, the source code is split
into individual complete expressions and \textbf{evaluate} will examine
possible plot changes in snapshots after each single expression has
been evaluated. For example, the code below consists of three expressions,
out of which two are related to drawing plots, therefore \textbf{evaluate}
will produce two plots by default:

<<low-level-plots, fig.keep='all', dev='tikz', fig.width=2.5, fig.height=2.5, out.width='.3\\textwidth', cache=TRUE>>=
par(mar=c(3,3,.1,.1))
plot(1:10, ann=FALSE,las=1)
text(5,9,'mass $\\rightarrow$ energy\n$E=mc^2$')
@

This brings a significant difference with traditional tools in R for
dynamic report generation, since low-level plotting changes can also
be recorded. The option \texttt{fig.keep} controls which plots to
keep in the output; \texttt{fig.keep='all'} will keep low-level changes
as separate plots; by default (\texttt{fig.keep='high'}), \textbf{knitr}
will merge low-level plot changes into the previous high-level plot,
like most graphics devices do. This feature may be useful for teaching
R graphics step by step. Note, however, low-level plotting commands
in a single expression (a typical case is a loop) will not be recorded
accumulatively, but high-level plotting commands, regardless of where
they are, will always be recorded. For example, this chunk will only
produce 2 plots instead of 21 plots because there are 2 complete expressions:

<<low-plot-loop, eval=FALSE>>=
plot(0,0,type='n',ann=FALSE)
for(i in seq(0, 2*pi,length=20)) points(cos(i),sin(i))
@

But this will produce 20 plots as expected:

<<high-plot-loop, eval=FALSE>>=
for(i in seq(0, 2*pi,length=20)) {plot(cos(i),sin(i),xlim=c(-1,1),ylim=c(-1,1))}
@

As I showed in the beginning of this manual, it is straightforward
to let \textbf{knitr} keep all the plots in a chunk and insert them
into the output document, so we no longer need the \texttt{cat('\textbackslash{}\textbackslash{}includegraphics\{\}')}
trick.

We can discard all previous plots and keep the last one only by \texttt{fig.keep='last'},
or keep only the first plot by \texttt{fig.keep='first'}, or discard
all plots by \texttt{fig.keep='none'}.

\subsubsection{Plot Rearrangement}

The option \texttt{fig.show} can decide whether to hold all plots
while evaluating the code and ``flush'' all of them to the end of
a chunk (\texttt{fig.show='hold'}), or just insert them to the place
where they were created (by default \texttt{fig.show='asis'}). Here
is an example of \texttt{fig.show='asis'}:

<<fig-hold, fig.show='asis', dev='pdf', fig.width=6, fig.height=4, out.width='.35\\linewidth'>>=
contour(volcano)  # contour lines
filled.contour(volcano)  # fill contour plot with colors
@

Beside \texttt{hold} and \texttt{asis}, the option \texttt{fig.show}
can take a third value: \texttt{animate}, which makes it possible
to insert animations into the output document. In \LaTeX{}, the package
\textbf{animate} is used to put together image frames as an animation.
For animations to work, there must be more than one plot produced
in a chunk. The option \texttt{interval} controls the time interval
between animation frames; by default it is 1 second. Note you have
to add \texttt{\textbackslash{}usepackage\{animate\}} in the \LaTeX{}
preamble, because \textbf{knitr} will not add it automatically. Animations
in the PDF output can only be viewed in Adobe Reader.

As a simple demonstration, here is a \href{http://en.wikipedia.org/wiki/Mandelbrot_set}{Mandelbrot animation}
taken from the \textbf{animation} package \citep{R-animation}; note
the PNG device is used because PDF files are too large. You should
be able to see the animation immediately with Acrobat Reader since
it was set to play automatically:

<<animate-demo, fig.show='animate', dev='png', out.width='.45\\linewidth', interval=.5, aniopts='controls,loop,autoplay',  cache=TRUE>>=
library(animation)
demo('Mandelbrot', echo = FALSE, package = 'animation')
@

\subsubsection{Plot Size}

The \texttt{fig.width} and \texttt{fig.height} options specify the
size of plots in the graphics device, and the real size in the output
document can be different (see \texttt{out.width} and \texttt{out.height}).
When there are multiple plots per chunk, it is possible to arrange
more than one plot per line in \LaTeX{} \textendash{} just specify
\texttt{out.width} to be less than half of the current line width,
e.g. \texttt{out.width='.49\textbackslash{}\textbackslash{}linewidth'}.

\subsubsection{The tikz Device}

Beside PDF, PNG and other traditional R graphical devices, \textbf{knitr}
has special support to tikz graphics via the \textbf{tikzDevice} package
\citep{R-tikzDevice}, which is similar to \textbf{pgfSweave}. If
we set the chunk option \texttt{dev='tikz'}, the \emph{tikz()} device
in \textbf{tikzDevice} will be used to save plots. Options \texttt{sanitize}
and \texttt{external} are related to the tikz device: see the documentation
of \emph{tikz()} for details. Note \texttt{external=TRUE} in \textbf{knitr}
has a different meaning with \textbf{pgfSweave} \textendash{} it means
\texttt{standAlone=TRUE} in \emph{tikz()}, and the tikz graphics output
will be compiled to PDF \emph{immediately} after it is created, so
the ``externalization'' does not depend on the \textbf{tikz} package;
to maintain consistency in (font) styles, \textbf{knitr} will read
the preamble of the input document and use it in the tikz device.
At the moment, I'm not sure if this is a faithful way to externalize
tikz graphics, but I have not seen any problems so far. The assumption
to make, however, is that you declare all the styles in the preamble;
\textbf{knitr} is agnostic of \emph{local} style changes in the body
of the document.

Below is an example taken from StackOverflow\footnote{\url{http://stackoverflow.com/q/8190087/559676}};
we usually have to write R code like this to obtain a math expression
$\mathrm{d}\mathbf{x}_{t}=\alpha[(\theta-\mathbf{x}_{t})\mathrm{d}t+4]\mathrm{d}B_{t}$
in R graphics:

<<math-expr-R, eval=FALSE>>=
qplot(1:10, 1:10) + opts(title = substitute(paste(d *
    bolditalic(x)[italic(t)] == alpha * (theta - bolditalic(x)[italic(t)]) *
    d * italic(t) + lambda * d * italic(B)[italic(t)]), list(lambda = 4)))
@

With the tikz device, it is both straightforward and more beautiful:

<<math-expr-tikz, dev='tikz', fig.width=5, fig.height=3, out.width='.55\\linewidth', cache=TRUE, message=FALSE>>=
library(ggplot2)
qplot(1:10, 1:10) +
labs(title = sprintf('$\\mathrm{d}\\mathbf{x}_{t} = \\alpha[(\\theta - \\mathbf{x}_{t})\\mathrm{d}t + %d]\\mathrm{d}B_{t}$', 4))
@

The advantage of tikz graphics is the consistency of styles\footnote{Users are encouraged to read the vignette of \textbf{tikzDevice},
which is the most beautiful vignette I have ever seen in R packages:
\url{http://cran.r-project.org/web/packages/tikzDevice/vignettes/tikzDevice.pdf}}, and one disadvantage is that \LaTeX{} may not be able to handle
too large tikz files (it can run out of memory). For example, an R
plot with tens of thousands of graphical elements may fail to compile
in \LaTeX{} if we use the tikz device. In such cases, we can switch
to the PDF or PNG device, or reconsider our decision on the type of
plots, e.g., a scatter plot with millions of points is usually difficult
to read, and a contour plot or a hexagon plot showing the 2D density
can be a better alternative (they are smaller in size).

The graphics manual contains more detailed information and you can
check it out in the \href{https://yihui.org/knitr/demo/graphics/}{website}.

\subsection{Cache}

The feature of cache is not a new idea \textendash{} both \textbf{cacheSweave}
and \textbf{weaver} have implemented it based on Sweave, with the
former using \textbf{filehash} and the latter using \textsf{.RData}
images; \textbf{cacheSweave} also supports lazy-loading of objects
based on \textbf{filehash}. The \textbf{knitr} package directly uses
internal base R functions to save (\emph{tools:::makeLazyLoadDB()})
and lazy-load objects (\emph{lazyLoad()}). These functions are either
undocumented or marked as internal, but as far as I understand, they
are the tools to implement lazy-loading for packages. The \textbf{cacheSweave}
vignette has clearly explained lazy-loading, and roughly speaking,
lazy-loading means an object will not be really loaded into memory
unless it is really used somewhere. This is very useful for cache;
sometimes we read a large object and cache it, then take a subset
for analysis and this subset is also cached; in the future, the initial
large object will not be loaded into R if our computation is only
based on the object of its subset.

The paths of cache files are determined by the chunk option \texttt{cache.path};
by default all cache files are created under a directory \textsf{cache}
relative to the current working directory, and if the option value
contains a directory (e.g. \texttt{cache.path='cache/abc-'}), cache
files will be stored under that directory (automatically created if
it does not exist). The cache is invalidated and purged on any changes
to the code chunk, including both the R code and chunk options\footnote{One exception is the \texttt{include} option, which is not cached
because \texttt{include=TRUE/FALSE} does not affect code evaluation;
meanwhile, the value \texttt{getOption('width')} is also cached, so
if you change this option, the cache will also be invalidated (this
option affects the width of text output)}; this means previous cache files of this chunk are removed (filenames
are identified by the chunk label). Unlike \textbf{pgfSweave}, cache
files will never accumulate since old cache files will always be removed
in \textbf{knitr}. Unlike \textbf{weaver} or \textbf{cacheSweave},
\textbf{knitr} will try to preserve these side-effects:
\begin{enumerate}
\item printed results: meaning that any output of a code chunk will be loaded
into the output document for a cached chunk, although it is not really
evaluated. The reason is \textbf{knitr} also cache the output of a
chunk as a character string. Note this means graphics output is also
cached since it is part of the output. It has been a pain for me for
a long time to have to lose output to gain cache;
\item loaded packages: after the evaluation of each cached chunk, the list
of packages used in the current R session is written to a file under
the cache path named \textsf{\_\_packages}; next time if a cached
chunk needs to be rebuilt, these packages will be loaded first. The
reasons for caching package names are, it can be slow to load some
packages, and a package might be loaded in a previous cached chunk
which is not available to the next cached chunk when only the latter
needs to be rebuilt. Note this only applies to cached chunks, and
for uncached chunks, you must always use \emph{library()} to load
packages explicitly;
\end{enumerate}
Although \textbf{knitr} tries to keep some side-effects, there are
still other types of side-effects like setting \emph{par()} or \emph{options()}
which are not cached. Users should be aware of these special cases,
and make sure to clearly separate the code which is not meant to be
cached to other chunks which are not cached, e.g., set all global
options in the first chunk of a document and do not cache that chunk.

Sometimes a cached chunk may need to use objects from other cached
chunks, which can bring a serious problem \textendash{} if objects
in previous chunks have changed, this chunk will not be aware of the
changes and will still use old cached results, unless there is a way
to detect such changes from other chunks. There is an option called
\texttt{dependson} in \textbf{cacheSweave} which does this job. We
can explicitly specify which other chunks this chunk depends on by
setting an option like \texttt{dependson='chunkA;chunkB'} or equivalently
\texttt{dependson=c('chunkA', 'chunkB')}. Each time the cache of a
chunk is rebuilt, all other chunks which depend on this chunk will
lose cache, hence their cache will be rebuilt as well.

Another way to specify the dependencies among chunks is to use the
chunk option \texttt{autodep} and the function \emph{dep\_auto()}.
This is an experimental feature borrowed from \textbf{weaver} which
frees us from setting chunk dependencies manually. The basic idea
is, if a latter chunk uses any objects created from a previous chunk,
the latter chunk is said to depend on the previous one. The function
\emph{findGlobals()} in the \textbf{codetools} package is used to
find out all global objects in a chunk, and according to its documentation,
the result is an approximation. Global objects roughly mean the ones
which are not created locally, e.g. in the expression \texttt{function()
\{y <- x\}}, \texttt{x} should be a global object, whereas \texttt{y}
is local. Meanwhile, we also need to save the list of objects created
in each cached chunk, so that we can compare them to the global objects
in latter chunks. For example, if chunk A created an object \texttt{x}
and chunk B uses this object, chunk B must depend on A, i.e. whenever
A changes, B must also be updated. When \texttt{autodep=TRUE}, \textbf{knitr}
will write out the names of objects created in a cached chunk as well
as those global objects in two files named \textsf{\_\_objects} and
\textsf{\_\_globals} respectively; later we can use the function \emph{dep\_auto()}
to analyze the object names to figure out the dependencies automatically.
See \url{https://yihui.org/knitr/demo/cache/} for examples.

Yet another way to specify dependencies is \emph{dep\_prev()}: this
is a conservative approach which sets the dependencies so that a cached
chunk will depend on all its previous chunks, i.e. whenever a previous
chunk is updated, all later chunks will be updated accordingly.

\subsection{Code Externalization}

It can be more convenient to write R code in a separate file, rather
than mixing it into a \LaTeX{} document; for example, we can run
R code successively in a pure R script from one chunk to the other
without jumping through other texts. Since I prefer using \LyX{}
to write reports, Sweave is even more inconvenient because I have
to recompile the whole document each time, even if I only want to
know the results of a single chunk. Therefore \textbf{knitr} introduced
the feature of code externalization to a separate R script. Currently
the setting is like this: the R script also uses chunk labels (marked
in the form \texttt{\#\# -{}-{}-{}- chunk-label} by default); if the
code chunk in the input document is empty, \textbf{knitr} will match
its label with the label in the R script to input external R code.
For example, suppose this is a code chunk labelled as \texttt{Q1}
in an R script named \textsf{homework1-xie.R} which is under the same
directory as the Rnw document:

<<ext-r-code, eval=FALSE>>=
## ---- Q1 ---------------------
gcd = function(m, n) {
    while ((r <- m %% n) != 0) {
       m = n; n = r
    }
    n
}
@

In the Rnw document, we can first read the script using the function
\emph{read\_chunk()}:

<<read-chunk, eval=FALSE>>=
read_chunk('homework1-xie.R')
@

This is usually done in an early chunk, and we can use the chunk \texttt{Q1}
later in the Rnw document:

<<use-ext-chunk, echo=FALSE, comment=NA>>=
cat('<<Q1, echo=TRUE, tidy=TRUE>>=','@',sep='\n')
@

Different documents can read the same R script, so the R code can
be reusable across different input documents.

\subsection{Evaluation of Chunk Options\label{subsec:conditional}}

By default \textbf{knitr} uses a new syntax to parse chunk options:
it treats them as function arguments instead of a text string to be
split to obtain option values. This gives the user much more power
than the old syntax; we can pass arbitrary R objects to chunk options
besides simple ones like \texttt{TRUE}/\texttt{FALSE}, numbers and
character strings. The page \url{https://yihui.org/knitr/demo/sweave/}
has given two examples to show the advantages of the new syntax. Here
we show yet another useful application.

Before \textbf{knitr} 0.3, there was a feature named ``conditional
evaluation''\footnote{request from \url{https://plus.google.com/u/0/116405544829727492615/posts/43WrRUffjzK}}.
The idea is, instead of setting chunk options \texttt{eval} and \texttt{echo}
to be \texttt{TRUE} or \texttt{FALSE} (constants), their values can
be controlled by global variables in the current R session. This enables
\textbf{knitr} to conditionally evaluate code chunks according to
variables. For example, here we assign \texttt{TRUE} to a variable
\texttt{dothis}:

<<cond-variable>>=
dothis=TRUE
@

In the next chunk, we set chunk options \texttt{eval=dothis} and \texttt{echo=!dothis},
both are valid R expressions since the variable \texttt{dothis} exists.
As we can see, the source code is hidden, but it was indeed evaluated:

<<cond-out1, eval=dothis, echo=!dothis>>=
print('you cannot see my source because !dothis is FALSE')
@

Then we set \texttt{eval=dothis} and \texttt{echo=dothis} for another
chunk:

<<cond-out2,eval=dothis,echo=dothis>>=
dothis
@

If we change the value of \texttt{dothis} to \texttt{FALSE}, neither
of the above chunks will be evaluated any more. Therefore we can control
many chunks with a single variable, and present results selectively.

This old feature requires \textbf{knitr} to treat \texttt{eval} and
\texttt{echo} specially, and we can easily see that it is no longer
necessary with the new syntax: \texttt{eval=dothis} will tell R to
find the variable \texttt{dothis} automatically just like we call
a function \texttt{foobar(eval = dothis)}. What is more, all options
will be evaluated as R expressions unless they are already constants
which do not need to be evaluated, so this old feature has been generalized
to all other options naturally.

\subsection{Customization}

The \textbf{knitr} package is ready for customization. Both the patterns
and hooks can be customized; see the package website for details.
Here I show an example on how to save \textbf{rgl} plots \citep{R-rgl}
using a customized hook function. First we define a hook named \texttt{rgl}
using the function \emph{hook\_rgl()} in \textbf{rgl}:

<<rgl-demo>>=
library(rgl)
knit_hooks$set(rgl = hook_rgl)
head(hook_rgl)  # the hook function is defined as this
@

Then we only have to set the chunk option \texttt{rgl=TRUE}:

<<fancy-rgl, rgl=TRUE, dev='png', fig.width=5, fig.height=5, out.width='2in', message=FALSE, warning=FALSE, cache=TRUE>>=
library(rgl)
demo('bivar', package='rgl', echo=FALSE)
par3d(zoom=.7)
@

Due to the flexibility of output hooks, \textbf{knitr} supports several
different output formats. The implementation is fairly easy, e.g.,
for \LaTeX{} we put R output in \texttt{verbatim} environments, and
in HTML, it is only a matter of putting output in \texttt{div} layers.
These are simply character string operations. Many demos in \url{https://yihui.org/knitr/demo/}
show this idea clearly. This manual did not cover all the features
of \textbf{knitr}, and users are encouraged to thumb through the website
to know more possible features.

\section{Editors}

You can use any text editors to write the source documents, but some
have built-in support for \textbf{knitr}. Both RStudio (\url{http://www.rstudio.org})
and \LyX{} (\url{http://www.lyx.org}) have full support for \textbf{knitr},
and you can compile the document to PDF with just one click. See \url{https://yihui.org/knitr/demo/rstudio/}
and \url{https://yihui.org/knitr/demo/lyx/} respectively. It is also
possible to support other editors like \href{https://yihui.org/knitr/demo/eclipse/}{Eclipse},
\href{https://yihui.org/knitr/demo/editors/}{Texmaker and WinEdt};
see the demo list in the website for configuration instructions.

\section*{About This Document}

This manual was written in \LyX{} and compiled with \textbf{knitr}
(version \Sexpr{packageVersion('knitr')}). The \LyX{} source and
the Rnw document exported from \LyX{} can be found under these directories:

<<source-location, eval=FALSE>>=
system.file('examples', 'knitr-manual.lyx', package='knitr')  # lyx source
system.file('examples', 'knitr-manual.Rnw', package='knitr')  # Rnw source
@

You can use the function \emph{knit()} to knit the Rnw document (remember
to put the two \textsf{.bib} files under the same directory), and
you need to make sure all the R packages used in this document are
installed:

<<required-packages, eval=FALSE>>=
install.packages(c('animation', 'rgl', 'tikzDevice', 'ggplot2'))
@

Feedback and comments on this manual and the package are always welcome.
Bug reports and feature requests can be sent to \url{https://github.com/yihui/knitr/issues},
and questions can be delivered to the \href{mailto:knitr@googlegroups.com}{mailing list}
\url{https://groups.google.com/group/knitr}.

% when knitr is updated, this chunk will be updated; why?
<<auto-bib, version=packageVersion('knitr'), echo=FALSE, cache=TRUE, message=FALSE, warning=FALSE>>=
# write all packages in the current session to a bib file
write_bib(c(.packages(), 'evaluate', 'formatR', 'highr'), file = 'knitr-packages.bib')
@

\bibliographystyle{jss}
\bibliography{knitr-manual,knitr-packages}

\end{document}

%% LyX 2.2.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[T1]{fontenc}
\usepackage[latin9]{inputenc}
\usepackage{url}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
\providecommand{\LyX}{L\kern-.1667em\lower.25em\hbox{Y}\kern-.125emX\@}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\@ifundefined{showcaptionsetup}{}{%
 \PassOptionsToPackage{caption=false}{subfig}}
\usepackage{subfig}
\makeatother

\begin{document}
<<setup, include=FALSE>>=
library(knitr)
opts_chunk$set(fig.path='figure/subfloats-')
@

\title{Using subfloats in \LyX{} with knitr}

\author{Yihui Xie}

\maketitle
If you insert a float figure inside a float figure in \LyX{}, you
will get subfloats. To make sure the two subfloats can stand side
by side, you have to:
\begin{enumerate}
\item remove new lines between the two subfloat environments in \LyX{}
(if you are not sure if there are blank lines between them, check
the \LaTeX{} source);
\item use the chunk option \texttt{echo=FALSE} to hide the R source code
in the output;
\item set \texttt{out.width} to be less than half of the linewidth if you
do not want the sub figures to exceed the page margin;
\item most importantly, leave a blank line in the beginning of the subfloat
environment, so that \texttt{<\textcompwordmark{}<>\textcompwordmark{}>=}
starts in a new line and \textbf{knitr} can recognize it;
\end{enumerate}
Figure \ref{fig:knitr-subfloat} is an example.

This document was compiled with \textbf{knitr} \Sexpr{packageVersion('knitr')}.
Rnw source at \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-subfloats.Rnw}.

\begin{figure}
\subfloat[this is a subfloat]{
<<test-subfloat, echo=FALSE, out.width='.45\\linewidth'>>=
plot(1)
@

}\subfloat[another float]{
<<test-another, echo=FALSE, out.width='.45\\linewidth'>>=
plot(rnorm(100))
@

}

\caption{one figure with 2 sub figures\label{fig:knitr-subfloat}}
\end{figure}

\end{document}

%% LyX 2.2.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[nohyper,justified]{tufte-handout}
\usepackage[T1]{fontenc}
\usepackage{url}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.

\title{Customizing Syntax Highlighting Themes}
\author{Yihui Xie \& Ramnath Vaidyanathan}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[breaklinks=true,pdfstartview=FitH]{hyperref}

\makeatother

\begin{document}
<<setup, include=FALSE, cache=FALSE>>=
library(knitr)
opts_chunk$set(fig.path='figure/theme-', cache.path='cache/theme-', cache=TRUE)
options(formatR.arrow=TRUE,width=78)
knit_hooks$set(par=function(before, options, envir){if (before) par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3)})
@
<<solarized-theme, cache=FALSE, echo=FALSE>>=
thm = knit_theme$get("solarized-dark")
knit_theme$set(thm)
@
\maketitle
\begin{abstract}
This manual\footnote{source code at \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-themes.Rnw}}
shows how to customize syntax highlighting of source code using themes.
It walks the user through the basics of syntax highlighting in \textbf{knitr},
and the use of built-in themes.
\end{abstract}
The \textbf{knitr} package uses the \textbf{highr} package to highlight
source code in a document. In short, the \textbf{highr} package parses
the source code, tokenizes it into grammar symbols, and formats their
appearance using CSS or \LaTeX{} commands. 

\section{Usage}

We can use the object \texttt{knit\_theme} to set / get a theme. See
\texttt{?knit\_theme} for the usage. For example, we set the theme
of this document to be \texttt{solarized-dark}:

<<solarized-theme>>=
@

\section{Built-in Themes}

\begin{margintable}
<<eclipse-css, comment=NA, echo=FALSE>>=
cat(c(readLines('../themes/edit-eclipse.css', n=30),'...'),sep='\n')
@
\end{margintable}

The listing on the right shows the CSS file for one of the themes,
\texttt{edit-eclipse}, which was adapted from Andre Simon's excellent
highlighter\footnote{\url{http://www.andre-simon.de/}}. The \textbf{knitr}
package comes pre-loaded with a number of themes based on this highlighter.
Here is list of all available code themes\footnote{For a preview of all themes, see \url{https://gist.github.com/yihui/3422133}}:

<<all-themes>>=
knit_theme$get()
@

Shown below is how the \texttt{solarized-dark} theme looks like when
applied to R code:

<<demo-code, eval = FALSE>>=
library(XML)
library(plyr)
library(reshape)
# SCRAPE THE DATA FROM WEB 

base_url = "http://www.mlsoccer.com/stats/%s/reg"
years    = 1996:2010
options(width = 40)

#' Function to save data for each year
save_data = function(y){
  url  = sprintf(base_url, y)
  tab  = readHTMLTable(url, header = FALSE, stringsAsFactors = FALSE);
  pos  = max(grep("United", tab));
  tab  = tab[[pos]];
  tab$year   = y;
  return(tab)            
}
	
team.list  = llply(years, save_data);
mls        = merge_recurse(team.list);
@

\section{Misc}

One thing to consider is the foreground color of plots when we use
dark themes; we need to make it lighter, otherwise the graphical elements
will be difficult to be read. We can access the foreground color of
the theme in the list returned by \texttt{knit\_theme\$get(theme)},
e.g., for this document:

<<fg-color>>=
## the object thm is from the first chunk
thm$foreground  # foreground color
thm$background  # background color
@

When we make plots, we may use these colors, e.g.

<<plot-color, fig.width=6, fig.height=4, out.width='.8\\linewidth', par=TRUE>>=
## can design a chunk hook to set foreground color automatically
fgcolor=thm$foreground
par(fg=fgcolor, col.axis=fgcolor, col.lab=fgcolor)
plot(rnorm(100),pch=19)
@

Of course, we do not need to worry about these colors when we use
a white background for the plots.
\end{document}

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[sc]{mathpazo}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\setcounter{secnumdepth}{2}
\setcounter{tocdepth}{2}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}
\begin{document}

<<setup, include=FALSE, cache=FALSE>>=
# set global chunk options
opts_chunk$set(fig.path='figure/minimal-', fig.align='center', fig.show='hold')
options(replace.assign=TRUE,width=90)
@


\title{A Minimal Demo of knitr}


\author{Yihui Xie}

\maketitle
You can test if \textbf{knitr} works with this minimal demo. OK, let's
get started with some boring random numbers:

<<boring-random>>=
set.seed(1121)
(x=rnorm(20))
mean(x);var(x)
@

The first element of \texttt{x} is \Sexpr{x[1]}. Boring boxplots
and histograms recorded by the PDF device:

<<boring-plots, fig.width=4, fig.height=4, out.width='.4\\linewidth'>>=
## two plots side by side (option fig.show='hold')
par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3,las=1)
boxplot(x)
hist(x,main='')
@

Do the above chunks work? You should be able to compile the \TeX{}
document and get a PDF file like this one: \url{https://bitbucket.org/stat/knitr/downloads/knitr-minimal.pdf}.
The Rnw source of this document is at \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-minimal.Rnw}.

\end{document}

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[10pt]{beamer}
\usepackage[T1]{fontenc}
\setcounter{secnumdepth}{3}
\setcounter{tocdepth}{3}
\usepackage{url}
\ifx\hypersetup\undefined
  \AtBeginDocument{%
    \hypersetup{unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 0},backref=false,colorlinks=false}
  }
\else
  \hypersetup{unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 0},backref=false,colorlinks=false}
\fi
\usepackage{breakurl}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
\providecommand{\LyX}{\texorpdfstring%
  {L\kern-.1667em\lower.25em\hbox{Y}\kern-.125emX\@}
  {LyX}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
 % this default might be overridden by plain title style
 \newcommand\makebeamertitle{\frame{\maketitle}}%
 \AtBeginDocument{
   \let\origtableofcontents=\tableofcontents
   \def\tableofcontents{\@ifnextchar[{\origtableofcontents}{\gobbletableofcontents}}
   \def\gobbletableofcontents#1{\origtableofcontents}
 }
 \def\lyxframeend{} % In case there is a superfluous frame end
 \long\def\lyxframe#1{\@lyxframe#1\@lyxframestop}%
 \def\@lyxframe{\@ifnextchar<{\@@lyxframe}{\@@lyxframe<*>}}%
 \def\@@lyxframe<#1>{\@ifnextchar[{\@@@lyxframe<#1>}{\@@@lyxframe<#1>[]}}
 \def\@@@lyxframe<#1>[{\@ifnextchar<{\@@@@@lyxframe<#1>[}{\@@@@lyxframe<#1>[<*>][}}
 \def\@@@@@lyxframe<#1>[#2]{\@ifnextchar[{\@@@@lyxframe<#1>[#2]}{\@@@@lyxframe<#1>[#2][]}}
 \long\def\@@@@lyxframe<#1>[#2][#3]#4\@lyxframestop#5\lyxframeend{%
   \frame<#1>[#2][#3]{\frametitle{#4}#5}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\usetheme{PaloAlto}

\makeatother

\begin{document}

<<setup, include=FALSE>>=
opts_chunk$set(fig.path='figure/beamer-',fig.align='center',fig.show='hold',size='footnotesize')
@


\title[knitr, Beamer, and FragileFrame]{A Minimal Demo of knitr with Beamer and Fragile Frames}


\author{Yihui Xie%
\thanks{I thank Richard E. Goldberg for providing this demo.%
}}

\makebeamertitle

\lyxframeend{}


\lyxframeend{}\lyxframe{Background}
\begin{itemize}
\item The \textbf{knitr}\textbf{\emph{ }}package allows you to embed R code
and figures in \LaTeX{} documents

\begin{itemize}
\item It has functionality similar to Sweave but looks nicer and gives you
more control
\end{itemize}
\item If you already have Sweave working in \LyX{}, getting \textbf{knitr}
to work is trivial

\begin{enumerate}
\item Install the \textbf{knitr} package in \emph{R}
\item Read \url{http://yihui.name/knitr/demo/lyx/}
\end{enumerate}
\item If you use Sweave or \textbf{knitr} with Beamer in \LyX{}, you probably
use the \emph{Beamer Fragile} module%
\footnote{\url{http://www.lyx.org/trac/ticket/7273}%
} too. Let's see if \textbf{knitr} works with Beamer in this small
demo. 
\end{itemize}

\lyxframeend{}\section{First Test}

\begin{frame}
[fragile]
\frametitle{First Test}

OK, let's get started with just some text:

<<echo=FALSE,results='hide'>>=
# some setup
options(width=60)  # make the printing fit on the page
set.seed(1121)   # make the results repeatable
@

<<>>= 
# create some random numbers
(x=rnorm(20))  
mean(x);var(x)  
@

BTW, the first element of \texttt{x} is \Sexpr{x[1]}. (Did you notice
the use of\texttt{ \textbackslash{}Sexpr\{\}}.)
\end{frame}



\lyxframeend{}\section{Second Test}

\begin{frame}
[fragile]
\frametitle{Second Test}

Text is nice but let's see what happens if we make a couple of plots
in our chunk:

<<boring-plots,fig.width=4,fig.height=4,out.width='.45\\linewidth'>>=
par(las=1,mar=c(4,4,.1,.1))  # tick labels direction
boxplot(x) 
hist(x,main='',col="blue",probability=TRUE) 
lines(density(x),col="red")
@
\end{frame}



\lyxframeend{}\section{The Big Question}


\lyxframeend{}\lyxframe{The Big Question}

Do the above chunks work? You should be able to compile the \LyX{}
document and get a nice-looking PDF slide presentation. If not, time
to double-check everything...


\lyxframeend{}

\end{document}

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[T1]{fontenc}
\usepackage[latin9]{inputenc}
\usepackage{url}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
\providecommand{\LyX}{L\kern-.1667em\lower.25em\hbox{Y}\kern-.125emX\@}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\@ifundefined{showcaptionsetup}{}{%
 \PassOptionsToPackage{caption=false}{subfig}}
\usepackage{subfig}
\makeatother

\begin{document}

<<setup, include=FALSE>>=
opts_chunk$set(fig.path='figure/subfloats-')
@


\title{Using subfloats in \LyX{} with knitr}


\author{Yihui Xie}

\maketitle
If you insert a float figure inside a float figure in \LyX{}, you
will get subfloats. To make sure the two subfloats can stand side
by side, you have to:
\begin{enumerate}
\item remove new lines between the two subfloat environments in \LyX{} (if
you are not sure if there are blank lines between them, check the
\LaTeX{} source);
\item use the chunk option \texttt{echo=FALSE} to hide the R source code
in the output;
\item set \texttt{out.width} to be less than half of the linewidth if you
do not want the sub figures to exceed the page margin;
\item most importantly, leave a blank line in the beginning of the subfloat
environment, so that \texttt{<\textcompwordmark{}<>\textcompwordmark{}>=}
starts in a new line and \textbf{knitr} can recognize it;
\end{enumerate}
Figure \ref{fig:knitr-subfloat} is an example.

This document was compiled with \textbf{knitr} \Sexpr{packageVersion('knitr')}.
Rnw source at \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-subfloats.Rnw}.

\begin{figure}
\subfloat[this is a subfloat]{
<<test-subfloat, echo=FALSE, out.width='.45\\linewidth'>>=
plot(1)
@

}\subfloat[another float]{
<<test-another, echo=FALSE, out.width='.45\\linewidth'>>=
plot(rnorm(100))
@

}

\caption{one figure with 2 sub figures\label{fig:knitr-subfloat}}
\end{figure}


\end{document}

\documentclass{article}
\usepackage[sc]{mathpazo}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\setcounter{secnumdepth}{2}
\setcounter{tocdepth}{2}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\makeatother

\begin{document}

<<setup,include=FALSE,cache=FALSE>>=
options(width=90)
opts_chunk$set(out.width = '.6\\linewidth')
.knitr.title = if (exists('.knitr.title')) paste('\\title{', .knitr.title, '}', sep = '') else ''
.knitr.author = if (nzchar(.knitr.title) && exists('.knitr.author')) {
  paste('\\author{', .knitr.author, '%\n',
        '\\thanks{This report is automatically generated with the R package \\textbf{knitr}
        (version ', packageVersion('knitr'), ').}}', sep = '')
} else ''
@

\Sexpr{.knitr.title}

\Sexpr{.knitr.author}

\Sexpr{if (nzchar(.knitr.title)) '\\maketitle'}
The results below are generated from an R script.

<<%sCHUNK_LABEL_HERE>>=
@

The R session information (including the OS info, R version and all
packages used):

<<session-info, cache=FALSE>>=
sessionInfo()
Sys.time()
@
<<clean-up, include=FALSE>>=
rm(.knitr.author); rm(.knitr.author)
@

\end{document}

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[twocolumn]{article}
\usepackage[sc]{mathpazo}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=3cm,bmargin=3cm,lmargin=3cm,rmargin=3cm}
\setcounter{secnumdepth}{2}
\setcounter{tocdepth}{2}
\setlength{\parskip}{\bigskipamount}
\setlength{\parindent}{0pt}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\makeatother

\begin{document}

<<setup, include=FALSE, cache=FALSE>>=
opts_chunk$set(fig.path='figure/twocolumn-', fig.align='center', fig.show='hold')
render_listings()
@


\title{Using knitr in A Two-column Document}


\author{Yihui Xie}

\maketitle
We can use the \texttt{figure{*}} environment in \LaTeX{} so that
figures can span over two columns. Because the output hooks use the
\textbf{framed} package which does not work well with the \texttt{figure{*}}
environment, we can either modify the output hooks or use other packages
like \textbf{listings}. For example,

<<chunk-hook, echo=2>>=
<<setup>>
@

\begin{figure*}
<<boring-plots, fig.width=5, fig.height=5, out.width='.45\\linewidth'>>=
x=rnorm(100)
par(mar=c(4,4,.1,.1))
boxplot(x)
hist(x,main='')
@

\caption{Two plots spanning over two columns.}
\end{figure*}


The output PDF file will be like: \url{https://bitbucket.org/stat/knitr/downloads/knitr-twocolumn.pdf}.
The Rnw source of this document is at \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-twocolumn.Rnw}.

\end{document}

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[nohyper,justified]{tufte-handout}
\usepackage[T1]{fontenc}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=true,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview=FitH}
\usepackage{breakurl}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.

\title{knitr Graphics Manual}
\author{Yihui Xie}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[buttonsize=1em]{animate}

\makeatother

\begin{document}

<<setup, include=FALSE, cache=FALSE>>=
options(replace.assign=TRUE,width=50)
opts_chunk$set(fig.path='figure/graphics-', cache.path='cache/graphics-', fig.align='center', dev='tikz', fig.width=5, fig.height=5, fig.show='hold', cache=TRUE, par=TRUE)
knit_hooks$set(par=function(before, options, envir){
if (before && options$fig.show!='none') par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3)
}, crop=hook_pdfcrop)
@

\maketitle
\begin{abstract}
This manual shows features of graphics in the \textbf{knitr} package
(version \Sexpr{packageVersion('knitr')}) in detail, including the
graphical devices, plot recording, plot rearrangement, control of
plot sizes, the tikz device, figure captions, animations and other
types of plots such as \textbf{rgl} or GGobi plots.
\end{abstract}
Before reading this specific manual%
\footnote{\url{http://bit.ly/knitr-graphics-src} (Rnw source)%
}, you must have finished the main manual%
\footnote{\url{http://bit.ly/knitr-main-pdf}%
}.


\section{Graphical Devices}

The \textbf{knitr} package comes with more than 20 built-in graphical
devices, and you can specify them through the \texttt{dev} option.
This document uses the global option \texttt{dev='tikz'}, i.e., the
plots are recorded by the tikz device by default, but we can change
the device locally. Since tikz will be used extensively throughout
this manual and you will see plenty of tikz graphics later, now we
first show a few other devices.

<<test-plot, eval=FALSE>>=
with(trees, symbols(Height, Volume, circles = Girth/16, inches = FALSE, bg = 'deeppink', fg = 'gray30'))
@

\begin{marginfigure}
<<pdf-dev, ref.label='test-plot', dev='pdf', out.width='\\linewidth', echo=FALSE>>=
@

\caption{The default PDF device.\label{mar:pdf-dev}}
\end{marginfigure}


\begin{marginfigure}
<<png-dev, ref.label='test-plot', dev='png', out.width='\\linewidth', echo=FALSE>>=
@

\caption{The PNG device.\label{mar:png-dev}}
\end{marginfigure}


Figure \ref{mar:pdf-dev} and \ref{mar:png-dev} show two standard
devices in the \textbf{grDevices} package. We can also use devices
in the \textbf{Cairo} or \textbf{cairoDevice} package, e.g., the chunk
below uses the \emph{CairoPNG()} device in the \textbf{Cairo} package.

<<cairo-png-dev, ref.label='test-plot', dev='CairoPNG', out.width='.5\\linewidth', echo=FALSE>>=
@


\section{Plot Recording}

As mentioned in the main manual, \textbf{knitr} uses the \textbf{evaluate}
package to record plots. There are two sources of plots: first, whenever
\emph{plot.new()} or \emph{grid.newpage()} is called, \textbf{evaluate}
will try to save a snapshot of the current plot%
\footnote{For technical details, see \texttt{?setHook} and \texttt{?recordPlot}%
}; second, after each complete expression is evaluated, a snapshot
is also saved. To speed up recording, the null graphical device \texttt{pdf(file
= NULL)} is used. Figure \ref{fig:two-high} shows two expressions
producing two high-level plots.

\begin{figure}
<<two-high, fig.width=3, fig.height=2.5, out.width='.49\\linewidth'>>=
plot(cars)
boxplot(cars$dist,xlab='dist')
@

\caption{Two high-level plots are captured. The key to arrange two plots side
by side is to specify the \texttt{out.width} option so that each plot
takes less than half of the line width. We do not have to use the
\texttt{par(mfrow)} trick, and it may not work in some cases (e.g.
to put base graphics and \textbf{ggplot2} side by side; recall Figure
1 in the main manual).\label{fig:two-high}}
\end{figure}


Figure \ref{fig:low-plot-loop} shows another example of two R expressions,
but the second expression only involves with low-level plotting changes.
By default, low-level plot changes are discarded, but you can retain
them with the option \texttt{fig.keep='all'}.

\begin{figure}
<<low-plot-loop, fig.width=3, fig.height=2.5, out.width='.49\\linewidth', fig.keep='all'>>=
plot(0,0,type='n',ann=FALSE)
for(i in seq(0, 2*pi,length=20)) {points(cos(i),sin(i))}
@

\caption{Two complete R expressions will produce at most two plots, as long
as there are not multiple high-level plotting calls in each expression;
option \texttt{fig.keep='all'} here.\label{fig:low-plot-loop}}
\end{figure}


Together with \texttt{fig.show='asis'}, we can show the process of
plotting step by step like Figure \ref{fig:high-with-low}.

\begin{figure}
<<high-with-low, fig.width=3, fig.height=2.5, out.width='.5\\linewidth', fig.keep='all', fig.show='asis'>>=
plot(cars)
abline(lm(dist~speed, data=cars))  # a regression line
@

\caption{Low-level plot changes in base graphics can be recorded separately,
and plots can be put in the places where they were produced.\label{fig:high-with-low}}
\end{figure}


A further note on plot recording: \textbf{knitr} examines all recorded
plots (as R objects) and compares them sequentially; if the previous
plot is a ``subset'' of the next plot (= previous plot + low-level
changes), the previous plot will be removed when \texttt{fig.keep='high'}.
If two successive plots are identical, the second one will be removed
by default, so it might be a little bit surprising that the following
chunk will only produce one plot by default%
\footnote{adapted from \url{https://github.com/yihui/knitr/issues/41}%
}:

<<identical-plots,eval=FALSE>>=
m = matrix(1:100, ncol = 10)
image(m)
image(m*2)  # exactly the same as previous plot
@


\section{Plot Rearrangement}

We can rearrange the plots in chunks in several ways. They can be
inserted right after the line(s) of R code which produced them, or
accumulated till the end of the chunk. There is an example in the
main manual demonstrating \texttt{fig.show='asis'} for two high-level
plots, and Figure \ref{fig:high-with-low} in this manual also demonstrates
this option for a high-level plot followed by a low-level change.

Here is an example demonstrating the option \texttt{fig.keep='last'}
(only the last plot is kept):

\begin{figure}
<<fig-last, fig.width=5.2, fig.height=3, out.width='.7\\linewidth', fig.keep='last', message=FALSE>>=
library(ggplot2)
pie <- ggplot(diamonds, aes(x = factor(1), fill = cut)) + xlab('cut') + geom_bar(width = 1) 
pie + coord_polar(theta = "y") # a pie chart
pie + coord_polar() # the bullseye chart 
@

\caption{Two plots were produced in this chunk, but only the last one is kept.
This can be useful when we experiment with many plots, but only want
the last result. (Adapted from the \textbf{ggplot2} website)}
\end{figure}


When multiple plots are produced by a code chunk, we may want to show
them as an animation with the option \texttt{fig.show='animate'}.
Figure \ref{fig:clock-animation} shows a simple clock animation;
you may compare the code to Figure \ref{fig:high-with-low} to understand
that high-level plots are always recorded, regardless of where they
appeared.

\begin{figure}
<<clock-animation, fig.width=3, fig.height=3, out.width='.4\\linewidth', fig.show='animate', tidy=FALSE, crop=TRUE>>=
par(mar = rep(3, 4))
for (i in seq(pi/2, -4/3 * pi, length = 12)) {
    plot(0, 0, pch = 20, ann = FALSE, axes = FALSE)
    arrows(0, 0, cos(i), sin(i))
    axis(1, 0, "VI"); axis(2, 0, "IX")
    axis(3, 0, "XII"); axis(4, 0, "III"); box()
}
@

\caption{A clock animation. You have to view it in Adobe Reader: click to play/pause;
there are also buttons to speed up or slow down the animation.\label{fig:clock-animation}}
\end{figure}


We can also set the alignment of plots easily with the \texttt{fig.align}
option; this document uses \texttt{fig.align='center'} as a global
option, and we can also set plots to be left/right-aligned. Figure
\ref{fig:maruko-plot} is an example of a left-aligned plot.

\begin{figure}
<<maruko-plot, fig.width=7, fig.height=4, out.width='.45\\linewidth', fig.align='left', crop=TRUE>>=
stars(cbind(1:16,10*(16:1)),draw.segments=TRUE)
@

\caption{A left-aligned plot adapted from \texttt{?stars} (I call this the
``Maruko'' plot, and it is one of my favorites; see \protect\url{http://en.wikipedia.org/wiki/Chibi_Maruko-chan}).\label{fig:maruko-plot}}
\end{figure}



\section{Plot Size}

We have seen several examples in which two or more plots can be put
side by side, and this is because the plots were resized in the output
document; with the chunk option \texttt{out.width} less than half
of the line width, \LaTeX{} will arrange two plots in one line; if
it is less than $1/3$ of the line width, three plots can be put in
one line. Of course we can also set it to be an absolute width like
\texttt{3in} (3 inches). This option is used extensively in this document
to control the size of plots in the output document.


\section{The tikz Device}

The main advantage of using tikz graphics is the consistency of styles
between texts in plots and those in the main document. Since we can
use native \LaTeX{} commands in plots, the styles of texts in plots
can be very sophisticated (see Figure \ref{fig:math-formula-tt} for
an example).

\begin{figure}
<<math-formula-tt, fig.width=6, fig.height=2, out.width='\\linewidth'>>=
plot(0:1,0:1,type='n', ylab='origin of statistics', xlab='statistical presentation rocks with \\LaTeX{}')
text(.5,c(.8,.5,.2), c('\\texttt{lm(y \\textasciitilde{} x)}', '$\\hat{\\beta}=(X^{\\prime}X)^{-1}X^{\\prime}y$',  '$(\\Omega,\\mathcal{F},\\mu)$'))
@

\caption{A plot created by \textbf{tikzDevice} with math expressions and typewriter
fonts. Note the font style consistency -- we write the same expressions
in \protect\LaTeX{} here: $\hat{\beta}=(X^{\prime}X)^{-1}X^{\prime}y$ and
$(\Omega,\mathcal{F},\mu)$; also \texttt{lm(y \textasciitilde{} x)}.\label{fig:math-formula-tt}}
\end{figure}


When using Xe\LaTeX{} instead of PDF\LaTeX{} to compile the document,
we need to tell the \textbf{tikzDevice} package by setting the \texttt{tikzDefaultEngine}
option before all plot chunks (preferably in the first chunk):

<<xetex-tikz, eval=FALSE>>=
options(tikzDefaultEngine='xetex')
@

This is useful and often necessary to compile tikz plots which contain
(UTF8) multi-byte characters.


\section{Figure Caption}

If the chunk option \texttt{fig.cap} is not \texttt{NULL} or \texttt{NA},
the plots will be put in a \texttt{figure} environment when the output
format is \LaTeX{}, and this option is used to write a caption in
this environment using \texttt{\textbackslash{}caption\{\}}. The other
two related options are \texttt{fig.scap} and \texttt{fig.lp} which
set the short caption and a prefix string for the figure label. The
default short caption is extracted from the caption by truncating
it at the first period or colon or semi-colon. The label is a combination
of \texttt{fig.lp} and the chunk label. Because \texttt{figure} is
a float environment, it can float away from the chunk output to other
places such as the top or bottom of a page when the \TeX{} document
is compiled.


\section{Other Features}

The \textbf{knitr} package can be extended with hook functions, and
here I give a few examples illustrating the flexibility.


\subsection{Cropping PDF Graphics}

Some R users may have been suffering from the extra margins in R plots,
especially in base graphics (\textbf{ggplot2} is much better in this
aspect). The default graphical option \texttt{mar} is about \texttt{c(5,
4, 4, 2)} (see \texttt{?par}), which is often too big. Instead of
endlessly setting \texttt{par(mar)}, you may consider the program
\texttt{pdfcrop}, which can crop the white margin automatically%
\footnote{\url{http://www.ctan.org/pkg/pdfcrop}%
}. In \textbf{knitr}, we can set up the hook \emph{hook\_pdfcrop()}
to work with a chunk option, say, \texttt{crop}.

<<crop-hook, cache=FALSE>>=
knit_hooks$set(crop=hook_pdfcrop)
@

Now we compare two plots below. The first one is not cropped (Figure
\ref{fig:pdf-nocrop}); then the same plot is produced but with a
chunk option \texttt{crop=TRUE} which will call the cropping hook
(Figure \ref{fig:pdf-crop}).

\begin{figure}
\begin{kframe}
<<pdf-nocrop, fig.width=3.2, fig.height=3.2, out.width='.5\\linewidth'>>=
par(mar=c(5,4,4,2),bg='white')  # large margin
plot(lat~long,data=quakes,pch=20,col=rgb(0,0,0,.2))
@
\end{kframe}

\caption{The original plot produced in R, with a large margin.\label{fig:pdf-nocrop}}
\end{figure}


\begin{figure}
\begin{kframe}
<<pdf-crop, ref.label='pdf-nocrop', echo=FALSE, fig.width=3.2, fig.height=3.2, out.width='.5\\linewidth', crop=TRUE>>=
@
\end{kframe}

\caption{The cropped plot; it fits better in the document.\label{fig:pdf-crop}}
\end{figure}


As we can see, the white margins are gone. If we use \emph{par()},
it might be hard and tedious to figure out a reasonable amount of
margin in order that neither is any label cropped nor do we get a
too large margin. My experience is that \texttt{pdfcrop} works well
with base graphics, but barely works with \textbf{grid} graphics (therefore
\textbf{lattice} and \textbf{ggplot2} are ruled out).


\subsection{Manually Saved Plots}

We have explained how R plots are recorded before. In some cases,
it is not possible to capture plots by \emph{recordPlot()} (such as
\textbf{rgl} plots), but we can save them using other functions. To
insert these plots into the output, we need to set up a hook first
like this (see \texttt{?hook\_plot\_custom} for details):

<<custom-plot-hook, cache=FALSE>>=
knit_hooks$set(custom.plot = hook_plot_custom)
@

Then we set the chunk option \texttt{custom.plot=TRUE}, and manually
write plot files in the chunk. Here we show an example of capturing
GGobi plots using the function \emph{ggobi\_display\_save\_picture()}
in the \textbf{rggobi} package:

<<ggobi-plot, custom.plot=TRUE, fig.ext='png', out.width='2.5in', results='hide', warning=FALSE>>=
library(rggobi)
ggobi(ggobi_find_file('data', 'flea.csv'))
Sys.sleep(1)  # wait for snapshot
ggobi_display_save_picture(path=fig_path('.png'))
@

One thing to note here is we have to make sure the plot filename is
from \emph{fig\_path()}, which is a convenience function to return
the figure path for the current chunk.

We can do whatever normal R plots can do with this hook, and we give
another example below to show how to work with animations.

<<rgl-animation, custom.plot=TRUE, fig.ext='png', out.width='2.5in', fig.show='animate', fig.num=20, interval=.4,  aniopts='controls,autoplay', results='hide'>>=
library(animation) # adapted from demo('rgl_animation')
data(pollen)
uM = matrix(c(-0.37, -0.51, -0.77, 0, -0.73, 0.67, -0.1, 0, 0.57, 0.53, -0.63, 0, 0, 0, 0, 1), 4, 4)
library(rgl)
open3d(userMatrix = uM, windowRect = c(0, 0, 400, 400))
plot3d(pollen[, 1:3])
zm = seq(1, 0.05, length = 20)
par3d(zoom = 1)  # change the zoom factor gradually later
for (i in 1:length(zm)) {
    par3d(zoom = zm[i]); Sys.sleep(.05)
    rgl.snapshot(paste(fig_path(i), 'png', sep = '.'))
}
@


\subsection{rgl Plots}

With the hook \emph{hook\_rgl()}, we can easily save snapshots from
the \textbf{rgl} package. We have shown an example in the main manual,
and here we add some details. The rgl hook is a good example of taking
care of details by carefully using the \texttt{options} argument in
the hook; for example, we cannot directly set the width and height
of rgl plots in \emph{rgl.snapshot()} or \emph{rgl.postscript()},
so we make use of the options \texttt{fig.width}, \texttt{fig.height}
and \texttt{dpi} to calculate the expected size of the window, then
resize the current window by \emph{par3d()}, and finally save the
plot.

This hook is actually built upon \emph{hook\_plot\_custom()} -- first
it saves the \textbf{rgl} snapshot, then it calls \emph{hook\_plot\_custom()}
to write the output code.

\appendix

\section{How to Compile This Manual}

This manual has a long chain of dependencies, so it may not be easy
to compile. These packages are required (all of them are free software):
\begin{description}
\item [{R}] Cairo, ggplot2, tikzDevice, rgl, rggobi, animation (all available
on CRAN except tikzDevice which is on R-Forge for the time being)
\item [{\LaTeX{}}] animate, hyperref and the tufte-handout class
\item [{Other}] GGobi, pdfcrop\end{description}


\end{document}

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[sc]{mathpazo}
\usepackage[T1]{fontenc}
\usepackage{listings}
\lstset{language=R}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\usepackage{url}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}
\begin{document}

<<setup, include=FALSE, cache=FALSE>>=
opts_chunk$set(fig.path = 'figure/listings-')
options(replace.assign = TRUE)
render_listings()
@


\title{Using listings with knitr}


\author{Yihui Xie}

\maketitle
To use the \textbf{listings} package with \textbf{knitr}, all you
need to do is to call a function in your first setup chunk (that chunk
should be hidden from the output with \texttt{include=FALSE} and should
not be cached):

% I just want to echo the 2nd line
<<setup, echo=2>>=
@

This function modifies the output hooks and header information so
that the output is written in \textbf{listings} environments, which
are kindly provided by Frank Harrell and can be found in \url{https://github.com/yihui/knitr/blob/master/inst/misc/Sweavel.sty}.
Of course you have to install the \textbf{listings} package if your
\LaTeX{} toolset does not include this package.

Now we can see how the results look like with our new settings:

<<boring-random>>=
set.seed(1121)  # for reproducibility
options(width=85)
x=rnorm(20)
x
mean(x)
sqrt(-1)  # this will give you a warning message
@

Another chunk:

<<rgl-hook-demo,eval=FALSE>>=
hook_rgl = function(before, options, envir) {
    ## after a chunk has been evaluated
    if (before || !require('rgl') || rgl.cur() == 0) return()  # no active device
    name = paste(valid_prefix(options$fig.path), options$label, sep = '')
    par3d(windowRect = 100 + options$dpi * c(0, 0, options$width, options$height))
    Sys.sleep(.05) # need time to respond to window size change

    fmt = opts_knit$get('out.format')
    if (fmt %in% c('html', 'markdown', 'gfm', 'jekyll')) options$dev = 'png'

    ## support 3 formats: eps, pdf and png (default)
    switch(options$dev,
           postscript = rgl.postscript(paste(name, '.eps', sep = ''), fmt = 'eps'),
           pdf = rgl.postscript(paste(name, '.pdf', sep = ''), fmt = 'pdf'),
           rgl.snapshot(paste(name, '.png', sep = ''), fmt = 'png'))

    if (fmt == 'html') return(.plot.hook.html(c(name, 'png'), options))
    if (fmt %in% c('markdown', 'gfm', 'jekyll'))
        return(.plot.hook.markdown(c(name, 'png'), options))

    paste(ifelse(options$align == 'center', '\\centering{}', ''), '\\includegraphics[',
          sprintf('width=%s', options$out.width), ']{', name, '}\n', sep = '')
}
@

Well, we can do crazier things with \textbf{knitr}. Here we use the
\texttt{fig.cap} option to write plots into the \texttt{figure} environment
automatically, and the caption is generated from data dynamically
(see Figure \ref{fig:boxplot-ex}):

% set an option first
<<eval-opts, echo=FALSE>>=
opts_knit$set(eval.after = 'fig.cap') # evaluate fig.cap after the chunk

<<boxplot-ex, fig.height=2, out.width='.8\\linewidth', fig.cap=sprintf('This is a boxplot; the median is %.2f.', median(x))>>=
par(mar=c(4,4,.1,.1))
boxplot(x, horizontal=TRUE)
@

As we know, \texttt{figure} is a float environment, so it has floated
away from the R code to the top of this page. This should not be surprising
to \LaTeX{} users.

You should be able to compile the \TeX{} document and get a PDF file
like this one: \url{https://bitbucket.org/stat/knitr/downloads/knitr-listings.pdf}.
For more information about out hooks, see \url{http://yihui.name/knitr/hooks}.

\end{document}

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass{article}
\usepackage[sc]{mathpazo}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage{geometry}
\geometry{verbose,tmargin=2.5cm,bmargin=2.5cm,lmargin=2.5cm,rmargin=2.5cm}
\setcounter{secnumdepth}{2}
\setcounter{tocdepth}{2}
\usepackage{url}
\usepackage[authoryear]{natbib}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=true,bookmarksopen=true,bookmarksopenlevel=2,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}
\hypersetup{
 pdfstartview={XYZ null null 1}}
\usepackage{breakurl}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
\providecommand{\LyX}{\texorpdfstring%
  {L\kern-.1667em\lower.25em\hbox{Y}\kern-.125emX\@}
  {LyX}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[buttonsize=1em]{animate}

\makeatother

\begin{document}

<<setup, include=FALSE, cache=FALSE>>=
## set global chunk options
opts_chunk$set(fig.path='figure/manual-', cache.path='cache/manual-', fig.align='center', fig.show='hold', par=TRUE)
## I use = but I can replace it with <-; set code/output width to be 68
options(replace.assign=TRUE, width=68)
## tune details of base graphics (http://yihui.name/knitr/hooks)
knit_hooks$set(par=function(before, options, envir){
if (before && options$fig.show!='none') par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3)
})
@


\title{knitr: A General-Purpose Tool for Dynamic Report Generation in R}


\author{Yihui Xie}

\maketitle
The original paradigm of literate programming was brought forward
mainly for software development, or specifically, to mix source code
(for computer) and documentation (for human) together. Early systems
include \href{http://www.literateprogramming.com/web.pdf}{WEB} and
\href{http://www.cs.tufts.edu/~nr/noweb/}{Noweb}; Sweave \citep{leisch2002}
was derived from the latter, but it is less focused on documenting
software, instead it is mainly used for reproducible data analysis
and generating statistical reports. The \textbf{knitr} package \citep{R-knitr}
is following the steps of Sweave. For this manual, I assume readers
have some background knowledge of Sweave to understand the technical
details; for a reference of available options, hooks and demos, see
the package homepage \url{http://yihui.name/knitr/}.


\section{Hello World}

A natural question is why to reinvent the wheel. The short answer
is that extending Sweave by hacking \textsf{SweaveDrivers.R} in the
\textbf{utils} package is a difficult job to me. Many features in
\textbf{knitr} come naturally as users would have expected. Figure
\ref{fig:cars-demo} is a simple demo of some features of \textbf{knitr}.

\begin{figure}
<<cars-demo,dev='tikz',fig.width=4,fig.height=2.8,out.width='.45\\textwidth',message=FALSE,cache=TRUE>>=
fit=lm(dist~speed,data=cars) # linear regression
par(mar=c(4, 4, 1, .1), mgp=c(2,1,0))
with(cars,plot(speed,dist,panel.last=abline(fit)))
text(10,100,'$Y = \\beta_0 + \\beta_1x + \\epsilon$')
library(ggplot2)
qplot(speed, dist, data=cars)+geom_smooth()
@

\caption{\label{fig:cars-demo}A simple demo of possible output in \textbf{knitr}:
(1) multiple plots per chunk; (2) no need to \emph{print()} objects
in \textbf{ggplot2}; (3) device size is $4\times2.8$ (inches) but
output size is adjusted to \texttt{.45\textbackslash{}textwidth} in
chunk options; (4) base graphics and \textbf{ggplot2} can sit side
by side; (5) use the \emph{tikz()} device in \textbf{tikzDevice} by
setting chunk option \texttt{dev='tikz'} (hence can write native \protect\LaTeX{}
expressions in R plots); (6) code highlighting.}
\end{figure}


I would have chosen to hide the R code if this were a real report,
but here I show the code just for the sake of demonstration. If we
type \emph{qplot()} in R, we get a plot, and the same thing happens
in \textbf{knitr}. If we draw two plots in the code, \textbf{knitr}
will show two plots and we do not need to tell it how many plots are
there in the code in advance. If we set \texttt{out.width='.49\textbackslash{}\textbackslash{}textwidth'}
in chunk options, we get it in the final output document. If we say
\texttt{fig.align='center'}, the plots are centered. That's it. Many
enhancements and new features will be introduced later. If you come
from the Sweave land, you can take a look at the page of transition
first: \url{http://yihui.name/knitr/demo/sweave/}.


\section{Design}

The flow of processing an input file is similar to Sweave, and two
major differences are that \textbf{knitr} provides more flexibility
to the users to customize the processing, and has many built-in options
such as the support to a wide range of graphics devices and cache.
Below is a brief description of the process:
\begin{enumerate}
\item \textbf{knitr} takes an input file and automatically determines an
appropriate set of \href{http://yihui.name/knitr/patterns}{patterns}
to use if they are not provided in advance (e.g. \textsf{file.Rnw}
will use \texttt{knit\_patterns\$get('rnw')});
\item a set of output \href{http://yihui.name/knitr/hooks}{hooks} will
also be set up automatically according to the filename extension (e.g.
use \LaTeX{} environments or HTML elements to wrap up R results);
\item the input file is read in and split into pieces consisting of R code
chunks and normal texts; the former will be executed one after the
other, and the latter may contain global chunk options or inline R
code;
\item for each chunk, the code is evaluated using the \textbf{evaluate}
package \citep{R-evaluate}, and the results may be filtered according
to chunk options (e.g. \texttt{echo=FALSE} will remove the R source
code)

\begin{enumerate}
\item if \texttt{cache=TRUE} for this chunk, \textbf{knitr} will first check
if there are previously cached results under the cache directory before
really evaluating the chunk; if cached results exist and this code
chunk has not been changed since last run (use MD5 sum to verify),
the cached results will be (lazy-) loaded, otherwise new cache will
be built; if a cached chunk depends on other chunks (see the \texttt{dependson}
\href{http://yihui.name/knitr/options}{option}) and any one of these
chunks has changed, this chunk must be forcibly updated (old cache
will be purged)
\item there are six types of possible output from \textbf{evaluate}, and
their classes are \texttt{character} (normal text output), \texttt{source}
(source code), \texttt{warning}, \texttt{message}, \texttt{error}
and \texttt{recordedplot}; an internal S3 generic function \emph{wrap()}
is used to deal with different types of output, using output hooks
defined in the object \texttt{knit\_hooks}
\item note plots are recorded as R objects before they are really saved
to files, so graphics devices will not be opened unless plots have
really been produced in a chunk
\item a code chunk is evaluated in a separate empty environment with the
global environment as its parent, and all the objects in this environment
after the evaluation will be saved if \texttt{cache=TRUE}
\item chunk hooks can be run before and/or after a chunk
\end{enumerate}
\item for normal texts, \textbf{knitr} will find out if there are chunk
options set among them (e.g. \texttt{\textbackslash{}SweaveOpts\{\}}),
and evaluate inline R code (e.g. \texttt{\textbackslash{}Sexpr\{\}});
the latter is involved with the \texttt{inline} hook;
\end{enumerate}
The hooks play important roles in \textbf{knitr}: this package makes
almost everything accessible to the users. Consider the following
extremely simple example which may demonstrate this freedom:

<<simple-example>>=
1+1
@

There are two parts in the final output: the source code \texttt{1
+ 1} and the output \texttt{{[}1{]} 2}; the comment characters \texttt{\#\#}
are from the default chunk option \texttt{comment}. Users may define
a hook function for the source code like this to use the \texttt{lstlisting}
environment:

<<hook-source, eval=FALSE>>=
knit_hooks$set(source = function(x, options) {
paste('\\begin{lstlisting}\n', x, '\\end{lstlisting}\n', sep = '')
})
@

Similarly we can put other types of output into other environments.
There is no need to hack at \textsf{Sweave.sty} for \textbf{knitr}
and you can put the output in any environments. What is more, the
output hooks make \textbf{knitr} ready for other types of output,
and a typical one is HTML (there are built-in hooks). The website
has provided many examples demonstrating the flexibility of the output.


\section{Features}

The \textbf{knitr} package borrowed features such as tikz graphics
and cache from \textbf{pgfSweave} and \textbf{cacheSweave} respectively,
but the implementations are different. New features like code reference
from an external R script as well as output customization are also
introduced. The feature of hook functions in Sweave is re-implemented
and hooks have new usage now. There are several other small features
which are motivated from my everyday use of Sweave. For example, a
progress bar is provided when knitting a file so we roughly know how
long we still need to wait; output from inline R code (e.g. \texttt{\textbackslash{}Sexpr\{x{[}1{]}\}})
is automatically formatted in \TeX{} math notation (like \Sexpr{123456789})
if the result is numeric, and we will not get too many digits by default.
You may check out a number of specific manuals dedicated to specific
features such as graphics in the website: \url{http://yihui.name/knitr/demos}.


\subsection{Code Decoration}

An internal syntax highlighter is used to highlight R code, and the
\textbf{formatR} package \citep{R-formatR} is used to reformat R
code (like \texttt{keep.source=FALSE} in Sweave but will also try
to retain comments). For \LaTeX{} output, the \textbf{framed} package
is used to decorate code chunks with a light gray background. If this
\LaTeX{} package is not found in the system, a version will be copied
directly from \textbf{knitr}. The prompt characters are removed by
default because they mangle the R source code in the output and make
it difficult to copy R code. The R output is masked in comments by
default based on the same rationale. It is easy to revert to the output
with prompts (set option \texttt{prompt=TRUE}), and you will quickly
realize the inconvenience to the readers if they want to copy and
run the code in the output document:

<<stupid-prompts, prompt=TRUE, comment=NA, highlight=FALSE>>=
x=rnorm(5)
x
var(x)
@

The example below shows the effect of \texttt{tidy=TRUE/FALSE}:

<<tidy-no, eval=FALSE, tidy=FALSE>>=
## option tidy=FALSE
for(k in 1:10){j=cos(sin(k)*k^2)+3;print(j-5)}
@
<<tidy-yes, eval=FALSE, tidy=TRUE>>=
## option tidy=TRUE
for(k in 1:10){j=cos(sin(k)*k^2)+3;print(j-5)}
@

Note \texttt{=} is replaced by \texttt{<-} because \texttt{options('replace.assign')}
was set to be \texttt{TRUE} in this document; see the documentation
of \emph{tidy.source()} in \textbf{formatR} for details.

Many highlighting themes can be used in \textbf{knitr}, which are
borrowed from the \textbf{highlight} package by \href{http://www.andre-simon.de/}{Andre Simon}%
\footnote{not the R package mentioned before; for a preview of these themes,
see \url{http://www.andre-simon.de/dokuwiki/doku.php?id=theme_examples}%
}; it is also possible to use themes from \url{http://www.eclipsecolorthemes.org/}
by providing a theme id to \textbf{knitr}%
\footnote{many thanks to \href{https://github.com/ramnathv}{Ramnath Vaidyanathan}
for the work on themes%
}. See \texttt{?knit\_theme} for details.


\subsection{Graphics}

Graphics is an important part of reports, and several enhancements
have been made in \textbf{knitr}. For example, grid graphics may not
need to be explicitly printed as long as the same code can produce
plots in R (in some cases, however, they have to be printed, e.g.
in a loop, because you have to do so in an R terminal).


\subsubsection{Graphical Devices}

Over a long time, a frequently requested feature for Sweave was the
support for other graphics devices, which has been implemented since
R 2.13.0. Instead of using logical options like \texttt{png} or \texttt{jpeg}
(this list can go on and on), \textbf{knitr} uses a single option
\texttt{dev} (like \texttt{grdevice} in Sweave) which has support
for more than 20 devices. For instance, \texttt{dev='png'} will use
the \emph{png()} device, and \texttt{dev='CairoJPEG'} uses the \emph{CairoJPEG()}
device in the \textbf{Cairo} package (it has to be installed first,
of course). If none of these devices is satisfactory, you can provide
the name of a customized device function, which must have been defined
before it is called.


\subsubsection{Plot Recording}

As mentioned before, all the plots in a code chunk are first recorded
as R objects and then ``replayed'' inside a graphical device to
generate plot files. The \textbf{evaluate} package will record plots
per \emph{expression} basis, in other words, the source code is split
into individual complete expressions and \textbf{evaluate} will examine
possible plot changes in snapshots after each single expression has
been evaluated. For example, the code below consists of three expressions,
out of which two are related to drawing plots, therefore \textbf{evaluate}
will produce two plots by default:

<<low-level-plots, fig.keep='all', dev='tikz', fig.width=2.5, fig.height=2.5, out.width='.3\\textwidth', cache=TRUE>>=
par(mar=c(3,3,.1,.1))
plot(1:10, ann=FALSE,las=1)
text(5,9,'mass $\\rightarrow$ energy\n$E=mc^2$')
@

This brings a significant difference with traditional tools in R for
dynamic report generation, since low-level plotting changes can also
be recorded. The option \texttt{fig.keep} controls which plots to
keep in the output; \texttt{fig.keep='all'} will keep low-level changes
as separate plots; by default (\texttt{fig.keep='high'}), \textbf{knitr}
will merge low-level plot changes into the previous high-level plot,
like most graphics devices do. This feature may be useful for teaching
R graphics step by step. Note, however, low-level plotting commands
in a single expression (a typical case is a loop) will not be recorded
accumulatively, but high-level plotting commands, regardless of where
they are, will always be recorded. For example, this chunk will only
produce 2 plots instead of 21 plots because there are 2 complete expressions:

<<low-plot-loop, eval=FALSE>>=
plot(0,0,type='n',ann=FALSE)
for(i in seq(0, 2*pi,length=20)) points(cos(i),sin(i))
@

But this will produce 20 plots as expected:

<<high-plot-loop, eval=FALSE>>=
for(i in seq(0, 2*pi,length=20)) {plot(cos(i),sin(i),xlim=c(-1,1),ylim=c(-1,1))}
@

As I showed in the beginning of this manual, it is straightforward
to let \textbf{knitr} keep all the plots in a chunk and insert them
into the output document, so we no longer need the \texttt{cat('\textbackslash{}\textbackslash{}includegraphics\{\}')}
trick.

We can discard all previous plots and keep the last one only by \texttt{fig.keep='last'},
or keep only the first plot by \texttt{fig.keep='first'}, or discard
all plots by \texttt{fig.keep='none'}.


\subsubsection{Plot Rearrangement}

The option \texttt{fig.show} can decide whether to hold all plots
while evaluating the code and ``flush'' all of them to the end of
a chunk (\texttt{fig.show='hold'}), or just insert them to the place
where they were created (by default \texttt{fig.show='asis'}). Here
is an example of \texttt{fig.show='asis'}:

<<fig-hold, fig.show='asis', dev='pdf', fig.width=6, fig.height=4, out.width='.35\\linewidth'>>=
contour(volcano)  # contour lines
filled.contour(volcano)  # fill contour plot with colors
@

Beside \texttt{hold} and \texttt{asis}, the option \texttt{fig.show}
can take a third value: \texttt{animate}, which makes it possible
to insert animations into the output document. In \LaTeX{}, the package
\textbf{animate} is used to put together image frames as an animation.
For animations to work, there must be more than one plot produced
in a chunk. The option \texttt{interval} controls the time interval
between animation frames; by default it is 1 second. Note you have
to add \texttt{\textbackslash{}usepackage\{animate\}} in the \LaTeX{}
preamble, because \textbf{knitr} will not add it automatically. Animations
in the PDF output can only be viewed in Adobe Reader.

As a simple demonstration, here is a \href{http://en.wikipedia.org/wiki/Mandelbrot_set}{Mandelbrot animation}
taken from the \textbf{animation} package \citep{R-animation}; note
the PNG device is used because PDF files are too large. You should
be able to see the animation immediately with Acrobat Reader since
it was set to play automatically:

<<animate-demo, fig.show='animate', dev='png', out.width='.45\\linewidth', interval=.5, aniopts='controls,loop,autoplay',  cache=TRUE>>=
library(animation)
demo('Mandelbrot', echo = FALSE, package = 'animation')
@


\subsubsection{Plot Size}

The \texttt{fig.width} and \texttt{fig.height} options specify the
size of plots in the graphics device, and the real size in the output
document can be different (see \texttt{out.width} and \texttt{out.height}).
When there are multiple plots per chunk, it is possible to arrange
more than one plot per line in \LaTeX{} -- just specify \texttt{out.width}
to be less than half of the current line width, e.g. \texttt{out.width='.49\textbackslash{}\textbackslash{}linewidth'}.


\subsubsection{The tikz Device}

Beside PDF, PNG and other traditional R graphical devices, \textbf{knitr}
has special support to tikz graphics via the \textbf{tikzDevice} package
\citep{R-tikzDevice}, which is similar to \textbf{pgfSweave}. If
we set the chunk option \texttt{dev='tikz'}, the \emph{tikz()} device
in \textbf{tikzDevice} will be used to save plots. Options \texttt{sanitize}
and \texttt{external} are related to the tikz device: see the documentation
of \emph{tikz()} for details. Note \texttt{external=TRUE} in \textbf{knitr}
has a different meaning with \textbf{pgfSweave} -- it means \texttt{standAlone=TRUE}
in \emph{tikz()}, and the tikz graphics output will be compiled to
PDF \emph{immediately} after it is created, so the ``externalization''
does not depend on the \textbf{tikz} package; to maintain consistency
in (font) styles, \textbf{knitr} will read the preamble of the input
document and use it in the tikz device. At the moment, I'm not sure
if this is a faithful way to externalize tikz graphics, but I have
not seen any problems so far. The assumption to make, however, is
that you declare all the styles in the preamble; \textbf{knitr} is
agnostic of \emph{local} style changes in the body of the document.

Below is an example taken from StackOverflow%
\footnote{\url{http://stackoverflow.com/q/8190087/559676}%
}; we usually have to write R code like this to obtain a math expression
$\mathrm{d}\mathbf{x}_{t}=\alpha[(\theta-\mathbf{x}_{t})\mathrm{d}t+4]\mathrm{d}B_{t}$
in R graphics:

<<math-expr-R, eval=FALSE>>=
qplot(1:10, 1:10) + opts(title = substitute(paste(d * 
    bolditalic(x)[italic(t)] == alpha * (theta - bolditalic(x)[italic(t)]) * 
    d * italic(t) + lambda * d * italic(B)[italic(t)]), list(lambda = 4))) 
@

With the tikz device, it is both straightforward and more beautiful: 

<<math-expr-tikz, dev='tikz', fig.width=5, fig.height=3, out.width='.55\\linewidth', cache=TRUE, message=FALSE>>=
library(ggplot2)
qplot(1:10, 1:10) + 
labs(title = sprintf('$\\mathrm{d}\\mathbf{x}_{t} = \\alpha[(\\theta - \\mathbf{x}_{t})\\mathrm{d}t + %d]\\mathrm{d}B_{t}$', 4))
@

The advantage of tikz graphics is the consistency of styles%
\footnote{Users are encouraged to read the vignette of \textbf{tikzDevice},
which is the most beautiful vignette I have ever seen in R packages:
\url{http://cran.r-project.org/web/packages/tikzDevice/vignettes/tikzDevice.pdf}%
}, and one disadvantage is that \LaTeX{} may not be able to handle
too large tikz files (it can run out of memory). For example, an R
plot with tens of thousands of graphical elements may fail to compile
in \LaTeX{} if we use the tikz device. In such cases, we can switch
to the PDF or PNG device, or reconsider our decision on the type of
plots, e.g., a scatter plot with millions of points is usually difficult
to read, and a contour plot or a hexagon plot showing the 2D density
can be a better alternative (they are smaller in size).

The graphics manual contains more detailed information and you can
check it out in the \href{http://yihui.name/knitr/demo/graphics/}{website}.


\subsection{Cache}

The feature of cache is not a new idea -- both \textbf{cacheSweave}
and \textbf{weaver} have implemented it based on Sweave, with the
former using \textbf{filehash} and the latter using \textsf{.RData}
images; \textbf{cacheSweave} also supports lazy-loading of objects
based on \textbf{filehash}. The \textbf{knitr} package directly uses
internal base R functions to save (\emph{tools:::makeLazyLoadDB()})
and lazy-load objects (\emph{lazyLoad()}). These functions are either
undocumented or marked as internal, but as far as I understand, they
are the tools to implement lazy-loading for packages. The \textbf{cacheSweave}
vignette has clearly explained lazy-loading, and roughly speaking,
lazy-loading means an object will not be really loaded into memory
unless it is really used somewhere. This is very useful for cache;
sometimes we read a large object and cache it, then take a subset
for analysis and this subset is also cached; in the future, the initial
large object will not be loaded into R if our computation is only
based on the object of its subset.

The paths of cache files are determined by the chunk option \texttt{cache.path};
by default all cache files are created under a directory \textsf{cache}
relative to the current working directory, and if the option value
contains a directory (e.g. \texttt{cache.path='cache/abc-'}), cache
files will be stored under that directory (automatically created if
it does not exist). The cache is invalidated and purged on any changes
to the code chunk, including both the R code and chunk options%
\footnote{One exception is the \texttt{include} option, which is not cached
because \texttt{include=TRUE/FALSE} does not affect code evaluation;
meanwhile, the value \texttt{getOption('width')} is also cached, so
if you change this option, the cache will also be invalidated (this
option affects the width of text output)%
}; this means previous cache files of this chunk are removed (filenames
are identified by the chunk label). Unlike \textbf{pgfSweave}, cache
files will never accumulate since old cache files will always be removed
in \textbf{knitr}. Unlike \textbf{weaver} or \textbf{cacheSweave},
\textbf{knitr} will try to preserve these side-effects:
\begin{enumerate}
\item printed results: meaning that any output of a code chunk will be loaded
into the output document for a cached chunk, although it is not really
evaluated. The reason is \textbf{knitr} also cache the output of a
chunk as a character string. Note this means graphics output is also
cached since it is part of the output. It has been a pain for me for
a long time to have to lose output to gain cache;
\item loaded packages: after the evaluation of each cached chunk, the list
of packages used in the current R session is written to a file under
the cache path named \textsf{\_\_packages}; next time if a cached
chunk needs to be rebuilt, these packages will be loaded first. The
reasons for caching package names are, it can be slow to load some
packages, and a package might be loaded in a previous cached chunk
which is not available to the next cached chunk when only the latter
needs to be rebuilt. Note this only applies to cached chunks, and
for uncached chunks, you must always use \emph{library()} to load
packages explicitly;
\end{enumerate}
Although \textbf{knitr} tries to keep some side-effects, there are
still other types of side-effects like setting \emph{par()} or \emph{options()}
which are not cached. Users should be aware of these special cases,
and make sure to clearly separate the code which is not meant to be
cached to other chunks which are not cached, e.g., set all global
options in the first chunk of a document and do not cache that chunk.

Sometimes a cached chunk may need to use objects from other cached
chunks, which can bring a serious problem -- if objects in previous
chunks have changed, this chunk will not be aware of the changes and
will still use old cached results, unless there is a way to detect
such changes from other chunks. There is an option called \texttt{dependson}
in \textbf{cacheSweave} which does this job. We can explicitly specify
which other chunks this chunk depends on by setting an option like
\texttt{dependson='chunkA;chunkB'} or equivalently \texttt{dependson=c('chunkA',
'chunkB')}. Each time the cache of a chunk is rebuilt, all other chunks
which depend on this chunk will lose cache, hence their cache will
be rebuilt as well.

Another way to specify the dependencies among chunks is to use the
chunk option \texttt{autodep} and the function \emph{dep\_auto()}.
This is an experimental feature borrowed from \textbf{weaver} which
frees us from setting chunk dependencies manually. The basic idea
is, if a latter chunk uses any objects created from a previous chunk,
the latter chunk is said to depend on the previous one. The function
\emph{findGlobals()} in the \textbf{codetools} package is used to
find out all global objects in a chunk, and according to its documentation,
the result is an approximation. Global objects roughly mean the ones
which are not created locally, e.g. in the expression \texttt{function()
\{y <- x\}}, \texttt{x} should be a global object, whereas \texttt{y}
is local. Meanwhile, we also need to save the list of objects created
in each cached chunk, so that we can compare them to the global objects
in latter chunks. For example, if chunk A created an object \texttt{x}
and chunk B uses this object, chunk B must depend on A, i.e. whenever
A changes, B must also be updated. When \texttt{autodep=TRUE}, \textbf{knitr}
will write out the names of objects created in a cached chunk as well
as those global objects in two files named \textsf{\_\_objects} and
\textsf{\_\_globals} respectively; later we can use the function \emph{dep\_auto()}
to analyze the object names to figure out the dependencies automatically.
See \url{http://yihui.name/knitr/demo/cache/} for examples.

Yet another way to specify dependencies is \emph{dep\_prev()}: this
is a conservative approach which sets the dependencies so that a cached
chunk will depend on all its previous chunks, i.e. whenever a previous
chunk is updated, all later chunks will be updated accordingly.


\subsection{Code Externalization}

It can be more convenient to write R code in a separate file, rather
than mixing it into a \LaTeX{} document; for example, we can run R
code successively in a pure R script from one chunk to the other without
jumping through other texts. Since I prefer using \LyX{} to write
reports, Sweave is even more inconvenient because I have to recompile
the whole document each time, even if I only want to know the results
of a single chunk. Therefore \textbf{knitr} introduced the feature
of code externalization to a separate R script. Currently the setting
is like this: the R script also uses chunk labels (marked in the form
\texttt{\#\# @knitr chunk-label} by default); if the code chunk in
the input document is empty, \textbf{knitr} will match its label with
the label in the R script to input external R code. For example, suppose
this is a code chunk labelled as \texttt{Q1} in an R script named
\textsf{homework1-xie.R} which is under the same directory as the
Rnw document:

<<ext-r-code, eval=FALSE>>=
## @knitr Q1
gcd = function(m, n) {
    while ((r <- m %% n) != 0) {
       m = n; n = r
    }
    n
}
@

In the Rnw document, we can first read the script using the function
\emph{read\_chunk()}:

<<read-chunk, eval=FALSE>>=
read_chunk('homework1-xie.R')
@

This is usually done in an early chunk, and we can use the chunk \texttt{Q1}
later in the Rnw document:

<<use-ext-chunk, echo=FALSE, comment=NA>>=
cat('<<Q1, echo=TRUE, tidy=TRUE>>=','@',sep='\n')
@

Different documents can read the same R script, so the R code can
be reusable across different input documents.


\subsection{Evaluation of Chunk Options\label{sub:conditional}}

By default \textbf{knitr} uses a new syntax to parse chunk options:
it treats them as function arguments instead of a text string to be
split to obtain option values. This gives the user much more power
than the old syntax; we can pass arbitrary R objects to chunk options
besides simple ones like \texttt{TRUE}/\texttt{FALSE}, numbers and
character strings. The page \url{http://yihui.name/knitr/demo/sweave/}
has given two examples to show the advantages of the new syntax. Here
we show yet another useful application.

Before \textbf{knitr} 0.3, there was a feature named ``conditional
evaluation''%
\footnote{request from \url{https://plus.google.com/u/0/116405544829727492615/posts/43WrRUffjzK}%
}. The idea is, instead of setting chunk options \texttt{eval} and
\texttt{echo} to be \texttt{TRUE} or \texttt{FALSE} (constants), their
values can be controlled by global variables in the current R session.
This enables \textbf{knitr} to conditionally evaluate code chunks
according to variables. For example, here we assign \texttt{TRUE}
to a variable \texttt{dothis}:

<<cond-variable>>=
dothis=TRUE
@

In the next chunk, we set chunk options \texttt{eval=dothis} and \texttt{echo=!dothis},
both are valid R expressions since the variable \texttt{dothis} exists.
As we can see, the source code is hidden, but it was indeed evaluated:

<<cond-out1, eval=dothis, echo=!dothis>>=
print('you cannot see my source because !dothis is FALSE')
@

Then we set \texttt{eval=dothis} and \texttt{echo=dothis} for another
chunk:

<<cond-out2,eval=dothis,echo=dothis>>=
dothis
@

If we change the value of \texttt{dothis} to \texttt{FALSE}, neither
of the above chunks will be evaluated any more. Therefore we can control
many chunks with a single variable, and present results selectively.

This old feature requires \textbf{knitr} to treat \texttt{eval} and
\texttt{echo} specially, and we can easily see that it is no longer
necessary with the new syntax: \texttt{eval=dothis} will tell R to
find the variable \texttt{dothis} automatically just like we call
a function \texttt{foobar(eval = dothis)}. What is more, all options
will be evaluated as R expressions unless they are already constants
which do not need to be evaluated, so this old feature has been generalized
to all other options naturally.


\subsection{Customization}

The \textbf{knitr} package is ready for customization. Both the patterns
and hooks can be customized; see the package website for details.
Here I show an example on how to save \textbf{rgl} plots \citep{R-rgl}
using a customized hook function. First we define a hook named \texttt{rgl}
using the function \emph{hook\_rgl()} in \textbf{knitr}:

<<rgl-demo>>=
knit_hooks$set(rgl = hook_rgl)
head(hook_rgl)  # the hook function is defined as this
@

Then we only have to set the chunk option \texttt{rgl=TRUE}:

<<fancy-rgl, rgl=TRUE, dev='png', fig.width=5, fig.height=5, out.width='2in', message=FALSE, cache=TRUE>>=
library(rgl)
demo('bivar', package='rgl', echo=FALSE) 
par3d(zoom=.7)
@

Due to the flexibility of output hooks, \textbf{knitr} supports several
different output formats. The implementation is fairly easy, e.g.,
for \LaTeX{} we put R output in \texttt{verbatim} environments, and
in HTML, it is only a matter of putting output in \texttt{div} layers.
These are simply character string operations. Many demos in \url{http://yihui.name/knitr/demos}
show this idea clearly. This manual did not cover all the features
of \textbf{knitr}, and users are encouraged to thumb through the website
to know more possible features.


\section{Editors}

You can use any text editors to write the source documents, but some
have built-in support for \textbf{knitr}. Both RStudio (\url{http://www.rstudio.org})
and \LyX{} (\url{http://www.lyx.org}) have full support for \textbf{knitr},
and you can compile the document to PDF with just one click. See \url{http://yihui.name/knitr/demo/rstudio/}
and \url{http://yihui.name/knitr/demo/lyx/} respectively. It is also
possible to support other editors like \href{http://yihui.name/knitr/demo/eclipse/}{Eclipse},
\href{http://yihui.name/knitr/demo/editors/}{Texmaker and WinEdt};
see the demo list in the website for configuration instructions.


\section*{About This Document}

This manual was written in \LyX{} and compiled with \textbf{knitr}
(version \Sexpr{packageVersion('knitr')}). The \LyX{} source and
the Rnw document exported from \LyX{} can be found under these directories:

<<source-location, eval=FALSE>>=
system.file('examples', 'knitr-manual.lyx', package='knitr')  # lyx source
system.file('examples', 'knitr-manual.Rnw', package='knitr')  # Rnw source
@

You can use the function \emph{knit()} to knit the Rnw document (remember
to put the two \textsf{.bib} files under the same directory), and
you need to make sure all the R packages used in this document are
installed:

<<required-packages, eval=FALSE>>=
install.packages(c('animation', 'rgl', 'tikzDevice', 'ggplot2'))
@

Feedback and comments on this manual and the package are always welcome.
Bug reports and feature requests can be sent to \url{https://github.com/yihui/knitr/issues},
and questions can be delivered to the \href{mailto:knitr@googlegroups.com}{mailing list}
\url{https://groups.google.com/group/knitr}.

% when knitr is updated, this chunk will be updated; why?
<<auto-bib, version=packageVersion('knitr'), echo=FALSE, cache=TRUE, message=FALSE, warning=FALSE>>=
# write all packages in the current session to a bib file
write_bib(c(.packages(), 'evaluate', 'formatR'), file = 'knitr-packages.bib')
@

\bibliographystyle{jss}
\bibliography{knitr-manual,knitr-packages}


\end{document}

%% LyX 2.0.5.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[nohyper,justified]{tufte-handout}
\usepackage[T1]{fontenc}
\usepackage{booktabs}
\usepackage{url}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.

\title{Customizing Syntax Highlighting Themes}
\author{Yihui Xie \& Ramnath Vaidyanathan}
\newcommand{\noun}[1]{\textsc{#1}}
%% Because html converters don't know tabularnewline
\providecommand{\tabularnewline}{\\}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.8}
\renewcommand{\bottomfraction}{0.8}
\renewcommand{\floatpagefraction}{0.75}

\usepackage[breaklinks=true,pdfstartview=FitH]{hyperref}

\makeatother

\begin{document}

<<setup, include=FALSE, cache=FALSE>>=
opts_chunk$set(fig.path='figure/theme-', cache.path='cache/theme-', cache=TRUE)
options(replace.assign=TRUE,width=78)
knit_hooks$set(par=function(before, options, envir){if (before) par(mar=c(4,4,.1,.1),cex.lab=.95,cex.axis=.9,mgp=c(2,.7,0),tcl=-.3)})
@
<<solarized-theme, cache=FALSE, echo=FALSE>>=
thm = knit_theme$get("solarized-dark")
knit_theme$set(thm)
@

\maketitle
\begin{abstract}
This manual%
\footnote{source code at \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-themes.Rnw}%
} shows how to customize syntax highlighting of source code using themes.
It walks the user through the basics of syntax highlighting in \textbf{knitr},
the use of built-in themes as well as those from \url{http://www.eclipsecolorthemes.org/}.
\end{abstract}
The \textbf{knitr} package uses the \textbf{highlight} package by
Romain Francois to highlight source code in a document. In short,
the \textbf{highlight} package parses the source code using the \noun{bison}
parser, tokenizes it into grammar symbols, and formats their appearance
based on a CSS style file. 


\section{Usage}

We can use the object \texttt{knit\_theme} to set / get a theme. See
\texttt{?knit\_theme} for the usage. For example, we set the theme
of this document to be \texttt{solarized-dark}:

<<solarized-theme>>=
@


\section{Built-in Themes}

\begin{margintable}
<<eclipse-css, comment=NA, echo=FALSE>>=
cat(c(readLines('../themes/edit-eclipse.css', n=30),'...'),sep='\n')
@
\end{margintable}


The listing on the right shows the CSS file for one of the themes,
\texttt{edit-eclipse}, which was adapted from Andre Simon's excellent
highlighter%
\footnote{\url{http://www.andre-simon.de/}%
}. The \textbf{knitr} package comes pre-loaded with a number of themes
based on this highlighter. Here is list of all available code themes:

<<all-themes>>=
knit_theme$get()
@

Shown below is how the \texttt{solarized-dark} theme looks like when
applied to R code:

<<demo-code, eval = FALSE>>=
# SCRAPE THE DATA FROM WEB 

base_url = "http://www.mlsoccer.com/stats/%s/reg"
years    = 1996:2010
options(width = 40)

#' Function to save data for each year
save_data = function(y){
  url  = sprintf(base_url, y)
  tab  = XML::readHTMLTable(url, header = FALSE, stringsAsFactors = FALSE);
  pos  = max(grep("United", tab));
  tab  = tab[[pos]];
  tab$year   = y;
  return(tab)            
}
	
team.list  = plyr::llply(years, save_data);
mls        = reshape::merge_recurse(team.list);
@


\section{Eclipse Color Themes}

A rich repository of syntax highlighting themes for use with Eclipse
IDE is available at \url{http://www.eclipsecolorthemes.org/}. In
order to use one of these themes with \textbf{knitr}, it needs to
be converted into a CSS file that can be parsed by \textbf{highlight}.
This is achieved by mapping language tokens across the two formats
(Table \ref{mar:Token-Mapping}) to create a CSS file.

\begin{margintable}
\renewcommand\arraystretch{1.3}

\caption{Token mapping\label{mar:Token-Mapping}}


\noindent %
\begin{tabular}{ll}
\toprule 
\textbf{Highlight (CSS)} & \textbf{Eclipse (XML)}\tabularnewline
\midrule
background & background\tabularnewline
number & number\tabularnewline
functioncall & method\tabularnewline
string & string\tabularnewline
keyword & keyword\tabularnewline
argument & localVariable\tabularnewline
comment & singleLineComment\tabularnewline
roxygencomment & javadoc\tabularnewline
formalargs & localVariable\tabularnewline
eqformalargs & foreground\tabularnewline
assignement & foreground\tabularnewline
package & class\tabularnewline
slot & foreground\tabularnewline
symbol & localVariableDeclaration\tabularnewline
prompt & foreground\tabularnewline
\bottomrule
\end{tabular}
\end{margintable}


If you want to use an eclipse theme that is not shipped with \textbf{knitr},
you can use the utility function \emph{eclipse\_theme()} to automatically
download, map and save the theme as a CSS file. For example, if you
want to use the theme Oughsumm%
\footnote{\url{http://www.eclipsecolorthemes.org/?view=theme&id=1885} (theme
id is 1885)%
}, you can use the code below in your first chunk:

<<eclipse-theme, eval=FALSE>>=
css = eclipse_theme(1885)
thm = knit_theme$get(css)
knit_theme$set(thm)
@

Note this requires Internet connection, and we can put the code to
download the theme in a cached chunk, so it will be downloaded only
once, then we set the theme in an uncached chunk using the parsed
theme object. For example, download the theme:

%% please use cache=TRUE and echo=FALSE for this chunk
<<download-theme, eval=FALSE, message=FALSE>>=
(css = eclipse_theme(1885))
thm = knit_theme$get(css)
@

Then apply the theme:

%% cache=FALSE for this chunk
<<apply-theme, eval=FALSE>>=
knit_theme$set(thm)
@

Even we do not have Internet connection the next time, we can still
use the theme.


\section{Misc}

One thing to consider is the foreground color of plots when we use
dark themes; we need to make it lighter, otherwise the graphical elements
will be difficult to be read. We can access the foreground color of
the theme in the list returned by \texttt{knit\_theme\$get(theme)},
e.g., for this document:

<<fg-color>>=
## the object thm is from the first chunk
thm$foreground  # foreground color
thm$background  # background color
@

When we make plots, we may use these colors, e.g.

<<plot-color, fig.width=6, fig.height=4, out.width='.8\\linewidth', par=TRUE>>=
## can design a chunk hook to set foreground color automatically
fgcolor=thm$foreground
par(fg=fgcolor, col.axis=fgcolor, col.lab=fgcolor)
plot(rnorm(100),pch=19)
@

Of course, we do not need to worry about these colors when we use
a white background for the plots.

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks-extra.R
\name{hook_pdfcrop}
\alias{hook_pdfcrop}
\alias{hook_optipng}
\alias{hook_pngquant}
\alias{hook_mogrify}
\alias{hook_plot_custom}
\alias{hook_purl}
\title{Built-in chunk hooks to extend knitr}
\usage{
hook_pdfcrop(before, options, envir)

hook_optipng(before, options, envir)

hook_pngquant(before, options, envir)

hook_mogrify(before, options, envir)

hook_plot_custom(before, options, envir)

hook_purl(before, options, envir)
}
\arguments{
\item{before, options, envir}{See \emph{References} below.}
}
\description{
Hook functions are called when the corresponding chunk options are not
\code{NULL} to do additional jobs beside the R code in chunks. This package
provides a few useful hooks, which can also serve as examples of how to
define chunk hooks in \pkg{knitr}.
}
\details{
The function \code{hook_pdfcrop()} can use the program \command{pdfcrop} to
crop the extra white margin when the plot format is PDF to make better use of
the space in the output document, otherwise we often have to struggle with
\code{\link[graphics]{par}} to set appropriate margins. Note
\command{pdfcrop} often comes with a LaTeX distribution such as MiKTeX or
TeXLive, and you may not need to install it separately (use
\code{Sys.which('pdfcrop')} to check it; if it not empty, you are able to use
it). Similarly, when the plot format is not PDF (e.g. PNG), the \pkg{magick}
package is used to crop the plot.

The function \code{hook_optipng()} calls the program \command{optipng} to
optimize PNG images. Note the chunk option \code{optipng} can be used to
provide additional parameters to the program \command{optipng}, e.g.
\code{optipng = '-o7'}.

The function \code{hook_pngquant()} calls the program \command{pngquant} to
optimize PNG images. Note the chunk option \code{pngquant} can be used to
provide additional parameters to the program \command{pngquant}, e.g.
\code{pngquant = '--speed=1 --quality=0-50'}.

The function \code{hook_mogrify()} calls the program \command{mogrify}.  Note
the chunk option \code{mogrify} can be used to provide additional parameters
to the program \command{mogrify} (with default \code{-trim} to trim PNG
files).

When the plots are not recordable via \code{\link[grDevices]{recordPlot}} and
we save the plots to files manually via other functions (e.g. \pkg{rgl}
plots), we can use the chunk hook \code{hook_plot_custom} to help write code
for graphics output into the output document.

The hook \code{hook_purl()} can be used to write the code chunks to an R
script. It is an alternative approach to \code{\link{purl}}, and can be more
reliable when the code chunks depend on the execution of them (e.g.
\code{\link{read_chunk}()}, or \code{\link{opts_chunk}$set(eval = FALSE)}).
To enable this hook, it is recommended to associate it with the chunk option
\code{purl}, i.e. \code{knit_hooks$set(purl = hook_purl)}. When this hook is
enabled, an R script will be written while the input document is being
\code{\link{knit}}. Currently the code chunks that are not R code or have the
chunk option \code{purl=FALSE} are ignored. Please note when the cache is
turned on (the chunk option \code{cache = TRUE}), no chunk hooks will be
executed, hence \code{hook_purl()} will not work, either. To solve this
problem, we need \code{cache = 2} instead of \code{TRUE} (see
\url{https://yihui.org/knitr/demo/cache/} for the meaning of \code{cache =
2}).
}
\note{
The two hook functions \code{hook_rgl()} and \code{hook_webgl()} were
  moved from \pkg{knitr} to the \pkg{rgl} package (>= v0.95.1247) after
  \pkg{knitr} v1.10.5, and you can \code{library(rgl)} to get them.
}
\examples{
if (require("rgl") && exists("hook_rgl")) knit_hooks$set(rgl = hook_rgl)
# then in code chunks, use the option rgl=TRUE
}
\references{
\url{https://yihui.org/knitr/hooks/#chunk_hooks}
}
\seealso{
\code{\link[rgl]{rgl.snapshot}}, \code{\link[rgl]{rgl.postscript}},
  \code{\link[rgl]{hook_rgl}}, \code{\link[rgl]{hook_webgl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parser.R
\docType{data}
\name{knit_code}
\alias{knit_code}
\title{The code manager to manage code in all chunks}
\format{An object of class \code{list} of length 6.}
\usage{
knit_code
}
\description{
This object provides methods to manage code (as character vectors) in all
chunks in \pkg{knitr} source documents. For example,
\code{knitr::knit_code$get()} returns a named list of all code chunks (the
names are chunk labels), and \code{knitr::knit_code$get('foo')} returns the
character vector of the code in the chunk with the label \code{foo}.
}
\note{
The methods on this object include the \code{set()} method (i.e., you
  could do something like \code{knitr::knit_code$set(foo = "'my precious new
  code'")}), but we recommend that you do not use this method to modify the
  content of code chunks, unless you are
  \href{https://emitanaka.rbind.io/post/knitr-knitr-code/}{as creative as Emi
  Tanaka} and know what you are doing.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\docType{data}
\name{opts_template}
\alias{opts_template}
\title{Template for creating reusable chunk options}
\format{An object of class \code{list} of length 6.}
\usage{
opts_template
}
\description{
Creates a template binding a label to a set of chunk options. Every chunk
that references the template label will have the specificed set of options
applied to it.
}
\examples{
opts_template$set(myfigures = list(fig.height = 4, fig.width = 4))
# later you can reuse these chunk options by 'opts.label', e.g.

# <<foo, opts.label='myfigures'>>=

# the above is equivalent to <<foo, fig.height=4, fig.width=4>>=
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern.R
\name{pat_rnw}
\alias{pat_rnw}
\alias{pat_brew}
\alias{pat_tex}
\alias{pat_html}
\alias{pat_md}
\alias{pat_rst}
\alias{pat_asciidoc}
\alias{pat_textile}
\title{Set regular expressions to read input documents}
\usage{
pat_rnw()

pat_brew()

pat_tex()

pat_html()

pat_md()

pat_rst()

pat_asciidoc()

pat_textile()
}
\value{
The patterns object \code{\link{knit_patterns}} is modified as a side
  effect.
}
\description{
These are convenience functions to set pre-defined pattern lists (the syntax
to read input documents). The function names are built from corresponding
file extensions, e.g. \code{pat_rnw()} can set the Sweave syntax to read Rnw
documents.
}
\examples{
# see how knit_patterns is modified
knit_patterns$get()
pat_rnw()
knit_patterns$get()

knit_patterns$restore()  # empty the list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/template.R
\name{stitch}
\alias{stitch}
\alias{stitch_rhtml}
\alias{stitch_rmd}
\title{Automatically create a report based on an R script and a template}
\usage{
stitch(
  script,
  template = system.file("misc", "knitr-template.Rnw", package = "knitr"),
  output = NULL,
  text = NULL,
  envir = parent.frame()
)

stitch_rhtml(..., envir = parent.frame())

stitch_rmd(..., envir = parent.frame())
}
\arguments{
\item{script}{Path to the R script.}

\item{template}{Path of the template to use. By default, the Rnw template in
this package; there is also an HTML template in \pkg{knitr}.}

\item{output}{Output filename, passed to \code{\link{knit}}). By default,
the base filename of the script is used.}

\item{text}{A character vector. This is an alternative way to provide the input
file.}

\item{envir}{Environment in which code chunks are to be evaluated,
for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
\code{\link{globalenv}()}).}

\item{...}{Arguments passed to \code{stitch()}.}
}
\value{
path of the output document
}
\description{
This is a convenience function for small-scale automatic reporting based on
an R script and a template. The default template is an Rnw file (LaTeX);
\code{stitch_rhtml()} and \code{stitch_rmd()} are wrappers on top of
\code{stitch()} using the R HTML and R Markdown templates respectively.
}
\details{
The first two lines of the R script can contain the title and author of the
report in comments of the form \samp{## title:} and \samp{## author:}. The
template must have a token \samp{\%sCHUNK_LABEL_HERE}, which will be used to
input all the R code from the script. See the examples below.

The R script may contain chunk headers of the form \samp{## ---- label,
opt1=val1, opt2=val2}, which will be copied to the template; if no chunk
headers are found, the whole R script will be inserted into the template as
one code chunk.
}
\examples{
s = system.file("misc", "stitch-test.R", package = "knitr")
if (interactive()) stitch(s)  # compile to PDF

# HTML report
stitch(s, system.file("misc", "knitr-template.Rhtml", package = "knitr"))

# or convert markdown to HTML
stitch(s, system.file("misc", "knitr-template.Rmd", package = "knitr"))

unlink(c("stitch-test.html", "stitch-test.md", "figure"), recursive = TRUE)
}
\seealso{
\code{\link{spin}} (turn a specially formatted R script to a report)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{load_cache}
\alias{load_cache}
\title{Load the cache database of a code chunk}
\usage{
load_cache(
  label,
  object,
  notfound = "NOT AVAILABLE",
  path = opts_chunk$get("cache.path"),
  lazy = TRUE
)
}
\arguments{
\item{label}{The chunk label of the code chunk that has a cache database.}

\item{object}{The name of the object to be fetched from the database. If it is
missing, \code{NULL} is returned).}

\item{notfound}{A value to use when the \code{object} cannot be found.}

\item{path}{Path of the cache database (normally set in the global chunk
option \code{cache.path}).}

\item{lazy}{Whether to \code{\link{lazyLoad}} the cache database (depending
on the chunk option \code{cache.lazy = TRUE} or \code{FALSE} of that code
chunk).}
}
\value{
Invisible \code{NULL} when \code{object} is not specified (the cache
  database will be loaded as a side effect), otherwise the value of the
  object if found.
}
\description{
If a code chunk has turned on the chunk option \code{cache = TRUE}, a cache
database will be established after the document is compiled. You can use this
function to manually load the database anywhere in the document (even before
the code chunk). This makes it possible to use objects created later in the
document earlier, e.g. in an inline R expression before the cached code
chunk, which is normally not possible because \pkg{knitr} compiles the
document in a linear fashion, and objects created later cannot be used before
they are created.
}
\note{
Apparently this function loads the value of the object from the
  \emph{previous} run of the document, which may be problematic when the
  value of the object becomes different the next time the document is
  compiled. Normally you must compile the document twice to make sure the
  cache database is created, and the object can be read from it. Please use
  this function with caution.
}
\references{
See the example #114 at
  \url{https://github.com/yihui/knitr-examples}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-conversion.R
\name{knit_watch}
\alias{knit_watch}
\title{Watch an input file continuously and knit it when it is updated}
\usage{
knit_watch(input, compile = knit, interval = 1, ...)
}
\arguments{
\item{input}{An input file path, or a character vector of mutiple input file paths.}

\item{compile}{A function to compile the \code{input} file. This could be e.g.
\code{\link{knit}} or \code{\link{knit2pdf}}, depending on the input file
and the output you want.}

\item{interval}{A time interval to pause in each cycle of the infinite loop.}

\item{...}{Other arguments to be passed to the \code{compile} function.}
}
\description{
Check the modification time of an input file continously in an infinite loop.
Whenever the time indicates the file has been modified, call a function to
recompile the input file.
}
\details{
This is actually a general function not necessarily restricted to
applications in \pkg{knitr}. You may specify any \code{compile} function to
process the \code{input} file. To stop the infinite loop, press the
\samp{Escape} key or \samp{Ctrl + C} (depending on your editing environment
and operating system).
}
\examples{
# knit_watch('foo.Rnw', knit2pdf)

# knit_watch('foo.Rmd', rmarkdown::render)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-vignettes.R
\name{knit_filter}
\alias{knit_filter}
\title{Spell check filter for source documents}
\usage{
knit_filter(ifile, encoding = "UTF-8")
}
\arguments{
\item{ifile}{Filename of the source document.}

\item{encoding}{Ignored (the file \code{ifile} must be encoded in UTF-8).}
}
\value{
A character vector of the file content, excluding code chunks and
  inline expressions.
}
\description{
When performing spell checking on source documents, we may need to skip R
code chunks and inline R expressions, because many R functions and symbols
are likely to be identified as typos. This function is designed for the
\code{filter} argument of \code{\link{aspell}()} to filter out code chunks
and inline expressions.
}
\examples{
library(knitr)
knitr_example = function(...) system.file("examples", ..., package = "knitr")
\donttest{
if (Sys.which("aspell") != "") {
    # -t means the TeX mode
    utils::aspell(knitr_example("knitr-minimal.Rnw"), knit_filter, control = "-t")
    
    # -H is the HTML mode
    utils::aspell(knitr_example("knitr-minimal.Rmd"), knit_filter, control = "-H -t")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output.R
\name{knit}
\alias{knit}
\alias{purl}
\title{Knit a document}
\usage{
knit(
  input,
  output = NULL,
  tangle = FALSE,
  text = NULL,
  quiet = FALSE,
  envir = parent.frame(),
  encoding = "UTF-8"
)

purl(..., documentation = 1L)
}
\arguments{
\item{input}{Path to the input file.}

\item{output}{Path to the output file for \code{knit()}. If \code{NULL}, this
function will try to guess a default, which will be under the current working
directory.}

\item{tangle}{Boolean; whether to tangle the R code from the input file (like
\code{\link[utils]{Stangle}}).}

\item{text}{A character vector. This is an alternative way to provide the input
file.}

\item{quiet}{Boolean; suppress the progress bar and messages?}

\item{envir}{Environment in which code chunks are to be evaluated,
for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
\code{\link{globalenv}()}).}

\item{encoding}{Encoding of the input file; always assumed to be UTF-8 (i.e.,
this argument is effectively ignored).}

\item{...}{arguments passed to \code{\link{knit}()} from \code{purl()}}

\item{documentation}{An integer specifying the level of documentation to add to
the tangled script. \code{0} means to output pure code, discarding all text chunks);
\code{1} (the default) means to add the chunk headers to the code; \code{2} means to
add all text chunks to code as roxygen comments.}
}
\value{
The compiled document is written into the output file, and the path
  of the output file is returned. If the \code{text} argument is not
  \code{NULL}, the compiled output is returned as a character vector. In
  other words, if you provide a file input, you get an output filename; if
  you provide a character vector input, you get a character vector output.
}
\description{
This function takes an input file, extracts the R code in it according to a
list of patterns, evaluates the code and writes the output in another file.
It can also tangle R source code from the input document (\code{purl()} is a
wrapper to \code{knit(..., tangle = TRUE)}). The \code{knitr.purl.inline}
option can be used to also tangle the code of inline expressions (disabled
by default).
}
\details{
For most of the time, it is not necessary to set any options outside the
input document; in other words, a single call like
\code{knit('my_input.Rnw')} is usually enough. This function will try to
determine many internal settings automatically. For the sake of
reproducibility, it is better practice to include the options inside the
input document (to be self-contained), instead of setting them before
knitting the document.

First the filename of the output document is determined in this way:
\file{foo.Rnw} generates \file{foo.tex}, and other filename extensions like
\file{.Rtex}, \file{.Rhtml} (\file{.Rhtm}) and \file{.Rmd}
(\file{.Rmarkdown}) will generate \file{.tex}, \file{.html} and \file{.md}
respectively. For other types of files, if the filename contains
\samp{_knit_}, this part will be removed in the output file, e.g.,
\file{foo_knit_.html} creates the output \file{foo.html}; if \samp{_knit_} is
not found in the filename, \file{foo.ext} will produce \file{foo.txt} if
\code{ext} is not \code{txt}, otherwise the output is \file{foo-out.txt}. If
\code{tangle = TRUE}, \file{foo.ext} generates an R script \file{foo.R}.

We need a set of syntax to identify special markups for R code chunks and R
options, etc. The syntax is defined in a pattern list. All built-in pattern
lists can be found in \code{all_patterns} (call it \code{apat}). First
\pkg{knitr} will try to decide the pattern list based on the filename
extension of the input document, e.g. \samp{Rnw} files use the list
\code{apat$rnw}, \samp{tex} uses the list \code{apat$tex}, \samp{brew} uses
\code{apat$brew} and HTML files use \code{apat$html}; for unkown extensions,
the content of the input document is matched against all pattern lists to
automatically determine which pattern list is being used. You can also manually
set the pattern list using the \code{\link{knit_patterns}} object or the
\code{\link{pat_rnw}} series functions in advance and \pkg{knitr} will
respect the setting.

According to the output format (\code{opts_knit$get('out.format')}), a set of
output hooks will be set to mark up results from R (see
\code{\link{render_latex}}). The output format can be LaTeX, Sweave and HTML,
etc. The output hooks decide how to mark up the results (you can customize
the hooks).

The name \code{knit} comes from its counterpart \samp{weave} (as in Sweave),
and the name \code{purl} (as \samp{tangle} in Stangle) comes from a knitting
method `knit one, purl one'.

If the input document has child documents, they will also be compiled
recursively. See \code{\link{knit_child}}.

See the package website and manuals in the references to know more about
\pkg{knitr}, including the full documentation of chunk options and demos,
etc.
}
\note{
The working directory when evaluating R code chunks is the directory of
  the input document by default, so if the R code involves external files
  (like \code{read.table()}), it is better to put these files under the same
  directory of the input document so that we can use relative paths. However,
  it is possible to change this directory with the package option
  \code{\link{opts_knit}$set(root.dir = ...)} so all paths in code chunks are
  relative to this \code{root.dir}. It is not recommended to change the
  working directory via \code{\link{setwd}()} in a code chunk, because it may
  lead to terrible consequences (e.g. figure and cache files may be written to
  wrong places). If you do use \code{setwd()}, please note that \pkg{knitr}
  will always restore the working directory to the original one. Whenever you
  feel confused, print \code{getwd()} in a code chunk to see what the working
  directory really is.

  If the \code{output} argument is a file path, it is strongly recommended to
  be in the current working directory (e.g. \file{foo.tex} instead of
  \file{somewhere/foo.tex}), especially when the output has external
  dependencies such as figure files. If you want to write the output to a
  different directory, it is recommended to set the working directory to
  that directory before you knit a document. For example, if the source
  document is \file{foo.Rmd} and the expected output is \file{out/foo.md},
  you can write \code{setwd('out/'); knit('../foo.Rmd')} instead of
  \code{knit('foo.Rmd', 'out/foo.md')}.

  N.B. There is no guarantee that the R script generated by \code{purl()} can
  reproduce the computation done in \code{knit()}. The \code{knit()} process
  can be fairly complicated (special values for chunk options, custom chunk
  hooks, computing engines besides R, and the \code{envir} argument, etc). If
  you want to reproduce the computation in a report generated by
  \code{knit()}, be sure to use \code{knit()}, instead of merely executing
  the R script generated by \code{purl()}. This seems to be obvious, but some
  people \href{http://bit.ly/SnLi6h}{just do not get it}.
}
\examples{
library(knitr)
(f = system.file("examples", "knitr-minimal.Rnw", package = "knitr"))
knit(f)  # compile to tex

purl(f)  # tangle R code
purl(f, documentation = 0)  # extract R code only
purl(f, documentation = 2)  # also include documentation

unlink(c("knitr-minimal.tex", "knitr-minimal.R", "figure"), recursive = TRUE)
}
\references{
Package homepage: \url{https://yihui.org/knitr/}. The \pkg{knitr}
  \href{https://yihui.org/knitr/demo/manual/}{main manual}: and
  \href{https://yihui.org/knitr/demo/graphics/}{graphics manual}.

  See \code{citation('knitr')} for the citation information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-conversion.R
\name{knit2pandoc}
\alias{knit2pandoc}
\title{Convert various input files to various output files using \code{knit()} and
Pandoc}
\usage{
knit2pandoc(
  input,
  output = NULL,
  tangle = FALSE,
  text = NULL,
  quiet = FALSE,
  envir = parent.frame(),
  to = "html",
  pandoc_wrapper = NULL,
  ...,
  encoding = "UTF-8"
)
}
\arguments{
\item{input}{Path to the input file.}

\item{output}{Path to the output file for \code{knit()}. If \code{NULL}, this
function will try to guess a default, which will be under the current working
directory.}

\item{tangle}{Boolean; whether to tangle the R code from the input file (like
\code{\link[utils]{Stangle}}).}

\item{text}{A character vector. This is an alternative way to provide the input
file.}

\item{quiet}{Boolean; suppress the progress bar and messages?}

\item{envir}{Environment in which code chunks are to be evaluated,
for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
\code{\link{globalenv}()}).}

\item{to}{Character string giving the Pandoc output format to use.}

\item{pandoc_wrapper}{An R function used to call Pandoc; by default, if
\pkg{rmarkdown} is installed, this uses
\code{rmarkdown::\link[rmarkdown]{pandoc_convert}()}, otherwise it
\code{\link{pandoc}()}.}

\item{...}{Options to be passed to the \code{pandoc_wrapper} function.}

\item{encoding}{Ignored (always assumes UTF-8).}
}
\value{
Returns the output of the \code{pandoc_wrapper} function.
}
\description{
Knits the input file and compiles to an output format using Pandoc.
}
\author{
Trevor L. Davis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\docType{data}
\name{opts_hooks}
\alias{opts_hooks}
\title{Hooks for code chunk options}
\format{An object of class \code{list} of length 6.}
\usage{
opts_hooks
}
\description{
Like \code{\link{knit_hooks}}, this object can be used to set hook functions
to manipulate chunk options.
}
\details{
For every code chunk, if the chunk option named, say, \code{FOO}, is not
\code{NULL}, and a hook function with the same name has been set via
\code{opts_hooks$set(FOO = function(options) { options })} (you can manipuate
the \code{options} argument in the function and return it), the hook function
will be called to update the chunk options.
}
\examples{
# make sure the figure width is no smaller than fig.height
opts_hooks$set(fig.width = function(options) {
    if (options$fig.width < options$fig.height) {
        options$fig.width = options$fig.height
    }
    options
})
# remove all hooks
opts_hooks$restore()
}
\references{
\url{https://yihui.org/knitr/hooks/}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-rd2html.R
\name{knit_rd}
\alias{knit_rd}
\alias{knit_rd_all}
\title{Knit package documentation}
\usage{
knit_rd(pkg, links = tools::findHTMLlinks(), frame = TRUE)

knit_rd_all()
}
\arguments{
\item{pkg}{Package name.}

\item{links}{A character vector of links to be passed to
\code{\link[tools]{Rd2HTML}}.}

\item{frame}{Boolean: whether to put a navigation frame on the left of the
index page.}
}
\value{
All HTML pages corresponding to topics in the package are written
  under the current working directory. An \file{index.html} is also written
  as a table of content.
}
\description{
Run examples in a package and insert output into the examples code;
\code{knit_rd_all()} is a wrapper around \code{knit_rd()} to build static
HTML help pages for all packages under the \file{html} directory of them.
}
\note{
Ideally the html pages should be put under the \file{html} directory of
  an installed package which can be found via \code{system.file('html',
  package = 'your_package_name')}, otherwise some links may not work (e.g.
  the link to the DESCRITION file).
}
\examples{
library(knitr)
\dontrun{

knit_rd("maps")
knit_rd("rpart")
setwd(system.file("html", package = "ggplot2"))
knit_rd("ggplot2")  # time-consuming!

knit_rd_all()  # this may take really long time if you have many packages installed
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{clean_cache}
\alias{clean_cache}
\title{Clean cache files that are probably no longer needed}
\usage{
clean_cache(clean = FALSE, path = opts_chunk$get("cache.path"))
}
\arguments{
\item{clean}{Boolean; whether to remove the files.}

\item{path}{Path to the cache.}
}
\description{
If you remove or rename some cached code chunks, their original cache files
will not be automatically cleaned. You can use this function to identify
these possible files, and clean them if you are sure they are no longer
needed.
}
\note{
The identification is not guaranteed to be correct, especially when
  multiple documents share the same cache directory. You are recommended to
  call \code{clean_cache(FALSE)} and carefully check the list of files (if
  any) before you really delete them (\code{clean_cache(TRUE)}).

  This function must be called within a code chunk in a source document,
  since it needs to know all chunk labels of the current document to
  determine which labels are no longer present, and delete cache
  corresponding to these labels.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{combine_words}
\alias{combine_words}
\title{Combine multiple words into a single string}
\usage{
combine_words(...)
}
\arguments{
\item{...}{Arguments passed to \code{xfun::\link[xfun]{join_words}()}.}
}
\description{
This is a wrapper function of \code{xfun::join_words()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks-html.R
\name{hook_ffmpeg_html}
\alias{hook_ffmpeg_html}
\alias{hook_gifski}
\alias{hook_scianimator}
\alias{hook_r2swf}
\title{Hooks to create animations in HTML output}
\usage{
hook_ffmpeg_html(x, options)

hook_gifski(x, options)

hook_scianimator(x, options)

hook_r2swf(x, options)
}
\arguments{
\item{x}{Filename for the plot (a character string).}

\item{options}{A list of the current chunk options.}
}
\description{
\code{hook_ffmpeg_html()} uses FFmpeg to convert images to a video;
\code{hook_gifski()} uses the \pkg{gifski} to convert images to a GIF
animation; \code{hook_scianimator()} uses the JavaScript library SciAnimator
to create animations; \code{hook_r2swf()} uses the \pkg{R2SWF} package.
}
\details{
These hooks are mainly for the package option \code{animation.fun}, e.g. you
can set \code{opts_knit$set(animation.fun = hook_scianimator)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\docType{data}
\name{opts_knit}
\alias{opts_knit}
\title{Options for the knitr package}
\format{An object of class \code{list} of length 6.}
\usage{
opts_knit
}
\description{
Options including whether to use a progress bar when knitting a document, and
the base directory of images, etc.
}
\details{
Besides the standard usage (\code{opts_knit$set()}), we can also set package
options prior to loading \code{knitr} or calling \code{knit()} using
\code{\link{options}()} in base R. A global option \code{knitr.package.foo}
in \code{options()} will be set as an option \code{foo} in \code{opts_knit},
i.e. global options in base R with the prefix \code{knitr.package.}
correspond to options in \code{opts_knit}. This can be useful to set package
options in \file{~/.Rprofile} without loading \pkg{knitr}.

See \code{str(knitr::opts_knit$get())} for a list of default package options.
}
\examples{
opts_knit$get("verbose")
opts_knit$set(verbose = TRUE)  # change it
if (interactive()) {
    # for unnamed chunks, use 'fig' as the figure prefix
    opts_knit$set(unnamed.chunk.label = "fig")
    knit("001-minimal.Rmd")  # from https://github.com/yihui/knitr-examples
}
}
\references{
Usage: \url{https://yihui.org/knitr/objects/}

  A list of available options:
  \url{https://yihui.org/knitr/options/#package_options}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks-html.R, R/hooks-asciidoc.R,
%   R/hooks-latex.R, R/hooks-md.R, R/hooks-rst.R, R/hooks-textile.R
\name{render_html}
\alias{render_html}
\alias{hooks_html}
\alias{render_asciidoc}
\alias{hooks_asciidoc}
\alias{render_latex}
\alias{hooks_latex}
\alias{render_sweave}
\alias{hooks_sweave}
\alias{render_listings}
\alias{hooks_listings}
\alias{render_markdown}
\alias{hooks_markdown}
\alias{render_jekyll}
\alias{hooks_jekyll}
\alias{render_rst}
\alias{hooks_rst}
\alias{render_textile}
\alias{hooks_textile}
\title{Set or get output hooks for different output formats}
\usage{
render_html()

hooks_html()

render_asciidoc()

hooks_asciidoc()

render_latex()

hooks_latex()

render_sweave()

hooks_sweave(envirs = c("Sinput", "Soutput", "Schunk"))

render_listings()

hooks_listings(envirs = c("Sinput", "Soutput", "Schunk"))

render_markdown(strict = FALSE, fence_char = "`")

hooks_markdown(strict = FALSE, fence_char = "`")

render_jekyll(highlight = c("pygments", "prettify", "none"), extra = "")

hooks_jekyll(highlight = c("pygments", "prettify", "none"), extra = "")

render_rst(strict = FALSE)

hooks_rst(strict = FALSE)

render_textile()

hooks_textile()
}
\arguments{
\item{envirs}{Names of LaTeX environments for code input, output, and chunk.}

\item{strict}{Boolean; whether to use strict markdown or reST syntax. For markdown, if
\code{TRUE}, code blocks will be indented by 4 spaces, otherwise they are
put in fences made by three backticks. For reST, if \code{TRUE}, code is
put under two colons and indented by 4 spaces, otherwise it is put under the
\samp{sourcecode} directive (this is useful for e.g. Sphinx).}

\item{fence_char}{A single character to be used in the code blocks fence.
This can be e.g. a backtick or a tilde, depending on your Markdown rendering
engine.}

\item{highlight}{Which code highlighting engine to use: if \code{pygments},
the Liquid syntax is used (default approach Jekyll); if \code{prettify},
the output is prepared for the JavaScript library \file{prettify.js}; if
\code{none}, no highlighting engine will be used, and code blocks are simply
indented by 4 spaces).}

\item{extra}{Extra tags for the highlighting engine. For \code{pygments}, this
can be \code{'linenos'}; for \code{prettify}, it can be \code{'linenums'}.}
}
\value{
\code{NULL} for \code{render_*} functions; corresponding hooks are
  set as a side effect. A list of output hooks for \code{hooks_*()}
  functions.
}
\description{
The \code{render_*()} functions set built-in output hooks for LaTeX, HTML,
Markdown, reStructuredText, AsciiDoc, and Textile. The \code{hooks_*()}
functions return a list of the output hooks for the corresponding format.
}
\details{
There are three variants of Markdown documents: ordinary Markdown
(\code{render_markdown(strict = TRUE)}, which calls
\code{hooks_markdown(strict = TRUE)}), extended Markdown (e.g., GitHub
Flavored Markdown and Pandoc; \code{render_markdown(strict = FALSE)}, which
calls \code{hooks_markdown(strict = FALSE)}), and Jekyll (a blogging system
on GitHub; \code{render_jekyll()}, which calls \code{hooks_jekyll()}).

For LaTeX output, there are three variants: \pkg{knitr}'s default style
(\code{render_latex()}, which calls \code{hooks_latex()} and uses the LaTeX
\pkg{framed} package), Sweave style (\code{render_sweave()}, which calls
\code{hooks_sweave()} and uses \file{Sweave.sty}), and listings style
(\code{render_listings()}, which calls \code{hooks_listings()} and uses LaTeX
\pkg{listings} package).

Default HTML output hooks are set by \code{render_html()} (which calls
\code{hooks_html()}); \code{render_rst()} (which calls \code{hooks_rst()}) is
for reStructuredText; \code{render_textile()} (which calls
\code{hooks_textile()}) is for Textile, and \code{render_asciidoc()} (which
calls \code{hooks_asciidoc()}) is AsciiDoc.

The \code{render_*()} functions can be used before \code{knit()} or in the
first chunk of the input document (ideally this chunk has options
\code{include = FALSE} and \code{cache = FALSE}) so that all the following
chunks will be formatted as expected.

You can also use \code{\link{knit_hooks}} to set the format's hooks with the
\code{hooks_*()} functions; see references for more info on further
customizing output hooks.
}
\examples{
# below is pretty much what knitr::render_markdown() does:
knitr::knit_hooks$set(knitr::hooks_markdown())

# you can retrieve a subset of the hooks and set them, e.g.,
knitr::knit_hooks$set(knitr::hooks_markdown()["source"])

knitr::knit_hooks$restore()
}
\references{
See output hooks in \url{https://yihui.org/knitr/hooks/}, and
  some examples in
  \url{https://bookdown.org/yihui/rmarkdown-cookbook/output-hooks.html}

  Jekyll and Liquid:
  \url{https://github.com/jekyll/jekyll/wiki/Liquid-Extensions}; prettify.js:
  \url{https://code.google.com/archive/p/google-code-prettify}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output.R
\name{sew}
\alias{sew}
\title{Wrap evaluated results for output}
\usage{
sew(x, options = list(), ...)
}
\arguments{
\item{x}{Output from \code{evaluate::\link{evaluate}()}.}

\item{options}{A list of chunk options used to control output.}

\item{...}{Other arguments to pass to methods.}
}
\description{
This function is mainly for internal use: it is called on each part of the
output of the code chunk (code, messages, text output, and plots, etc.) after
all statements in the code chunk have been evaluated, and will sew these
pieces of output together into a character vector.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parser.R
\name{all_labels}
\alias{all_labels}
\alias{all_rcpp_labels}
\title{Get all chunk labels in a document}
\usage{
all_labels(...)

all_rcpp_labels(...)
}
\arguments{
\item{...}{A vector of R expressions, each of which should return \code{TRUE}
or \code{FALSE}. The expressions are evaluated using the \emph{local} chunk
options of each code chunk as the environment, which means global chunk
options are not considered when evaluating these expressions. For example,
if you set the global chunk option \code{opts_chunk$set(purl = TRUE)},
\code{all_labels(purl == TRUE)} will \emph{not} return the labels of all
code chunks, but will only return the labels of those code chunks that have
local chunk options \code{purl = TRUE}.}
}
\value{
A character vector.
}
\description{
The function \code{all_labels()} returns all chunk labels as a character
vector. Optionally, you can specify a series of conditions to filter the
labels. The function `all_rcpp_labels()` is a wrapper function for
\code{all_labels(engine == 'Rcpp')}.
}
\details{
For example, suppose the condition expression is \code{engine == 'Rcpp'}, the
object \code{engine} is the local chunk option \code{engine}. If an
expression fails to be evaluated (e.g. when a certain object does not exist),
\code{FALSE} is returned and the label for this chunk will be filtered out.
}
\note{
Empty code chunks are always ignored, including those chunks that are
  empty in the original document but filled with code using chunk options
  such as \code{ref.label} or \code{code}.
}
\examples{
# the examples below are meaningless unless you put them in a knitr document
all_labels()
all_labels(engine == "Rcpp")
all_labels(echo == FALSE && results != "hide")
# or separate the two conditions
all_labels(echo == FALSE, results != "hide")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks-latex.R
\name{hook_movecode}
\alias{hook_movecode}
\title{Some potentially useful document hooks}
\usage{
hook_movecode(x)
}
\arguments{
\item{x}{A character string (the whole output document).}
}
\value{
The post-processed document as a character string.
}
\description{
A document hook is a function to post-process the output document.
}
\details{
\code{hook_movecode()} is a document hook to move code chunks out of LaTeX
floating environments like \samp{figure} and \samp{table} when the chunks
were actually written inside the floats. This function is primarily designed
for LyX: we often insert code chunks into floats to generate figures or
tables, but in the final output we do not want the code to float with the
environments, so we use regular expressions to find out the floating
environments, extract the code chunks and move them out. To disable this
behavior, use a comment \code{\% knitr_do_not_move} in the floating
environment.
}
\note{
These functions are hackish. Also note \code{hook_movecode()} assumes
  you to use the default output hooks for LaTeX (not Sweave or listings), and
  every figure/table environment must have a label.
}
\examples{
\dontrun{
knit_hooks$set(document = hook_movecode)
}
# see example 103 at https://github.com/yihui/knitr-examples
}
\references{
\url{https://yihui.org/knitr/hooks/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-vignettes.R
\name{vignette_engines}
\alias{vignette_engines}
\title{Package vignette engines}
\description{
Since R 3.0.0, package vignettes can use non-Sweave engines, and \pkg{knitr}
has provided a few engines to compile vignettes via \code{\link{knit}()} with
different templates. See \url{https://yihui.org/knitr/demo/vignette/} for
more information.
}
\note{
If you use the \code{knitr::rmarkdown} engine, please make sure that
  you put \pkg{rmarkdown} in the \samp{Suggests} field of your
  \file{DESCRIPTION} file. Also make sure the executables \command{pandoc}
  and \command{pandoc-citeproc} can be found by \pkg{rmarkdown} during
  \command{R CMD build}. If you build your package from RStudio, this is
  normally not a problem. If you build the package outside RStudio, run
  \command{which pandoc} and \command{which pandoc-citeproc} in the terminal
  (or \code{Sys.which('pandoc')} and \code{Sys.which('pandoc-citeproc')} in
  R) to check if \command{pandoc} and \command{pandoc-citeproc} can be found.
  If you use Linux, you may make symlinks to the Pandoc binaries in RStudio:
  \url{https://rmarkdown.rstudio.com/docs/articles/pandoc.html}, or
  install \command{pandoc} and \command{pandoc-citeproc} separately.

  When the \pkg{rmarkdown} package is not installed or not available, or
  \command{pandoc} or \command{pandoc-citeproc} cannot be found, the
  \code{knitr::rmarkdown} engine will fall back to the \code{knitr::knitr}
  engine, which uses R Markdown v1 based on the \pkg{markdown} package.
}
\examples{
library(knitr)
vig_list = tools::vignetteEngine(package = "knitr")
str(vig_list)
vig_list[["knitr::knitr"]][c("weave", "tangle")]
vig_list[["knitr::knitr_notangle"]][c("weave", "tangle")]
vig_list[["knitr::docco_classic"]][c("weave", "tangle")]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern.R
\docType{data}
\name{all_patterns}
\alias{all_patterns}
\title{All built-in patterns}
\format{An object of class \code{list} of length 8.}
\usage{
all_patterns
}
\description{
This object is a named list of all built-in patterns.
}
\examples{
all_patterns$rnw
all_patterns$html

str(all_patterns)
}
\references{
Usage: \url{https://yihui.org/knitr/patterns/}
}
\seealso{
\code{\link{knit_patterns}}
}
\keyword{datasets}

\name{eclipse_theme}
\alias{eclipse_theme}
\title{Download and convert a theme from eclipsecolorthemes.org to CSS}
\usage{
eclipse_theme(id)
}
\arguments{
  \item{id}{id of theme to save as CSS}
}
\value{
  Path to the CSS file converted from the website.
}
\description{
  This function uses the \pkg{XML} package to parse the
  theme as an XML file, then converts to a CSS file using a
  brew template in the \pkg{knitr} package. The CSS file
  can be further parsed with \code{knit_theme$get()}, and
  the result will be ready for \code{knit_theme$set()} to
  set the highlighting theme.
}
\examples{
## http://www.eclipsecolorthemes.org/?view=theme&id=1
\dontrun{
opts_knit$set(out.format = "latex")
(css = eclipse_theme(1))
thm = knit_theme$get(css)
knit_theme$set(thm)
}
}
\author{
  Ramnath Vaidyanathan
}
\references{
  \url{http://www.eclipsecolorthemes.org/}
}
\seealso{
  \code{\link{knit_theme}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parser.R
\name{read_chunk}
\alias{read_chunk}
\alias{read_demo}
\title{Read chunks from an external script}
\usage{
read_chunk(
  path,
  lines = read_utf8(path),
  labels = NULL,
  from = NULL,
  to = NULL,
  from.offset = 0L,
  to.offset = 0L,
  roxygen_comments = TRUE
)

read_demo(topic, package = NULL, ...)
}
\arguments{
\item{path}{Path to the R script.}

\item{lines}{Character vector of lines of code. By default, this is read from
\code{path}.}

\item{labels}{Character vector of chunk labels (default \code{NULL}).}

\item{from, to}{Numeric vector specifying the starting/ending line numbers of
code chunks, or a character vector; see Details.}

\item{from.offset, to.offset}{Offsets to be added to \code{from}/\code{to}.}

\item{roxygen_comments}{Logical dictating whether to keep trailing
roxygen-style comments from code chunks in addition to whitespace}

\item{topic, package}{Name of the demo and the package. See \code{\link[utils]{demo}}.}

\item{...}{Arguments passed to \code{\link{read_chunk}}.}
}
\value{
As a side effect, code chunks are read into the current session so
  that future chunks can (re)use the code by chunk label references. If an
  external chunk has the same label as a chunk in the current session, chunk
  label references by future chunks will refer to the external chunk.
}
\description{
Chunks can be put in an external script, and this function reads chunks into
the current \pkg{knitr} session; \code{read_demo()} is a convenience function
to read a demo script from a package.
}
\details{
There are two approaches to read external code into the current session: (1)
Use a special separator of the from \code{## ---- chunk-label} (at least four
dashes before the chunk label) in the script; (2) Manually specify the
labels, starting and ending positions of code chunks in the script.

The second approach will be used only when \code{labels} is not \code{NULL}.
For this approach, if \code{from} is \code{NULL}, the starting position is 1;
if \code{to} is \code{NULL}, each of its element takes the next element of
\code{from} minus 1, and the last element of \code{to} will be the length of
\code{lines} (e.g. when \code{from = c(1, 3, 8)} and the script has 10 lines
in total, \code{to} will be \code{c(2, 7, 10)}). Alternatively, \code{from}
and \code{to} can be character vectors as regular expressions to specify the
positions; when their length is 1, the single regular expression will be
matched against the \code{lines} vector, otherwise each element of
\code{from}/\code{to} is matched against \code{lines} and the match is
supposed to be unique so that the numeric positions returned from
\code{grep()} will be of the same length of \code{from}/\code{to}. Note
\code{labels} always has to match the length of \code{from} and \code{to}.
}
\note{
This function can only be used in a chunk which is \emph{not} cached
  (chunk option \code{cache = FALSE}), and the code is read and stored in the
  current session \emph{without} being executed (to actually run the code,
  you have to use a chunk with a corresponding label).
}
\examples{
## put this in foo.R and read_chunk('foo.R')

## ---- my-label ----
1 + 1
lm(y ~ x, data = data.frame(x = 1:10, y = rnorm(10)))

## later you can use <<my-label>>= to reference this chunk

## the 2nd approach
code = c("#@a", "1+1", "#@b", "#@a", "rnorm(10)", "#@b")
read_chunk(lines = code, labels = "foo")  # put all code into one chunk named foo
read_chunk(lines = code, labels = "foo", from = 2, to = 2)  # line 2 into chunk foo
read_chunk(lines = code, labels = c("foo", "bar"), from = c(1, 4), to = c(3, 6))
# automatically figure out 'to'
read_chunk(lines = code, labels = c("foo", "bar"), from = c(1, 4))
read_chunk(lines = code, labels = c("foo", "bar"), from = "^#@a", to = "^#@b")
read_chunk(lines = code, labels = c("foo", "bar"), from = "^#@a", to = "^#@b", 
    from.offset = 1, to.offset = -1)

## later you can use, e.g., <<foo>>=
knitr:::knit_code$get()  # use this to check chunks in the current session
knitr:::knit_code$restore()  # clean up the session
}
\references{
\url{https://yihui.org/knitr/demo/externalization/}
}
\author{
Yihui Xie; the idea of the second approach came from Peter
  Ruckdeschel (author of the \pkg{SweaveListingUtils} package)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-conversion.R
\name{knit2wp}
\alias{knit2wp}
\title{Knit an R Markdown document and post it to WordPress}
\usage{
knit2wp(
  input,
  title = "A post from knitr",
  ...,
  envir = parent.frame(),
  shortcode = FALSE,
  action = c("newPost", "editPost", "newPage"),
  postid,
  publish = TRUE
)
}
\arguments{
\item{input}{Filename of the Rmd document.}

\item{title}{Title of the post.}

\item{...}{Other meta information of the post, e.g. \code{categories = c('R',
'Stats')} and \code{mt_keywords = c('knitr', 'wordpress')}, et cetera.}

\item{envir}{Environment in which code chunks are to be evaluated,
for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
\code{\link{globalenv}()}).}

\item{shortcode}{A length-2 logical vector: whether to use the shortcode
\samp{[sourcecode lang='lang']}, which can be useful to WordPress.com users
for syntax highlighting of source code and output. The first element
applies to source code, and the second applies to text output. By default,
both are \code{FALSE}.}

\item{action}{Whether to create a new post, update an existing post, or create a new
page.}

\item{postid}{If \code{action} is \code{editPost}, the post id \code{postid} must be
specified.}

\item{publish}{Boolean: publish the post immediately?}
}
\description{
This function is a wrapper around the \pkg{RWordPress} package. It compiles
an R Markdown document to HTML and post the results to WordPress.
}
\note{
This function will convert the encoding of the post and the title to
  UTF-8 internally. If you have additional data to send to WordPress (e.g.
  keywords and categories), you may have to manually convert them to the
  UTF-8 encoding with the \code{\link{iconv}(x, to = 'UTF-8')} function
  (especially when using Windows).
}
\examples{
# see the reference
}
\references{
\url{https://yihui.org/knitr/demo/wordpress/}
}
\author{
William K. Morris, Yihui Xie, and Jared Lander
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{knitr-package}
\alias{knitr-package}
\alias{knitr}
\title{A general-purpose tool for dynamic report generation in R}
\description{
The \pkg{knitr} package is an implementation of Literate Programming, a
programming paradigm that intermingle code chunks (for computing) with prose
(for documentation) in the same document.
}
\details{
When the document is compiled, the code chunks can be executed, and the
results from computing (text or graphics) are automatically written to the
output along with the prose.

This package is an alternative tool to Sweave with a more flexible design and
new features like caching and finer control of graphics. It is not limited to
LaTeX and is ready to be customized to process other file formats. See the
package website in the references for more information and examples.
}
\note{
The pronunciation of \pkg{knitr} is similar to \emph{neater} or you can
  think of \emph{knitter} (but it is \emph{single t}). The name comes from
  \code{knit} + \code{R} (while \code{Sweave} = \code{S} + \code{weave}).
}
\references{
Full documentation and demos: \url{https://yihui.org/knitr/};
  FAQ's: \url{https://yihui.org/knitr/faq/}
}
\seealso{
The core function in this package: \code{\link{knit}}. If you are an
  Sweave user, see \code{\link{Sweave2knitr}} on how to convert Sweave files
  to \pkg{knitr}.
}
\author{
Yihui Xie <\url{https://yihui.org}>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-sweave.R
\name{Sweave2knitr}
\alias{Sweave2knitr}
\title{Convert Sweave to knitr documents}
\usage{
Sweave2knitr(
  file,
  output = gsub("[.]([^.]+)$", "-knitr.\\\\1", file),
  text = NULL
)
}
\arguments{
\item{file}{Path to the Rnw file (must be encoded in UTF-8).}

\item{output}{Output file path. By default, \file{file.Rnw} produces
\file{file-knitr.Rnw}); if \code{text} is not NULL, no output file will be
produced.}

\item{text}{An alternative way to provide the Sweave code as a character
string. If \code{text} is provided, \code{file} will be ignored.}
}
\value{
If \code{text} is \code{NULL}, the \code{output} file is written and
  \code{NULL} is returned. Otherwise, the converted text string is returned.
}
\description{
This function converts an Sweave document to a \pkg{knitr}-compatible
document.
}
\details{
The pseudo command \samp{\\SweaveInput{file.Rnw}} is converted to a code
chunk header \code{<<child='file.Rnw'>>=}.

Similarly \samp{\\SweaveOpts{opt = value}} is converted to a code chunk
\samp{opts_chunk$set(opt = value)} with the chunk option \code{include =
FALSE}; the options are automatically fixed in the same way as local chunk
options (explained below).

The Sweave package \samp{\\usepackage{Sweave}} in the preamble is removed
because it is not required.

Chunk options are updated if necessary: option values \code{true} and
\code{false} are changed to \code{TRUE} and \code{FALSE} respectively;
\code{fig=TRUE} is removed because it is not necessary for \pkg{knitr} (plots
will be automatically generated); \code{fig=FALSE} is changed to
\code{fig.keep='none'}; the devices \code{pdf/jpeg/png/eps/tikz=TRUE} are
converted to \code{dev='pdf'/'jpeg'/'png'/'postscript'/'tikz'};
\code{pdf/jpeg/png/eps/tikz=FALSE} are removed;
\code{results=tex/verbatim/hide} are changed to
\code{results='asis'/'markup'/'hide'}; \code{width/height} are changed to
\code{fig.width/fig.height}; \code{prefix.string} is changed to
\code{fig.path}; \code{print/term/prefix=TRUE/FALSE} are removed; most of the
character options (e.g. \code{engine} and \code{out.width}) are quoted;
\code{keep.source=TRUE/FALSE} is changed to \code{tidy=FALSE/TRUE} (note the
order of values).

If a line \code{@} (it closes a chunk) directly follows a previous
\code{@}, it is removed; if a line \code{@} appears before a code chunk and
no chunk is before it, it is also removed, because \pkg{knitr} only uses one
\samp{@} after \samp{<<>>=} by default (which is not the original Noweb
syntax but more natural).
}
\note{
If \samp{\\SweaveOpts{}} spans across multiple lines, it will not be
  fixed, and you have to fix it manually. The LaTeX-style syntax of Sweave
  chunks are ignored (see \code{?SweaveSyntaxLatex}); only the Noweb syntax
  is supported.
}
\examples{
Sweave2knitr(text = "<<echo=TRUE>>=")  # this is valid
Sweave2knitr(text = "<<png=true>>=")  # dev='png'
Sweave2knitr(text = "<<eps=TRUE, pdf=FALSE, results=tex, width=5, prefix.string=foo>>=")
Sweave2knitr(text = "<<,png=false,fig=TRUE>>=")
Sweave2knitr(text = "\\\\SweaveOpts{echo=false}")
Sweave2knitr(text = "\\\\SweaveInput{hello.Rnw}")
# Sweave example in utils
testfile = system.file("Sweave", "Sweave-test-1.Rnw", package = "utils")
Sweave2knitr(testfile, output = "Sweave-test-knitr.Rnw")
if (interactive()) knit("Sweave-test-knitr.Rnw")  # or knit2pdf() directly
unlink("Sweave-test-knitr.Rnw")
}
\references{
The motivation of the changes in the syntax:
  \url{https://yihui.org/knitr/demo/sweave/}
}
\seealso{
\code{\link{Sweave}}, \code{\link{gsub}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{wrap_rmd}
\alias{wrap_rmd}
\title{Wrap long lines in Rmd files}
\usage{
wrap_rmd(file, width = 80, text = NULL, backup)
}
\arguments{
\item{file}{The input Rmd file.}

\item{width}{The expected line width.}

\item{text}{A character vector of text lines, as an alternative to \code{file}. If
\code{text} is not \code{NULL}, \code{file} is ignored.}

\item{backup}{Path to back up the original file in case anything goes
wrong. If set to \code{NULL}, no backup is made. The default value is constructed
from \code{file} by adding \code{__} before the base filename.}
}
\value{
If \code{file} is provided, it is overwritten; if \code{text} is
  provided, a character vector is returned.
}
\description{
This function wraps long paragraphs in an R Markdown file. Other elements are
not wrapped: the YAML preamble, fenced code blocks, section headers and
indented elements. The main reason for wrapping long lines is to make it
easier to review differences in version control.
}
\note{
Currently it does not wrap blockquotes or lists (ordered or unordered).
  This feature may or may not be added in the future.
}
\examples{
wrap_rmd(text = c("```", "1+1", "```", "- a list item", "> a quote", "", 
    paste(rep("this is a normal paragraph", 5), collapse = " ")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{dep_prev}
\alias{dep_prev}
\title{Make later chunks depend on previous chunks}
\usage{
dep_prev()
}
\value{
\code{NULL}; the internal dependency structure is updated as a side
  effect.
}
\description{
This function can be used to build dependencies among chunks so that all
later chunks depend on previous chunks, i.e. whenever the cache of a previous
chunk is updated, the cache of all its later chunks will be updated.
}
\references{
\url{https://yihui.org/knitr/demo/cache/}
}
\seealso{
\code{\link{dep_auto}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/header.R
\name{set_header}
\alias{set_header}
\title{Set the header information}
\usage{
set_header(...)
}
\arguments{
\item{...}{Header components; currently possible components are
\code{highlight}, \code{tikz} and \code{framed}, which contain the
necessary commands to be used in the HTML header or LaTeX preamble. Note that
HTML output does not use the \code{tikz} and \code{framed} components, since
they do not make sense in the context of HTML.}
}
\value{
The header vector in \code{opts_knit} is set.
}
\description{
Some output documents may need appropriate header information. For example,
for LaTeX output, we need to write \samp{\\usepackage{tikz}} into the
preamble if we use tikz graphics; this function sets the header information
to be written into the output.
}
\details{
By default, \pkg{knitr} will set up the header automatically. For example, if
the tikz device is used, \pkg{knitr} will add \samp{\\usepackage{tikz}} to
the LaTeX preamble, and this is done by setting the header component
\code{tikz} to be a character string: \code{set_header(tikz =
'\\usepackage{tikz}')}. Similary, when we highlight R code using the
\pkg{highlight} package (i.e. the chunk option \code{highlight = TRUE}),
\pkg{knitr} will set the \code{highlight} component of the header vector
automatically; if the output type is HTML, this component will be different
-- instead of LaTeX commands, it contains CSS definitions.

For power users, all the components can be modified to adapt to a customized
type of output. For instance, we can change \code{highlight} to LaTeX
definitions of the \pkg{listings} package (and modify the output hooks
accordingly), so we can decorate R code using the \pkg{listings} package.
}
\examples{
set_header(tikz = "\\\\usepackage{tikz}")
opts_knit$get("header")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\docType{data}
\name{cache_engines}
\alias{cache_engines}
\title{Cache engines of other languages}
\format{An object of class \code{list} of length 6.}
\usage{
cache_engines
}
\description{
This object controls how to load cached environments from languages other
than R (when the chunk option \code{engine} is not \code{'R'}). Each
component in this object is a function that takes the current path to the
chunk cache and loads it into the language environment.
}
\details{
The cache engine function has one argument \code{options}, a list containing
all chunk options. Note that \code{options$hash} is the path to the current
chunk cache with the chunk's hash, but without any file extension, and the
language engine may write a cache database to this path (with an extension).

The cache engine function should load the cache environment and should know
the extension appropriate for the language.
}
\references{
See \url{https://github.com/rstudio/reticulate/pull/167} for an
  implementation of a cache engine for Python.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output.R
\name{knit_print}
\alias{knit_print}
\alias{normal_print}
\title{A custom printing function}
\usage{
knit_print(x, ...)

normal_print(x, ...)
}
\arguments{
\item{x}{An R object to be printed}

\item{...}{Additional arguments passed to the S3 method. Currently ignored,
except two optional arguments \code{options} and \code{inline}; see
the references below.}
}
\value{
The value returned from the print method should be a character vector
  or can be converted to a character value. You can wrap the value in
  \code{\link{asis_output}()} so that \pkg{knitr} writes the character value
  as is in the output.
}
\description{
The S3 generic function \code{knit_print} is the default printing function in
\pkg{knitr}. The chunk option \code{render} uses this function by default.
The main purpose of this S3 generic function is to customize printing of R
objects in code chunks. We can fall back to the normal printing behavior by
setting the chunk option \code{render = normal_print}.
}
\details{
Users can write custom methods based on this generic function. For example,
if we want to print all data frames as tables in the output, we can define a
method \code{knit_print.data.frame} that turns a data.frame into a table (the
implementation may use other R packages or functions, e.g. \pkg{xtable} or
\code{\link{kable}()}).
}
\note{
It is recommended to leave a \code{...} argument in your method, to
  allow future changes of the \code{knit_print()} API without breaking your
  method.
}
\examples{
library(knitr)
# write tables for data frames
knit_print.data.frame = function(x, ...) {
    res = paste(c("", "", kable(x, output = FALSE)), collapse = "\n")
    asis_output(res)
}
# register the method
registerS3method("knit_print", "data.frame", knit_print.data.frame)
# after you define and register the above method, data frames will be printed as
# tables in knitr, which is different with the default print() behavior
}
\references{
See \code{vignette('knit_print', package = 'knitr')}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\docType{data}
\name{knit_engines}
\alias{knit_engines}
\title{Engines of other languages}
\format{An object of class \code{list} of length 6.}
\usage{
knit_engines
}
\description{
This object controls how to execute the code from languages other than R
(when the chunk option \code{engine} is not \code{'R'}). Each component in
this object is a function that takes a list of current chunk options
(including the source code) and returns a character string to be written into
the output.
}
\details{
The engine function has one argument \code{options}: the source code of the
current chunk is in \code{options$code}. Usually we can call external
programs to run the code via \code{\link{system2}}. Other chunk options are
also contained in this argument, e.g. \code{options$echo} and
\code{options$eval}, etc.

In most cases, \code{options$engine} can be directly used in command line to
execute the code, e.g. \code{python} or \code{ruby}, but sometimes we may
want to specify the path of the engine program, in which case we can pass it
through the \code{engine.path} option. For example, \code{engine='ruby',
engine.path='/usr/bin/ruby1.9.1'}. Additional command line arguments can be
passed through \code{options$engine.opts}, e.g. \code{engine='ruby',
engine.opts='-v'}.

See \code{str(knitr::knit_engines$get())} for a list of built-in language
engines.
}
\note{
The Leiningen engine \code{lein} requires lein-exec plugin; see
  \url{https://github.com/yihui/knitr/issues/1176} for details.
}
\examples{
knit_engines$get("python")
knit_engines$get("awk")
names(knit_engines$get())
}
\references{
Usage: \url{https://yihui.org/knitr/objects/}; examples:
  \url{https://yihui.org/knitr/demo/engines/}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern.R
\docType{data}
\name{knit_patterns}
\alias{knit_patterns}
\title{Patterns to match and extract R code in a document}
\format{An object of class \code{list} of length 6.}
\usage{
knit_patterns
}
\description{
Patterns are regular expressions and will be used in functions like
\code{\link[base]{grep}} to extract R code and chunk options. The object
\code{knit_patterns} controls the patterns currently used; see the references
and examples for usage.  All built-in patterns are available in the list
\link{all_patterns}.
}
\examples{
library(knitr)
opat = knit_patterns$get()  # old pattern list (to restore later)

apats = all_patterns  # a list of all built-in patterns
str(apats)
knit_patterns$set(apats[["rnw"]])  # set pattern list from apats

knit_patterns$get(c("chunk.begin", "chunk.end", "inline.code"))

# a customized pattern list; has to empty the original patterns first!
knit_patterns$restore()
# we may want to use this in an HTML document
knit_patterns$set(list(chunk.begin = "<!--helloR\\\\s+(.*)", chunk.end = "^byeR-->"))
str(knit_patterns$get())

knit_patterns$set(opat)  # put the old patterns back
}
\references{
Usage: \url{https://yihui.org/knitr/objects/}

Components in \code{knit_patterns}: \url{https://yihui.org/knitr/patterns/}
}
\seealso{
\code{\link{all_patterns}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output.R
\name{asis_output}
\alias{asis_output}
\title{Mark an R object with a special class}
\usage{
asis_output(x, meta = NULL, cacheable = NA)
}
\arguments{
\item{x}{An R object. Typically a character string, or an object which can
be converted to a character string via \code{\link{as.character}()}.}

\item{meta}{Additional metadata of the object to be printed. The metadata
will be collected when the object is printed, and accessible via
\code{knit_meta()}.}

\item{cacheable}{Boolean indicating whether this object is cacheable. If
\code{FALSE}, \pkg{knitr} will stop when caching is enabled on code chunks
that contain \code{asis_output()}.}
}
\description{
This is a convenience function that assigns the input object a class named
\code{knit_asis}, so that \pkg{knitr} will treat it as is (the effect is the
same as the chunk option \code{results = 'asis'}) when it is written to the
output.
}
\details{
This function is normally used in a custom S3 method based on the printing
function \code{\link{knit_print}()}.

For the \code{cacheable} argument, you need to be careful when printing the
object involves non-trivial side effects, in which case it is strongly
recommended to use \code{cacheable = FALSE} to instruct \pkg{knitr} that this
object should not be cached using the chunk option \code{cache = TRUE},
otherwise the side effects will be lost the next time the chunk is knitted.
For example, printing a \pkg{shiny} input element or an HTML widget in an R
Markdown document may involve registering metadata about some JavaScript
libraries or stylesheets, and the metadata may be lost if we cache the code
chunk, because the code evaluation will be skipped the next time. This
particular issue has been solved in \pkg{knitr} after v1.13 (the metadata
will be saved and loaded automatically when caching is enabled), but not all
metadata can be saved and loaded next time and still works in the new R
session.
}
\note{
This function only works in top-level R expressions, and it will not
  work when it is called inside another expression, such as a for-loop. See
  \url{https://github.com/yihui/knitr/issues/1137} for a discussion.
}
\examples{
# see ?knit_print
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spin.R
\name{spin_child}
\alias{spin_child}
\title{Spin a child R script}
\usage{
spin_child(input, format)
}
\arguments{
\item{input}{Filename of the input R script.}

\item{format}{Passed to \code{format} in \code{spin()}. If not
provided, it will be guessed from the current knitting process.}
}
\value{
A character string of the knitted R script.
}
\description{
This function is similar to \code{\link{knit_child}()} but is used in R
scripts instead. When the main R script is not called via
\code{\link{spin}()}, this function simply executes the child script via
\code{\link{sys.source}()}, otherwise it calls \code{\link{spin}()} to spin
the child script into a source document, and uses \code{\link{knit_child}()}
to compile it. You can call this function in R code, or using the syntax of
inline R expressions in \code{\link{spin}()} (e.g.
\code{{{knitr::spin_child('script.R')}}}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parser.R
\name{partition_chunk}
\alias{partition_chunk}
\title{Partition chunk options from the code chunk body}
\usage{
partition_chunk(engine, code)
}
\description{
This is a wrapper function calling \code{xfun::\link[xfun]{divide_chunk}()}
under the hood.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/template.R
\name{knit_expand}
\alias{knit_expand}
\title{A simple macro preprocessor for templating purposes}
\usage{
knit_expand(file, ..., text = read_utf8(file), delim = c("{{", "}}"))
}
\arguments{
\item{file}{The template file.}

\item{...}{A list of variables to be used for the code in the template; note that
the variables will be searched for in the parent frame as well.}

\item{text}{Character vector of lines of code. An alternative way to specify
the template code directly. If \code{text} is provided, \code{file} will be ignored.}

\item{delim}{A pair of opening and closing delimiters for the templating tags.}
}
\value{
A character vector, with the tags evaluated and replaced by their
  values.
}
\description{
This function expands a template based on the R expressions in \code{{{}}}
(this tag can be customized by the \code{delim} argument). These expressions
are extracted, evaluated and replaced by their values in the original
template.
}
\examples{
# see the knit_expand vignette
if (interactive()) browseVignettes(package = "knitr")
}
\references{
This function was inspired by the pyexpander and m4
  (\url{http://www.gnu.org/software/m4/}), thanks to Frank Harrell.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{include_url}
\alias{include_url}
\alias{include_app}
\title{Embed a URL as an HTML iframe or a screenshot in \pkg{knitr} documents}
\usage{
include_url(url, height = "400px")

include_app(url, height = "400px")
}
\arguments{
\item{url}{Character string containing a URL.}

\item{height}{Character string with the height of the iframe.}
}
\value{
An R object with a special class that \pkg{knitr} recognizes
  internally to generate the iframe or screenshot.
}
\description{
When the output format is HTML, \code{include_url()} inserts an iframe in the
output; otherwise it takes a screenshot of the URL and insert the image in
the output. \code{include_app()} takes the URL of a Shiny app and adds
\samp{?showcase=0} to it (to disable the showcase mode), then passes the URL
to \code{include_url()}.
}
\seealso{
\code{\link{include_graphics}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output.R
\name{knit_exit}
\alias{knit_exit}
\title{Exit knitting early}
\usage{
knit_exit(append)
}
\arguments{
\item{append}{A character vector to be appended to the results from
\code{knit()} so far. By default, this is \samp{\end{document}} for LaTeX
output, and \samp{</body></html>} for HTML output, to make the output
document complete. For other types of output, it is an empty string.}
}
\value{
Invisible \code{NULL}. An internal signal is set up (as a side
  effect) to notify \code{knit()} to quit as if it had reached the end of the
  document.
}
\description{
Sometimes we may want to exit the knitting process early, and completely
ignore the rest of the document. This function provides a mechanism to
terminate \code{\link{knit}()}.
}
\examples{
# see https://github.com/yihui/knitr-examples/blob/master/096-knit-exit.Rmd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-upload.R
\name{imgur_upload}
\alias{imgur_upload}
\title{Upload an image to imgur.com}
\usage{
imgur_upload(file, key = xfun::env_option("knitr.imgur.key"), ...)
}
\arguments{
\item{file, key, ...}{See \code{xfun::\link[xfun]{upload_imgur}()}.}
}
\description{
This function is an alias to \code{xfun::upload_imgur()}. It is kept in
\pkg{knitr} only for backward-compatibility reasons. You are recommended to
use \code{xfun::upload_imgur()} directly instead.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-conversion.R
\name{knit2pdf}
\alias{knit2pdf}
\title{Convert Rnw or Rrst files to PDF}
\usage{
knit2pdf(
  input,
  output = NULL,
  compiler = NULL,
  envir = parent.frame(),
  quiet = FALSE,
  ...
)
}
\arguments{
\item{input}{Path to the input file.}

\item{output}{Path to the output file for \code{knit()}. If \code{NULL}, this
function will try to guess a default, which will be under the current working
directory.}

\item{compiler}{A character string giving the LaTeX engine used to
compile the tex document to PDF. For an Rrst file, setting \code{compiler} to
\code{'rst2pdf'} will use \code{\link{rst2pdf}} to compile the rst file to
PDF using the ReportLab open-source library.}

\item{envir}{Environment in which code chunks are to be evaluated,
for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
\code{\link{globalenv}()}).}

\item{quiet}{Boolean; suppress the progress bar and messages?}

\item{...}{Options to be passed to \code{tinytex::\link[tinytex]{latexmk}} or
\code{\link{rst2pdf}}.}
}
\value{
The filename of the PDF file.
}
\description{
Knit the input Rnw or Rrst document, and compile to PDF using
\code{tinytex::\link[tinytex]{latexmk}()} or \code{\link{rst2pdf}()}.
}
\note{
The \code{output} argument specifies the output filename to be passed
  to the PDF compiler (e.g. a tex document) instead of the PDF filename.
}
\examples{
#' compile with xelatex
## knit2pdf(..., compiler = 'xelatex')

#' compile a reST file with rst2pdf
## knit2pdf(..., compiler = 'rst2pdf')
}
\author{
Ramnath Vaidyanathan, Alex Zvoleff and Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/params.R
\name{knit_params}
\alias{knit_params}
\title{Extract knit parameters from a document}
\usage{
knit_params(text, evaluate = TRUE)
}
\arguments{
\item{text}{Character vector containing the document text.}

\item{evaluate}{Boolean. If \code{TRUE} (the default), expression values embedded
within the YAML will be evaluated. If \code{FALSE}, parameters defined with an
expression will have the parsed but unevaluated expression in their \code{value}
field.}
}
\value{
List of objects of class \code{knit_param} that correspond to the
  parameters declared in the \code{params} section of the YAML front matter.
  These objects have the following fields:

  \describe{
    \item{\code{name}}{The parameter name.}
    \item{\code{value}}{The default value for the parameter.}
    \item{\code{expr}}{The R expression (if any) that yielded the default value.}
  }

  In addition, other fields included in the YAML may also be present
  alongside the name, type, and value fields (e.g. a \code{label} field
  that provides front-ends with a human readable name for the parameter).
}
\description{
This function reads the YAML front-matter section of a document and returns a
list of any parameters declared there. This function exists primarily to
support the parameterized reports feature of the \pkg{rmarkdown} package,
however is also used by the knitr \code{\link{purl}} function to include
the default parameter values in the R code it emits.
}
\details{
Parameters are included in YAML front matter using the \code{params} key.
This key can have any number of subkeys each of which represents a
parameter. For example:

\preformatted{
---
title: My Document
output: html_document
params:
  frequency: 10
  show_details: true
---
}

Parameter values can be provided inline as illustrated above or can be
included in a \code{value} sub-key. For example:

\preformatted{
---
title: My Document
output: html_document
params:
  frequency:
    value: 10
---
}

This second form is useful when you need to provide additional details
about the parameter (e.g. a \code{label} field as describe above).

You can also use R code to yield the value of a parameter by prefacing the value
with \code{!r}, for example:

\preformatted{
---
title: My Document
output: html_document
params:
  start: !r Sys.Date()
---
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-conversion.R
\name{rst2pdf}
\alias{rst2pdf}
\title{A wrapper for rst2pdf}
\usage{
rst2pdf(input, command = "rst2pdf", options = "")
}
\arguments{
\item{input}{The input rst file.}

\item{command}{Character string giving the path of the
\command{rst2pdf} program. If the program is not in your PATH, the full path has to be
given here.}

\item{options}{Extra command line options, e.g. \code{'-v'}.}
}
\value{
An input file \file{*.rst} will produce \file{*.pdf} and this output
  filename is returned if the conversion was successful.
}
\description{
Convert reST to PDF using \command{rst2pdf} (which converts from rst to PDF
using the ReportLab open-source library).
}
\references{
\url{https://github.com/rst2pdf/rst2pdf}
}
\seealso{
\code{\link{knit2pdf}}
}
\author{
Alex Zvoleff and Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rocco.R
\name{rocco}
\alias{rocco}
\title{Knit R Markdown using the classic Docco style}
\usage{
rocco(input, ...)
}
\arguments{
\item{input}{Path of the input R Markdown file.}

\item{...}{Arguments to be passed to \code{\link{knit2html}}}
}
\value{
An HTML file is written, and its name is returned.
}
\description{
The classic Docco style is a two-column layout, with text in the left and
code in the right column.
}
\details{
The output HTML page supports resizing and hiding/showing the two columns.
Move the cursor to the center of the page, and it will change to a
bidirectional resize cursor; drag the cursor to resize the two columns. Press
the key \code{t} to hide the code column (show the text column only), and
press again to hide the text column (show code).
}
\examples{
rocco_view = function(input) {
    owd = setwd(tempdir())
    on.exit(setwd(owd))
    if (!file.exists(input)) 
        return()
    o = rocco(input, header = "", quiet = TRUE)
    if (interactive()) 
        browseURL(o)
}
# knit these two vignettes using the docco style
rocco_view(system.file("doc", "docco-classic.Rmd", package = "knitr"))
rocco_view(system.file("doc", "knit_expand.Rmd", package = "knitr"))
}
\references{
The Docco package by Jeremy Ashkenas:
  \url{https://github.com/jashkenas/docco}
}
\author{
Weicheng Zhu and Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{set_parent}
\alias{set_parent}
\title{Specify the parent document of child documents}
\usage{
set_parent(parent)
}
\arguments{
\item{parent}{Path to the parent document, relative to the current child
document.}
}
\value{
The preamble is extracted and stored to be used later when the
  complete output is written.
}
\description{
This function extracts the LaTeX preamble of the parent document to use for
the child document, so that the child document can be compiled as an
individual document.
}
\details{
When the preamble of the parent document also contains code chunks and inline
R code, they will be evaluated as if they were in this child document. For
examples, when \pkg{knitr} hooks or other options are set in the preamble of
the parent document, it will apply to the child document as well.
}
\note{
Obviously this function is only useful when the output format is LaTeX.
  This function only works when the child document is compiled in a
  standalone mode using \code{\link{knit}()} (instead of being called in
  \code{\link{knit_child}()}); when the parent document is compiled, this
  function in the child document will be ignored.
}
\examples{
## can use, e.g. \Sexpr{set_parent('parent_doc.Rnw')} or

# <<setup-child, include=FALSE>>=

# set_parent('parent_doc.Rnw')

# @
}
\references{
\url{https://yihui.org/knitr/demo/child/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table.R
\name{kable}
\alias{kable}
\alias{kables}
\title{Create tables in LaTeX, HTML, Markdown and reStructuredText}
\usage{
kable(
  x,
  format,
  digits = getOption("digits"),
  row.names = NA,
  col.names = NA,
  align,
  caption = opts_current$get("tab.cap"),
  label = NULL,
  format.args = list(),
  escape = TRUE,
  ...
)

kables(x, format, caption = NULL, label = NULL)
}
\arguments{
\item{x}{For \code{kable()}, \code{x} is an R object, which is typically a
matrix or data frame. For \code{kables()}, a list with each element being a
returned value from \code{kable()}.}

\item{format}{A character string. Possible values are \code{latex},
\code{html}, \code{pipe} (Pandoc's pipe tables), \code{simple} (Pandoc's
simple tables), \code{rst}, \code{jira}, and \code{org} (Emacs Org-mode).
The value of this argument will be automatically determined if the function
is called within a \pkg{knitr} document. The \code{format} value can also
be set in the global option \code{knitr.table.format}. If \code{format} is
a function, it must return a character string.}

\item{digits}{Maximum number of digits for numeric columns, passed to
\code{round()}. This can also be a vector of length \code{ncol(x)}, to set
the number of digits for individual columns.}

\item{row.names}{Logical: whether to include row names. By default, row names
are included if \code{rownames(x)} is neither \code{NULL} nor identical to
\code{1:nrow(x)}.}

\item{col.names}{A character vector of column names to be used in the table.}

\item{align}{Column alignment: a character vector consisting of \code{'l'}
(left), \code{'c'} (center) and/or \code{'r'} (right). By default or if
\code{align = NULL}, numeric columns are right-aligned, and other columns
are left-aligned. If \code{length(align) == 1L}, the string will be
expanded to a vector of individual letters, e.g. \code{'clc'} becomes
\code{c('c', 'l', 'c')}, unless the output format is LaTeX.}

\item{caption}{The table caption. By default, it is retrieved from the chunk
option \code{tab.cap}.}

\item{label}{The table reference label. By default, the label is obtained
from \code{knitr::\link{opts_current}$get('label')} (i.e., the current
chunk label). To disable the label, use \code{label = NA}.}

\item{format.args}{A list of arguments to be passed to \code{\link{format}()}
to format table values, e.g. \code{list(big.mark = ',')}.}

\item{escape}{Boolean; whether to escape special characters when producing
HTML or LaTeX tables. When \code{escape = FALSE}, you have to make sure
that special characters will not trigger syntax errors in LaTeX or HTML.}

\item{...}{Other arguments (see Examples and References).}
}
\value{
A character vector of the table source code.
}
\description{
A very simple table generator, and it is simple by design. It is not intended
to replace any other R packages for making tables. The \code{kable()}
function returns a single table for a single data object, and returns a table
that contains multiple tables if the input object is a list of data objects.
The \code{kables()} function is similar to \code{kable(x)} when \code{x} is a
list of data objects, but \code{kables()} accepts a list of \code{kable()}
values directly instead of data objects (see examples below).
}
\details{
Missing values (\code{NA}) in the table are displayed as \code{NA} by
default. If you want to display them with other characters, you can set the
option \code{knitr.kable.NA}, e.g. \code{options(knitr.kable.NA = '')} to
hide \code{NA} values.

You can set the option \code{knitr.kable.max_rows} to limit the number of
rows to show in the table, e.g., \code{options(knitr.kable.max_rows = 30)}.
}
\note{
When using \code{kable()} as a \emph{top-level} expression, you do not
  need to explicitly \code{print()} it due to R's automatic implicit
  printing. When it is wrapped inside other expressions (such as a
  \code{\link{for}} loop), you must explicitly \code{print(kable(...))}.
}
\examples{
d1 = head(iris)
d2 = head(mtcars)
# pipe tables by default
kable(d1)
kable(d2[, 1:5])
# no inner padding
kable(d2, format = "pipe", padding = 0)
# more padding
kable(d2, format = "pipe", padding = 2)
kable(d1, format = "latex")
kable(d1, format = "html")
kable(d1, format = "latex", caption = "Title of the table")
kable(d1, format = "html", caption = "Title of the table")
# use the booktabs package
kable(mtcars, format = "latex", booktabs = TRUE)
# use the longtable package
kable(matrix(1000, ncol = 5), format = "latex", digits = 2, longtable = TRUE)
# change LaTeX default table environment
kable(d1, format = "latex", caption = "My table", table.envir = "table*")
# add some table attributes
kable(d1, format = "html", table.attr = "id=\"mytable\"")
# reST output
kable(d2, format = "rst")
# no row names
kable(d2, format = "rst", row.names = FALSE)
# Pandoc simple tables
kable(d2, format = "simple", caption = "Title of the table")
# format numbers using , as decimal point, and ' as thousands separator
x = as.data.frame(matrix(rnorm(60, 1e+06, 10000), 10))
kable(x, format.args = list(decimal.mark = ",", big.mark = "'"))
# save the value
x = kable(d2, format = "html")
cat(x, sep = "\n")
# can also set options(knitr.table.format = 'html') so that the output is HTML

# multiple tables via either kable(list(x1, x2)) or kables(list(kable(x1),
# kable(x2)))
kable(list(d1, d2), caption = "A tale of two tables")
kables(list(kable(d1, align = "l"), kable(d2)), caption = "A tale of two tables")
}
\references{
See
  \url{https://bookdown.org/yihui/rmarkdown-cookbook/kable.html} for some
  examples about this function, including specific arguments according to the
  \code{format} selected.
}
\seealso{
Other R packages such as \pkg{huxtable}, \pkg{xtable},
  \pkg{kableExtra}, \pkg{gt} and \pkg{tables} for HTML and LaTeX tables, and
  \pkg{ascii} and \pkg{pander} for different flavors of markdown output and
  some advanced features and table styles. For more on other packages for
  creating tables, see
  \url{https://bookdown.org/yihui/rmarkdown-cookbook/table-other.html}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\docType{data}
\name{opts_chunk}
\alias{opts_chunk}
\alias{opts_current}
\title{Default and current chunk options}
\format{An object of class \code{list} of length 6.}
\usage{
opts_chunk

opts_current
}
\description{
Options for R code chunks. When running R code, the object \code{opts_chunk}
(default options) is not modified by chunk headers (local chunk options are
merged with default options), whereas \code{opts_current} (current options)
changes with different chunk headers and it always reflects the options for
the current chunk.
}
\details{
Normally we set up the global options once in the first code chunk in a
document using \code{opts_chunk$set()}, so that all \emph{latter} chunks will
use these options. Note the global options set in one chunk will not affect
the options in this chunk itself, and that is why we often need to set global
options in a separate chunk.

See \code{str(knitr::opts_chunk$get())} for a list of default chunk options.
}
\note{
\code{opts_current} should be treated as read-only and you are supposed
  to only query its values via \code{opts_current$get()}. Technically you
  could also call \code{opts_current$set()} to change the values, but you are
  not recommended to do so unless you understand the consequences.
}
\examples{
opts_chunk$get("prompt")
opts_chunk$get("fig.keep")
}
\references{
Usage: \url{https://yihui.org/knitr/objects/}

  A list of available options:
  \url{https://yihui.org/knitr/options/#chunk_options}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes.R
\docType{data}
\name{knit_theme}
\alias{knit_theme}
\title{Syntax highlighting themes}
\format{An object of class \code{list} of length 2.}
\usage{
knit_theme
}
\description{
This object can be used to set or get themes in \pkg{knitr} for syntax
highlighting.
}
\details{
We can use \code{knit_theme$set(theme)} to set the theme, and
\code{knit_theme$get(theme)} to get a theme. The \code{theme} is a character
string for both methods (either the name of the theme, or the path to the CSS
file of a theme), and for the \code{set()} method, it can also be a list
returned by the \code{get()} method. See examples below.
}
\note{
The syntax highlighting here only applies to \file{.Rnw} (LaTeX) and
  \file{.Rhtml} (HTML) documents, and it does not work for other types of
  documents, such as \file{.Rmd} (R Markdown, which has its own syntax
  highlighting themes; see \url{http://rmarkdown.rstudio.com}).
}
\examples{
opts_knit$set(out.format = "latex")
knit_theme$set("edit-vim")

knit_theme$get()  # names of all available themes

thm = knit_theme$get("acid")  # parse the theme to a list
knit_theme$set(thm)

opts_knit$set(out.format = NULL)  # restore option
}
\references{
For a preview of all themes, see
  \url{https://gist.github.com/yihui/3422133}.
}
\author{
Ramnath Vaidyanathan and Yihui Xie
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/params.R
\name{knit_params_yaml}
\alias{knit_params_yaml}
\title{Extract knit parameters from YAML text}
\usage{
knit_params_yaml(yaml, evaluate = TRUE)
}
\arguments{
\item{yaml}{Character vector containing the YAML text.}

\item{evaluate}{If \code{TRUE} (the default) expression values
embedded within the YAML will be evaluated. If \code{FALSE}, parameters
defined with an expression will have the parsed but unevaluated expression
in their \code{value} field.}
}
\value{
List of objects of class \code{knit_param} that correspond to the
parameters declared in the \code{params} section of the YAML. See
\code{\link{knit_params}} for a full description of these objects.
}
\description{
This function reads the YAML front-matter that has already been extracted
from a document and returns a list of any parameters declared there.
}
\seealso{
\code{\link{knit_params}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\name{engine_output}
\alias{engine_output}
\title{An output wrapper for language engine output}
\usage{
engine_output(options, code, out, extra = NULL)
}
\arguments{
\item{options}{A list of chunk options. Usually this is just the object
\code{options} passed to the engine function; see
\code{\link{knit_engines}}.}

\item{code}{Source code of the chunk, to which the output hook
\code{source} is applied, unless the chunk option \code{echo} is \code{FALSE}.}

\item{out}{Text output from the engine, to which the hook \code{output}
is applied, unless the chunk option \code{results} is \code{'hide'}}

\item{extra}{Any additional text output that you want to include.}
}
\value{
A character string generated from the source code and output using
  the appropriate output hooks.
}
\description{
If you have designed a language engine, you may call this function in the end
to format and return the text output from your engine.
}
\details{
For expert users, an advanced usage of this function is
\code{engine_output(options, out = LIST)} where \code{LIST} is a list that
has the same structure as the output of \code{evaluate::evaluate()}. In this
case, the arguments \code{code} and \code{extra} are ignored, and the list is
passed to an internal function \code{knitr:::wrap()} to return a character
vector of final output.
}
\examples{
library(knitr)
engine_output(opts_chunk$merge(list(engine = "Rscript")), 
    code = "1 + 1", out = "[1] 2")
engine_output(opts_chunk$merge(list(echo = FALSE, engine = "Rscript")), 
    code = "1 + 1", out = "[1] 2")

# expert use only
engine_output(opts_chunk$merge(list(engine = "python")), 
    out = list(structure(list(src = "1 + 1"), class = "source"), 
        "2"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parser.R
\name{inline_expr}
\alias{inline_expr}
\title{Wrap code using the inline R expression syntax}
\usage{
inline_expr(code, syntax)
}
\arguments{
\item{code}{Character string of the inline R source code.}

\item{syntax}{A character string to specify the syntax, e.g. \code{rnw},
\code{html}, or \code{md}. If not specified, this will be guessed from
the knitting context.}
}
\value{
A character string marked up using the inline R code syntax.
}
\description{
This is a convenience function to write the "source code" of inline R
expressions. For example, if you want to write \samp{`r 1+1`} literally in an
R Markdown document, you may write \samp{`` `r knitr::inline_expr('1+1')`
``}; for Rnw documents, this may be
\samp{\verb|\Sexpr{knitr::inline_expr{'1+1'}}|}.
}
\examples{
library(knitr)
inline_expr("1+1", "rnw")
inline_expr("1+1", "html")
inline_expr("1+1", "md")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{current_input}
\alias{current_input}
\title{Query the current input filename}
\usage{
current_input(dir = FALSE)
}
\arguments{
\item{dir}{Boolean; whether to prepend the current working directory to the file path,
i.e. whether to return an absolute path or a relative path.}
}
\value{
A character string, if this function is called inside an input
  document. Otherwise \code{NULL}.
}
\description{
Returns the name of the input file passed to \code{\link{knit}()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_latex_output}
\alias{is_latex_output}
\alias{is_html_output}
\alias{pandoc_to}
\alias{pandoc_from}
\title{Check the current input and output type}
\usage{
is_latex_output()

is_html_output(fmt = pandoc_to(), excludes = NULL)

pandoc_to(fmt)

pandoc_from()
}
\arguments{
\item{fmt}{A character vector of output formats to be checked against. If not
provided, \code{is_html_output()} uses \code{pandoc_to()}, and
\code{pandoc_to()} returns the output format name.}

\item{excludes}{A character vector of output formats that should not be
considered as HTML format. Options are: markdown, epub, html, html4, html5,
revealjs, s5, slideous, slidy, and gfm.}
}
\description{
The function \code{is_latex_output()} returns \code{TRUE} when the output
format is LaTeX; it works for both \file{.Rnw} and R Markdown documents (for
the latter, the two Pandoc formats \code{latex} and \code{beamer} are
considered LaTeX output). The function \code{is_html_output()} only works for
R Markdown documents and will test for several Pandoc HTML based output
formats (by default, these formats are considered as HTML formats:
\code{c('markdown', 'epub', 'html', 'html4', 'html5', 'revealjs', 's5',
'slideous', 'slidy', 'gfm')}).
}
\details{
The function \code{pandoc_to()} returns the Pandoc output format, and
\code{pandoc_from()} returns Pandoc input format. \code{pandoc_to(fmt)}
allows to check the current output format against a set of format names. Both
are to be used with R Markdown documents.

These functions may be useful for conditional output that depends on the
output format. For example, you may write out a LaTeX table in an R Markdown
document when the output format is LaTeX, and an HTML or Markdown table when
the output format is HTML. Use \code{pandoc_to(fmt)} to test a more specific
Pandoc format.

Internally, the Pandoc output format of the current R Markdown document is
stored in \code{knitr::\link{opts_knit}$get('rmarkdown.pandoc.to')}, and the
Pandoc input format in
\code{knitr::\link{opts_knit}$get('rmarkdown.pandoc.from')}
}
\note{
See available Pandoc formats, in
  \href{https://pandoc.org/MANUAL.html}{Pandoc's Manual}
}
\examples{
# check for output formats type
knitr::is_latex_output()
knitr::is_html_output()
knitr::is_html_output(excludes = c("markdown", "epub"))
# Get current formats
knitr::pandoc_from()
knitr::pandoc_to()
# Test if current output format is 'docx'
knitr::pandoc_to("docx")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{include_graphics}
\alias{include_graphics}
\title{Embed external images in \pkg{knitr} documents}
\usage{
include_graphics(
  path,
  auto_pdf = getOption("knitr.graphics.auto_pdf", FALSE),
  dpi = NULL,
  error = TRUE
)
}
\arguments{
\item{path}{A character vector of image paths.}

\item{auto_pdf}{Whether to use PDF images automatically when the output
format is LaTeX. If \code{TRUE}, then e.g. \file{foo/bar.png} will be
replaced by \file{foo/bar.pdf} if the latter exists. This can be useful
since normally PDF images are of higher quality than raster images like
PNG, when the output is LaTeX/PDF.}

\item{dpi}{DPI (dots per inch) value. Used to calculate the output width (in
inches) of the images. This will be their actual width in pixels, divided
by \code{dpi}. If not provided, the chunk option \code{dpi} is used; if
\code{NA}, the output width will not be calculated.}

\item{error}{Whether to signal an error if any files specified in the
\code{path} argument do not exist and are not web resources.}
}
\value{
The same as the input character vector \code{path} but it is marked
  with special internal S3 classes so that \pkg{knitr} will convert the file
  paths to proper output code according to the output format.
}
\description{
When plots are not generated from R code, there is no way for \pkg{knitr} to
capture plots automatically. In this case, you may generate the images
manually and pass their file paths to this function to include them in the
output. The major advantage of using this function is that it is portable in
the sense that it works for all document formats that \pkg{knitr} supports,
so you do not need to think if you have to use, for example, LaTeX or
Markdown syntax, to embed an external image. Chunk options related to
graphics output that work for normal R plots also work for these images, such
as \code{out.width} and \code{out.height}.
}
\note{
This function is supposed to be used in R code chunks or inline R code
  expressions. You are recommended to use forward slashes (\verb{/}) as path
  separators instead of backslashes in the image paths.

  The automatic calculation of the output width requires the \pkg{png}
  package (for PNG images) or the \pkg{jpeg} package (for JPEG images). The
  width will not be calculated if the chunk option \code{out.width} is
  already provided or \code{dpi = NA}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-base64.R
\name{image_uri}
\alias{image_uri}
\title{Encode an image file to a data URI}
\usage{
image_uri(f)
}
\arguments{
\item{f}{Path to the image file.}
}
\value{
The data URI as a character string.
}
\description{
This function takes an image file and uses the \pkg{markdown} package to
encode it as a base64 string, which can be used in the \code{img} tag in
HTML.
}
\examples{
uri = image_uri(file.path(R.home("doc"), "html", "logo.jpg"))
if (interactive()) {
    cat(sprintf("<img src=\"\%s\" />", uri), file = "logo.html")
    browseURL("logo.html")  # you can check its HTML source
}
}
\references{
\url{http://en.wikipedia.org/wiki/Data_URI_scheme}
}
\author{
Wush Wu and Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output.R
\name{knit_child}
\alias{knit_child}
\title{Knit a child document}
\usage{
knit_child(..., options = NULL, envir = knit_global())
}
\arguments{
\item{...}{Arguments passed to \code{\link{knit}}.}

\item{options}{A list of chunk options to be used as global options inside
the child document. When one uses the \code{child}
option in a parent chunk, the chunk options of the parent chunk will be
passed to the \code{options} argument here.  Ignored if not a list.}

\item{envir}{Environment in which code chunks are to be evaluated,
for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
\code{\link{globalenv}()}).}
}
\value{
A character string of the content of the compiled child document is
  returned as a character string so it can be written back to the parent
  document directly.
}
\description{
This function knits a child document and returns a character string to input
the result into the main document. It is designed to be used in the chunk
option \code{child} and serves as the alternative to the
\command{SweaveInput} command in Sweave.
}
\note{
This function is not supposed be called directly like
  \code{\link{knit}()}; instead it must be placed in a parent document to let
  \code{\link{knit}()} call it indirectly.

  The path of the child document is determined relative to the parent document.
}
\examples{
# you can write \Sexpr{knit_child('child-doc.Rnw')} in an Rnw file 'main.Rnw' to
# input results from child-doc.Rnw in main.tex

# comment out the child doc by \Sexpr{knit_child('child-doc.Rnw', eval = FALSE)}
}
\references{
\url{https://yihui.org/knitr/demo/child/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{read_rforge}
\alias{read_rforge}
\title{Read source code from R-Forge}
\usage{
read_rforge(path, project, extra = "")
}
\arguments{
\item{path}{Relative path to the source script on R-Forge.}

\item{project}{Name of the R-Forge project.}

\item{extra}{Extra parameters to be passed to the URL (e.g. \code{extra =
'&revision=48'} to check out the source of revision 48).}
}
\value{
A character vector of the source code.
}
\description{
This function reads source code from the SVN repositories on R-Forge.
}
\examples{
library(knitr)
\donttest{
# relies on r-forge.r-project.org being accessible
read_rforge("rgl/R/axes.R", project = "rgl")
read_rforge("rgl/R/axes.R", project = "rgl", extra = "&revision=519")
}
}
\author{
Yihui Xie and Peter Ruckdeschel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output.R
\name{knit_meta}
\alias{knit_meta}
\alias{knit_meta_add}
\title{Metadata about objects to be printed}
\usage{
knit_meta(class = NULL, clean = TRUE)

knit_meta_add(meta, label = "")
}
\arguments{
\item{class}{Optionally return only metadata entries that inherit from the
specified class. The default, \code{NULL}, returns all entries.}

\item{clean}{Whether to clean the collected metadata. By default, the
metadata stored in \pkg{knitr} is cleaned up once retrieved, because we may
not want the metadata to be passed to the next \code{knit()} call; to be
defensive (i.e. not to have carryover metadata), you can call
\code{knit_meta()} before \code{knit()}.}

\item{meta}{A metadata object to be added to the session.}

\item{label}{A chunk label to indicate which chunk the metadata belongs to.}
}
\value{
\code{knit_meta()} returns the matched metadata specified by
  \code{class}; \code{knit_meta_add()} returns all current metadata.
}
\description{
As an object is printed, \pkg{knitr} will collect metadata about it (if
available). After knitting is done, all the metadata is accessible via this
function. You can manually add metadata to the \pkg{knitr} session via
\code{knit_meta_add()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.R
\docType{data}
\name{knit_hooks}
\alias{knit_hooks}
\title{Hooks for R code chunks, inline R code and output}
\format{An object of class \code{list} of length 6.}
\usage{
knit_hooks
}
\description{
A hook is a function of a pre-defined form (arguments) that takes values of
arguments and returns desired output. The object \code{knit_hooks} is used to
access or set hooks in this package.
}
\examples{
knit_hooks$get("source")
knit_hooks$get("inline")
}
\references{
Usage: \url{https://yihui.org/knitr/objects/}

Components in \code{knit_hooks}: \url{https://yihui.org/knitr/hooks/}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{is_low_change}
\alias{is_low_change}
\title{Compare two recorded plots}
\usage{
is_low_change(p1, p2)
}
\arguments{
\item{p1, p2}{Plot objects.}
}
\value{
Logical value indicating whether \code{p2} is a low-level update of
  \code{p1}.
}
\description{
Check if one plot only contains a low-level update of another plot.
}
\examples{
pdf(NULL)
dev.control("enable")  # enable plot recording
plot(1:10)
p1 = recordPlot()
abline(0, 1)  # add a line (a low-level change)
p2 = recordPlot()
plot(rnorm(100))
p3 = recordPlot()  # draw a completely new plot
dev.off()
knitr::is_low_change(p1, p2)  # true
knitr::is_low_change(p1, p3)  # false
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-conversion.R
\name{knit2html}
\alias{knit2html}
\title{Convert markdown to HTML using knit() and markdownToHTML()}
\usage{
knit2html(
  input,
  output = NULL,
  ...,
  envir = parent.frame(),
  text = NULL,
  quiet = FALSE,
  encoding = "UTF-8",
  force_v1 = FALSE
)
}
\arguments{
\item{input}{Path to the input file.}

\item{output}{Path to the output file for \code{knit()}. If \code{NULL}, this
function will try to guess a default, which will be under the current working
directory.}

\item{...}{Options passed to \code{\link[markdown]{markdownToHTML}}.}

\item{envir}{Environment in which code chunks are to be evaluated,
for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
\code{\link{globalenv}()}).}

\item{text}{A character vector. This is an alternative way to provide the input
file.}

\item{quiet}{Boolean; suppress the progress bar and messages?}

\item{encoding}{Encoding of the input file; always assumed to be UTF-8 (i.e.,
this argument is effectively ignored).}

\item{force_v1}{Boolean; whether to force rendering the input document as an R
Markdown v1 document, even if it is for v2.}
}
\value{
If the argument \code{text} is NULL, a character string (HTML code)
  is returned; otherwise the result is written into a file and the filename
  is returned.
}
\description{
This is a convenience function to knit the input markdown source and call
\code{\link[markdown]{markdownToHTML}()} in the \pkg{markdown} package to
convert the result to HTML.
}
\note{
The \pkg{markdown} package is for R Markdown v1, which is much less
  powerful than R Markdown v2, i.e. the \pkg{rmarkdown} package
  (\url{https://rmarkdown.rstudio.com}). To render R Markdown v2 documents to
  HTML, please use \code{rmarkdown::render()} instead.
}
\examples{
# a minimal example
writeLines(c("# hello markdown", "```{r hello-random, echo=TRUE}", "rnorm(5)", "```"), 
    "test.Rmd")
knit2html("test.Rmd")
if (interactive()) browseURL("test.html")

unlink(c("test.Rmd", "test.html", "test.md"))
}
\seealso{
\code{\link{knit}}, \code{\link[markdown]{markdownToHTML}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spin.R
\name{spin}
\alias{spin}
\title{Spin goat's hair into wool}
\usage{
spin(
  hair,
  knit = TRUE,
  report = TRUE,
  text = NULL,
  envir = parent.frame(),
  format = c("Rmd", "Rnw", "Rhtml", "Rtex", "Rrst"),
  doc = "^#+'[ ]?",
  inline = "^[{][{](.+)[}][}][ ]*$",
  comment = c("^[# ]*/[*]", "^.*[*]/ *$"),
  precious = !knit && is.null(text)
)
}
\arguments{
\item{hair}{Path to the R script. The script must be encoded in UTF-8 if it
contains multibyte characters.}

\item{knit}{Logical; whether to compile the document after conversion.}

\item{report}{Logical; whether to generate a report for \file{Rmd},
\file{Rnw} and \file{Rtex} output. Ignored if \code{knit = FALSE}.}

\item{text}{A character vector of code, as an alternative way to provide the
R source. If \code{text} is not \code{NULL}, \code{hair} will be ignored.}

\item{envir}{Environment for \code{\link{knit}()} to evaluate the code.}

\item{format}{Character; the output format. The default is R Markdown.}

\item{doc}{A regular expression to identify the documentation lines; by
default it follows the roxygen convention, but it can be customized, e.g.
if you want to use \code{##} to denote documentation, you can use
\code{'^##\\\\s*'}.}

\item{inline}{A regular expression to identify inline R expressions; by
default, code of the form \code{\{\{code\}\}} on its own line is treated as
an inline expression.}

\item{comment}{A pair of regular expressions for the start and end delimiters
of comments; the lines between a start and an end delimiter will be
ignored. By default, the delimiters are \verb{/*} at the beginning of a
line, and \verb{*/} at the end, following the convention of C comments.}

\item{precious}{logical: whether intermediate files (e.g., \code{.Rmd} files
when \code{format} is \code{"Rmd"}) should be preserved. The default is
\code{FALSE} if \code{knit} is \code{TRUE} and the input is a file.}
}
\value{
If \code{text} is \code{NULL}, the path of the final output document,
  otherwise the content of the output.
}
\description{
This function takes a specially formatted R script and converts it to a
literate programming document. By default normal text (documentation) should
be written after the roxygen comment (\code{#'}) and code chunk options are
written after \code{#+} or \code{#-} or \code{# ----} or any of these
combinations replacing \code{#} with \code{--}.
}
\details{
Obviously the goat's hair is the original R script, and the wool is the
literate programming document (ready to be knitted).
}
\note{
If the output format is Rnw and no document class is specified in
  roxygen comments, this function will automatically add the \code{article}
  class to the LaTeX document so that it is complete and can be compiled. You
  can always specify the document class and other LaTeX settings in roxygen
  comments manually.

  When the output format is Rmd, it is compiled to HTML via
  \code{\link{knit2html}()}, which uses R Markdown v1 instead of v2. If you
  want to use the latter, you should call
  \code{rmarkdown::\link[rmarkdown]{render}()} instead.
}
\references{
\url{https://yihui.org/knitr/demo/stitch/}
}
\seealso{
\code{\link{stitch}} (feed a template with an R script)
}
\author{
Yihui Xie, with the original idea from Richard FitzJohn (who named it
  as \code{sowsear()} which meant to make a silk purse out of a sow's ear)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/citation.R
\name{write_bib}
\alias{write_bib}
\title{Generate BibTeX bibliography databases for R packages}
\usage{
write_bib(..., prefix = getOption("knitr.bib.prefix", "R-"))
}
\arguments{
\item{..., prefix}{Arguments passed to \code{xfun::\link[xfun]{pkg_bib}()}.}
}
\description{
A wrapper function of \code{xfun::pkg_bib()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{extract_raw_output}
\alias{extract_raw_output}
\alias{restore_raw_output}
\alias{raw_output}
\title{Mark character strings as raw output that should not be converted}
\usage{
extract_raw_output(text, markers = raw_markers)

restore_raw_output(text, chunks, markers = raw_markers)

raw_output(x, markers = raw_markers, ...)
}
\arguments{
\item{text}{For \code{extract_raw_output()}, the content of the input file
(e.g. Markdown); for \code{restore_raw_output()}, the content of the output
file (e.g. HTML generated by Pandoc from Markdown).}

\item{markers}{A length-2 character vector to be used to wrap \code{x};
see \code{knitr:::raw_markers} for the default value.}

\item{chunks}{A named character vector returned from
\code{extract_raw_output()}.}

\item{x}{The character vector to be protected.}

\item{...}{Arguments to be passed to \code{\link{asis_output}()}.}
}
\value{
For \code{extract_raw_output()}, a list of two components:
  \code{value} (the \code{text} with raw output replaced by MD5 digests) and
  \code{chunks} (a named character vector, of which the names are MD5 digests
  and values are the raw output). For \code{restore_raw_output()}, the
  restored \code{text}.
}
\description{
These functions provide a mechanism to protect the character output of R code
chunks. The output is annotated with special markers in \code{raw_output};
\code{extract_raw_output()} will extract raw output wrapped in the markers,
and replace the raw output with its MD5 digest; \code{restore_raw_output()}
will restore the MD5 digest with the original raw output.
}
\details{
This mechanism is designed primarily for R Markdown pre/post-processors. In
an R code chunk, you generate \code{raw_output()} to the Markdown output. In
the pre-processor, you can \code{extract_raw_output()} from the Markdown
file, store the raw output and MD5 digests, and remove the actual raw output
from Markdown so Pandoc will never see it. In the post-processor, you can
read the Pandoc output (e.g., an HTML or RTF file), and restore the raw
output.
}
\examples{
library(knitr)
out = c("*hello*", raw_output("<special>content</special> *protect* me!"), 
    "*world*")
pre = extract_raw_output(out)
str(pre)
pre$value = gsub("[*]([^*]+)[*]", "<em>\\\\1</em>", 
    pre$value)  # think this as Pandoc conversion
pre$value
# raw output was protected from the conversion (e.g.
# *protect* was not converted)
restore_raw_output(pre$value, pre$chunks)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc}
\alias{pandoc}
\title{A Pandoc wrapper to convert documents to other formats}
\usage{
pandoc(input, format, config = getOption("config.pandoc"), ext = NA)
}
\arguments{
\item{input}{A character vector of Markdown filenames (must be encoded in
UTF-8).}

\item{format}{Name of the output format (see References). This can be a
character vector of multiple formats; by default, it is obtained from the
\code{t} field in the configuration. If the configuration is empty or the
\code{t} field is not found, the default output format will be
\code{'html'}.}

\item{config}{Path to the Pandoc configuration file. If missing, it is
assumed to be a file with the same base name as the \code{input} file and
an extension \code{.pandoc} (e.g. for \file{foo.md} it looks for
\file{foo.pandoc})}

\item{ext}{Filename extensions. By default, the extension is inferred from
the \code{format}, e.g. \code{latex} creates \code{pdf}, \code{dzslides}
creates \code{html}, and so on}
}
\value{
The output filename(s) (or an error if the conversion failed).
}
\description{
This function calls Pandoc to convert documents to other formats such as
HTML, LaTeX/PDF and Word, etc, (optionally) based on a configuration file or
in-file configurations which specify the options to use for Pandoc.
}
\details{
There are two ways to input the Pandoc configurations -- through a config
file, or embed the configurations in the input file as special comments
between \verb{<!--pandoc} and \verb{-->}.

The configuration file is a DCF file (see \code{\link{read.dcf}}). This file
must contain a field named \code{t} which means the output format. The
configurations are written in the form of \code{tag:value} and passed to
Pandoc (if no value is needed, just leave it empty, e.g. the option
\code{standalone} or \code{s} for short). If there are multiple output
formats, write each format and relevant configurations in a block, and
separate blocks with blank lines.

If there are multiple records of the \code{t} field in the configuration, the
input markdown file will be converted to all these formats by default, unless
the \code{format} argument is specified as one single format.
}
\examples{
system("pandoc -h")  # see possible output formats
}
\references{
Pandoc: \url{https://pandoc.org}; Examples and rules of the
  configurations: \url{https://yihui.org/knitr/demo/pandoc/}

  Also see R Markdown (v2) at \url{http://rmarkdown.rstudio.com}. The
  \pkg{rmarkdown} package has several convenience functions and templates
  that make it very easy to use Pandoc. The RStudio IDE also has
  comprehensive support for it, so I'd recommend users who are not familiar
  with command-line tools to use the \pkg{rmarkdown} package instead.
}
\seealso{
\code{\link{read.dcf}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\docType{data}
\name{rand_seed}
\alias{rand_seed}
\title{An unevaluated expression to return .Random.seed if exists}
\usage{
rand_seed
}
\description{
This expression returns \code{.Random.seed} when \code{eval(rand_seed)} and
\code{NULL} otherwise.
}
\details{
It is designed to work with \code{opts_chunk$set(cache.extra = rand_seed)}
for reproducibility of chunks that involve with random number generation. See
references.
}
\examples{
eval(rand_seed)
rnorm(1)  # .Random.seed is created (or modified)
eval(rand_seed)
}
\references{
\url{https://yihui.org/knitr/demo/cache/}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{fig_path}
\alias{fig_path}
\title{Path for figure files}
\usage{
fig_path(suffix = "", options = opts_current$get(), number)
}
\arguments{
\item{suffix}{A filename suffix; if it is non-empty and does not
contain a dot \code{.}, it will be treated as the filename extension (e.g.
\code{png} will be used as \code{.png})}

\item{options}{A list of options; by default the options of the current chunk.}

\item{number}{The current figure number. The default is the internal chunk option
\code{fig.cur}, if this is available.}
}
\value{
A character vector of the form \file{fig.path-label-i.suffix}.
}
\description{
The filename of figure files is the combination of options \code{fig.path}
and \code{label}. This function returns the path of figures for the current
chunk by default.
}
\note{
When there are special characters (not alphanumeric or \samp{-} or
  \samp{_}) in the path, they will be automatically replaced with \samp{_}.
  For example, \file{a b/c.d-} will be sanitized to \file{a_b/c_d-}. This
  makes the filenames safe to LaTeX.
}
\examples{
fig_path(".pdf", options = list(fig.path = "figure/abc-", label = "first-plot"))
fig_path(".png", list(fig.path = "foo-", label = "bar"), 1:10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{download_image}
\alias{download_image}
\title{Download an image from the web and include it in a document}
\usage{
download_image(
  url,
  path = xfun::url_filename(url),
  use_file = !pandoc_to("html"),
  ...
)
}
\arguments{
\item{url}{The URL of an image.}

\item{path}{The download path (inferred from the URL by default). If the file
exists, it will not be downloaded (downloading can take time and requires
Internet connection). If you are sure the file needs to be downloaded
again, delete it beforehand.}

\item{use_file}{Whether to use the URL or the download path to include the
image. By default, the URL is used for HTML output formats, and the file
path is used for other output formats.}

\item{...}{Other arguments to be passed to \code{\link{include_graphics}()}.}
}
\description{
When including images in non-HTML output formats such as LaTeX/PDF, URLs will
not work as image paths. In this case, we have to download the images. This
function is a wrapper of \code{xfun::\link[xfun]{download_file}()} and
\code{\link{include_graphics}()}.
}
\examples{\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
knitr::download_image("https://www.r-project.org/Rlogo.png")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\name{set_alias}
\alias{set_alias}
\title{Set aliases for chunk options}
\usage{
set_alias(...)
}
\arguments{
\item{...}{Named arguments. Argument names are aliases, and argument values
are real option names.}
}
\value{
\code{NULL}. \code{opts_knit$get('aliases')} is modified as the side effect.
}
\description{
We do not have to use the chunk option names given in \pkg{knitr}; we can set
aliases for them. The aliases are a named character vector; the names are
aliases and the elements in this vector are the real option names.
}
\examples{
set_alias(w = "fig.width", h = "fig.height")
# then we can use options w and h in chunk headers instead of fig.width and
# fig.height
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{raw_block}
\alias{raw_block}
\alias{raw_latex}
\alias{raw_html}
\title{Mark character strings as raw blocks in R Markdown}
\usage{
raw_block(x, type = "latex", ...)

raw_latex(x, ...)

raw_html(x, ...)
}
\arguments{
\item{x}{The character vector to be protected.}

\item{type}{The type of raw blocks (i.e., the Pandoc output format). If you
are not sure about the Pandoc output format of your document, insert a code
chunk \code{knitr:::pandoc_to()} and see what it returns after the document
is compiled.}

\item{...}{Arguments to be passed to \code{\link{asis_output}()}.}
}
\description{
Wraps content in a raw attribute block, which protects it from being escaped
by Pandoc. See \url{https://pandoc.org/MANUAL.html#generic-raw-attribute}.
Functions \code{raw_latex()} and \code{raw_html()} are shorthands of
\code{raw_block(x, 'latex')} and \code{raw_block(x, 'html')}, respectively.
}
\examples{
knitr::raw_latex("\\\\emph{some text}")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_crop}
\alias{plot_crop}
\title{Crop a plot (remove the edges) using PDFCrop or ImageMagick}
\usage{
plot_crop(x, quiet = TRUE)
}
\arguments{
\item{x}{Filename of the plot.}

\item{quiet}{Whether to suppress standard output from the command.}
}
\value{
The original filename.
}
\description{
The program \command{pdfcrop} (often shipped with a LaTeX distribution) is
executed on a PDF plot file, and
\code{magick::\link[magick:transform]{image_trim}()} is executed for other
types of plot files.
}
\details{
The program \command{pdfcrop} can crop the extra white margins when the plot
format is PDF, to make better use of the space in the output document,
otherwise we often have to struggle with \code{graphics::\link{par}()} to set
appropriate margins. Note \command{pdfcrop} often comes with a LaTeX
distribution such as TinyTeX, MiKTeX, or TeX Live, and you may not need to
install it separately (use \code{Sys.which('pdfcrop')} to check it; if it not
empty, you are able to use it). Note that \command{pdfcrop} depends on
GhostScript. You can check if GhostScript is installed via
\code{tools::find_gs_cmd()}.
}
\references{
PDFCrop: \url{https://www.ctan.org/pkg/pdfcrop}. If you use
  TinyTeX, you may install \command{pdfcrop} with
  \code{tinytex::tlmgr_install('pdfcrop')}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{fig_chunk}
\alias{fig_chunk}
\title{Obtain the figure filenames for a chunk}
\usage{
fig_chunk(label, ext = "", number, fig.path = opts_chunk$get("fig.path"))
}
\arguments{
\item{label}{The chunk label.}

\item{ext}{The figure file extension, e.g. \code{png} or \code{pdf}.}

\item{number}{The figure number (by default \code{1}).}

\item{fig.path}{Passed to \code{\link{fig_path}}. By default, the chunk
option \code{fig.path} is used.}
}
\value{
A character vector of filenames.
}
\description{
Given a chunk label, the figure file extension, the figure number(s), and the
chunk option \code{fig.path}, return the filename(s).
}
\details{
This function can be used in an inline R expression to write out the figure
filenames without hard-coding them. For example, if you created a plot in a
code chunk with the label \code{foo} and figure path \file{my-figure/}, you
are not recommended to use hard-coded figure paths like
\samp{\includegraphics{my-figure/foo-1.pdf}} (in \file{.Rnw} documents) or
\samp{![](my-figure/foo-1.png)} (R Markdown) in your document. Instead, you
should use \samp{\\Sexpr{fig_chunk('foo', 'pdf')}} or \samp{![](`r
fig_chunk('foo', 'png')`)}.

You can generate plots in a code chunk but not show them inside the code
chunk by using the chunk option \code{fig.show = 'hide'}. Then you can use
this function if you want to show them elsewhere.
}
\examples{
library(knitr)
fig_chunk("foo", "png")
fig_chunk("foo", "pdf")
fig_chunk("foo", "svg", 2)  # the second plot of the chunk foo
fig_chunk("foo", "png", 1:5)  # if the chunk foo produced 5 plots
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-conversion.R
\name{rnw2pdf}
\alias{rnw2pdf}
\title{Convert an \file{Rnw} document to PDF}
\usage{
rnw2pdf(
  input,
  output = with_ext(input, "pdf"),
  compiler = "xelatex",
  envir = parent.frame(),
  quiet = FALSE,
  clean = TRUE,
  error = FALSE,
  ...
)
}
\arguments{
\item{input}{Path to the input file.}

\item{output}{Path of the PDF output file. By default, it uses the same name
as the \code{input}, but changes the file extension to ".pdf".}

\item{compiler, ...}{The LaTeX engine and other arguments to be passed to
\code{tinytex::\link[tinytex]{latexmk}()}. The default compiler is
\code{xelatex}.}

\item{envir}{Environment in which code chunks are to be evaluated, for
example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
\code{\link{globalenv}()}).}

\item{quiet}{Boolean; suppress the progress bar and messages?}

\item{clean}{If \code{TRUE}, the intermediate files will be removed.}

\item{error}{If \code{FALSE}, knitting stops when any error occurs.}
}
\value{
The \code{output} file path.
}
\description{
Call \code{\link{knit}()} to compile the \file{.Rnw} input to \file{.tex},
and then \code{tinytex::\link[tinytex]{latexmk}()} to convert \file{.tex} to
\file{.pdf}.
}
\details{
This function is similar to \code{\link{knit2pdf}()}, with the following differences:
\enumerate{
  \item The default compiler is "xelatex" instead of "pdflatex".
  \item \code{output} uses the file extension ".pdf" instead of ".tex".
  \item Before knitting, it tries to remove the \code{output} file and will throw a clear error if the file cannot be removed.
  \item \code{output} could be under any dir, not necessarily the same directory as \code{input}.
  \item It cleans up intermediate files by default, including the ".tex" file.
  \item It stops knitting when any error occurs (by setting the chunk option \code{error = FALSE}).
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks-html.R, R/hooks-asciidoc.R,
%   R/hooks-latex.R, R/hooks-md.R, R/hooks-rst.R, R/hooks-textile.R
\name{hook_plot_html}
\alias{hook_plot_html}
\alias{hook_plot_asciidoc}
\alias{hook_plot_tex}
\alias{hook_plot_md}
\alias{hook_plot_rst}
\alias{hook_plot_textile}
\title{Default plot hooks for different output formats}
\usage{
hook_plot_html(x, options)

hook_plot_asciidoc(x, options)

hook_plot_tex(x, options)

hook_plot_md(x, options)

hook_plot_rst(x, options)

hook_plot_textile(x, options)
}
\arguments{
\item{x}{Filename for the plot (a character string).}

\item{options}{A list of the current chunk options.}
}
\value{
A character string of code, with plot filenames wrapped.
}
\description{
These hook functions define how to mark up graphics output in different
output formats.
}
\details{
Depending on the options passed over, \code{hook_plot_tex} may return the
normal \samp{\\includegraphics{}} command, or \samp{\\input{}} (for tikz
files), or \samp{\\animategraphics{}} (for animations); it also takes many
other options into consideration to align plots and set figure sizes, etc.
Similarly, \code{hook_plot_html}, \code{hook_plot_md} and
\code{hook_plot_rst} return character strings which are HTML, Markdown, reST
code.

In most cases we do not need to call these hooks explicitly, and they were
designed to be used internally. Sometimes we may not be able to record R
plots using \code{\link[grDevices]{recordPlot}}, and we can make use of these
hooks to insert graphics output in the output document; see
\code{\link{hook_plot_custom}} for details.
}
\examples{
# this is what happens for a chunk like this

# <<foo-bar-plot, dev='pdf', fig.align='right'>>=
hook_plot_tex("foo-bar-plot.pdf", opts_chunk$merge(list(fig.align = "right")))

# <<bar, dev='tikz'>>=
hook_plot_tex("bar.tikz", opts_chunk$merge(list(dev = "tikz")))

# <<foo, dev='pdf', fig.show='animate', interval=.1>>=

# 5 plots are generated in this chunk
hook_plot_tex("foo5.pdf", opts_chunk$merge(list(fig.show = "animate", interval = 0.1, 
    fig.cur = 5, fig.num = 5)))
}
\references{
\url{https://yihui.org/knitr/hooks/}
}
\seealso{
\code{\link{hook_plot_custom}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{dep_auto}
\alias{dep_auto}
\title{Build automatic dependencies among chunks}
\usage{
dep_auto(path = opts_chunk$get("cache.path"))
}
\arguments{
\item{path}{Path to the dependency file.}
}
\value{
\code{NULL}. The dependencies are built as a side effect.
}
\description{
When the chunk option \code{autodep = TRUE}, all names of objects created in
a chunk will be saved in a file named \file{__objects} and all global objects
used in a chunk will be saved to \file{__globals}. This function can analyze
object names in these files to automatically build cache dependencies, which
is similar to the effect of the \code{dependson} option. It is supposed to be
used in the first chunk of a document and this chunk must not be cached.
}
\note{
Be cautious about \code{path}: because this function is used in a
  chunk, the working directory when the chunk is evaluated is the directory
  of the input document in \code{\link{knit}}, and if that directory differs
  from the working directory before calling \code{knit()}, you need to adjust
  the \code{path} argument here to make sure this function can find the cache
  files \file{__objects} and \file{__globals}.
}
\references{
\url{https://yihui.org/knitr/demo/cache/}
}
\seealso{
\code{\link{dep_prev}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parser.R
\name{convert_chunk_header}
\alias{convert_chunk_header}
\title{Convert the in-header chunk option syntax to the in-body syntax}
\usage{
convert_chunk_header(
  input,
  output = NULL,
  type = c("multiline", "wrap", "yaml"),
  width = 0.9 * getOption("width")
)
}
\arguments{
\item{input}{File path to the document with code chunks to convert.}

\item{output}{The default \code{NULL} will output to console. Other values
can be a file path to write the converted content into or a function which
takes \code{input} as argument and returns a file path to write into (e.g.,
\code{output = identity} to overwrite the input file).}

\item{type}{This determines how the in-body options will be formatted.
\code{"mutiline"} (the default, except for \file{qmd} documents, for which
the default is \code{"yaml"}) will write each chunk option on a separate
line. Long chunk option values will be wrapped onto several lines, and you
can use \code{width = 0} to keep one line per option only. \code{"wrap"}
will wrap all chunk options together using
\code{\link[base:strwrap]{base::strwrap}()}. \code{"yaml"} will convert
chunk options to YAML.}

\item{width}{An integer passed to \code{base::strwrap()} for \code{type =
"wrap"} and \code{type = "multiline"}. If set to \code{0}, deactivate the
wrapping (for \code{type = "multiline"} only).}
}
\value{
A character vector of converted \code{input} when \code{output =
  NULL}. The output file path with converted content otherwise.
}
\description{
This is a helper function for moving chunk options from the chunk header to
the chunk body using the new syntax.
}
\note{
Learn more about the new chunk option syntax in
  \url{https://yihui.org/en/2022/01/knitr-news/}
}
\section{About \pkg{knitr} option syntax}{


Historical chunk option syntax have chunk option in the chunk header using
valid R syntax. This is an example for \verb{.Rmd} document
\preformatted{
```\{r, echo = FALSE, fig.width: 10\}
```
}

New syntax allows to pass option inside the chunk using several variants
\itemize{
\item Passing options one per line using valid R syntax. This corresponds to \code{convert_chunk_header(type = "multiline")}.
\preformatted{
```\{r\}
#| echo = FALSE,
#| fig.width = 10
```
}

\item Passing option part from header in-chunk with several line if wrapping is
needed. This corresponds to \code{convert_chunk_header(type = "wrap")}
\preformatted{
```\{r\}
#| echo = FALSE, fig.width = 10
```
}
\item Passing options key value pairs in-chunk using YAML syntax. Values are no
more R expression but valid YAML syntax. This corresponds to
\code{convert_chunk_header(type = "yaml")} (not implement yet).
\preformatted{```\{r\}
#| echo: false,
#| fig.width: 10
```
}
}
}

\examples{
knitr_example = function(...) system.file("examples", ..., package = "knitr")
# Convert a document for multiline type
convert_chunk_header(knitr_example("knitr-minimal.Rmd"))
# Convert a document for wrap type
convert_chunk_header(knitr_example("knitr-minimal.Rmd"), type = "wrap")
# Reduce default wrapping width
convert_chunk_header(knitr_example("knitr-minimal.Rmd"), type = "wrap", width = 0.6 *
    getOption("width"))
\dontrun{
# Explicitly name the output
convert_chunk_header("test.Rmd", output = "test2.Rmd")
# Overwrite the input
convert_chunk_header("test.Rmd", output = identity)
# Use a custom function to name the output
convert_chunk_header("test.Rmd", output = \(f) sprintf("\%s-new.\%s",
    xfun::sans_ext(f), xfun::file_ext(f)))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{knit_global}
\alias{knit_global}
\title{The global environment for evaluating code}
\usage{
knit_global(envir = NULL)
}
\arguments{
\item{envir}{If \code{NULL}, the function returns the \code{envir} argument
of \code{\link{knit}}, otherwise it should be a new environment for
evaluating code, in which case the function returns the old environment
after setting the new environment.}
}
\description{
Get or set the environment in which code chunks are evaluated.
}
